package gov.va.med.pharmacy.pecs.persistence;

import gov.va.med.pharmacy.pecs.customupdate.FdbVersion;
import gov.va.med.pharmacy.pecs.customupdate.FdbVersionDAO;
import gov.va.med.pharmacy.pecs.domain.impl.va.DrugPairVersionVa;
import gov.va.med.pharmacy.pecs.query.Page;
import gov.va.med.pharmacy.pecs.service.reports.impl.NullDrugDrugPairsCustomizationReport;
import gov.va.med.pharmacy.pecs.test.AbstractUnitTest;

import java.util.Iterator;
import java.util.List;

import javax.annotation.Resource;

import junit.framework.Assert;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.junit.Test;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.transaction.annotation.Transactional;

/**
 * Unit Tests for the DrugPairDAOImpl class.
 * 
 * @author vhaisaduffg
 *
 */
public class DrugPairDAOImplTest extends AbstractUnitTest implements InitializingBean {

    private static final Log LOGGER = LogFactory.getLog(DrugPairDAOImplTest.class);
    
    @Resource
    private DrugPairDAOImpl drugPairDao;
    @Resource
    private FdbVersionDAO fdbVersionDao;
    @Resource
    private NullDrugDrugPairsCustomizationReport report;
    
    private static String COMMENT_TEXT = "FDB Deleted On ";
    private static String NO_DATA_FOUND_TEXT = "No data found.";
    private static String REPORT_DATA_FOUND = "The null drug pair report data is not empty"; 

    @Test
    @Transactional
    public void testDeletedDrugPairRemoval() throws Exception {
        try {
            drugPairDao.deleteDrugPairsWithDeletedRoutedGeneric("VERSION_USER", "VERSION_USER", getComment(), COMMENT_TEXT);
            
            // Check to make sure all Drug Pairs have been removed
            List<List<Object>> nullDrugPairReportData = report.findData();
            if ( ! nullDrugPairReportData.isEmpty()) {
                // A Null Drug Pair Report will only contain a header and "No data found" message if no Drug Pairs are returned.
                if ( nullDrugPairReportData.size() == 2) {
                    String message = (String) nullDrugPairReportData.get(1).get(0);
                    if ( ! NO_DATA_FOUND_TEXT.equals(message)) {
                        Assert.fail(REPORT_DATA_FOUND);
                    }
                } else {
                    Assert.fail(REPORT_DATA_FOUND);
                }
            }
        } catch(Exception e) {
            LOGGER.error(e.getMessage(), e);
            throw e;
        }
    }
    
    /**
     * Checks to make sure all injectable beans have been injected.
     * 
     * @throws Exception with a message showing what bean has not been injected
     */
    @Override
    public void afterPropertiesSet() throws Exception {
        if ( drugPairDao == null ) {
            throw new Exception("DrugPairDAOImpl has not been injected in to DrugPairDAOImplTest.");
        }
        if ( fdbVersionDao == null ) {
            throw new Exception("FdbVersionDAO has not been injected in to DrugPairDAOImplTest.");
        }
        if ( report == null ) {
            throw new Exception("NullDrugDrugPairsCustomizationReport has not been injected in to DrugPairDAOImplTest.");
        }
    }

    private String getComment() {
        StringBuilder sb = new StringBuilder();
        sb.append(COMMENT_TEXT);
        sb.append(" ");
        sb.append(findIssueDate());
        return sb.toString();
    }
    
    /**
     * Returns the latest FDB Issue Date.
     * 
     * @return String
     */
    private String findIssueDate() {
        String issueDate = null;
        FdbVersion fdbVersion = fdbVersionDao.findById(new Long("1"));
        issueDate = fdbVersion.getIssuedate();

        return issueDate;
    }

    /**
     * Sets the DrugPairDAOImpl object to use.
     * 
     * @param drugPairDao the DrugPairDAOImpl object
     */
    public void setDrugPairDao(DrugPairDAOImpl drugPairDao) {
        this.drugPairDao = drugPairDao;
    }
    
    /**
     * Tests the paging capabilities in HibernateDAOImpl and Page. Cycles through the pages of a multi-page query, ensuring
     * total count equals the reported query size and that the last row of one page is not repeated as the first row of the
     * second page. Note that the query includes sorting.
     */
    @SuppressWarnings("unchecked")
    @Test
    @Transactional
    public void testFindByQueryByPage() {
        String query =
                "from DrugPairVersionVa where versionId = versionedObject.exportManager.currentVersion.versionId AND ( lower(versionedObject.routedGeneric1.description) like '%aspirin%' OR lower(versionedObject.routedGeneric2.description) like '%aspirin%' ) order by date desc, id asc";
        int pageNum = 1;
        int totCount = 0;
        Page page = drugPairDao.findByQueryByPage(query, pageNum);

        Assert.assertNotNull("page is null", page);
        Assert.assertTrue("The query does not require multiple pages", 1 < page.getTotalPages());

        totCount += page.getResults().size();

        // we want to avoid any indexing with +1 or -1 in our tests since that could just mask erroneous +1/-1 in the real code,
        // so we iterate to find the last item.
        DrugPairVersionVa lastFromPreviousPage = null;

        @SuppressWarnings("unchecked")
        Iterator<DrugPairVersionVa> it = page.getResults().iterator();
        while (it.hasNext()) {
            lastFromPreviousPage = it.next();
        }

        while (page.isNextPage()) {
            pageNum++;
            page = drugPairDao.findByQueryByPage(query, pageNum);
            Assert.assertNotNull(page);
            totCount += page.getResults().size();
            DrugPairVersionVa firstFromCurrentPage = (DrugPairVersionVa) page.getResults().get(0);
            Assert.assertFalse("Last row from prevous page is same as first from this page!",
                    compareIds(lastFromPreviousPage, firstFromCurrentPage));
            
            DrugPairVersionVa newLastFromPreviousPage = null; 
            
            it = page.getResults().iterator();
            while (it.hasNext()) {
                newLastFromPreviousPage = it.next();
                Assert.assertFalse("Last row from prevous page is same as first from this page!",
                        compareIds(lastFromPreviousPage, newLastFromPreviousPage));                                
            }            
            
            lastFromPreviousPage = newLastFromPreviousPage;
        }

        Assert.assertTrue("Number of records returned from all the pages does not equal the querySize.",
                totCount == page.getQuerySize());
    }

    /**
     * Compares the key fields from two DrugPairVersionVa objects.
     * @param lastFromPreviousPage
     * @param firstFromCurrentPage
     * @return true if they are the same; false if they are different.
     */
    private boolean compareIds(DrugPairVersionVa lastFromPreviousPage, DrugPairVersionVa firstFromCurrentPage) {
        if (!lastFromPreviousPage.getRoutedGeneric1().equals(firstFromCurrentPage.getRoutedGeneric1())){
            return false;
        }
        if (!lastFromPreviousPage.getRoutedGeneric2().equals(firstFromCurrentPage.getRoutedGeneric2())){
            return false;
        }
        if (!lastFromPreviousPage.getInteraction().getFdbId().equals(firstFromCurrentPage.getInteraction().getFdbId())){
            return false;
        }
        return true;    
    }    
    
}
