package gov.va.med.pharmacy.pecs.web.customization;

import gov.va.med.pharmacy.pecs.dto.DrugPairDTO;
import gov.va.med.pharmacy.pecs.dto.PecsDTO;
import gov.va.med.pharmacy.pecs.test.AbstractUnitTest;

import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;

import junit.framework.Assert;

import org.junit.Test;
import org.springframework.mock.web.MockHttpServletResponse;
import org.springframework.transaction.annotation.Transactional;

/**
 * JUnit test class for CustomizationQueryController.  This is more of an integration test driver.
 *
 */
public class CustomizationQueryControllerTest extends AbstractUnitTest{

    @Resource
    private CustomizationQueryController controller;  

    /**
     * Test submitQuery for FDB Drug Pairs, expecting paging, ordering results by some field, and checking for duplication between pages.
     * @throws Exception
     */
    @Test
    @Transactional
    public void testSubmitQueryDrugPairFDB() throws Exception {
        testSubmitQuery("FDB", "DRUG_PAIR", 
                "[{ \"field\":\"RTGENID1_DESC\", \"operator\":\"CONTAINS\", \"value\":\"zith\" },\"AND\",{ \"field\":\"SEVERITYLEVELCODE\", \"operator\":\"EQ\", \"value\":\"3\" }]/[/@['rtgenid2Desc']]", true);
    }    

    /**
     * Test submitQuery for VA Drug Pairs, expecting paging, ordering results by some field, and checking for duplication between pages.
     * @throws Exception
     */
    @Test
    @Transactional
    public void testSubmitQueryDrugPairVA() throws Exception {
        testSubmitQuery("VA", "DRUG_PAIR", 
                "[{ \"field\":\"RTGENID1_DESC\", \"operator\":\"CONTAINS\", \"value\":\"aspirin\" }]/[/@['rtgenid1Desc']]", true);
    }    
    
    /**
     * Test submitQuery for FDB Drug Drug Interactions, expecting paging, ordering results by some field, and checking for duplication between pages.
     * @throws Exception
     */   
    @Test
    @Transactional
    public void testSubmitQueryDrugDrugInteractionFDB() throws Exception {
        testSubmitQuery("FDB", "DRUG_DRUG_INTERACTION", 
                "[{ \"field\":\"INTERACTIONID\", \"operator\":\"CONTAINS\", \"value\":\"1\" },\"AND\",{ \"field\":\"SEVERITYLEVELCODE\", \"operator\":\"CONTAINS\", \"value\":\"1\" }]/[/@['severitylevelcode']", true);
    }                

    /**
     * Test submitQuery for VA Drug Drug Interactions, expecting paging, ordering results by some field, and checking for duplication between pages.
     * @throws Exception
     */       
    @Test
    @Transactional
    public void testSubmitQueryDrugDrugInteractionVA() throws Exception {
        testSubmitQuery("VA", "DRUG_DRUG_INTERACTION", 
                "[{ \"field\":\"INTERACTIONID\", \"operator\":\"CONTAINS\", \"value\":\"1\" },\"AND\",{ \"field\":\"ACTION_STATUS\", \"operator\":\"CONTAINS\", \"value\":\"Approved\" }]/[/@['interactionid']", true);
    }        

    /**
     * Test submitQuery for FDB Dose Range, expecting paging, ordering results by some field, and checking for duplication between pages.
     * @throws Exception
     */          
    @Test
    @Transactional
    public void testSubmitQueryDoseRangeFDB() throws Exception {
        testSubmitQuery("FDB", "DOSE_RANGE", 
                "[{ \"field\":\"CONCEPTTYPE\", \"operator\":\"EQ\", \"value\":\"6\" },\"AND\",{ \"field\":\"AGEHIGHINDAYS\", \"operator\":\"LE\", \"value\":\"365\" },\"AND\",{ \"field\":\"AGELOWINDAYS\", \"operator\":\"GE\", \"value\":\"0\" }]/[/@['agehighindays']", true);
    }        

    /**
     * Test submitQuery for VA Dose Range, expecting paging, ordering results by some field, and checking for duplication between pages.
     * @throws Exception
     */      
    @Test
    @Transactional
    public void testSubmitQueryDoseRangeVA() throws Exception {
        testSubmitQuery("VA", "DOSE_RANGE", 
                "[{ \"field\":\"CONCEPTTYPE\", \"operator\":\"EQ\", \"value\":\"6\" }]/[/@['agehighindays']", false);
    }        

    /**
     * Test submitQuery for FDB Professional Monograph, expecting paging, ordering results by some field, and checking for duplication between pages.
     * @throws Exception
     */     
    @Test
    @Transactional
    public void testSubmitQueryProfessionalMonographFDB() throws Exception {
        testSubmitQuery("FDB", "PROFESSIONAL_MONOGRAPH", 
                "[{ \"field\":\"TITLE\", \"operator\":\"CONTAINS\", \"value\":\"a\" }]/[/@['monographTitle']]", true);                
    }               
    
    /**
     * Test submitQuery for VA Professional Monograph, expecting paging, ordering results by some field, and checking for duplication between pages.
     * Note: there were not enough VA records to cause paging.
     * @throws Exception
     */
    @Test
    @Transactional
    public void testSubmitQueryProfessionalMonographVA() throws Exception {
        testSubmitQuery("VA", "PROFESSIONAL_MONOGRAPH", 
                "[{ \"field\":\"ACTION_DATE_DISPLAY_STRING\", \"operator\":\"GT\", \"value\":\"20000101\" }]/[/@['actionDateDisplayString']]", false);                
    }           
       
    /**
     * Test submitQuery for FDB Duplicate Therapy, expecting paging, ordering results by some field, and checking for duplication between pages.
     * @throws Exception
     */       
    @Test
    @Transactional
    public void testSubmitQueryDuplicateTherapyFDB() throws Exception {
        testSubmitQuery("FDB", "DUPLICATE_THERAPY", 
                "[{ \"field\":\"CUSTOM_DUP_ALLOWANCE\", \"operator\":\"CONTAINS\", \"value\":\"0\" }]/[/@['customstring']]", true);                
    }              

    /**
     * Test submitQuery for VA Duplicate Therapy, expecting paging, ordering results by some field, and checking for duplication between pages.
     * @throws Exception
     */       
    @Test
    @Transactional
    public void testSubmitQueryDuplicateTherapyVA() throws Exception {
        testSubmitQuery("VA", "DUPLICATE_THERAPY", 
                "[{ \"field\":\"CUSTOM_DUP_ALLOWANCE\", \"operator\":\"CONTAINS\", \"value\":\"0\" }]/[/@['customstring']]", false);                
    }                 
    
    /**
     * Common method to submitQuery in the controller.
     * Reads the data page by page, comparing all records from the current page against all previously retrieved
     * records, looking for duplicate keys.  Also, compares the actual count of retrieved rows against the total 
     * query count specified in the Content-Range header.  
     * @param recordType "VA" or "FDB"
     * @param conceptType DOSE_RANGE, DRUG_DRUG_INTERACTION, DRUG_PAIR, DUPLICATE_THERAPY, or PROFESSIONAL_MONOGRAPH
     * @param json The json string, minus any paging range (the test will loop through all pages.  
     *          Tip - run the query in the UI and copy the json string from Explorer F12 tools Network tab
     * @param expectMultiplePages true indicates more than one page should be returned; false indicates that there may not be more than one page
     * @throws Exception If any errors encountered during the test
     */
    private void testSubmitQuery(String recordType, String conceptType, String json, boolean expectMultiplePages) throws Exception {
        String responseRange;
        ConceptRecordForm conceptRecordForm = new ConceptRecordForm();
        
        HttpServletRequest request = new org.springframework.mock.web.MockHttpServletRequest();
        request.getSession().setAttribute("queryCriteria", conceptRecordForm);
        
        MockHttpServletResponse response = new org.springframework.mock.web.MockHttpServletResponse();                                        
        List<PecsDTO> recordsPreviouslyRetrieved = new ArrayList<PecsDTO>();
        List<PecsDTO> recordsOnCurrentPage;
        int startRowInPage = 0;
        int lastRowInPage = 0;
        int totalQueryRows = 0;
        int totalRowsRetrieved = 0;
        String range = "[:100]";
        do{
            
            recordsOnCurrentPage = controller.submitQuery(recordType, conceptType, json + range, false, response, request);
                           
            totalRowsRetrieved = totalRowsRetrieved + recordsOnCurrentPage.size();
            
            for (PecsDTO currPecsDTO : recordsOnCurrentPage) {
                for (PecsDTO prevPecsDTO : recordsPreviouslyRetrieved) {                    
                    if (prevPecsDTO.equals(currPecsDTO)){
                        //special case since the DrugPairDTO only compares rg1id and rg2id and doesn't compare interaction id. 
                        if (currPecsDTO instanceof DrugPairDTO  
                                && !((DrugPairDTO)currPecsDTO).getInteractionid().equals(((DrugPairDTO)prevPecsDTO).getInteractionid())){                            
                            continue;
                        }
                        else{
                            Assert.fail("Duplicate record found.  " + currPecsDTO + " and " + prevPecsDTO);
                        }                            
                    }                        
                }
            }
            
            recordsPreviouslyRetrieved.addAll(recordsOnCurrentPage);
            
            responseRange = (String) response.getHeader("Content-Range");

            Matcher matcher = Pattern.compile("(\\d*)-(\\d*)/(\\d*)").matcher(responseRange);
            while(matcher.find())
            {
                startRowInPage = Integer.valueOf(matcher.group(1));
                lastRowInPage = Integer.valueOf(matcher.group(2));
                totalQueryRows = Integer.valueOf(matcher.group(3));                
            }            
            range = "[" + (startRowInPage + 100) + ":" + (lastRowInPage + 100) + "]";
        } while (lastRowInPage < totalQueryRows);
        
        if (expectMultiplePages){
            Assert.assertTrue("Total Rows needs to be greater than 100 to cause paging.  Please choose different data.", totalQueryRows > 100);
        }
        Assert.assertEquals("Total Rows retrieved does not match the number of rows the query says it has.", totalQueryRows, totalRowsRetrieved);
                
    }        
}
