package gov.va.med.pharmacy.pecs.web.lookup.drugpair;

import gov.va.med.pharmacy.pecs.dto.DrugPairDTO;
import gov.va.med.pharmacy.pecs.test.AbstractUnitTest;
import gov.va.med.pharmacy.pecs.web.customization.ConceptRecordForm;

import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;

import junit.framework.Assert;

import org.junit.Test;
import org.springframework.mock.web.MockHttpServletResponse;
import org.springframework.transaction.annotation.Transactional;


//@RunWith(SpringJUnit4ClassRunner.class)
//
////classpath resource
//@WebAppConfiguration("classpath:test-web-resources")

/**
 * JUnit test class for DrugPairLookupController.  This is more of an integration test driver.
 *
 */
public class DrugPairLookupControllerTest extends AbstractUnitTest{

    @Resource
    private DrugPairLookupController controller;  

    /**
     * Test populateFdbData for FDB Drug Pairs, expecting paging, ordering results by some field, and checking for duplication between pages.
     * @throws Exception
     */
    @Test
    @Transactional
    public void testPopulateFdbData() throws Exception {
        testSubmitQuery("FDB", "[{\"field\":\"RTGENID1_DESC\",\"operator\":\"CONTAINS\",\"value\":\"zith\"},\"AND\",{\"field\":\"SEVERITYLEVELCODE\",\"operator\":\"EQ\",\"value\":\"3\"},\"AND\"]/[/@['rtgenid2Desc']]", 
                true);
    }    
    
    /**
     * Test populateStagingData for VA Drug Pairs, expecting paging, ordering results by some field, and checking for duplication between pages.
     * @throws Exception
     */
    @Test
    @Transactional
    public void testPopulateStagingData() throws Exception {
        testSubmitQuery("VA", "[{\"field\":\"RTGENID1_DESC\",\"operator\":\"CONTAINS\",\"value\":\"aspirin\"},\"AND\",{\"field\":\"SEVERITYLEVELCODE\",\"operator\":\"EQ\",\"value\":\"2\"},\"AND\"]/[/@['rtgenid2Desc']]", 
                true);
    }                   
    
    /**
     * Common method to invoke populateStagingData or populateFdbData in the controller.
     * Reads the data page by page, comparing all records from the current page against all previously retrieved
     * records, looking for duplicate keys.  Also, compares the actual count of retrieved rows against the total 
     * query count specified in the Content-Range header.   
     * @param recordType "VA" or "FDB"
     * @param json The json string, minus any paging range (the test will loop through all pages.  
     *          Tip - run the query in the UI and copy the json string from Explorer F12 tools Network tab
     * @param expectMultiplePages true indicates more than one page should be returned; false indicates that there may not be more than one page
     * @throws Exception If any errors encountered during the test
     */
    private void testSubmitQuery(String recordType, String json, boolean expectMultiplePages) throws Exception {
        String responseRange;

        ConceptRecordForm conceptRecordForm = new ConceptRecordForm();
        
        HttpServletRequest request = new org.springframework.mock.web.MockHttpServletRequest();
        request.getSession().setAttribute("queryCriteria", conceptRecordForm);
        
        MockHttpServletResponse response = new org.springframework.mock.web.MockHttpServletResponse();                                        
        List<DrugPairDTO> recordsPreviouslyRetrieved = new ArrayList<DrugPairDTO>();
        List<DrugPairDTO> recordsOnCurrentPage;
        int startRowInPage = 0;
        int lastRowInPage = 0;
        int totalQueryRows = 0;
        int totalRowsRetrieved = 0;
        String range = "[:100]";
        do{
            if (recordType == "VA"){
                recordsOnCurrentPage = controller.populateStagingData(json + range, response);
            }
            else{                
                recordsOnCurrentPage = controller.populateFdbData(json + range, response); 
            }
        
            totalRowsRetrieved = totalRowsRetrieved + recordsOnCurrentPage.size();
            
            for (DrugPairDTO currDrugPairDTO : recordsOnCurrentPage) {
                for (DrugPairDTO prevDrugPairDTO : recordsPreviouslyRetrieved) {                    
                    if (prevDrugPairDTO.equals(currDrugPairDTO)){
                        //special case since the DrugPairDTO only compares rg1id and rg2id and doesn't compare interaction id. 
                        if (currDrugPairDTO instanceof DrugPairDTO  
                                && !(currDrugPairDTO.getInteractionid().equals(prevDrugPairDTO.getInteractionid()))){                            
                            continue;
                        }
                        else{
                            Assert.fail("Duplicate record found.  " + currDrugPairDTO + " and " + prevDrugPairDTO);
                        }   
                    }                        
                }
            }
            
            recordsPreviouslyRetrieved.addAll(recordsOnCurrentPage);
            
            responseRange = (String) response.getHeader("Content-Range");

            Matcher matcher = Pattern.compile("(\\d*)-(\\d*)/(\\d*)").matcher(responseRange);
            while(matcher.find())
            {
                startRowInPage = Integer.valueOf(matcher.group(1));
                lastRowInPage = Integer.valueOf(matcher.group(2));
                totalQueryRows = Integer.valueOf(matcher.group(3));                
            }            
            range = "[" + (startRowInPage + 100) + ":" + (lastRowInPage + 100) + "]";
        } while (lastRowInPage < totalQueryRows);
        
        if (expectMultiplePages){
            Assert.assertTrue("Total Rows needs to be greater than 100 to cause paging.  Please choose different data.", totalQueryRows > 100);
        }
        Assert.assertEquals("Total Rows retrieved does not match the number of rows the query says it has.", totalQueryRows, totalRowsRetrieved);
                
    }        
}
