package gov.va.med.pharmacy.pecs.customupdate;

import gov.va.med.pharmacy.pecs.domain.impl.va.AbstractCustomizable;
import gov.va.med.pharmacy.pecs.exception.CustomizationServiceException;
import gov.va.med.pharmacy.pecs.persistence.GenericDAO;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Date;
import java.util.zip.ZipOutputStream;

import javax.annotation.Resource;

import org.apache.log4j.Logger;

/**
 * The Class BaseUpdateFileSupport.
 * 
 * @author rick.bloyer
 */
public abstract class BaseUpdateFileSupport {

    /** The Constant logger. */
    private static final Logger LOGGER = Logger.getLogger(BaseUpdateFileSupport.class);

    /** The Constant HEADER_RECORD_TYPE. */
    protected static final String HEADER_RECORD_TYPE = "H";

    /** The Constant FIELD_DEF_RECORD_TYPE. */
    protected static final String FIELD_DEF_RECORD_TYPE = "F";

    /** The Constant PK_DEF_RECORD_TYPE. */
    protected static final String PK_DEF_RECORD_TYPE = "P";

    /** The Constant DELETE_RECORD_TYPE. */
    protected static final String DELETE_RECORD_TYPE = "D";

    /** The Constant CHANGE_RECORD_TYPE. */
    protected static final String CHANGE_RECORD_TYPE = "C";

    /** The Constant ADD_RECORD_TYPE. */
    protected static final String ADD_RECORD_TYPE = "A";

    /** The Constant FIELD_SEPARATOR. */
    protected static final String FIELD_SEPARATOR = "|";

    /** The Constant UPDATE_EMPTY_FIELD. */
    protected static final String UPDATE_EMPTY_FIELD = "''";

    /** The Constant NEW_LINE_STR. */
    protected static final String NEW_LINE_STR = "\r\n";

    @Resource
    private GenericDAO genericDao;

    /**
     * Instantiates a new base update file support.
     */
    public BaseUpdateFileSupport() {
    }

    /**
     * This method takes the parameters given in the signature and creates a record in FDB header record format. The method
     * creates the record and writes it to the output stream given. An example header record is as follows:
     * 
     * H|fdb_custom_ddim|66470|0|0|3.2|W|20090209|USA
     * 
     * Please refer to FDB update file format documentation.
     * 
     * @param zout
     *            The zip output stream that this header record will be written to.
     * @param recordCounts
     *            the record counts
     * @param versionStr
     *            The FDB database update version taken from the FDB_VERSION table.
     * @return An output stream containing this header record.
     */
    public ZipOutputStream generateHeaderRecord(ZipOutputStream zout, RecordCounts recordCounts, String versionStr) {
        if (zout == null || versionStr == null)
            throw new CustomizationServiceException("A required parameter is null or undefined.");

        // Create header record
        String headerRec =
                HEADER_RECORD_TYPE + FIELD_SEPARATOR + getTableName() + FIELD_SEPARATOR + recordCounts.getAddCount()
                        + FIELD_SEPARATOR + recordCounts.getChangeCount() + FIELD_SEPARATOR + recordCounts.getDeleteCount()
                        + FIELD_SEPARATOR + versionStr + NEW_LINE_STR;

        try {
            // Write header to the zip entry
            zout.write(headerRec.getBytes(), 0, headerRec.length());
        } catch (IOException ioe) {
            LOGGER.error("Unable to create header record", ioe);
            throw new CustomizationServiceException("Unable to create header record", ioe);
        }

        return zout;
    }

    /**
     * This method creates a record in FDB update ADD record format. The list parameter contains records to be formatted. The
     * list is iterated, and for each field in each record, the field is appended to the record.
     * 
     * Please refer to FDB update file format documentation for ADD record field definitions.
     * 
     * @param record
     *            the record
     * @param isFull
     *            the is full
     * @return StringBuffer containing the set of records in the FDB update ADD record format.
     */
    protected final StringBuffer formatAddRecord(Object[] record, boolean isFull) {
        StringBuffer buf = new StringBuffer();
        Object field = null;
        String recPrefix = ADD_RECORD_TYPE + FIELD_SEPARATOR;

        // For each record
        if (!isFull) {
            // Only place record prefix on Incremental updates
            buf.append(recPrefix);
        }

        // For each field in record
        for (int i = 0; i < record.length; i++) {
            if (i > 0) {
                buf.append(FIELD_SEPARATOR);
            }
            field = record[i];
            if (field == null) {
                if (!isFull) {
                    buf.append(UPDATE_EMPTY_FIELD);
                }
            } else {
                buf.append(formatField(field, isFull));
            }
        }

        buf.append(NEW_LINE_STR);

        return buf;
    }

    /**
     * This method creates a record in FDB update CHANGE record format. The list parameter contains records to be formatted. The
     * list is iterated, and for each field in each record, the field is appended to the record.
     * 
     * Please refer to FDB update file format documentation for CHANGE record field definitions.
     * 
     * @param record
     *            the record
     * @return StringBuffer containing the set of records in the FDB update CHANGE record format.
     */
    protected StringBuffer formatChangeRecord(Object[] record) {
        StringBuffer buf = new StringBuffer();
        Object field = null;
        String recPrefix = CHANGE_RECORD_TYPE + FIELD_SEPARATOR;
        buf.append(recPrefix);

        // For each field in record
        for (int i = 0; i < record.length; i++) {
            if (i > 0) {
                buf.append(FIELD_SEPARATOR);
            }
            field = record[i];

            // process key fields
            if (i < getKeyFieldCount()) {
                buf.append(formatField(field, false));
            } else {
                buf.append(i + 1);
                buf.append(FIELD_SEPARATOR);
                buf.append(formatField(field, false));
            }
        }

        buf.append(NEW_LINE_STR);

        return buf;
    }

    /**
     * This method creates a record in FDB update DELETE record format. The list parameter contains records to be formatted. The
     * list is iterated, and for each field in each record, the field is appended to the record.
     * 
     * Please refer to FDB update file format documentation for DELETE record field definitions.
     * 
     * @param record
     *            the record
     * @return StringBuffer containing the set of records in the FDB update DELETE record format.
     */
    protected StringBuffer formatDeleteRecord(Object[] record) {
        StringBuffer buf = new StringBuffer();
        Object field = null;
        String recPrefix = DELETE_RECORD_TYPE + FIELD_SEPARATOR;
        buf.append(recPrefix);

        // For each field in record
        for (int i = 0; i < record.length; i++) {
            if (i > 0) {
                buf.append(FIELD_SEPARATOR);
            }
            field = record[i];
            buf.append(formatField(field, false));
        }
        buf.append(NEW_LINE_STR);

        return buf;
    }

    /**
     * This method formats the Object parameter according to FDB update file format rules in preparation to be inserted into an
     * Oracle database. If the Object is not an instance of the String class, the Object is returned. If the Object parameter is
     * an instance of the String class, the parameter is cast into a String instance. The String is searched for any
     * apostrophes, and if found are escaped with the Oracle apostrophe escape character. The String is then enclosed with
     * apostrophes and returned. An example return value would be:
     * 
     * 'ST. JOHN''S WORT'
     * 
     * @param obj
     *            the obj
     * @param isFull
     *            the is full
     * @return the object
     */
    protected Object formatField(Object obj, boolean isFull) {
        if (obj == null) {
            // For a full update, don't include anything. For an incremental
            // update
            // data needs to be in quotes.
            if (!isFull) {
                return UPDATE_EMPTY_FIELD;
            } else {
                return null;
            }
        } else if (!(obj instanceof String))
            return obj;

        if (!isFull) {
            StringBuffer field = new StringBuffer(((String) obj).replaceAll("'", "''"));
            field.insert(0, "'");
            field.append("'");
            return field.toString();
        }
        return obj;
    }

    /**
     * Return the tableName value;.
     * 
     * @return String containing the class field value.
     */
    public abstract String getTableName();

    /**
     * Return an integer of the number of fields in the key for this Order Check type.
     * 
     * @return An integer of the number of fields in the key.
     */
    public abstract int getKeyFieldCount();

    /**
     * The inheriting instance method returns the FDB update file Field Definition Record. The record will be specific to the
     * Order Check type.
     * 
     * Please refer to FDB update file format documentation for the Field Definition Record definition.
     * 
     * @return String containing the FDB update file Field Definition Record for this Order Check type.
     */
    public abstract String generateFieldDefRecord();

    /**
     * An inheriting instance method returns the FDB update file Primary Key Record. The record will be specific to the Order
     * Check type.
     * 
     * Please refer to FDB update file format documentation for the Field Definition Record definition.
     * 
     * @return String containing the FDB update file Primary Key Record for this Order Check type.
     */
    public abstract String generatePrimaryKeyRecord();

    /**
     * Do generate add records.
     * 
     * @param zout
     *            the zout
     * @param fileDate
     *            the file date
     */
    protected abstract void doGenerateAddRecords(ZipOutputStream zout, Date fileDate);

    /**
     * Do generate change records.
     * 
     * @param out
     *            the out
     * @param fileDate
     *            the file date
     */
    protected abstract void doGenerateChangeRecords(ZipOutputStream out, Date fileDate);

    /**
     * Creates the new byte buffer.
     * 
     * @return the byte[]
     */
    private byte[] createNewByteBuffer() {
        byte[] buf = new byte[16 * 1024];
        return buf;
    }

    /**
     * Mehtod to generate delete records.
     * 
     * @param out
     *            the out
     * @param fileDate
     *            the file date
     */
    protected abstract void doGenerateDeleteRecords(ZipOutputStream out, Date fileDate);

    /**
     * An inheriting instance method will generate records in the full FDB update file format. The generated records are
     * appended to the output stream that is given in the method parameter.
     * 
     * @param zout
     *            The output stream that the generated records will be written to.
     * @param file
     *            the file
     */
    public void generateFullRecords(ZipOutputStream zout, File file) {
        if (file != null) {
            try {
                byte[] buf = createNewByteBuffer();
                InputStream in = new FileInputStream(file);
                int r;
                while ((r = in.read(buf)) != -1) {
                    zout.write(buf, 0, r);
                }
                in.close();
            } catch (IOException e) {
                String msg = "Error adding FULL records";
                LOGGER.error(msg, e);
                throw new CustomizationServiceException(msg, e);
            }
        }
    }

    /**
     * An inheriting instance method will return a count of the number of records that will be included in the current full
     * custom update file.
     * 
     * @return Integer count of the number of ADD records to be included in this custom update file.
     */

    public File generateFullRecords() {
        try {
            File fullRecordsTempFile = File.createTempFile("PECS_full_", null);
            OutputStream out = new BufferedOutputStream(new FileOutputStream(fullRecordsTempFile));
            doGenerateFullRecords(out);
            out.close();

            return fullRecordsTempFile;
        } catch (IOException e) {
            String msg = "Failed to get FULL record";
            LOGGER.error(msg, e);
            throw new CustomizationServiceException(msg, e);
        }

    }

    /**
     * Do generate full records.
     * 
     * @param zout
     *            the zout
     */
    protected abstract void doGenerateFullRecords(OutputStream zout);

    /**
     * The name of the file to be used in the custom update archive.
     * 
     * @return the file name
     */
    public abstract String getUpdateFileName();

    /**
     * Gets the generic dao.
     * 
     * @return the generic dao
     */
    public final GenericDAO getGenericDao() {
        return genericDao;
    }

    /**
     * Sets the generic dao.
     * 
     * @param genericDao
     *            the new generic dao
     */
    public void setGenericDao(GenericDAO genericDao) {
        this.genericDao = genericDao;
    }

    /**
     * Save version update.
     * 
     * @param <T>
     *            -
     * 
     * @param fileDate
     *            the file date
     * @param record
     *            the record
     */
    public <T> void saveVersionUpdate(Date fileDate, AbstractCustomizable<T> record) {
        record.getExportableVersion().setExportDate(fileDate);
        record.setExportedVersion(record.getExportableVersion());
        getGenericDao().saveOrUpdate(record);
    }

    /**
     * Gets the all record counts. Two step process to allow Drug Pair to override the getTotalRecordCounts method.
     * 
     * @return the all record counts
     */
    public final RecordCounts getAllRecordCounts() {
        RecordCounts counts = new RecordCounts();
        counts = getIncrementalRecordCounts();
        counts.setTotalCount(getTotalRecordCounts());
        return counts;
    }

    /**
     * Gets the incremental record counts.
     * 
     * @return the incremental record counts
     */
    public RecordCounts getIncrementalRecordCounts() {
        return getGenericDao().findCustomUpdateCounts(getDomainObjectName());
    }

    /**
     * Gets the total record counts. Will be overridden Drug Pairs.
     * 
     * @return the total record counts
     */
    public Long getTotalRecordCounts() {
        return getGenericDao().findCustomUpdateFullCounts(getDomainObjectName());

    }

    /**
     * Gets the domain object name.
     * 
     * @return the domain object name
     */
    public abstract String getDomainObjectName();

    /**
     * Generate records all statuses.
     * 
     * @param fileDate
     *            the file date
     * @param recordCounts
     *            the record counts
     * @param zout
     *            the zout
     */
    public void generateRecordsAllStatuses(Date fileDate, RecordCounts recordCounts, ZipOutputStream zout) {
        try {
            doGenerateDeleteRecords(zout, fileDate);
            doGenerateChangeRecords(zout, fileDate);
            doGenerateAddRecords(zout, fileDate);
        } catch (Exception e) {
            String msg = "Failed to generate incremental Records.";
            LOGGER.error(msg, e);
            throw new CustomizationServiceException(msg, e);
        }

    }
}
