package gov.va.med.pharmacy.pecs.customupdate;

import gov.va.med.pharmacy.pecs.persistence.HibernateDAOImpl;

import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Collections;
import java.util.Date;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.Query;
import org.springframework.stereotype.Repository;

/**
 * DAO class used for Custom Update persistence operations.
 * 
 * @author vhaisaduffg
 * 
 */
@Repository
public final class CustomUpdateDAO extends HibernateDAOImpl<CustomUpdateFileVa> {
    private static final String FIND_BY_MAX_ID_QUERY = "findByMaxId";
    private static final Log LOGGER = LogFactory.getLog(CustomUpdateDAO.class);
    private static final String FIND_ALL_YEARS_OF_UPDATES = "findAllYearsOfUpdates";
    private static final String FIND_MONTHS_BY_YEAR = "findMonthsByYear";
    private static final String FIND_FILES_BY_MONTH_AND_YEAR = "findFilesByMonthAndYear";
    private static final String FIND_ALL_ORDERED_BY_ISSUE_DATE = "findAllOrderedByIssueDate";

    /**
     * Returns the persistent class used in this DAO.
     * 
     * @return Class
     */
    @Override
    protected Class<CustomUpdateFileVa> getPersistentClass() {
        return CustomUpdateFileVa.class;
    }

    /**
     * Returns the CtVersionVa having the highest identifier currently in the database.
     * 
     * @return CtVersionVa
     */
    public CtVersionVa findMaxId() {
        if (LOGGER.is   ugEnabled()) {
            LOGGER.debug("finding CtVersionVa instance with max id");
        }
        try {
            Query queryObject = getSession().getNamedQuery(FIND_BY_MAX_ID_QUERY);
            return (CtVersionVa) queryObject.uniqueResult();
        } catch (RuntimeException re) {
            LOGGER.error("find by max id failed", re);
            throw re;
        }
    }

    /**
     * Find all years of Custom Updates.
     * 
     * @return the list of Dates of custom updates.
     */
    @SuppressWarnings("unchecked")
    public List<Date> findAllYears() {
        if (LOGGER.is   ugEnabled()) {
            LOGGER.debug("finding all CtVersionVa by Year");
        }
        try {
            Query queryObject = getSession().getNamedQuery(FIND_ALL_YEARS_OF_UPDATES);
            return queryObject.list();
        } catch (RuntimeException re) {
            LOGGER.error("Failed To Get The Years Of The Custom Update Records", re);
            throw re;
        }
    }

    /**
     * Find All Months By Year selected.
     * 
     * @param year
     *            String
     * @return the list of Dates of custom updates.
     */
    @SuppressWarnings("unchecked")
    public List<Date> findMonthsByYear(String year) {
        DateFormat formatter = new SimpleDateFormat("yyyy");
        Date date;
        try {
            date = formatter.parse(year);
        } catch (ParseException e) {
            return Collections.EMPTY_LIST;
        }
        if (LOGGER.is   ugEnabled()) {
            LOGGER.debug("finding all CtVersionVa by Year");
        }
        try {
            Query queryObject = getSession().getNamedQuery(FIND_MONTHS_BY_YEAR);
            queryObject.setDate("year", date);
            return queryObject.list();
        } catch (RuntimeException re) {
            LOGGER.error("Failed To Get The Years Of The Custom Update Records", re);
            throw re;
        }
    }

    /**
     * Find All Files By Month and Year.
     * 
     * @param year
     *            String
     * @param month
     *            String
     * @return the list of Dates of custom updates.
     */
    @SuppressWarnings("unchecked")
    public List<CtVersionVa> findFilesByMonthsAndYear(String year, String month) {
        DateFormat formatter = new SimpleDateFormat("yyyyMM");
        Date date;
        try {
            date = formatter.parse(year + month);
        } catch (ParseException e) {
            return Collections.EMPTY_LIST;
        }
        if (LOGGER.is   ugEnabled()) {
            LOGGER.debug("finding all CtVersionVa by Year");
        }
        try {
            Query queryObject = getSession().getNamedQuery(FIND_FILES_BY_MONTH_AND_YEAR);
            queryObject.setDate("date", date);
            return queryObject.list();
        } catch (RuntimeException re) {
            LOGGER.error("Failed To Get The Years Of The Custom Update Records", re);
            throw re;
        }
    }

    /**
     * Returns a List of CtVersionVa order in descending order by Issue Date.
     * 
     * @return List
     */
    @SuppressWarnings("unchecked")
    public List<CtVersionVa> findAllOrderByIssueDateDesc() {
        if (LOGGER.is   ugEnabled()) {
            LOGGER.debug("finding all CtVersionVa ordering by issue date in descending order");
        }
        try {
            Query queryObject = getSession().getNamedQuery(FIND_ALL_ORDERED_BY_ISSUE_DATE);
            return queryObject.list();
        } catch (RuntimeException re) {
            LOGGER.error("find by max id failed", re);
            throw re;
        }
    }
}
