package gov.va.med.pharmacy.pecs.customupdate;

import gov.va.med.pharmacy.pecs.domain.impl.va.DrugDrugInteractionVa;
import gov.va.med.pharmacy.pecs.domain.impl.va.DrugDrugInteractionVersionVa;
import gov.va.med.pharmacy.pecs.exception.CustomizationServiceException;

import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.zip.ZipOutputStream;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.stereotype.Service;

/**
 * The Class DrugDrugInteractionStringsUpdateFileSupport.
 */
@Service
public final class DrugDrugInteractionStringsUpdateFileSupport extends BaseUpdateFileSupport {

    /** The logger. */
    private static Log LOGGER = LogFactory.getLog(DrugDrugInteractionStringsUpdateFileSupport.class);

    /** The Constant FIELD_DEF_RECORD. */
    private static final String FIELD_DEF_RECORD = "F|3|1|category|2|interactionid|3|customstring";

    /** The Constant PRIMARY_KEY_RECORD. */
    private static final String PRIMARY_KEY_RECORD = "P|2|1|2";

    /** The Constant PRIMARY_KEY_RECORD_FIELD_CNT. */
    private static final int PRIMARY_KEY_RECORD_FIELD_CNT = 2;

    /** The Constant TABLE_NAME. */
    private static final String TABLE_NAME = "fdb_custom_ddimstrings";

    /** The Constant DDIS_FILE_NAME. */
    public static final String DDIS_FILE_NAME = "FDBCUSTOMDDIMSTRINGS";

    /** The Constant CATEGORY. */
    private static final String CATEGORY = "FDB_ID";

    /**
     * This instance method returns the FDB update file Field Definition Record. The record will be specific to the Order Check
     * type.
     * 
     * Please refer to FDB update file format documentation for the Field Definition Record definition.
     * 
     * @return String containing the FDB update file Field Definition Record for this Order Check type.
     */
    public String generateFieldDefRecord() {
        return FIELD_DEF_RECORD;
    }

    /**
     * this method returns the FDB update file Primary Key Record. The record will be specific to the Order Check type.
     * 
     * Please refer to FDB update file format documentation for the Field Definition Record definition.
     * 
     * @return String containing the FDB update file Primary Key Record for this Order Check type.
     */
    public String generatePrimaryKeyRecord() {
        return PRIMARY_KEY_RECORD;
    }

    /**
     * Method to generate the all records.
     * 
     * @param zout
     *            the zout
     * @see gov.va.med.pharmacy.pecs.customupdate.BaseUpdateFileSupport#doGenerateFullRecords(java.io.OutputStream)
     */
    @SuppressWarnings("unchecked")
    protected void doGenerateFullRecords(OutputStream zout) {
        List<DrugDrugInteractionVa> list =
                (List<DrugDrugInteractionVa>) getGenericDao().findDrugDrugInteractionStringsCustomUpdateFullRecords(
                        getDomainObjectName());

        if (!list.isEmpty()) {
            for (DrugDrugInteractionVa record : list) {
                // for full updates we're using the same field order as for ADD
                for (Object[] values : getFields(record.getExportableVersion())) {
                    StringBuffer buf = formatAddRecord(values, true);
                    try {
                        zout.write(buf.toString().getBytes());

                    } catch (IOException ioe) {
                        String msg = "Unable to write FULL record to output";
                        LOGGER.error(msg, ioe);
                        throw new CustomizationServiceException(msg, ioe);
                    }
                }
            }
        }

        return;
    }

    /**
     * Method to generate add records.
     * 
     * @param zout
     *            the zout
     * @param fileDate
     *            the file date
     * @see gov.va.med.pharmacy.pecs.customupdate.BaseUpdateFileSupport#doGenerateAddRecords(java.util.zip.ZipOutputStream,
     *      java.util.Date)
     */
    @SuppressWarnings("unchecked")
    protected void doGenerateAddRecords(ZipOutputStream zout, Date fileDate) {
        List<DrugDrugInteractionVa> list =
                (List<DrugDrugInteractionVa>) getGenericDao().findDrugDrugInteractionStringsCustomUpdateAddRecords(
                        getDomainObjectName());

        if (!list.isEmpty()) {
            for (DrugDrugInteractionVa record : list) {
                for (Object[] values : getFields(record.getExportableVersion())) {
                    StringBuffer buf = formatAddRecord(values, false);
                    try {
                        zout.write(buf.toString().getBytes());

                    } catch (IOException ioe) {
                        String msg = "Unable to write ADD record to output";
                        LOGGER.error(msg, ioe);
                        throw new CustomizationServiceException(msg, ioe);
                    }
                }
            }
        }
    }

    /**
     * This method will generate records in the FDB update CHANGE file format. The generated records are appended to the output
     * stream that is given in the method parameter. CHANGE records are approved records that have had a field value change
     * since the last file generation process executed.
     * 
     * @param out
     *            the out
     * @param fileDate
     *            the file date
     */
    @Override
    protected void doGenerateChangeRecords(ZipOutputStream out, Date fileDate) {
        // Old File Showed Changed SQL had been Commented out so no code will be present for this method.
        return;

    }

    /**
     * Extract the values needed for a record from this Drug-Drug Interaction version.
     * 
     * @param version
     *            the version from which the data is to be extracted
     * @return an array of values
     */
    private Object[][] getFields(DrugDrugInteractionVersionVa version) {
        // For an add/ change, we need to specify first the PKs of the records
        // generated for this version, then the other fields;
        // category|interactionid (FDB)|customstring (Custom Interaction Id)";
        // Generate Forward Drug-Drug Interaction
        ArrayList<Object[]> res = new ArrayList<Object[]>();

        Object[] v = new Object[3];
        v[0] = CATEGORY;
        v[1] = version.getVersionedObject().getFdbId();
        v[2] = version.getVersionedObject().getInteractionId();

        res.add(v);

        // Generate Reverse Drug-Drug Interaction
        v = new Object[3];
        v[0] = CATEGORY;
        // The Reverse FDB ID is calculated as 32000 - the Forwards FDB ID
        v[1] = (32000L - version.getVersionedObject().getFdbId());
        v[2] = version.getVersionedObject().getInteractionId();

        res.add(v);
        return res.toArray(new Object[res.size()][]);
    }

    /**
     * Gets the delete fields.
     * 
     * @param version
     *            the version
     * @return the delete fields
     */
    private Object[][] getDeleteFields(DrugDrugInteractionVersionVa version) {
        // For an add/ change, we need to specify first the PKs of the records
        // generated for this version, then the other fields;
        // category|interactionid (FDB)|customstring (Custom Interaction Id)";
        // Generate Forward Drug-Drug Interaction
        ArrayList<Object[]> res = new ArrayList<Object[]>();

        Object[] v = new Object[2];
        v[0] = CATEGORY;
        v[1] = version.getVersionedObject().getFdbId();

        res.add(v);

        // Generate Reverse Drug-Drug Interaction
        v = new Object[2];
        v[0] = CATEGORY;
        // The Reverse FDB ID is calculated as 32000 - the Forwards FDB ID
        v[1] = (32000L - version.getVersionedObject().getFdbId());

        res.add(v);
        return res.toArray(new Object[res.size()][]);
    }

    /**
     * Method to generate the delete records.
     * 
     * @param out
     *            the out
     * @param fileDate
     *            the file date
     * @see gov.va.med.pharmacy.pecs.customupdate.BaseUpdateFileSupport#doGenerateDeleteRecords(java.util.zip.ZipOutputStream,
     *      java.util.Date)
     */
    @SuppressWarnings("unchecked")
    protected void doGenerateDeleteRecords(ZipOutputStream out, Date fileDate) {
        List<DrugDrugInteractionVa> list =
                (List<DrugDrugInteractionVa>) getGenericDao().findDrugDrugInteractionStringsCustomUpdateDeleteRecords(
                        getDomainObjectName());
        if (!list.isEmpty()) {
            for (DrugDrugInteractionVa record : list) {
                for (Object[] values : getDeleteFields(record.getExportableVersion())) {
                    StringBuffer buf = formatDeleteRecord(values);
                    try {
                        out.write(buf.toString().getBytes());

                    } catch (IOException ioe) {
                        String msg = "Unable to write DELETE record to output";
                        LOGGER.error(msg, ioe);
                        throw new CustomizationServiceException(msg, ioe);
                    }
                }
            }
        }
    }

    /**
     * Method to retrieve the update file name.
     * 
     * @return the update file name
     * @see gov.va.med.pharmacy.pecs.customupdate.BaseUpdateFileSupport#getUpdateFileName()
     */
    public String getUpdateFileName() {
        return DDIS_FILE_NAME;
    }

    /**
     * Method to retrieve the key field count.
     * 
     * @return the key field count
     * @see gov.va.med.pharmacy.pecs.customupdate.BaseUpdateFileSupport#getKeyFieldCount()
     */
    @Override
    public int getKeyFieldCount() {
        return PRIMARY_KEY_RECORD_FIELD_CNT;
    }

    /**
     * Method to retrieve the Table Name.
     * 
     * @return Table Name.
     * 
     * @see gov.va.med.pharmacy.pecs.customupdate.BaseUpdateFileSupport#getTableName()
     */
    @Override
    public String getTableName() {
        return TABLE_NAME;
    }

    /**
     * Returns the Concept for the SQL to run in the database.
     * 
     * @return DrugDrugInteractionVa - Concept Name
     * @see gov.va.med.pharmacy.pecs.customupdate.BaseUpdateFileSupport#getDomainObjectName()
     */
    @Override
    public String getDomainObjectName() {

        return "DrugDrugInteractionVa";
    }

    /**
     * Gets the total record counts.
     * 
     * @return Long the total record counts
     */
    @Override
    public Long getTotalRecordCounts() {
        return (getGenericDao().findDrugDrugInteractionStringsCustomUpdateFullCounts(getDomainObjectName()) * 2);

    }

    /**
     * Gets the incremental record counts. DDIMSTRING has no changed record counts so it is zeroed out.
     * 
     * @return the incremental record counts
     */
    @Override
    public RecordCounts getIncrementalRecordCounts() {
        RecordCounts recordCounts = getGenericDao().findCustomUpdateCountsDDIStrings(getDomainObjectName());
        recordCounts.setAddCount(recordCounts.getAddCount() * 2);
        recordCounts.setChangeCount(0L);
        recordCounts.setDeleteCount(recordCounts.getDeleteCount() * 2);
        return recordCounts;
    }

}
