package gov.va.med.pharmacy.pecs.customupdate;

import gov.va.med.pharmacy.pecs.domain.impl.va.DrugDrugInteractionVa;
import gov.va.med.pharmacy.pecs.domain.impl.va.DrugDrugInteractionVersionVa;
import gov.va.med.pharmacy.pecs.domain.impl.va.MonographPair;
import gov.va.med.pharmacy.pecs.domain.impl.va.MonographPairDAO;
import gov.va.med.pharmacy.pecs.exception.CustomizationServiceException;

import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.zip.ZipOutputStream;

import javax.annotation.Resource;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.stereotype.Service;

/**
 * This class creates output files for the fdb updates of drug drug interaction records.
 * 
 */
@Service
public final class DrugDrugInteractionUpdateFileSupport extends BaseUpdateFileSupport {

    /** The logger. */
    private static Log logger = LogFactory.getLog(DrugDrugInteractionUpdateFileSupport.class);

    /** The monograph pair dao. */
    @Resource
    private MonographPairDAO monographPairDAO;

    /** The monograph pairs. */
    private Map<Long, Long> monographPairs = null;

    /** The Constant FIELD_DEF_RECORD. */
    private static final String FIELD_DEF_RECORD =
            "F|18|1|interactionid|2|seqno|3|description1|4|description2|5|description3|6|description4|7|description5|8"
                    + "|description6|9|severitylevelcode|10|monographid|11|uicategory1|12|uicategory2|13|uicategory3|14"
                    + "|uicategory4|15|uicategory5|16|uicategory6|17|clinicaleffectcode1|18|clinicaleffectcode2";

    /** The Constant PRIMARY_KEY_RECORD. */
    private static final String PRIMARY_KEY_RECORD = "P|2|1|2";

    /** The Constant PRIMARY_KEY_RECORD_FIELD_CNT. */
    private static final int PRIMARY_KEY_RECORD_FIELD_CNT = 2;

    /** The Constant TABLE_NAME. */
    private static final String TABLE_NAME = "fdb_custom_ddiminteraction";

    /** The Constant DDI_FILE_NAME. */
    public static final String DDI_FILE_NAME = "FDBCUSTOMDDIMINTERACTION";

    /** The Constant UICATEGORY. */
    private static final String UICATEGORY = "VA";

    /**
     * This instance method returns the FDB update file Field Definition Record. The record will be specific to the Order Check
     * type.
     * 
     * Please refer to FDB update file format documentation for the Field Definition Record definition.
     * 
     * @return String containing the FDB update file Field Definition Record for this Order Check type.
     */
    @Override
    public final String generateFieldDefRecord() {
        return FIELD_DEF_RECORD;
    }

    /**
     * this method returns the FDB update file Primary Key Record. The record will be specific to the Order Check type.
     * 
     * Please refer to FDB update file format documentation for the Field Definition Record definition.
     * 
     * @return String containing the FDB update file Primary Key Record for this Order Check type.
     */
    @Override
    public final String generatePrimaryKeyRecord() {
        return PRIMARY_KEY_RECORD;
    }

    /**
     * Do generate full records.
     * 
     * @param zout
     *            the zout
     */
    @Override
    @SuppressWarnings("unchecked")
    protected void doGenerateFullRecords(OutputStream zout) {

        List<DrugDrugInteractionVa> list =
                (List<DrugDrugInteractionVa>) getGenericDao().findCustomUpdateRecords(getDomainObjectName());

        if (!list.isEmpty()) {
            for (DrugDrugInteractionVa m : list) {
                // for full updates we're using the same field order as for ADD
                for (Object[] values : getFields(m.getExportableVersion())) {
                    StringBuffer buf = formatAddRecord(values, true);
                    try {
                        zout.write(buf.toString().getBytes());

                    } catch (IOException ioe) {
                        String msg = "Unable to write FULL record to output";
                        logger.error(msg, ioe);
                        throw new CustomizationServiceException(msg, ioe);
                    }
                }
            }
        }

        return;
    }

    /**
     * Do generate add records.
     * 
     * @param zout
     *            the zout
     * @param fileDate
     *            the file date
     */
    @Override
    @SuppressWarnings("unchecked")
    protected void doGenerateAddRecords(ZipOutputStream zout, Date fileDate) {
        List<DrugDrugInteractionVa> list =
                (List<DrugDrugInteractionVa>) getGenericDao().findCustomUpdateAddRecords(getDomainObjectName());

        if (!list.isEmpty()) {
            for (DrugDrugInteractionVa interaction : list) {
                for (Object[] values : getFields(interaction.getExportableVersion())) {
                    StringBuffer buf = formatAddRecord(values, false);
                    try {
                        zout.write(buf.toString().getBytes());

                    } catch (IOException ioe) {
                        String msg = "Unable to write ADD record to output";
                        logger.error(msg, ioe);
                        throw new CustomizationServiceException(msg, ioe);
                    }
                }

                saveVersionUpdate(fileDate, interaction);

            }
        }
    }

    /**
     * Do generate change records.
     * 
     * @param out
     *            the out
     * @param fileDate
     *            the file date
     */
    @Override
    @SuppressWarnings("unchecked")
    protected void doGenerateChangeRecords(ZipOutputStream out, Date fileDate) {
        List<DrugDrugInteractionVa> list =
                (List<DrugDrugInteractionVa>) getGenericDao().findCustomUpdateChangeRecords(getDomainObjectName());

        if (!list.isEmpty()) {
            for (DrugDrugInteractionVa interaction : list) {
                for (Object[] values : getFields(interaction.getExportableVersion())) {
                    StringBuffer buf = formatChangeRecord(values);
                    try {
                        out.write(buf.toString().getBytes());

                    } catch (IOException ioe) {
                        String msg = "Unable to write CHANGE record to output";
                        logger.error(msg, ioe);
                        throw new CustomizationServiceException(msg, ioe);
                    }
                }

                saveVersionUpdate(fileDate, interaction);
            }
        }
    }

    /**
     * Extract the values needed for a record from this Drug-Drug Interaction version. This method creates both the Forward and
     * Reverse Drug-Drug Interaction.
     * 
     * @param version
     *            the version from which the data is to be extracted
     * @return an array of values
     */
    private Object[][] getFields(DrugDrugInteractionVersionVa version) {
        // For an add/ change, we need to specify first the PKs of the records
        // generated for this version, then the other fields;
        // interactionid|seqno|description1|description2|description3|description4|description5|description6|severitylevelcode
        // |monographid|uicategory1|uicategory2|uicategory3|uicategory4|uicategory5|uicategory6|clinicaleffectcode1
        // |clinicaleffectcode2"
        ArrayList<Object[]> res = new ArrayList<Object[]>();

        // Generate Forward Drug-Drug Interaction
        int seqno = 1;
        Object[] v = new Object[18];
        v[0] = version.getVersionedObject().getInteractionId();
        v[1] = seqno;
        v[2] = version.getDescription();
        // description2 through description6 are currently unused
        // v[3] = null;
        // v[4] = null;
        // v[5] = null;
        // v[6] = null;
        // v[7] = null;
        v[8] = version.getSeverityLevel().getCode();
        v[9] = version.getMonographId();
        v[10] = UICATEGORY;
        // uicategory2 through 6 are currently unused
        // v[11] = null;
        // v[12] = null;
        // v[13] = null;
        // v[14] = null;
        // v[15] = null;
        v[16] = version.getClinicalEffect1().getCode();
        if (version.getClinicalEffect2() != null) {
            v[17] = version.getClinicalEffect2().getCode();
        }

        res.add(v);

        // Generate Reverse Drug-Drug Interaction
        // Check for reverse monograph
        Long reverseMonograph = getMonographPairs().get(version.getMonographId());

        if (reverseMonograph == null) {
            reverseMonograph = version.getMonographId();
        }
        seqno = 2;
        v = new Object[18];
        v[0] = version.getVersionedObject().getInteractionId();
        v[1] = seqno;
        v[2] = version.getReverseDescription();
        // description2 through description6 are currently unused
        // v[3] = null;
        // v[4] = null;
        // v[5] = null;
        // v[6] = null;
        // v[7] = null;
        v[8] = version.getSeverityLevel().getCode();
        v[9] = reverseMonograph;
        v[10] = UICATEGORY;
        // uicategory2 through 6 are currently unused
        // v[11] = null;
        // v[12] = null;
        // v[13] = null;
        // v[14] = null;
        // v[15] = null;
        v[16] = version.getClinicalEffect1().getReverse().getCode();
        if (version.getClinicalEffect2() != null) {
            v[17] = version.getClinicalEffect2().getReverse().getCode();
        }

        res.add(v);
        return res.toArray(new Object[res.size()][]);
    }

    /**
     * Gets the delete fields.
     * 
     * @param version
     *            the version
     * @return the delete fields
     */
    private Object[][] getDeleteFields(DrugDrugInteractionVersionVa version) {
        // For an add/ change, we need to specify first the PKs of the records
        // generated for this version, then the other fields;
        // interactionid|seqno|description1|description2|description3|description4|description5|description6
        // |severitylevelcode|monographid|uicategory1|uicategory2|uicategory3|uicategory4|uicategory5
        // |uicategory6|clinicaleffectcode1|clinicaleffectcode2"
        ArrayList<Object[]> res = new ArrayList<Object[]>();

        // Generate Forward Drug-Drug Interaction
        int seqno = 1;
        Object[] v = new Object[2];
        v[0] = version.getVersionedObject().getInteractionId();
        v[1] = seqno;

        res.add(v);

        // Generate Reverse Drug-Drug Interaction
        seqno = 2;
        v = new Object[2];
        v[0] = version.getVersionedObject().getInteractionId();
        v[1] = seqno;

        res.add(v);
        return res.toArray(new Object[res.size()][]);
    }

    /**
     * Creates the monograph pairs.
     * 
     * @return the map
     */
    private Map<Long, Long> createMonographPairs() {
        Map<Long, Long> monographIdPairs = new HashMap<Long, Long>() {

            /**
             * 
             */
            private static final long serialVersionUID = 1L;
        };
        List<MonographPair> monoPairs = new ArrayList<MonographPair>();
        monoPairs = monographPairDAO.findAll();
        for (MonographPair monographPair : monoPairs) {
            monographIdPairs.put(monographPair.getForwardMonograph(), monographPair.getReverseMonograph());
            monographIdPairs.put(monographPair.getReverseMonograph(), monographPair.getForwardMonograph());
        }
        return monographIdPairs;

    }

    /**
     * Do generate delete records.
     * 
     * @param out
     *            the out
     * @param fileDate
     *            the file date
     */
    @Override
    @SuppressWarnings("unchecked")
    protected void doGenerateDeleteRecords(ZipOutputStream out, Date fileDate) {
        List<DrugDrugInteractionVa> list =
                (List<DrugDrugInteractionVa>) getGenericDao().findCustomUpdateDeleteRecords(getDomainObjectName());
        if (!list.isEmpty()) {
            for (DrugDrugInteractionVa interaction : list) {
                for (Object[] values : getDeleteFields(interaction.getExportableVersion())) {
                    StringBuffer buf = formatDeleteRecord(values);
                    try {
                        out.write(buf.toString().getBytes());

                    } catch (IOException ioe) {
                        String msg = "Unable to write DELETE record to output";
                        logger.error(msg, ioe);
                        throw new CustomizationServiceException(msg, ioe);
                    }
                }

                saveVersionUpdate(fileDate, interaction);
            }
        }
    }

    /**
     * Returns the Update File Name.
     * 
     * @return String - the file name
     */
    @Override
    public final String getUpdateFileName() {
        return DDI_FILE_NAME;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.customupdate.BaseUpdateFileSupport#getKeyFieldCount()
     */
    @Override
    public final int getKeyFieldCount() {
        return PRIMARY_KEY_RECORD_FIELD_CNT;
    }

    /**
     * Gets the record counts.
     * 
     * @return the record counts
     */

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.customupdate.BaseUpdateFileSupport#getTableName()
     */
    @Override
    public final String getTableName() {
        return TABLE_NAME;
    }

    /**
     * The monograph pairs to return.
     * 
     * @return the monographPairs
     */
    public final Map<Long, Long> getMonographPairs() {
        setMonographPairs(createMonographPairs());
        return monographPairs;
    }

    /**
     * The monograph pairs to set.
     * 
     * @param monographPairs
     *            the monographPairs to set
     */
    public final void setMonographPairs(Map<Long, Long> monographPairs) {
        this.monographPairs = monographPairs;
    }

    @Override
    public String getDomainObjectName() {

        return "DrugDrugInteractionVa";
    }

    /**
     * Gets the total record counts.
     * 
     * @return Long the total record counts
     */
    @Override
    public Long getTotalRecordCounts() {
        return (getGenericDao().findCustomUpdateFullCounts(getDomainObjectName()) * 2);

    }

    @Override
    public RecordCounts getIncrementalRecordCounts() {
        RecordCounts recordCounts = getGenericDao().findCustomUpdateCounts(getDomainObjectName());
        recordCounts.setAddCount(recordCounts.getAddCount() * 2);
        recordCounts.setChangeCount(recordCounts.getChangeCount() * 2);
        recordCounts.setDeleteCount(recordCounts.getDeleteCount() * 2);
        return recordCounts;
    }

}
