package gov.va.med.pharmacy.pecs.customupdate;

import gov.va.med.pharmacy.pecs.domain.MonographSection;
import gov.va.med.pharmacy.pecs.domain.SectionLine;
import gov.va.med.pharmacy.pecs.domain.impl.va.MonographVersion;
import gov.va.med.pharmacy.pecs.domain.impl.va.ProfessionalDrugDrugMonographVa;
import gov.va.med.pharmacy.pecs.exception.CustomizationServiceException;

import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.zip.ZipOutputStream;

import org.apache.log4j.Logger;
import org.springframework.stereotype.Service;

/**
 * MonographUpdateFileSupport Entity.
 * 
 * @author DOMAINOVERSD
 * 
 */
@Service
public final class MonographUpdateFileSupport extends BaseUpdateFileSupport {
    private static final Logger LOGGER = Logger.getLogger(MonographUpdateFileSupport.class);

    /** Table Name. */
    public static final String TABLE_NAME = "fdb_custom_monograph";
    /** Monograph File name. */
    public static final String MONOGRAPH_FILE_NAME = "FDBCUSTOMMONOGRAPH";
    private static final String FIELD_DEF_RECORD = "F|8|1|languageid|2|versioncode|3|categorycode|4|monographid|5"
            + "|sequencenumber|6|sectioncode|7|formatcode|8|linetext";
    private static final String PRIMARY_KEY_RECORD = "P|5|1|2|3|4|5";

    private static final int FDB_MONOGRAPH_LINE_SIZE = 78;

    /**
     * This instance method returns the FDB update file Field Definition Record. The record will be specific to the Order Check
     * type.
     * 
     * Please refer to FDB update file format documentation for the Field Definition Record definition.
     * 
     * @return String containing the FDB update file Field Definition Record for this Order Check type.
     */
    public final String generateFieldDefRecord() {
        return FIELD_DEF_RECORD;
    }

    /**
     * this method returns the FDB update file Primary Key Record. The record will be specific to the Order Check type.
     * 
     * Please refer to FDB update file format documentation for the Field Definition Record definition.
     * 
     * @return String containing the FDB update file Primary Key Record for this Order Check type.
     */
    public final String generatePrimaryKeyRecord() {
        return PRIMARY_KEY_RECORD;
    }

    /**
     * Generate Full Records.
     * 
     * @param zout
     *            OutputStream
     */
    @SuppressWarnings("unchecked")
    protected void doGenerateFullRecords(OutputStream zout) {

        List<ProfessionalDrugDrugMonographVa> list =
                (List<ProfessionalDrugDrugMonographVa>) getGenericDao().findCustomUpdateRecords(getDomainObjectName());

        if (!list.isEmpty()) {
            for (ProfessionalDrugDrugMonographVa m : list) {
                // for full updates we're using the same field order as for ADD

                for (Object[] values : getAddFields(m.getExportableVersion())) {
                    StringBuffer buf = formatAddRecord(values, true);
                    try {
                        zout.write(buf.toString().getBytes());
                    } catch (IOException ioe) {
                        String msg = "Unable to write FULL record to output";
                        LOGGER.error(msg, ioe);
                        throw new CustomizationServiceException(msg, ioe);
                    }
                }
            }
        }

        return;
    }

    /**
     * Method to generate the add records.
     * 
     * @param zout
     *            the zout
     * @param fileDate
     *            the file date
     * @see gov.va.med.pharmacy.pecs.customupdate.BaseUpdateFileSupport#doGenerateAddRecords(java.util.zip.ZipOutputStream,
     *      java.util.Date)
     */
    @SuppressWarnings("unchecked")
    protected void doGenerateAddRecords(ZipOutputStream zout, Date fileDate) {

        List<ProfessionalDrugDrugMonographVa> list =
                (List<ProfessionalDrugDrugMonographVa>) getGenericDao().findCustomUpdateAddRecords(getDomainObjectName());

        if (!list.isEmpty()) {
            for (ProfessionalDrugDrugMonographVa m : list) {
                for (Object[] values : getAddFields(m.getExportableVersion())) {
                    StringBuffer buf = formatAddRecord(values, false);
                    try {
                        zout.write(buf.toString().getBytes());
                    } catch (IOException ioe) {
                        String msg = "Unable to write ADD record to output";
                        LOGGER.error(msg, ioe);
                        throw new CustomizationServiceException(msg, ioe);
                    }
                }
                saveVersionUpdate(fileDate, m);
            }
        }
    }

    /**
     * Generate Records.
     * 
     * @param out
     *            - OutputStream
     * @param fileDate
     *            the file date
     */
    @SuppressWarnings("unchecked")
    protected void doGenerateChangeRecords(ZipOutputStream out, Date fileDate) {
        List<ProfessionalDrugDrugMonographVa> list =
                (List<ProfessionalDrugDrugMonographVa>) getGenericDao().findCustomUpdateChangeRecords(getDomainObjectName());

        if (!list.isEmpty()) {
            for (ProfessionalDrugDrugMonographVa m : list) {

                for (Object[] values : getDeleteFields(m.getExportedVersion())) {

                    StringBuffer buf = formatDeleteRecord(values);
                    try {
                        out.write(buf.toString().getBytes());

                    } catch (IOException ioe) {
                        String msg = "Unable to write DELETE record to output";
                        LOGGER.error(msg, ioe);
                        throw new CustomizationServiceException(msg, ioe);
                    }
                }
                for (Object[] values : getAddFields(m.getExportableVersion())) {

                    StringBuffer buf = formatAddRecord(values, false);
                    try {
                        out.write(buf.toString().getBytes());

                    } catch (IOException ioe) {
                        String msg = "Unable to write ADD record to output";
                        LOGGER.error(msg, ioe);
                        throw new CustomizationServiceException(msg, ioe);
                    }
                }
                saveVersionUpdate(fileDate, m);
            }
        }
    }

    /**
     * Extract the values needed for a CHANGE record from this monograph version.
     * 
     * @param version
     *            the version from which the data is to be extracted
     * @return an array of values
     */
    private Object[][] getAddFields(MonographVersion version) {
        int seqno = 1;
        ArrayList<Object[]> res = new ArrayList<Object[]>();
        for (MonographSection section : version.getSections()) {
            for (SectionLine line : section.splitText(FDB_MONOGRAPH_LINE_SIZE)) {
                Object[] v = new Object[8];
                v[0] = version.getVersionedObject().getLanguageId();
                v[1] = version.getVersionCode();
                v[2] = version.getCategoryCode();
                v[3] = version.getVersionedObject().getId();
                v[4] = seqno++;
                v[5] = section.getSectionCode().getCode();
                v[6] = line.getFormatCode();
                v[7] = line.getText();
                res.add(v);
            }
        }
        return res.toArray(new Object[res.size()][]);
    }

    /**
     * Generate Deleted Records.
     * 
     * @param out
     *            RecordCounts
     * @param fileDate
     *            the file date
     */
    @SuppressWarnings("unchecked")
    protected void doGenerateDeleteRecords(ZipOutputStream out, Date fileDate) {
        List<ProfessionalDrugDrugMonographVa> list =
                (List<ProfessionalDrugDrugMonographVa>) getGenericDao().findCustomUpdateDeleteRecords(getDomainObjectName());
        if (!list.isEmpty()) {
            for (ProfessionalDrugDrugMonographVa m : list) {
                for (Object[] values : getDeleteFields(m.getExportedVersion())) {
                    StringBuffer buf = formatDeleteRecord(values);
                    try {
                        out.write(buf.toString().getBytes());

                    } catch (IOException ioe) {
                        String msg = "Unable to write DELETE record to output";
                        LOGGER.error(msg, ioe);
                        throw new CustomizationServiceException(msg, ioe);
                    }
                }
                saveVersionUpdate(fileDate, m);
            }
        }
    }

    /**
     * Extract the values needed for a DELETE record from this monograph version.
     * 
     * @param version
     *            the version from which the data is to be extracted
     * @return an array of values
     */
    private Object[][] getDeleteFields(MonographVersion version) {
        // For a delete, we need to specify the PKs of the records
        // generated for this version; note that there are multiple
        // records per version (one for each section line, formatted for FDB).
        // languageid|versioncode|categorycode|monographid|sequencenumber
        int seqno = 1;
        ArrayList<Object[]> res = new ArrayList<Object[]>();
        for (MonographSection section : version.getSections()) {
            for (SectionLine line : section.splitText(FDB_MONOGRAPH_LINE_SIZE)) {
                Object[] v = new Object[5];
                v[0] = version.getVersionedObject().getLanguageId();
                v[1] = version.getVersionCode();
                v[2] = version.getCategoryCode();
                v[3] = version.getVersionedObject().getId();
                v[4] = seqno++;
                res.add(v);
            }
        }
        return res.toArray(new Object[res.size()][]);
    }

    /**
     * Update File Name.
     * 
     * @return MONOGRPAH_FILE_NAME
     * 
     */
    @Override
    public final String getUpdateFileName() {
        return MONOGRAPH_FILE_NAME;
    }

    /**
     * Get Key Field Count.
     * 
     * @return 5
     */
    @Override
    public final int getKeyFieldCount() {
        return 5;
    }

    @Override
    public final String getTableName() {
        return TABLE_NAME;
    }

    @Override
    public String getDomainObjectName() {

        return "ProfessionalDrugDrugMonographVa";
    }

    /**
     * Record Counts are calculated by getting the number of lines within the sections of the Monograph's Exportable Version.
     * 
     * @return the total record counts
     */

    public Long getTotalRecordCounts() {
        @SuppressWarnings("unchecked")
        List<ProfessionalDrugDrugMonographVa> list =
                (List<ProfessionalDrugDrugMonographVa>) getGenericDao().findCustomUpdateRecords(getDomainObjectName());
        Long counts = new Long(0);
        for (ProfessionalDrugDrugMonographVa item : list) {
            MonographVersion version = item.getExportableVersion();
            for (MonographSection section : version.getSections()) {
                counts += section.splitText(FDB_MONOGRAPH_LINE_SIZE).size();
            }
        }
        return counts;
    }

    public Long getAddRecordCounts() {
        @SuppressWarnings("unchecked")
        List<ProfessionalDrugDrugMonographVa> list =
                (List<ProfessionalDrugDrugMonographVa>) getGenericDao().findCustomUpdateAddRecords(getDomainObjectName());
        Long counts = new Long(0);
        for (ProfessionalDrugDrugMonographVa item : list) {
            MonographVersion version = item.getExportableVersion();
            for (MonographSection section : version.getSections()) {
                counts += section.splitText(FDB_MONOGRAPH_LINE_SIZE).size();
            }
        }
        return counts;
    }

    public Long getDeletedRecordCounts() {
        @SuppressWarnings("unchecked")
        List<ProfessionalDrugDrugMonographVa> list =
                (List<ProfessionalDrugDrugMonographVa>) getGenericDao().findCustomUpdateDeleteRecords(getDomainObjectName());
        Long counts = new Long(0);
        for (ProfessionalDrugDrugMonographVa item : list) {
            MonographVersion version = item.getExportableVersion();
            for (MonographSection section : version.getSections()) {
                counts += section.splitText(FDB_MONOGRAPH_LINE_SIZE).size();
            }
        }
        return counts;
    }

    public Long getChangeRecordCounts() {
        @SuppressWarnings("unchecked")
        List<ProfessionalDrugDrugMonographVa> list =
                (List<ProfessionalDrugDrugMonographVa>) getGenericDao().findCustomUpdateChangeRecords(getDomainObjectName());
        Long counts = new Long(0);
        for (ProfessionalDrugDrugMonographVa item : list) {
            MonographVersion version = item.getExportableVersion();
            for (MonographSection section : version.getSections()) {
                counts += section.splitText(FDB_MONOGRAPH_LINE_SIZE).size();
            }
        }
        return counts;
    }

    /**
     * Gets the incremental record counts. Change counts are processed as one add and one delete. Counts reflect the actual
     * transactions processed.
     * 
     * @return the incremental record counts
     */
    public RecordCounts getIncrementalRecordCounts() {
        RecordCounts counts = new RecordCounts();
        Long changeCount = getChangeRecordCounts();
        counts.setAddCount(getAddRecordCounts() + changeCount);
        counts.setChangeCount(0L);
        counts.setDeleteCount(getDeletedRecordCounts() + changeCount);
        return counts;
    }

}
