package gov.va.med.pharmacy.pecs.domain;

import gov.va.med.pharmacy.pecs.domain.impl.va.ConceptXref;
import gov.va.med.pharmacy.pecs.state.PecsStateMachine;
import gov.va.med.pharmacy.pecs.state.Status;

import java.util.Date;
import java.util.List;

/**
 * The interface that concepts that can be customized by users must implement.
 * 
 * @param <T>
 *            the generic type
 */
public interface Customizable<T> {
    /**
     * Obtain the date when a customization became externally visible.
     * 
     * @return the date of the customization
     */
    Date getEffectiveDate();

    /**
     * Obtain the last date when change to this customization request was made.
     * 
     * @return the date of the customization
     */
    Date getModifiedDate();

    /**
     * Record the date when a customization was made.
     * 
     * @param date
     *            the customization date
     */
    void setModifiedDate(Date date);

    /**
     * A comment that documents the reason for a specific action.
     * 
     * @return the comment text
     */
    String getComment();

    /**
     * Record the reason for the latest action.
     * 
     * @param reason
     *            the text
     */
    void setComment(String reason);

    /**
     * The user currently assigned to this customization request.
     * 
     * @return the user id
     */
    String getAssignedUser();

    /**
     * Assign a user to this customization request.
     * 
     * @param user
     *            the user id
     */
    void setAssignedUser(String user);

    // /**
    // * Medical references justifying the customization request.
    // *
    // * @return the text
    // */
    // String getReferenceText();
    //
    // /**
    // * Provide reference text.
    // *
    // * @param referenceText the text
    // */
    // void setReferenceText(String referenceText);

    /**
     * The user who last performed an action causing a status change on this customization request.
     * 
     * @return the user id
     */
    String getActionBy();

    /**
     * Record the user who last performed an action causing a status change on this customization request.
     * 
     * @param user
     *            the user id
     */
    void setActionBy(String user);

    /**
     * The user who initiated this customization request.
     * 
     * @return user the user id
     */
    public String getRequestor();

    /**
     * The status of this customization request. This indicates how far along in the workflow it got.
     * 
     * @return the status id
     */
    Status getStatus();

    /**
     * Update the status of this request. This is typically done by
     * 
     * @param status
     *            the status id {@link PecsStateMachine} as a result of applying an action that causes a state transition.
     */
    void setStatus(Status status);

    /**
     * The most recent version.
     * 
     * @return the current version
     */
    Version<T> getCurrentVersion();

    /**
     * The most recent version that could be exported. That typically means the version must be in an APPROVED or DELETED
     * status.
     * 
     * @return the last exportable version
     */
    Version<T> getExportableVersion();

    /**
     * Record the current version as the most current exportable version.
     */
    void setExportableVersion();

    /**
     * Rolls this instance back to the state of its most current exportable version. That means the current version will become
     * a copy of the exportable version.
     */
    void rollBack();

    /**
     * Obtain the version history for this instance.
     * 
     * @return a list of versions, in order, beginning with the oldest; the last item in the list is the current version, all
     *         others are considered historic (used for audit purposes)
     */
    List<Version<T>> getVersionHistory();

    /**
     * The user who last made a change to this customization request.
     * 
     * @return the user id
     */
    String getAuditUser();

    /**
     * Record the user who last last made a change to this customization request.
     * 
     * @param user
     *            the user id
     */
    void setAuditUser(String user);

    /**
     * Gets the concept xref.
     * 
     * @return the concept xref
     */
    ConceptXref getConceptXref();

    /**
     * Sets the concept xref.
     * 
     * @param xref
     *            the new concept xref
     */
    void setConceptXref(ConceptXref xref);

}
