package gov.va.med.pharmacy.pecs.domain;

import gov.va.med.pharmacy.pecs.domain.impl.lookup.DoseRoute;
import gov.va.med.pharmacy.pecs.domain.impl.lookup.DoseType;
import gov.va.med.pharmacy.pecs.domain.impl.lookup.HalfLifeUnit;

/**
 * This class represents Dose Range from the FDB database. From the FDB documentation:
 * 
 * Custom Dose Range Check information can be established at the RoutedDrug, RoutedGeneric, RoutedDoseFormDrug,
 * RoutedDoseFormGeneric, and GCNSeqNo level. This custom dosing information can be used for reference or screening. The
 * fdb_custom_dosing table is used to define custom Dose Range Check information.
 * 
 * Note that for a given drug, range parameters may not overlap. For example, if an age range of 10 to 29 days is already in
 * use, an age range of 15 to 29 days cannot be used.
 * 
 * Dosing for DispensableDrug, PackagedDrug, ManufacturedDrug, and RegionalPackagedDrug concepts will be dosed based on the
 * drugs GCNSeqNo. If custom information is not found at the Name-Based level for RoutedDrug and RoutedDoseFormDrug items the
 * DIFWK looks for custom dosing defined at the appropriate generic concept level.
 * 
 * 
 * @author VHAISPMETELA
 * 
 */
public interface DoseRange {

    /**
     * Dose route identifier.
     * 
     * @return the dose route identifier.
     */
    DoseRoute getDoseRoute();

    /**
     * Dose route identifier.
     * 
     * @param doseRoute
     *            to set.
     */
    void setDoseRoute(DoseRoute doseRoute);

    /**
     * Dose type identifier.
     * 
     * @return the dose type identifier.
     */
    DoseType getDoseType();

    /**
     * Dose type identifier.
     * 
     * @param doseType
     *            to set.
     */
    void setDoseType(DoseType doseType);

    /**
     * FDBDX type code to identify a Medical Condition.
     * 
     * @return the FDBDX type code to identify a Medical Condition.
     */
    String getFdbdx();

    /**
     * User-defined category code.
     * 
     * @param fdbdx
     *            to set.
     */
    void setFdbdx(String fdbdx);

    /**
     * User-defined category code.
     * 
     * @return the user-defined category code.
     */
    String getCategory();

    /**
     * User-defined category code.
     * 
     * @param category
     *            to set.
     */
    void setCategory(String category);

    /**
     * Number identifying the drug.
     * 
     * @return the number identifying the drug.
     */
    Long getConceptid();

    /**
     * Number identifying the drug.
     * 
     * @param conceptid
     *            to set.
     */
    void setConceptid(Long conceptid);

    /**
     * Minimum amount to be administered per day.
     * 
     * @return the minimum amount to be administered per day.
     */
    Double getDoseLow();

    /**
     * Minimum amount to be administered per day.
     * 
     * @param doseLow
     *            to set.
     */
    void setDoseLow(Double doseLow);

    /**
     * Low dose for a given dose form.
     * 
     * @return the low dose for a given dose form.
     */
    Double getDoseFormLow();

    /**
     * Low dose for a given dose form.
     * 
     * @param doseFormLow
     *            to set.
     */
    void setDoseFormLow(Double doseFormLow);

    /**
     * Highest amount to be administered per day.
     * 
     * @return the highest amount to be administered per day.
     */
    Double getDoseHigh();

    /**
     * Highest amount to be administered per day.
     * 
     * @param doseHigh
     *            to set.
     */
    void setDoseHigh(Double doseHigh);

    /**
     * High dose for a given dose form.
     * 
     * @return the high dose for a given dose form.
     */
    Double getDoseFormHigh();

    /**
     * High dose for a given dose form.
     * 
     * @param doseFormHigh
     *            to set.
     */
    void setDoseFormHigh(Double doseFormHigh);

    /**
     * Maximum amount to be administered in a single dose.
     * 
     * @return the maximum amount to be administered in a single dose.
     */
    Double getMaxSingleDose();

    /**
     * Number identifying the drugs concept type.
     * 
     * @return the number identifying the drugs concept type.
     */
    Long getConceptType();

    /**
     * Number identifying the drugs concept type.
     * 
     * @param conceptType
     *            to set.
     */
    void setConceptType(Long conceptType);

    /**
     * Maximum single dose for a given form.
     * 
     * @param maxSingleDose
     *            to set.
     */
    void setMaxSingleDose(Double maxSingleDose);

    /**
     * Maximum single dose for a given form.
     * 
     * @return the maximum single dose for a given form.
     */
    Double getMaxSingleDoseForm();

    /**
     * Maximum single dose for a given form.
     * 
     * @param maxSingleDoseForm
     *            to set.
     */
    void setMaxSingleDoseForm(Double maxSingleDoseForm);

    /**
     * Maximum amount to be administered per day.
     * 
     * @return the maximum amount to be administered per day.
     */
    Double getMaxDailyDose();

    /**
     * Maximum amount to be administered per day.
     * 
     * @param maxDailyDose
     *            to set.
     */
    void setMaxDailyDose(Double maxDailyDose);

    /**
     * Maximum amount to be administered per dose rate (hours or minutes).
     * 
     * @return the Maximum amount to be administered per dose rate (hours or minutes).
     */
    Double getMaxDailyDoseForm();

    /**
     * Maximum amount to be administered per dose rate (hours or minutes).
     * 
     * @param maxDailyDoseForm
     *            to set.
     */
    void setMaxDailyDoseForm(Double maxDailyDoseForm);

    /**
     * Maximum amount to be administered over a patients lifetime, if available.
     * 
     * @return the maximum amount to be administered over a patients lifetime, if available.
     */
    Double getMaxLifeTimeDose();

    /**
     * Maximum amount to be administered over a patients lifetime, if available.
     * 
     * @param maxLifeTimeDose
     *            to set.
     */
    void setMaxLifeTimeDose(Double maxLifeTimeDose);

    /**
     * Maximum lifetime dose for a given dose form.
     * 
     * @return the maximum lifetime dose for a given dose form.
     */
    Double getMaxLifeTimeDoseForm();

    /**
     * Maximum lifetime dose for a given dose form.
     * 
     * @param maxLifeTimeDoseForm
     *            to set.
     */
    void setMaxLifeTimeDoseForm(Double maxLifeTimeDoseForm);

    /**
     * Indicates the low end of the drugs half-life range.
     * 
     * @return the indicates the low end of the drugs half-life range.
     */
    Double getLowEliminationHalfLife();

    /**
     * Indicates the low end of the drugs half-life range.
     * 
     * @param lowEliminationHalfLife
     *            to set.
     */
    void setLowEliminationHalfLife(Double lowEliminationHalfLife);

    /**
     * Indicates the high end of the drugs half-life range.
     * 
     * @return the indicates the high end of the drugs half-life range.
     */
    Double getHighEliminationHalfLife();

    /**
     * Indicates the high end of the drugs half-life range.
     * 
     * @param highEliminationHalfLife
     *            to set.
     */
    void setHighEliminationHalfLife(Double highEliminationHalfLife);

    /**
     * Unit of time for the half-life range of a drug.
     * 
     * @return the unit of time for the half-life range of a drug.
     */
    HalfLifeUnit getHalfLifeUnits();

    /**
     * Unit of time for the half-life range of a drug.
     * 
     * @param halfLifeUnits
     *            to set.
     */
    void setHalfLifeUnits(HalfLifeUnit halfLifeUnits);

    /**
     * Indicates the low end of a drugs frequency of administration per day.
     * 
     * @return the indicates the low end of a drugs frequency of administration per day.
     */
    Double getFrequencyLow();

    /**
     * Indicates the low end of a drugs frequency of administration per day.
     * 
     * @param frequencyLow
     *            to set.
     */
    void setFrequencyLow(Double frequencyLow);

    /**
     * indicates the high end of a drugs frequency of administration per day.
     * 
     * @return the indicates the high end of a drugs frequency of administration per day.
     */
    Double getFrequencyHigh();

    /**
     * Indicates the high end of a drugs frequency of administration per day.
     * 
     * @param frequencyHigh
     *            to set.
     */
    void setFrequencyHigh(Double frequencyHigh);

    /**
     * Indicates the lowest recommended duration of therapy (in days).
     * 
     * @return the indicates the lowest recommended duration of therapy (in days).
     */
    Long getDurationLow();

    /**
     * Indicates the lowest recommended duration of therapy (in days).
     * 
     * @param durationLow
     *            to set.
     */
    void setDurationLow(Long durationLow);

    /**
     * Indicates the highest recommended duration of therapy (in days).
     * 
     * @return the indicates the highest recommended duration of therapy (in days.
     */
    Long getDurationHigh();

    /**
     * Indicates the highest recommended duration of therapy (in days).
     * 
     * @param durationHigh
     *            to set.
     */
    void setDurationHigh(Long durationHigh);

    /**
     * Indicates the maximum recommended duration of therapy (in days).
     * 
     * @return the indicates the maximum recommended duration of therapy (in days).
     */
    Long getMaxDuration();

    /**
     * Indicates the maximum recommended duration of therapy (in days).
     * 
     * @param maxDuration
     *            to set.
     */
    void setMaxDuration(Long maxDuration);

    /**
     * Boolean (0/1) indicating whether dosing needs to be adjusted for hepatic impairment.
     * 
     * @return the boolean (0/1) indicating whether dosing needs to be adjusted for hepatic impairment.
     */
    Long getHepaticImpairmentIndicator();

    /**
     * Boolean (0/1) indicating whether dosing needs to be adjusted for hepatic impairment.
     * 
     * @param hepaticImpairmentIndicator
     *            to set.
     */
    void setHepaticImpairmentIndicator(Long hepaticImpairmentIndicator);

    /**
     * Boolean (0/1) indicating whether dosing needs to be adjusted for renal impairment.
     * 
     * @return the boolean (0/1) indicating whether dosing needs to be adjusted for renal impairment.
     */
    Long getRenalImpairmentIndicator();

    /**
     * Boolean (0/1) indicating whether dosing needs to be adjusted for renal impairment.
     * 
     * @param renalImpairmentIndicator
     *            to set.
     */
    void setRenalImpairmentIndicator(Long renalImpairmentIndicator);

    /**
     * Boolean (0/1) indicating whether weight is required for dosing.
     * 
     * @return the Boolean (0/1) indicating whether weight is required for dosing.
     */
    Long getWeightImpairmentIndicator();

    /**
     * Boolean (0/1) indicating whether weight is required for dosing.
     * 
     * @param weightImpairmentIndicator
     *            to set.
     * 
     */
    void setWeightImpairmentIndicator(Long weightImpairmentIndicator);

    /**
     * Boolean (0/1) indicating whether body surface area is required for dosing.
     * 
     * @return the boolean (0/1) indicating whether body surface area is required for dosing.
     */
    Long getBsaRequiredIndicator();

    /**
     * Boolean (0/1) indicating whether body surface area is required for dosing.
     * 
     * @param bsaRequiredIndicator
     *            to set.
     * 
     */
    void setBsaRequiredIndicator(Long bsaRequiredIndicator);

    /**
     * Warning code.
     * 
     * @return the warning code.
     */
    String getWarningCode();

    /**
     * Warning code.
     * 
     * @param warningCode
     *            to set.
     */
    void setWarningCode(String warningCode);

    /**
     * Minimum amount to be administered per dose rate (hours or minutes).
     * 
     * @return the Minimum amount to be administered per dose rate (hours or minutes).
     */
    Double getDoseRateLow();

    /**
     * Minimum amount to be administered per dose rate (hours or minutes).
     * 
     * @param doseRateLow
     *            to set.
     */
    void setDoseRateLow(Double doseRateLow);

    /**
     * Highest amount to be administered per dose rate (hours or minutes).
     * 
     * @return the highest amount to be administered per dose rate (hours or minutes).
     */
    Double getDoseRateHigh();

    /**
     * Highest amount to be administered per dose rate (hours or minutes).
     * 
     * @param doseRateHigh
     *            to set.
     */
    void setDoseRateHigh(Double doseRateHigh);

    /**
     * Low dose for a given dose form rate (hours or minutes).
     * 
     * @return the low dose for a given dose form rate (hours or minutes).
     */
    Double getDoseFormRateLow();

    /**
     * Low dose for a given dose form rate (hours or minutes).
     * 
     * @param doseFormRateLow
     *            to set.
     */
    void setDoseFormRateLow(Double doseFormRateLow);

    /**
     * High dose for a given dose form rate (hours or minutes).
     * 
     * @return the high dose for a given dose form rate (hours or minutes).
     */
    Double getDoseFormRateHigh();

    /**
     * High dose for a given dose form rate (hours or minutes).
     * 
     * @param doseFormRateHigh
     *            to set.
     */
    void setDoseFormRateHigh(Double doseFormRateHigh);

    /**
     * Maximum amount to be administered in a single dose rate (hours or minutes).
     * 
     * @return the maximum amount to be administered in a single dose rate (hours or minutes).
     */
    Double getMaxSingleDoseRate();

    /**
     * Maximum amount to be administered in a single dose rate (hours or minutes).
     * 
     * @param maxSingleDoseRate
     *            to set.
     */
    void setMaxSingleDoseRate(Double maxSingleDoseRate);

    /**
     * Maximum single dose for a given dose form rate (hours or minutes).
     * 
     * @return the maximum single dose for a given dose form rate (hours or minutes).
     */
    Double getMaxSingleDoseFormRate();

    /**
     * Maximum single dose for a given dose form rate (hours or minutes).
     * 
     * @param maxSingleDoseFormRate
     *            to set.
     */
    void setMaxSingleDoseFormRate(Double maxSingleDoseFormRate);

    /**
     * Maximum amount to be administered per dose rate (hours or minutes).
     * 
     * @return the maximum amount to be administered per dose rate (hours or minutes).
     */
    Double getMaxDailyDoseRate();

    /**
     * Maximum amount to be administered per dose rate (hours or minutes).
     * 
     * @param maxDailyDoseRate
     *            to set.
     */
    void setMaxDailyDoseRate(Double maxDailyDoseRate);

    /**
     * Maximum single dose for a given dose form rate (hours or minutes).
     * 
     * @return the maximum single dose for a given dose form rate (hours or minutes).
     */
    Double getMaxDailyDoseFormRate();

    /**
     * Maximum single dose for a given dose form rate (hours or minutes).
     * 
     * @param maxDailyDoseFormRate
     *            to set.
     */
    void setMaxDailyDoseFormRate(Double maxDailyDoseFormRate);

    /**
     * DXID type code to identify a Medical Condition.
     * 
     * @return the DXID type code to identify a Medical Condition.
     */
    String getDxid();

    /**
     * DXID type code to identify a Medical Condition.
     * 
     * @param dxid
     *            to set.
     */
    void setDxid(String dxid);

    /**
     * Number indicating lowest creatinine clearance to which dosing applies.
     * 
     * @return the number indicating lowest creatinine clearance to which dosing applies.
     */
    Long getCrclThreshold();

    /**
     * Number indication lowest creatinine clearance to which dosing applies.
     * 
     * @param crclThreshold
     *            to set.
     */
    void setCrclThreshold(Long crclThreshold);

    /**
     * Lowest patient age in days to which dosing information applies.
     * 
     * @return the lowest patient age in days to which dosing information applies.
     */
    Long getAgeLowInDays();

    /**
     * Lowest patient age in days to which dosing information applies.
     * 
     * @param ageLowInDays
     *            to set.
     */
    void setAgeLowInDays(Long ageLowInDays);

    /**
     * Highest patient age in days to which dosing information applies.
     * 
     * @return the highest patient age in days to which dosing information applies.
     */
    Long getAgeHighInDays();

    /**
     * Highest patient age in days to which dosing information applies.
     * 
     * @param ageHighInDays
     *            to set.
     */
    void setAgeHighInDays(Long ageHighInDays);

    /**
     * Unit of measure for low dose per day.
     * 
     * @return the unit of measure for low dose per day.
     */
    String getDoseLowUnits();

    /**
     * Unit of measure for low dose per day.
     * 
     * @param doseLowUnits
     *            to set.
     */
    void setDoseLowUnits(String doseLowUnits);

    /**
     * Unit of measure for high dose per day.
     * 
     * @return the unit of measure for high dose per day.
     */
    String getDoseHighUnits();

    /**
     * Unit of measure for high dose per day.
     * 
     * @param doseHighUnits
     *            to set.
     */
    void setDoseHighUnits(String doseHighUnits);

    /**
     * Unit of measure for the dose form (EA/KG/DAY).
     * 
     * @return the unit of measure for the dose form (EA/KG/DAY).
     */
    String getDoseFormLowUnits();

    /**
     * Unit of measure for the dose form (EA/KG/DAY).
     * 
     * @param doseFormLowUnits
     *            to set.
     */
    void setDoseFormLowUnits(String doseFormLowUnits);

    /**
     * Unit of measure for the dose form (EA/KG/DAY).
     * 
     * @return the unit of measure for the dose form (EA/KG/DAY).
     */
    String getDoseFormHighUnits();

    /**
     * Unit of measure for the dose form (EA/KG/DAY).
     * 
     * @param doseFormHighUnits
     *            to set.
     */
    void setDoseFormHighUnits(String doseFormHighUnits);

    /**
     * Unit of measure for the maximum single dose.
     * 
     * @return the Unit of measure for the maximum single dose.
     */
    String getMaxSingleDoseUnits();

    /**
     * Unit of measure for the maximum single dose.
     * 
     * @param maxSingleDoseUnits
     *            to set.
     */
    void setMaxSingleDoseUnits(String maxSingleDoseUnits);

    /**
     * Unit of measure for the dose form (EA/KG/DAY).
     * 
     * @return the unit of measure for the dose form (EA/KG/DAY).
     */
    String getMaxSingleDoseFormUnits();

    /**
     * Unit of measure for the dose form (EA/KG/DAY).
     * 
     * @param maxSingleDoseFormUnits
     *            to set.
     */
    void setMaxSingleDoseFormUnits(String maxSingleDoseFormUnits);

    /**
     * Unit of measure for the maximum daily dose.
     * 
     * @return the unit of measure for the maximum daily dose.
     */
    String getMaxDailyDoseUnits();

    /**
     * Unit of measure for the maximum daily dose.
     * 
     * @param maxDailyDoseUnits
     *            to set.
     */
    void setMaxDailyDoseUnits(String maxDailyDoseUnits);

    /**
     * Unit of measure for the dose form (EA/KG/DAY).
     * 
     * @return the unit of measure for the dose form (EA/KG/DAY).
     */
    String getMaxDailyDoseFormUnits();

    /**
     * Unit of measure for the dose form (EA/KG/DAY).
     * 
     * @param maxDailyDoseFormUnits
     *            to set.
     */
    void setMaxDailyDoseFormUnits(String maxDailyDoseFormUnits);

    /**
     * Unit of measure for maximum lifetime dose.
     * 
     * @return the unit of measure for maximum lifetime dose.
     */
    String getMaxLifeTimeDoseUnits();

    /**
     * Unit of measure for maximum lifetime dose.
     * 
     * @param maxLifeTimeDoseUnits
     *            to set.
     */
    void setMaxLifeTimeDoseUnits(String maxLifeTimeDoseUnits);

    /**
     * Unit of measure for the dose form (EA/KG/DAY).
     * 
     * @return the unit of measure for the dose form (EA/KG/DAY).
     */
    String getMaxLifeTimeDoseFormUnits();

    /**
     * Unit of measure for the dose form (EA/KG/DAY).
     * 
     * @param maxLifeTimeDoseFormUnits
     *            to set.
     */
    void setMaxLifeTimeDoseFormUnits(String maxLifeTimeDoseFormUnits);

    /**
     * Unit of measure for the creatinine clearance threshold.
     * 
     * @return the unit of measure for the creatinine clearance threshold.
     */
    String getCrclThresholdUnits();

    /**
     * Unit of measure for the creatinine clearance threshold.
     * 
     * @param crclThreshholdUnits
     *            to set.
     */
    void setCrclThresholdUnits(String crclThreshholdUnits);

    /**
     * Unit of measure for low dose rate (hours or minutes).
     * 
     * @return the unit of measure for low dose rate (hours or minutes).
     */
    String getDoseRateLowUnits();

    /**
     * Unit of measure for low dose rate (hours or minutes).
     * 
     * @param doseRateLowUnits
     *            to set.
     */
    void setDoseRateLowUnits(String doseRateLowUnits);

    /**
     * Unit of measure for high dose rate (hours or minutes).
     * 
     * @return the unit of measure for high dose rate (hours or minutes).
     */
    String getDoseRateHighUnits();

    /**
     * Unit of measure for high dose rate (hours or minutes).
     * 
     * @param doseRateHighUnits
     *            to set.
     */
    void setDoseRateHighUnits(String doseRateHighUnits);

    /**
     * Unit of measure for the dose form rate (hours or minutes).
     * 
     * @return the Unit of measure for the dose form rate (hours or minutes).
     */
    String getDoseFormRateLowUnits();

    /**
     * Unit of measure for the dose form rate (hours or minutes).
     * 
     * @param doseFormRateLowUnits
     *            to set.
     */
    void setDoseFormRateLowUnits(String doseFormRateLowUnits);

    /**
     * Unit of measure for the dose form rate (hours or minutes).
     * 
     * @return the unit of measure for the dose form rate (hours or minutes).
     */
    String getDoseFormRateHighUnits();

    /**
     * Unit of measure for the dose form rate (hours or minutes).
     * 
     * @param doseFormRateHighUnits
     *            to set.
     */
    void setDoseFormRateHighUnits(String doseFormRateHighUnits);

    /**
     * Unit of measure for the maximum single dose rate (hours or minutes).
     * 
     * @return the unit of measure for the maximum single dose rate (hours or minutes).
     */
    String getMaxSingleDoseRateUnits();

    /**
     * Unit of measure for the maximum single dose rate (hours or minutes).
     * 
     * @param maxSingleDoseRateUnits
     *            to set.
     */
    void setMaxSingleDoseRateUnits(String maxSingleDoseRateUnits);

    /**
     * Unit of measure for the dose form rate (hours or minutes).
     * 
     * @return the Unit of measure for the dose form rate (hours or minutes).
     */
    String getMaxSingleDoseFormRateUnits();

    /**
     * Unit of measure for the dose form rate (hours or minutes).
     * 
     * @param maxSingleDoseFormRateUnits
     *            to set.
     */
    void setMaxSingleDoseFormRateUnits(String maxSingleDoseFormRateUnits);

    /**
     * Unit of measure for the maximum daily dose rate (hours or minutes).
     * 
     * @return the unit of measure for the maximum daily dose rate (hours or minutes).
     */
    String getMaxDailyDoseRateUnits();

    /**
     * Unit of measure for the maximum daily dose rate (hours or minutes).
     * 
     * @param maxDailyDoseRateUnits
     *            to set.
     */
    void setMaxDailyDoseRateUnits(String maxDailyDoseRateUnits);

    /**
     * Unit of measure for the dose form rate (hours or minutes).
     * 
     * @return the unit of measure for the dose form rate (hours or minutes).
     */
    String getMaxDailyDoseFormRateUnits();

    /**
     * Unit of measure for the dose form rate (hours or minutes).
     * 
     * @param maxDailyDoseFormRateUnits
     *            to set.
     */
    void setMaxDailyDoseFormRateUnits(String maxDailyDoseFormRateUnits);

    /**
     * Gets the max single nte dose.
     * 
     * @return the max single nte dose
     */
    Double getMaxSingleNTEDose();

    /**
     * Gets the max single nte dose unit.
     * 
     * @return the max single nte dose unit
     */
    String getMaxSingleNTEDoseUnit();

    /**
     * Gets the max single nte dose form.
     * 
     * @return the max single nte dose form
     */
    Double getMaxSingleNTEDoseForm();

    /**
     * Gets the max single nte dose form unit.
     * 
     * @return the max single nte dose form unit
     */
    String getMaxSingleNTEDoseFormUnit();

    /**
     * Sets the max single nte dose.
     * 
     * @param maxSingleNTEDose
     *            the new max single nte dose
     */
    void setMaxSingleNTEDose(Double maxSingleNTEDose);

    /**
     * Sets the max single nte dose unit.
     * 
     * @param maxSingleNTEDoseUnit
     *            the new max single nte dose unit
     */
    void setMaxSingleNTEDoseUnit(String maxSingleNTEDoseUnit);

    /**
     * Sets the max single nte dose form.
     * 
     * @param maxSingleNTEDoseForm
     *            the new max single nte dose form
     */
    void setMaxSingleNTEDoseForm(Double maxSingleNTEDoseForm);

    /**
     * Sets the max single nte dose form unit.
     * 
     * @param maxSingleNTEDoseFormUnit
     *            the new max single nte dose form unit
     */
    void setMaxSingleNTEDoseFormUnit(String maxSingleNTEDoseFormUnit);

}
