package gov.va.med.pharmacy.pecs.domain;

import java.util.List;

/**
 * A section in a monograph describes a specific aspect, for example, the
 * mechanism of action, or clinical effects.
 */
public interface MonographSection {
    /**
     * Format the text of this section for display.
     * 
     * @return the formatted text
     */
    String formatText();

    /**
     * Split the formatted text for this section into lines.
     * 
     * @param lineSize
     *            the maximum line size
     * @return a list of lines
     */
    public List<SectionLine> splitText(int lineSize);

    /**
     * Get the raw text of this section.
     * 
     * @return the formatted text
     */
    String getText();

    /**
     * Set this section's text.
     * 
     * @param text
     *            the section text
     */
    void setText(String text);

    /**
     * The section code indicates what this section describes (e.g. mechanism of
     * action, side effects etc).
     * 
     * @return the code
     */
    SectionCode getSectionCode();

    /**An enumeration of section codes handled by PECS.
     *
     */
    enum SectionCode {
        
        /**Variables.*/
        TITLE("T"), MECHANISM_OF_ACTION("A"), BLANK("B"), DISCUSSION("D"), CLINICAL_EFFECTS("E"), SEVERITY_LEVEL("L"),
        /**Variables.*/
        PATIENT_MANAGEMENT("M"), PREDISPOSING_FACTORS("P"), REFERENCE("R"), DISCLAIMER("Z");

        private String code;

        /**
         * Get a section code by its character code.
         * 
         * @param val
         *            the character code
         * @return the section code
         */
        public static SectionCode getByValue(String val) {
            for (SectionCode s : values()) {
                if (s.code.equals(val)) {
                    return s;
                }
            }
            throw new IllegalArgumentException("Invalid section code: <" + val + ">");
        }
        /**Section Code.
         * 
         * @param c - String
         */
        private SectionCode(String c) {
            code = c;
        }
        /**Get Code.
         * 
         * @return code
         */
        public String getCode() {
            return code;
        }
        /**To Sting.
         * @return code
         */
        public String toString() {
            return code;
        }
    }
}
