package gov.va.med.pharmacy.pecs.domain;

import gov.va.med.pharmacy.pecs.state.Status;

import java.util.Date;

/**
 * A version records the state of a customization request at a given point in time. The version history is maintained for audit
 * purposes.
 * 
 * @param <T>
 *            Version
 */
public interface Version<T> {
    /**
     * The customization request whose state this instance captured.
     * 
     * @return the versioned object
     */
    T getVersionedObject();

    /**
     * A comment meant to document why this version was created.
     * 
     * @return the comment text
     */
    String getComment();

    /**
     * Set Comment.
     * 
     * @param comment
     *            - String
     */
    void setComment(String comment);

    /**
     * The date when this version was created.
     * 
     * @return the creation date
     */
    Date getDate();

    /**
     * Set Date.
     * 
     * @param date
     *            Date
     */
    void setDate(Date date);

    /**
     * The user assigned to the owning versioned object at the time when this version was current.
     * 
     * @return the user id
     */
    String getAssignedUser();

    /**
     * Set Assigned User.
     * 
     * @param user
     *            - String
     */
    void setAssignedUser(String user);

    /**
     * The user who created this version.
     * 
     * @return the user id
     */
    String getUser();

    /**
     * Set User.
     * 
     * @param user
     *            - String
     */
    void setUser(String user);

    /**
     * The user who last performed a status-changing operation.
     * 
     * @return the user id
     */
    String getActionBy();

    /**
     * Set ActionBy.
     * 
     * @param user
     *            - String
     */
    void setActionBy(String user);

    /**
     * The status of the customization request when the version was created.
     * 
     * @return the status id
     */
    Status getStatus();

    /**
     * Set Status.
     * 
     * @param status
     *            Status
     */
    void setStatus(Status status);

    /**
     * Is this the current version of the customization request?
     * 
     * @return true if it's the current version (latest), false otherwise
     */
    boolean isCurrent();

    /**
     * Is this version of the customization request modifiable?
     * 
     * @return true if the version is modifiable, false otherwise
     */
    boolean isModifiable();

    /**
     * Format the comments obtained from the version history of the given concept, up to the specified version.
     * 
     * concept the customized record for which the comment history is desired version the version up to which we want comments
     * for
     * 
     * @return the formatted comment history (descending chronological order)
     */
    String getCommentHistory();

    /**
     * Gets the export date.
     * 
     * @return the export date
     */
    Date getExportDate();

    /**
     * Sets the export date.
     * 
     * @param exportDate
     *            the new export date
     */
    void setExportDate(Date exportDate);

}
