package gov.va.med.pharmacy.pecs.domain.impl.va;

import gov.va.med.pharmacy.pecs.domain.Customizable;
import gov.va.med.pharmacy.pecs.domain.UserComment;
import gov.va.med.pharmacy.pecs.domain.Version;
import gov.va.med.pharmacy.pecs.state.Status;

import java.util.Date;
import java.util.HashSet;
import java.util.Set;

/**
 * The Class AbstractCustomizable.
 * 
 * @param <T>
 *            the generic type
 */
public abstract class AbstractCustomizable<T> implements Customizable<T> {

    /** The requestor. */
    private String requestor;
    // non-persistent field used to indicate a roll-back was performed
    /** The rolled back. */
    private boolean rolledBack;

    /**
     * Gets the modifiable current version.
     * 
     * @return the modifiable current version
     */
    protected abstract Version<T> getModifiableCurrentVersion();

    /**
     * Sets the exportable version.
     * 
     * @param v
     *            the new exportable version
     */
    protected abstract void setExportableVersion(Version<T> v);

    /**
     * Sets the exported version.
     * 
     * @param v
     *            the new exported version
     */
    public abstract void setExportedVersion(Version<T> v);

    /** The concept xref. */
    protected ConceptXref conceptXref;

    /** The fdb reference id. */
    private Long fdbReferenceId;

    /** The export date. */
    private Date exportDate;

    /**
     * Gets the fdb reference id.
     * 
     * @return the fdb reference id
     */
    public Long getFdbReferenceId() {
        return fdbReferenceId;
    }

    /**
     * Sets the fdb reference id.
     * 
     * @param fdbReferenceId
     *            the new fdb reference id
     */
    public void setFdbReferenceId(Long fdbReferenceId) {
        this.fdbReferenceId = fdbReferenceId;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.domain.Customizable#getActionBy()
     */
    @Override
    public String getActionBy() {
        return getCurrentVersion().getActionBy();
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.domain.Customizable#setActionBy(java.lang.String)
     */
    @Override
    public void setActionBy(String actionUser) {
        getModifiableCurrentVersion().setActionBy(actionUser);
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.domain.Customizable#getAuditUser()
     */
    @Override
    public String getAuditUser() {
        return getCurrentVersion().getUser();
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.domain.Customizable#setAuditUser(java.lang.String)
     */
    @Override
    public void setAuditUser(String actionUser) {
        getModifiableCurrentVersion().setUser(actionUser);
        // if this is a new customization request,
        // record the user as the requestor
        if (getVersionHistory().size() == 1) {
            setRequestor(actionUser);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.domain.Customizable#getRequestor()
     */
    @Override
    public String getRequestor() {
        return requestor;
    }

    /**
     * Sets the requestor.
     * 
     * @param requestor
     *            the new requestor
     */
    protected void setRequestor(String requestor) {
        this.requestor = requestor;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.domain.Customizable#getAssignedUser()
     */
    @Override
    public String getAssignedUser() {
        return getCurrentVersion().getAssignedUser();
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.domain.Customizable#setAssignedUser(java.lang.String)
     */
    @Override
    public void setAssignedUser(String assignedUser) {
        getModifiableCurrentVersion().setAssignedUser(assignedUser);
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.domain.Customizable#getComment()
     */
    @Override
    public String getComment() {
        return getCurrentVersion().getComment();
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.domain.Customizable#setComment(java.lang.String)
     */
    @Override
    public void setComment(String reason) {
        getModifiableCurrentVersion().setComment(reason);
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.domain.Customizable#getEffectiveDate()
     */
    @Override
    public Date getEffectiveDate() {
        Version<T> v = getExportableVersion();
        return v == null ? null : v.getDate();
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.domain.Customizable#getModifiedDate()
     */
    @Override
    public Date getModifiedDate() {
        return getCurrentVersion().getDate();
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.domain.Customizable#setModifiedDate(java.util.Date)
     */
    @Override
    public void setModifiedDate(Date date) {
        getModifiableCurrentVersion().setDate(date);
    }

    // @Override
    // public String getReferenceText() {
    // return getCurrentVersion().getReferenceText();
    // }
    //
    // @Override
    // public void setReferenceText(String referenceText) {
    // getModifiableCurrentVersion().setReferenceText(referenceText);
    // }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.domain.Customizable#getStatus()
     */
    @Override
    public Status getStatus() {
        return getCurrentVersion().getStatus();
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.domain.Customizable#setStatus(gov.va.med.pharmacy.pecs.state.Status)
     */
    @Override
    public void setStatus(Status status) {
        getModifiableCurrentVersion().setStatus(status);
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.domain.Customizable#setExportableVersion()
     */
    @Override
    public void setExportableVersion() {
        setExportableVersion(getCurrentVersion());
    }

    /**
     * Was rolled back.
     * 
     * @return true, if successful
     */
    public boolean wasRolledBack() {
        return rolledBack;
    }

    /**
     * Sets the rolled back.
     * 
     * @param flag
     *            the new rolled back
     */
    protected void setRolledBack(boolean flag) {
        rolledBack = flag;
    }

    /**
     * Creates the concept xref.
     * 
     * @return the concept xref
     */
    protected abstract ConceptXref createConceptXref();

    /**
     * Gets the concept xref.
     * 
     * @return the concept xref
     */
    public ConceptXref getConceptXref() {
        return conceptXref;
    }

    /**
     * Sets the concept xref.
     * @param conceptXref
     *     the concept Xref.
     */
    public void setConceptXref(ConceptXref conceptXref) {
        this.conceptXref = conceptXref;
    }

    /**
     * Gets the user comment history.
     * 
     * @return the user comment history
     */
    public Set<UserComment> getUserCommentHistory() {
        if (getConceptXref() == null) {
            return new HashSet<UserComment>(0);
        }

        return getConceptXref().getUserCommentHistory();
    }

    /**
     * Sets the user comment history.
     * 
     * @param userCommentHistory
     *            the new comment history
     */
    public void setUserCommentHistory(Set<UserComment> userCommentHistory) {
        getConceptXref().setUserCommentHistory(userCommentHistory);
    }

    /**
     * Adds the user comment.
     * 
     * @param userComment
     *            the user comment
     */
    public void addUserComment(UserComment userComment) {
        getConceptXref().addUserComment(userComment);
    }

}
