package gov.va.med.pharmacy.pecs.domain.impl.va;

import gov.va.med.pharmacy.pecs.domain.Customizable;
import gov.va.med.pharmacy.pecs.domain.Version;
import gov.va.med.pharmacy.pecs.state.Status;
import gov.va.med.pharmacy.pecs.util.PecsDateFormat;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.ListIterator;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.time.DateFormatUtils;

/**
 * The Class AbstractVersion.
 * 
 * @param <T>
 *            the generic type
 */
public abstract class AbstractVersion<T> implements Version<T>, ModifiableEntity {

    /** The df. */
    private final SimpleDateFormat df = new SimpleDateFormat("yyyy/MM/dd kk:mm:ss");

    /** The version id. */
    private Long versionId = 0L;

    /** The comment. */
    private String comment;

    /** The date. */
    private Date date;

    /** The assigned user. */
    private String assignedUser;

    /** The action by. */
    private String actionBy;

    /** The user. */
    private String user;

    /** The status. */
    private Status status;

    /** The modifiable. */
    private boolean modifiable = true;

    /** The export Date */
    private Date exportDate;

    /**
     * Get the Export Date.
     * 
     * @return the exportDate
     */
    @Override
    public Date getExportDate() {
        return exportDate;
    }

    /**
     * Get the export date of the the record formatted in the <code>PecsDateFormat.PECS_DEFAULT_DISPLAY_TIMEZONE</code>.
     * 
     * @return the exportDate represented in the <code>PecsDateFormat.PECS_DEFAULT_DISPLAY_TIMEZONE</code>
     */
    public String getExportDateDisplayString() {
        if(getExportDate() == null){
            return null;
        }
        return DateFormatUtils.format(getExportDate(),PecsDateFormat.PECS_DATE_TIME_FORMAT, 
                PecsDateFormat.PECS_DEFAULT_DISPLAY_TIMEZONE);
    }
    
    /**
     * Set the Export Date.
     * 
     * @param exportDate
     *            the exportDate to set
     */
    @Override
    public void setExportDate(Date exportDate) {
        this.exportDate = exportDate;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.domain.Version#getVersionedObject()
     */
    @Override
    public abstract T getVersionedObject();

    // public abstract Version<T> getCurrentVersion();

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.domain.Version#isCurrent()
     */
    @SuppressWarnings("unchecked")
    @Override
    public boolean isCurrent() {
        // return (getCustomizableConcept().getCurrentVersion() == this;
        return ((Customizable<T>) getVersionedObject()).getCurrentVersion() == this;
    }

    /**
     * Gets the version id.
     * 
     * @return the version id
     */
    public Long getVersionId() {
        return versionId;
    }

    /**
     * Sets the version id.
     * 
     * @param versionId
     *            the new version id
     */
    public void setVersionId(Long versionId) {
        this.versionId = versionId;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.domain.Version#getComment()
     */
    @Override
    public String getComment() {
        if (comment != null && comment.length() > 22) {
            String possibleDate = comment.substring(0, df.toPattern().length());
            try {
                // does the comment really start with a date?
                df.parse(possibleDate);
                // if we got here, this is probably an imported comment
                String label = getCommentLabel(this);
                // was it created for this version?
                if (comment.startsWith(label)) {
                    // now extract just this version's comment text
                    // and ignore other comments that may be appended
                    int startIndex = label.length();
                    int endIndex = comment.indexOf('\r');
                    return endIndex > 0 ? comment.substring(startIndex, endIndex) : comment.substring(startIndex);
                } else {
                    return null;
                }
            } catch (ParseException e) {
                // the comment doesn't start with a date,
                // so it's not an imported one
            } catch (NumberFormatException e) {

            }
        }
        return comment;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.domain.Version#setComment(java.lang.String)
     */
    @Override
    public void setComment(String comment) {
        this.comment = comment;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.domain.Version#getDate()
     */
    @Override
    public Date getDate() {
        return date;
    }

    /**
     * Get the date of the the record formatted in the <code>PecsDateFormat.PECS_DEFAULT_DISPLAY_TIMEZONE</code>.
     * 
     * @return the date represented in the <code>PecsDateFormat.PECS_DEFAULT_DISPLAY_TIMEZONE</code>
     */
    public String getDateDisplayString() {
        if(getDate() == null){
            return null;
        }
        return DateFormatUtils.format(getDate(),PecsDateFormat.PECS_DATE_TIME_FORMAT, 
                PecsDateFormat.PECS_DEFAULT_DISPLAY_TIMEZONE);
    }
    
    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.domain.Version#setDate(java.util.Date)
     */
    @Override
    public void setDate(Date date) {
        this.date = date;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.domain.Version#getAssignedUser()
     */
    @Override
    public String getAssignedUser() {
        return assignedUser;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.domain.Version#setAssignedUser(java.lang.String)
     */
    @Override
    public void setAssignedUser(String assignedUser) {
        this.assignedUser = assignedUser;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.domain.Version#getUser()
     */
    @Override
    public String getUser() {
        return user;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.domain.Version#setUser(java.lang.String)
     */
    @Override
    public void setUser(String user) {
        this.user = user;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.domain.Version#getStatus()
     */
    @Override
    public Status getStatus() {
        return status;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.domain.Version#setStatus(gov.va.med.pharmacy.pecs.state.Status)
     */
    @Override
    public void setStatus(Status status) {
        this.status = status;
    }

    // public String getReferenceText() {
    // return referenceText;
    // }
    // public void setReferenceText(String referenceText) {
    // this.referenceText = referenceText;
    // }

    /**
     * Checks if this version is modifiable. The only time when a version is modifiable is after it has been created and while
     * it is being populated. Once it has been persisted, it cannot be changed anymore. This implementation relies on the fact
     * that persistent ids are assigned only when objects are persisted.
     * 
     * @return true, if is modifiable
     */
    @Override
    public boolean isModifiable() {
        return modifiable;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.domain.impl.va.ModifiableEntity#setModifiable(boolean)
     */
    @Override
    public void setModifiable(boolean modifiable) {
        this.modifiable = modifiable;
    }

    /**
     * A method to copy data from one AbstractVersion to another. This method should be overridden and executed to copy data
     * specific to a Versioned object.
     * 
     * @param other
     *            the other
     */
    public void copy(Version<T> other) {
        // this.comment = other.getComment();
        date = other.getDate();
        assignedUser = other.getAssignedUser();
        actionBy = other.getActionBy();
        user = other.getUser();
        status = other.getStatus();
    }

    /**
     * Format the comments obtained from the version history of the given concept, up to the specified version.
     * 
     * @return the formatted comment history (descending chronological order)
     */
    @SuppressWarnings("unchecked")
    @Override
    public String getCommentHistory() {
        StringBuffer commentBuffer = new StringBuffer();
        // Customizable<T> concept = getCustomizableConcept();
        Customizable<T> concept = (Customizable<T>) getVersionedObject();
        int index = concept.getVersionHistory().indexOf(this);
        for (ListIterator<Version<T>> iterator = concept.getVersionHistory().listIterator(index + 1); iterator.hasPrevious();) {
            // for (Version<T> v : concept.getVersionHistory()) {
            Version<T> version = iterator.previous();
            if (StringUtils.isBlank(version.getComment())) {
                continue;
            }
            commentBuffer.append(getCommentLabel((AbstractVersion<T>)version));
            commentBuffer.append(version.getComment());
            commentBuffer.append("\n");
            // if (v.equals(version)) {
            // break;
            // }
        }
        return commentBuffer.toString();
    }

    /**
     * Builds the comment label, which contains the standardized date and user.
     * 
     * @param version
     *            the version from which the comment label should be built
     * @return the comment label
     */
    private String getCommentLabel(AbstractVersion<T> version) {
        StringBuffer labelBuffer = new StringBuffer(version.getDateDisplayString());
        labelBuffer.append(" ");
        labelBuffer.append(version.getUser());
        labelBuffer.append(": ");
        return labelBuffer.toString();
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.domain.Version#setActionBy(java.lang.String)
     */
    @Override
    public void setActionBy(String actionUser) {
        this.actionBy = actionUser;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.domain.Version#getActionBy()
     */
    @Override
    public String getActionBy() {
        return actionBy;
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Object#toString()
     */
    @Override
    public String toString() {
        return "AbstractVersion [getExportDate()=" + getExportDate() + ", isCurrent()=" + isCurrent() + ", getVersionId()="
                + getVersionId() + ", getComment()=" + getComment() + ", getDate()=" + getDate() + ", getAssignedUser()="
                + getAssignedUser() + ", getUser()=" + getUser() + ", getStatus()=" + getStatus() + ", isModifiable()="
                + isModifiable() + ", getCommentHistory()=" + getCommentHistory() + ", getActionBy()=" + getActionBy() + "]";
    }

}
