package gov.va.med.pharmacy.pecs.domain.impl.va;

import gov.va.med.pharmacy.pecs.domain.MonographSection;
import gov.va.med.pharmacy.pecs.domain.SectionLine;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang.StringUtils;

/**
 * An implementation of a custom monograph section. This class is not mapped to the persistent store, instead instances are
 * synthesized on the fly, based on persistent section lines.
 * 
 */
public final class MonographSectionVa implements MonographSection, Serializable {
    private static final int DATA_LENGTH = 64;
    private static final int LINE = 72;
    /**
     * 
     */
    private static final long serialVersionUID = 1112790460363539960L;
    private static final String NOFORMAT_CODE = "0";
    private static final String NEWLINE_CODE = "1";
    private static final String NEWPARA_CODE = "2";
    // private static final String CRLF = "\r\n";
    private static final int CHUNK_SIZE = 4000;

    private final MonographSection.SectionCode sectionCode;
    private final MonographVersion version;
    private final List<MonographSectionLine> lines = new ArrayList<MonographSectionLine>();

    /**
     * Monograph Section Va Constructor.
     * 
     * @param code
     *            - MonographSection
     * @param version
     *            - MonographVersion
     */
    MonographSectionVa(MonographSection.SectionCode code, MonographVersion version) {
        this.sectionCode = code;
        this.version = version;
    }

    /**
     * Generate the original text of the section, assembled from the lines.
     * 
     * @return the section text
     */
    @Override
    public String getText() {
        StringBuffer buf = new StringBuffer();
        for (MonographSectionLine line : lines) {
            buf.append(line.getText());
        }
        return buf.length() == 0 ? null : buf.toString();
    }

    /**
     * Format the section text for display (this means for now prepend with section label).
     * 
     * @return the formatted section text
     */
    @Override
    public String formatText() {
        String text = getText();
        if (StringUtils.isBlank(text)) {
            return null;
        }
        String label = version.getSupportedSections().get(sectionCode);
        return label + text;
    }

    /**
     * Set the section's text.
     * 
     * @param text
     *            - String
     */
    @Override
    public void setText(String text) {
        version.removeLines(lines);
        lines.clear();
        for (String chunk : chunkText(text, CHUNK_SIZE)) {
            MonographSectionLine line = new MonographSectionLine();
            line.setSectionCode(sectionCode.getCode());
            line.setFormatCode(NOFORMAT_CODE);
            line.setText(chunk);
            lines.add(line);
        }
        version.addLines(lines);
    }

    @Override
    public MonographSection.SectionCode getSectionCode() {
        return sectionCode;
    }

    /**
     * Add Line to the Monograph Section.
     * 
     * @param line
     *            Monograph Section Line
     */
    void addLine(MonographSectionLine line) {
        if (!sectionCode.getCode().equals(line.getSectionCode())) {
            String msg = "Cannot add line with format code " + line.getSectionCode() + " to section with code " + sectionCode;
            throw new IllegalArgumentException(msg);
        }
        lines.add(line);
    }

    /**
     * Split the text in chunks of the given size. We don't care about splitting in the middle of a word, we don't care about
     * newlines...
     * 
     * @param text
     *            the text to split
     * @param chunkSize
     *            the chunk size
     * @return a list of text chunks
     */
    private List<String> chunkText(String text, int chunkSize) {
        ArrayList<String> res = new ArrayList<String>();
        if (!StringUtils.isBlank(text)) {
            int start = 0;
            while (text.length() - start > chunkSize) {
                res.add(text.substring(start, start + chunkSize));
                start += chunkSize;
            }
            res.add(text.substring(start));
        }
        return res;
    }

    /**
     * Split the given text along line boundaries, in chunks of the given maximum size. The text is first split in lines, then
     * each line is further divided into smaller pieces, if necessary.
     * <p>
     * The returned objects mimic FDB's way of representing sections.
     * 
     * @param lineSize
     *            the maximum size of a text chunk to be produced
     * @return a list of objects from which the original text (including new lines) can be reconstructed
     */

    @Override
    public List<SectionLine> splitText(/* String text, */int lineSize) {
        ArrayList<SectionLine> res = new ArrayList<SectionLine>();
        String text = formatText();
        if (StringUtils.isBlank(text)) {
            return res;
        }

        String[] sectionText = text.split("\n|\r\n|\n\r");
        boolean emptyLine = false;

        for (int i = 0; i < sectionText.length; i++) {
            String sText = sectionText[i];
            int origLen = sText.length();
            String formatCode = NEWLINE_CODE;
            if (emptyLine) {
                formatCode = NEWPARA_CODE;
            }
            emptyLine = false;

            if (origLen <= lineSize/* 72 */) {
                if (!"".equals(sText)) {
                    MonographSectionLine line = new MonographSectionLine();
                    line.setFormatCode(formatCode);
                    line.setSectionCode(sectionCode.getCode());
                    line.setText(sText);
                    res.add(line);
                } else if ((i + 1) < sectionText.length) {
                    emptyLine = true;
                    continue;
                }
            } else {
                String[] words = sText.split(" ");
                StringBuffer lineData = new StringBuffer(words[0]);
                for (int j = 1; j < words.length; j++) {
                    while (j < words.length && lineData.length() <= lineSize - (words[j].length() + 1)/* 64 */) {
                        lineData.append(" ").append(words[j++]);
                    }
                    MonographSectionLine line = new MonographSectionLine();
                    line.setFormatCode(formatCode);
                    line.setSectionCode(sectionCode.getCode());
                    line.setText(lineData.toString());
                    res.add(line);

                    j--;
                    lineData.setLength(0);
                    formatCode = NOFORMAT_CODE;
                }
            }
        }
        return res;
    }

    /**
     * Splits the content of a monograph section into lines that can fit in FDB's LINETEXT column.
     * 
     * @param text
     *            String
     * @return a list of lines
     */
    public static List<String> splitText1(String text) {
        ArrayList<String> sections = new ArrayList<String>();

        if (StringUtils.isBlank(text))
            return sections;

        String[] sectionText = text.split("\n|\r\n|\n\r");
        for (int i = 0; i < sectionText.length; i++) {
            String stext = sectionText[i];
            int origLen = stext.length();
            if (origLen <= LINE) {
                if (!stext.equals("")) {
                    sections.add(stext);
                } else /* if ((i+1) < sectionText.length) */{
                    sections.add("");
                    continue;
                }
            } else {
                String[] words = stext.split(" ");
                StringBuffer lineData = new StringBuffer(words[0]);
                for (int j = 1; j < words.length; j++) {
                    while (lineData.length() <= DATA_LENGTH && j < words.length) {
                        lineData.append(" ").append(words[j++]);
                    }
                    sections.add(lineData.toString());
                    j--;
                    lineData.delete(0, lineData.length());
                }
            }
        }

        return sections;
    }

}
