package gov.va.med.pharmacy.pecs.domain.impl.va;

import gov.va.med.pharmacy.pecs.domain.Monograph;
import gov.va.med.pharmacy.pecs.domain.MonographSection;
import gov.va.med.pharmacy.pecs.domain.MonographSection.SectionCode;
import gov.va.med.pharmacy.pecs.domain.UserComment;
import gov.va.med.pharmacy.pecs.domain.Version;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.log4j.Logger;

/**
 * A version of a monograph customization request.
 */
//@formatter:off 
public final class MonographVersion extends AbstractVersion<Monograph> implements 
Version<Monograph>, Monograph, java.io.Serializable {
//@formatter:on 
    /** The Constant serialVersionUID. */
    private static final long serialVersionUID = -2592477858808585008L;

    /** The Constant LOGGER. */
    private static final Logger LOGGER = Logger.getLogger(MonographVersion.class);

    /** The monograph. */
    private AbstractMonographVa monograph;

    /** The reference text. */
    private String referenceText;

    /** The title. */
    private String title;

    /** The lines. */
    private List<MonographSectionLine> lines = new ArrayList<MonographSectionLine>();

    /** The sections. */
    private Map<MonographSection.SectionCode, MonographSectionVa> sections =
            new HashMap<MonographSection.SectionCode, MonographSectionVa>();

    /**
     * Instantiates a new monograph version.
     */
    public MonographVersion() {
    }

    /**
     * Sets the versioned object.
     * 
     * @param abstractMonographVa
     *            the new versioned object
     */
    public void setVersionedObject(AbstractMonographVa abstractMonographVa) {
        this.monograph = abstractMonographVa;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.domain.impl.va.AbstractVersion#getVersionedObject()
     */
    @Override
    public AbstractMonographVa getVersionedObject() {
        return this.monograph;
    }

    /**
     * Gets the reference text.
     * 
     * @return the reference text
     */
    public String getReferenceText() {
        return referenceText;
    }

    /**
     * Sets the reference text.
     * 
     * @param referenceText
     *            the new reference text
     */
    public void setReferenceText(String referenceText) {
        this.referenceText = referenceText;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.domain.Monograph#getTitle()
     */
    @Override
    public String getTitle() {
        return title;
    }

    /**
     * Set the monograph's title.
     * 
     * @param title
     *            the title to set
     */
    public void setTitle(String title) {
        this.title = title;
        getSection(MonographSection.SectionCode.TITLE).setText(title);
    }

    /**
     * Sets the lines.
     * 
     * @param lines
     *            the new lines
     */
    public void setLines(List<MonographSectionLine> lines) {
        this.lines = lines;
        sections.clear();
        for (MonographSectionLine line : lines) {
            try {
                MonographSection.SectionCode code = MonographSection.SectionCode.getByValue(line.getSectionCode());
                MonographSectionVa section = sections.get(code);
                if (section == null) {
                    section = new MonographSectionVa(code, this);
                    sections.put(code, section);
                }
                section.addLine(line);
            } catch (IllegalArgumentException e) {
                String msg = "Error loading monograph section line " + line + "; ignoring";
                LOGGER.error(msg, e);
            }
        }
    }

    /**
     * Gets the lines.
     * 
     * @return the lines
     */
    public List<MonographSectionLine> getLines() {
        return lines;
    }

    /**
     * Gets the section.
     * 
     * @param code
     *            the code
     * @return the section
     */
    public MonographSection getSection(MonographSection.SectionCode code) {
        MonographSectionVa s = sections.get(code);
        if (s == null) {
            s = new MonographSectionVa(code, this);
            sections.put(code, s);
        }
        return s;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.domain.Monograph#getSections()
     */
    @Override
    public Set<MonographSection> getSections() {
        return new HashSet<MonographSection>(sections.values());
    }

    /*
     * BACKUP public void copy(MonographVersion other) { super.copy(other); monograph = other.getVersionedObject(); // comment =
     * other.getComment(); // date = new Date(); // user = other.getUser(); // assignedUser = other.getAssignedUser(); // status
     * = other.getStatus(); title = other.getTitle(); // make a new collection with the old version's lines // and set it on the
     * clone through the setter (to make // sure sections are created) sections = new HashMap<MonographSection.SectionCode,
     * MonographSectionVa>(); setLines(new ArrayList<MonographSectionLine>(other.getLines())); referenceText =
     * other.getReferenceText(); }
     *//* BACKUP */

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.domain.impl.va.AbstractVersion#copy(gov.va.med.pharmacy.pecs.domain.Version)
     */
    @Override
    public void copy(Version<Monograph> other) {
        super.copy(other);
        MonographVersion mv = (MonographVersion) other;
        monograph = mv.getVersionedObject();
        title = mv.getTitle();
        // make a new collection with the old version's lines
        // and set it on the clone through the setter (to make
        // sure sections are created)
        sections = new HashMap<MonographSection.SectionCode, MonographSectionVa>();
        setLines(new ArrayList<MonographSectionLine>(mv.getLines()));
        referenceText = mv.getReferenceText();
    }

    /**
     * Checks if this version is modifiable. The only time when a version is modifiable is after it has been created and while
     * it is being populated. Once it has been persisted, it cannot be changed anymore. This implementation relies on the fact
     * that persistent ids are assigned only when objects are persisted.
     * 
     * @param someLines
     *            the some lines
     */
    // public boolean isModifiable() {
    // return versionId == 0;
    // }

    void removeLines(List<MonographSectionLine> someLines) {
        lines.removeAll(someLines);
    }

    /**
     * Adds the lines.
     * 
     * @param someLines
     *            the some lines
     */
    void addLines(List<MonographSectionLine> someLines) {
        lines.addAll(someLines);
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Object#toString()
     */
    @Override
    public String toString() {
        return "[monograph id " + monograph.getId() + "/version id " + getVersionId() + "]";
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.domain.Monograph#getCategoryCode()
     */
    @Override
    public String getCategoryCode() {
        return getVersionedObject().getCategoryCode();
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.domain.Monograph#getVersionCode()
     */
    @Override
    public String getVersionCode() {
        return getVersionedObject().getVersionCode();
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.domain.Monograph#getSupportedSections()
     */
    @Override
    public Map<SectionCode, String> getSupportedSections() {
        return monograph.getSupportedSections();
    }

    /**
     * Get the description of clinical effects.
     * 
     * @return the discussion text
     */
    public String getClinicalEffects() {
        return getSection(MonographSection.SectionCode.CLINICAL_EFFECTS).getText();
    }

    /**
     * Get the description of the severity level.
     * 
     * @return the severity level
     */
    public String getSeverityLevel() {
        return getSection(MonographSection.SectionCode.SEVERITY_LEVEL).getText();
    }

    @Override
    public void setUserCommentHistory(Set<UserComment> userCommentHistory) {
        // TODO Auto-generated method stub

    }

    @Override
    public Set<UserComment> getUserCommentHistory() {
        // Do not retrieve the User Comment History through the version.
        return getVersionedObject().getUserCommentHistory();
    }

}
