package gov.va.med.pharmacy.pecs.fdbreports;

import java.io.BufferedOutputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.CreationHelper;
import org.apache.poi.ss.usermodel.DataFormat;
import org.apache.poi.ss.usermodel.FillPatternType;
import org.apache.poi.ss.usermodel.IndexedColors;
import org.apache.poi.xssf.usermodel.XSSFCellStyle;
import org.apache.poi.xssf.usermodel.XSSFColor;
import org.apache.poi.xssf.usermodel.XSSFFont;
import org.apache.poi.xssf.usermodel.XSSFRow;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;

/**
 * A helper class used to create FDB Update Comparison Report (Excel spreadsheets).
 * 
 * The default style is black, non-bolded text. The updated style is red, bolded text. The header style is black, bolded text.
 * 
 * All row and column numbers are 0 based.
 * 
 * @author vhaisaduffg
 * 
 */

public final class ExcelUpdateReportHelper {

    private static final Log LOGGER = LogFactory.getLog(ExcelUpdateReportHelper.class);

    /**
     * The extension to be used in file names for this type of file.
     */
    public static final String EXTENSION = "xlsx";
    private static final String LATEST_FDB_ROW_HEADER = "FDB After Update";
    private static final String PREVIOUS_FDB_ROW_HEADER = "FDB Before Update";
    private static final String VA_CUSTOM_ROW_HEADER = "VA Custom";
    private static final String NO_DATA_FOUND = "NO DATA FOUND";

    private static final String FDB_UPDATE_DATE_HEADER = "FDB Update Received:";
    private static final String NOT_CUSTOMIZED_MESSAGE = "Not customized";
    private static final String UPDATE_FIELD_NOTE = "Note: *  indicates changed FDB data";
    private static final String UPDATE_DP_DESC_FIELD_NOTE = "Note: *  Indicates new Routed Generic 1 or 2 Description";
    // private static final Integer UPDATE_FIELD_NOTE_CELL = 4;
    private static final Integer UPDATE_FIELD_NOTE_CELL = 6;
    private static final String DOUBLE_FORMAT = "0.00000";
    private static final String DATE_FORMAT = "yyyy-MM-dd";
    private static final String TIME_FORMAT = "HH:mm:ss";
    private static final String DATE_TIME_FORMAT = DATE_FORMAT + " " + TIME_FORMAT;
    private static final String UPDATED_INDICATOR = " *";
    private static final Float SEPARATOR_ROW_HEIGHT = 7f;
    private static final Integer HEADING_ROW = 1;
    private static final byte[] RGB_COLOR_VALUES = new byte[] { (byte) 82, (byte) 164, (byte) 213 };

    private XSSFSheet currentWorksheet;
    private XSSFWorkbook workbook;
    private XSSFCellStyle defaultCellStyle;
    private XSSFCellStyle defaultDateCellStyle;
    private XSSFCellStyle headerCellStyle;
    private XSSFCellStyle headerDateCellStyle;
    private XSSFCellStyle updatedCellStyle;
    private XSSFCellStyle updatedDateCellStyle;
    private XSSFCellStyle conceptSeparatorStyle;

    private final DecimalFormat decimalFormatter = new DecimalFormat(DOUBLE_FORMAT);
    private final SimpleDateFormat dateTimeFormatter = new SimpleDateFormat(DATE_TIME_FORMAT);
    private final SimpleDateFormat dateFormatter = new SimpleDateFormat(DATE_FORMAT);

    /**
     * A method used to retrieve a row, create a cell and set the value of that cell.
     * 
     * @param rowNumber
     *            the row to create
     * @param colNumber
     *            the column to create
     * @param value
     *            the value to set
     * @param cellStyle
     *            the style of the cell that should be set
     */
    private void addCell(int rowNumber, int colNumber, Object value, CellStyle cellStyle) {
        Cell cell = createCell(rowNumber, colNumber, cellStyle);
        if (value == null) {
            return;
        }

        if (value instanceof Date) {
            cell.setCellValue((Date) value);
        } else if (value instanceof Number) {
            cell.setCellValue(((Number) value).doubleValue());
        } else {
            cell.setCellValue(value.toString());
        }
    }

    /**
     * Adds a new cell at the rowNumber/colNumber and sets the cell style to headerCellStyle.
     * 
     * @param rowNumber
     *            the row
     * @param colNumber
     *            the column
     * @param value
     *            the cell value
     */
    private void addHeaderCell(int rowNumber, int colNumber, String value) {
        addCell(rowNumber, colNumber, value, headerCellStyle);
    }

    /**
     * Creates a new cell that contains a Date value. The default style is used.
     * 
     * @param rowNumber
     *            the row
     * @param colNumber
     *            the column
     * @param value
     *            the value
     */

    public void addCell(int rowNumber, int colNumber, Date value) {
        addCell(rowNumber, colNumber, getFormattedDate(value), defaultDateCellStyle);
    }

    /**
     * Creates a new cell that contains a Double value. The default style is used.
     * 
     * @param rowNumber
     *            the row
     * @param colNumber
     *            the column
     * @param value
     *            the value
     */
    public void addCell(int rowNumber, int colNumber, Double value) {
        addCell(rowNumber, colNumber, getFormattedDecimal(value), defaultCellStyle);
    }

    /**
     * Creates a new cell that contains an Integer value. The default style is used.
     * 
     * @param rowNumber
     *            the row
     * @param colNumber
     *            the column
     * @param value
     *            the value
     */
    public void addCell(int rowNumber, int colNumber, Integer value) {
        addCell(rowNumber, colNumber, value, defaultCellStyle);
    }

    /**
     * Creates a new cell that contains a Long value. The default style is used.
     * 
     * @param rowNumber
     *            the row
     * @param colNumber
     *            the column
     * @param value
     *            the value
     */
    public void addCell(int rowNumber, int colNumber, Long value) {
        addCell(rowNumber, colNumber, value, defaultCellStyle);
    }

    /**
     * Create a new cell that contains a String value. The default style is used.
     * 
     * @param rowNumber
     *            the row
     * @param colNumber
     *            the column
     * @param value
     *            the value
     */
    public void addCell(int rowNumber, int colNumber, String value) {
        addCell(rowNumber, colNumber, value, defaultCellStyle);
    }

    /**
     * Creates a new cell that contains a Date value. The updated style is used.
     * 
     * @param rowNumber
     *            the row
     * @param colNumber
     *            the column
     * @param value
     *            the value
     */

    public void addUpdatedCell(int rowNumber, int colNumber, Date value) {
        String updatedValue = getUpdatedFormattedDate(value);
        addCell(rowNumber, colNumber, updatedValue, updatedDateCellStyle);
    }

    /**
     * Formats a Date value using the dateTimeFormatter.
     * 
     * @param value
     *            the date
     * @return String
     */
    private String getFormattedDateTime(Date value) {
        return dateTimeFormatter.format(value);
    }

    /**
     * Formats a Date value using the dateFormatter.
     * 
     * @param value
     *            the date
     * @return String
     */
    private String getFormattedDate(Date value) {
        return dateFormatter.format(value);
    }

    /**
     * Formats a Date value using the dateTimeFormatter and adds the UPDATED_INDICATOR to the end of the return value.
     * 
     * @param value
     *            the Date
     * @return String
     */
    @SuppressWarnings("unused")
    private String getUpdatedFormattedDateTime(Date value) {
        return getFormattedDateTime(value) + UPDATED_INDICATOR;
    }

    /**
     * Formats a Date value using the dateFormatter and adds the UPDATED_INDICATOR to the end of the return value.
     * 
     * @param value
     *            the Date
     * @return String
     */
    private String getUpdatedFormattedDate(Date value) {
        return getFormattedDate(value) + UPDATED_INDICATOR;
    }

    /**
     * Creates a new cell that contains a Double value. The updated style is used.
     * 
     * @param rowNumber
     *            the row
     * @param colNumber
     *            the column
     * @param value
     *            the value
     */
    public void addUpdatedCell(int rowNumber, int colNumber, Double value) {
        addCell(rowNumber, colNumber, getUpdatedFormattedDecimal(value), updatedCellStyle);
    }

    /**
     * Formats a Double value using the DECIMAL_FORMATTER.
     * 
     * @param value
     *            the Double
     * @return String
     */
    private String getFormattedDecimal(Double value) {
        return decimalFormatter.format(value);
    }

    /**
     * Formats a Double value using the DECIMAL_FORMATTER and appends the UPDATED_INDICATOR to the return value.
     * 
     * @param value
     *            the Double
     * @return String
     */
    private String getUpdatedFormattedDecimal(Double value) {
        return getFormattedDecimal(value) + UPDATED_INDICATOR;
    }

    /**
     * Creates a new cell that contains an Integer value. The updated style is used.
     * 
     * @param rowNumber
     *            the row
     * @param colNumber
     *            the column
     * @param value
     *            the value
     */
    public void addUpdatedCell(int rowNumber, int colNumber, Integer value) {
        addCell(rowNumber, colNumber, value + UPDATED_INDICATOR, updatedCellStyle);
    }

    /**
     * Creates a new cell that contains a Long value. The updated style is used.
     * 
     * @param rowNumber
     *            the row
     * @param colNumber
     *            the column
     * @param value
     *            the value
     */
    public void addUpdatedCell(int rowNumber, int colNumber, Long value) {
        addCell(rowNumber, colNumber, value + UPDATED_INDICATOR, updatedCellStyle);
    }

    /**
     * Creates a new cell that contains a String value. The updated style is used.
     * 
     * @param rowNumber
     *            the row
     * @param colNumber
     *            the column
     * @param value
     *            the value
     */
    public void addUpdatedCell(int rowNumber, int colNumber, String value) {
        addCell(rowNumber, colNumber, value + UPDATED_INDICATOR, updatedCellStyle);
    }

    /**
     * Creates a new row in a worksheet and sets the row header.
     * 
     * @param row
     *            the row number to create
     * 
     * @return row that was Created
     */
    public XSSFRow createLatestFdbRow(int row) {
        XSSFRow rowCreated = currentWorksheet.createRow(row);
        addHeaderCell(row, 0, LATEST_FDB_ROW_HEADER);
        return rowCreated;
    }

    /**
     * Create Drug Pair Row.
     * 
     * @param row
     *            int
     * @return Drug Pair Row Created.
     */
    public XSSFRow createDpRow(int row) {
        XSSFRow rowCreated = currentWorksheet.createRow(row);
        return rowCreated;
    }

    /**
     * Creates a new row in a worksheet and sets the row header.
     * 
     * @param row
     *            the row number to create
     * 
     * @return Fdb Row Created
     */
    public XSSFRow createPreviousFdbRow(int row) {
        XSSFRow rowCreated = currentWorksheet.createRow(row);
        addHeaderCell(row, 0, PREVIOUS_FDB_ROW_HEADER);
        return rowCreated;
    }

    /**
     * Creates a new row in a worksheet and set the row header.
     * 
     * @param row
     *            the row number to create
     * @return Va Custom Row Created.
     */
    public XSSFRow createVaCustomRow(int row) {
        XSSFRow rowCreated = currentWorksheet.createRow(row);
        addHeaderCell(row, 0, VA_CUSTOM_ROW_HEADER);
        return rowCreated;
    }

    /**
     * Removes a row in the worksheet.
     * 
     * @param row
     *            the row number to create
     */
    public void removeRow(XSSFRow row) {
        currentWorksheet.removeRow(row);
    }

    /**
     * Removes multiple rows.
     * 
     * @param rows
     *            the rows to remove
     */
    public void removeRows(List<XSSFRow> rows) {
        for (XSSFRow row : rows) {
            removeRow(row);
        }
    }

    /**
     * Creates a new row in a worksheet and set the row header. this row is if there is no difference at all
     * 
     * @param row
     *            the row number to create
     */
    public void createNoDataFoundRow(int row) {
        currentWorksheet.createRow(row);
        addHeaderCell(row, 0, NO_DATA_FOUND);
    }

    /**
     * Creates a new VA Custom row in a worksheet and sets Column B to the NOT_CUSTOMIZED_MESSAGE.
     * 
     * @param row
     *            the row number to create
     * @return Create Not Customizable Row
     */
    public XSSFRow createNotCustomizedRow(int row) {
        XSSFRow rowCreated = createVaCustomRow(row);
        addCell(row, 1, NOT_CUSTOMIZED_MESSAGE);
        return rowCreated;
    }

    /**
     * Creates a separator row to separate concept values.
     * 
     * @param rowNumber
     *            the row
     * @param numberOfColumns
     *            the number of columns to fill in the row
     */
    public void createSeparatorRow(int rowNumber, int numberOfColumns) {
        XSSFRow row = currentWorksheet.createRow(rowNumber);
        row.setHeightInPoints(SEPARATOR_ROW_HEIGHT);
        int colsToCreate = numberOfColumns + 1; // Account for the header column
        for (int i = 0; i < colsToCreate; i++) {
            addCell(rowNumber, i, null, conceptSeparatorStyle);
        }
    }

    /**
     * Creates a default FDB Update report header. The header values are placed in to the spreadsheet starting at Column B.
     * 
     * @param headerValues
     *            the list of headers
     * @param fdbUpdateDate
     *            the FDB Update Date to display in the header
     * @return the last row number created in the header. Any data rows should start 1 greater than this value.
     */
    public int createHeader(List<String> headerValues, String fdbUpdateDate) {
        currentWorksheet.createRow(0);
        addCell(0, 0, FDB_UPDATE_DATE_HEADER, headerCellStyle);
        addCell(0, 1, fdbUpdateDate, headerDateCellStyle);
        addCell(0, UPDATE_FIELD_NOTE_CELL, UPDATE_FIELD_NOTE, defaultCellStyle);

        currentWorksheet.createRow(1);
        int col = 1;
        for (String header : headerValues) {
            addCell(HEADING_ROW, col++, header, headerCellStyle);
        }

        return 1;
    }

    /**
     * Create Header 2.
     * 
     * @param headerValues
     *            List<String>
     * @param fdbUpdateDate
     *            String
     * @return 1
     */
    public int createHeader2(List<String> headerValues, String fdbUpdateDate) {
        currentWorksheet.createRow(0);
        addCell(0, 1, UPDATE_DP_DESC_FIELD_NOTE, defaultCellStyle);
        currentWorksheet.createRow(1);
        int col = 0;
        for (String header : headerValues) {
            addCell(1, col++, header, headerCellStyle);
        }
        return 1;
    }

    /**
     * Obtains the specified row from the current worksheet and adds a new cell.
     * 
     * @param rowNumber
     *            the row to obtain
     * @param colNumber
     *            the column to create
     * @param cellStyle
     *            the style of the cell
     * @return Cell
     */
    private Cell createCell(int rowNumber, int colNumber, CellStyle cellStyle) {
        XSSFRow row = currentWorksheet.getRow(rowNumber);
        Cell cell = row.createCell(colNumber);
        cell.setCellStyle(cellStyle);
        return cell;
    }

    /**
     * Creates a new Excel workbook and initializes the styles used.
     */
    public void createWorkbook() {
        workbook = new XSSFWorkbook();
        createFonts();
    }

    /**
     * Creates a new Worksheet in the Workbook and assigns it a name.
     * 
     * @param worksheetName
     *            the name of the worksheet
     */
    public void createWorksheet(String worksheetName) {
        currentWorksheet = workbook.createSheet(worksheetName);
    }

    /**
     * Autosizes the columns in all worksheets and creates a freeze pane for the heading.
     */
    private void setupForRetrieval() {
        final int numberOfSheets = workbook.getNumberOfSheets();
        for (int i = 0; i < numberOfSheets; i++) {
            final XSSFSheet sheet = workbook.getSheetAt(i);

            if (null != sheet.getRow(HEADING_ROW)) {
                final int numberOfColumns = sheet.getRow(HEADING_ROW).getLastCellNum();
                for (int j = 0; j < numberOfColumns; j++) {
                    sheet.autoSizeColumn(j);
                }
            }
            sheet.createFreezePane(0, 2);
        }
        workbook.setActiveSheet(0);
        workbook.setFirstVisibleTab(0);
    }

    /**
     * Creates the fonts/styles used in this workbook.
     */
    private void createFonts() {
        CreationHelper createHelper = workbook.getCreationHelper();
        DataFormat format = createHelper.createDataFormat();

        defaultCellStyle = workbook.createCellStyle();
        defaultCellStyle.setWrapText(true);

        defaultDateCellStyle = workbook.createCellStyle();
        defaultDateCellStyle.setWrapText(false);
        defaultDateCellStyle.setDataFormat(format.getFormat(DATE_TIME_FORMAT));

        XSSFFont updatedFont = workbook.createFont();
        updatedFont.setBold(true);
        updatedFont.setColor(IndexedColors.RED.getIndex());
        updatedCellStyle = workbook.createCellStyle();
        updatedCellStyle.setWrapText(true);
        updatedCellStyle.setFont(updatedFont);

        updatedDateCellStyle = workbook.createCellStyle();
        updatedDateCellStyle.setWrapText(true);
        updatedDateCellStyle.setDataFormat(format.getFormat(DATE_TIME_FORMAT));
        updatedDateCellStyle.setFont(updatedFont);

        XSSFFont headerFont = workbook.createFont();
        headerFont.setBold(true);
        headerCellStyle = workbook.createCellStyle();
        headerCellStyle.setFont(headerFont);

        headerDateCellStyle = workbook.createCellStyle();
        headerDateCellStyle.setWrapText(true);
        headerDateCellStyle.setDataFormat(format.getFormat(DATE_FORMAT));

        conceptSeparatorStyle = workbook.createCellStyle();
        XSSFColor separatorColor = new XSSFColor();
        separatorColor.setRgb(RGB_COLOR_VALUES);
        conceptSeparatorStyle.setFillBackgroundColor(separatorColor);
        conceptSeparatorStyle.setFillForegroundColor(separatorColor);
        conceptSeparatorStyle.setFillPattern(FillPatternType.SOLID_FOREGROUND);
    }

    /**
     * Returns the workbook currently being used.
     * 
     * @return workbook
     */
    public XSSFWorkbook getWorkbook() {
        setupForRetrieval();
        return workbook;
    }

    /**
     * Returns the Workbook as a byte array that can be used for storage purposes.
     * 
     * @return byte[] a null value can be returned if no Workbook is in use
     * @throws IOException
     *             if an exception occurs writing the byte array
     */
    public byte[] toByteArray() throws IOException {
        try {
            if (workbook != null) {
                setupForRetrieval();
                ByteArrayOutputStream dest = new ByteArrayOutputStream();
                BufferedOutputStream out = new BufferedOutputStream(dest);
                workbook.write(out);
                return dest.toByteArray();
            } else {
                return null;
            }
        } catch (IOException ioe) {
            LOGGER.error(ioe.getMessage(), ioe);
            throw ioe;
        }
    }

    /**
     * sets the sheet index.
     * 
     * @param activeSheetIndex
     *            the active sheet index.
     */
    public void setSheetIndex(int activeSheetIndex) {
        currentWorksheet = workbook.getSheetAt(activeSheetIndex);
    }

}
