package gov.va.med.pharmacy.pecs.fdbreports;

import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.apache.log4j.Logger;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

/**
 * Encapsulates the extraction of the metadata from the FDB update file.
 * 
 * @author VHAISACARUSR
 * 
 */
public final class FDBUpdateMetadataParser {
    private static final String PRODUCT_TABLES_TABLE = "/Product/Tables/Table";
    private static final String PRIMARY_KEY = "PrimaryKey";
    private static final String FILE = "File";
    private static final String TYPE = "Type";
    private static final String COLUMN = "Column";
    private static final String NAME = "Name";
    private static final String FDB_METADATA_FILE = "PRODDEFINITION.XML";
    private static final String PRODUCT_TABLES_TABLE_PRIMARY_KEY = "/Product/Tables/Table/PrimaryKey";
    private static final Logger LOGGER = Logger.getLogger(FDBUpdateMetadataParser.class);
    private final ZipFile zipFile;

    /**
     * parses the update metadata file.
     * 
     * @param zipFile
     *            the data associated with the PRODDEFINITION.XML file
     */
    public FDBUpdateMetadataParser(ZipFile zipFile) {
        this.zipFile = zipFile;
    }

    /**
     * Parse the XML file representing the metadata for the FDB database schema.
     * 
     * @return the metadata
     */
    public FdbUpdateMetadata parseMetadata() {
        try {
            FdbUpdateMetadata fdbUpdateMetadata = new FdbUpdateMetadata();
            ZipEntry entry = zipFile.getEntry(FDB_METADATA_FILE);
            InputStream zipFileInputStream = zipFile.getInputStream(entry);
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            factory.setNamespaceAware(true);
            DocumentBuilder db = factory.newDocumentBuilder();
            Document doc = db.parse(zipFileInputStream);
            XPathFactory xFactory = XPathFactory.newInstance();
            XPath xpath = xFactory.newXPath();
            XPathExpression expr = xpath.compile(PRODUCT_TABLES_TABLE);
            NodeList nodes = (NodeList) expr.evaluate(doc, XPathConstants.NODESET);

            for (int i = 0; i < nodes.getLength(); i++) {
                Node n = nodes.item(i);
                FdbTableMetadata table = new FdbTableMetadata();

                String tableName = n.getAttributes().getNamedItem(NAME).getNodeValue();

                table.setTableName(tableName);
                fdbUpdateMetadata.addTable(table);
                // loop the column child node
                NodeList list2 = n.getChildNodes();
                for (int j = 0; j < list2.getLength(); j++) {
                    Node node = list2.item(j);

                    String nodeName = node.getNodeName();
                    String column = "";
                    FdbTableColumn fdbTableColumn = new FdbTableColumn();
                    if (nodeName.equals(COLUMN)) {
                        column = node.getAttributes().getNamedItem(NAME).getNodeValue();
                        fdbTableColumn.setName(column);
                        fdbTableColumn.setKey(false); // reset after verifying
                        String dataType = node.getAttributes().getNamedItem(TYPE).getNodeValue();
                        fdbTableColumn.setDataType(FdbColumnType.fromString(dataType));
                        table.addColumn(fdbTableColumn);
                    } else if (nodeName.equals(FILE)) {
                        String fileName = node.getAttributes().getNamedItem(NAME).getNodeValue();
                        fileName = fileName.toUpperCase() + ".UPD";
                        table.setFileName(fileName);
                    } else if (nodeName.equals(PRIMARY_KEY)) {
                        String pkName = node.getAttributes().getNamedItem(NAME).getNodeValue();
                        table.setPkName(pkName);
                    }

                }
            }

            getPKs(doc, xpath, fdbUpdateMetadata);
            return fdbUpdateMetadata;
        } catch (Exception e) {
            String msg = "File " + zipFile.getName() + " does not appear to be a valid FDB incremental update";
            LOGGER.error(msg, e);
            return null;
        }
    }

    /**
     * @param doc
     *            the document
     * @param xpath
     *            the XPath
     * @param fdbUpdateMetadata
     *            the meta data
     * @return a map maps keyed by strings
     * @throws IOException
     *             if there is a problem with the stream
     * @throws ParserConfigurationException
     *             if the file cannot be parsed
     * @throws SAXException
     *             if there is an exception with SAX
     * @throws XPathExpressionException
     *             if there is an exception with XPath
     */
    private Map<String, Map<String, String>> getPKs(Document doc, XPath xpath, FdbUpdateMetadata fdbUpdateMetadata)
            throws IOException, ParserConfigurationException, SAXException, XPathExpressionException {
        XPathExpression expr = xpath.compile(PRODUCT_TABLES_TABLE_PRIMARY_KEY);
        NodeList nodes = (NodeList) expr.evaluate(doc, XPathConstants.NODESET);
        Map<String, Map<String, String>> pkMap = new HashMap<String, Map<String, String>>();

        for (int i = 0; i < nodes.getLength(); i++) {
            Node n = nodes.item(i);
            // FdbTableMetadata table = new FdbTableMetadata();

            String pkName = n.getAttributes().getNamedItem(NAME).getNodeValue();

            // loop the PK Col child node
            NodeList list2 = n.getChildNodes();
            Map<String, String> pkColsMap = new HashMap<String, String>();
            for (int j = 0; j < list2.getLength(); j++) {
                // pkColsMap = new HashMap<String, String>();
                Node node = list2.item(j);

                String nodeName = node.getNodeName();
                if (nodeName.equals("PKCol")) {
                    String pkColName = node.getAttributes().getNamedItem(NAME).getNodeValue();
                    pkColsMap.put(pkColName, "true");
                }
            }
            pkMap.put(pkName, pkColsMap);
        }

        // set the key values
        // ///////////////////////////////////////////////
        List<FdbTableMetadata> tables = fdbUpdateMetadata.getTables();
        for (FdbTableMetadata fdbTableMetadata : tables) {
            String tableName = fdbTableMetadata.getTableName();
            if (tableName != null) {
                List<FdbTableColumn> cols = fdbTableMetadata.getColumns();
                String pkName = fdbTableMetadata.getPkName();
                for (FdbTableColumn fdbTableColumn : cols) {
                    String colName = fdbTableColumn.getName();
                    Map<String, String> a = pkMap.get(pkName);
                    String b = a.get(colName);
                    if ((b != null) && (!b.equals(""))) {
                        fdbTableColumn.setKey(true);
                    }

                }// end for
            }// end if
        }// end for

        return pkMap;
    }
}
