package gov.va.med.pharmacy.pecs.fdbreports;

import java.io.BufferedReader;
import java.io.DataInputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

import org.apache.log4j.Logger;

/**
 * Parses the FDB Update File.
 * 
 * @author VHAISACARUSR
 * 
 */
public final class FdbUpdateParser {

    /**
     * please change the name of this VAR if you know what 8 is referring to!
     */
    private static final int EIGHT = 8;

    // private static final String FDB_METADATA_FILE = "PRODDEFINITION.XML";
    // private static final String FDB_DUPLICATE_THERAPY = "FDBDUPLICATETHERAPY.UPD";
    // private static final String FDB_DOSING = "FDBDOSING.UPD";
    // private static final String FDB_DDIM_INTERACTION = "FDBDDIMINTERACTION.UPD";
    /**
     * File name for Routed Generic.
     */
    public static final String FDB_GENERIC_ROUTEDDRUG = "FDBGENERICROUTEDDRUG.UPD";
    private static final String FDB_CONTROL_FILE = "FDBUPDCONTROL.DAT";
    private static final Logger LOGGER = Logger.getLogger(FdbUpdateParser.class);

    private final FdbUpdateMetadata meta;
    private final ZipFile zip;
    // private String fileDt;
    private String fdbIssueDate;
    private String fdbPreviousIssueDate;

    /**
     * Processes a ZIP file.
     * 
     * @param file
     *            the file to be processed
     * @throws Exception
     *             if an error occurs
     */
    public FdbUpdateParser(File file) throws Exception {
        zip = new ZipFile(file, ZipFile.OPEN_READ);
        FDBUpdateMetadataParser metadataParser = new FDBUpdateMetadataParser(zip);
        this.meta = metadataParser.parseMetadata();
        if (this.meta != null) {
            parseControlFile();
        }
    }

    /**
     * Parse the data file.
     * 
     * @param fileName
     *            the name of the file to parse
     * @return the parsed file
     * @throws IOException
     *             if an error occurs
     */
    public File parseConcept(String fileName) throws IOException {
        File f = File.createTempFile("report_", ".tmp");
        ZipEntry entry = zip.getEntry(fileName);
        InputStream in = zip.getInputStream(entry);
        OutputStream out = new FileOutputStream(f);
        ObjectOutputStream outputStream = new ObjectOutputStream(out);

        DataInputStream ds = new DataInputStream(in);
        BufferedReader br = new BufferedReader(new InputStreamReader(ds));
        // Read File Line By Line
        List<FdbTableMetadata> listMeta = this.meta.getTables();
        for (FdbTableMetadata fdbTableMetadata : listMeta) {
            String fileNameFromMeta = fdbTableMetadata.getFileName();
            if (fileNameFromMeta.equals(fileName)) {
                processConcept(br, outputStream, fdbTableMetadata);
            }
        }

        // write an end of stream marker
        outputStream.writeObject(null);
        in.close();
        outputStream.close();
        out.close();
        return f;
    }

    /**
     * Reads the text file one line at a time and creates an object.
     * 
     * @param bufferedReader
     *            the buffered reader
     * @param outputStream
     *            the output stream
     * @param tableMeta
     *            the FDB metadata table
     * @throws IOException
     *             if a problem occurs
     */
    private void processConcept(BufferedReader bufferedReader, ObjectOutputStream outputStream, FdbTableMetadata tableMeta)
            throws IOException {
        String strLine = "";
        Map<String, String> fieldMap = new HashMap<String, String>();
        Map<String, String> pkMap = new HashMap<String, String>();
        FdbUpdateRecord fdbUpdateRecord;
        int lineCount = 0;
        while ((strLine = bufferedReader.readLine()) != null) {
            lineCount = lineCount + 1;

            fdbUpdateRecord = new FdbUpdateRecord();
            fdbUpdateRecord.setTable(tableMeta);
            List<FdbColumnUpdate> newList = new ArrayList<FdbColumnUpdate>();
            if (strLine.startsWith("F")) {
                processLineStartsWithF(strLine, fieldMap);
            } else if (strLine.startsWith("P")) {
                processLineStartsWithP(strLine, fieldMap, pkMap);
                //@formatter:off
            } else if (strLine.startsWith(FdbUpdateRecord.REC_TYPE_ADD)
                    || strLine.startsWith(FdbUpdateRecord.REC_TYPE_DELETE)) {
                //@formatter:on
                processLineStartsWithAddOrDelete(tableMeta, strLine, fieldMap, fdbUpdateRecord, newList);

                outputStream.writeObject(fdbUpdateRecord);
            } else if (strLine.startsWith(FdbUpdateRecord.REC_TYPE_CHANGE)) { // change
                processLineforChangeRecord(tableMeta, strLine, fieldMap, pkMap, fdbUpdateRecord, newList);
                outputStream.writeObject(fdbUpdateRecord);
            }
            // Print the content on the console
        }
    }

    /**
     * @param tableMeta
     *            the table metadata
     * @param line
     *            the line from the update file
     * @param fieldMap
     *            the field map
     * @param pkMap
     *            the primary key map
     * @param fdbUpdateRecord
     *            the update record
     * @param newList
     *            the new list
     */
    private void processLineforChangeRecord(FdbTableMetadata tableMeta, String line, Map<String, String> fieldMap,
            Map<String, String> pkMap, FdbUpdateRecord fdbUpdateRecord, List<FdbColumnUpdate> newList) {
        FdbTableColumn fdbTableColumn;
        FdbColumnUpdate fdbColumnUpdate;
        String[] tokens = line.split("\\|");
        for (int i = 0; i < tokens.length; i++) {
            String value = tokens[i];
            if (i == 0) {
                fdbUpdateRecord.setType(value);
            } else {
                fdbColumnUpdate = new FdbColumnUpdate();
                if (i <= pkMap.size()) {
                    fdbTableColumn = tableMeta.getColumn(fieldMap.get(String.valueOf(i)));
                } else {
                    // data becomes index--- only for "Change". this tells which column to update with new data.
                    String updateIndex = value;
                    fdbTableColumn = tableMeta.getColumn(fieldMap.get(updateIndex));
                    if (++i >= tokens.length) {
                        // this can happen when the data for the last field is null
                        value = null;
                    } else {
                        value = nullIfEmpty(tokens[i]);
                    }
                }
                fdbColumnUpdate.setColumn(fdbTableColumn);
                fdbColumnUpdate.setCurrentValue(value);
                newList.add(fdbColumnUpdate);
            }
        }
        fdbUpdateRecord.setValues(newList);
    }

    /**
     * @param tableMeta
     *            the FDB table metadata
     * @param line
     *            the line from the update file
     * @param fieldMap
     *            the field map
     * @param fdbUpdateRecord
     *            the update record
     * @param newList
     *            the new list
     */
    private void processLineStartsWithAddOrDelete(FdbTableMetadata tableMeta, String line, Map<String, String> fieldMap,
            FdbUpdateRecord fdbUpdateRecord, List<FdbColumnUpdate> newList) {
        FdbTableColumn fdbTableColumn;
        FdbColumnUpdate fdbColumnUpdate;
        String[] tokens = line.split("\\|");
        for (int i = 0; i < tokens.length; i++) {
            String value = tokens[i];
            if (i == 0) {
                fdbUpdateRecord.setType(value);
            } else {
                fdbColumnUpdate = new FdbColumnUpdate();
                fdbTableColumn = tableMeta.getColumn(fieldMap.get(String.valueOf(i)));
                fdbColumnUpdate.setColumn(fdbTableColumn);
                fdbColumnUpdate.setCurrentValue(value);
                newList.add(fdbColumnUpdate);
            }
        }
        fdbUpdateRecord.setValues(newList);
    }

    /**
     * @param line
     *            the line
     * @param fieldMap
     *            the field map
     * @param pkMap
     *            the primary key map
     */
    private void processLineStartsWithP(String line, Map<String, String> fieldMap, Map<String, String> pkMap) {
        StringTokenizer tok;
        tok = new StringTokenizer(line, "|");
        int counter = 1;
        while (tok.hasMoreTokens()) {
            String key = tok.nextToken();

            if (counter > 2) {
                String value = fieldMap.get(key);
                pkMap.put(key, value);
            }
            counter = counter + 1;
        }
    }

    /**
     * @param line
     *            the line passed in
     * @param fieldMap
     *            the map of key values
     */
    private void processLineStartsWithF(String line, Map<String, String> fieldMap) {
        StringTokenizer tok;
        tok = new StringTokenizer(line, "|");
        while (tok.hasMoreTokens()) {
            String key = tok.nextToken();
            String value = tok.nextToken();
            fieldMap.put(key, value);
        }
        fieldMap.remove("F");
    }

    /**
     * @param str
     *            the string to test
     * @return null if the string is empty
     */
    private String nullIfEmpty(String str) {
        return str != null && str.length() == 0 ? null : str;
    }

    /**
     * the purpose of this method is to get the file date of zip file.
     * 
     * @throws IOException
     *             if a problem occurs
     */
    private void parseControlFile() throws IOException {
        ZipEntry entry = zip.getEntry(FDB_CONTROL_FILE);
        if (entry != null) {
            InputStream in = zip.getInputStream(entry);
            DataInputStream ds = new DataInputStream(in);
            BufferedReader br = new BufferedReader(new InputStreamReader(ds));
            String strLine = "";
            while ((strLine = br.readLine()) != null) {
                if (strLine.startsWith("E")) {
                    StringTokenizer tok = new StringTokenizer(strLine, "|");
                    while (tok.hasMoreTokens()) {
                        String value = tok.nextToken();
                        if (value.startsWith("'Weekly production data")) {
                            int dateIndex = value.indexOf("2");
                            if (dateIndex > 0) {
                                setFdbIssueDate(value.substring(dateIndex, dateIndex + EIGHT));
                            }
                        }
                    }
                } else if (strLine.startsWith("V")) {
                    StringTokenizer tok = new StringTokenizer(strLine, "|");
                    while (tok.hasMoreTokens()) {
                        String value = tok.nextToken();
                        if (value.startsWith("previous database issue date")) {
                            fdbPreviousIssueDate = tok.nextToken();
                        }
                    }
                }
            }
        }
    }

    /**
     * Returns the Previous Issue Date from the control file.
     * 
     * @return the fdbPreviousIssueDate
     */
    public final String getFdbPreviousIssueDate() {
        return fdbPreviousIssueDate;
    }

    /**
     * Sets the Previous Issue Date.
     * 
     * @param fdbPreviousIssueDate
     *            the fdbPreviousIssueDate to set
     */
    public final void setFdbPreviousIssueDate(String fdbPreviousIssueDate) {
        this.fdbPreviousIssueDate = fdbPreviousIssueDate;
    }

    /**
     * Gets the fdbIssueDate.
     * 
     * @return the fdbIssueDate
     */
    public String getFdbIssueDate() {
        return fdbIssueDate;
    }

    /**
     * Sets the fdbIssueDate.
     * 
     * @param fdbIssueDate
     *            the fdbIssueDate to set
     */
    public void setFdbIssueDate(String fdbIssueDate) {
        this.fdbIssueDate = fdbIssueDate;
    }

    /*
     * Test method, remove after test
     * 
     * public void testPopulateMetaData(FdbUpdateMetadata meta) { // test code to verify all meta data is correctly set into the
     * Objects. // 1 duplicate therapy List<FdbTableMetadata> tables = meta.getTables(); for (FdbTableMetadata fdbTableMetadata
     * : tables) { String tableName = fdbTableMetadata.getTableName(); if (tableName != null) { if
     * ((tableName.equals("fdb_duplicatetherapy")) || (tableName.equals("fdb_dosing")) ||
     * (tableName.equals("fdb_ddiminteraction"))) { List<FdbTableColumn> cols = fdbTableMetadata.getColumns(); for
     * (FdbTableColumn fdbTableColumn : cols) { String colName = fdbTableColumn.getName(); boolean isKey =
     * fdbTableColumn.isKey(); String dataType = fdbTableColumn.getDataType(); } String fileName =
     * fdbTableMetadata.getFileName(); String pkName = fdbTableMetadata.getPkName(); }
     * 
     * }// end if
     * 
     * }// end for }
     */
}
