package gov.va.med.pharmacy.pecs.metadata;

import gov.va.med.pharmacy.pecs.exception.ValidationException;

import java.lang.reflect.Field;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import org.springframework.expression.EvaluationException;
import org.springframework.expression.Expression;
import org.springframework.expression.ParseException;

/**
 * A user-exposed field in a concept.
 */
public final class ConceptField // implements ApplicationContextAware
{

    /** The concept metadata. */
    private final ConceptMetadata conceptMetadata;

    /** The static metadata. */
    private final FieldInfo staticMetadata;

    /** The dynamic metadata. */
    private final UserModifiableFieldMetadata dynamicMetadata;

    /** The field. */
    private Field field;

    /** The expression. */
    private Expression expression;

    /** The custom expression. */
    private Expression customExpression;

    /** The fdb expression. */
    private Expression fdbExpression;
    /**
     * The Date Format to use.
     */
    private final SimpleDateFormat dateFormatter = new SimpleDateFormat("yyyy-MM-dd hh:mm:ss");

    /** The default values. */
    private static Map<Class<?>, Object> defaultValues;
    static {
        defaultValues = new HashMap<Class<?>, Object>();
        defaultValues.put(int.class, new Integer(0));
        defaultValues.put(long.class, new Long(0));
        defaultValues.put(float.class, new Float(0));
        defaultValues.put(double.class, new Double(0));
    }

    /** The override. */
    private final Map<String, Object> override = new HashMap<String, Object>();

    /**
     * Editable setting.
     */
    public static final String EDITABLE = "editable";

    /**
     * Constructor.
     * 
     * @param f
     *            Field
     * @param conceptMetadata
     *            ConceptMetadata
     * @param dynamicInfo
     *            UserModifiableFieldMetadata
     * @param prefix
     *            String
     */
    ConceptField(Field f, ConceptMetadata conceptMetadata, UserModifiableFieldMetadata dynamicInfo, String prefix) {

        this.conceptMetadata = conceptMetadata;
        staticMetadata = f.getAnnotation(FieldInfo.class);
        // check if we're dealing with a nested field
        String fn = staticMetadata.nestedFieldName();
        if (ConceptMetadata.noValue(fn)) {
            field = f;
        } else {
            for (Field fld : f.getType().getDeclaredFields()) {
                if (fn.equals(fld.getName())) {
                    field = fld;
                }
            }
            if (field == null) {
                String msg =
                        "Nested field " + fn + " cannot be accessed for field " + staticMetadata.name() + " (type "
                                + f.getType().getName() + ")";
                ValidationException e = new ValidationException(msg);
                ConceptMetadata.LOGGER.error(msg, e);
                throw e;
            }
            prefix = ("".equals(prefix) ? f.getName() : prefix + f.getName() + '.');
        }
        dynamicMetadata = dynamicInfo;
        try {
            expression = conceptMetadata.getParser().parseExpression(makeNullSafe(prefix + getPropertyName()));
            customExpression = conceptMetadata.getParser().parseExpression(makeNullSafe(getPersistentPath(true)));
            fdbExpression = conceptMetadata.getParser().parseExpression(makeNullSafe(getPersistentPath(false)));
        } catch (ParseException e) {
            String msg =
                    "Property " + getPropertyName() + " is invalid for " + this.conceptMetadata.getDtoClass().getName() + "/"
                            + getName();
            ConceptMetadata.LOGGER.error(msg, e);
            throw new ValidationException(msg, e);
        }
    }

    /**
     * Makes a null value safe.
     * 
     * @param expr
     *            String
     * @return String
     */
    private String makeNullSafe(String expr) {
        StringBuffer newExpr = new StringBuffer();
        String[] elements = expr.split("\\.");
        for (int i = 0; i < elements.length - 1; i++) {
            newExpr.append(elements[i]).append("?.");
        }
        newExpr.append(elements[elements.length - 1]);
        return newExpr.toString();
    }

    /**
     * Extract the value corresponding to this field for the given DTO.
     * 
     * @param data
     *            the object to get the value from
     * @return the value
     */
    public Object getValue(Object data) {
        try {
            Object val = expression.getValue(data);
            if (val == null) {
                val = getDefaultValue(getType());
            }
            return val;
        } catch (EvaluationException e) {
            String msg = "Failed to access field " + getName() + " of " + data + " (property name = " + getPropertyName() + ")";
            throw new ValidationException(msg, e);
        }
    }

    /**
     * Extract the value corresponding to this field for the given custom persistent object.
     * 
     * @param data
     *            the object to get the value from
     * @return the value
     */
    private Object getCustomValue(Object data) {
        try {
            Object val = customExpression.getValue(data);
            if (val == null) {
                val = getDefaultValue(getType());
            }
            return val;
        } catch (EvaluationException e) {
            String msg =
                    "Failed to access field " + getName() + " of " + data + " (property name = " + getPersistentPath(true)
                            + ")";
            throw new ValidationException(msg, e);
        }
    }

    /**
     * Extract the value corresponding to this field for the given FDB persistent object.
     * 
     * @param data
     *            the object to get the value from
     * @return the value
     */
    private Object getFdbValue(Object data) {
        try {
            Object val = fdbExpression.getValue(data);
            if (val == null) {
                val = getDefaultValue(getType());
            }
            return val;
        } catch (EvaluationException e) {
            String msg =
                    "Failed to access field " + getName() + " of " + data + " (property name = " + getPersistentPath(false)
                            + ")";
            throw new ValidationException(msg, e);
        }
    }

    /**
     * Returns the default value.
     * 
     * @param type
     *            Class<?>
     * @return Object
     */
    private Object getDefaultValue(Class<?> type) {
        return defaultValues.get(type);
    }

    /**
     * Returns the value as a String.
     * 
     * @param data
     *            Object
     * @return String
     */
    public String getValueAsString(Object data) {
        Object val = getValue(data);
        if (val == null) {
            return "";
        }
        if (val instanceof Date) {
            return dateFormatter.format(val);
        } else {
            return String.valueOf(val);
        }
    }

    /**
     * Returns if the field is required.
     * 
     * @return boolean
     */
    public boolean isRequired() {
        return staticMetadata.required();
    }

    /**
     * Returns the display name.
     * 
     * @return String
     */
    public String getDisplayName() {
        return dynamicMetadata == null ? "" : dynamicMetadata.getDisplayName();
    }

    /**
     * Returns the name.
     * 
     * @return String
     */
    public String getName() {
        return staticMetadata.name();
    }

    /**
     * Returns if the field is internal.
     * 
     * @return boolean
     */
    public boolean isInternal() {
        return staticMetadata.internal();
    }

    /**
     * Returns if the field is a lookup value.
     * 
     * @return boolean
     */
    public boolean isLookup() {
        return staticMetadata.lookup();
    }

    /**
     * Returns the lookup type.
     * 
     * @return Class
     */
    public Class<?> getLookupType() {
        return staticMetadata.lookupType();
    }

    /**
     * Returns the persistent path.
     * 
     * @param custom
     *            if true, returns the Custom persistent path. If false, the FDB persistent path.
     * @return String
     */
    public String getPersistentPath(boolean custom) {
        return custom ? staticMetadata.customPersistentPath() : staticMetadata.fdbPersistentPath();
    }

    /**
     * Returns the persistent value.
     * 
     * @param custom
     *            if true, return Custom. If false, FDB is returned.
     * @param data
     *            the data object.
     * @return Object
     */
    public Object getPersistentValue(boolean custom, Object data) {
        return custom ? getCustomValue(data) : getFdbValue(data);
    }

    /**
     * Returns the FieldId attribute from the dynamic metadata. If the dynamic metadata has not been set, a -1 is returned.
     * 
     * @return the field id
     */
    public long getFieldId() {
        return dynamicMetadata == null ? -1L : dynamicMetadata.getFieldId();
    }

    /**
     * Include in reports.
     * 
     * @return true, if successful
     */
    public boolean includeInReports() {
        return dynamicMetadata == null ? false : dynamicMetadata.isIncludeInReports();
    }

    /**
     * Include in query results.
     * 
     * @return true, if successful
     */
    public boolean includeInQueryResults() {
        return dynamicMetadata == null ? false : dynamicMetadata.isIncludeInQueryResults();
    }

    /**
     * Include in detail display.
     * 
     * @return true, if successful
     */
    public boolean includeInDetailDisplay() {
        return !staticMetadata.hideFromDetailDisplay() && dynamicMetadata == null ? false : dynamicMetadata
                .isIncludeInDetailDisplay();
    }

    /**
     * Gets the display order.
     * 
     * @return the display order
     */
    public int getDisplayOrder() {
        return dynamicMetadata == null ? 100 : dynamicMetadata.getDisplayOrder();
    }

    /**
     * Gets the type.
     * 
     * @return the type
     */
    public Class<?> getType() {
        return field.getType();
    }

    /**
     * Gets the concept metadata.
     * 
     * @return the concept metadata
     */
    public ConceptMetadata getConceptMetadata() {
        return this.conceptMetadata;
    }

    /**
     * Gets the property name.
     * 
     * @return the property name
     */
    public String getPropertyName() {
        String propertyName = staticMetadata.property();
        if (ConceptMetadata.noValue(propertyName)) {
            propertyName = field.getName();
        }
        return propertyName;
    }

    /**
     * Checks if is editable.
     * 
     * @return true, if is editable
     */
    public boolean isEditable() {
        if (override.containsKey(EDITABLE)) {
            return (Boolean) override.get(EDITABLE);
        }
        return this.staticMetadata.editable();
    }

    /**
     * Gets the display lines.
     * 
     * @return the display lines
     */
    public int getDisplayLines() {
        return this.staticMetadata.displayLines();

    }

    /**
     * Checks if is queriable.
     * 
     * @param custom
     *            the custom
     * @return true, if is queriable
     */
    public boolean isQueriable(boolean custom) {
        return !staticMetadata.nonQueriable() && !ConceptMetadata.noValue(getPersistentPath(custom));
    }

    /**
     * Checks if is reportable.
     * 
     * @param custom
     *            the custom
     * @return true, if is reportable
     */
    public boolean isReportable(boolean custom) {
        return !staticMetadata.nonReportable() && !ConceptMetadata.noValue(getPersistentPath(custom));
    }

    /**
     * Allows the code to temporarily override the annotation setting for the field. This may be needed in certain business
     * situations. For example, the Concept Id on the Dose Range should be read-only, except when a Dose Range is being created
     * from scratch. Currently, the only annotation that can be overridden is the editable annotation.
     * 
     * @param key
     *            the annotation to override
     * @param value
     *            the new annotation value
     */
    public void setOverride(String key, Boolean value) {
        override.put(key, value);
    }

    /**
     * Is this field included in a Report History report?.
     * 
     * @return true, if is report history
     */
    public boolean isReportHistory() {
        return staticMetadata.reportHistory();
    }

    /**
     * If field is in Report History report, what order should it be displayed.
     * 
     * @return the report history order
     */
    public int getReportHistoryOrder() {
        return staticMetadata.reportHistoryOrder();
    }

    /**
     * If field is in Report History report, should it's old and new values be compared?.
     * 
     * @return true, if is report history compare
     */
    public boolean isReportHistoryCompare() {
        return staticMetadata.reportHistoryCompare();
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (o instanceof ConceptField) {
            ConceptField other = (ConceptField) o;
            return field.equals(other.field);
        }
        return false;
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + ((field == null) ? 0 : field.hashCode());
        return result;
    }

    @Override
    public String toString() {
        return "[field " + staticMetadata.name() + "]";
    }
}
