package gov.va.med.pharmacy.pecs.metadata;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.SortedSet;
import java.util.TreeSet;

import org.apache.log4j.Logger;
import org.springframework.expression.ExpressionParser;
import org.springframework.expression.spel.standard.SpelExpressionParser;

/**
 * Class Concept MetaData.
 * 
 * @author DOMAINOVERSD
 * 
 */
public final class ConceptMetadata {
    /** Logger. */
    static final Logger LOGGER = Logger.getLogger(ConceptMetadata.class);
    private static final Comparator<ConceptField> COMPARATOR = new Comparator<ConceptField>() {
        @Override
        public int compare(ConceptField o1, ConceptField o2) {
            return o1.getDisplayOrder() - o2.getDisplayOrder();
        }
    };
    private static final Comparator<ConceptField> REPORT_HISTORY_COMPARATOR = new Comparator<ConceptField>() {
        @Override
        public int compare(ConceptField o1, ConceptField o2) {
            return o1.getReportHistoryOrder() - o2.getReportHistoryOrder();
        }
    };
    private final Class<?> dtoClass;
    private final ClassInfo classInfo;
    private final HashMap<String, ConceptField> fields;
    private ExpressionParser parser;

    /**
     * Constructor creates the metadata for a concept.
     * 
     * @param concept
     *            the concept the metadata pertains to
     * @param dynamicInfo
     *            the metadata that is stored in the database. This is user defined and modifiable in the application itself
     */
    public ConceptMetadata(Class<?> concept, List<UserModifiableFieldMetadata> dynamicInfo) {
        dtoClass = concept;
        classInfo = dtoClass.getAnnotation(ClassInfo.class);
        setParser(new SpelExpressionParser());
        fields = new HashMap<String, ConceptField>();
        HashMap<String, UserModifiableFieldMetadata> dynamicMap = new HashMap<String, UserModifiableFieldMetadata>();
        for (UserModifiableFieldMetadata d : dynamicInfo) {
            dynamicMap.put(d.getFieldName(), d);
        }
        Map<String, Field> fieldMap = getAllFields();
        for (String name : fieldMap.keySet()) {
            Field f = fieldMap.get(name);
            FieldInfo fieldInfo = f.getAnnotation(FieldInfo.class);
            if (fieldInfo != null) {
                String fieldName = fieldInfo.name();
                // if the map key is longer than the field name, we're dealing
                // with
                // a field that's not a member of the DTO class, so we need to
                // specify the path used to reach it (that's what the prefix is)
                int nl = name.length();
                int fl = f.getName().length();
                String prefix = (nl > fl ? name.substring(0, nl - fl) : "");
                ConceptField field = new ConceptField(f, this, dynamicMap.get(fieldName), prefix);
                fields.put(fieldName, field);
            }
        }
    }

    /**
     * Get All Fields Map<STRING, Field>.
     * 
     * @return Gets Fields
     */
    private Map<String, Field> getAllFields() {
        return getFields(dtoClass, null);
    }

    /**
     * Get Fields.
     * 
     * @param cls
     *            Class<X>
     * @param prefix
     *            String
     * @return fields
     */
    private static Map<String, Field> getFields(Class<?> cls, String prefix) {
        Map<String, Field> fields;
        if (cls.getSuperclass() != null) {
            fields = getFields(cls.getSuperclass(), null);
        } else {
            fields = new HashMap<String, Field>();
        }
        for (Field f : cls.getDeclaredFields()) {
            CompositeField composite = f.getAnnotation(CompositeField.class);
            if (composite != null) {
                Map<String, Field> nestedFields = getFields(f.getType(), f.getName());
                fields.putAll(nestedFields);
            } else {
                fields.put((prefix == null ? "" : prefix + '.') + f.getName(), f);
            }
        }

        return fields;
    }

    /**
     * Get Persistent Class.
     * 
     * @param custom
     *            boolean
     * @return val
     */
    public Class<?> getPersistentClass(boolean custom) {
        return custom ? classInfo.customPersistentClass() : classInfo.fdbPersistentClass();
    }

    /**
     * No Value.
     * 
     * @param val
     *            String
     * @return String val
     */
    public static boolean noValue(String val) {
        return "".equals(val);
    }

    /**
     * Get Fields.
     * 
     * @return res
     */
    public List<ConceptField> getFields() {
        ArrayList<ConceptField> res = new ArrayList<ConceptField>(fields.values());
        // sort list based on display order
        Collections.sort(res, COMPARATOR);
        return res;
    }

    /**
     * Get FieldMap.
     * 
     * @return HashMap
     */
    public Map<String, ConceptField> getFieldMap() {
        return new HashMap<String, ConceptField>(fields);
    }

    /**
     * Get Query Fields.
     * 
     * @param custom
     *            boolean
     * @return ArrayList<ConceptField> Res
     */
    public List<ConceptField> getQueryFields(boolean custom) {
        ArrayList<ConceptField> res = new ArrayList<ConceptField>();
        for (ConceptField f : fields.values()) {
            if (f.isQueriable(custom) && f.includeInQueryResults()) {
                res.add(f);
            }
        }
        // sort list based on display order
        Collections.sort(res, COMPARATOR);
        return res;
    }

    /**
     * Get Query Fields.
     * 
     * @return res
     */
    public List<ConceptField> getQueryFields() {
        SortedSet<ConceptField> fields = new TreeSet<ConceptField>(new Comparator<ConceptField>() {
            @Override
            public int compare(ConceptField f1, ConceptField f2) {
                return f1.getName().compareTo(f2.getName());
            }
        });
        fields.addAll(new HashSet<ConceptField>(getQueryFields(true)));
        fields.addAll(new HashSet<ConceptField>(getQueryFields(false)));
        ArrayList<ConceptField> res = new ArrayList<ConceptField>(fields);
        // sort list based on display order
        Collections.sort(res, COMPARATOR);
        return res;
    }

    /**
     * Get Report Fields.
     * 
     * @param custom
     *            boolean
     * @return Concept Field
     */
    public List<ConceptField> getReportFields(boolean custom) {
        ArrayList<ConceptField> res = new ArrayList<ConceptField>();
        for (ConceptField f : fields.values()) {
            if (f.isReportable(custom) && f.includeInReports()) {
                res.add(f);
            }
        }
        // sort list based on display order
        Collections.sort(res, COMPARATOR);
        return res;
    }

    /**
     * Get Report Fields.
     * 
     * @return fields
     */
    public List<ConceptField> getReportFields() {
        SortedSet<ConceptField> fields = new TreeSet<ConceptField>(new Comparator<ConceptField>() {
            @Override
            public int compare(ConceptField f1, ConceptField f2) {
                return f1.getName().compareTo(f2.getName());
            }
        });
        fields.addAll(new HashSet<ConceptField>(getReportFields(true)));
        fields.addAll(new HashSet<ConceptField>(getReportFields(false)));
        ArrayList<ConceptField> res = new ArrayList<ConceptField>(fields);
        // sort list based on display order
        Collections.sort(res, COMPARATOR);
        return res;
    }

    /**
     * Get Report History Fields.
     * 
     * @return Concept Field
     */
    public List<ConceptField> getReportHistoryFields() {
        ArrayList<ConceptField> res = new ArrayList<ConceptField>();
        for (ConceptField f : fields.values()) {
            if (f.isReportHistory()) {
                res.add(f);
            }
        }
        // sort list based on Report History Order order
        Collections.sort(res, REPORT_HISTORY_COMPARATOR);
        return res;
    }

    /**
     * Get Required Fields.
     * 
     * @return ConceptField Array res
     */
    public List<ConceptField> getRequiredFields() {
        ArrayList<ConceptField> res = new ArrayList<ConceptField>();
        for (ConceptField f : fields.values()) {
            if (f.isRequired()) {
                res.add(f);
            }
        }
        return res;
    }

    /**
     * ConceptField Getter.
     * 
     * @param name
     *            - String
     * @return fields.get (name)
     */
    public ConceptField getField(String name) {
        return fields.get(name);
    }

    /**
     * Get Field By Property.
     * 
     * @param name
     *            String
     * @return Concept Field
     */
    public ConceptField getFieldByProperty(String name) {
        for (ConceptField f : fields.values()) {
            if (f.getPropertyName().equals(name)) {
                return f;
            }
        }
        return null;
    }

    /**
     * Get Concept Name.
     * 
     * @return classInfo.name
     */
    public String getConceptName() {
        return classInfo.name();
    }

    /**
     * Get DTO CLass.
     * 
     * @return dtoClass
     */
    Class<?> getDtoClass() {
        return dtoClass;
    }

    /**
     * Set Parser.
     * 
     * @param parser
     *            ExpressionParser
     */
    public void setParser(ExpressionParser parser) {
        this.parser = parser;
    }

    /**
     * ExpressionParser Getter.
     * 
     * @return parser
     */
    public ExpressionParser getParser() {
        return parser;
    }

}
