package gov.va.med.pharmacy.pecs.metadata;

import gov.va.med.pharmacy.pecs.domain.Lookup;

import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

/**
 * Information about a concept field.
 */
@Retention(RetentionPolicy.RUNTIME)
@Target(ElementType.FIELD)
public @interface FieldInfo {
    /**
     * A label for the field (legacy name).
     */
    String name();

    /**
     * Is this a required field?
     */
    boolean required() default false;

    /**
     * Is this field editable?
     */
    boolean editable() default true;

    /**
     * Is this an internal field? Internal fields are those that are not included in custom updates.
     */
    boolean internal() default false;

    /**
     * The property name for this DTO field. This should only be specified if it's different from the field name.
     */
    String property() default ""; // annotation params cannot be null

    /**
     * Is this a lookup field? Lookup fields can only take values from a restricted set, provided by {@link #lookupMethod()}.
     */
    boolean lookup() default false;

    /**
     * Is this a lookup type? Lookup type can only support object(s) found in the pecs.domain.lookup package.
     */
    Class<?> lookupType() default Lookup.class;

    /**
     * The lookup method that provides the allowable values for this field (if it is a lookup field).
     */
    String lookupMethod() default ""; // annotation params cannot be null

    /**
     * Number of lines needed to display the data for this field.
     */
    int displayLines() default 1;

    /**
     * The path to access the value for this field in the custom persistent class.
     */
    String customPersistentPath();

    /**
     * The path to access the value for this field in the FDB persistent class.
     */
    String fdbPersistentPath() default "";

    /**
     * This is used when the annotation actually refers not to the entire annotated field, but only to one of its instance
     * variables (this obviously makes sense only for non-primitive fields). It's mostly a hack for dealing with situations
     * where the same Java field can represent different metadata fields.
     */
    String nestedFieldName() default "";

    /**
     * Provides a way to prevent fields from being displayed on detail pages. This is useful for fields that are FDB-only.
     */
    boolean hideFromDetailDisplay() default false;

    /**
     * Provides a way to handle navigation through null fields. If the property corresponding to this field evaluates to null
     * for an object, the value returned by evaluating this expression should be used instead.
     */
    String nullPropertyReplacement() default "";

    /**
     * Is this field prohibited from being included in query results?
     */
    boolean nonQueriable() default false;

    /**
     * Is this field prohibited from being included in reports?
     */
    boolean nonReportable() default false;

    /**
     * Is this field included in a Report History report?
     */
    boolean reportHistory() default false;

    /**
     * If field is in Report History report, what order should it be displayed. Do not skip or repeat numbers for these values.
     * This value is also used to determine the actual column in the spreadsheet to determine if the value has been updated.
     */
    int reportHistoryOrder() default -1;

    /**
     * If field is in Report History report, should it's old and new values be compared?
     */
    boolean reportHistoryCompare() default false;
}
