package gov.va.med.pharmacy.pecs.net;

import gov.va.med.pharmacy.pecs.net.FtpClient.FtpPath;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;

import org.apache.commons.net.ftp.FTPFile;

/**
 * An interface to define commands available for the File Transfer Protocol.
 * 
 * @author vhaisaduffg
 * 
 */
public interface FileTransferCommands {

    /**
     * Change to the specified paths, creating them if necessary.
     * 
     * @param path
     *            one or more paths
     * @param create
     *            true if create
     * @return true is successful
     * @throws IOException
     *             if the path is not accessible or unable to create
     */
    boolean changeTo(FtpPath path, boolean create) throws IOException;

    /**
     * Change the current working directory of the FTP session.
     * 
     * @param dir
     *            the directory to change to
     * @return True if successfully completed, false if not.
     * @throws IOException
     *             If an I/O Exception occurs while communicating with the server.
     */
    boolean changeWorkingDirectory(String dir) throws IOException;

    /**
     * Creates a FTP socket connection to the passed in FTP Server.
     * 
     * @param server
     *            the FTP server to connect to
     * @throws IOException
     *             If an I/O Exception occurs while communicating with the server.
     */
    void connect(String server) throws IOException;

    /**
     * Sets the file type to be transferred.
     * 
     * @param fileType
     *            The file type (ASCII or Binary) of the file to be transferred.
     * @return True if successfully completed, false if not.
     * @throws IOException
     *             If an I/O Exception occurs while communicating with the server.
     */
    boolean setFileType(int fileType) throws IOException;

    /**
     * Logs in to the connected FTP server using the supplied user and password.
     * 
     * @param user
     *            the userName
     * @param password
     *            the password
     * @return True if successfully completed, false if not.
     * @throws IOException
     *             If an I/O Exception occurs while communicating with the server.
     */
    boolean login(String user, String password) throws IOException;

    /**
     * Logs out of the currently connected FTP server.
     * 
     * @return True if successfully completed, false if not.
     * @throws IOException
     *             If an I/O Exception occurs while communicating with the server.
     */
    boolean logout() throws IOException;

    /**
     * Returns a list of files in the current directory.
     * 
     * @return A list of files.
     * @throws IOException
     *             If an I/O Exception occurs while communicating with the server.
     */
    FTPFile[] listFiles() throws IOException;

    /**
     * Retrieves the fName file from the FTP server in to a temporary file on the local server.
     * 
     * @param fName
     *            the temporary file
     * @return True if successfully completed, false if not.
     * @throws IOException
     *             If an I/O Exception occurs while communicating with the server.
     */
    File retrieveToTemporaryFile(String fName) throws IOException;

    /**
     * A convenience method that combines the connect and login functionality.
     * 
     * @param server
     *            the FTP server to connect
     * @param user
     *            the username for login purposes
     * @param password
     *            the password for login purposes
     * @return True if successfully completed, false if not.
     * @throws IOException
     *             If an I/O Exception occurs while communicating with the server.
     */
    boolean connectAndLogin(String server, String user, String password) throws IOException;

    /**
     * Allows a file to be transferred to the FTP server and stored in the current directory.
     * 
     * @param name
     *            the name of the file being transferred
     * @param in
     *            the InputStream to the file
     * @return True if successfully completed, false if not.
     * @throws IOException
     *             If an I/O Exception occurs while communicating with the server.
     */
    boolean storeFile(String name, InputStream in) throws IOException;
}
