package gov.va.med.pharmacy.pecs.persistence;

import gov.va.med.pharmacy.pecs.domain.impl.va.DrugDrugInteractionVa;
import gov.va.med.pharmacy.pecs.domain.impl.va.DrugDrugInteractionVersionVa;
import gov.va.med.pharmacy.pecs.exception.CustomizationServiceException;

import java.util.List;

import org.apache.log4j.Logger;
import org.hibernate.Query;
import org.springframework.stereotype.Repository;

/**
 * Responsible for getting data out of of the database.
 * 
 * @author vhaisacarusr
 * 
 */
@Repository
public final class DrugDrugInteractionDAOImpl extends HibernateDAOImpl<DrugDrugInteractionVa> {

    /** The logger. */
    private static Logger logger = Logger.getLogger(DrugDrugInteractionDAOImpl.class);

    /** The duplicate check. */
    private static String DUPLICATE_CHECK = "duplicateCheck";

    /** The find by interaction id. */
    private static String FIND_BY_INTERACTION_ID = "findByInteractionId";

    /** The find ddi by fdb id. */
    private static String FIND_DDI_BY_FDB_ID = "findDDIByFdbId";

    /** The find ddi by interaction id. */
    private static String FIND_DDI_BY_INTERACTION_ID = "findDDIByInteractionId";

    /** The find version by id. */
    private static String FIND_VERSION_BY_ID = "findVersionById";

    /** The delete monograph query. */
    private static String DELETE_MONOGRAPH_QUERY = "deletedMonographReportData";

    /** The find interaction descriptions. */
    private static String FIND_INTERACTION_DESCRIPTIONS = "findInteractionDescriptions";

    // private static String[] invisibleDrugPairStatuses = new String[]
    // {"Deleted", "Rejected"};

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.persistence.HibernateDAOImpl#getPersistentClass()
     */
    @Override
    protected Class<DrugDrugInteractionVa> getPersistentClass() {
        return DrugDrugInteractionVa.class;
    }

    /**
     * Checks to see if another Custom Drug-Drug Interaction exists with the same fdbId, or reverse fdbId, Severity Level Code,
     * description or reverse description. If one or more is found, the first one is returned. If none are found, a null value
     * is returned.
     * 
     * @param fdbId
     *            the FDB id of the interaction in question
     * @param reverseFdbId
     *            the FDB ID of the reverse interaction
     * @param severityLevelCode
     *            the severity level of hte interaction.
     * @return a list of interactions
     */
    @SuppressWarnings("unchecked")
    public List<DrugDrugInteractionVa> checkForDuplicate(Long fdbId, Long reverseFdbId, String severityLevelCode) {
        Query query = getSession().getNamedQuery(DUPLICATE_CHECK);
        query.setLong("fdbId", fdbId);
        query.setLong("reverseFdbId", reverseFdbId);
        query.setString("severityLevelCode", severityLevelCode);
        List<DrugDrugInteractionVa> list = query.list();
        if (list.size() > 0) {
            return list;
        } else {
            return null;
        }
    }

    /**
     * Retrieves the custom interactions for associated with a FDB record.
     * 
     * @param fdbid
     *            the ID of the FDB record
     * @return a list of custom interactions
     */
    @SuppressWarnings("unchecked")
    public List<DrugDrugInteractionVa> findDDIByFdbId(Long fdbid) {
        Query query = getSession().getNamedQuery(FIND_DDI_BY_FDB_ID);
        query.setLong("fdbId", fdbid);
        return query.list();
    }

    /**
     * Retrieves all the custom interactions by Interaction Id.
     * 
     * @param interactionId
     *            the interaction id
     * @return the list
     */
    @SuppressWarnings("unchecked")
    public List<DrugDrugInteractionVa> findDDIListByInteractionID(Long interactionId) {
        Query query = getSession().getNamedQuery(FIND_DDI_BY_INTERACTION_ID);
        query.setLong("interactionId", interactionId);
        return query.list();
    }

    /**
     * retrieves the Drug Drug Interaction for a interaction ID. This is the object that contains all versions.
     * 
     * @param interactionId
     *            the interaction id to serch on
     * @return a object that holds all of the custom drug drug interaction information
     */
    @SuppressWarnings("unchecked")
    public DrugDrugInteractionVa findDDIByInteractionId(Long interactionId) {
        Query query = getSession().getNamedQuery(FIND_DDI_BY_INTERACTION_ID);
        query.setLong("interactionId", interactionId);
        List<DrugDrugInteractionVa> list = query.list();
        if (list.size() != 1) {
            String msg;
            if (list.size() == 0) {
                msg = "Could not find DrugDrugInteraction with Interaction Id = " + interactionId;
            } else {
                msg = "Too many DrugDrugInteractions found with Interaction Id = " + interactionId;
            }
            logger.info(msg);
            throw new CustomizationServiceException(msg);
        }
        return list.get(0);
    }

    /**
     * retrieves the current version of the DDI for the interaction ID.
     * 
     * @param interactionId
     *            the interaction ID of the DDI
     * @return a single DDI
     */
    @SuppressWarnings("unchecked")
    public DrugDrugInteractionVersionVa findByInteractionId(Long interactionId) {
        Query query = getSession().getNamedQuery(FIND_BY_INTERACTION_ID);
        query.setLong("interactionId", interactionId);
        List<DrugDrugInteractionVersionVa> list = query.list();
        if (list.size() != 1) {
            String msg;
            if (list.size() == 0) {
                msg = "Could not find DrugDrugInteraction with Interaction Id = " + interactionId;
            } else {
                msg = "Too many DrugDrugInteractions found with Interaction Id = " + interactionId;
            }
            logger.info(msg);
            throw new CustomizationServiceException(msg);
        }
        return list.get(0);
    }

    /**
     * Retrieves a Custom DDI of a specific version.
     * 
     * @param versionId
     *            the version ID (recordId)
     * @return a custom drug drug interaction record
     */
    @SuppressWarnings("unchecked")
    public DrugDrugInteractionVersionVa findVersionById(Long versionId) {
        Query query = getSession().getNamedQuery(FIND_VERSION_BY_ID);
        query.setLong("versionId", versionId);
        List<DrugDrugInteractionVersionVa> list = query.list();
        if (list.size() != 1) {
            String msg;
            if (list.size() == 0) {
                msg = "Could not find DrugDrugInteractionVersion with Version Id = " + versionId.toString();
            } else {
                msg = "Too many DrugDrugInteractionVersions found with Version Id = " + versionId.toString();
            }
            logger.info(msg);
            throw new CustomizationServiceException(msg);
        }
        return list.get(0);
    }

    /**
     * 
     * Checks if the interaction has drug pairs that are not rejected or deleted.
     * 
     * @param ddi
     *            the drug drug interaction
     * @return true if the interaction has drug pairs that are not rejected or deleted
     */
    public boolean hasVisibleDrugPairs(DrugDrugInteractionVa ddi) {
        return hasDrugPairsInOtherStatuses(ddi, "Deleted", "Rejected");
    }

    /**
     * Checks the interaction has drug pairs that are only reviewed and approved.
     * 
     * @param ddi
     *            the drug drug interaction
     * @return true if the interaction has drug pairs that are only reviewed and approved
     */
    public boolean allDrugPairsReviewedApproved(DrugDrugInteractionVa ddi) {
        return !hasDrugPairsInOtherStatuses(ddi, "Deleted", "Rejected", "Reviewed", "Approved");
    }

    /**
     * Checks if all drug pairs for an interaction are approved.
     * 
     * @param ddi
     *            the drug drug interaction
     * @return true if all drug pairs are approved
     */
    public boolean allDrugPairsApproved(DrugDrugInteractionVa ddi) {
        return !hasDrugPairsInOtherStatuses(ddi, "Deleted", "Rejected", "Approved");
    }

    /**
     * Checks for drug pairs in other statuses.
     * 
     * @param ddi
     *            the drug drug interaction
     * @param statusNames
     *            the status' to check for
     * @return true if the interaction has drug pairs in the supplied status
     */
    private boolean hasDrugPairsInOtherStatuses(DrugDrugInteractionVa ddi, String... statusNames) {
        if (statusNames.length == 0) {
            return false;
        }
        StringBuffer inValues = new StringBuffer();
        for (String status : statusNames) {
            if (inValues.length() == 0) {
                // first element
                inValues.append('(');
            } else {
                inValues.append(',');
            }
            inValues.append('\'').append(status).append('\'');
        }
        inValues.append(')');
        String hql =
                "select count(*) from DrugPairVa as t1 where t1.interaction.interactionId = " + ddi.getInteractionId()
                        + " and t1.exportManager.currentVersion.status.name not in " + inValues;
        long count = (Long) getSession().createQuery(hql).iterate().next();
        return count > 0;
    }

    /**
     * Finds the DDI with monographs that have been deleted.
     * 
     * @return list of interactions.
     */
    @SuppressWarnings("unchecked")
    public List<DrugDrugInteractionVersionVa> getDeleteMonographData() {
        return getSession().getNamedQuery(DELETE_MONOGRAPH_QUERY).list();
    }

    /**
     * I forget what this returns but it belive it is a list of string arrays.
     * 
     * @return a list of descriptions
     */
    @SuppressWarnings("rawtypes")
    public List getInteractionDescriptions() {
        return getSession().getNamedQuery(FIND_INTERACTION_DESCRIPTIONS).list();
    }

}
