package gov.va.med.pharmacy.pecs.persistence;

import gov.va.med.pharmacy.pecs.domain.impl.fdb.RoutedGenericFdb;
import gov.va.med.pharmacy.pecs.domain.impl.fdb.RoutedGenericFdbLite;
import gov.va.med.pharmacy.pecs.domain.impl.va.DrugPairVa;
import gov.va.med.pharmacy.pecs.exception.CustomizationServiceException;

import java.math.BigDecimal;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.ObjectNotFoundException;
import org.hibernate.Query;
import org.springframework.stereotype.Repository;

/**
 * Implementation of the Drug Pair DAO.
 * 
 * @author VHAISACARUSR
 */
@Repository
public class DrugPairDAOImpl extends HibernateDAOImpl<DrugPairVa> {
    private static final Log LOGGER = LogFactory.getLog(DrugPairDAOImpl.class);
    private static final String FIND_DP_QUERY = "findDrugPairQuery";
    private static final String FIND_DP_FDB_QUERY = "findDrugPairFdbQuery";
    private static final String FIND_DP_BY_DDI_QUERY = "findDrugPairsByInteractionIdQuery";
    private static final String FIND_DP_STATUS_COUNTS_BY_DDI_QUERY = "findDrugPairsStatusCountsByInteractionIdQuery";
    private static final String DELETE_DRUG_PAIRS_STEP1 = "deleteDrugPairsStep1";
    private static final String DELETE_DRUG_PAIRS_STEP2 = "deleteDrugPairsStep2";
    private static final String DELETE_DRUG_PAIRS_STEP3 = "deleteDrugPairsStep3";
    private static final String FIND_DP_BY_ROUTE_QUERY = "findDrugPairByRouteQuery";
    private static final String UPDATE_DRUG_PAIR_VERSION_EXPORTED = "updateVersionAsExported";
    private static final String UPDATE_DRUG_PAIR_EXPORTED = "updateAsExported";

    @Override
    protected final Class<DrugPairVa> getPersistentClass() {
        return DrugPairVa.class;
    }

    /**
     * Finds a drug pair by interaction ID.
     * 
     * @param rtgenId1
     *            The routed generic 1 ID
     * @param rtgenId2
     *            The routed generic 2 ID
     * @param interactionId
     *            the interaction ID
     * @return a single <code>DrugPairVa</code>
     */
    @SuppressWarnings("unchecked")
    public final DrugPairVa findDrugPair(long rtgenId1, long rtgenId2, long interactionId) {
        Query query = getSession().getNamedQuery(FIND_DP_QUERY);
        query.setLong("rtgenId1", rtgenId1);
        query.setLong("rtgenId2", rtgenId2);
        query.setLong("interactionId", interactionId);
        Iterator<DrugPairVa> iter = query.iterate();
        DrugPairVa res = iter.hasNext() ? iter.next() : null;
        if (iter.hasNext()) {
            String msg =
                    "More than one drug pair found for the combination: " + rtgenId1 + '/' + rtgenId2 + '/' + interactionId;
            throw new CustomizationServiceException(msg);
        }

        return res;
    }

    /**
     * Finds a drug pair by FDB ID.
     * 
     * @param rtgenId1
     *            The routed generic 1 ID
     * @param rtgenId2
     *            The routed generic 2 ID
     * @param fdbId
     *            The FDB ID
     * @return a single <code>DrugPairVa</code>
     */
    @SuppressWarnings("unchecked")
    public final DrugPairVa findDrugPairByFdb(long rtgenId1, long rtgenId2, long fdbId) {
        Query query = getSession().getNamedQuery(FIND_DP_FDB_QUERY);
        query.setLong("rtgenId1", rtgenId1);
        query.setLong("rtgenId2", rtgenId2);
        query.setLong("fdbId", fdbId);
        Iterator<DrugPairVa> iter = query.iterate();
        DrugPairVa res = new DrugPairVa();

        try {

            res = iter.hasNext() ? iter.next() : null;

        } catch (Exception e) {

            if (iter.hasNext()) {

                String msg =

                "More than one drug pair found for the combination: " + rtgenId1 + '/' + rtgenId2 + '/' + fdbId;

                throw new CustomizationServiceException(msg);

            }

        }
        return res;
    }

    /**
     * Finds a drug pair by FDB ID. This query being called here has been updated to find all Forward and Reverse Drug Pairs
     * with the passed in Routed Generic Ids and FDB Id. The reverse FDB Id is calculated within the query.
     * 
     * @param rtgenId1
     *            The routed generic 1 ID
     * @param rtgenId2
     *            The routed generic 2 ID
     * @param fdbId
     *            The FDB ID
     * @return a single <code>DrugPairVa</code>
     */
    @SuppressWarnings("unchecked")
    public final List<DrugPairVa> findDrugPairsByFdb(long rtgenId1, long rtgenId2, long fdbId) {
        Query query = getSession().getNamedQuery(FIND_DP_FDB_QUERY);
        query.setLong("rtgenId1", rtgenId1);
        query.setLong("rtgenId2", rtgenId2);
        query.setLong("fdbId", fdbId);
        List<DrugPairVa> drugPairs = query.list();

        return drugPairs;
    }

    /**
     * Find all drug pairs associated with the interaction having the given id, that are in states other than deleted or
     * rejected.
     * 
     * @param interactionId
     *            the id of the interaction
     * @return List of <code>DrugPairVa</code>'s
     */
    @SuppressWarnings("unchecked")
    public final List<DrugPairVa> findDrugPairsByInteractionId(long interactionId) {
        Query query = getSession().getNamedQuery(FIND_DP_BY_DDI_QUERY);
        query.setLong("interactionId", interactionId);
        return query.list();
    }

    /**
     * Provides a Map of all the statuses and counts of drug pairs associated with the interaction having the given id.
     * 
     * @param interactionId
     *            the id of the interaction
     * @return <code>Map\<String, long\></code> where the key is the status description and the value is the count of drug pairs
     *         with that status.
     */
    @SuppressWarnings("rawtypes")
    public final Map<String, Long> findDrugPairsStatusCountsByInteractionId(long interactionId) {
        Query query = getSession().getNamedQuery(FIND_DP_STATUS_COUNTS_BY_DDI_QUERY);
        query.setLong("interactionId", interactionId);
        Iterator results = query.list().iterator();

        Map<String, Long> resultMap = new HashMap<String, Long>();
        while (results.hasNext()) {
            Object[] row = (Object[]) results.next();
            String status = (String) row[0];
            // Oracle returns counts as BigDecimals, so we need to convert to Long for ease of use.
            BigDecimal bdCount = (BigDecimal) row[1];
            Long count = bdCount.longValue();
            resultMap.put(status, count);
        }
        return resultMap;
    }

    /**
     * Executes the SQL necessary to delete Drug Pairs associated with a Routed Generic that has been deleted from the FDB
     * database.
     * 
     * @param versionUser
     *            the name of the user executing the process
     * @param actionBy
     *            the user who should be placed in to the Action By column
     * @param comment
     *            the comment to place on the deleted Drug Pairs
     * @param filterComment
     *            a comment that will filter out old Deleted Drug Pairs
     */
    public final void deleteDrugPairsWithDeletedRoutedGeneric(String versionUser, String actionBy, String comment,
            String filterComment) {
        Query step1 = getSession().getNamedQuery(DELETE_DRUG_PAIRS_STEP1);
        Query step2 = getSession().getNamedQuery(DELETE_DRUG_PAIRS_STEP2);
        Query step3 = getSession().getNamedQuery(DELETE_DRUG_PAIRS_STEP3);
        step1.setParameter("versionUser", versionUser);
        step1.setParameter("actionBy", actionBy);
        step1.setParameter("comment", comment);
        step2.setParameter("comment", comment);
        step3.setParameter("comment", comment);
        step1.executeUpdate();
        step2.executeUpdate();
        step3.executeUpdate();
    }

    /**
     * Find drug pairs by routed generic id.
     * 
     * @param routedGenericId
     *            the routed generic id
     * @return the list
     */
    @SuppressWarnings("unchecked")
    public final List<DrugPairVa> findDrugPairsByRoutedGenericId(Long routedGenericId) {
        Query query = getSession().getNamedQuery(FIND_DP_BY_ROUTE_QUERY);
        query.setLong("rtgenId1", routedGenericId);
        return query.list();
    }

    /**
     * Updates the exportdDate for a DrugPairVersionVa object.
     * 
     * @param interactionId
     *            the Interaction Identifier
     * @param routedGenericId1
     *            a Routed Generic Identifier
     * @param routedGenericId2
     *            a Routed Generic Identifier
     * @param fileDate
     *            the date to set exportedDate
     * @return the number of rows that were updated
     */
    public final int updateDrugPairAsExported(Long interactionId, Long routedGenericId1, Long routedGenericId2, Date fileDate) {
        Query query = getSession().getNamedQuery(UPDATE_DRUG_PAIR_VERSION_EXPORTED);
        query.setParameter("exportDate", fileDate);
        query.setParameter("rtgenId1", routedGenericId1);
        query.setParameter("rtgenId2", routedGenericId2);
        query.setParameter("interactionId", interactionId);
        query.executeUpdate();

        query = getSession().getNamedQuery(UPDATE_DRUG_PAIR_EXPORTED);
        query.setParameter("rtgenId1", routedGenericId1);
        query.setParameter("rtgenId2", routedGenericId2);
        query.setParameter("interactionId", interactionId);
        return query.executeUpdate();
    }

    /**
     * Gets the routed generic drugs.
     * 
     * @return the routed generic drugs
     */
    public final List<RoutedGenericFdb> getRoutedGenericDrugs() {

        try {
            return queryRoutedGenericDrugs();
        } catch (ObjectNotFoundException e) {
            LOGGER.error("ObjectNotFoundException caught trying to retrieve Routed Generic Drugs. Clearing cached drugs...");
            getSessionFactory().getCache().evictQueryRegion("RoutedGenericFdbQuery");

        }
        return queryRoutedGenericDrugs();
    }

    /**
     * Query routed generic drugs.
     * 
     * @return the list
     */
    @SuppressWarnings("unchecked")
    private List<RoutedGenericFdb> queryRoutedGenericDrugs() {
        String queryString = "from RoutedGenericFdb order by description";
        Query query = getSession().createQuery(queryString);

        query.setCacheable(true);
        query.setCacheRegion("RoutedGenericFdbQuery");
        return query.list();
    }

    /**
     * Gets the routed generic drugs lite.
     * 
     * @return the routed generic drugs
     */
    public final List<RoutedGenericFdbLite> getRoutedGenericDrugsLite() {

        try {
            return queryRoutedGenericDrugsLite();
        } catch (ObjectNotFoundException e) {
            LOGGER.error("ObjectNotFoundException caught trying to retrieve Routed Generic Drugs. Clearing cached drugs...");
            getSessionFactory().getCache().evictQueryRegion("RoutedGenericFdbLiteQuery");

        }
        return queryRoutedGenericDrugsLite();
    }

    /**
     * Query routed generic drugs lite.
     * 
     * @return the list
     */
    @SuppressWarnings("unchecked")
    private List<RoutedGenericFdbLite> queryRoutedGenericDrugsLite() {
        String queryString = "from RoutedGenericFdbLite order by description";
        Query query = getSession().createQuery(queryString);

        query.setCacheable(true);
        query.setCacheRegion("RoutedGenericFdbLiteQuery");
        return query.list();
    }

}
