package gov.va.med.pharmacy.pecs.persistence;

import gov.va.med.pharmacy.pecs.query.ConceptQuery;
import gov.va.med.pharmacy.pecs.query.Page;

import java.io.Serializable;
import java.util.Collections;
import java.util.List;

import javax.annotation.Resource;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.criterion.Example;
import org.springframework.orm.hibernate3.HibernateTemplate;

/**
 * HibernateDAOImpl implements HibernateDAO.
 * 
 * @param <T>
 *            the generic type
 * @author DOMAINOVERSD
 */
public abstract class HibernateDAOImpl<T> implements HibernateDAO<T> {

    /** The Constant LOGGER. */
    private static final Log LOGGER = LogFactory.getLog(HibernateDAOImpl.class);
    /**
     * Hibernate Template.
     * 
     */
    protected HibernateTemplate hibernateTemplate;

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.persistence.HibernateDAO#getSession()
     */
    @Override
    public Session getSession() {
        return hibernateTemplate.getSessionFactory().getCurrentSession();
    }

    /**
     * Sesstion Factory Getter.
     * 
     * @return hiberntateTemplate Session Factory.
     */
    public SessionFactory getSessionFactory() {
        return hibernateTemplate.getSessionFactory();
    }

    /**
     * Set SessionFactory.
     * 
     * @param sessionFactory
     *            the new session factory
     */
    @Resource(name = "pecsSessionFactory")
    public void setSessionFactory(SessionFactory sessionFactory) {
        hibernateTemplate = new HibernateTemplate(sessionFactory);
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.persistence.HibernateDAO#findByQuery(gov.va.med.pharmacy.pecs.query.ConceptQuery, boolean)
     */
    @Override
    public List<?> findByQuery(final ConceptQuery query, boolean custom) {
        if (query == null) {
            throw new IllegalArgumentException("Required query parameter is null");
        }
        if (LOGGER.is   ugEnabled()) {
            LOGGER.debug("Base DAO executing HQL query: " + query.toHQL(custom));
        }

        List<?> modelList = getSession().createQuery(query.toHQL(custom)).list();
        if (modelList == null || modelList.size() == 0) {
            return Collections.emptyList();
        }

        return modelList;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.persistence.HibernateDAO#findByQuery(java.lang.String, java.lang.Class)
     */
    @Override
    @SuppressWarnings("unchecked")
    public <X> List<X> findByQuery(String query, Class<X> clz) {
        if (query == null) {
            throw new IllegalArgumentException("Required query parameter is null");
        }
        if (LOGGER.is   ugEnabled()) {
            LOGGER.debug("Base DAO executing HQL query: " + query);
        }

        List<X> modelList = getSession().createQuery(query).list();
        if (modelList.isEmpty()) {
            return Collections.emptyList();
        }

        return modelList;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.persistence.HibernateDAO#findByQueryByPage(java.lang.String, int)
     */
    @Override
    public Page findByQueryByPage(String query, int page) {
        if (query == null || query.equals("")) {
            throw new IllegalArgumentException("Required query parameter is null");
        }
        if (page < 1) {
            throw new IllegalArgumentException("Required page parameter cannot be less than 1");
        }

        Page queryPage = new Page(query, getSession(), page, Page.DEFAULT_SIZE);
        if (queryPage == null || queryPage.getResults() == null) {
            return null;
        }

        return queryPage;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.persistence.HibernateDAO#findAll(java.lang.Class)
     */
    @Override
    @SuppressWarnings("unchecked")
    public <X> List<X> findAll(Class<X> clz) {
        LOGGER.debug("finding all " + clz.getName() + " instances");
        Collections.emptyList();
        try {
            String queryString = "from " + clz.getName();
            Query queryObject = getSession().createQuery(queryString);
            return queryObject.list();
        } catch (RuntimeException re) {
            LOGGER.error("find all failed", re);
            throw re;
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.persistence.HibernateDAO#findAll()
     */
    @Override
    @SuppressWarnings("unchecked")
    public List<T> findAll() {
        LOGGER.debug("finding all " + getPersistentClass().getSimpleName() + " instances");
        try {
            String queryString = "from " + getPersistentClass().getName();
            Query queryObject = getSession().createQuery(queryString);
            return queryObject.list();
        } catch (RuntimeException re) {
            LOGGER.error("find all failed", re);
            throw re;
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.persistence.HibernateDAO#findById(java.io.Serializable)
     */
    @Override
    @SuppressWarnings("unchecked")
    public T findById(Serializable id) {
        LOGGER.debug("getting " + getPersistentClass().getSimpleName() + " instance with id: " + id);
        try {
            T instance = (T) getSession().get(getPersistentClass(), id);
            return instance;
        } catch (RuntimeException re) {
            LOGGER.error("get failed for " + getPersistentClass().getSimpleName() + " instance with id: " + id, re);
            throw re;
        }
    }

    /**
     * Find by id.
     * 
     * @param <X>
     *            the generic type
     * @param id
     *            the id
     * @param clz
     *            the clz
     * @return the x
     */
    @SuppressWarnings("unchecked")
    public <X> X findById(Serializable id, Class<X> clz) {
        LOGGER.debug("getting " + clz.getName() + " instance with id: " + id);
        try {
            X instance = (X) getSession().get(clz, id);
            return instance;
        } catch (RuntimeException re) {
            LOGGER.error("get failed", re);
            throw re;
        }
    }

    /**
     * Gets the persistent class.
     * 
     * @return the persistent class
     */
    protected abstract Class<T> getPersistentClass();

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.persistence.HibernateDAO#merge(java.lang.Object)
     */
    @Override
    @SuppressWarnings("unchecked")
    public T merge(T detachedInstance) {
        if (detachedInstance == null) {
            return null;
        }

        try {
            T result = (T) getSession().merge(detachedInstance);
            LOGGER.debug("merge successful");
            return result;
        } catch (RuntimeException re) {
            LOGGER.error("merge failed", re);
            throw re;
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.persistence.HibernateDAO#saveOrUpdate(java.lang.Object)
     */
    @Override
    public void saveOrUpdate(Object/* T */instance) {
        if (instance == null) {
            return;
        }
        LOGGER.debug("attaching dirty " + instance.getClass().getSimpleName() + " instance");
        try {
            getSession().saveOrUpdate(instance);
            LOGGER.debug("attach successful");
        } catch (RuntimeException re) {
            LOGGER.error("attach failed", re);
            throw re;
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.persistence.HibernateDAO#update(java.lang.Object)
     */
    @Override
    public void update(Object/* T */detachedInstance) {
        if (detachedInstance == null) {
            return;
        }
        LOGGER.debug("update " + detachedInstance.getClass().getSimpleName() + " detachedInstance");
        try {
            getSession().update(detachedInstance);
            LOGGER.debug("update successful");
        } catch (RuntimeException re) {
            LOGGER.error("update failed", re);
            throw re;
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.persistence.HibernateDAO#save(java.lang.Object)
     */
    @Override
    public Serializable save(Object/* T */transientInstance) {
        if (transientInstance == null) {
            return null;
        }
        LOGGER.debug("saving " + transientInstance.getClass().getSimpleName() + " instance");
        try {
            Serializable s = getSession().save(transientInstance); // save the
                                                                   // object
            getSession().flush(); // force the insert
            getSession().refresh(transientInstance); // re-read (after triggers
                                                     // execute)
            return s;
        } catch (RuntimeException re) {
            LOGGER.error("save failed", re);
            throw re;
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.persistence.HibernateDAO#delete(java.lang.Object)
     */
    @Override
    public void delete(Object/* T */persistentInstance) {
        if (persistentInstance == null) {
            return;
        }
        LOGGER.debug("deleting " + persistentInstance.getClass().getSimpleName() + " instance");
        try {
            getSession().delete(persistentInstance);
            getSession().flush();
            LOGGER.debug("delete successful");
        } catch (RuntimeException re) {
            LOGGER.error("delete failed", re);
            throw re;
        }
    }

    /**
     * Find by example.
     * 
     * @param instance
     *            the instance
     * @return the list
     */
    @SuppressWarnings("unchecked")
    public List<T> findByExample(T instance) {
        LOGGER.debug("finding ConfigQueryVa instance by example");
        try {
            List<T> results = getSession().createCriteria(instance.getClass().getName()).add(Example.create(instance)).list();
            LOGGER.debug("find by example successful, result size: " + results.size());
            return results;
        } catch (RuntimeException re) {
            LOGGER.error("find by example failed", re);
            throw re;
        }
    }

    /**
     * Determine the number of results for the given query.
     * 
     * @param hqlQuery
     *            a Hibernate query string (no select, should start with 'from')
     * @return the result count
     */
    @Override
    public long resultCount(String hqlQuery) {
        return (Long) getSession().createQuery("select count(*) " + hqlQuery).iterate().next();
    }

}
