package gov.va.med.pharmacy.pecs.persistence;

import gov.va.med.pharmacy.pecs.domain.Lookup;
import gov.va.med.pharmacy.pecs.domain.impl.va.ReferenceTextVa;
import gov.va.med.pharmacy.pecs.exception.CustomizationServiceException;

import java.util.ArrayList;
import java.util.List;

import javax.annotation.Resource;

import org.apache.log4j.Logger;
import org.hibernate.SessionFactory;
import org.springframework.orm.hibernate3.support.HibernateDaoSupport;
import org.springframework.stereotype.Repository;

/**
 * A DAO implementation to retrieve Terminology (Lookup) objects from the database.
 * 
 * @author vhaisaduffg
 * 
 */
@Repository
public class TerminologyDAOImpl extends HibernateDaoSupport implements TerminologyDAO {

    private final Logger logger = Logger.getLogger(TerminologyDAOImpl.class);

    private static final String GETREFERENCE = "getReference";
    private static final String GETREFERENCEBYID = "getReferenceById";
    private static final String GETREFERENCEBYNAME = "getReferenceByName";

    /**
     * A private method to actually perform the retrieval from the database.
     * 
     * @param hql
     *            the HQL to execute
     * @param clazz
     *            the Class being retrieved. Used only in a debug message.
     * 
     * @return List
     */
    @SuppressWarnings("rawtypes")
    private List getLookups(String hql, Class clazz) {
        // getHibernateTemplate().setCacheQueries(true);
        List lookup = getHibernateTemplate().find(hql);

        if (logger.is   ugEnabled()) {
            logger.debug("Returning " + lookup.size() + " " + clazz.getName() + " from getLookup");
        }
        return lookup;
    }

    /**
     * A method to retrieve the objects for a particular class.
     * 
     * @param clazz
     *            the persistent class to retrieve.
     * @return List
     */
    @Override
    @SuppressWarnings("rawtypes")
    public final List getLookups(Class clazz) {
        StringBuffer hql = initHQL(clazz);
        return getLookups(hql.toString(), clazz);
    }

    /**
     * A method to retrieve the objects for a particular class, order by the sortOrder attribute.
     * 
     * @param clazz
     *            the persistent class to retrieve.
     * @return List
     */
    @Override
    @SuppressWarnings("rawtypes")
    public final List getSortedLookups(Class clazz) {
        StringBuffer hql = initHQL(clazz);
        hql.append(" c order by c.sortOrder ASC");
        return getLookups(hql.toString(), clazz);
    }

    /**
     * A method to retrieve the active objects for a particular class, order by the sortOrder attribute.
     * 
     * @param clazz
     *            the persistent class to retrieve.
     * @return List
     */
    @Override
    @SuppressWarnings({ "rawtypes", "unchecked" })
    public final List<?> getActiveSortedLookups(Class clazz) {
        List<Lookup> types = getSortedLookups(clazz);
        List returnTypes = new ArrayList();
        for (Lookup lookup : types) {
            if (lookup.isActive()) {
                returnTypes.add(lookup);
            }
        }
        return returnTypes;
    }

    /**
     * A method to retrieve a particular object by the identifier.
     * 
     * @param clazz
     *            the persistent class to retrieve
     * @param id
     *            the identifier of the object to retrieve.
     * @return Object
     */
    @Override
    @SuppressWarnings("unchecked")
    public final Object getById(@SuppressWarnings("rawtypes") Class clazz, Long id) {
        try {
            return getHibernateTemplate().get(clazz, id);
        } catch (Exception e) {
            logger.error(e.getMessage(), e);
            throw new CustomizationServiceException(e.getMessage(), e);
        }
    }

    /**
     * Allows a Lookup to be retrieved by the code value. An exception is thrown if more than one Lookup object has that same
     * code. A null value is returned if no Lookups contain the code.
     * 
     * @param clazz
     *            the persistent class to retrieve
     * @param code
     *            the code of the object to retrieve.
     * @return Object
     */
    @Override
    @SuppressWarnings("rawtypes")
    public final Object getByCode(Class clazz, String code) {
        try {
            if (code == null) {
                return null;
            }
            StringBuffer hql = initHQL(clazz);
            hql.append(" c where code = ?");
            List lookups = getHibernateTemplate().find(hql.toString(), code);
            if (lookups.size() == 1) {
                return lookups.get(0);
            } else if (lookups.size() == 0) {
                return null;
            } else {
                throw new CustomizationServiceException("More than one " + clazz.getName() + " object was found for code = "
                        + code);
            }
        } catch (Exception e) {
            logger.error(e.getMessage(), e);
            throw new CustomizationServiceException(e.getMessage(), e);
        }
    }

    /**
     * Allows a Lookup to be retrieved by the code value. An exception is thrown if more than one Lookup object has that same
     * code. A null value is returned if no Lookups contain the code.
     * 
     * @param clazz
     *            the persistent class to retrieve
     * @param code
     *            the code of the object to retrieve.
     * @return Object
     */
    @Override
    @SuppressWarnings("rawtypes")
    public final Object getByCode(Class clazz, Long code) {
        try {
            if (code == null) {
                return null;
            }
            StringBuffer hql = initHQL(clazz);
            hql.append(" c where code = ?");
            List lookups = getHibernateTemplate().find(hql.toString(), code);
            if (lookups.size() == 1) {
                return lookups.get(0);
            } else if (lookups.size() == 0) {
                return null;
            } else {
                throw new CustomizationServiceException("More than one " + clazz.getName() + " object was found for code = "
                        + code);
            }
        } catch (Exception e) {
            logger.error(e.getMessage(), e);
            throw new CustomizationServiceException(e.getMessage(), e);
        }
    }

    /**
     * Initializes the HQL that will be executed by setting up the "FROM" HQL statement.
     * 
     * @param clazz
     *            the persistent class that is being acted upon
     * @return StringBuffer
     */
    @SuppressWarnings("rawtypes")
    private final StringBuffer initHQL(Class clazz) {
        StringBuffer sb = new StringBuffer();
        sb.append("from ");
        sb.append(clazz.getName());
        return sb;
    }

    /**
     * Retrieves the specified type of reference data from the database.
     * 
     * @param type
     *            the type of reference data to retrieve.
     * @return List
     */
    @Override
    @SuppressWarnings("rawtypes")
    public final List getReferenceData(String type) {
        return getHibernateTemplate().findByNamedQuery(GETREFERENCE, type);
    }

    /**
     * Retrieves a specific refernce type by name.
     * 
     * @param type
     *            the type of reference data to retrieve.
     * @param name
     *            the name of the reference object to retrieve
     * @return ReferenceTextVa
     */
    @Override
    public final ReferenceTextVa getReferenceByName(String type, String name) {
        List<Object> params = new ArrayList<Object>();
        params.add(type);
        params.add(name);
        @SuppressWarnings("rawtypes")
        List list = getHibernateTemplate().findByNamedQuery(GETREFERENCEBYNAME, params.toArray());
        if (list.size() == 0) {
            return null;
        } else if (list.size() > 1) {
            String msg = "Too many Reference Data rows found for Type = " + type + " with the Name = " + name;
            throw new CustomizationServiceException(msg);
        }
        return (ReferenceTextVa) list.get(0);
    }

    /**
     * Retrieves a specific refernce type by its identifier.
     * 
     * @param type
     *            the type of reference data to retrieve.
     * @param id
     *            the identifier of the reference object to retrieve
     * @return ReferenceTextVa
     */
    @Override
    public final ReferenceTextVa getReferenceById(String type, Long id) {
        List<Object> params = new ArrayList<Object>();
        params.add(type);
        params.add(id);
        @SuppressWarnings("rawtypes")
        List list = getHibernateTemplate().findByNamedQuery(GETREFERENCEBYID, params.toArray());
        if (list.size() == 0) {
            return null;
        } else if (list.size() > 1) {
            String msg = "Too many Reference Data rows found for Type = " + type + " with the Id = " + Long.toString(id);
            throw new CustomizationServiceException(msg.toString());
        }
        return (ReferenceTextVa) list.get(0);
    }

    /**
     * Sets the SessionFactory to utilize.
     * 
     * @param sessionFactory
     *            the SessionFactory to use.
     */
    @Resource(name = "pecsSessionFactory")
    public void setPecsSessionFactory(SessionFactory sessionFactory) {
        super.setSessionFactory(sessionFactory);
    }
}
