package gov.va.med.pharmacy.pecs.persistence.fdb;

import java.util.ArrayList;
import java.util.List;

import javax.annotation.Resource;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.stereotype.Repository;

import firstdatabank.database.FDBDataManager;
import firstdatabank.database.FDBSQLException;
import firstdatabank.dif.DispensableDrug;
import firstdatabank.dif.DispensableGeneric;
import firstdatabank.dif.DispensableGenerics;
import firstdatabank.dif.DrugSearchFilter;
import firstdatabank.dif.FDBDispensableDrugLoadType;
import firstdatabank.dif.FDBNotAvailableException;
import firstdatabank.dif.FDBPhoneticSearch;
import firstdatabank.dif.FDBRoutedDoseFormDrugLoadType;
import firstdatabank.dif.FDBRoutedDrugLoadType;
import firstdatabank.dif.FDBSearchMethod;
import firstdatabank.dif.FDBUnknownIDException;
import firstdatabank.dif.Navigation;
import firstdatabank.dif.RoutedDoseFormDrug;
import firstdatabank.dif.RoutedDoseFormDrugs;
import firstdatabank.dif.RoutedDoseFormGeneric;
import firstdatabank.dif.RoutedDoseFormGenerics;
import firstdatabank.dif.RoutedDrug;
import firstdatabank.dif.RoutedDrugs;
import firstdatabank.dif.RoutedGeneric;
import firstdatabank.dif.RoutedGenerics;
import firstdatabank.dif.SearchFilter;
import gov.va.med.pharmacy.pecs.domain.impl.fdb.DrugConceptSearchResult;

/**
 * A DAO class to interact with the FDB DIF APIs in order to obtain Drug information.
 * 
 * @author vhaisaduffg
 * 
 */
@Repository
public final class DrugConceptDao implements InitializingBean {

    private static final Log LOGGER = LogFactory.getLog(DrugConceptDao.class);

    @Resource
    private Navigation fdbNavigation;
    @Resource
    private FDBDataManager fdbDataManager;

    /**
     * Concept Type for a Routed Drug.
     */
    public static final Long ROUTED_DRUG_TYPE = 2L;
    /**
     * Concept Type for a Routed Generic.
     */
    public static final Long ROUTED_GENERIC_TYPE = 5L;
    /**
     * Concept Type for a Dispensable Generic.
     */
    public static final Long DISPENSABLE_GENERIC_TYPE = 6L;
    /**
     * Concept Type for a Routed Dose Form Drug.
     */
    public static final Long ROUTED_DOSAGE_FORM_DRUG_TYPE = 7L;
    /**
     * Concept Type for a Routed Dose Form Generic.
     */
    public static final Long ROUTED_DOSAGE_FORM_GENERIC_TYPE = 8L;

    private static final Long ENGLISH_BASE_LANGUAGE_ID = 1L;

    /**
     * Returns the routed generic ID from a GenericDispensable object.
     * 
     * @param id
     *            identifier
     * @return String
     */
    public Long getRoutedGenericFromGCNID(Long id) {
        Long routedGenericID = null;
        try {
            DispensableGeneric drugConcept = new DispensableGeneric(fdbDataManager);
            drugConcept.load(id, StringUtils.EMPTY, StringUtils.EMPTY);
            routedGenericID = drugConcept.getRoutedGenericID();
        } catch (FDBUnknownIDException fuie) {
            // This is okay, we will not return a Description.
            LOGGER.info(fuie.getMessage(), fuie);
        } catch (FDBSQLException fse) {
            // This is okay, we will not return a Description.
            LOGGER.info(fse.getMessage(), fse);
        }
        return routedGenericID;
    }

    /**
     * Returns the routed generic ID from a Med ID.
     * 
     * @param id
     *            identifier
     * @return String
     */
    public Long getRoutedGenericFromMEDID(Long id) {
        Long routedGenericID = null;
        try {
            DispensableDrug drugConcept = new DispensableDrug(fdbDataManager);
            drugConcept.load(id.longValue(), FDBDispensableDrugLoadType.fdbDDLTMedID, StringUtils.EMPTY, StringUtils.EMPTY,
                    StringUtils.EMPTY);
            Long gcnSeqNo = drugConcept.getGCNSeqNo();

            if (gcnSeqNo != null && gcnSeqNo != 0) {
                routedGenericID = getRoutedGenericFromGCNID(gcnSeqNo); // Should this method name be GCNID?
            }

        } catch (FDBUnknownIDException fuie) {
            // This is okay, we will not return a Description.
            LOGGER.info(fuie.getMessage(), fuie);
        } catch (FDBSQLException fse) {
            // This is okay, we will not return a Description.
            LOGGER.info(fse.getMessage(), fse);
        } catch (FDBNotAvailableException e) {
            LOGGER.info(e.getMessage(), e);
        }
        return routedGenericID;
    }

    /**
     * Retrieves the description of the Concept.
     * 
     * @param id
     *            the identifier of the Concept
     * @param type
     *            the type of the Concept
     * @return String
     */
    public final String getDrugConceptDescription(Long id, Long type) {
        String description = null;

        if (ROUTED_DRUG_TYPE.equals(type)) {
            description = getRoutedDrugDescription(id);
        } else if (ROUTED_GENERIC_TYPE.equals(type)) {
            description = getRoutedGenericDescription(id);
        } else if (DISPENSABLE_GENERIC_TYPE.equals(type)) {
            description = getGenericDispensableDescription(id);
        } else if (ROUTED_DOSAGE_FORM_DRUG_TYPE.equals(type)) {
            description = getRoutedDoseFormDrugDescription(id);
        } else if (ROUTED_DOSAGE_FORM_GENERIC_TYPE.equals(type)) {
            description = getRoutedDoseFormGenericDescription(id);
        }

        return description;

    }

    /**
     * Retrieves the description from a RoutedDoseFormGeneric object.
     * 
     * @param id
     *            identifier
     * @return String
     */
    private String getRoutedDoseFormGenericDescription(Long id) {
        String description = null;
        try {
            RoutedDoseFormGeneric drugConcept = new RoutedDoseFormGeneric(fdbDataManager);
            drugConcept.load(id, StringUtils.EMPTY, StringUtils.EMPTY);
            description = drugConcept.getDescription();
        } catch (FDBUnknownIDException fuie) {
            // This is okay, we will not return a Description.
            LOGGER.info(fuie.getMessage(), fuie);
        } catch (FDBSQLException fse) {
            // This is okay, we will not return a Description.
            LOGGER.info(fse.getMessage(), fse);
        }
        return description;
    }

    /**
     * Retrieves the description from a RoutedDoseFormDrug object.
     * 
     * @param id
     *            identifier
     * @return String
     */
    private String getRoutedDoseFormDrugDescription(Long id) {
        String description = null;
        try {
            RoutedDoseFormDrug drugConcept = new RoutedDoseFormDrug(fdbDataManager);
            drugConcept.load(id, FDBRoutedDoseFormDrugLoadType.fdbRDFDLTRTDFMedID, StringUtils.EMPTY, StringUtils.EMPTY,
                    StringUtils.EMPTY);
            description = drugConcept.getDescription();
        } catch (FDBNotAvailableException fnae) {
            // This is okay, we will not return a Description.
            LOGGER.info(fnae.getMessage(), fnae);
        } catch (FDBUnknownIDException fuie) {
            // This is okay, we will not return a Description.
            LOGGER.info(fuie.getMessage(), fuie);
        } catch (FDBSQLException fse) {
            // This is okay, we will not return a Description.
            LOGGER.info(fse.getMessage(), fse);
        }
        return description;
    }

    /**
     * Returns the description from a GenericDispensable object.
     * 
     * @param id
     *            identifier
     * @return String
     */
    private String getGenericDispensableDescription(Long id) {
        String description = null;
        try {
            DispensableGeneric drugConcept = new DispensableGeneric(fdbDataManager);
            drugConcept.load(id, StringUtils.EMPTY, StringUtils.EMPTY);
            description = drugConcept.getDescription();
        } catch (FDBUnknownIDException fuie) {
            // This is okay, we will not return a Description.
            LOGGER.error(fuie.getMessage(), fuie);
        } catch (FDBSQLException fse) {
            // This is okay, we will not return a Description.
            LOGGER.error(fse.getMessage(), fse);
        }
        return description;
    }

    /**
     * Returns the description from a RoutedGeneric object.
     * 
     * @param id
     *            identifier
     * @return String
     */
    private String getRoutedGenericDescription(Long id) {
        String description = null;
        try {
            RoutedGeneric routedGeneric = new RoutedGeneric(fdbDataManager);
            routedGeneric.load(id, StringUtils.EMPTY, StringUtils.EMPTY);
            description = routedGeneric.getDescription();
        } catch (FDBUnknownIDException fuie) {
            // This is okay, we will not return a Description.
            LOGGER.info(fuie.getMessage(), fuie);
        } catch (FDBSQLException fse) {
            // This is okay, we will not return a Description.
            LOGGER.info(fse.getMessage(), fse);
        }
        return description;
    }

    /**
     * Returns the description from a RoutedDrug object.
     * 
     * @param id
     *            identifier
     * @return String
     */
    private String getRoutedDrugDescription(Long id) {
        String description = null;
        try {
            RoutedDrug routedDrug = new RoutedDrug(fdbDataManager);
            routedDrug.load(id, FDBRoutedDrugLoadType.fdbRDLTRMID, StringUtils.EMPTY, StringUtils.EMPTY, StringUtils.EMPTY);
            description = routedDrug.getDescription();
        } catch (FDBNotAvailableException fnae) {
            // This is okay, we will not return a Description.
            LOGGER.info(fnae.getMessage(), fnae);
        } catch (FDBUnknownIDException fuie) {
            // This is okay, we will not return a Description.
            LOGGER.info(fuie.getMessage(), fuie);
        } catch (FDBSQLException fse) {
            // This is okay, we will not return a Description.
            LOGGER.info(fse.getMessage(), fse);
        }
        return description;
    }

    /**
     * Accepts a Concept Id Description, condition and optional Concept Type in order to perform a query using the FDB DIF APIs.
     * 
     * @param description
     *            the items being searched for
     * @param operatorName
     *            the operator name
     * @return List<DrugConceptSearchResult>
     */
    public final synchronized List<DrugConceptSearchResult> queryForDescription(String description, String operatorName) {
        List<DrugConceptSearchResult> searchResults = new ArrayList<DrugConceptSearchResult>();
        synchronized (firstdatabank.dif.GlobalObjectManager.class) { // FDB locking contention
            searchResults.addAll(queryForRoutedDrugByDescription(description, operatorName));
            searchResults.addAll(queryForRoutedGenericByDescription(description, operatorName));
            searchResults.addAll(queryForDispensableGenericByDescription(description, operatorName));
            searchResults.addAll(queryForRoutedDoseFormDrugByDescription(description, operatorName));
            searchResults.addAll(queryForRoutedDoseFormGenericByDescription(description, operatorName));
        }

        return searchResults;
    }

    /**
     * Performs a search for Routed Drugs using the FDB DIF API.
     * 
     * @param description
     *            the items being searched for
     * @param condition
     *            the condition being used
     * @return List<DrugConceptSearchResult>
     */
    private final List<DrugConceptSearchResult> queryForRoutedDrugByDescription(String description, String condition) {
        RoutedDrugs drugs = null;
        try {
            drugs = fdbNavigation.routedDrugSearch(description, getSearchFilter(condition), getDrugSearchFilter());
        } catch (FDBSQLException fse) {
            LOGGER.error(fse.getMessage(), fse);
        } catch (RuntimeException re) {
            LOGGER.error(re.getMessage(), re);
        }
        return buildSearchResults(drugs);
    }

    /**
     * Performs a search for Routed Generics using the FDB DIF API.
     * 
     * @param description
     *            the items being searched for
     * @param condition
     *            the condition being used
     * @return List<DrugConceptSearchResult>
     */
    private final List<DrugConceptSearchResult> queryForRoutedGenericByDescription(String description, String condition) {
        RoutedGenerics drugs = null;
        try {
            drugs = fdbNavigation.routedGenericDrugSearch(description, getSearchFilter(condition), getDrugSearchFilter());
        } catch (FDBSQLException fse) {
            LOGGER.error(fse.getMessage(), fse);
        } catch (RuntimeException re) {
            LOGGER.error(re.getMessage(), re);
        }
        return buildSearchResults(drugs);
    }

    /**
     * Performs a search for Dispensable Generics using the FDB DIF API.
     * 
     * @param description
     *            the items being searched for
     * @param condition
     *            the condition being used
     * @return List<DrugConceptSearchResult>
     */
    private final List<DrugConceptSearchResult> queryForDispensableGenericByDescription(String description, String condition) {
        DispensableGenerics drugs = null;
        try {
            drugs = fdbNavigation.dispensableGenericDrugSearch(description, getSearchFilter(condition), getDrugSearchFilter());
        } catch (FDBSQLException fse) {
            LOGGER.error(fse.getMessage(), fse);
        } catch (RuntimeException re) {
            LOGGER.error(re.getMessage(), re);
        }
        return buildSearchResults(drugs);
    }

    /**
     * Performs a search for Routed Dose Form Drugs using the FDB DIF API.
     * 
     * @param description
     *            the items being searched for
     * @param condition
     *            the condition being used
     * @return List<DrugConceptSearchResult>
     */
    private final List<DrugConceptSearchResult> queryForRoutedDoseFormDrugByDescription(String description, String condition) {
        RoutedDoseFormDrugs drugs = null;
        try {
            drugs = fdbNavigation.routedDoseFormDrugSearch(description, getSearchFilter(condition), getDrugSearchFilter());
        } catch (FDBSQLException fse) {
            LOGGER.error(fse.getMessage(), fse);
        } catch (RuntimeException re) {
            LOGGER.error(re.getMessage(), re);
        }
        return buildSearchResults(drugs);
    }

    /**
     * Performs a search for Routed Dose Form Generics using the FDB DIF API.
     * 
     * @param description
     *            the items being searched for
     * @param condition
     *            the condition being used
     * @return List<DrugConceptSearchResult>
     */
    private final List<DrugConceptSearchResult>
            queryForRoutedDoseFormGenericByDescription(String description, String condition) {
        RoutedDoseFormGenerics drugs = null;
        try {
            drugs = fdbNavigation.routedDoseFormGenericSearch(description, getSearchFilter(condition), getDrugSearchFilter());
        } catch (FDBSQLException fse) {
            LOGGER.error(fse.getMessage(), fse);
        } catch (RuntimeException re) {
            LOGGER.error(re.getMessage(), re);
        }
        return buildSearchResults(drugs);
    }

    /**
     * Builds a list of DrugConceptSearchResult objects from the RoutedDrugs passed in.
     * 
     * @param drugs
     *            the FDB DIF API RoutedDrugs
     * @return List<DrugConceptSearchResult>
     */
    private List<DrugConceptSearchResult> buildSearchResults(RoutedDrugs drugs) {
        List<DrugConceptSearchResult> results = new ArrayList<DrugConceptSearchResult>();
        if (drugs != null) {
            final int numberOfDrugs = drugs.count();
            for (int i = 0; i < numberOfDrugs; i++) {
                RoutedDrug drug = drugs.item(i);
                DrugConceptSearchResult result = new DrugConceptSearchResult();
                result.setId(drug.getID());
                result.setConceptType(ROUTED_DRUG_TYPE);
                result.setDescription(drug.getDescription());
                results.add(result);
            }
        }
        return results;
    }

    /**
     * Builds a list of DrugConceptSearchResult objects from the RoutedGenerics passed in.
     * 
     * @param drugs
     *            the FDB DIF API RoutedGenerics
     * @return List<DrugConceptSearchResult>
     */
    private List<DrugConceptSearchResult> buildSearchResults(RoutedGenerics drugs) {
        List<DrugConceptSearchResult> results = new ArrayList<DrugConceptSearchResult>();
        if (drugs != null) {
            final int numberOfDrugs = drugs.count();
            for (int i = 0; i < numberOfDrugs; i++) {
                RoutedGeneric drug = drugs.item(i);
                DrugConceptSearchResult result = new DrugConceptSearchResult();
                result.setId(drug.getID());
                result.setConceptType(ROUTED_GENERIC_TYPE);
                result.setDescription(drug.getDescription());
                results.add(result);
            }
        }
        return results;
    }

    /**
     * Builds a list of DrugConceptSearchResult objects from the DispensableGenerics passed in.
     * 
     * @param drugs
     *            the FDB DIF API DispensableGenerics
     * @return List<DrugConceptSearchResult>
     */
    private List<DrugConceptSearchResult> buildSearchResults(DispensableGenerics drugs) {
        List<DrugConceptSearchResult> results = new ArrayList<DrugConceptSearchResult>();
        if (drugs != null) {
            final int numberOfDrugs = drugs.count();
            for (int i = 0; i < numberOfDrugs; i++) {
                DispensableGeneric drug = drugs.item(i);
                DrugConceptSearchResult result = new DrugConceptSearchResult();
                result.setId(drug.getID());
                result.setConceptType(DISPENSABLE_GENERIC_TYPE);
                result.setDescription(drug.getDescription());
                results.add(result);
            }
        }
        return results;
    }

    /**
     * Builds a list of DrugConceptSearchResult objects from the RoutedDoseFormDrugs passed in.
     * 
     * @param drugs
     *            the FDB DIF API RoutedDoseFormDrugs
     * @return List<DrugConceptSearchResult>
     */
    private List<DrugConceptSearchResult> buildSearchResults(RoutedDoseFormDrugs drugs) {
        List<DrugConceptSearchResult> results = new ArrayList<DrugConceptSearchResult>();
        if (drugs != null) {
            final int numberOfDrugs = drugs.count();
            for (int i = 0; i < numberOfDrugs; i++) {
                RoutedDoseFormDrug drug = drugs.item(i);
                DrugConceptSearchResult result = new DrugConceptSearchResult();
                result.setId(drug.getID());
                result.setConceptType(ROUTED_DOSAGE_FORM_DRUG_TYPE);
                result.setDescription(drug.getDescription());
                results.add(result);
            }
        }
        return results;
    }

    /**
     * Builds a list of DrugConceptSearchResult objects from the RoutedDoseFormGenerics passed in.
     * 
     * @param drugs
     *            the FDB DIF API RoutedDoseFormGenerics
     * @return List<DrugConceptSearchResult>
     */
    private List<DrugConceptSearchResult> buildSearchResults(RoutedDoseFormGenerics drugs) {
        List<DrugConceptSearchResult> results = new ArrayList<DrugConceptSearchResult>();
        if (drugs != null) {
            final int numberOfDrugs = drugs.count();
            for (int i = 0; i < numberOfDrugs; i++) {
                RoutedDoseFormGeneric drug = drugs.item(i);
                DrugConceptSearchResult result = new DrugConceptSearchResult();
                result.setId(drug.getID());
                result.setConceptType(ROUTED_DOSAGE_FORM_GENERIC_TYPE);
                result.setDescription(drug.getDescription());
                results.add(result);
            }
        }
        return results;
    }

    /**
     * Creates an initializes a SearchFilter for a FDB DIF API search.
     * 
     * @param operatorName
     *            the name of the operator being queried for
     * @return SearchFilter
     */
    private SearchFilter getSearchFilter(String operatorName) {
        SearchFilter sf = new SearchFilter();

        sf.setBaseLanguageID(ENGLISH_BASE_LANGUAGE_ID);
        sf.setPhoneticSearch(FDBPhoneticSearch.fdbPSNoPhonetic);
        if ("EQ".equals(operatorName) || "BW".equals(operatorName)) {
            sf.setSearchMethod(FDBSearchMethod.fdbSMSimple);
        } else if ("CONTAINS".equals(operatorName)) {
            sf.setSearchMethod(FDBSearchMethod.fdbSMContainsExhaustive);
        }
        return sf;
    }

    /**
     * Creates and initializes a DrugSearchFilter for FDB DIF API search.
     * 
     * @return DrugSearchFilter
     */
    private DrugSearchFilter getDrugSearchFilter() {
        DrugSearchFilter dsf = new DrugSearchFilter();

        dsf.setIncludeObsoleteDrugs(false);

        return dsf;
    }

    /**
     * Sets the Navigation object to use for FDB navigation.
     * 
     * @param navigation
     *            Navigation
     */
    public final void setFdbNavigation(Navigation navigation) {
        fdbNavigation = navigation;
    }

    /**
     * Sets the FDBDataManager.
     * 
     * @param manager
     *            FDBDataManager
     */
    public final void setFdbDataManager(FDBDataManager manager) {
        fdbDataManager = manager;
    }

    /**
     * Checks to make sure all beans have been injected.
     * 
     * @throws Exception
     *             if a problem is found
     */
    @Override
    public final void afterPropertiesSet() throws Exception {
        if (fdbNavigation == null) {
            throw new Exception("FdbNavigation has not been injected in to DrugConceptDao.");
        }
        if (fdbDataManager == null) {
            throw new Exception("FDBDataManager has not been injected in to DrugConceptDao.");
        }
    }
}
