package gov.va.med.pharmacy.pecs.query;

import gov.va.med.pharmacy.pecs.exception.CustomizationServiceException;
import gov.va.med.pharmacy.pecs.metadata.ConceptField;

import java.text.ParsePosition;
import java.text.SimpleDateFormat;
import java.util.Date;

import org.apache.commons.lang.StringUtils;

/**
 * AnyTypeOperator implements Condition Operator and is the base class for other type operators.
 * 
 */
public abstract class AnyTypeOperator implements ConditionOperator {
    /** Date formatter used to format HQL and XML dates as Strings. */
    private final SimpleDateFormat dateFormatter = new SimpleDateFormat("yyyy-MM-dd");

    @Override
    public Object parseValue(String val, ConceptField field) {
        if (StringUtils.isBlank(val)) {
            return null;
        }
        Class<?> type = field.getType();
        try {
            if (type.equals(Integer.class) || type.equals(Long.class) || type.equals(Short.class) || type.equals(int.class)
                    || type.equals(long.class) || type.equals(short.class)) {
                return Long.parseLong(val);
            } 
            if (type.equals(Date.class)) {
                return parseBasicOrExtendedDate(val, field.getDisplayName());
            } 
            if (type.equals(Double.class) || type.equals(Float.class) || type.equals(double.class)
                    || type.equals(float.class)) {
                return Double.parseDouble(val);
            }
            
            return val.trim();
        } catch (NumberFormatException e) {
            String msg = "Value for field " + field.getName() + " must be a number";
            throw new CustomizationServiceException(msg, e);
        }
    }

    /**
     * Parses a String value in the format 'yyyy-MM-dd' or 'yyyyMMdd' into a java.util.Date.  
     * Uses strict parsing, meaning it does not allow for dates such as 20140230 (February 30). 
     * The logic in this method could be replaced by Apache Commons Lang DateUtils.parseDateStrictly once that 
     * library is upgraded to version 2.5 or later.
     * @param value contains the date value to parse
     * @param fieldName the name of the field
     * @return Parsed Date
     */
    protected Date parseBasicOrExtendedDate(String value, String fieldName){
        final SimpleDateFormat dateFormatterWithDashes = new SimpleDateFormat("yyyy-MM-dd");
        final SimpleDateFormat dateFormatterWithoutDashes = new SimpleDateFormat("yyyyMMdd");
        dateFormatterWithDashes.setLenient(false);
        dateFormatterWithoutDashes.setLenient(false);
        ParsePosition parsePosition = new ParsePosition(0);
        String trimmedValue = value.trim();
           
        Date date = (Date) dateFormatterWithDashes.parse(trimmedValue, parsePosition);

        if (date == null || parsePosition.getIndex() != trimmedValue.length()){
            parsePosition = new ParsePosition(0);
            date = (Date) dateFormatterWithoutDashes.parse(trimmedValue, parsePosition);

            if (date == null || parsePosition.getErrorIndex() >=0 
                    || parsePosition.getIndex() != trimmedValue.length()){                

                    String msg = "Value for field " + fieldName + " must be a valid date formatted as " 
                            + dateFormatterWithDashes.toPattern() + " or " + dateFormatterWithoutDashes.toPattern();
                    throw new CustomizationServiceException(msg);                
            }                
        }
            
        return date;
    }
    
    @Override
    public String formatHQLValue(Object val, ConceptField field) {
        Class<?> type = field.getType();
        if (val == null) {
            return "null";
        }
        if (type.equals(String.class)) {
            return "'" + ((String) val).trim().toLowerCase() + "'";
        }
        else if (type.equals(Date.class)) {
            return "to_date('" + dateFormatter.format(val) + "', '" + dateFormatter.toPattern().toUpperCase() + "')";
        }  
        else {
            return String.valueOf(val);
        }
    }

    @Override
    public String formatXMLValue(Object val, ConceptField field) {
        Class<?> type = field.getType();
        if (type.equals(Date.class)) {
            return dateFormatter.format(val);
        } else {
            return String.valueOf(val);
        }
    }

}
