package gov.va.med.pharmacy.pecs.query;

import gov.va.med.pharmacy.pecs.metadata.ConceptField;
import gov.va.med.pharmacy.pecs.metadata.ConceptMetadata;

import java.util.ArrayList;
import java.util.List;

/**
 * Represents a query for a concept. A query is expressed in terms of fields defined in the meta data. So the same query can be
 * used to query both custom and FDB data. The concept to which the query applies to is defined by its meta data.
 * 
 * @author vhaisaalbua
 */
public final class ConceptQuery {

    /** The include history. */
    private boolean includeHistory;

    /** The metadata. */
    private ConceptMetadata metadata;

    /** The condition. */
    private QueryCondition condition;

    /** The sort fields. */
    private List<ConceptField> sortFields;

    /** The sort asc. */
    private boolean sortAsc;

    /** Denotes whether the query will be used for paged results. */
    private boolean isPaged;    
    
    /**
     * ConceptQuery.
     * 
     * @param meta
     *            - ConceptMetaData.
     */
    public ConceptQuery(ConceptMetadata meta) {
        metadata = meta;
    }

    /**
     * ConceptQuery.
     * 
     * @param meta
     *            ConceptMetadata
     * @param cond
     *            QueryCondtion
     */
    public ConceptQuery(ConceptMetadata meta, QueryCondition cond) {
        metadata = meta;
        condition = cond;
    }

    /**
     * Creates an HQL query based on the properties of this query instance.
     * 
     * @param custom true = VA custom entity query; false = FDB entity query.
     * @return String containing the HQL.
     */
    public String toHQL(boolean custom) {
        StringBuffer hqlBuffer = new StringBuffer("from ");
        hqlBuffer.append(metadata.getPersistentClass(custom).getSimpleName());
        if (condition != null) {
            QueryConditionVisitor visitor = new HqlSerializationVisitor(custom);
            accept(visitor);
            hqlBuffer.append(" where").append(visitor);
        }

        if (sortFields != null && !sortFields.isEmpty()) {                       
            boolean alreadyHasOrderByClause = addSortFieldsToHQL(custom, hqlBuffer);            
            addUniqueSortToHQLIfPaged(hqlBuffer, alreadyHasOrderByClause);               
        }
        return hqlBuffer.toString();
    }

    /**
     * Adds the sort fields to the HQL.
     * @param custom true = VA custom entity query; false = FDB entity query.
     * @param hqlBuffer contains the HQL to which this sort will be appended.
     * @return true = an order by clause has been added; false = no order by clause has been added.
     */
    private boolean addSortFieldsToHQL(boolean custom, StringBuffer hqlBuffer) {
        boolean alreadyHasOrderByClause = false;        
        ArrayList<ConceptField> orderBy = filterSortFieldsForOrderBy(custom);
        
        for (ConceptField f : orderBy) {
            if (!alreadyHasOrderByClause) {
                hqlBuffer.append(" order by ");
                alreadyHasOrderByClause = true;
            } else {
                hqlBuffer.append(", ");
            }
            
            boolean stringField = f.getType().getSimpleName().equals("String");
            if (stringField) {
                hqlBuffer.append("LOWER(");
            }
            hqlBuffer.append(f.getPersistentPath(custom));
            if (stringField) {
                hqlBuffer.append(")");
            }
            hqlBuffer.append(sortAsc ? " asc" : " desc");
        }
        return alreadyHasOrderByClause;
    }

    /**
     * Filters the list of ConceptFields, ignoring those which do not map to a persistent field for the type of record 
     * specified.
     * 
     * @param custom true = VA custom entity query; false = FDB entity query.
     * @return a list of ConceptFields that map to a persistent field for the record type.
     */
    private ArrayList<ConceptField> filterSortFieldsForOrderBy(boolean custom) {
        ArrayList<ConceptField> orderBy = new ArrayList<ConceptField>();
        for (ConceptField f : sortFields) {  
            f = stripDisplayStringFromFieldName(f);
            
            if (!ConceptMetadata.noValue(f.getPersistentPath(custom))) {
                orderBy.add(f);
            }
        }
        return orderBy;
    }

    /**
     *  Removes DISPLAY_STRING from the field name.  Some fields, like Action Date and Export Date, have parallel 
     *  fields for display only.  We need to remove the _DISPLAY_STRING from those field names so they map properly 
     *  to the persistence field.
     * 
     * @param f the field from which to remove DISPLAY_STRING from the field name, if it is part of the name.
     * @return the field with the field name stripped, or the original field unchanged if _DISPLAY_STRING was not part of the 
     *      field name.
     */
    private ConceptField stripDisplayStringFromFieldName(ConceptField f) {
        if (f.getName().endsWith("_DISPLAY_STRING"))
        {
            f = metadata.getField(f.getName().substring(0, f.getName().length() - "_DISPLAY_STRING".length()));
        }
        return f;
    }

    /**
     * Adds a sort by the entity's id fields to the HQL if the query isPaged is true.
     * @param hqlBuffer Stringbuffer containing the HQL to which this sort will be appended.
     * @param alreadyHasOrderByClause whether the buffer already contains an order by clause
     */
    private void addUniqueSortToHQLIfPaged(StringBuffer hqlBuffer, boolean alreadyHasOrderByClause) {
        if (isPaged){
            if (alreadyHasOrderByClause) { // there already is an order by clause                    
                hqlBuffer.append(", id asc");
            }
            else{                    
                hqlBuffer.append(" order by id asc");
            }                    
        }
    }

    /**
     * toXML.
     * 
     * @return String query condtion.XMl
     */
    public String toXML() {
        return "<query>" + (condition == null ? "" : condition.toXML()) + "</query>";
    }

    /**
     * toText.
     * 
     * @return String metadata.getConceptName
     */
    public String toText() {
        return "from " + metadata.getConceptName() + (condition == null ? "" : " where " + condition.toText());
    }

    /**
     * toJSON.
     * 
     * @return String visitor
     */
    public String toJSON() {
        JsonSerializationVisitor visitor = new JsonSerializationVisitor();
        accept(visitor);
        return visitor.toString();
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Object#toString()
     */
    @Override
    public String toString() {
        return toText();
    }

    /**
     * addCondtion.
     * 
     * @param cond
     *            QueryCondition.
     * @param conj
     *            Conjunction.
     * @return condition.
     */
    public ConceptQuery addCondition(QueryCondition cond, Conjunction conj) {
        if (condition == null) {
            condition = cond;
        } else {
            condition = condition.combine(cond, conj);
        }
        return this;
    }

    /**
     * include History.
     * 
     * @return includeHistory
     */
    public boolean includeHistory() {
        return includeHistory;
    }

    /**
     * Set Include History.
     * 
     * @param includeHistory
     *            - boolean
     */
    public void setIncludeHistory(boolean includeHistory) {
        this.includeHistory = includeHistory;
    }

    /**
     * Concept Metadata.
     * 
     * @return getMetadata.
     */
    public ConceptMetadata getMetadata() {
        return metadata;
    }

    /**
     * Set Metadata.
     * 
     * @param meta
     *            - conceptMetadata
     */
    public void setMetadata(ConceptMetadata meta) {
        this.metadata = meta;
    }

    /**
     * Get SortFields.
     * 
     * @return List of SortField
     */
    public List<ConceptField> getSortFields() {
        return sortFields;
    }

    /**
     * Specify the fields that should be used when sorting the query results.
     * 
     * @param fieldNames
     *            the field names
     * @param asc
     *            true, if ascending order is desired, false otherwise
     */
    public void setSortFields(List<String> fieldNames, boolean asc) {
        ArrayList<ConceptField> list = new ArrayList<ConceptField>();
        for (String name : fieldNames) {
            ConceptField f = metadata.getField(name);
            if (f == null) {
                throw new IllegalArgumentException("Unknown field " + name);
            } else {
                list.add(f);
            }
        }
        this.sortFields = list;
        sortAsc = asc;
    }

    /**
     * Set Sort Fields.
     * 
     * @param list
     *            List<ConceptField>
     */
    public void setSortFields(List<ConceptField> list) {
        sortFields = list;
    }

    /**
     * Set Sort Order.
     * 
     * @param asc
     *            sortAsc
     */
    public void setSortOrder(boolean asc) {
        sortAsc = asc;
    }

    /**
     * Gets isPaged.
     * @return the isPaged
     */
    public boolean isPaged() {
        return isPaged;
    }

    /**
     * Sets isPaged.
     * @param newIsPagedValue the isPaged to set
     */
    public void setPaged(boolean newIsPagedValue) {
        this.isPaged = newIsPagedValue;
    }

    /**
     * isCondtionSet.
     * 
     * @return condition if not null
     */
    public boolean isConditionSet() {
        return condition != null;
    }

    /**
     * Accept QueryConditionVisitor.
     * 
     * @param visitor
     *            QueryCondtionVisitor
     */
    public void accept(QueryConditionVisitor visitor) {
        if (condition == null) {
            return;
        }
        condition.accept(visitor);
        condition = visitor.getResult();
    }

    /**
     * ConceptQuery Copy.
     * 
     * @return dolly
     */
    public ConceptQuery copy() {
        ConceptQuery dolly = new ConceptQuery(metadata);
        dolly.condition = condition;
        dolly.includeHistory = includeHistory;
        dolly.sortFields = sortFields;
        dolly.sortAsc = sortAsc;
        dolly.isPaged = isPaged;
        return dolly;
    }

}
