package gov.va.med.pharmacy.pecs.query;

import java.util.ArrayList;
import java.util.List;

/**
 * A conjunction condition groups together a list of conditions connected by the same logical conjunction (AND, OR).
 * 
 * @author vhaisaalbua
 */
public abstract class ConjunctionCondition implements QueryCondition {
    private final Conjunction conjunction;
    private List<QueryCondition> conditions;

    /**
     * Conjunction Getter.
     * 
     * @return conjunction
     */
    protected Conjunction getConjunction() {
        return conjunction;
    }

    /**
     * Conjunction Condition createCOnjunctionCondition.
     * 
     * @param conj
     *            - Conjunction
     * @param c1
     *            - Query Condition
     * @param c2
     *            - Query Condition
     * @return ConjunctionCondition
     */
    public static ConjunctionCondition createConjunctionCondition(Conjunction conj, QueryCondition c1, QueryCondition c2) {
        switch (conj) {
        case OR:
            return new OrConjunctionCondition(c1, c2);
        case AND:
            return new AndConjunctionCondition(c1, c2);
        default:
            throw new IllegalArgumentException("Invalid logical conjunction: " + conj);
        }
    }

    /**
     * ConjunctionCondition Create.
     * 
     * @param conj
     *            - Conjunction
     * @param conds
     *            - List
     * @return conjunction condition
     */
    public static ConjunctionCondition createConjunctionCondition(Conjunction conj, List<QueryCondition> conds) {
        switch (conj) {
        case OR:
            return new OrConjunctionCondition(conds);
        case AND:
            return new AndConjunctionCondition(conds);
        default:
            throw new IllegalArgumentException("Invalid logical conjunction: " + conj);
        }
    }

    /**
     * Conjunction Condition.
     * 
     * @param conj
     *            Conjunction.
     * @param c1
     *            QueryCondition
     * @param c2
     *            Query Condition
     */
    public ConjunctionCondition(Conjunction conj, QueryCondition c1, QueryCondition c2) {
        conjunction = conj;
        conditions = new ArrayList<QueryCondition>();
        conditions.add(c1);
        conditions.add(c2);
    }

    /**
     * Conjunction Condition.
     * 
     * @param conj
     *            Conjunction
     * @param conds
     *            List
     */
    public ConjunctionCondition(Conjunction conj, List<QueryCondition> conds) {
        conjunction = conj;
        if (conds.size() < 2) {
            throw new IllegalArgumentException("At least 2 conditions must be passed");
        } else {
            conditions = conds;
        }
    }

    @Override
    public ConjunctionCondition combine(QueryCondition cond, Conjunction conj) {
        conditions.add(cond);
        return this;
    }

    @Override
    public StringBuffer toHQL(final boolean custom) {
        return format(new ConjunctionConditionFormatter() {
            @Override
            public StringBuffer formatCondition(QueryCondition cond) {
                return cond.toHQL(custom);
            }

            @Override
            public StringBuffer formatConjunction() {
                return new StringBuffer(" " + conjunction + " ");
            }
        });
    }

    @Override
    public StringBuffer toXML() {
        return format(new ConjunctionConditionFormatter() {
            @Override
            public StringBuffer formatCondition(QueryCondition cond) {
                return cond.toXML();
            }

            @Override
            public StringBuffer formatConjunction() {
                return new StringBuffer("<" + conjunction + "/>");
            }
        });

    }

    @Override
    public StringBuffer toText() {
        return format(new ConjunctionConditionFormatter() {
            @Override
            public StringBuffer formatCondition(QueryCondition cond) {
                return cond.toText();
            }

            @Override
            public StringBuffer formatConjunction() {
                return new StringBuffer(" " + conjunction + " ");
            }
        });
    }

    @Override
    public String toString() {
        return toText().toString();
    }

    /**
     * Format String Buffer.
     * 
     * @param formatter
     *            Conjunction Condition formatter.
     * @return String Buffer
     */
    private StringBuffer format(ConjunctionConditionFormatter formatter) {
        StringBuffer res = new StringBuffer();
        StringBuffer conj = formatter.formatConjunction();
        for (QueryCondition c : conditions) {
            if (res.length() != 0) {
                res.append(conj);
            }
            res.append(formatter.formatCondition(c));
        }
        return res;
    }

    /**
     * Get Conditions.
     * 
     * @return List of Condition.
     */
    public List<QueryCondition> getConditions() {
        return conditions;
    }

    /**
     * Set Conditions.
     * 
     * @param conditions
     *            - List
     */
    public void setConditions(List<QueryCondition> conditions) {
        this.conditions = conditions;
    }

    /**
     * Inner interface ConjunctionCondtionFormatter.
     * 
     * @author DOMAINOVERSD
     * 
     */
    interface ConjunctionConditionFormatter {
        /**
         * String Buffer.
         * 
         * @return formatConjunction
         */
        StringBuffer formatConjunction();

        /**
         * FormatCondtion StringBuffer.
         * 
         * @param cond
         *            QueryCondtition
         * 
         * @return n/a
         */
        StringBuffer formatCondition(QueryCondition cond);
    }
}
