package gov.va.med.pharmacy.pecs.query;

import gov.va.med.pharmacy.pecs.domain.impl.fdb.DrugConceptSearchResult;
import gov.va.med.pharmacy.pecs.dto.DoseRangeDTO;
import gov.va.med.pharmacy.pecs.metadata.ConceptField;
import gov.va.med.pharmacy.pecs.metadata.ConceptMetadata;
import gov.va.med.pharmacy.pecs.persistence.fdb.DrugConceptDao;
import gov.va.med.pharmacy.pecs.service.MetadataService;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

import javax.annotation.Resource;

/**
 * A visitor that modifies a Dose Range interaction query so that Concept Id Description can be translated to Concept Ids that
 * are found in FDB.
 * 
 * @author vhaisaduffg
 */
public class DoseRangeVisitor implements QueryConditionVisitor {

    @Resource
    private final MetadataService metadataService;
    @Resource
    private final DrugConceptDao drugConceptDao;

    private ConceptMetadata meta = null;
    private ConceptField conceptTypeField = null;
    private ConceptField conceptIdField = null;
    private static final Integer MAX_IN_SIZE = 999; // The maximum number of values that can be specified in an IN clause.
    private QueryCondition result;
    private boolean firstPass = true;

    /**
     * Constructor since this class cannot be set up as a bean in Spring.
     * 
     * @param service
     *            MetadataService
     * @param dao
     *            DrugConceptDao
     */
    public DoseRangeVisitor(MetadataService service, DrugConceptDao dao) {
        metadataService = service;
        drugConceptDao = dao;
    }

    /**
     * A private method to return the Metadata for the DoseRangeDTO class.
     */
    private void getMetadata() {
        if (meta == null) {
            meta = metadataService.getMetadata(DoseRangeDTO.class);
        }
    }

    /**
     * A private method to find the Concept Id metadata.
     * 
     * @return ConceptField containing Concept Id metadata
     */
    private final ConceptField findConceptIdMeta() {
        if (conceptIdField == null) {
            getMetadata();
            conceptIdField = meta.getField("CONCEPTID");
        }
        return conceptIdField;
    }

    /**
     * A private method to find the Concept Type metadata.
     * 
     * @return ConceptField containing Concept Type metadata
     */
    private final ConceptField findConceptTypeMeta() {
        if (conceptTypeField == null) {
            getMetadata();
            conceptTypeField = meta.getField("CONCEPTTYPE");
        }
        return conceptTypeField;
    }

    /**
     * Executed by the visitFieldCondition method. This method calls the DrugConceptDao queryForDescription to obtain a list of
     * drugs and their Concept Ids. The HQL where clause is built and replaced in the query.
     * 
     * @param operator
     *            The operator the user selected (i.e. Contains, Begins With, etc).
     * @param value
     *            The value being queried for
     * @return An HQL where condition clause
     */
    private final QueryCondition process(ConditionOperator operator, String value) {
        QueryCondition condition = null;
        List<DrugConceptSearchResult> drugs = drugConceptDao.queryForDescription(value, operator.getName());
        Collections.sort(drugs);
        Iterator<DrugConceptSearchResult> drugIterator = drugs.iterator();
        Long previousConceptType = 0L;
        List<Long> values = new ArrayList<Long>();
        if (drugIterator.hasNext()) {
            while (drugIterator.hasNext()) {
                DrugConceptSearchResult drug = drugIterator.next();
                if (previousConceptType == 0L) {
                    previousConceptType = drug.getConceptType();
                }
                if (!previousConceptType.equals(drug.getConceptType()) || values.size() > MAX_IN_SIZE) {
                    condition = addCondition(previousConceptType, values, condition);
                    previousConceptType = drug.getConceptType();
                    values = new ArrayList<Long>();
                }
                values.add(drug.getId());
            }
        } else {
            // If no drugs were found, set the concept Id to a value that will never be found.
            values.add(new Long(-1));
        }

        condition = addCondition(previousConceptType, values, condition);

        return new ParenthesisCondition(condition);
    }

    /**
     * Creates a new HQL where condition based upon the Concept Type and list of values passed in.
     * 
     * @param conceptType
     *            Concept Type
     * @param values
     *            list of Concept Ids
     * @param condition
     *            the current QueryCondition to append to
     * @return a new QueryCondition
     */
    private QueryCondition addCondition(Long conceptType, List<Long> values, QueryCondition condition) {
        FieldCondition conceptTypeCondition =
                new FieldCondition(findConceptTypeMeta(), OperatorFactory.getInstance().getOperator(ConditionOperator.EQ),
                        conceptType);
        //@formatter:off
        FieldCondition conceptIdCondition =
                new FieldCondition(findConceptIdMeta(),
                        OperatorFactory.getInstance().getOperator(ConditionOperator.IN), values);
        //@formatter:on
        AndConjunctionCondition andCondition = new AndConjunctionCondition(conceptTypeCondition, conceptIdCondition);
        ParenthesisCondition grouping = new ParenthesisCondition(andCondition);
        if (condition == null) {
            condition = grouping; // new ParenthesisCondition(grouping);
        } else {
            condition = condition.combine(grouping, Conjunction.OR);
        }
        return condition;
    }

    /**
     * Visitor method used to construct a new QueryCondition if the field being acted upon is the Concept Id Description.
     * 
     * @param cond
     *            the Field being acted upon
     * @return List<QueryCondition>
     */
    @Override
    public final List<QueryCondition> visitFieldCondition(FieldCondition cond) {
        QueryCondition c = cond;
        ConceptField field = cond.getField();
        if (DoseRangeDTO.CONCEPTDESCRIPTION.equalsIgnoreCase(field.getName())) {
            String v = (String) cond.getValue();
            ConditionOperator operator = cond.getOperator();
            c = process(operator, v);
        }

        result = c;
        return Collections.singletonList(c);
    }

    /**
     * Checks to see if this is the first time this query condition was analyzed.
     * 
     * @param cond
     *            QueryCondition
     * @return boolean
     */
    protected final boolean isFirstPass(QueryCondition cond) {
        if (firstPass) {
            result = cond;
            firstPass = false;
            return true;
        }
        return false;
    }

    /**
     * Used to build an AND conjunction condition.
     * 
     * @param cond
     *            AndConjunctionCondition
     * @return List<QueryCondition>
     */
    @Override
    public final List<QueryCondition> visitAndConjunctionCondition(AndConjunctionCondition cond) {
        ArrayList<QueryCondition> temp = new ArrayList<QueryCondition>();
        for (QueryCondition c : cond.getConditions()) {
            List<QueryCondition> c1 = c.accept(this);
            temp.addAll(c1);
        }
        QueryCondition c = ConjunctionCondition.createConjunctionCondition(Conjunction.AND, temp);
        result = c;

        return Collections.singletonList(c);
    }

    /**
     * Used to build an OR conjunction condition.
     * 
     * @param cond
     *            OrConjunctionCondition
     * @return List<QueryCondition>
     */
    @Override
    public final List<QueryCondition> visitOrConjunctionCondition(OrConjunctionCondition cond) {
        ArrayList<QueryCondition> temp = new ArrayList<QueryCondition>();
        for (QueryCondition c : cond.getConditions()) {
            List<QueryCondition> list = c.accept(this);
            temp.addAll(list);
        }

        QueryCondition c = ConjunctionCondition.createConjunctionCondition(Conjunction.OR, temp);
        result = c;

        return Collections.singletonList(c);
    }

    /**
     * Method to visit a HibernateCondition.
     * 
     * @param cond
     *            HibernateCondition
     * @return List<QueryCondition>
     */
    @Override
    public final List<QueryCondition> visitHibernateCondition(HibernateCondition cond) {
        return Collections.singletonList((QueryCondition) cond);
    }

    /**
     * Method to visit a ParenthesisCondition.
     * 
     * @param cond
     *            ParenthesisCondition
     * @return List<QueryCondition>
     */
    @Override
    public final List<QueryCondition> visitParenthesisCondition(ParenthesisCondition cond) {
        // remember if first pass, but don't change the flag,
        // so the nested condition believes it's the first one
        boolean wasFirstPass = firstPass;
        // process nested condition
        List<QueryCondition> temp = cond.getNested().accept(this);
        if (wasFirstPass) {
            // parenthesize result
            return Collections.singletonList((QueryCondition) temp);
        } else {
            switch (temp.size()) {
            case 0:
                throw new IllegalStateException("0 conditions returned");
            case 1:
                result = temp.get(0);
                return Collections.singletonList(result);
            default:
                // the nested condition must have been an OR conjunction
                result = new ParenthesisCondition(ConjunctionCondition.createConjunctionCondition(Conjunction.OR, temp));
                return Collections.singletonList(result);
            }
        }
    }

    /**
     * Returns the result.
     * 
     * @return QueryCondition
     */
    @Override
    public final QueryCondition getResult() {
        return result;
    }
}
