package gov.va.med.pharmacy.pecs.query;

import gov.va.med.pharmacy.pecs.domain.impl.va.DrugDrugInteractionVersionVa;
import gov.va.med.pharmacy.pecs.dto.DrugDrugInteractionDTO;
import gov.va.med.pharmacy.pecs.metadata.ConceptField;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * A visitor that modifies a drug drug interaction query so that it becomes insensitive to the order of drugs in the description
 * or the corresponding FDB interaction id.
 * 
 * @author vhaisaalbua
 */
public final class DrugDrugInteractionVisitor implements QueryConditionVisitor {

    /** The result. */
    private QueryCondition result;

    /** The first pass. */
    private boolean firstPass = true;

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.query.QueryConditionVisitor#getResult()
     */
    @Override
    public QueryCondition getResult() {
        return result;
    }

    /**
     * Boolean Check on QueryCondition.
     * 
     * @param cond
     *            QueryCondition
     * @return boolean T/F
     */
    private boolean isFirstPass(QueryCondition cond) {
        if (firstPass) {
            result = cond;
            firstPass = false;
            return true;
        }
        return false;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.query.QueryConditionVisitor#visitAndConjunctionCondition(gov.va.med.pharmacy.pecs.query.
     * AndConjunctionCondition)
     */
    @Override
    public List<QueryCondition> visitAndConjunctionCondition(AndConjunctionCondition cond) {
        boolean setResult = isFirstPass(cond);
        ArrayList<QueryCondition> temp = new ArrayList<QueryCondition>();
        for (QueryCondition c : cond.getConditions()) {
            List<QueryCondition> c1 = c.accept(this);
            temp.addAll(c1);
        }
        QueryCondition c;
        if (cond.getConditions().equals(temp)) {
            // there were no changes
            c = null;
        } else {
            c = ConjunctionCondition.createConjunctionCondition(Conjunction.AND, temp);
        }
        if (setResult && c != null) {
            result = result.combine(c, Conjunction.OR);
        }

        if (c == null) {
            return Collections.emptyList();
        } else {
            return Collections.singletonList(c);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.query.QueryConditionVisitor#visitOrConjunctionCondition(gov.va.med.pharmacy.pecs.query.
     * OrConjunctionCondition)
     */
    @Override
    public List<QueryCondition> visitOrConjunctionCondition(OrConjunctionCondition cond) {
        boolean setResult = isFirstPass(cond);
        ArrayList<QueryCondition> temp = new ArrayList<QueryCondition>();
        for (QueryCondition c : cond.getConditions()) {
            List<QueryCondition> list = c.accept(this);
            // eliminate duplicates of original condition (could happen for
            // conditions that don't need processing)
            for (QueryCondition c1 : list) {
                if (!c.equals(c1)) {
                    temp.add(c1);
                }
            }
        }

        if (setResult) {
            for (QueryCondition c : temp) {
                result = result.combine(c, Conjunction.OR);
            }
        }
        return temp;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * gov.va.med.pharmacy.pecs.query.QueryConditionVisitor#visitFieldCondition(gov.va.med.pharmacy.pecs.query.FieldCondition)
     */
    @Override
    public List<QueryCondition> visitFieldCondition(FieldCondition cond) {
        boolean setResult = isFirstPass(cond);
        QueryCondition c = cond;
        ConceptField field = cond.getField();
        if (DrugDrugInteractionDTO.DESCRIPTION1.equals(field.getName())) {
            Object v = cond.getValue();
            if (v != null && v.toString().contains("/")) {
                FieldCondition copy = cond.copy();
                copy.setValue(reverseString(v.toString()));
                c = copy;
            }
        } else if (DrugDrugInteractionDTO.FDB_ID.equals(field.getName())) {
            // we need to use the same condition for the reverse FDB id
            // we'll generate a Hibernate condition like:
            // (32000 - FDB_ID) <operator> <value>
            StringBuffer buf = new StringBuffer("(32000 - ");
            // we know this is just for custom queries
            buf.append(cond.getField().getPersistentPath(true)).append(")");
            buf.append(" ").append(cond.getOperator().getSymbol()).append(" ");
            buf.append(cond.getOperator().formatHQLValue(cond.getValue(), cond.getField()));
            c = new HibernateCondition(buf.toString());
        }

        if (setResult && c != cond) {
            result = result.combine(c, Conjunction.OR);
        }
        return Collections.singletonList(c);
    }

    /**
     * Reverse the String.
     * 
     * @param description
     *            -String
     * @return description + drug drug interacton Version Va
     */
    private Object reverseString(String description) {
        final String[] descriptionParts = description.split(DrugDrugInteractionVersionVa.DESCRIPTION_DELIMITER);
        return descriptionParts[1] + DrugDrugInteractionVersionVa.DESCRIPTION_DELIMITER + descriptionParts[0];
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.query.QueryConditionVisitor#visitHibernateCondition(gov.va.med.pharmacy.pecs.query.
     * HibernateCondition)
     */
    @Override
    public List<QueryCondition> visitHibernateCondition(HibernateCondition cond) {
        return Collections.singletonList((QueryCondition) cond);
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.query.QueryConditionVisitor#visitParenthesisCondition(gov.va.med.pharmacy.pecs.query.
     * ParenthesisCondition)
     */
    @Override
    public List<QueryCondition> visitParenthesisCondition(ParenthesisCondition cond) {
        // remember if first pass, but don't change the flag,
        // so the nested condition believes it's the first one
        boolean wasFirstPass = firstPass;
        // process nested condition
        List<QueryCondition> temp = cond.getNested().accept(this);
        if (wasFirstPass) {
            // parenthesize result
            result = new ParenthesisCondition(result);
            return Collections.singletonList(result);
        } else {
            switch (temp.size()) {
            case 0:
                throw new IllegalStateException("0 conditions returned");
            case 1:
                return Collections.singletonList((QueryCondition) new ParenthesisCondition(temp.get(0)));
            default:
                // the nested condition must have been an OR conjunction
                return Collections.singletonList((QueryCondition) new ParenthesisCondition(ConjunctionCondition
                        .createConjunctionCondition(Conjunction.OR, temp)));
            }
        }
    }

}
