package gov.va.med.pharmacy.pecs.query;

import gov.va.med.pharmacy.pecs.dto.DrugPairDTO;
import gov.va.med.pharmacy.pecs.metadata.ConceptField;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * A visitor that modifies a drug pair query so that it becomes insensitive to the order of drugs in a drug pair or the
 * corresponding FDB interaction id.
 * 
 * @author vhaisaalbua
 */
public class DrugPairVisitor implements QueryConditionVisitor {

    /** The result. */
    private QueryCondition result;

    /** The first pass. */
    private boolean firstPass = true;

    /**
     * Get Result.
     * 
     * @return result
     */
    @Override
    public final QueryCondition getResult() {
        return result;
    }

    /**
     * First Pass.
     * 
     * @param cond
     *            - Query Condition
     * @return true/false
     */
    private boolean isFirstPass(QueryCondition cond) {
        if (firstPass) {
            result = cond;
            firstPass = false;
            return true;
        }
        return false;
    }

    /**
     * Visit and Conjunction Condition.
     * 
     * @param cond
     *            - AndConjunctionCondition
     * @return Collections.emptyList/singletonlist
     */
    @Override
    public final List<QueryCondition> visitAndConjunctionCondition(AndConjunctionCondition cond) {
        boolean setResult = isFirstPass(cond);
        ArrayList<QueryCondition> temp = new ArrayList<QueryCondition>();
        for (QueryCondition c : cond.getConditions()) {
            List<QueryCondition> c1 = c.accept(this);
            temp.addAll(c1);
        }
        QueryCondition c;
        if (cond.getConditions().equals(temp)) {
            c = null;
        } else {
            c = ConjunctionCondition.createConjunctionCondition(Conjunction.AND, temp);
        }
        if (setResult && c != null) {
            result = result.combine(c, Conjunction.OR);
        }

        if (c == null) {
            return Collections.emptyList();
        } else {
            return Collections.singletonList(c);
        }
    }

    /**
     * VisitOrConjunctionCondition.
     * 
     * @param cond
     *            OrConjunctionCondition
     * @return temp
     */
    @Override
    public final List<QueryCondition> visitOrConjunctionCondition(OrConjunctionCondition cond) {
        boolean setResult = isFirstPass(cond);
        ArrayList<QueryCondition> temp = new ArrayList<QueryCondition>();
        for (QueryCondition c : cond.getConditions()) {
            List<QueryCondition> list = c.accept(this);

            for (QueryCondition c1 : list) {
                if (!c.equals(c1)) {
                    temp.add(c1);
                }
            }
        }

        if (setResult) {
            for (QueryCondition c : temp) {
                result = result.combine(c, Conjunction.OR);
            }
        }
        return temp;
    }

    /**
     * VisitFieldCondition.
     * 
     * @param cond
     *            FieldCondition
     * @return Collection/result
     */
    @Override
    public final List<QueryCondition> visitFieldCondition(FieldCondition cond) {
        boolean setResult = isFirstPass(cond);
        QueryCondition c = cond;
        ConceptField field = cond.getField();

        if (DrugPairDTO.RTGENID1.equals(field.getName())) {
            FieldCondition copy = cond.copy();
            copy.setField(field.getConceptMetadata().getField(DrugPairDTO.RTGENID2));
            c = copy;
        } else if (DrugPairDTO.RTGENID2.equals(field.getName())) {
            FieldCondition copy = cond.copy();
            copy.setField(field.getConceptMetadata().getField(DrugPairDTO.RTGENID1));
            c = copy;
        } else if (DrugPairDTO.RTGENID1_DESC.equals(field.getName())) {
            FieldCondition copy = cond.copy();
            copy.setField(field.getConceptMetadata().getField(DrugPairDTO.RTGENID2_DESC));
            c = copy;
        } else if (DrugPairDTO.RTGENID2_DESC.equals(field.getName())) {
            FieldCondition copy = cond.copy();
            copy.setField(field.getConceptMetadata().getField(DrugPairDTO.RTGENID1_DESC));
            c = copy;
        } else if (DrugPairDTO.FDB_ID.equals(field.getName())) {
            StringBuffer buf = new StringBuffer("(32000 - ");
            buf.append(cond.getField().getPersistentPath(true)).append(")");
            buf.append(" ").append(cond.getOperator().getSymbol()).append(" ");
            buf.append(cond.getOperator().formatHQLValue(cond.getValue(), cond.getField()));
            c = new HibernateCondition(buf.toString());
        }

        if (setResult && c != cond) {
            result = result.combine(c, Conjunction.OR);
        }
        return Collections.singletonList(c);
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.query.QueryConditionVisitor#visitHibernateCondition(gov.va.med.pharmacy.pecs.query.
     * HibernateCondition)
     */
    @Override
    public final List<QueryCondition> visitHibernateCondition(HibernateCondition cond) {
        return Collections.singletonList((QueryCondition) cond);
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.query.QueryConditionVisitor#visitParenthesisCondition(gov.va.med.pharmacy.pecs.query.
     * ParenthesisCondition)
     */
    @Override
    public final List<QueryCondition> visitParenthesisCondition(ParenthesisCondition cond) {
        boolean wasFirstPass = firstPass;
        List<QueryCondition> temp = cond.getNested().accept(this);
        if (wasFirstPass) {
            result = new ParenthesisCondition(result);
            return Collections.singletonList(result);
        } else {
            switch (temp.size()) {
            case 0:
                throw new IllegalStateException("0 conditions returned");
            case 1:
                return Collections.singletonList((QueryCondition) new ParenthesisCondition(temp.get(0)));
            default:
                return Collections.singletonList((QueryCondition) new ParenthesisCondition(ConjunctionCondition
                        .createConjunctionCondition(Conjunction.OR, temp)));
            }
        }
    }

}
