package gov.va.med.pharmacy.pecs.query;

import gov.va.med.pharmacy.pecs.metadata.ConceptField;
import gov.va.med.pharmacy.pecs.metadata.ConceptMetadata;

import java.io.File;
import java.io.FileOutputStream;
import java.util.Date;
import java.util.List;

import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.CreationHelper;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;


/**
 * The Class ExcelFormatter.
 * 
 * @param <T>
 *            the generic type
 */
public class ExcelFormatter<T> {

    /** The Constant MAX_XLS_2003_ROWS. */
    private static final int MAX_XLS_2003_ROWS = 65536;

    /**
     * Gets the max rows.
     * 
     * @return the max rows
     */
    public int getMaxRows() {
        return MAX_XLS_2003_ROWS;
    }

    /**
     * Sets the cell value.
     * 
     * @param cell
     *            the cell
     * @param val
     *            the val
     * @param dateCellStyle
     *            the date cell style
     */
    private void setCellValue(Cell cell, Object val, CellStyle dateCellStyle) {
        if (val == null) {
            cell.setCellValue("");
        } else if (val instanceof Date) {
            cell.setCellValue((Date) val);
            cell.setCellStyle(dateCellStyle);
        } else if (val instanceof Number) {
            cell.setCellValue(((Number) val).doubleValue());
        } else {
            cell.setCellValue(val.toString());
        }
    }

    /**
     * Gets the file.
     * 
     * @param data
     *            the data
     * @param meta
     *            the meta
     * @param custom
     *            the custom
     * @param filePath
     *            the file path
     * @param fileName
     *            the file name
     * @return the file
     * @throws Exception
     *             the exception
     */
    public File getFile(List<Object[]> data, ConceptMetadata meta, boolean custom, String filePath, String fileName)
            throws Exception {
        Workbook wb = new HSSFWorkbook();
        CreationHelper createHelper = wb.getCreationHelper();
        CellStyle cellStyle = wb.createCellStyle();
        cellStyle.setDataFormat(createHelper.createDataFormat().getFormat("m/d/yy h:mm"));
        Sheet sheet = wb.createSheet("Query Results");
        int rowNum = 0;
        Row row = sheet.createRow(rowNum++);
        int colNum = 0;
        // header
        for (ConceptField f : meta.getQueryFields(custom)) {
            Cell cell = row.createCell(colNum++);
            cell.setCellValue(f.getDisplayName());
        }

        for (Object[] d : data) {
            if (rowNum == getMaxRows()) {
                break;
            }
            row = sheet.createRow(rowNum++);
            colNum = 0;
            for (ConceptField f : meta.getQueryFields(custom)) {
                Cell cell = row.createCell(colNum);
                setCellValue(cell, d[colNum], cellStyle);
                colNum++;
            }
        }
        File path = filePath != null ? new File(filePath) : null;
        File tmpFile = File.createTempFile(fileName + "_" + System.currentTimeMillis(), ".xls", path);
        FileOutputStream fileOut = new FileOutputStream(tmpFile);
        wb.write(fileOut);
        fileOut.close();

        return tmpFile;
    }

    /**
     * Gets the file using object.
     * 
     * @param data
     *            the data
     * @param meta
     *            the meta
     * @param custom
     *            the custom
     * @param filePath
     *            the file path
     * @param fileName
     *            the file name
     * @return the file using object
     * @throws Exception
     *             the exception
     */
    public File getFileUsingObject(List<T> data, ConceptMetadata meta, boolean custom, String filePath, String fileName)
            throws Exception {
        Workbook wb = new HSSFWorkbook();
        CreationHelper createHelper = wb.getCreationHelper();
        CellStyle cellStyle = wb.createCellStyle();
        cellStyle.setDataFormat(createHelper.createDataFormat().getFormat("m/d/yy h:mm"));
        Sheet sheet = wb.createSheet("Query Results");
        int rowNum = 0;
        Row row = sheet.createRow(rowNum++);
        int colNum = 0;
        // header

        for (ConceptField f : meta.getQueryFields(custom)) {
            Cell cell = row.createCell(colNum++);
            cell.setCellValue(f.getDisplayName());
        }

        for (T d : data) {
            if (rowNum == getMaxRows()) {
                break;
            }
            row = sheet.createRow(rowNum++);
            colNum = 0;
            for (ConceptField f : meta.getQueryFields(custom)) {
                Cell cell = row.createCell(colNum++);
                setCellValue(cell, f.getValue(d), cellStyle);
            }
        }

        File path = filePath != null ? new File(filePath) : null;
        File tmpFile = File.createTempFile(fileName + "_" + System.currentTimeMillis(), ".xls", path);
        FileOutputStream fileOut = new FileOutputStream(tmpFile);
        wb.write(fileOut);
        fileOut.close();

        return tmpFile;
    }

}
