package gov.va.med.pharmacy.pecs.query;

import java.util.Calendar;
import java.util.Date;
import java.util.List;


/**
 * A visitor that builds a JSON representation of this query.
 * 
 * @author vhaisaalbua
 */
public final class HqlSerializationVisitor extends StringSerializationVisitor {
    private final boolean custom;
    private final StringBuffer buf = new StringBuffer();

    /**
     * HqlSerializationVisitor.
     * 
     * @param custom
     *            boolean
     */
    public HqlSerializationVisitor(boolean custom) {
        this.custom = custom;
    }

    @Override
    protected List<QueryCondition> doVisitConjunctionCondition(ConjunctionCondition cond, Conjunction conj) {
        boolean first = true;
        for (QueryCondition c : cond.getConditions()) {
            if (!first) {
                buf.append(" " + conj);
            } else {
                first = false;
            }
            c.accept(this);
        }

        return null;
    }

    @Override
    protected List<QueryCondition> doVisitFieldCondition(FieldCondition cond) {
        buf.append(" ");
        
        if (Date.class.equals(cond.getField().getType())){
            appendHQLForDateCondition(cond);
            return null;
        }
        if (String.class.equals(cond.getField().getType())) {
            buf.append("lower(");
        }
        buf.append(cond.getField().getPersistentPath(custom));
        if (String.class.equals(cond.getField().getType())) {
            buf.append(")");
        }
        buf.append(" ").append(cond.getOperator().getSymbol()).append(" ");
        buf.append(cond.getOperator().formatHQLValue(cond.getValue(), cond.getField()));

        return null;
    }

    @Override
    public List<QueryCondition> doVisitHibernateCondition(HibernateCondition cond) {
        buf.append(" ");
        buf.append(cond.getHql());

        return null;
    }

    @Override
    public List<QueryCondition> visitParenthesisCondition(ParenthesisCondition cond) {
        buf.append(" (");
        cond.getNested().accept(this);
        buf.append(" )");

        return null;
    }

    @Override
    public String toString() {
        return buf.toString();
    }
    
    /**
     * Appends the HQL for a Date condition.  Special treatment is needed for EQ and LE conditions with Dates, 
     * since we want to ignore the time portion of dates when querying.  
     * EQ will be transformed to: db_date >= query_date and db_date < (query_date + 1 day).
     * LE will be transformed to: db_date < (query_date + 1 day).
     * GT will be transformed to: db_date >= (query_date + 1 day).
     * @param cond FieldCondition containing a date field type.
     */
    private void appendHQLForDateCondition(FieldCondition cond) {
        if (ConditionOperator.EQ.equals(cond.getOperator().getName())){
            buf.append( " ( ");
            buf.append(cond.getField().getPersistentPath(custom));
            buf.append( " >= ");
            buf.append(cond.getOperator().formatHQLValue(cond.getValue(), cond.getField()));
            buf.append( " AND ");
            buf.append(cond.getField().getPersistentPath(custom));
            buf.append( " < ");            
            
            Date nextDay = getNextDay(cond);
            buf.append(cond.getOperator().formatHQLValue(nextDay, cond.getField()));

            buf.append( " ) ");
        }
        else if (ConditionOperator.LE.equals(cond.getOperator().getName())){
            Date nextDay = getNextDay(cond);
            buf.append(cond.getField().getPersistentPath(custom));
            buf.append( " < ");
            buf.append(cond.getOperator().formatHQLValue(nextDay, cond.getField()));
        }
        else if (ConditionOperator.GT.equals(cond.getOperator().getName())){
            Date nextDay = getNextDay(cond);
            buf.append(cond.getField().getPersistentPath(custom));
            buf.append( " >= ");
            buf.append(cond.getOperator().formatHQLValue(nextDay, cond.getField()));
        }
        else if (ConditionOperator.NEQ.equals(cond.getOperator().getName())){
            Date nextDay = getNextDay(cond);
            buf.append( " ( ");
            buf.append(cond.getField().getPersistentPath(custom));
            buf.append( " < ");            
            buf.append(cond.getOperator().formatHQLValue(cond.getValue(), cond.getField()));
            buf.append( " OR ");
            buf.append(cond.getField().getPersistentPath(custom));
            buf.append( " >= ");
            buf.append(cond.getOperator().formatHQLValue(nextDay, cond.getField()));
            buf.append( " ) ");
        }        
        else{
            buf.append(cond.getField().getPersistentPath(custom));
            buf.append(" ").append(cond.getOperator().getSymbol()).append(" ");
            buf.append(cond.getOperator().formatHQLValue(cond.getValue(), cond.getField()));            
        }        
    }

    /**
     * Calculates a Date that is one day greater than the date specified in the specified condition's value.
     * @param cond FieldCondition containing a date field type.
     * @return Date that is one day greater than the date specified in the condition's value.
     */
    private Date getNextDay(FieldCondition cond) {
        Calendar calendar = Calendar.getInstance();    
        calendar.setTime((Date)cond.getValue());
        calendar.add(Calendar.DATE, 1);
        Date nextDay = new Date(calendar.getTimeInMillis());
        return nextDay;
    }
}
