package gov.va.med.pharmacy.pecs.query;

import gov.va.med.pharmacy.pecs.metadata.ConceptField;
import gov.va.med.pharmacy.pecs.metadata.ConceptMetadata;

import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;
import org.codehaus.jackson.map.ObjectMapper;
import org.springframework.stereotype.Service;

/**
 * JsonConceptQueryFactory implements ConceptQueryFactory.
 * 
 * @author DOMAINOVERSD
 * 
 */
@Service
public final class JsonConceptQueryFactory implements ConceptQueryFactory {
    private static final Logger LOGGER = Logger.getLogger(JsonConceptQueryFactory.class);
    private final ObjectMapper mapper = new ObjectMapper();

    @SuppressWarnings("unchecked")
    @Override
    public ConceptQuery createQuery(ConceptMetadata meta, String json) {
        try {
            List<?> list = mapper.readValue(json, List.class);
            if (list.isEmpty()) {
                return new ConceptQuery(meta);
            }
            Iterator<?> iter = list.iterator();
            Map<String, String> map = (Map<String, String>) iter.next();
            QueryCondition condition = parseConditionNode(map, meta);
            ConceptQuery query = new ConceptQuery(meta, condition);

            for (; iter.hasNext();) {
                Conjunction conj = parseConjunctionNode((String) iter.next());
                // sometimes the query ends with a conjunction
                if (iter.hasNext()) {
                    @SuppressWarnings("rawtypes")
                    QueryCondition cond = parseConditionNode((Map) iter.next(), meta);
                    query.addCondition(cond, conj);
                }
            }

            return query;
        } catch (Exception e) {
            String msg = "Error parsing JSON query: " + json;
            LOGGER.error(msg, e);
            throw new RuntimeException(msg, e);
        }
    }

    /**
     * ParseConditionNode.
     * 
     * @param node
     *            Map<String,String>
     * @param meta
     *            ConceptMetadata
     * @return FieldCondition
     */
    private QueryCondition parseConditionNode(final Map<String, String> node, ConceptMetadata meta) {
        String fieldName = node.get(QueryCondition.FIELD);
        if (fieldName.endsWith("_DISPLAY_STRING")){
            fieldName = fieldName.substring(0, fieldName.length() - "_DISPLAY_STRING".length());
        }
        ConceptField field = meta.getField(fieldName);
        if (field == null) {
            throw new IllegalArgumentException("Unknown field " + fieldName);
        }
        String op = node.get(QueryCondition.OPERATOR);
        ConditionOperator operator = OperatorFactory.getInstance().getOperator(op);
        if (operator == null) {
            throw new IllegalArgumentException("Invalid operator " + op);
        }
        String value = node.get(QueryCondition.VALUE);

        return new FieldCondition(field, operator, operator.parseValue(value, field));
    }

    /**
     * Parse Conjunction Node.
     * 
     * @param conj
     *            String
     * @return Conjunction value of conj
     */
    private Conjunction parseConjunctionNode(final String conj) {
        return Conjunction.valueOf(conj);
    }

}
