package gov.va.med.pharmacy.pecs.query;

import java.util.List;

/**
 * A visitor that builds a JSON representation of this query.
 * 
 * @author vhaisaalbua
 */
public final class JsonSerializationVisitor implements QueryConditionVisitor {
    private QueryCondition result;
    private boolean firstPass = true;
    private StringBuffer buf = new StringBuffer();

    @Override
    public QueryCondition getResult() {
        return result;
    }

    /**
     * isFirstPass.
     * 
     * @param cond
     *            - Query
     * @return boolean T/F
     */
    private boolean isFirstPass(QueryCondition cond) {
        if (firstPass) {
            firstPass = false;
            // remember original query condition;
            // this is what we want getResult() to return
            // (so that this visitor has no effect of the query state)
            result = cond;
            return true;
        }
        return false;
    }

    @Override
    public List<QueryCondition> visitAndConjunctionCondition(AndConjunctionCondition cond) {
        return visitConjunctionCondition(cond, Conjunction.AND);
    }

    @Override
    public List<QueryCondition> visitOrConjunctionCondition(OrConjunctionCondition cond) {
        return visitConjunctionCondition(cond, Conjunction.OR);
    }

    /**
     * VisitConjunctionCondition.
     * 
     * @param cond
     *            ConjunctionCondition
     * @param conj
     *            Conjunction
     * @return null
     */
    private List<QueryCondition> visitConjunctionCondition(ConjunctionCondition cond, Conjunction conj) {
        isFirstPass(cond);
        boolean first = true;
        for (QueryCondition c : cond.getConditions()) {
            if (!first) {
                buf.append(", ");
                buf = asJson(conj, buf);
                buf.append(", ");
            } else {
                first = false;
            }
            c.accept(this);
        }

        return null;
    }

    @Override
    public List<QueryCondition> visitFieldCondition(FieldCondition cond) {
        isFirstPass(cond);
        buf.append("{ ");
        buf = asJson(QueryCondition.FIELD, cond.getField().getName(), buf).append(", ");
        buf = asJson(QueryCondition.OPERATOR, cond.getOperator().getName(), buf).append(", ");
        buf = asJson(QueryCondition.VALUE, cond.getOperator().formatXMLValue(cond.getValue(), cond.getField()), buf);
        buf.append(" }");
        return null;
    }

    @Override
    public List<QueryCondition> visitHibernateCondition(HibernateCondition cond) {
        // don't process
        isFirstPass(cond);
        return null;
    }

    @Override
    public List<QueryCondition> visitParenthesisCondition(ParenthesisCondition cond) {
        throw new UnsupportedOperationException("JSON query format doesn't support paranthesized queries");
    }

    /**
     * String Buffer asJson.
     * 
     * @param name
     *            Object
     * @param value
     *            Object
     * @param buffer
     *            StringBuffer
     * @return buf
     */
    private StringBuffer asJson(Object name, Object value, StringBuffer buffer) {
        buffer = asJson(name, buffer).append(" : ");
        asJson(value, buffer);
        return buffer;
    }

    /**
     * String Buffer as Json.
     * 
     * @param value
     *            Object
     * @param buffer
     *            StringBuffer
     * @return buf
     */
    private StringBuffer asJson(Object value, StringBuffer buffer) {
        buffer.append('"').append(value).append('"');
        return buffer;
    }

    @Override
    public String toString() {
        return "[ " + buf + " ]";
    }

}
