package gov.va.med.pharmacy.pecs.query;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.apache.log4j.Logger;
import org.hibernate.Query;

/**
 * The Class Page.
 */
public class Page {

    /** seconds in one minute. Used for query timeout. */
    private static final int ONE_MINUTE = 60;

    /** The Constant log. */
    private static final Logger LOG = Logger.getLogger(Page.class);

    /** The Constant DEFAULT_SIZE. */
    public static final int DEFAULT_SIZE = 100;

    /** The results. */
    private List<?> results;

    /** The flat results. */
    private ArrayList<List<String>> flatResults;

    /** The page size. */
    private final int pageSize;

    /** The page. */
    private final int page;

    /** The query size. */
    private int querySize = -1;

    /** The total pages. */
    private int totalPages = -1;

    /**
     * Instantiates a new page.
     * 
     * @param query
     *            the query
     * @param session
     *            the session
     * @param page
     *            the page
     * @param pageSize
     *            the page size
     */
    public Page(String query, org.hibernate.Session session, int page, int pageSize) {
        if (LOG.is   ugEnabled())
            LOG.debug("Page object received page: " + page + " and pagesize: " + pageSize + " and query: " + query);

        validatePagingParameters(page, pageSize);

        this.page = page;
        this.pageSize = pageSize;
        
        String countQuery = " Select count(*) " + query;
        int inxOrderBy = countQuery.indexOf("order by");
        if (inxOrderBy > 0){
            countQuery = countQuery.substring(0,inxOrderBy);
        }
        org.hibernate.Query qry = session.createQuery(countQuery).setTimeout(ONE_MINUTE);
        querySize = ((Number) qry.uniqueResult()).intValue();

        logQuerySize();

        if (querySize > 0) {
            // Hibernate API is 0 based, while the GUI is 1 based
            page--;
            Query hQuery = session.createQuery(query);

            int startRow = page * pageSize;
            int howManyToFetch = pageSize;
            results = hQuery.setFirstResult(startRow).setMaxResults(howManyToFetch).list();            
            LOG.debug("Querying startRow " + startRow + " for " + howManyToFetch + " with parameters: " 
                    + Arrays.asList(hQuery.getNamedParameters()));
            org.hibernate.stat.Statistics statistics = session.getSessionFactory().getStatistics();
            statistics.setStatisticsEnabled(true);

            long hit0 = statistics.getQueryCacheHitCount();
            long miss0 = statistics.getSecondLevelCacheMissCount();
            LOG.debug("hit" + hit0 + " missed " + miss0);
        } else {
            results = new ArrayList();
        }
    }

    /**
     * Instantiates a new page.
     * 
     * @param queryResults
     *            the query results
     * @param page
     *            the page
     * @param pageSize
     *            the page size
     */
    public Page(ArrayList<List<String>> queryResults, int page, int pageSize) {
        if (LOG.is   ugEnabled())
            LOG.debug("Page object received page: " + page + " and pagesize: " + pageSize);

        validatePagingParameters(page, pageSize);

        this.flatResults = new ArrayList<List<String>>();
        this.page = page;
        this.pageSize = pageSize;

        if (queryResults != null) {
            querySize = queryResults.size();
        }

        logQuerySize();

        if (querySize > 0) {
            if (querySize <= pageSize) {
                for (int i = 0; i < querySize; i++) {
                    flatResults.add(queryResults.get(i));
                }
            } else if (page == 1) {
                for (int i = 0; i < pageSize; i++) {
                    flatResults.add(queryResults.get(i));
                }
            } else {
                int tempPageSize = pageSize;
                if (page == getTotalPages()) {
                    tempPageSize = (querySize % pageSize);
                }
                int step = (page - 1) * pageSize;
                for (int i = step, j = 0; j < tempPageSize; i++, j++) {
                    flatResults.add(queryResults.get(i));
                }
            }
        } else {
            flatResults = new ArrayList<List<String>>();
            querySize = 0;
            results = new ArrayList();
        }
    }

    /**
     * Checks if is next page.
     * 
     * @return true, if is next page
     */
    public boolean isNextPage() {
        if (results == null || results.size() == 0) {
            return false;
        }

        return results.size() >= pageSize;
    }

    /**
     * Checks if is previous page.
     * 
     * @return true, if is previous page
     */
    public boolean isPreviousPage() {
        return page > 0;
    }

    /**
     * Gets the results.
     * 
     * @return the results
     */
    public List getResults() {
        return isNextPage() ? results.subList(0, pageSize) : results;
    }

    /**
     * Gets the flat results.
     * 
     * @return the flat results
     */
    public ArrayList<List<String>> getFlatResults() {
        return flatResults;
    }

    /**
     * Sets the results.
     * 
     * @param results
     *            the new results
     */
    public void setResults(List results) {
        this.results = results;
    }

    /**
     * Gets the total pages.
     * 
     * @return the total pages
     */
    public int getTotalPages() {
        if (totalPages == -1) {
            if (querySize == -1) {
                totalPages = 0;
            } else {
                if (pageSize > querySize) {
                    totalPages = 1;
                } else {
                    totalPages = querySize / pageSize;
                    if ((querySize % pageSize) > 0) {
                        totalPages++;
                    }
                }
            }
        }

        if (LOG.is   ugEnabled())
            LOG.debug("Paging computed total pages at: " + totalPages);

        return totalPages;
    }

    /**
     * Gets the query size.
     * 
     * @return the query size
     */
    public int getQuerySize() {
        return querySize;
    }

    /**
     * Sets the query size.
     * 
     * @param querySize
     *            the new query size
     */
    public void setQuerySize(int querySize) {
        this.querySize = querySize;
    }

    /**
     * Validates the paging parameters, page number and size.
     * 
     * @param page
     *            - the page number
     * @param pageSize
     *            - the page size
     */
    private void validatePagingParameters(int page, int pageSize) {
        if (page < 1 || pageSize < 1) {
            throw new IllegalArgumentException("the page and number of rows retrieved cannot be less than one");
        }
    }

    /**
     * Logs the querySize if debug is enabled.
     */
    private void logQuerySize() {
        if (LOG.is   ugEnabled()) {
            LOG.debug("Paging found total query size: " + querySize);
        }
    }
}
