package gov.va.med.pharmacy.pecs.query;

import java.util.List;

/**
 * A visitor used to process a concept query's condition. The contract between the visitor and the query is that after the
 * visitor has done its job, it will return a new {@link QueryCondition} the the {@link ConceptQuery} will use to replace its
 * own condition.
 * <p>
 * Visitors don't necessarily have to modify the condition. An example of such processing is serializing the query to various
 * formats (e.g. JSON, HQL). In that case, the original condition must be returned.
 * 
 * 
 * @author vhaisaalbua
 */
public interface QueryConditionVisitor {
    /**
     * Visit And Conjunction Condition.
     * 
     * @param cond
     *            - AndConjunctionCondition
     * @return no return
     */
    List<QueryCondition> visitAndConjunctionCondition(AndConjunctionCondition cond);

    /**
     * Visit Or Conjunction Condition.
     * 
     * @param cond
     *            OrConjunctionCondition.
     * @return no return
     */
    List<QueryCondition> visitOrConjunctionCondition(OrConjunctionCondition cond);

    /**
     * Visit Field Conjunction Condition.
     * 
     * @param cond
     *            - FieldCondition
     * @return none
     */
    List<QueryCondition> visitFieldCondition(FieldCondition cond);

    /**
     * Visit Hibernate Condition.
     * 
     * @param cond
     *            -HibernateCondition
     * @return List
     */
    List<QueryCondition> visitHibernateCondition(HibernateCondition cond);

    /**
     * Visit Parenthesis Condition.
     * 
     * @param cond
     *            - ParenthesisCondition
     * @return List
     */
    List<QueryCondition> visitParenthesisCondition(ParenthesisCondition cond);

    /**
     * The processing result. As mentioned before, for visitors that don't alter the query condition, this will be the first
     * condition visited.
     * 
     * @return the condition that should replace the original query condition
     */
    QueryCondition getResult();

}
