package gov.va.med.pharmacy.pecs.query;

import gov.va.med.pharmacy.pecs.metadata.ConceptField;
import gov.va.med.pharmacy.pecs.metadata.ConceptMetadata;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * A visitor that modifies a query so that it only contains either custom or FDB conditions.
 * 
 * @author vhaisaalbua
 */
public final class QueryTypeVisitor implements QueryConditionVisitor {
    private QueryCondition result;
    private boolean firstPass = true;
    private final boolean custom;

    /**
     * QueryTypeVisitor.
     * 
     * @param custom
     *            - boolean
     */
    public QueryTypeVisitor(boolean custom) {
        this.custom = custom;
    }

    @Override
    public QueryCondition getResult() {
        return result;
    }

    /**
     * Boolean Check Is First Pass.
     * 
     * @param cond
     *            - QueryCondition
     * @return boolean -T/F
     */
    private boolean isFirstPass(QueryCondition cond) {
        if (firstPass) {
            firstPass = false;
            return true;
        }
        return false;
    }

    @Override
    public List<QueryCondition> visitAndConjunctionCondition(AndConjunctionCondition cond) {
        boolean setResult = isFirstPass(cond);
        ArrayList<QueryCondition> temp = new ArrayList<QueryCondition>();
        for (QueryCondition c : cond.getConditions()) {
            List<QueryCondition> list = c.accept(this);
            temp.addAll(list);
        }
        if (temp.size() < cond.getConditions().size()){            
            return Collections.emptyList();
        }
        else{
            QueryCondition newCond = ConjunctionCondition.createConjunctionCondition(Conjunction.AND, temp);
            if (setResult) {
                result = newCond;
            }
            return Collections.singletonList(newCond);
        }
    }

    @Override
    public List<QueryCondition> visitOrConjunctionCondition(OrConjunctionCondition cond) {
        boolean setResult = isFirstPass(cond);
        ArrayList<QueryCondition> temp = new ArrayList<QueryCondition>();
        for (QueryCondition c : cond.getConditions()) {
            List<QueryCondition> list = c.accept(this);
            temp.addAll(list);
        }

        QueryCondition newCond;
        switch (temp.size()) {
        case 0:
            newCond = null;
            break;
        case 1:
            newCond = temp.get(0);
            break;
        default:
            newCond = ConjunctionCondition.createConjunctionCondition(Conjunction.OR, temp);
        }
        if (setResult) {
            if (!temp.isEmpty()) {
                result = newCond;
            }
        }
        return Collections.singletonList(newCond);
    }

    @Override
    public List<QueryCondition> visitFieldCondition(FieldCondition cond) {
        boolean setResult = isFirstPass(cond);
        ConceptField field = cond.getField();
        if (ConceptMetadata.noValue(field.getPersistentPath(custom))) {
            if (setResult) {
                result = null;
            }
            return Collections.emptyList();
        } else {
            if (setResult) {
                result = cond;
            }
            return Collections.singletonList((QueryCondition) cond);
        }
    }

    @Override
    public List<QueryCondition> visitHibernateCondition(HibernateCondition cond) {
        return Collections.singletonList((QueryCondition) cond);
    }

    @Override
    public List<QueryCondition> visitParenthesisCondition(ParenthesisCondition cond) {
        boolean setResult = isFirstPass(cond);
        List<QueryCondition> temp = cond.getNested().accept(this);

        QueryCondition newCond;
        switch (temp.size()) {
        case 0:
            newCond = null;
            break;
        case 1:
            newCond = new ParenthesisCondition(temp.get(0));
            break;
        default:
            throw new IllegalStateException("More than one condition returned");
        }
        if (setResult) {
            if (!temp.isEmpty()) {
                result = newCond;
            }
        }
        return Collections.singletonList(newCond);
    }

}
