package gov.va.med.pharmacy.pecs.rules;

import gov.va.med.pharmacy.pecs.metadata.ConceptMetadata;

import java.util.ArrayList;
import java.util.List;

/**
 * An attempt to encapsulate the enforcement of various business rules for a concept. Business rules are registered along with
 * triggers that verify when they should be checked. At run time the caller will present object to be validated for the
 * application of a specific action.
 * 
 * @author vhaisacarusr
 * 
 */
public final class BusinessRulesValidationImpl implements BusinessRulesValidation {
    private List<Entry> rules = new ArrayList<Entry>();
    private ConceptMetadata metadata;

    /**
     * Constructor that takes the metadata for a argument.
     * 
     * @param meta
     *            the concept metadata
     */
    public BusinessRulesValidationImpl(ConceptMetadata meta) {
        metadata = meta;
    }

    @Override
    public void registerRule(BusinessRule rule, BusinessRuleTrigger trigger) {
        rule.setMetadata(metadata);
        trigger.setMetadata(metadata);
        rules.add(new Entry(rule, trigger));
    }

    @Override
    public List<ValidationResult> validate(Object crt, Object prev, String action) {
        List<ValidationResult> res = new ArrayList<ValidationResult>();
        for (Entry entry : rules) {
            if (entry.getTrigger().shouldFire(crt, prev, action)) {
                res.add(entry.getRule().validate(crt, prev));
            }
        }

        return res;
    }

    @Override
    public ConceptMetadata getMetadata() {
        return metadata;
    }

    private static final class Entry {
        private BusinessRule rule;
        private BusinessRuleTrigger trigger;

        /**
         * @param r
         *            the business rule
         * @param t
         *            the business rule trigger
         */
        private Entry(BusinessRule r, BusinessRuleTrigger t) {
            setRule(r);
            setTrigger(t);
        }

        /**
         * Gets the rule.
         * 
         * @return the rule
         */
        public BusinessRule getRule() {
            return rule;
        }

        /**
         * Sets the rule.
         * 
         * @param rule
         *            the rule to set
         */
        public void setRule(BusinessRule rule) {
            this.rule = rule;
        }

        /**
         * Gets the trigger.
         * 
         * @return the trigger
         */
        public BusinessRuleTrigger getTrigger() {
            return trigger;
        }

        /**
         * Sets the trigger.
         * 
         * @param trigger
         *            the trigger to set
         */
        public void setTrigger(BusinessRuleTrigger trigger) {
            this.trigger = trigger;
        }
    }

}
