/**
 * 
 */
package gov.va.med.pharmacy.pecs.rules.validation;

import gov.va.med.pharmacy.pecs.dto.PecsDTO;
import gov.va.med.pharmacy.pecs.metadata.ConceptMetadata;
import gov.va.med.pharmacy.pecs.rules.ActionTrigger;
import gov.va.med.pharmacy.pecs.rules.AllRequiredFieldsRule;
import gov.va.med.pharmacy.pecs.rules.AlwaysFailRule;
import gov.va.med.pharmacy.pecs.rules.AlwaysFireTrigger;
import gov.va.med.pharmacy.pecs.rules.BusinessRulesValidation;
import gov.va.med.pharmacy.pecs.rules.BusinessRulesValidationImpl;
import gov.va.med.pharmacy.pecs.rules.FieldValueTrigger;
import gov.va.med.pharmacy.pecs.rules.InternalFieldsRule;
import gov.va.med.pharmacy.pecs.rules.LargeTextFieldRule;
import gov.va.med.pharmacy.pecs.rules.NotTrigger;
import gov.va.med.pharmacy.pecs.rules.RequiredFieldRule;
import gov.va.med.pharmacy.pecs.state.Status;

/**
 * This class encapsulates the validation rules that need to be run across all concepts.
 * 
 * @author vhaisacarusr
 * 
 */
public final class GenericBusinessRules {
    /**
     * private constructor to prevent instantiation.
     */
    private GenericBusinessRules() {
    }

    /**
     * The generic data validation across all concepts.
     * 
     * @param meta
     *            the metadata for the concept
     * @return the resutls of the
     */
    public static BusinessRulesValidation getGenericDataValidation(ConceptMetadata meta) {
        // define constants for action string values
        BusinessRulesValidationImpl validator = new BusinessRulesValidationImpl(meta);
        // all required fields must be present
        validator.registerRule(new AllRequiredFieldsRule(), new AlwaysFireTrigger());
        // comments can have at most 4000 characters
        validator.registerRule(new LargeTextFieldRule(PecsDTO.CURRENT_ACTION_REASON, "it can have at most 4000 characters"),
                new AlwaysFireTrigger());
        // rejecting a record requires a comment
        validator.registerRule(new RequiredFieldRule(PecsDTO.CURRENT_ACTION_REASON), new ActionTrigger("Reject"));
        // creating a record requires a comment
        validator.registerRule(new RequiredFieldRule(PecsDTO.CURRENT_ACTION_REASON), new ActionTrigger("New"));
        // approving deletion requires a comment
        validator.registerRule(new RequiredFieldRule(PecsDTO.CURRENT_ACTION_REASON), new ActionTrigger("Delete"));
        // any action taken on an approved record requires a comment
        validator.registerRule(new RequiredFieldRule(PecsDTO.CURRENT_ACTION_REASON), new FieldValueTrigger(
                PecsDTO.ACTION_STATUS, Status.STATUS_APPROVED));
        // any action taken on a deleted record requires a comment
        validator.registerRule(new RequiredFieldRule(PecsDTO.CURRENT_ACTION_REASON), new FieldValueTrigger(
                PecsDTO.ACTION_STATUS, Status.STATUS_DELETED));
        // any action taken on a rejected record requires a comment
        validator.registerRule(new RequiredFieldRule(PecsDTO.CURRENT_ACTION_REASON), new FieldValueTrigger(
                PecsDTO.ACTION_STATUS, Status.STATUS_REJECTED));
        // second approval cannot change any FDB fields
        validator.registerRule(new InternalFieldsRule(), new ActionTrigger("Approve"));
        validator.registerRule(new InternalFieldsRule(), new ActionTrigger("Delete"));
        String msg1 =
                "Action 'Submit as Reviewed' cannot be performed on modified records. "
                        + "Please click the 'Modify' button after changing fields.";
        validator.registerRule(new InternalFieldsRule(msg1), new ActionTrigger("Submit_as_Reviewed"));
        String msg2 =
                "Action 'Submit for Delete' cannot be performed on modified records. "
                        + "Please click the 'Modify' button after changing fields.";
        validator.registerRule(new InternalFieldsRule(msg2), new ActionTrigger("Submit_for_Delete"));
        return validator;
    }

    /**
     * Used to determine whether the action should cause a status change. The rule is that modifying only internal fields should
     * not cause a status change.
     * 
     * @param meta
     *            the metadata for the concept this rule is checked
     * @return the validator
     */
    public static BusinessRulesValidation getGenericStatusChangeValidation(ConceptMetadata meta) {
        BusinessRulesValidationImpl validator = new BusinessRulesValidationImpl(meta);
        // for 'Modify', return success if only internal fields are modified
        validator.registerRule(new InternalFieldsRule(), new ActionTrigger("Modify"));
        // if the action is not 'Modify', fail (status change required)
        validator.registerRule(new AlwaysFailRule(), new NotTrigger(new ActionTrigger("Modify")));
        return validator;
    }
}
