package gov.va.med.pharmacy.pecs.service;

import gov.va.med.pharmacy.pecs.domain.Lookup;
import gov.va.med.pharmacy.pecs.domain.impl.lookup.ClinicalEffect;
import gov.va.med.pharmacy.pecs.domain.impl.lookup.ComparisonReportConcept;
import gov.va.med.pharmacy.pecs.domain.impl.lookup.DoseRoute;
import gov.va.med.pharmacy.pecs.domain.impl.lookup.DoseType;
import gov.va.med.pharmacy.pecs.domain.impl.lookup.DoseUnitType;
import gov.va.med.pharmacy.pecs.domain.impl.lookup.DuplicateAllowance;
import gov.va.med.pharmacy.pecs.domain.impl.lookup.EasySearchSelectType;
import gov.va.med.pharmacy.pecs.domain.impl.lookup.HalfLifeUnit;
import gov.va.med.pharmacy.pecs.domain.impl.lookup.SeverityLevel;
import gov.va.med.pharmacy.pecs.domain.impl.va.ReferenceTextVa;

import java.util.List;

/**
 * The Interface TerminologyService.
 */
public interface TerminologyService {

    /**
     * Gets the comparison report concepts.
     * 
     * @return the comparison report concepts @ the customization service exception
     */
    public List<ComparisonReportConcept> getComparisonReportConcepts();

    /**
     * Gets the dose unit type data.
     * 
     * @return the dose unit type data @ the customization service exception
     */
    public List<DoseUnitType> getDoseUnitTypeData();

    /**
     * Gets the duplicate allowances.
     * 
     * @return the duplicate allowances
     */
    public List<DuplicateAllowance> getDuplicateAllowances();

    /**
     * Gets the duplicate allowance by code.
     * 
     * @param duplicateAllowance
     *            the duplicate allowance code
     * @return the duplicate allowance by code
     */
    public DuplicateAllowance getDuplicateAllowanceByCode(Long duplicateAllowance);

    /**
     * Gets the half life units.
     * 
     * @return the half life units
     */
    public List<HalfLifeUnit> getHalfLifeUnits();

    /**
     * Gets the half life unit by code.
     * 
     * @param halfLifeUnitCode
     *            the half life unit code
     * @return the half life unit by code @ the customization service exception
     */
    public HalfLifeUnit getHalfLifeUnitByCode(String halfLifeUnitCode);

    /**
     * Gets the clinical effects.
     * 
     * @return the clinical effects @ the customization service exception
     */
    public List<ClinicalEffect> getClinicalEffects();

    /**
     * Gets the clinical effect by code.
     * 
     * @param clinicalEffectCode
     *            the clinical effect code
     * @return the clinical effect by code @ the customization service exception
     */
    public ClinicalEffect getClinicalEffectByCode(String clinicalEffectCode);

    /**
     * Gets the by code.
     * 
     * @param clazz
     *            the clazz
     * @param code
     *            the code
     * @return the by code @ the customization service exception
     */
    @SuppressWarnings("rawtypes")
    public Lookup getByCode(Class clazz, String code);

    /**
     * Gets the dose routes.
     * 
     * @return the dose routes @ the customization service exception
     */
    public List<DoseRoute> getDoseRoutes();

    /**
     * Gets the dose route.
     * 
     * @param id
     *            the id
     * @return the dose route @ the customization service exception
     */
    public DoseRoute getDoseRoute(Long id);

    /**
     * Gets the dose route by code.
     * 
     * @param code
     *            the code
     * @return the dose route by code @ the customization service exception
     */
    public DoseRoute getDoseRouteByCode(String code);

    /**
     * Gets the dose types.
     * 
     * @return the dose types @ the customization service exception
     */
    public List<DoseType> getDoseTypes();

    /**
     * Gets the dose type.
     * 
     * @param id
     *            the id
     * @return the dose type @ the customization service exception
     */
    public DoseType getDoseType(Long id);

    /**
     * Gets the dose type by code.
     * 
     * @param code
     *            the code
     * @return the dose type by code @ the customization service exception
     */
    public DoseType getDoseTypeByCode(String code);

    /**
     * Gets the easy search select types.
     * 
     * @return the easy search select types @ the customization service exception
     */
    public List<EasySearchSelectType> getEasySearchSelectTypes();

    /**
     * Gets the easy search select type by code.
     * 
     * @param code
     *            the code
     * @return the easy search select type by code @ the customization service exception
     */
    public EasySearchSelectType getEasySearchSelectTypeByCode(String code);

    // public List<PeriodOfTime> getPeriodOfTime() ;
    // public PeriodOfTime getPeriodOfTime(Long id) ;
    /**
     * Gets the severity levels.
     * 
     * @return the severity levels @ the customization service exception
     */
    public List<SeverityLevel> getSeverityLevels();

    /**
     * Gets the severity level by code.
     * 
     * @param severityLevelCode
     *            the severity level code
     * @return the severity level by code @ the customization service exception
     */
    public SeverityLevel getSeverityLevelByCode(String severityLevelCode);

    /**
     * Gets the aIDS guidelines.
     * 
     * @return the aIDS guidelines @ the customization service exception
     */
    public List<ReferenceTextVa> getAIDSGuidelines();

    /**
     * Gets the aIDS guideline by id.
     * 
     * @param id
     *            the id
     * @return the aIDS guideline by id @ the customization service exception
     */
    public ReferenceTextVa getAIDSGuidelineById(Long id);

    /**
     * Gets the aIDS guideline by name.
     * 
     * @param name
     *            the name
     * @return the aIDS guideline by name @ the customization service exception
     */
    public ReferenceTextVa getAIDSGuidelineByName(String name);

    /**
     * Gets the drug interaction facts.
     * 
     * @return the drug interaction facts @ the customization service exception
     */
    public List<ReferenceTextVa> getDrugInteractionFacts();

    /**
     * Gets the drug interaction fact by id.
     * 
     * @param id
     *            the id
     * @return the drug interaction fact by id @ the customization service exception
     */
    public ReferenceTextVa getDrugInteractionFactById(Long id);

    /**
     * Gets the drug interaction fact by name.
     * 
     * @param name
     *            the name
     * @return the drug interaction fact by name @ the customization service exception
     */
    public ReferenceTextVa getDrugInteractionFactByName(String name);

    /**
     * Gets the drug interaction fact documentation.
     * 
     * @return the drug interaction fact documentation @ the customization service exception
     */
    public List<ReferenceTextVa> getDrugInteractionFactDocumentation();

    /**
     * Gets the drug interaction fact documentation by id.
     * 
     * @param id
     *            the id
     * @return the drug interaction fact documentation by id @ the customization service exception
     */
    public ReferenceTextVa getDrugInteractionFactDocumentationById(Long id);

    /**
     * Gets the drug interaction fact documentation by name.
     * 
     * @param name
     *            the name
     * @return the drug interaction fact documentation by name @ the customization service exception
     */
    public ReferenceTextVa getDrugInteractionFactDocumentationByName(String name);

    /**
     * Gets the drug interaction fact onsets.
     * 
     * @return the drug interaction fact onsets @ the customization service exception
     */
    public List<ReferenceTextVa> getDrugInteractionFactOnsets();

    /**
     * Gets the drug interaction fact onset by id.
     * 
     * @param id
     *            the id
     * @return the drug interaction fact onset by id @ the customization service exception
     */
    public ReferenceTextVa getDrugInteractionFactOnsetById(Long id);

    /**
     * Gets the drug interaction fact onset by name.
     * 
     * @param name
     *            the name
     * @return the drug interaction fact onset by name @ the customization service exception
     */
    public ReferenceTextVa getDrugInteractionFactOnsetByName(String name);

    /**
     * Gets the drug interaction fact severities.
     * 
     * @return the drug interaction fact severities @ the customization service exception
     */
    public List<ReferenceTextVa> getDrugInteractionFactSeverities();

    /**
     * Gets the drug interaction fact severity by id.
     * 
     * @param id
     *            the id
     * @return the drug interaction fact severity by id @ the customization service exception
     */
    public ReferenceTextVa getDrugInteractionFactSeverityById(Long id);

    /**
     * Gets the drug interaction fact severity by name.
     * 
     * @param name
     *            the name
     * @return the drug interaction fact severity by name @ the customization service exception
     */
    public ReferenceTextVa getDrugInteractionFactSeverityByName(String name);

    /**
     * Gets the evaluation of drug interactions.
     * 
     * @return the evaluation of drug interactions @ the customization service exception
     */
    public List<ReferenceTextVa> getEvaluationOfDrugInteractions();

    /**
     * Gets the evaluation of drug interaction by id.
     * 
     * @param id
     *            the id
     * @return the evaluation of drug interaction by id @ the customization service exception
     */
    public ReferenceTextVa getEvaluationOfDrugInteractionById(Long id);

    /**
     * Gets the evaluation of drug interaction by name.
     * 
     * @param name
     *            the name
     * @return the evaluation of drug interaction by name @ the customization service exception
     */
    public ReferenceTextVa getEvaluationOfDrugInteractionByName(String name);

    /**
     * Gets the interaction sources.
     * 
     * @return the interaction sources @ the customization service exception
     */
    public List<ReferenceTextVa> getInteractionSources();

    /**
     * Gets the interaction source by id.
     * 
     * @param id
     *            the id
     * @return the interaction source by id @ the customization service exception
     */
    public ReferenceTextVa getInteractionSourceById(Long id);

    /**
     * Gets the interaction source by name.
     * 
     * @param name
     *            the name
     * @return the interaction source by name @ the customization service exception
     */
    public ReferenceTextVa getInteractionSourceByName(String name);

    /**
     * Gets the interaction types.
     * 
     * @return the interaction types @ the customization service exception
     */
    public List<ReferenceTextVa> getInteractionTypes();

    /**
     * Gets the interaction type by id.
     * 
     * @param id
     *            the id
     * @return the interaction type by id @ the customization service exception
     */
    public ReferenceTextVa getInteractionTypeById(Long id);

    /**
     * Gets the interaction type by name.
     * 
     * @param name
     *            the name
     * @return the interaction type by name @ the customization service exception
     */
    public ReferenceTextVa getInteractionTypeByName(String name);

    /**
     * Gets the level of evidence.
     * 
     * @return the level of evidence @ the customization service exception
     */
    public List<ReferenceTextVa> getLevelOfEvidence();

    /**
     * Gets the level of evidence by id.
     * 
     * @param id
     *            the id
     * @return the level of evidence by id @ the customization service exception
     */
    public ReferenceTextVa getLevelOfEvidenceById(Long id);

    /**
     * Gets the level of evidence by name.
     * 
     * @param name
     *            the name
     * @return the level of evidence by name @ the customization service exception
     */
    public ReferenceTextVa getLevelOfEvidenceByName(String name);

    /**
     * Gets the medline hits.
     * 
     * @return the medline hits @ the customization service exception
     */
    public List<ReferenceTextVa> getMedlineHits();

    /**
     * Gets the medline hit by id.
     * 
     * @param id
     *            the id
     * @return the medline hit by id @ the customization service exception
     */
    public ReferenceTextVa getMedlineHitById(Long id);

    /**
     * Gets the medline hit by name.
     * 
     * @param name
     *            the name
     * @return the medline hit by name @ the customization service exception
     */
    public ReferenceTextVa getMedlineHitByName(String name);

    /**
     * Gets the micromedex onsets.
     * 
     * @return the micromedex onsets @ the customization service exception
     */
    public List<ReferenceTextVa> getMicromedexOnsets();

    /**
     * Gets the micromedex onset by id.
     * 
     * @param id
     *            the id
     * @return the micromedex onset by id @ the customization service exception
     */
    public ReferenceTextVa getMicromedexOnsetById(Long id);

    /**
     * Gets the micromedex onset by name.
     * 
     * @param name
     *            the name
     * @return the micromedex onset by name @ the customization service exception
     */
    public ReferenceTextVa getMicromedexOnsetByName(String name);

    /**
     * Gets the micromedex severities.
     * 
     * @return the micromedex severities @ the customization service exception
     */
    public List<ReferenceTextVa> getMicromedexSeverities();

    /**
     * Gets the micromedex severity by id.
     * 
     * @param id
     *            the id
     * @return the micromedex severity by id @ the customization service exception
     */
    public ReferenceTextVa getMicromedexSeverityById(Long id);

    /**
     * Gets the micromedex severity by name.
     * 
     * @param name
     *            the name
     * @return the micromedex severity by name @ the customization service exception
     */
    public ReferenceTextVa getMicromedexSeverityByName(String name);

    /**
     * Gets the micromedex substantiations.
     * 
     * @return the micromedex substantiations @ the customization service exception
     */
    public List<ReferenceTextVa> getMicromedexSubstantiations();

    /**
     * Gets the micromedex substantiation by id.
     * 
     * @param id
     *            the id
     * @return the micromedex substantiation by id @ the customization service exception
     */
    public ReferenceTextVa getMicromedexSubstantiationById(Long id);

    /**
     * Gets the micromedex substantiation by name.
     * 
     * @param name
     *            the name
     * @return the micromedex substantiation by name @ the customization service exception
     */
    public ReferenceTextVa getMicromedexSubstantiationByName(String name);

    /**
     * Gets the package inserts.
     * 
     * @return the package inserts @ the customization service exception
     */
    public List<ReferenceTextVa> getPackageInserts();

    /**
     * Gets the package insert by id.
     * 
     * @param id
     *            the id
     * @return the package insert by id @ the customization service exception
     */
    public ReferenceTextVa getPackageInsertById(Long id);

    /**
     * Gets the package insert by name.
     * 
     * @param name
     *            the name
     * @return the package insert by name @ the customization service exception
     */
    public ReferenceTextVa getPackageInsertByName(String name);

    /**
     * Gets the pBM criteria.
     * 
     * @return the pBM criteria @ the customization service exception
     */
    public List<ReferenceTextVa> getPBMCriteria();

    /**
     * Gets the pBM criteria by id.
     * 
     * @param id
     *            the id
     * @return the pBM criteria by id @ the customization service exception
     */
    public ReferenceTextVa getPBMCriteriaById(Long id);

    /**
     * Gets the pBM criteria by name.
     * 
     * @param name
     *            the name
     * @return the pBM criteria by name @ the customization service exception
     */
    public ReferenceTextVa getPBMCriteriaByName(String name);

    /**
     * Gets the dose unit type by code.
     * 
     * @param doseUnit
     *            the dose unit
     * @return the dose unit type by code @ the customization service exception
     */
    public DoseUnitType getDoseUnitTypeByCode(String doseUnit);

    // The following are reserved for future use
    // public List<AIDSGuideline> getAIDSGuidelines() ;
    // public AIDSGuideline getAIDSGuidelineByCode(String severityLevelCode) ;
    // public List<DrugInteractionsFact> getDrugInteractionFacts() ;
    // public DrugInteractionsFact getDrugInteractionsFactByCode(String severityLevelCode) ;
    // public List<DrugInteractionsFactDocumentation> getDrugInteractionsFactDocumentation()
    // ;
    // public DrugInteractionsFactDocumentation getDrugInteractionsFactDocumentationByCode(String severityLevelCode)
    // ;
    // public List<DrugInteractionsFactOnset> getDrugInteractionsFactOnsets() ;
    // public DrugInteractionsFactOnset getDrugInteractionsFactOnsetByCode(String severityLevelCode)
    // ;
    // public List<DrugInteractionsFactSeverity> getDrugInteractionsFactSeverities() ;
    // public DrugInteractionsFactSeverity getDrugInteractionsFactSeverityByCode(String severityLevelCode)
    // ;
    // public List<EvaluationOfDrugInteraction> getEvaluationOfDrugInteractions() ;
    // public EvaluationOfDrugInteraction getEvaluationOfDrugInteractionByCode(String severityLevelCode)
    // ;
    // public List<InteractionSource> getInteractionSources() ;
    // public InteractionSource getInteractionSourceByCode(String severityLevelCode) ;
    // public List<InteractionType> getInteractionTypes() ;
    // public InteractionType getInteractionTypeByCode(String severityLevelCode) ;
    // public List<LevelOfEvidence> getLevelOfEvidence() ;
    // public LevelOfEvidence getLevelOfEvidenceByCode(String severityLevelCode) ;
    // public List<MedlineHit> getMedlineHits() ;
    // public MedlineHit getMedlineHitByCode(String severityLevelCode) ;
    // public List<MicromedexOnset> getMicromedexOnsets() ;
    // public MicromedexOnset getMicromedexOnsetByCode(String severityLevelCode) ;
    // public List<MicromedexSeverity> getMicromedexSeverities() ;
    // public MicromedexSeverity getMicromedexSeverityByCode(String severityLevelCode) ;
    // public List<MicromedexSubstantiation> getMicromedexSubstantiations() ;
    // public MicromedexSubstantiation getMicromedexSubstantiationByCode(String severityLevelCode)
    // ;
    // public List<PackageInsert> getPackageInserts() ;
    // public PackageInsert getPackageInsertByCode(String severityLevelCode) ;
    // public List<PBMCriteria> getPBMCriteria() ;
    // public PBMCriteria getPBMCriteriaByCode(String severityLevelCode) ;

}
