package gov.va.med.pharmacy.pecs.service.impl;

import gov.va.med.pharmacy.pecs.exception.CustomizationServiceException;
import gov.va.med.pharmacy.pecs.metadata.Concept;
import gov.va.med.pharmacy.pecs.metadata.ConceptField;
import gov.va.med.pharmacy.pecs.metadata.ConceptMetadata;
import gov.va.med.pharmacy.pecs.metadata.UserModifiableFieldMetadata;
import gov.va.med.pharmacy.pecs.metadata.UserModifiableFieldMetadataDAO;
import gov.va.med.pharmacy.pecs.metadata.UserModifiableFieldMetadataDTO;
import gov.va.med.pharmacy.pecs.service.MetadataService;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Map.Entry;

import javax.annotation.Resource;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * Meta Data Service Impl.
 * 
 * @author DOMAINOVERSD
 * 
 */
@Service(value="metadataService")
public final class MetadataServiceImpl implements MetadataService, InitializingBean {
    private static final Logger LOGGER = Logger.getLogger(MetadataServiceImpl.class);
    
    @Resource
    private UserModifiableFieldMetadataDAO userModifiableFieldMetadataDao;

    @Override
    @Transactional
    public ConceptMetadata getMetadata(Class<?> conceptClass) {
        try {
            String query = "from UserModifiableFieldMetadata where conceptName = '" + conceptClass.getSimpleName() + "'";
            List<UserModifiableFieldMetadata> dynamic =
                    userModifiableFieldMetadataDao.findByQuery(query, UserModifiableFieldMetadata.class);
            return new ConceptMetadata(conceptClass, dynamic);
        } catch (RuntimeException e) {
            String msg =
                    "Unable to determine field metadata for concept identified by class name " + conceptClass.getSimpleName();
            LOGGER.error(msg, e);
            throw e;
        }
    }

    /**
     * Find Concepts.
     * 
     * @return userModifiableFieldMetadataDao.findConcepts
     */
    @Override
    @Transactional(readOnly = true)
    public List<String> findConcepts() {
        return userModifiableFieldMetadataDao.findConcepts();
    }

    /**
     * Array List findbyConcept.
     * 
     * @param conceptName
     *            - String
     * @return list
     */
    @Override
    @Transactional(readOnly = true)
    public ArrayList<UserModifiableFieldMetadataDTO> findByConcept(String conceptName) {
        ArrayList<UserModifiableFieldMetadataDTO> list = new ArrayList<UserModifiableFieldMetadataDTO>();
        ConceptMetadata meta = getMetadata(findConcept(conceptName));
        list = buildDTO(meta);
        Collections.sort(list);
        return list;
    }

    /**
     * FindConcept.
     * 
     * @param conceptName
     *            - String
     * @return concept
     */
    private Class<?> findConcept(String conceptName) {
        Class<?> concept = null;
        if (conceptName.equals("DrugDrugInteractionDTO")) {
            concept = Concept.DRUG_DRUG_INTERACTION.getConceptClass();
        } else if (conceptName.equals("DrugPairDTO")) {
            concept = Concept.DRUG_PAIR.getConceptClass();
        } else if (conceptName.equals("DoseRangeDTO")) {
            concept = Concept.DOSE_RANGE.getConceptClass();
        } else if (conceptName.equals("DuplicateTherapyDTO")) {
            concept = Concept.DUPLICATE_THERAPY.getConceptClass();
        } else if (conceptName.equals("MonographDTO")) {
            concept = Concept.PROFESSIONAL_MONOGRAPH.getConceptClass();
        }
        return concept;
    }

    /**
     * Returns a list of UserModifiableFieldMetadataDTO from the ConceptMetadata for fields that are not Internal only.
     * 
     * @param meta
     *            - ConceptMetadata
     * @return list
     */
    private ArrayList<UserModifiableFieldMetadataDTO> buildDTO(ConceptMetadata meta) {
        ArrayList<UserModifiableFieldMetadataDTO> list = new ArrayList<UserModifiableFieldMetadataDTO>();
        Iterator<Entry<String, ConceptField>> iter = meta.getFieldMap().entrySet().iterator();
        while (iter.hasNext()) {
            Entry<String, ConceptField> entry = iter.next();
            list.add(buildDTO(entry.getValue()));
        }
        return list;
    }

    /**
     * UserModifiableFieldMetadataDTO - buildDTO.
     * 
     * @param field
     *            - ConceptField
     * @return meta
     */
    private UserModifiableFieldMetadataDTO buildDTO(ConceptField field) {
        UserModifiableFieldMetadataDTO meta = new UserModifiableFieldMetadataDTO();
        meta.setConceptName(field.getConceptMetadata().getConceptName());
        meta.setDisplayOrder("" + field.getDisplayOrder());
        meta.setFieldId(field.getFieldId());
        meta.setFieldName(field.getName());
        meta.setIncludeInDetailDisplay(field.includeInDetailDisplay());
        meta.setIncludeInQueryResults(field.includeInQueryResults());
        meta.setIncludeInReports(field.includeInReports());
        meta.setRequired(field.isRequired());
        meta.setDisplayName(field.getDisplayName());
        meta.setCanIncludeInQueryResults(field.isQueriable(true) || field.isQueriable(false));
        meta.setCanIncludeInReports(field.isReportable(true) || field.isReportable(false));
        return meta;
    }

    /**
     * modifyConfiguration.
     * 
     * @param metadata
     *            - List<UserModifiableFieldMetadataDTO>
     */
    @Override
    @Transactional
    public void modifyConfiguration(List<UserModifiableFieldMetadataDTO> metadata) {
        for (UserModifiableFieldMetadataDTO field : metadata) {
            try {
                UserModifiableFieldMetadata foundConfigColumn = userModifiableFieldMetadataDao.findById(field.getFieldId());

                if (!foundConfigColumn.equals(field)) {
                    // fieldName and conceptName cannot be changed, so don't try
                    // to here.
                    foundConfigColumn.setIncludeInDetailDisplay(field.isIncludeInDetailDisplay());
                    foundConfigColumn.setIncludeInQueryResults(field.isIncludeInQueryResults());
                    foundConfigColumn.setIncludeInReports(field.isIncludeInReports());
                    foundConfigColumn.setDisplayOrder(Integer.parseInt(field.getDisplayOrder()));
                    foundConfigColumn.setDisplayName(field.getDisplayName());
                    userModifiableFieldMetadataDao.save(foundConfigColumn);
                }
            } catch (Exception e) {
                throw new CustomizationServiceException("A problem was encountered saving the field " + field.getFieldName());
            }
        }
    }

    /**
     * UserModifiableFieldMetadataDAO getUserModifiableFieldMetadataDao.
     * 
     * @return userModifiableFieldMetadataDao
     */
    public UserModifiableFieldMetadataDAO getUserModifiableFieldMetadataDao() {
        return userModifiableFieldMetadataDao;
    }

    /**
     * Set User Modifiable Field Meta Data Dao.
     * 
     * @param userModifiableFieldMetadataDao
     *            - UserModifiableFieldMetadataDAO
     */
    public void setUserModifiableFieldMetadataDao(UserModifiableFieldMetadataDAO userModifiableFieldMetadataDao) {
        this.userModifiableFieldMetadataDao = userModifiableFieldMetadataDao;
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        if (userModifiableFieldMetadataDao == null) {
            throw new Exception("The UserModifiableFieldMetadataDAO property has not been injected in to MetadataServiceImpl.");
        }
    }
}
