package gov.va.med.pharmacy.pecs.service.impl;

import gov.va.med.pharmacy.pecs.domain.Lookup;
import gov.va.med.pharmacy.pecs.domain.impl.lookup.ClinicalEffect;
import gov.va.med.pharmacy.pecs.domain.impl.lookup.ComparisonReportConcept;
import gov.va.med.pharmacy.pecs.domain.impl.lookup.DoseRoute;
import gov.va.med.pharmacy.pecs.domain.impl.lookup.DoseType;
import gov.va.med.pharmacy.pecs.domain.impl.lookup.DoseUnitType;
import gov.va.med.pharmacy.pecs.domain.impl.lookup.DuplicateAllowance;
import gov.va.med.pharmacy.pecs.domain.impl.lookup.EasySearchSelectType;
import gov.va.med.pharmacy.pecs.domain.impl.lookup.HalfLifeUnit;
import gov.va.med.pharmacy.pecs.domain.impl.lookup.SeverityLevel;
import gov.va.med.pharmacy.pecs.domain.impl.va.ReferenceTextVa;
import gov.va.med.pharmacy.pecs.exception.CustomizationServiceException;
import gov.va.med.pharmacy.pecs.persistence.TerminologyDAO;
import gov.va.med.pharmacy.pecs.service.TerminologyService;

import java.util.List;

import javax.annotation.Resource;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.stereotype.Service;

/**
 * The Class TerminologyServiceImpl.
 */
@Service(value = "terminologyService")
public final class TerminologyServiceImpl implements InitializingBean, TerminologyService {

    /** The Constant LOGGER. */
    private static final Log LOGGER = LogFactory.getLog(TerminologyServiceImpl.class);

    /** The terminology dao. */
    @Resource
    private TerminologyDAO terminologyDAO;

    /**
     * Gets the terminology dao.
     * 
     * @return the terminology dao
     */
    public TerminologyDAO getTerminologyDAO() {
        return terminologyDAO;
    }

    /**
     * Sets the terminology dao.
     * 
     * @param terminologyDAO
     *            the new terminology dao
     */
    public void setTerminologyDAO(TerminologyDAO terminologyDAO) {
        this.terminologyDAO = terminologyDAO;
    }

    /**
     * Retrieves a List of active ComparisonReportConcept objects. The objects are returned in sorted order by sortOrder.
     * 
     * @return List
     */
    @Override
    @SuppressWarnings("unchecked")
    public List<ComparisonReportConcept> getComparisonReportConcepts() {
        return terminologyDAO.getActiveSortedLookups(ComparisonReportConcept.class);
    }

    @Override
    @SuppressWarnings("unchecked")
    public List<DoseUnitType> getDoseUnitTypeData() {
        return terminologyDAO.getActiveSortedLookups(DoseUnitType.class);
    }

    @Override
    @SuppressWarnings("unchecked")
    public List<ReferenceTextVa> getAIDSGuidelines() {
        return terminologyDAO.getReferenceData(ReferenceTextVa.AIDS_GUIDELINE_TYPE);
    }

    @Override
    public ReferenceTextVa getAIDSGuidelineById(Long id) {
        return terminologyDAO.getReferenceById(ReferenceTextVa.AIDS_GUIDELINE_TYPE, id);
    }

    @Override
    public ReferenceTextVa getAIDSGuidelineByName(String name) {
        return terminologyDAO.getReferenceByName(ReferenceTextVa.AIDS_GUIDELINE_TYPE, name);
    }

    @Override
    @SuppressWarnings("unchecked")
    public List<ClinicalEffect> getClinicalEffects() {
        return terminologyDAO.getLookups(ClinicalEffect.class);
    }

    @Override
    public ClinicalEffect getClinicalEffectByCode(String code) {
        return (ClinicalEffect) terminologyDAO.getByCode(ClinicalEffect.class, code);
    }

    @Override
    @SuppressWarnings("unchecked")
    public List<DoseRoute> getDoseRoutes() {
        return terminologyDAO.getLookups(DoseRoute.class);
    }

    @Override
    public DoseRoute getDoseRoute(Long id) {
        try {
            return (DoseRoute) terminologyDAO.getById(DoseRoute.class, id);
        } catch (Exception e) {
            throw new CustomizationServiceException(e);
        }
    }

    @Override
    public DoseRoute getDoseRouteByCode(String code) {
        return (DoseRoute) terminologyDAO.getByCode(DoseRoute.class, code);
    }

    @Override
    @SuppressWarnings("unchecked")
    public List<DoseType> getDoseTypes() {
        return terminologyDAO.getLookups(DoseType.class);
    }

    @Override
    public DoseType getDoseType(Long id) {
        try {
            return (DoseType) terminologyDAO.getById(DoseType.class, id);
        } catch (Exception e) {
            throw new CustomizationServiceException(e);
        }
    }

    @Override
    public DoseType getDoseTypeByCode(String code) {
        return (DoseType) terminologyDAO.getByCode(DoseType.class, code);
    }

    @Override
    public DoseUnitType getDoseUnitTypeByCode(String code) {
        return (DoseUnitType) terminologyDAO.getByCode(DoseUnitType.class, code);
    }

    @Override
    @SuppressWarnings("unchecked")
    public List<ReferenceTextVa> getDrugInteractionFacts() {
        return terminologyDAO.getReferenceData(ReferenceTextVa.DI_FACT_NUMBER_TYPE);
    }

    @Override
    public ReferenceTextVa getDrugInteractionFactById(Long id) {
        return terminologyDAO.getReferenceById(ReferenceTextVa.DI_FACT_NUMBER_TYPE, id);
    }

    @Override
    public ReferenceTextVa getDrugInteractionFactByName(String name) {
        return terminologyDAO.getReferenceByName(ReferenceTextVa.DI_FACT_NUMBER_TYPE, name);
    }

    @Override
    @SuppressWarnings("unchecked")
    public List<ReferenceTextVa> getDrugInteractionFactDocumentation() {
        return terminologyDAO.getReferenceData(ReferenceTextVa.DI_FACT_DOCUMENTATION_TYPE);
    }

    @Override
    public ReferenceTextVa getDrugInteractionFactDocumentationById(Long id) {
        return terminologyDAO.getReferenceById(ReferenceTextVa.DI_FACT_DOCUMENTATION_TYPE, id);
    }

    @Override
    public ReferenceTextVa getDrugInteractionFactDocumentationByName(String name) {
        return terminologyDAO.getReferenceByName(ReferenceTextVa.DI_FACT_DOCUMENTATION_TYPE, name);
    }

    @Override
    @SuppressWarnings("unchecked")
    public List<ReferenceTextVa> getDrugInteractionFactOnsets() {
        return terminologyDAO.getReferenceData(ReferenceTextVa.DI_FACT_ONSET_TYPE);
    }

    @Override
    public ReferenceTextVa getDrugInteractionFactOnsetById(Long id) {
        return terminologyDAO.getReferenceById(ReferenceTextVa.DI_FACT_ONSET_TYPE, id);
    }

    @Override
    public ReferenceTextVa getDrugInteractionFactOnsetByName(String name) {
        return terminologyDAO.getReferenceByName(ReferenceTextVa.DI_FACT_ONSET_TYPE, name);
    }

    @Override
    @SuppressWarnings("unchecked")
    public List<ReferenceTextVa> getDrugInteractionFactSeverities() {
        return terminologyDAO.getReferenceData(ReferenceTextVa.DI_FACT_SEVERITY_TYPE);
    }

    @Override
    public ReferenceTextVa getDrugInteractionFactSeverityById(Long id) {
        return terminologyDAO.getReferenceById(ReferenceTextVa.DI_FACT_SEVERITY_TYPE, id);
    }

    @Override
    public ReferenceTextVa getDrugInteractionFactSeverityByName(String name) {
        return terminologyDAO.getReferenceByName(ReferenceTextVa.DI_FACT_SEVERITY_TYPE, name);
    }

    @Override
    @SuppressWarnings("unchecked")
    public List<ReferenceTextVa> getEvaluationOfDrugInteractions() {
        return terminologyDAO.getReferenceData(ReferenceTextVa.EVALUATION_OF_DRUG_INTERACTION_TYPE);
    }

    @Override
    public ReferenceTextVa getEvaluationOfDrugInteractionById(Long id) {
        return terminologyDAO.getReferenceById(ReferenceTextVa.EVALUATION_OF_DRUG_INTERACTION_TYPE, id);
    }

    @Override
    public ReferenceTextVa getEvaluationOfDrugInteractionByName(String name) {
        return terminologyDAO.getReferenceByName(ReferenceTextVa.EVALUATION_OF_DRUG_INTERACTION_TYPE, name);
    }

    @Override
    @SuppressWarnings("unchecked")
    public List<ReferenceTextVa> getInteractionSources() {
        return terminologyDAO.getReferenceData(ReferenceTextVa.INTERACTION_SOURCE_TYPE);
    }

    @Override
    public ReferenceTextVa getInteractionSourceById(Long id) {
        return terminologyDAO.getReferenceById(ReferenceTextVa.INTERACTION_SOURCE_TYPE, id);
    }

    @Override
    public ReferenceTextVa getInteractionSourceByName(String name) {
        return terminologyDAO.getReferenceByName(ReferenceTextVa.INTERACTION_SOURCE_TYPE, name);
    }

    @Override
    @SuppressWarnings("unchecked")
    public List<ReferenceTextVa> getInteractionTypes() {
        return terminologyDAO.getReferenceData(ReferenceTextVa.INTERACTION_TYPE_TYPE);
    }

    @Override
    public ReferenceTextVa getInteractionTypeById(Long id) {
        return terminologyDAO.getReferenceById(ReferenceTextVa.INTERACTION_TYPE_TYPE, id);
    }

    @Override
    public ReferenceTextVa getInteractionTypeByName(String name) {
        return terminologyDAO.getReferenceByName(ReferenceTextVa.INTERACTION_TYPE_TYPE, name);
    }

    @Override
    @SuppressWarnings("unchecked")
    public List<ReferenceTextVa> getLevelOfEvidence() {
        return terminologyDAO.getReferenceData(ReferenceTextVa.LEVEL_OF_EVIDENCE_TYPE);
    }

    @Override
    public ReferenceTextVa getLevelOfEvidenceById(Long id) {
        return terminologyDAO.getReferenceById(ReferenceTextVa.LEVEL_OF_EVIDENCE_TYPE, id);
    }

    @Override
    public ReferenceTextVa getLevelOfEvidenceByName(String name) {
        return terminologyDAO.getReferenceByName(ReferenceTextVa.LEVEL_OF_EVIDENCE_TYPE, name);
    }

    @Override
    @SuppressWarnings("unchecked")
    public List<ReferenceTextVa> getMedlineHits() {
        return terminologyDAO.getReferenceData(ReferenceTextVa.MEDLINE_HIT_TYPE);
    }

    @Override
    public ReferenceTextVa getMedlineHitById(Long id) {
        return terminologyDAO.getReferenceById(ReferenceTextVa.MEDLINE_HIT_TYPE, id);
    }

    @Override
    public ReferenceTextVa getMedlineHitByName(String name) {
        return terminologyDAO.getReferenceByName(ReferenceTextVa.MEDLINE_HIT_TYPE, name);
    }

    @Override
    @SuppressWarnings("unchecked")
    public List<ReferenceTextVa> getMicromedexOnsets() {
        return terminologyDAO.getReferenceData(ReferenceTextVa.MICROMEDEX_ONSET_TYPE);
    }

    @Override
    public ReferenceTextVa getMicromedexOnsetById(Long id) {
        return terminologyDAO.getReferenceById(ReferenceTextVa.MICROMEDEX_ONSET_TYPE, id);
    }

    @Override
    public ReferenceTextVa getMicromedexOnsetByName(String name) {
        return terminologyDAO.getReferenceByName(ReferenceTextVa.MICROMEDEX_ONSET_TYPE, name);
    }

    @Override
    @SuppressWarnings("unchecked")
    public List<ReferenceTextVa> getMicromedexSeverities() {
        return terminologyDAO.getReferenceData(ReferenceTextVa.MICROMEDEX_SEVERITY_TYPE);
    }

    @Override
    public ReferenceTextVa getMicromedexSeverityById(Long id) {
        return terminologyDAO.getReferenceById(ReferenceTextVa.MICROMEDEX_SEVERITY_TYPE, id);
    }

    @Override
    public ReferenceTextVa getMicromedexSeverityByName(String name) {
        return terminologyDAO.getReferenceByName(ReferenceTextVa.MICROMEDEX_SEVERITY_TYPE, name);
    }

    @Override
    @SuppressWarnings("unchecked")
    public List<ReferenceTextVa> getMicromedexSubstantiations() {
        return terminologyDAO.getReferenceData(ReferenceTextVa.MICROMEDEX_SUBSTANTIATION_TYPE);
    }

    @Override
    public ReferenceTextVa getMicromedexSubstantiationById(Long id) {
        return terminologyDAO.getReferenceById(ReferenceTextVa.MICROMEDEX_SUBSTANTIATION_TYPE, id);
    }

    @Override
    public ReferenceTextVa getMicromedexSubstantiationByName(String name) {
        return terminologyDAO.getReferenceByName(ReferenceTextVa.MICROMEDEX_SUBSTANTIATION_TYPE, name);
    }

    @Override
    @SuppressWarnings("unchecked")
    public List<ReferenceTextVa> getPackageInserts() {
        return terminologyDAO.getReferenceData(ReferenceTextVa.PACKAGE_INSERT_TYPE);
    }

    @Override
    public ReferenceTextVa getPackageInsertById(Long id) {
        return terminologyDAO.getReferenceById(ReferenceTextVa.PACKAGE_INSERT_TYPE, id);
    }

    @Override
    public ReferenceTextVa getPackageInsertByName(String name) {
        return terminologyDAO.getReferenceByName(ReferenceTextVa.PACKAGE_INSERT_TYPE, name);
    }

    @Override
    @SuppressWarnings("unchecked")
    public List<ReferenceTextVa> getPBMCriteria() {
        return terminologyDAO.getReferenceData(ReferenceTextVa.PBM_CRITERIA_TYPE);
    }

    @Override
    public ReferenceTextVa getPBMCriteriaById(Long id) {
        return terminologyDAO.getReferenceById(ReferenceTextVa.PBM_CRITERIA_TYPE, id);
    }

    @Override
    public ReferenceTextVa getPBMCriteriaByName(String name) {
        return terminologyDAO.getReferenceByName(ReferenceTextVa.PBM_CRITERIA_TYPE, name);
    }

    @Override
    @SuppressWarnings("unchecked")
    public List<SeverityLevel> getSeverityLevels() {
        return terminologyDAO.getLookups(SeverityLevel.class);
    }

    @Override
    public SeverityLevel getSeverityLevelByCode(String code) {
        return (SeverityLevel) terminologyDAO.getByCode(SeverityLevel.class, code);
    }

    /**
     * Gets the by code.
     * 
     * @param clazz
     *            the clazz
     * @param code
     *            the code
     * @return the by code
     */
    @Override
    @SuppressWarnings("rawtypes")
    public Lookup getByCode(Class clazz, String code) {

        return (Lookup) terminologyDAO.getByCode(clazz, code);
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        if (terminologyDAO == null) {
            throw new Exception("terminologyDAO has not been set");
        }
        // Cache data
        // initializeCache();
    }

    /**
     * Initializes the cache.
     */
    private void initializeCache() {
        if (LOGGER.is   ugEnabled()) {
            LOGGER.debug("TerminologyService is initializing cache");
        }
        getAIDSGuidelines();
        getClinicalEffects();
        getDoseRoutes();
        getDoseTypes();
        getDoseUnitTypeData();
        getDrugInteractionFacts();
        getDrugInteractionFactDocumentation();
        getDrugInteractionFactOnsets();
        getDrugInteractionFactSeverities();
        getDuplicateAllowances();
        getEasySearchSelectTypes();
        getEvaluationOfDrugInteractions();
        getHalfLifeUnits();
        getInteractionSources();
        getInteractionTypes();
        getLevelOfEvidence();
        getMedlineHits();
        getMicromedexOnsets();
        getMicromedexSeverities();
        getMicromedexSubstantiations();
        getPackageInserts();
        getPBMCriteria();
        // getPeriodOfTime();
        getSeverityLevels();
    }

    @SuppressWarnings("unchecked")
    @Override
    public List<EasySearchSelectType> getEasySearchSelectTypes() {
        List<EasySearchSelectType> list = terminologyDAO.getLookups(EasySearchSelectType.class);

        // Add a blank entry if it does not exist.
        if (list.get(0).getCode().length() > 0) {
            EasySearchSelectType blank = new EasySearchSelectType();
            blank.setCode("");
            blank.setName("");
            list.add(0, blank);
        }
        return list;
    }

    @Override
    public EasySearchSelectType getEasySearchSelectTypeByCode(String code) {
        return (EasySearchSelectType) terminologyDAO.getByCode(EasySearchSelectType.class, code);
    }

    @SuppressWarnings("unchecked")
    @Override
    public List<DuplicateAllowance> getDuplicateAllowances() {
        return terminologyDAO.getLookups(DuplicateAllowance.class);
    }

    @Override
    public DuplicateAllowance getDuplicateAllowanceByCode(Long duplicateAllowanceCode) {
        return (DuplicateAllowance) terminologyDAO.getByCode(DuplicateAllowance.class, duplicateAllowanceCode);
    }

    @SuppressWarnings("unchecked")
    @Override
    public List<HalfLifeUnit> getHalfLifeUnits() {
        return terminologyDAO.getLookups(HalfLifeUnit.class);
    }

    @Override
    public HalfLifeUnit getHalfLifeUnitByCode(String halfLifeUnitCode) {
        return (HalfLifeUnit) terminologyDAO.getByCode(HalfLifeUnit.class, halfLifeUnitCode);
    }
}
