package gov.va.med.pharmacy.pecs.state;

import gov.va.med.pharmacy.pecs.domain.Customizable;
import gov.va.med.pharmacy.pecs.domain.impl.va.UserProfile;
import gov.va.med.pharmacy.pecs.persistence.HibernateDAO;
import gov.va.med.pharmacy.pecs.util.jexl.JexlContext;
import gov.va.med.pharmacy.pecs.util.jexl.JexlEvaluator;

import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.scxml.EventDispatcher;
import org.apache.commons.scxml.SCXMLExecutor;
import org.apache.commons.scxml.SCXMLExpressionException;
import org.apache.commons.scxml.TriggerEvent;
import org.apache.commons.scxml.env.SimpleDispatcher;
import org.apache.commons.scxml.env.SimpleErrorReporter;
import org.apache.commons.scxml.model.ModelException;
import org.apache.commons.scxml.model.SCXML;
import org.apache.commons.scxml.model.State;
import org.apache.commons.scxml.model.Transition;

/**
 * Pecs StateMachine Entity.
 * 
 * @author DOMAINOVERSD
 * 
 */
public class PecsStateMachine {

    /** The engine. */
    private final SCXMLExecutor engine;

    /** The factory. */
    private final PecsStateMachineFactory factory;

    /**
     * PecsStateMachine.
     * 
     * @param stateMachine
     *            -SCXML
     * @param factory
     *            - PecsStateMachineFactory
     */
    PecsStateMachine(SCXML stateMachine, /* String initialState, */PecsStateMachineFactory factory) {
        engine = new SCXMLExecutor(new JexlEvaluator(), new SimpleDispatcher(), new SimpleErrorReporter());
        engine.setStateMachine(stateMachine);
        engine.setSuperStep(true);
        engine.setRootContext(new JexlContext());

        try {
            engine.go();
        } catch (ModelException e) {
            throw new IllegalArgumentException("failed to create state machine instance", e);
        }

        this.factory = factory;
    }

    /**
     * Sets the initial state, so execution can pick up where it left off.
     * 
     * @param exec
     *            -SCXMLExecutor
     * @param initialState
     *            the initial state
     */
    @SuppressWarnings("unchecked")
    private void setInitialState(SCXMLExecutor exec, String initialState) {
        if (initialState != null) {
            Map<String, State> allStates = exec.getStateMachine().getTargets();

            State state = allStates.get(initialState);
            if (state == null) {
                throw new IllegalStateException("Unknown state " + initialState);
            }

            @SuppressWarnings("rawtypes")
            Set states = exec.getCurrentStatus().getStates();
            states.clear();
            states.add(state);
        }
    }

    /**
     * Execute a state transition for the given action. If no transition fires for this event (ignoring conditions), this will
     * is considered an error (attempt to perform an invalid action).
     * 
     * @param action
     *            The action name.
     * @return the id of the new current state of the machine.
     * @throws ModelException
     *             the model exception
     * @throws SCXMLExpressionException
     *             the sCXML expression exception
     */
    public/* Status */String applyAction(final String action) throws ModelException, SCXMLExpressionException {
        boolean legalTransition = false;
        for (Transition t : getPossibleTransitions()) {
            if (t.getEvent().equals(action)) {
                legalTransition = true;
                break;
            }
        }
        if (!legalTransition) {
            String msg = "Illegal action " + action + " for status " + getCurrentStatus()/*
                                                                                          * . getDisplayName ( )
                                                                                          */;
            throw new IllegalArgumentException(msg);
        }
        TriggerEvent[] evts = { new TriggerEvent(action, TriggerEvent.SIGNAL_EVENT, null) };
        engine.triggerEvents(evts);
        return getCurrentStatus();
    }

    /**
     * Status that Gets Current Status.
     * 
     * @return s -String object
     * @throws SCXMLExpressionException
     *             the sCXML expression exception
     */
    public/* Status */String getCurrentStatus() throws SCXMLExpressionException {
        String s = getStateInternal().getId();
        // return statuses.get(s);
        return s;
    }

    /**
     * Get State Internal.
     * 
     * @return (State) Get Status
     */
    private State getStateInternal() {
        return (State) engine.getCurrentStatus().getStates().iterator().next();
    }

    /**
     * List get Possible Transitions.
     * 
     * @return Transitions List
     */
    @SuppressWarnings("unchecked")
    private List<Transition> getPossibleTransitions() {
        State crt = getStateInternal();
        return crt.getTransitionsList();
    }

    /**
     * The set of actions that would cause a transition to fire, given the state machine's current state.
     * 
     * @return the set of actions
     * @throws SCXMLExpressionException
     *             the sCXML expression exception
     */
    public Set<String> getAllowableActions() throws SCXMLExpressionException {
        HashSet<String> res = new HashSet<String>();
        for (Transition t : getPossibleTransitions()) {
            String cond = t.getCond();
            if (cond == null || engine.getEvaluator().evalCond(engine.getRootContext(), cond)) {
                res.add(t.getEvent());
            }
        }
        return res;
    }

    /**
     * Set the object on which this state machine operates.
     * 
     * @param concept
     *            the object whose state is managed by this state machine
     */
    public void setConcept(Customizable<?> concept) {
        setInitialState(engine, concept.getStatus().getCode());
        engine.getRootContext().set("concept", concept);
        engine.getRootContext().set("statusFactory", factory);
    }

    /**
     * Set the user who performs the action.
     * 
     * @param user
     *            the user processing the object
     */
    public void setUser(UserProfile user) {
        engine.getRootContext().set("crtUser", user);
    }

    /**
     * Configure a DAO that can be used for operations involving the persistent store.
     * 
     * @param dao
     *            the DAO instance
     */
    public void setDAO(HibernateDAO<?> dao) {
        engine.getRootContext().set("dao", dao);
    }

    /**
     * SimpleEvent Dispatcher Entity.
     * 
     * @author DOMAINOVERSD
     * 
     */
    class SimpleEventDispatcher implements EventDispatcher {

        /*
         * (non-Javadoc)
         * 
         * @see org.apache.commons.scxml.EventDispatcher#cancel(java.lang.String)
         */
        @Override
        public void cancel(String sendId) {
            // nothing
        }

        /*
         * (non-Javadoc)
         * 
         * @see org.apache.commons.scxml.EventDispatcher#send(java.lang.String, java.lang.String, java.lang.String,
         * java.lang.String, java.util.Map, java.lang.Object, long, java.util.List)
         */
        @Override
        public void send(String sendId, String target, String targetType, String event, Map params, Object hints, long delay,
                List externalNodes) {
            // try {
            // engine.getEvaluator().eval(engine.getRootContext(), "");
            // } catch (SCXMLExpressionException e) {
            // // TODO Auto-generated catch block
            // e.printStackTrace();
            // }
        }

    }
}
