package gov.va.med.pharmacy.pecs.util.config;

import java.util.Map.Entry;
import java.util.Properties;
import java.util.Set;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.BeanInitializationException;
import org.springframework.beans.factory.config.ConfigurableListableBeanFactory;
import org.springframework.beans.factory.config.PropertyPlaceholderConfigurer;

/**
 * Recursive Property Place Holder Configure Entity.
 * 
 * @author Madhu Katikala (@user) A property resource configurer that resolves placeholders in bean property values of context
 *         definitions. It <i>pulls</i> values from a properties file into bean definitions. In addition to porocessing beans in
 *         the current factory, it recursivley processes the beans in the parent factories as well.
 */
public final class RecursivePropertyPlaceholderConfigurer extends PropertyPlaceholderConfigurer {

    private static Logger logger = Logger.getLogger(RecursivePropertyPlaceholderConfigurer.class);

    private static String encryptedIdentifier = "ENC";

    /**
     * Recursive Property Place Holder. Invokes parent's default constructor
     */
    public RecursivePropertyPlaceholderConfigurer() {
        super();
    }

    /**
     * Recursively process the beans in the current and all its parent factories.
     * 
     * @param beanFactoryToProcess
     *            - ConfigurableListableBeanFactory
     * @param props
     *            - Properties
     */
    @Override
    protected void processProperties(ConfigurableListableBeanFactory beanFactoryToProcess, Properties props) {
        ConfigurableListableBeanFactory currentFactory = beanFactoryToProcess;
        while (currentFactory != null) {
            super.processProperties(currentFactory, decryptProperties(props));
            currentFactory = (ConfigurableListableBeanFactory) currentFactory.getParentBeanFactory();
        }
    }

    /**
     * Loops through the supplied properties looking for any values that begin with the encryptedIdentifier. For each property
     * found, it calls the decryption method and stores the decrypted value back in the Properties.
     * 
     * @param props
     *            - Properties
     * @return props
     */
    @SuppressWarnings({ "unchecked", "rawtypes" })
    private Properties decryptProperties(Properties props) {
        Set<Entry<Object, Object>> properties = props.entrySet();
        for (Entry entry : properties) {
            String value = (String) entry.getValue();
            if (value.startsWith(encryptedIdentifier)) {
                //@formatter:off
                String encryptedValue = value.substring(4, value.length());
                //@formatter:on
                try {
                    entry.setValue(PECSEncrypter.decrypt(encryptedValue));
                } catch (Exception e) {
                    final String message = "Error trying to decrypt value for key " + entry.getKey();
                    logger.error(message, e);
                    throw new BeanInitializationException(message, e);
                }
            }
        }
        return props;
    }

}
