package gov.va.med.pharmacy.pecs.service;

import gov.va.med.pharmacy.pecs.domain.impl.fdb.RoutedGenericFdb;
import gov.va.med.pharmacy.pecs.domain.impl.fdb.RoutedGenericFdbLite;
import gov.va.med.pharmacy.pecs.domain.impl.va.UserProfile;
import gov.va.med.pharmacy.pecs.dto.DrugPairDTO;

import java.util.List;
import java.util.Map;

import org.springframework.validation.BindingResult;

/**
 * The drug pair service.
 * 
 * @author VHAISACARUSR
 * 
 */
public interface DrugPairService extends PecsService<DrugPairDTO> {
    /**
     * Gets the custom drug pair for the ID.
     * 
     * @param id
     *            the custom drug pair ID
     * @return a drug pair
     */
    DrugPairDTO getCustomModel(long id);

    /**
     * Gets a list of routed generic drugs.
     * 
     * @return a list of routed generic drugs
     */
    List<RoutedGenericFdb> getRoutedGenericDrugs();

    /**
     * Creates a new record for each item in the list.
     * 
     * @param records
     *            the list of records to be created.
     * @param err
     *            holds the errors if there are any
     */
    void createNewRecords(List<DrugPairDTO> records, BindingResult err);

    /**
     * Returns a list of all the rejected or deleted drug pairs of an FDB DDI excluding the passed drug pair.
     * 
     * @param routedGeneric1
     *            routed generic 1 ID
     * @param routedGeneric2
     *            routed generic 2 ID
     * 
     * @param fdbId
     *            the FDB interaction ID
     * 
     * @return pairs
     */
    List<DrugPairDTO> findDrugPairsForFDBInteraction(Long routedGeneric1, Long routedGeneric2, Long fdbId);

    /**
     * Looks up the drug pair description.
     * 
     * @param rtgenId1
     *            routed generic 1 ID
     * 
     * @param rtgenId2
     *            routed generic 2 ID
     * @param interactionId
     *            the FDB interaction ID
     * @return a string array with the description
     */
    String[] lookupDpDesc(long rtgenId1, long rtgenId2, long interactionId);

    /**
     * Gets the current drug pair.
     * 
     * @param rtgenId1
     *            routed generic 1 ID
     * @param rtgenId2
     *            routed generic 2 ID
     * @param interactionId
     *            the VA custom interaction ID
     * @return the drug pair
     */
    DrugPairDTO getCurrentDrugPair(long rtgenId1, long rtgenId2, long interactionId);

    /**
     * Apply the given action to a list of records.
     * 
     * @param action
     *            the action to be performed
     * @param records
     *            the drug pairs for the action to be performed on
     * @param err
     *            holds the errors if there is one
     */
    void applyAction(String action, List<DrugPairDTO> records, BindingResult err);

    /**
     * Find all 'visible' drug pairs (not deleted or rejected) for the given interaction. Also annotate the DPs with the
     * allowable actions for the give user.
     * 
     * @param interactionId
     *            the VA custom interaction ID
     * @param profile
     *            the user profile
     * @return a list of drug pairs
     */
    List<DrugPairDTO> getDrugPairsForInteraction(long interactionId, UserProfile profile);

    /**
     * Find all statuses and counts of drug pairs associated with the interaction having the given id.
     * 
     * @param interactionId 
     *            the VA custom interaction ID
     * @return <code>{@literal Map<String, long>}</code> where the key is the status name and the value is 
     * the count of drug pairs with that status.
     */
    public Map<String, Long> getDrugPairsStatusCountsByInteractionId(long interactionId);
    
    /**
     * Find all drug pairs (regardless of status) for the given interaction.
     * 
     * @param interactionId
     *            the VA custom interaction ID
     * @return a list of drug pairs
     */
    List<DrugPairDTO> getAllDrugPairsForInteraction(long interactionId);

    /**
     * Gets the current drug pair for the FDB id provided.
     * 
     * @param rtgenId1
     *            routed generic 1 ID
     * 
     * @param rtgenId2
     *            routed generic 2 ID
     * @param fdbId
     *            the FDB ID
     * @return a drug pair
     */
    DrugPairDTO getCurrentDrugPairByFdb(long rtgenId1, long rtgenId2, long fdbId);

    /**
     * Gets the FDB data from the database and returns it as a DTO.
     * 
     * @param interactionId
     *            the FDB interaction ID the FDB interaction ID
     * @param rtgenId1
     *            routed generic 1 ID
     * @param rtgenId2
     *            routed generic 2 ID
     * @return a drug pair
     */
    DrugPairDTO getFdbData(Long interactionId, Long rtgenId1, Long rtgenId2);

    /**
     * Gets the FDB drug pairs for the interaction provided.
     * 
     * @param interactionId
     *            the FDB interaction ID
     * @return a list of drug pairs
     */
    List<DrugPairDTO> getFdbData(Long interactionId);

    /**
     * Filter out drug pairs associated with interactions. Only drug pairs that can be added to this interaction will be
     * returned
     * 
     * @param fdbId
     *            the FDB interaction id
     * @return a list of drug pairs. never <code>null</code>
     */
    List<DrugPairDTO> getFilteredFdbData(Long fdbId);

    /**
     * Gets the custom drug pairs for the custom interaction ID.
     * 
     * @param interactionId
     *            the custom interaction ID
     * @return the custom drug pairs
     */
    List<DrugPairDTO> getStagingData(Long interactionId);

    /**
     * Retrieves the interaction ID's for drug pairs that need action on approved interactions.
     * 
     * @param userName
     *            name of the logged in user
     * @param status
     *            status of the drug pairs. More than one status will be used in an in clause.
     * @param actionUser
     *            user who applied the action. Can be <code>null</code>
     * @return a list of <code>Long</code>'s. Never <code>null</code>
     */
    List<Long> getInteractionsWithWork(String userName, List<String> status, String actionUser);

    /**
     * Checks to see if the drug pairs exist on another interaction.
     * 
     * @param fdbId
     *            the FDB id for the interaction;
     * @return true if there is an interaction with drug pairs.
     */
    boolean hasFdbDrugPairsOnOtherInteraction(long fdbId);

    /**
     * Gets the va drug pairs by routed generic.
     * 
     * @param routedGenId
     *            the routed gen id
     * @return the va drug pairsby routed generic
     */
    List<DrugPairDTO> getVaDrugPairsByRoutedGenericId(long routedGenId);

    /**
     * Gets the fDB drug pairs by routed generic.
     * 
     * @param routedGenId
     *            the routed gen id
     * @return the fDB drug pairs by routed generic
     */
    List<DrugPairDTO> getFDBDrugPairsByRoutedGenericId(long routedGenId);

    /**
     * Gets a list of routed generic drugs containing only their lite attributes.
     * 
     * @return a list of routed generic drugs
     */   
    List<RoutedGenericFdbLite> getRoutedGenericDrugsLite();
}
