package gov.va.med.pharmacy.pecs.service;

import gov.va.med.pharmacy.pecs.criteria.ConceptHistoryCriteria;
import gov.va.med.pharmacy.pecs.domain.UserComment;
import gov.va.med.pharmacy.pecs.domain.impl.va.UserProfile;
import gov.va.med.pharmacy.pecs.dto.PecsDTO;
import gov.va.med.pharmacy.pecs.metadata.ConceptMetadata;
import gov.va.med.pharmacy.pecs.persistence.HibernateDAO;
import gov.va.med.pharmacy.pecs.query.ConceptQuery;
import gov.va.med.pharmacy.pecs.query.Page;
import gov.va.med.pharmacy.pecs.service.dto.FieldHistoryLineItem;

import java.io.File;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.springframework.validation.BindingResult;

/**
 * The interface that the services dealing with various concepts need to implement.
 * 
 * @param <T>
 *            the class of the DTO used to exchange information between the presentation and service tiers
 */
public interface PecsService<T extends PecsDTO> {

    /**
     * Returns the staging data for the query.
     * 
     * @param query
     *            the query to run
     * @return a list of results
     */
    List<T> getStagingData(ConceptQuery query);

    /**
     * * Returns the staging data for the query in pages for the UI.
     * 
     * @param query
     *            the query to run
     * @param page
     *            the page to be returned
     * @return a page object holding information about the page.
     */
    Page getStagingData(ConceptQuery query, int page);

    /**
     * * Returns the staging data for the query in pages for the UI.
     * 
     * @param query
     *            the query to run in the form of a string
     * @param page
     *            the page to be returned
     * @return a page object holding information about the page.
     */
    Page getStagingData(String query, int page);

    /**
     * Returns the FDB data returned by the query.
     * 
     * @param query
     *            the query to run
     * @return a list of results
     */
    List<T> getFdbData(ConceptQuery query);

    /**
     * * Returns the FDB data for the query in pages for the UI.
     * 
     * @param query
     *            the query to run
     * @param page
     *            the page to be returned
     * @return a page object holding information about the page.
     */
    Page getFdbData(ConceptQuery query, int page);

    /**
     * * Returns the FDB data for the query in pages for the UI.
     * 
     * @param query
     *            the query to run in the form of a string
     * @param page
     *            the page to be returned
     * @return a page object holding information about the page.
     */
    Page getFdbData(String query, int page);

    /**
     * Creates a new record.
     * 
     * @param record
     *            the record to create
     * @param err
     *            errors to be populated if one or more occur
     * @return the newly created concept object
     */
    T createNewRecord(T record, BindingResult err);

    /**
     * Applies an action to change the state of a concept.
     * 
     * @param action
     *            the action to perform
     * @param record
     *            the record to perform the action on
     * @param err
     *            an object to store errors
     * @return the new concept object
     */
    T applyAction(String action, T record, BindingResult err);

    /**
     * Retrieves the actions that are able to be performed on an object.
     * 
     * @param record
     *            the record to check
     * @param user
     *            the user performing the action
     * @return the list of actions that can be performed
     */
    Set<String> getAllowableActions(T record, UserProfile user);

    /**
     * I believe that this takes a json string from the user interface and creates a Excel file out of the results. Someone
     * should verify this.
     * 
     * @param json
     *            the query as a json stringl
     * @param filePath
     *            the path to the file that is created
     * @param fileName
     *            the name of the file to be created
     * @param custom
     *            true if this query is for custom records
     * @return a file
     */
    File getDelimitedFile(String json, String filePath, String fileName, boolean custom);

    /**
     * Aggregate counts for customization requests satisfying various conditions.
     * 
     * @param userName
     *            the user name on behalf of which the counts are determined
     * @return a map, in which the values are the counts and the keys are strings describing what the counts represent
     */
    Map<String, Integer> getStagingDataCounts(String userName);

    /**
     * The metadata describing the concept this service handles.
     * 
     * @return the metadata
     */
    ConceptMetadata getMetadata();

    /**
     * returns the dao for the concept.
     * 
     * @param custom
     *            true if you want the custom DAO
     * @return a instance of the DAO
     */
    HibernateDAO<?> getDao(boolean custom);

    /**
     * Retrieves the history for all required objects. ConceptHistoryCriteria.id is expected to be populated for all concepts
     * except Dose Range which would expect ConceptHistoryCriteria.versionId to be populated.
     * 
     * @param criteria
     *            The ConceptHistoryCriteria
     * @return a map keyed by field name and holding a list of FieldHistoryLineItem. Never <code>null</code>
     */
    Map<String, List<FieldHistoryLineItem>> getRequiredHistory(ConceptHistoryCriteria criteria);

    /**
     * Creates the user comment.
     * 
     * @param fdbId
     *            the FDB id the comment should be associated with
     * @param comment
     *            the comment
     * @param bindingResult
     *            the binding results
     */
    void createUserComment(Object fdbId, UserComment comment, BindingResult bindingResult);

}
