package gov.va.med.pharmacy.pecs.service.easy;

import gov.va.med.pharmacy.pecs.domain.impl.easy.DoseCheck;
import gov.va.med.pharmacy.pecs.domain.impl.easy.DosingInfo;
import gov.va.med.pharmacy.pecs.domain.impl.easy.DrugCheckResult;
import gov.va.med.pharmacy.pecs.domain.impl.easy.DrugDrugInteractionInfo;
import gov.va.med.pharmacy.pecs.domain.impl.easy.DrugIdentifier;
import gov.va.med.pharmacy.pecs.domain.impl.easy.DuplicateTherapyInfo;
import gov.va.med.pharmacy.pecs.domain.impl.easy.PecsDrugInfo;
import gov.va.med.pharmacy.pecs.domain.impl.lookup.DoseRoute;
import gov.va.med.pharmacy.pecs.domain.impl.lookup.DoseType;
import gov.va.med.pharmacy.pecs.domain.impl.lookup.DoseUnitType;
import gov.va.med.pharmacy.pecs.domain.impl.lookup.SeverityLevel;
import gov.va.med.pharmacy.pecs.dto.DrugPairDTO;

import java.util.Collection;
import java.util.List;

/**
 * An interface for the Easy Search Service.
 * 
 * @author vhaisacarusr
 * 
 */
public interface EasySearchService {
    /**
     * Searches the FDB database for a drug that matches the text passed in.
     * 
     * @param text
     *            the string to match
     * @return a list of drugs that match the text passed in
     */
    Collection<PecsDrugInfo> searchDrugs(String text);

    /**
     * Gets the Check Orders for duplicate therapy.
     * 
     * @param ids
     *            the drug identifiers
     * @return a list of <code>DuplicateTherapyInfo</code> objects
     */
    DrugCheckResult<DuplicateTherapyInfo> checkDuplicateTherapy(Collection<DrugIdentifier> ids);

    /**
     * Check drug drug interaction.
     * 
     * @param ids
     *            the ids
     * @return the drug check result
     */
    DrugCheckResult<DrugDrugInteractionInfo> checkDrugDrugInteraction(Collection<DrugIdentifier> ids);

    /**
     * Check dosing.
     * 
     * @param doseCheck
     *            the dose check
     * @return the drug check result
     */
    DrugCheckResult<DosingInfo> checkDosing(DoseCheck doseCheck);

    /**
     * Gets the drug.
     * 
     * @param id
     *            the id
     * @return the drug
     */
    PecsDrugInfo getDrug(DrugIdentifier id);

    /**
     * Gets the vista severity levels.
     * 
     * @return the vista severity levels
     */
    List<SeverityLevel> getVistaSeverityLevels();

    /**
     * Gets the dose routes.
     * 
     * @param id
     *            the id
     * @return the dose routes
     */
    List<DoseRoute> getDoseRoutes(DrugIdentifier id);

    /**
     * Gets the dose types.
     * 
     * @param id
     *            the id
     * @return the dose types
     */
    List<DoseType> getDoseTypes(DrugIdentifier id);

    /**
     * Gets the dose units.
     * 
     * @param id
     *            the id
     * @return the dose units
     */
    List<DoseUnitType> getDoseUnits(DrugIdentifier id);

    /**
     * Generate reports.
     * 
     * @param severityLevel
     *            the severity level
     * @param id
     *            the id
     * @return the list
     * @throws Exception
     *             the exception
     */
    List<DrugPairDTO> generateReports(DrugIdentifier id, String severityLevel) throws Exception;

}
