package gov.va.med.pharmacy.pecs.service.easy.impl;

import firstdatabank.database.FDBDataManager;
import firstdatabank.database.FDBException;
import firstdatabank.database.FDBSQLException;
import firstdatabank.dif.DRCLookupResult;
import firstdatabank.dif.DRCLookupResults;
import firstdatabank.dif.DTClass;
import firstdatabank.dif.DTClasses;
import firstdatabank.dif.DispensableDrug;
import firstdatabank.dif.DispensableDrugs;
import firstdatabank.dif.DispensableGeneric;
import firstdatabank.dif.DispensableGenerics;
import firstdatabank.dif.DrugSearchFilter;
import firstdatabank.dif.FDBDispensableDrugLoadType;
import firstdatabank.dif.FDBDosingSource;
import firstdatabank.dif.FDBNotAvailableException;
import firstdatabank.dif.FDBPhoneticSearch;
import firstdatabank.dif.FDBSearchMethod;
import firstdatabank.dif.FDBUnknownIDException;
import firstdatabank.dif.FilterCodes;
import firstdatabank.dif.Navigation;
import firstdatabank.dif.SearchFilter;
import gov.va.med.pharmacy.pecs.domain.impl.easy.DoseCheck;
import gov.va.med.pharmacy.pecs.domain.impl.easy.DosingInfo;
import gov.va.med.pharmacy.pecs.domain.impl.easy.DrugCheckResult;
import gov.va.med.pharmacy.pecs.domain.impl.easy.DrugDrugInteractionInfo;
import gov.va.med.pharmacy.pecs.domain.impl.easy.DrugIdentifier;
import gov.va.med.pharmacy.pecs.domain.impl.easy.DrugIdentifierType;
import gov.va.med.pharmacy.pecs.domain.impl.easy.DuplicateTherapyInfo;
import gov.va.med.pharmacy.pecs.domain.impl.easy.PecsDrugInfo;
import gov.va.med.pharmacy.pecs.domain.impl.easy.ProfessionalMonograph;
import gov.va.med.pharmacy.pecs.domain.impl.easy.TherapeuticClass;
import gov.va.med.pharmacy.pecs.domain.impl.lookup.DoseRoute;
import gov.va.med.pharmacy.pecs.domain.impl.lookup.DoseType;
import gov.va.med.pharmacy.pecs.domain.impl.lookup.DoseUnitType;
import gov.va.med.pharmacy.pecs.domain.impl.lookup.SeverityLevel;
import gov.va.med.pharmacy.pecs.dto.DrugPairDTO;
import gov.va.med.pharmacy.pecs.exception.CustomizationServiceException;
import gov.va.med.pharmacy.pecs.persistence.fdb.DrugConceptDao;
import gov.va.med.pharmacy.pecs.service.DrugPairService;
import gov.va.med.pharmacy.pecs.service.TerminologyService;
import gov.va.med.pharmacy.pecs.service.easy.EasySearchService;
import gov.va.med.pharmacy.peps.common.vo.DrugCheckVo;
import gov.va.med.pharmacy.peps.common.vo.DrugDoseCheckResultVo;
import gov.va.med.pharmacy.peps.common.vo.DrugDrugCheckResultVo;
import gov.va.med.pharmacy.peps.common.vo.DrugTherapyCheckResultVo;
import gov.va.med.pharmacy.peps.common.vo.OrderCheckResultsVo;
import gov.va.med.pharmacy.peps.common.vo.OrderCheckVo;
import gov.va.med.pharmacy.peps.common.vo.ProfessionalMonographVo;
import gov.va.med.pharmacy.peps.external.common.drugdatavendor.capability.PerformDrugChecksCapability;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.TreeSet;

import javax.annotation.Resource;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * Implementation for the Easy Search Service.
 * 
 */
@Service
public final class EasySearchServiceImpl implements EasySearchService, InitializingBean {

    /** The Constant LOGGER. */
    private static final Logger LOGGER = Logger.getLogger(EasySearchServiceImpl.class);

    /** The Constant VISTA_SEVERITY_LEVEL_CODES. */
    private static final List<String> VISTA_SEVERITY_LEVEL_CODES = Arrays.asList(new String[] { "1", "2" });

    /** The drug check capability. */
    @Resource
    private PerformDrugChecksCapability drugCheckCapability;

    /** The fdb navigation. */
    @Resource
    private Navigation fdbNavigation;

    /** The fdb data manager. */
    @Resource
    private FDBDataManager fdbDataManager;

    /** The terminology service. */
    @Resource
    private TerminologyService terminologyService;

    /** The vista severity levels. */
    private final ArrayList<SeverityLevel> vistaSeverityLevels = new ArrayList<SeverityLevel>();

    /** The unit types map. */
    private Map<String, DoseUnitType> unitTypesMap;

    /** The drug concept dao. */
    @Resource
    private DrugConceptDao drugConceptDao;

    /** The drug pair service. */
    @Resource
    private DrugPairService drugPairService;

    /** The Constant DOSE_UNIT_DELIMITER. */
    private static final char DOSE_UNIT_DELIMITER = '/';

    /** The Constant DOSE_UNIT_ALT_DELIMITER. */
    private static final String DOSE_UNIT_ALT_DELIMITER = " per ";

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.service.easy.EasySearchService#generateReports(gov.va.med.pharmacy.pecs.domain.impl.easy.
     * DrugIdentifier, java.lang.String)
     */
    @Override
    @Transactional(readOnly = true)
    public List<DrugPairDTO> generateReports(DrugIdentifier id, String severityLevelIndicator) throws Exception {

        Long routedGenId = getRoutedGenericIdByType(id);

        List<DrugPairDTO> vaDrugPairs = drugPairService.getVaDrugPairsByRoutedGenericId(routedGenId);

        List<DrugPairDTO> fdbDrugPairs = drugPairService.getFDBDrugPairsByRoutedGenericId(routedGenId);

        List<DrugPairDTO> combinedDrugPairList = combineDrugPairLists(vaDrugPairs, fdbDrugPairs);
        List<DrugPairDTO> combinedDrugPairListFiltered = filterDataBySeverity(combinedDrugPairList, severityLevelIndicator);

        sortListByDesc(combinedDrugPairListFiltered);

        return combinedDrugPairListFiltered;
    }

    /**
     * Gets the routed generic id by type.
     * 
     * @param id
     *            the id
     * @return the routed generic id by type
     */
    private Long getRoutedGenericIdByType(DrugIdentifier id) {
        Long routedGenId;
        Long newId = Long.valueOf(id.getValue().toString());
        if (id.getType().equals(DrugIdentifierType.fromString("MED_ID"))) {

            routedGenId = drugConceptDao.getRoutedGenericFromMEDID(newId);
        } else {
            routedGenId = drugConceptDao.getRoutedGenericFromGCNID(newId);
        }
        return routedGenId;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.service.easy.EasySearchService#searchDrugs(java.lang.String)
     */
    @Override
    public Collection<PecsDrugInfo> searchDrugs(String text) {

        SearchFilter searchFilter = new SearchFilter();
        DrugSearchFilter drugSearchFilter = new DrugSearchFilter();

        drugSearchFilter.setIncludeDevices(false);
        drugSearchFilter.setIncludePrivateLabelers(true);
        drugSearchFilter.setIncludeRepackagers(true);

        FDBSearchMethod method = FDBSearchMethod.fdbSMContainsExhaustive;
        searchFilter.setSearchMethod(method);
        searchFilter.setPhoneticSearch(FDBPhoneticSearch.fdbPSNoPhonetic);

        // Do we really need to include all these (retired, replaced)?
        FilterCodes fc = new FilterCodes();
        fc.addItem("0"); // active
        fc.addItem("1"); // replaced
        fc.addItem("2"); // retired
        fc.addItem("3"); // inactive
        fc.addItem("9"); // unassociated
        drugSearchFilter.setIncludeStatusCodes(fc);

        try {
            DispensableDrugs searchResult = getFdbNavigation().dispensableDrugSearch(text, searchFilter, drugSearchFilter);
            DispensableGenerics dispensableGenerics =
                    getFdbNavigation().dispensableGenericDrugSearch(text, searchFilter, drugSearchFilter);
            Map<Long, PecsDrugInfo> map = new HashMap<Long, PecsDrugInfo>();
            convertDispensablesToPecsDrugInfos(map, searchResult);
            convertDispensableGenericsToPecsDrugInfos(map, dispensableGenerics);

            ArrayList<PecsDrugInfo> drugInfos = new ArrayList<PecsDrugInfo>(map.values());
            sortPecsDrugInfoByDesc(drugInfos);
            return drugInfos;
        } catch (FDBSQLException e) {
            throw new CustomizationServiceException("Error searching drugs for " + text, e);
        }
    }

    /**
     * Sort pecs drug info by desc.
     * 
     * @param drugInfos
     *            list of <code>PecsDrugInfo</code>'s
     */
    private void sortPecsDrugInfoByDesc(List<PecsDrugInfo> drugInfos) {
        Collections.sort(drugInfos, new Comparator<PecsDrugInfo>() {
            @Override
            public int compare(PecsDrugInfo o1, PecsDrugInfo o2) {
                return o1.getDescription().compareToIgnoreCase(o2.getDescription());
            }
        });
    }

    /**
     * Convert dispensables to pecs drug infos.
     * 
     * @param map
     *            a map of <code>PecsDrugInfo</code>'s keyed by <code>DrugIdentifier</code>
     * @param dispensableDrugs
     *            the drugs to convert
     */
    private void convertDispensablesToPecsDrugInfos(Map<Long, PecsDrugInfo> map, DispensableDrugs dispensableDrugs) {

        for (int i = 0; i < dispensableDrugs.count(); i++) {
            DispensableDrug dd = dispensableDrugs.item(i);
            // if there is no GCN Sequence we can't look up a routed generic so there is no point in adding it to the list.
            if (dd.getGCNSeqNo() != 0) {
                PecsDrugInfo dto = new PecsDrugInfo();
                dto.setDescription(dd.getDescription());
                dto.setId(new DrugIdentifier(dd.getID(), DrugIdentifierType.MED_ID));
                dto.addAlternateId(new DrugIdentifier(dd.getGCNSeqNo(), DrugIdentifierType.GCN_SEQNO));
                map.put(dd.getGCNSeqNo(), dto);
            }
        }
    }

    /**
     * Convert dispensable generics to pecs drug infos.
     * 
     * @param map
     *            a map of <code>PecsDrugInfo</code>'s keyed by <code>DrugIdentifier</code>
     * @param genericDrugs
     *            the drugs to convert
     */
    private void convertDispensableGenericsToPecsDrugInfos(Map<Long, PecsDrugInfo> map, DispensableGenerics genericDrugs) {

        for (int i = 0; i < genericDrugs.count(); i++) {
            DispensableGeneric dg = genericDrugs.item(i);
            PecsDrugInfo dto = new PecsDrugInfo();
            dto.setDescription(dg.getDescription());
            dto.setId(new DrugIdentifier(dg.getID(), DrugIdentifierType.GCN_SEQNO));

            map.put(dg.getID(), dto);
        }
    }

    /**
     * Gets the order check vo.
     * 
     * @param dtos
     *            the drug information
     * @return an order check
     */
    private OrderCheckVo getOrderCheckVo(Collection<PecsDrugInfo> dtos) {
        OrderCheckVo orderCheck = new OrderCheckVo();
        orderCheck.setProspectiveOnly(false);
        orderCheck.setCustomTables(true);
        Collection<DrugCheckVo> drugsToScreen = new ArrayList<DrugCheckVo>();
        for (PecsDrugInfo dto : dtos) {
            DrugCheckVo drug = new DrugCheckVo();
            drug.setGcnSeqNo(dto.getId(DrugIdentifierType.GCN_SEQNO).getValueAsString());
            drug.setDrugName(dto.getDescription());
            drug.setProspective(true);
            drugsToScreen.add(drug);
        }
        orderCheck.setDrugsToScreen(drugsToScreen);

        return orderCheck;
    }

    /**
     * Collect drugs.
     * 
     * @param ids
     *            the drug identifiers
     * @return a map of drug info keyed by strings
     */
    private Map<String, PecsDrugInfo> collectDrugs(Collection<DrugIdentifier> ids) {
        HashMap<String, PecsDrugInfo> inputDrugs = new HashMap<String, PecsDrugInfo>();
        for (DrugIdentifier id : ids) {
            try {
                PecsDrugInfo dto = getDrug(id);
                inputDrugs.put(dto.getDescription(), dto);
            } catch (Exception e) {
                LOGGER.error("Unable to check " + id, e);
            }
        }
        return inputDrugs;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.service.easy.EasySearchService#checkDuplicateTherapy(java.util.Collection)
     */
    @Override
    public DrugCheckResult<DuplicateTherapyInfo> checkDuplicateTherapy(Collection<DrugIdentifier> ids) {
        Map<String, PecsDrugInfo> map = collectDrugs(ids);
        OrderCheckVo orderCheck = getOrderCheckVo(map.values());
        orderCheck.setDrugTherapyCheck(true);
        orderCheck.setDuplicateAllowance(true);

        DrugCheckResult<DuplicateTherapyInfo> res = new DrugCheckResult<DuplicateTherapyInfo>();
        OrderCheckResultsVo response = getDrugCheckCapability().performDrugChecks(orderCheck);
        if (response.getDrugTherapyCheckResults().getChecks().isEmpty()) {
            return res;
        }

        if (LOGGER.is   ugEnabled()) {
            LOGGER.debug("Duplicate therapy checks for " + ids);
        }
        for (DrugTherapyCheckResultVo vo : response.getDrugTherapyCheckResults().getChecks()) {
            DuplicateTherapyInfo info = new DuplicateTherapyInfo();
            TherapeuticClass dtClass = new TherapeuticClass();
            dtClass.setId(vo.getId());
            dtClass.setDescription(vo.getDuplicateClass());
            info.setDtClass(dtClass);
            info.setAllowance(vo.getAllowance());
            info.setCustom(vo.isCustom());
            info.setShortText(vo.getShortText());
            if (LOGGER.is   ugEnabled()) {
                LOGGER.debug("Duplicate therapy class " + dtClass.getId() + " (" + dtClass.getDescription() + ")");
            }
            for (DrugCheckVo d : vo.getDrugs()) {
                // info.addDrug(getDrug(new DrugIdentifier(d.getGcnSeqNo(), DrugIdentifierType.GCN_SEQNO)));
                info.addDrug(map.get(d.getDrugName()));
            }
            res.addChecks(info);
        }
        return res;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.service.easy.EasySearchService#checkDrugDrugInteraction(java.util.Collection)
     */
    @Override
    public DrugCheckResult<DrugDrugInteractionInfo> checkDrugDrugInteraction(Collection<DrugIdentifier> ids) {
        Map<String, PecsDrugInfo> map = collectDrugs(ids);
        OrderCheckVo orderCheck = getOrderCheckVo(map.values());
        orderCheck.setDrugDrugCheck(true);

        DrugCheckResult<DrugDrugInteractionInfo> res = new DrugCheckResult<DrugDrugInteractionInfo>();
        OrderCheckResultsVo response = getDrugCheckCapability().performDrugChecks(orderCheck);
        if (response.getDrugDrugCheckResults().getChecks().isEmpty()) {
            return res;
        }

        if (LOGGER.is   ugEnabled()) {
            LOGGER.debug("Drug-drug interaction checks for " + ids);
        }
        for (DrugDrugCheckResultVo vo : response.getDrugDrugCheckResults().getChecks()) {
            DrugDrugInteractionInfo info = new DrugDrugInteractionInfo();
            info.setId(vo.getId());
            info.setCustom(vo.isCustom());
            info.setInteractionDescription(vo.getInteractionDescription());
            info.setSeverity(findSeverityLevelByDescription(vo.getSeverity()));
            info.setShortText(vo.getShortText());
            if (LOGGER.is   ugEnabled()) {
                LOGGER.debug("Duplicate therapy class " + info.getId() + " (" + info.getInteractionDescription() + ")");
            }
            for (DrugCheckVo d : vo.getDrugs()) {
                info.addDrug(map.get(d.getDrugName()));
            }

            ProfessionalMonographVo pmVo = vo.getProfessionalMonograph();
            if (pmVo != null) {
                ProfessionalMonograph pm = new ProfessionalMonograph();
                pm.setClinicalEffects(snipLabel(pmVo.getClinicalEffects()));
                pm.setDisclaimer(snipLabel(pmVo.getDisclaimer()));
                pm.setDiscussion(snipLabel(pmVo.getDiscussion()));
                pm.setMechanismOfAction(snipLabel(pmVo.getMechanismOfAction()));
                pm.setPatientManagement(snipLabel(pmVo.getPatientManagement()));
                pm.setPredisposingFactors(snipLabel(pmVo.getPredisposingFactors()));
                // the first element in the references array is the 'REFERENCES:' label
                if (pmVo.getReferences().size() > 2) {
                    pm.setReferences(pmVo.getReferences().subList(1, pmVo.getReferences().size()));
                }
                pm.setSeverityLevel(snipLabel(pmVo.getSeverityLevel()));
                pm.setTitle(snipLabel(pmVo.getMonographTitle()));
                info.setProfessionalMonograph(pm);
            }

            res.addChecks(info);
        }
        return res;
    }

    /**
     * Find severity level by description.
     * 
     * @param description
     *            the description
     * @return the severity level
     */
    private SeverityLevel findSeverityLevelByDescription(String description) {
        for (SeverityLevel sl : getTerminologyService().getSeverityLevels()) {
            if (sl.getDescription().equalsIgnoreCase(description)) {
                return sl;
            }
        }
        throw new IllegalArgumentException("Invalid severity description: " + description);
    }

    /**
     * Snip label.
     * 
     * @param text
     *            the label
     * @return the label
     */
    private String snipLabel(String text) {
        if (text == null) {
            return null;
        }
        int index = text.indexOf(':');
        if (index < 0) {
            return text;
        } else {
            return text.substring(index + 1);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * gov.va.med.pharmacy.pecs.service.easy.EasySearchService#checkDosing(gov.va.med.pharmacy.pecs.domain.impl.easy.DoseCheck)
     */
    @Override
    public DrugCheckResult<DosingInfo> checkDosing(DoseCheck doseCheck) {
        OrderCheckVo orderCheck = getOrderCheckVo(Collections.singleton(getDrug(doseCheck.getDrugId())));
        DrugCheckVo dc = orderCheck.getDrugsToScreen().iterator().next();
        DoseRoute doseRoute = getTerminologyService().getDoseRouteByCode(doseCheck.getDoseRoute());
        DoseType doseType = getTerminologyService().getDoseTypeByCode(doseCheck.getDoseType());

        dc.setDoseRate(doseCheck.getDoseRateUnit());
        dc.setDoseRoute(doseRoute.getName());
        dc.setDoseType(doseType.getName());
        dc.setDoseUnit(getTerminologyService().getDoseUnitTypeByCode(doseCheck.getDoseUnit()).getName());
        dc.setDuration(doseCheck.getDuration());
        dc.setDurationRate(doseCheck.getDurationRate());
        dc.setFrequency(doseCheck.getFrequency());
        dc.setSingleDoseAmount(doseCheck.getSingleDoseAmount());
        dc.setDoseCheck(true);

        orderCheck.setDrugDoseCheck(true);
        orderCheck.setAgeInDays(doseCheck.getAgeInDays());
        orderCheck.setWeightInKg(doseCheck.getWeightInKg());
        orderCheck.setBodySurfaceAreaInSqM(doseCheck.getBsaInSqM());
        OrderCheckResultsVo response = getDrugCheckCapability().performDrugChecks(orderCheck);

        DrugCheckResult<DosingInfo> res = new DrugCheckResult<DosingInfo>();
        if (response.getDrugDoseCheckResults().getChecks().isEmpty()) {
            return res;
        }
        for (DrugDoseCheckResultVo vo : response.getDrugDoseCheckResults().getChecks()) {
            DosingInfo info = new DosingInfo();
            info.setCustom(vo.isCustom());
            info.setChemoInjectable(vo.isChemoInjectable());
            info.setDailyDoseMessage(vo.getDailyDoseMessage());
            info.setDailyDoseStatus(vo.getDailyDoseStatus());
            info.setDailyDoseStatusCode(vo.getDailyDoseStatusCode());
            info.setDoseFormHigh(vo.getDoseFormHigh());
            info.setDoseFormHighUnit(vo.getDoseFormHighUnit());
            info.setDoseFormLow(vo.getDoseFormLow());
            info.setDoseFormLowUnit(vo.getDoseFormLowUnit());
            info.setDoseFrequencyHigh(vo.getDoseFrequencyHigh());
            info.setDoseFrequencyLow(vo.getDoseFrequencyLow());
            info.setDoseHigh(vo.getDoseHigh());
            info.setDoseHighUnit(vo.getDoseHighUnit());
            info.setDoseLow(vo.getDoseLow());
            info.setDoseLowUnit(vo.getDoseLowUnit());
            info.setDoseRouteDescription(vo.getDoseRouteDescription());
            info.setDoseTypeDescription(doseType.getDescription());
            info.setDurationMessage(vo.getDurationMessage());
            info.setDurationStatus(vo.getDurationStatus());
            info.setDurationStatusCode(vo.getDurationStatusCode());
            info.setFrequencyMessage(vo.getFrequencyMessage());
            info.setFrequencyStatus(vo.getFrequencyStatus());
            info.setFrequencyStatusCode(vo.getFrequencyStatusCode());
            info.setMaxDailyDoseMessage(vo.getMaxDailyDoseMessage());
            info.setMaxDailyDoseStatus(vo.getMaxDailyDoseStatus());
            info.setMaxDailyDoseStatusCode(vo.getMaxDailyDoseStatusCode());
            info.setMaxLifetimeDose(vo.getMaxLifetimeDose());
            info.setRangeDoseHigh(vo.getRangeDoseHigh());
            info.setRangeDoseLow(vo.getRangeDoseLow());
            info.setRangeDoseMessage(vo.getRangeDoseMessage());
            info.setRangeDoseStatus(vo.getRangeDoseStatus());
            info.setRangeDoseStatusCode(vo.getRangeDoseStatusCode());
            info.setSingleDoseMax(vo.getSingleDoseMax());
            info.setSingleDoseMessage(vo.getSingleDoseMessage());
            info.setSingleDoseStatus(vo.getSingleDoseStatus());
            info.setSingleDoseStatusCode(vo.getSingleDoseStatusCode());

            info.setDoseHigh(vo.getDoseHigh());
            info.setDoseHighUnit(vo.getDoseHighUnit());
            info.setDoseRateHigh("1");
            info.setDoseRateHighUnit("1");
            res.addChecks(info);
        }
        return res;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * gov.va.med.pharmacy.pecs.service.easy.EasySearchService#getDrug(gov.va.med.pharmacy.pecs.domain.impl.easy.DrugIdentifier)
     */
    @Override
    public PecsDrugInfo getDrug(DrugIdentifier id) {
        PecsDrugInfo dto = new PecsDrugInfo();
        try {
            populateDTOFromDispensableGeneric(id, dto);
        } catch (FDBUnknownIDException e) {
            try {
                populateDTOFromDispensableDrug(id, dto);
            } catch (NumberFormatException ex) {
                throw new CustomizationServiceException("Error retrieving drug " + id, ex);
            } catch (FDBSQLException ex) {
                throw new CustomizationServiceException("Error retrieving drug " + id, ex);
            } catch (FDBUnknownIDException ex) {

            } catch (FDBNotAvailableException ex) {
                throw new CustomizationServiceException("Error retrieving drug " + id, ex);
            }
        } catch (NumberFormatException e) {
            throw new CustomizationServiceException("Error retrieving drug " + id, e);
        } catch (FDBSQLException e) {
            throw new CustomizationServiceException("Error retrieving drug " + id, e);
        }

        return dto;

    }

    /**
     * Populate dto from dispensable drug.
     * 
     * @param id
     *            the id
     * @param dto
     *            the dto
     * @throws FDBSQLException
     *             the fDBSQL exception
     * @throws FDBUnknownIDException
     *             the fDB unknown id exception
     * @throws FDBNotAvailableException
     *             the fDB not available exception
     */
    private void populateDTOFromDispensableDrug(DrugIdentifier id, PecsDrugInfo dto) throws FDBSQLException,
            FDBUnknownIDException, FDBNotAvailableException {
        DispensableDrug drug = new DispensableDrug(getFdbDataManager());

        dto.setId(id);
        switch (id.getType()) {
        case GCN_SEQNO:
            drug.load(Long.valueOf(id.getValueAsString()), FDBDispensableDrugLoadType.fdbDDLTGCNSeqNo, "", "", "");
            break;
        case MED_ID:
            drug.load(Long.valueOf(id.getValueAsString()), FDBDispensableDrugLoadType.fdbDDLTMedID, "", "", "");
            dto.addAlternateId(new DrugIdentifier(drug.getGCNSeqNo(), DrugIdentifierType.GCN_SEQNO));
            break;
        default:
            break;
        }
        dto.setDescription(drug.getDescription());
        DTClasses dtClasses = drug.getDuplicateTherapyClasses(true, "");
        ArrayList<TherapeuticClass> dtc = new ArrayList<TherapeuticClass>();
        for (int i = 0; i < dtClasses.count(); i++) {
            DTClass item = dtClasses.item(i);
            TherapeuticClass dtClass = new TherapeuticClass();
            dtClass.setDescription(item.getDescription());
            dtClass.setId(item.getID());
            dtc.add(dtClass);
        }
        dto.setTherapeuticClasses(dtc);
    }

    /**
     * Populate dto from dispensable generic.
     * 
     * @param id
     *            the id
     * @param dto
     *            the dto
     * @throws FDBSQLException
     *             the fDBSQL exception
     * @throws FDBUnknownIDException
     *             the fDB unknown id exception
     */
    private void populateDTOFromDispensableGeneric(DrugIdentifier id, PecsDrugInfo dto) throws FDBSQLException,
            FDBUnknownIDException {
        DispensableGeneric generic = new DispensableGeneric(getFdbDataManager());
        dto.setId(id);

        generic.load(Long.valueOf(id.getValueAsString()), "", "");
        dto.setDescription(generic.getDescription());
        DTClasses dtClasses = generic.getDuplicateTherapyClasses(true, "");
        ArrayList<TherapeuticClass> dtc = new ArrayList<TherapeuticClass>();
        for (int i = 0; i < dtClasses.count(); i++) {
            DTClass item = dtClasses.item(i);
            TherapeuticClass dtClass = new TherapeuticClass();
            dtClass.setDescription(item.getDescription());
            dtClass.setId(item.getID());
            dtc.add(dtClass);
        }
        dto.setTherapeuticClasses(dtc);
    }

    /**
     * Load drug.
     * 
     * @param id
     *            the drug identifier
     * @return a dispensable drug
     */
    private DispensableDrug loadDrug(DrugIdentifier id) {
        try {
            DispensableDrug drug = new DispensableDrug(getFdbDataManager());
            switch (id.getType()) {
            case GCN_SEQNO:
                drug.load(Long.valueOf(id.getValueAsString()), FDBDispensableDrugLoadType.fdbDDLTGCNSeqNo, "", "", "");
                break;
            case MED_ID:
                drug.load(Long.valueOf(id.getValueAsString()), FDBDispensableDrugLoadType.fdbDDLTMedID, "", "", "");
                break;
            default:
                break;
            }
            return drug;
        } catch (FDBException e) {
            throw new CustomizationServiceException("Error loading drug " + id, e);
        }
    }

    /**
     * Sets the DrugConceptDao object.
     * 
     * @param dao
     *            the new drug concept dao
     */
    public void setDrugConceptDao(DrugConceptDao dao) {
        drugConceptDao = dao;
    }

    /**
     * Gets the drug concept dao.
     * 
     * @return the drug concept dao
     */
    public DrugConceptDao getDrugConceptDao() {
        return drugConceptDao;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.service.easy.EasySearchService#getVistaSeverityLevels()
     */
    @Override
    public List<SeverityLevel> getVistaSeverityLevels() {
        return vistaSeverityLevels;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.service.easy.EasySearchService#getDoseRoutes(gov.va.med.pharmacy.pecs.domain.impl.easy.
     * DrugIdentifier)
     */
    @Override
    public List<DoseRoute> getDoseRoutes(DrugIdentifier id) {
        try {
            DispensableDrug drug = loadDrug(id);
            firstdatabank.dif.DoseRoutes routes = drug.getDoseRoutes();
            ArrayList<DoseRoute> res = new ArrayList<DoseRoute>();
            for (int i = 0; i < routes.count(); i++) {
                firstdatabank.dif.DoseRoute route = routes.item(i);
                res.add(getTerminologyService().getDoseRouteByCode(route.getID()));
            }
            return res;
        } catch (Exception e) {
            throw new CustomizationServiceException("Could not retrieve dose routes for drug " + id, e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * gov.va.med.pharmacy.pecs.service.easy.EasySearchService#getDoseTypes
     * (gov.va.med.pharmacy.pecs.domain.impl.easy.DrugIdentifier)
     */
    @Override
    public List<DoseType> getDoseTypes(DrugIdentifier id) {
        try {
            DispensableDrug drug = loadDrug(id);
            firstdatabank.dif.DoseTypes types = drug.getDoseTypes();
            ArrayList<DoseType> res = new ArrayList<DoseType>();
            for (int i = 0; i < types.count(); i++) {
                firstdatabank.dif.DoseType type = types.item(i);
                res.add(getTerminologyService().getDoseTypeByCode(type.getID()));
            }
            return res;
        } catch (Exception e) {
            throw new CustomizationServiceException("Could not retrieve dose types for Drug " + id, e);
        }

    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * gov.va.med.pharmacy.pecs.service.easy.EasySearchService#getDoseUnits
     * (gov.va.med.pharmacy.pecs.domain.impl.easy.DrugIdentifier)
     */
    @Override
    public List<DoseUnitType> getDoseUnits(DrugIdentifier id) {
        try {
            TreeSet<DoseUnitType> res = new TreeSet<DoseUnitType>(new Comparator<DoseUnitType>() {
                @Override
                public int compare(DoseUnitType o1, DoseUnitType o2) {
                    return o1.getName().compareTo(o2.getName());
                }
            });
            DispensableDrug drug = loadDrug(id);
            DRCLookupResults drcDosing = drug.getDRCDosing("", "", "", 0, FDBDosingSource.fdbDSCustomWithFDBDefault, "");
            for (int i = 0; i < drcDosing.count(); i++) {
                DRCLookupResult drc = drcDosing.item(i);
                processDoseUnit(drc.getMaxSingleDoseUnit(), res);
                processDoseUnit(drc.getMaxSingleDoseFormUnit(), res);
                processDoseUnit(drc.getMaxDailyDoseUnit(), res);
                processDoseUnit(drc.getMaxDailyDoseFormUnit(), res);
                processDoseUnit(drc.getDoseLowUnit(), res);
                processDoseUnit(drc.getDoseFormLowUnit(), res);
                processDoseUnit(drc.getDoseHighUnit(), res);
                processDoseUnit(drc.getDoseFormHighUnit(), res);
            }
            return new ArrayList<DoseUnitType>(res);
        } catch (Exception e) {
            throw new CustomizationServiceException("Could not retrieve dose units for drug" + id, e);
        }
    }

    /**
     * Process dose unit.
     * 
     * @param unit
     *            the unit to process
     * @param res
     *            the dose type
     */
    private void processDoseUnit(String unit, TreeSet<DoseUnitType> res) {
        DoseUnitType dut = unitTypesMap.get(getSimpleDoseUnit(unit));
        if (dut != null) {
            res.add(dut);
        }
    }

    /**
     * Gets the simple dose unit.
     * 
     * The Dose Unit Type is expected to be in the format of 'Substance Unit of Measure/Body Weight Unit of Measure[/Duration]'.
     * With FDB DIF 3.3, an alternate format of 'Substance Unit of Measure per Body Weight Unit of Measure[ per Duration]' is
     * coming back. So if we do not find a slash in the doseUnit, search for a ' per '.
     * 
     * @param doseUnit
     *            the dose unit
     * @return an object?
     */
    private Object getSimpleDoseUnit(String doseUnit) {
        int index = doseUnit.indexOf(DOSE_UNIT_DELIMITER);
        if (index == -1) {
            index = doseUnit.indexOf(DOSE_UNIT_ALT_DELIMITER);
        }
        return index > 0 ? doseUnit.substring(0, index).toUpperCase() : doseUnit.toUpperCase();
    }

    /**
     * Gets the drugCheckCapability.
     * 
     * @return the drugCheckCapability
     */
    public PerformDrugChecksCapability getDrugCheckCapability() {
        return drugCheckCapability;
    }

    /**
     * Sets the drugCheckCapability.
     * 
     * @param drugCheckCapability
     *            the drugCheckCapability to set
     */
    public void setDrugCheckCapability(PerformDrugChecksCapability drugCheckCapability) {
        this.drugCheckCapability = drugCheckCapability;
    }

    /**
     * Gets the fdbNavigation.
     * 
     * @return the fdbNavigation
     */
    public Navigation getFdbNavigation() {
        return fdbNavigation;
    }

    /**
     * Sets the fdbNavigation.
     * 
     * @param fdbNavigation
     *            the fdbNavigation to set
     */
    public void setFdbNavigation(Navigation fdbNavigation) {
        this.fdbNavigation = fdbNavigation;
    }

    /**
     * Gets the fdbDataManager.
     * 
     * @return the fdbDataManager
     */
    public FDBDataManager getFdbDataManager() {
        return fdbDataManager;
    }

    /**
     * Sets the fdbDataManager.
     * 
     * @param fdbDataManager
     *            the fdbDataManager to set
     */
    public void setFdbDataManager(FDBDataManager fdbDataManager) {
        this.fdbDataManager = fdbDataManager;
    }

    /**
     * Gets the terminologyService.
     * 
     * @return the terminologyService
     */
    public TerminologyService getTerminologyService() {
        return terminologyService;
    }

    /**
     * Sets the terminologyService.
     * 
     * @param terminologyService
     *            the terminologyService to set
     */
    public void setTerminologyService(TerminologyService terminologyService) {
        this.terminologyService = terminologyService;
    }

    /**
     * Gets the drug pair service.
     * 
     * @return the drug pair service
     */
    public DrugPairService getDrugPairService() {
        return drugPairService;
    }

    /**
     * Sets the drug pair service.
     * 
     * @param drugPairService
     *            the new drug pair service
     */
    public void setDrugPairService(DrugPairService drugPairService) {
        this.drugPairService = drugPairService;
    }

    /**
     * Method will compare the severity level indicator from the front end each drug pair in the provided list.
     * 
     * indicator 1 is severity level 1, indicator 2 is severity level 2, indicator 12 is severity level 1 & 2
     * 
     * @param listToFilter
     *            the list to filter
     * @param severityLevelIndicator
     *            the severity level indicator
     * @return the list of DrugPairDTO
     */
    private List<DrugPairDTO> filterDataBySeverity(List<DrugPairDTO> listToFilter, String severityLevelIndicator) {

        List<DrugPairDTO> filteredList = new ArrayList<DrugPairDTO>();

        for (DrugPairDTO drugPair : listToFilter) {

            if (severityLevelIndicator.equals("12")) {
                if (drugPair.getSeveritylevelcode().equals("1") || drugPair.getSeveritylevelcode().equals("2")) {
                    filteredList.add(drugPair);
                    continue;
                }

                continue;
            }

            if (severityLevelIndicator.equals(drugPair.getSeveritylevelcode())) {
                filteredList.add(drugPair);
            }
        }
        return filteredList;
    }

    /**
     * Method will combine the VA and FDB drug lists.  
     * All VA drug pairs are added then used as a key to only add 
     * FDB drugs that are not already represented by a VA record.
     * 
     * @param vaDrugPairList
     *            the VA drug pair list
     * @param fdbDrugPairList
     *            the FDB drug pair list
     * @return the list
     */
    private List<DrugPairDTO> combineDrugPairLists(List<DrugPairDTO> vaDrugPairList, List<DrugPairDTO> fdbDrugPairList) {
  
        List<DrugPairDTO> targetList = new ArrayList<DrugPairDTO>();
        
        List<String> keyList = new ArrayList<String>(vaDrugPairList.size());
        for (DrugPairDTO vaDrugPair : vaDrugPairList) {
            targetList.add(vaDrugPair);
            keyList.add(vaDrugPair.getRtgenid1() + "|" + vaDrugPair.getRtgenid2() + "|" + vaDrugPair.getFdbId());
        }
             
        for (DrugPairDTO fdbDrugPair : fdbDrugPairList) {  
            //@formatter: off
            if (!keyList.contains(fdbDrugPair.getRtgenid1() + "|" 
            + fdbDrugPair.getRtgenid2() + "|" + fdbDrugPair.getInteractionid())){
                targetList.add(fdbDrugPair);
            }      
            //@formatter: on
        }
        
        return targetList;

    }

    /**
     * Sort list by desc.
     * 
     * @param allData
     *            the all data
     */

    private void sortListByDesc(List<DrugPairDTO> allData) {

        Collections.sort(allData, new Comparator<DrugPairDTO>() {
            @Override
            public int compare(DrugPairDTO o1, DrugPairDTO o2) {

                int result = 0;
                if (o1.getRtgenid1Desc() != null && o2.getRtgenid1Desc() != null) {
                    result = o1.getRtgenid1Desc().compareToIgnoreCase(o2.getRtgenid1Desc());
                    if (result == 0) {
                        if (o1.getRtgenid2Desc() != null && o2.getRtgenid2Desc() != null) {
                            return o1.getRtgenid2Desc().compareToIgnoreCase(o2.getRtgenid2Desc());
                        }
                    }
                }
                return result;
            }
        });
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.springframework.beans.factory.InitializingBean#afterPropertiesSet()
     */
    @Override
    public void afterPropertiesSet() throws Exception {
        if (drugConceptDao == null) {
            throw new Exception("drugConceptDao has not been injected into EasySearchServiceImpl.");
        }
        if (drugPairService == null) {
            throw new Exception("drugPairService has not been injected into EasySearchServiceImpl.");
        }

        for (String code : VISTA_SEVERITY_LEVEL_CODES) {
            vistaSeverityLevels.add(getTerminologyService().getSeverityLevelByCode(code));
        }
        unitTypesMap = new HashMap<String, DoseUnitType>();
        for (DoseUnitType unitType : getTerminologyService().getDoseUnitTypeData()) {
            unitTypesMap.put(unitType.getName(), unitType);
        }
    }

}
