package gov.va.med.pharmacy.pecs.service.fdbreports.impl;

import gov.va.med.pharmacy.pecs.domain.impl.lookup.ComparisonReportConcept;
import gov.va.med.pharmacy.pecs.domain.impl.va.FdbUpdateReport;
import gov.va.med.pharmacy.pecs.fdbreports.ExcelUpdateReportHelper;
import gov.va.med.pharmacy.pecs.persistence.FdbUpdateReportDao;
import gov.va.med.pharmacy.pecs.util.BlobHandler;

import java.io.IOException;
import java.sql.Date;

import javax.annotation.Resource;

import org.springframework.beans.factory.InitializingBean;
import org.springframework.stereotype.Service;

/**
 * A utility to cleanup the FDB Comparison Reports.
 * 
 * @author vhaisaduffg
 * 
 */
@Service
public final class ComparisonReportUtility implements InitializingBean {

    // Milliseconds * seconds in minute * minutes in hour * hours in day * days
    // in week * number of weeks
    private static final Long REPORT_RETENTION_PERIOD = new Long(1000L * 60L * 60L * 24L * 7L * 8L);

    @Resource
    private FdbUpdateReportDao fdbUpdateReportDao;

    /** The blob handler. */
    @Resource
    private BlobHandler blobHandler;

    /**
     * Calculates the Date that all reports since should be kept and passes that value to the FdbUpdateReportDao.cleanup method.
     */
    public void cleanup() {
        Date reportRetentionPeriod = new Date(System.currentTimeMillis() - REPORT_RETENTION_PERIOD);

        fdbUpdateReportDao.cleanup(reportRetentionPeriod);
    }

    /**
     * Saves the report to the file system.
     * 
     * @param helper
     *            The helper to create the excel report
     * @param fileNameToSave
     *            the name the file will be saved as
     * @param concept
     *            the concept in the report
     * @throws IOException
     *             if an error occurs
     */
    public void saveReport(ExcelUpdateReportHelper helper, String fileNameToSave, ComparisonReportConcept concept)
            throws IOException {
        FdbUpdateReport report = new FdbUpdateReport();

        if ( helper == null ) {
            throw new IllegalArgumentException("ExcelUpdateReportHelper cannot be null.");
        }
        if ( fileNameToSave == null || fileNameToSave.length() == 0 ) {
            throw new IllegalArgumentException("fileNameToSave cannot be null or have a length of zero.");
        }
        if ( concept == null ) {
            throw new IllegalArgumentException("ComparisonReportConcept cannot be null.");
        }
        
        report.setConcept(concept);
        report.setFdbUpdateDate(new Date(System.currentTimeMillis()));
        report.setContent(getBlobHandler().createBlob(helper.toByteArray()));
        report.setFilename(fileNameToSave);
        fdbUpdateReportDao.save(report);
    }

    /**
     * Sets the FdbUpdateReportDao to use.
     * 
     * @param dao
     *            FdbUpdateReportDao
     */
    public final void setFdbUpdateReportDao(FdbUpdateReportDao dao) {
        fdbUpdateReportDao = dao;
    }

    /**
     * Gets the blobHandler.
     * 
     * @return the blobHandler
     */
    public BlobHandler getBlobHandler() {
        return blobHandler;
    }

    /**
     * Sets the blobHandler.
     * 
     * @param blobHandler
     *            the blobHandler to set
     */
    public void setBlobHandler(BlobHandler blobHandler) {
        this.blobHandler = blobHandler;
    }

    /**
     * Allows the class to check to make sure all required objects have been injected.
     * 
     * @throws Exception
     *             if a property has not been injected
     */
    @Override
    public final void afterPropertiesSet() throws Exception {
        if (fdbUpdateReportDao == null) {
            throw new Exception("FdbUpdateReportDao has not been injected in to ComparisonReportUtility.");
        }
        if (blobHandler == null) {
            throw new Exception("BlobHandler has not been injected into FdbUpdateReportServiceImpl.");
        }
    }
}
