package gov.va.med.pharmacy.pecs.service.fdbreports.impl;

import gov.va.med.pharmacy.pecs.domain.impl.lookup.ClinicalEffect;
import gov.va.med.pharmacy.pecs.domain.impl.lookup.SeverityLevel;
import gov.va.med.pharmacy.pecs.dto.MonographDTO;
import gov.va.med.pharmacy.pecs.exception.CustomizationServiceException;
import gov.va.med.pharmacy.pecs.fdbreports.ExcelUpdateReportHelper;
import gov.va.med.pharmacy.pecs.fdbreports.FdbUpdateParser;
import gov.va.med.pharmacy.pecs.persistence.fdb.DrugConceptDao;
import gov.va.med.pharmacy.pecs.service.DrugDrugInteractionService;
import gov.va.med.pharmacy.pecs.service.DrugPairService;
import gov.va.med.pharmacy.pecs.service.MonographService;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.util.Arrays;
import java.util.List;

import javax.annotation.Resource;

import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.stereotype.Component;

/**
 * Creates and saves the Drug Drug Interaction Comparison report.
 * 
 * @author VHAISACARUSR
 * 
 */
@Component
public final class DrugDrugInteractionComparisonReportProcessor extends AbstractComparisonReportProcessor implements
        InitializingBean {

    private static final String FDBDDIMDRUGLINK_UPD = "FDBDDIMDRUGLINK.UPD";

    private static final String FDBDDIMINTERACTION_UPD = "FDBDDIMINTERACTION.UPD";

    private static final String DDI_CONCEPT_KEY = "DDI";

    /** The Constant LOGGER. */
    private static final Logger LOGGER = Logger.getLogger(DrugDrugInteractionComparisonReportProcessor.class);

    /** The monograph service. */
    @Resource
    private MonographService monographService;

    /** The drug drug interaction service. */
    @Resource
    private DrugDrugInteractionService drugDrugInteractionService;

    /** The routed generics processor. */
    @Resource
    private RoutedGenericsProcessor routedGenericsProcessor;

    /** The drug pair service. */
    @Resource
    private DrugPairService drugPairService;

    /** The drug concept dao. */
    @Resource
    private DrugConceptDao drugConceptDao;

    /**
     * Process drug drug interaction.
     * 
     * @param parser
     *            the parser
     */
    @Override
    public void processComparisonReport(FdbUpdateParser parser) {
        try {
            // Processing FDBDDUMDRUGLINK.UPD in two parts. First to create UPD ArrayList for comparing
            // the DP input records to the DDI input records for notes in the DDI report's Drug Pair column.
            List<SeverityLevel> severityList = getTerminologyService().getSeverityLevels();
            List<ClinicalEffect> clinicalList = getTerminologyService().getClinicalEffects();
            List<MonographDTO> monoList = monographService.getAllMonographs();
            File dpUpd = parser.parseConcept(FDBDDIMDRUGLINK_UPD);
            ObjectInputStream inDP = new ObjectInputStream(new FileInputStream(dpUpd));
            DPFdbUpdateDataAugmentation augDP = new DPFdbUpdateDataAugmentation(drugPairService);
            augDP.setInput(inDP);
            // Creating ArrayList to sort and hold UPD input records
            // It will first be used to match interactionids within ddi rpt generator
            List<SortedDrugPair> sortedUpdArrayList = SortedDrugPairHelper.buildDpArrayList(augDP);
            File f = parser.parseConcept(FDBDDIMINTERACTION_UPD);
            ObjectInputStream in = new ObjectInputStream(new FileInputStream(f));
            DDIFdbUpdateDataAugmentation aug = new DDIFdbUpdateDataAugmentation(drugDrugInteractionService);
            aug.setInput(in);
            DDIReportGenerator reportGenerator = new DDIReportGenerator(aug, sortedUpdArrayList);
            ExcelUpdateReportHelper helper =
                    reportGenerator.generateReport(parser.getFdbIssueDate(), severityList, clinicalList, monoList);
            // After processing FDBDDIMINTERACTION.UPD, the UPD ArrayList will be used to create
            // the worksheets.
            Long savedInteractionId = 0L;
            int row = 0;
            SortedDrugPair sdp;
            // Create and populate new spreadsheet for each interactionid/drug pair in FDBDDIMDRUGLINK.UPD
            for (int i = 0; i < sortedUpdArrayList.size(); i++) {
                sdp = sortedUpdArrayList.get(i);
                if (!sdp.getInteractionid().equals(savedInteractionId)) {
                    List<String> headerList =
                            Arrays.asList(new String[] { "Routed Generic 1 Description", "Routed Generic 2 Description",
                                    "DATUP action" });
                    helper.createWorksheet("FDB Interaction ID " + sdp.getInteractionid() + "-DP");
                    row = helper.createHeader2(headerList, parser.getFdbIssueDate());
                    savedInteractionId = sdp.getInteractionid();
                }
                helper.createLatestFdbRow(++row);
                helper.addCell(row, 0, findRoutedGenericDescription(sdp.getRtgenid1()));
                helper.addCell(row, 1, findRoutedGenericDescription(sdp.getRtgenid2()));
                helper.addCell(row, 2, sdp.getDatupAction());
            }
            String fileNameToSave = "DDIReport.xlsx";
            saveReport(helper, fileNameToSave, DDI_CONCEPT_KEY);
            getComparisonReportUtility().cleanup();
        } catch (Exception ex) {
            String msg = "Failed to process DDI inside processDrugDrugInteraction therapy.";
            LOGGER.error(msg, ex);
            throw new CustomizationServiceException(msg, ex);
        }

    }

    /**
     * A method to retrieve the Description for a Routed Generic.
     * 
     * @param rtGenId
     *            Routed Generic Identifier
     * @return String
     */
    private final String findRoutedGenericDescription(Long rtGenId) {
        String description = null;
        description = drugConceptDao.getDrugConceptDescription(rtGenId, DrugConceptDao.ROUTED_GENERIC_TYPE);
        if (description == null || StringUtils.EMPTY.equals(description)) {
            try {
                description = getRoutedGenericsProcessor().getDescription(rtGenId);
            } catch (IOException ioe) {
                LOGGER.error(ioe.getMessage(), ioe);
            }
        }
        return description;
    }

    /**
     * Gets the routedGenericsProcessor.
     * 
     * @return the routedGenericsProcessor
     */
    public RoutedGenericsProcessor getRoutedGenericsProcessor() {
        return routedGenericsProcessor;
    }

    /**
     * Sets the routedGenericsProcessor.
     * 
     * @param routedGenericsProcessor
     *            the routedGenericsProcessor to set
     */
    public void setRoutedGenericsProcessor(RoutedGenericsProcessor routedGenericsProcessor) {
        this.routedGenericsProcessor = routedGenericsProcessor;
    }

    @Override
    public void afterPropertiesSet() throws Exception {

        if (monographService == null) {
            throw new Exception("monograph service has not been injected into FdbUpdateReportServiceImpl.");
        }
    }
}
