package gov.va.med.pharmacy.pecs.service.fdbreports.impl;

import gov.va.med.pharmacy.pecs.dto.PecsDTO;
import gov.va.med.pharmacy.pecs.fdbreports.FdbColumnUpdate;
import gov.va.med.pharmacy.pecs.fdbreports.FdbTableColumn;
import gov.va.med.pharmacy.pecs.fdbreports.FdbTableMetadata;
import gov.va.med.pharmacy.pecs.fdbreports.FdbUpdateRecord;
import gov.va.med.pharmacy.pecs.persistence.HibernateDAO;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.util.ArrayList;
import java.util.List;

import org.hibernate.SQLQuery;

/**
 * A class used to prepare data for report generators. It's the model in an MVC pattern (the report generators being the view).
 * Specifically, for update records extracted from an FDB update, we need to pair the new values with the old ones from the
 * database, that are about to be overwritten; we need to bring in from the database values for the columns that are not
 * affected by the update; we need, in some cases, to retrieve the VA object(s) that are associated with the given FDB record.
 */
public abstract class FdbUpdateDataAugmentation {

    /** The dao. */
    private final HibernateDAO<?> dao;

    /** The input. */
    private ObjectInputStream input;

    /**
     * Instantiates a new fdb update data augmentation.
     * 
     * @param dao
     *            the dao
     */
    public FdbUpdateDataAugmentation(HibernateDAO<?> dao) {
        this.dao = dao;
    }

    /**
     * Find corresponding VA objects.
     * 
     * @param rec
     *            the record
     * @return the list<? extends {@link PecsDTO}>
     */
    public abstract List<? extends PecsDTO> findCorrespondingVaObjects(FdbUpdateRecord rec);

    /**
     * Next record.
     * 
     * @return the FDB update record
     * @throws IOException
     *             Signals that an I/O exception has occurred.
     * @throws ClassNotFoundException
     *             the class not found exception
     */
    public final FdbUpdateRecord nextRecord() throws IOException, ClassNotFoundException {
        FdbUpdateRecord rec = (FdbUpdateRecord) input.readObject();
        if (rec != null) {
            populateCorrespondingFdbValues(rec);
        }
        return rec;
    }

    /**
     * Populate the values from the database corresponding to the update record passed as argument.
     * 
     * @param rec
     *            the update record for which the previous values need to be populated from the database
     */
    public final void populateCorrespondingFdbValues(FdbUpdateRecord rec) {
        if (FdbUpdateRecord.REC_TYPE_ADD.equals(rec.getType())) {
            // no previous values if the record is being added
            return;
        }

        FdbTableMetadata table = rec.getTable();
        List<FdbTableColumn> columns = table.getColumns();
        List<FdbColumnUpdate> keys = rec.getKeyValues();
        // we will collect key values in this list
        List<Object> params = new ArrayList<Object>();

        String sql = createSQL(table, columns, keys, params);
        SQLQuery q = dao.getSession().createSQLQuery(sql);
        // bind key values; note that they were added to the list in
        // the same order the keys were added to the where clause
        for (int i = 0; i < params.size(); i++) {
            // I was under the impression parameter positions are 1-based,
            // but starting at 1 causes an exception (that says parameter
            // positions are 1-based - go figure!)
            q.setParameter(i, params.get(i));
        }

        // populate the values that came back from the database into
        // the update record;
        // the result should have only one entry, consisting of
        // values for the columns from the metadata (in the same order)
        Object[] dbValues = (Object[]) q.list().get(0);
        for (int i = 0; i < dbValues.length; i++) {
            FdbColumnUpdate c = rec.getValue(columns.get(i));
            if (c == null) {
                // this field was not present in the update record,
                // which means its value has not changed
                c = new FdbColumnUpdate();
                c.setColumn(columns.get(i));
                if (FdbUpdateRecord.REC_TYPE_CHANGE.equals(rec.getType())) {
                    // for change records, populate the current values with the old ones
                    Object old = dbValues[i];
                    c.setCurrentValue(old != null ? old.toString() : null);
                }
                rec.addValue(c);
            }
            Object old = dbValues[i];
            c.setPreviousValue(old != null ? old.toString() : null);
        }
    }

    /**
     * @param table
     *            the FDB table Meta Data
     * @param columns
     *            the list of columns
     * @param keys
     *            list of keys
     * @param params
     *            the parameters for the query
     * @return the query as a string
     */
    private final String createSQL(FdbTableMetadata table, List<FdbTableColumn> columns, List<FdbColumnUpdate> keys,
            List<Object> params) {
        StringBuffer sql = new StringBuffer("select ");
        boolean first = true;
        // build the select column name list
        for (FdbTableColumn col : columns) {
            if (first) {
                first = false;
            } else {
                sql.append(',');
            }
            sql.append(col.getName());
        }
        sql.append(" from ");
        sql.append(table.getTableName());
        sql.append(" where ");
        first = true;
        for (FdbColumnUpdate key : keys) {
            if (first) {
                first = false;
            } else {
                sql.append(" and ");
            }
            sql.append(key.getColumn().getName());
            sql.append("=?");
            params.add(key.getCurrentValueAsObject());
        }
        return sql.toString();
    }

    /**
     * Sets the input.
     * 
     * @param input
     *            the new input
     */
    public final void setInput(ObjectInputStream input) {
        this.input = input;
    }

    /**
     * Gets the input.
     * 
     * @return the input
     */
    public final ObjectInputStream getInput() {
        return input;
    }

}
