package gov.va.med.pharmacy.pecs.service.fdbreports.impl;

import gov.va.med.pharmacy.pecs.customupdate.FdbVersion;
import gov.va.med.pharmacy.pecs.domain.impl.va.FdbUpdateReport;
import gov.va.med.pharmacy.pecs.exception.CustomizationServiceException;
import gov.va.med.pharmacy.pecs.fdbreports.FdbUpdateParser;
import gov.va.med.pharmacy.pecs.net.FileTransferCommands;
import gov.va.med.pharmacy.pecs.net.FileTransferFactory;
import gov.va.med.pharmacy.pecs.net.FtpClient;
import gov.va.med.pharmacy.pecs.persistence.FdbUpdateReportDao;
import gov.va.med.pharmacy.pecs.service.CustomUpdateService;
import gov.va.med.pharmacy.pecs.service.fdbreports.FdbUpdateReportService;

import java.io.File;
import java.io.IOException;
import java.util.List;
import java.util.Properties;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.annotation.Resource;

import org.apache.commons.net.ftp.FTPFile;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * The Class FdbUpdateReportServiceImpl.
 */
@Service
public final class FdbUpdateReportServiceImpl implements FdbUpdateReportService, InitializingBean {

    /** The Constant LOGGER. */
    private static final Logger LOGGER = Logger.getLogger(FdbUpdateReportServiceImpl.class);

    /** The Constant ZIP_FILE_PATTERN. */
    protected static final Pattern ZIP_FILE_PATTERN = Pattern.compile("(.*\\.zip)(;\\d+)?", Pattern.CASE_INSENSITIVE);

    /** The export file properties. */
    @Resource
    private Properties exportFileProperties = null;

    /** The routed generics processor. */
    @Resource
    private RoutedGenericsProcessor routedGenericsProcessor;

    /** The custom update service. */
    @Resource
    private CustomUpdateService customUpdateService;

    /** The fdb update report dao. */
    @Resource
    private FdbUpdateReportDao fdbUpdateReportDao;

    /** The comparison report utility. */
    @Resource
    private ComparisonReportUtility comparisonReportUtility;

    @Resource
    private DrugDrugInteractionComparisonReportProcessor ddiReportProcessor;

    @Resource
    private DuplicateTherapyComparisonReportProcessor dtReportProcessor;

    @Resource
    private DoseRangeComparisonReportProcessor drReportProcessor;

    @Resource
    private FileTransferFactory fileTransferFactory;

    @Override
    @Transactional
    public void processUpdateFile() {
        File file;
        FileTransferCommands ftp = fileTransferFactory.createFtp();
        try {
            connectToFTPServer(ftp);
            // String dir = exportFileProperties.getProperty(CustomUpdateServiceImpl.FDB_FILE_DIR_PROPERTY);
            changeFtpDir(ftp);
            FTPFile[] ftpFiles = ftp.listFiles();
            String fdbIssueDate = null;

            for (int i = 0; i < ftpFiles.length; i++) {
                String fName = ftpFiles[i].getName();
                Matcher filenameMatcher = ZIP_FILE_PATTERN.matcher(fName);
                // ignore if not a ZIP file
                if (ftpFiles[i].isDirectory() || !filenameMatcher.matches()) {
                    LOGGER.debug("Skipping non-ZIP file: " + ftpFiles[i]);
                    continue;
                }
                LOGGER.debug("Downloading zip archive " + fName);
                file = ftp.retrieveToTemporaryFile(fName);
                FdbUpdateParser parser = new FdbUpdateParser(file);
                fdbIssueDate = parser.getFdbIssueDate();
                if (fdbIssueDate == null || !checkPreviousIssueDate(parser.getFdbPreviousIssueDate(), fName)) {
                    // not an FDB incremental update
                    LOGGER.debug(fName + " does not appear to be an incremental update file");
                    continue;
                }

                processReports(parser);
                LOGGER.debug("DrugDrugInteraction process is completed");

                // we're just going to process one update file; I'm not sure
                // how to handle the situation where more than one is found
                break;
            }

        } catch (Exception e) {
            String msg = "Failed to process FDB update file";// + f.getAbsolutePath();
            LOGGER.error(msg, e);
            e.printStackTrace();
            throw new CustomizationServiceException(msg, e);
        } finally {
            try {
                ftp.logout();
            } catch (Exception ignore) {
                LOGGER.warn(ignore.getMessage());
            }
        }
    }

    /**
     * @param parser
     *            the FdbUpdateParser
     */
    private void processReports(FdbUpdateParser parser) {
        String controlFileDate = parser.getFdbIssueDate();
        try {
            routedGenericsProcessor.processRoutedGenerics(parser, controlFileDate);
            LOGGER.debug("RoutedGenerics process is completed");
        } catch (Exception ex) {
            String msg = "Failed to generate report for Routed Generic excel";
            LOGGER.error(msg, ex);
            throw new CustomizationServiceException(msg, ex);
        }
        // processDosing(parser);
        try {
            getDtReportProcessor().processComparisonReport(parser);
        } catch (Exception ex) {
            String msg = "Failed to generate report for Duplicate therapy excel";
            LOGGER.error(msg, ex);
            throw new CustomizationServiceException(msg, ex);
        }

        try {
            ddiReportProcessor.processComparisonReport(parser);
        } catch (Exception ex) {
            String msg = "Failed to generate report for Drug Drug Interaction excel";
            LOGGER.error(msg, ex);
            throw new CustomizationServiceException(msg, ex);
        }
        try {
            drReportProcessor.processComparisonReport(parser);
        } catch (Exception ex) {
            String msg = "Failed to generate report for Drug Range excel";
            LOGGER.error(msg, ex);
            throw new CustomizationServiceException(msg, ex);
        }
    }

    /**
     * @param ftp
     *            the FTP client
     * @throws IOException
     *             if operation fails
     */
    private void changeFtpDir(FileTransferCommands ftp) throws IOException {
        String dir = exportFileProperties.getProperty(CustomUpdateService.EXPORT_FILE_DIR_PROPERTY);
        if (!ftp.setFileType(FtpClient.BINARY_FILE_TYPE)) {
            String msg = "Unable to set file transfer type to binary";
            throw new CustomizationServiceException(msg);
        }
        LOGGER.debug("Changing directory to " + dir);
        boolean success = ftp.changeWorkingDirectory(dir);
        if (!success) {
            String msg = "Failed to change directory to" + dir;
            throw new CustomizationServiceException(msg);
        }
    }

    /**
     * @param ftp
     *            the FTP Client
     * @throws IOException
     *             if operation fails
     */
    private void connectToFTPServer(FileTransferCommands ftp) throws IOException {
        String server = exportFileProperties.getProperty(CustomUpdateService.EXPORT_FILE_SERVER_PROPERTY);
        LOGGER.debug("Connecting to " + server);
        ftp.connect(server);
        String user = exportFileProperties.getProperty(CustomUpdateService.EXPORT_USER_NAME_PROPERTY);
        String password = exportFileProperties.getProperty(CustomUpdateService.EXPORT_PASSWORD_PROPERTY, user);
        LOGGER.debug("Logging in as " + user);
        boolean success = ftp.login(user, password);
        if (!success) {
            String msg = "Failed to log in";
            throw new CustomizationServiceException(msg);
        }
    }

    /**
     * Check previous issue date.
     * 
     * @param fdbPreviousIssueDate
     *            the fdb previous issue date
     * @param fileName
     *            the file name
     * @return true, if successful
     */
    private boolean checkPreviousIssueDate(String fdbPreviousIssueDate, String fileName) {
        FdbVersion fdbVersion = customUpdateService.getFdbVersion();
        if (fdbVersion != null && fdbVersion.getIssuedate().equals(fdbPreviousIssueDate)) {
            return true;
        }
        if (fdbVersion != null && LOGGER.isInfoEnabled()) {
            LOGGER.info("Skipping file " + fileName + " because the expected previous issue date (" + fdbPreviousIssueDate
                    + ") does not match what is in FDB (" + fdbVersion.getIssuedate() + ").");
        }
        return false;

    }

    @Override
    @Transactional
    public List<FdbUpdateReport> getReports() {
        List<FdbUpdateReport> rptList = fdbUpdateReportDao.findAll(FdbUpdateReport.class);
        return rptList;
    }

    @Override
    @Transactional
    public FdbUpdateReport getReport(Long id) {
        // final Long id = new Long(id);
        FdbUpdateReport reportRetrieved = fdbUpdateReportDao.findById(id, FdbUpdateReport.class);
        return reportRetrieved;
    }

    @Override
    @Transactional
    public void deleteReport(FdbUpdateReport rpt) {
        // final Long id = new Long(id);
        FdbUpdateReport rpt1 = new FdbUpdateReport();
        // fdbUpdateReportDao.delete(FdbUpdateReport.class);
        fdbUpdateReportDao.delete(rpt1);
    }

    /**
     * Gets the export file properties.
     * 
     * @return the export file properties
     */
    public Properties getExportFileProperties() {
        return exportFileProperties;
    }

    /**
     * Sets the export file properties.
     * 
     * @param exportFileProperties
     *            the new export file properties
     */
    public void setExportFileProperties(Properties exportFileProperties) {
        this.exportFileProperties = exportFileProperties;
    }

    /**
     * Allows the FdbUpdateReportDao to be set.
     * 
     * @param dao
     *            FdbUpdateReportDao
     */
    public void setFdbUpdateReportDao(FdbUpdateReportDao dao) {
        fdbUpdateReportDao = dao;
    }

    /**
     * Gets the comparison report utility.
     * 
     * @return the comparison report utility
     */
    public ComparisonReportUtility getComparisonReportUtility() {
        return comparisonReportUtility;
    }

    /**
     * Sets the comparison report utility.
     * 
     * @param comparisonReportUtility
     *            the new comparison report utility
     */
    public void setComparisonReportUtility(ComparisonReportUtility comparisonReportUtility) {
        this.comparisonReportUtility = comparisonReportUtility;
    }

    /**
     * Sets the RoutedGenericsProcessor to use.
     * 
     * @param routedGenericsProcessor
     *            the routedGenericsProcessor to set
     */
    public final void setRoutedGenericsProcessor(RoutedGenericsProcessor routedGenericsProcessor) {
        this.routedGenericsProcessor = routedGenericsProcessor;
    }

    /**
     * Sets the CustomUpdateServiceImpl.
     * 
     * @param service
     *            the CustomUpdateServiceImpl to set
     */
    public final void setCustomUpdateService(CustomUpdateService service) {
        this.customUpdateService = service;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.springframework.beans.factory.InitializingBean#afterPropertiesSet()
     */
    @Override
    public void afterPropertiesSet() throws Exception {

        if (exportFileProperties == null) {
            throw new Exception("exportFileProperties has not been injected into FdbUpdateReportServiceImpl.");
        }

        if (fdbUpdateReportDao == null) {
            throw new Exception("FdbUpdateReportDao has not been injected into FdbUpdateReportServiceImpl.");
        }
        if (comparisonReportUtility == null) {
            throw new Exception("ComparisonReportUtility has not been injected into FdbUpdateReportServiceImpl.");
        }

        if (routedGenericsProcessor == null) {
            throw new Exception("RoutedGenericsProcessor has not been injected into FdbUpdateReportServiceImpl.");
        }

        if (customUpdateService == null) {
            throw new Exception("CustomUpdateServiceImpl has not been injected into FdbUpdateReportServiceImpl.");
        }
    }

    /**
     * Gets the ddiReportProcessor.
     * 
     * @return the ddiReportProcessor
     */
    public DrugDrugInteractionComparisonReportProcessor getDdiReportProcessor() {
        return ddiReportProcessor;
    }

    /**
     * Sets the ddiReportProcessor.
     * 
     * @param ddiReportProcessor
     *            the ddiReportProcessor to set
     */
    public void setDdiReportProcessor(DrugDrugInteractionComparisonReportProcessor ddiReportProcessor) {
        this.ddiReportProcessor = ddiReportProcessor;
    }

    /**
     * Gets the dtReportProcessor.
     * 
     * @return the dtReportProcessor
     */
    public DuplicateTherapyComparisonReportProcessor getDtReportProcessor() {
        return dtReportProcessor;
    }

    /**
     * Sets the dtReportProcessor.
     * 
     * @param dtReportProcessor
     *            the dtReportProcessor to set
     */
    public void setDtReportProcessor(DuplicateTherapyComparisonReportProcessor dtReportProcessor) {
        this.dtReportProcessor = dtReportProcessor;
    }

    /**
     * Gets the drReportProcessor.
     * 
     * @return the drReportProcessor
     */
    public DoseRangeComparisonReportProcessor getDrReportProcessor() {
        return drReportProcessor;
    }

    /**
     * Sets the drReportProcessor.
     * 
     * @param drReportProcessor
     *            the drReportProcessor to set
     */
    public void setDrReportProcessor(DoseRangeComparisonReportProcessor drReportProcessor) {
        this.drReportProcessor = drReportProcessor;
    }

}
