package gov.va.med.pharmacy.pecs.service.fdbreports.impl;

import gov.va.med.pharmacy.pecs.exception.CustomizationServiceException;
import gov.va.med.pharmacy.pecs.fdbreports.FdbColumnUpdate;
import gov.va.med.pharmacy.pecs.fdbreports.FdbUpdateParser;
import gov.va.med.pharmacy.pecs.fdbreports.FdbUpdateRecord;
import gov.va.med.pharmacy.pecs.persistence.fdb.RoutedGenericFdbDao;
import gov.va.med.pharmacy.pecs.service.DrugPairService;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.util.HashMap;
import java.util.Map;

import javax.annotation.Resource;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.stereotype.Component;

/**
 * Handles the processing of Routed Generics. Basically, this processor is looking for adds and deletions to Routed Generics in
 * FDB. For a delete, the Routed Generic data will be saved in to the PECS schema for historical purposes. For Adds, the Routed
 * Generics in the PECS schema will be checked to see if a previously deleted FDB Routed Generic is being added back. If so, the
 * entry will be deleted from the PECS schema.
 * 
 * @author vhaisaduffg
 * 
 */
@Component
public final class RoutedGenericsProcessor implements InitializingBean {
    private FdbUpdateDataAugmentation aug;
    @Resource
    private RoutedGenericFdbDao fdbDao;

    @Resource
    private DrugPairService drugPairService;

    private final Map<Long, String> addedRoutedGenerics = new HashMap<Long, String>();

    private static final Log LOGGER = LogFactory.getLog(RoutedGenericsProcessor.class);

    private static final String ROUTED_GENERIC_KEY = "rtgenid";
    private static final String DESCRIPTION_KEY = "description1";

    private static final String NEW_DESCRIPTION_IND = " *";

    /**
     * For now, Routed Generics that are to be deleted from FDB are going to be stored in CTSTAGING for historical purposes. So
     * no report will be generated by this processing.
     * 
     * @param parser
     *            FdbUpdateParser
     * @param fileDate
     *            the date of the file being processed
     * @throws Exception
     *             if an error is encountered
     */
    public void processRoutedGenerics(FdbUpdateParser parser, String fileDate) throws Exception {
        try {
            File f = parser.parseConcept(FdbUpdateParser.FDB_GENERIC_ROUTEDDRUG);
            ObjectInputStream in = new ObjectInputStream(new FileInputStream(f));
            aug = new RoutedGenericUpdateDataAugmentation(drugPairService);
            aug.setInput(in);
            setDataAugmentation(aug);
            process();
        } catch (Exception ex) {
            String msg = "Failed to process Routed Generic inside processRoutedGenerics.";
            LOGGER.error(msg, ex);
            throw new CustomizationServiceException(msg, ex);
        }
    }

    /**
     * Processes the records retrieved from the FDB Incremental Update looking for adds and deletes. Changes are ignored.
     * 
     * @throws Exception
     *             if an error occurs
     */
    protected final void process() throws Exception {
        FdbUpdateRecord rec;
        while ((rec = aug.nextRecord()) != null) {
            if (rec.getType().equals(FdbUpdateRecord.REC_TYPE_ADD)) {
                processAdd(rec);
            }
        }
    }

    /**
     * Controls the processing of an Add to the FDB Routed Generic. The Routed Generic in the PECS schema will be checked to see
     * if it exists. If it does exist, the entry will be deleted.
     * 
     * @param rec
     *            data augmentation record
     */
    private void processAdd(FdbUpdateRecord rec) {
        Long rtgenid = findRtGenId(rec);
        addedRoutedGenerics.put(rtgenid, findDescription(rec));
    }

    /**
     * Returns the Routed Generic Identifier from the data augmentation record.
     * 
     * @param rec
     *            data augmentation record
     * @return Routed Generic Identifier
     */
    private Long findRtGenId(FdbUpdateRecord rec) {
        FdbColumnUpdate col = rec.getValue(rec.getTable().getColumn(ROUTED_GENERIC_KEY));
        return (Long) col.getCurrentValueAsObject();
    }

    /**
     * Returns the Routed Generic Description from the data augmentation record. The NEW_DESCRIPTION_IND is appended to the
     * description when it is returned.
     * 
     * @param rec
     *            data augmentation record
     * @return Routed Generic Identifier
     */
    private String findDescription(FdbUpdateRecord rec) {
        FdbColumnUpdate col = rec.getValue(rec.getTable().getColumn(DESCRIPTION_KEY));
        return col.getCurrentValue() + NEW_DESCRIPTION_IND;
    }

    /**
     * Searches the Routed Generics for the specified RTGENID and returns the description. If the RTGENID is not found, a null
     * value will be returned.
     * 
     * @param rtGenId
     *            the Routed Generic Identifier
     * @return Description
     * @throws IOException
     *             if a problem occurs reading the file
     */
    public final String getDescription(Long rtGenId) throws IOException {
        String foundDescription = null;
        if (addedRoutedGenerics.containsKey(rtGenId)) {
            foundDescription = addedRoutedGenerics.get(rtGenId);
        }
        return foundDescription;
    }

    /**
     * Sets the RoutedGenericDao to use.
     * 
     * @param dao
     *            RoutedGenericDao
     */
    public final void setRoutedGenericFdbDao(RoutedGenericFdbDao dao) {
        this.fdbDao = dao;
    }

    /**
     * Sets the FdbUpdateDataAugmentation object.
     * 
     * @param augmentation
     *            FdbUpdateDataAugmentation
     */
    public final void setDataAugmentation(FdbUpdateDataAugmentation augmentation) {
        this.aug = augmentation;
    }

    /**
     * Checks to make sure all Spring beans have been injected after object creation.
     * 
     * @throws Exception
     *             if a bean has not been injected
     */
    @Override
    public final void afterPropertiesSet() throws Exception {
        if (fdbDao == null) {
            throw new Exception("RoutedGenericFdbDao has not been injected in to the RoutedGenericsProcessor.");
        }
        if (drugPairService == null) {
            throw new Exception("DrugPairService has not been injected into FdbUpdateReportServiceImpl.");
        }
    }
}
