package gov.va.med.pharmacy.pecs.service.impl;

import gov.va.med.pharmacy.pecs.criteria.ConceptHistoryCriteria;
import gov.va.med.pharmacy.pecs.domain.Customizable;
import gov.va.med.pharmacy.pecs.domain.UserComment;
import gov.va.med.pharmacy.pecs.domain.impl.va.AbstractVersion;
import gov.va.med.pharmacy.pecs.domain.impl.va.ConceptXref;
import gov.va.med.pharmacy.pecs.domain.impl.va.UserProfile;
import gov.va.med.pharmacy.pecs.dto.PecsDTO;
import gov.va.med.pharmacy.pecs.exception.CustomizationServiceException;
import gov.va.med.pharmacy.pecs.exception.ValidationException;
import gov.va.med.pharmacy.pecs.metadata.ConceptField;
import gov.va.med.pharmacy.pecs.metadata.ConceptMetadata;
import gov.va.med.pharmacy.pecs.persistence.ConceptXrefDAO;
import gov.va.med.pharmacy.pecs.persistence.FdbHibernateDAOImpl;
import gov.va.med.pharmacy.pecs.persistence.HibernateDAO;
import gov.va.med.pharmacy.pecs.query.ConceptQuery;
import gov.va.med.pharmacy.pecs.query.ConceptQueryFactory;
import gov.va.med.pharmacy.pecs.query.CurrentRecordsVisitor;
import gov.va.med.pharmacy.pecs.query.ExcelFormatter;
import gov.va.med.pharmacy.pecs.query.Page;
import gov.va.med.pharmacy.pecs.query.QueryConditionVisitor;
import gov.va.med.pharmacy.pecs.query.QueryTypeVisitor;
import gov.va.med.pharmacy.pecs.rules.BusinessRulesValidation;
import gov.va.med.pharmacy.pecs.rules.ValidationResult;
import gov.va.med.pharmacy.pecs.rules.validation.GenericBusinessRules;
import gov.va.med.pharmacy.pecs.service.MetadataService;
import gov.va.med.pharmacy.pecs.service.PecsService;
import gov.va.med.pharmacy.pecs.service.RequestCountSupport;
import gov.va.med.pharmacy.pecs.service.TerminologyService;
import gov.va.med.pharmacy.pecs.service.dto.FieldHistoryLineItem;
import gov.va.med.pharmacy.pecs.state.PecsStateMachine;
import gov.va.med.pharmacy.pecs.state.PecsStateMachineFactory;
import gov.va.med.pharmacy.pecs.state.Status;

import java.io.File;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.annotation.Resource;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.BindingResult;
import org.springframework.validation.ObjectError;

/**
 * Superclass for PECS services.
 * 
 * @param <D>
 *            the DTO class (can be the same as the concept class)
 * @param <C>
 *            the class representing the concept that is being customized
 */
public abstract class AbstractPecsService<D extends PecsDTO, C> implements PecsService<D>, InitializingBean {

    /** The Constant LOGGER. */
    private static final Logger LOGGER = Logger.getLogger(AbstractPecsService.class);

    /** The state machine factory. */
    @Resource
    private PecsStateMachineFactory stateMachineFactory;

    /** The formatter. */
    private ExcelFormatter<D> formatter = new ExcelFormatter<D>();

    /** The generic data validation. */
    private BusinessRulesValidation genericDataValidation;

    /** The generic status change validation. */
    private BusinessRulesValidation genericStatusChangeValidation;

    /** The metadata service. */
    @Resource
    private MetadataService metadataService;

    /** The metadata. */
    private ConceptMetadata metadata;

    /** The Constant TIMEOUT. */
    private static final int TIMEOUT = 1200;

    /** The terminology service. */
    @Resource
    private TerminologyService terminologyService;

    /** The concept xref dao. */
    @Resource
    private ConceptXrefDAO conceptXrefDAO;

    /** The concept query factory. */
    @Resource(name = "jsonConceptQueryFactory")
    private ConceptQueryFactory conceptQueryFactory;

    /*
     * ResourceBundle messages = ResourceBundle.getBundle("pecs-service-messages");
     * 
     * public String getMessage(String message, Object[] args) {
     * 
     * MessageFormat formatter = new MessageFormat("");
     * 
     * formatter.applyPattern(messages.getString(message)); String formattedMessage = formatter.format(args);
     * 
     * return formattedMessage;
     * 
     * }
     * 
     * public void addMessageToBindingResult(String className, String message, Object[] args, BindingResult results) {
     * 
     * String formattedMessage = getMessage(message, args);
     * 
     * results.addError(new ObjectError(className, formattedMessage)); }
     */

    /**
     * Gets the conceptXrefDAO.
     * 
     * @return the conceptXrefDAO
     */
    public ConceptXrefDAO getConceptXrefDAO() {
        return conceptXrefDAO;
    }

    /**
     * Sets the Concept Xref user comment dao.
     * 
     * @param conceptXrefDAO
     *            the conceptXrefDAO
     */
    public void setConceptXrefDAO(ConceptXrefDAO conceptXrefDAO) {
        this.conceptXrefDAO = conceptXrefDAO;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.springframework.beans.factory.InitializingBean#afterPropertiesSet()
     */
    @Override
    public void afterPropertiesSet() throws Exception {
        if (terminologyService == null) {
            throw new Exception("terminologyService must be set.");
        }
        metadata = getMetadata();
        genericDataValidation = GenericBusinessRules.getGenericDataValidation(metadata);
        genericStatusChangeValidation = GenericBusinessRules.getGenericStatusChangeValidation(metadata);
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.service.PecsService#getStagingData(gov.va.med.pharmacy.pecs.query.ConceptQuery, int)
     */
    @Override
    @Transactional(readOnly = true)
    public final Page getStagingData(ConceptQuery query, int page) {
        if (query.isConditionSet()) {
            query = preProcessQuery(query, true);
            if (!query.isConditionSet()) {
                return getEmptyPage(page);
            }
        }
        return getStagingData(query.toHQL(true), page);
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.service.PecsService#getStagingData(gov.va.med.pharmacy.pecs.query.ConceptQuery)
     */
    @Override
    @Transactional(readOnly = true)
    public final List<D> getStagingData(ConceptQuery query) {
        ConceptQuery preProcessedQuery = preProcessQuery(query, true);
        return doGetStagingData(preProcessedQuery);
    }

    /**
     * Gets Staging Data in a List Object.
     * 
     * @param query
     *            - ConceptyQuery
     * @return - Page
     */
    protected abstract List<D> doGetStagingData(ConceptQuery query);

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.service.PecsService#getFdbData(gov.va.med.pharmacy.pecs.query.ConceptQuery, int)
     */
    @Override
    @Transactional(readOnly = true)
    public final Page getFdbData(ConceptQuery query, int page) {
        if (query.isConditionSet()) {
            query = preProcessQuery(query, false);
            if (!query.isConditionSet()) {
                return getEmptyPage(page);
            }
        }
        return getFdbData(query.toHQL(false), page);
    }

    /**
     * Get Empty Page.
     * 
     * @param pageNo
     *            the page number
     * @return Page
     */
    private Page getEmptyPage(int pageNo) {
        return new Page(null, pageNo, Page.DEFAULT_SIZE);
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.service.PecsService#getFdbData(java.lang.String, int)
     */
    @Override
    @Transactional(readOnly = true)
    public final Page getFdbData(String query, int page) {
        if (query == null || query.equals(""))
            throw new IllegalArgumentException("required query parameter is null");

        if (page < 1)
            throw new IllegalArgumentException("required page parameter cannot be less than one");
        Page pg = getDao(false).findByQueryByPage(query, page);
        List<D> dtos = convertFdbToCustom(pg.getResults());
        pg.setResults(dtos);
        return pg;
    }

    /** 
     * Gets the FDB Date for the specified query.
     * @see gov.va.med.pharmacy.pecs.service.PecsService#getFdbData(gov.va.med.pharmacy.pecs.query.ConceptQuery)
     */
    @Transactional(readOnly = true)
    public List<D> getFdbData(ConceptQuery query) {
        if (LOGGER.is   ugEnabled())
            LOGGER.debug("Performing lookup using query in Service Impl layer");
    
        if (query == null)
            throw new IllegalArgumentException("required query parameter is null");
        
        if (query.isConditionSet()) {
            query = preProcessQuery(query, false);
            if (!query.isConditionSet()) { 
                return new ArrayList<D>();
            }
        }
    
        List<?> fdbObjects = getFdbDao().findByQuery(query, false);
        return convertFdbToCustom(fdbObjects);
    }

    /**
     * List object for convertedFdbToCustom.
     * 
     * @param fdbObjects
     *            - List<?>
     * @return fdbObjects
     */
    protected abstract List<D> convertFdbToCustom(List<?> fdbObjects);

    /**
     * Perform the logic for applying the action.
     * 
     * @param concept
     *            the concept type the action is being performed on
     * @param newData
     *            the new object
     * @param oldData
     *            the old object
     * @param action
     *            the action to apply
     * @param user
     *            the user performing the action
     * @param err
     *            populated with any errors found
     * @return whether applying the action involved the state machine
     */
    protected final boolean doApplyAction(Customizable<C> concept, D newData, D oldData, String action, UserProfile user,
            BindingResult err) {
        try {
            validateAction(newData, oldData, action);
        } catch (ValidationException e) {
            for (String msg : e.getMessages()) {
                err.addError(new ObjectError(newData.getClass().getSimpleName(), msg));
            }
            return false;
        }
        if (!noStatusChange(newData, oldData, action)) {
            PecsStateMachine sm = getStateMachineFactory().getStateMachine();
            configureStateMachine(sm, concept, user);
            try {
                sm.applyAction(action);
            } catch (Exception e) {
                String message = "Failed to apply action '" + action + "' to " + concept;
                LOGGER.error(message, e);
                throw new CustomizationServiceException(message, e);
            }
            return true;
        }

        return false;
    }

    /**
     * Configure State Machine.
     * 
     * @param sm
     *            - PecsStateMachine
     * @param concept
     *            - Customizable
     * @param user
     *            - UserProfile
     */
    protected final void configureStateMachine(PecsStateMachine sm, Customizable<C> concept, UserProfile user) {
        sm.setConcept(concept);
        sm.setUser(user);
    }

    /**
     * Validate New Record.
     * 
     * @param data
     *            - D
     * @param err
     *            - BindingResult
     * @return false
     */
    protected final boolean validateNewRecord(D data, BindingResult err) {
        try {
            validateAction(data, data, "New");
        } catch (ValidationException e) {
            for (String msg : e.getMessages()) {
                err.addError(new ObjectError("dpCustomization", msg));
            }
            return false;
        }
        return true;
    }

    /**
     * Retrieve a delimited file representing the result set of the executed query given in the query parameter. The file
     * format, including delimiter, file name, file extension and the option to include column names in the file depend on
     * setting class field members, or accepting default values when the class is instantiated. A null value will be returned
     * if:
     * <ul>
     * <li>The HQL represented in the query parameter cannot be converted into SQL</li>
     * <li>The result of SQL execution is null</li>
     * <li>Generating delimited content from query execution is null</li>
     * </ul>
     * 
     * 
     * object encapsulating the HQL to be executed
     * 
     * @param json
     *            - String
     * @param filePath
     *            - String path of the result file
     * @param fileName
     *            - String name of the result file
     * @param custom
     *            - boolean
     * @return File representing the result set of the executed query
     */
    @Override
    @Transactional(readOnly = true)
    public File getDelimitedFile(final String json, final String filePath, final String fileName, final boolean custom) {
        File file = null;
        try {
            ConceptMetadata meta = getMetadata();
            ConceptQuery query = new ConceptQuery(meta);

            query = conceptQueryFactory.createQuery(meta, json);

            String hql = getSelectFields(meta, custom) + " " + preProcessQuery(query, custom).toHQL(custom);
            List<Object[]> list = executeQuery(hql, custom, formatter.getMaxRows());
            file = getFormatter().getFile(list, meta, custom, filePath, fileName);
        } catch (Exception ex) {
            String msg = "Failed to perform excel file generation";
            LOGGER.error(msg, ex);
            throw new CustomizationServiceException(msg, ex);
        }

        return file;
    }

    /**
     * Generate a select list containing the property paths of the fields that should be included in a 'table display'. This is
     * used to generate an HQL statement.
     * 
     * @param meta
     *            of type ConceptMetadata
     * @param custom
     *            - boolean true if custom
     * @return String
     */
    protected final String getSelectFields(ConceptMetadata meta, boolean custom) {
        StringBuffer buf = new StringBuffer();
        for (ConceptField f : meta.getQueryFields(custom)) {
            if (buf.length() == 0) {
                buf.append("select ");
            } else {
                buf.append(", ");
            }
            buf.append(f.getPersistentPath(custom));
        }
        return buf.toString();
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.service.PecsService#getStagingDataCounts(java.lang.String)
     */
    @Override
    @Transactional(readOnly = true)
    public final Map<String, Integer> getStagingDataCounts(String userName) {
        HashMap<String, Integer> counts = new HashMap<String, Integer>();
        RequestCountSupport support = getRequestCountSupport();

        counts.put("REQTOTAL", support.getCountsByRequestorAndStatus(userName, null));
        counts.put("REQAPPROVED", support.getCountsByRequestorAndStatus(userName, Status.STATUS_APPROVED));

        List<String> requiredReviewed = new ArrayList<String>();
        requiredReviewed.add(Status.STATUS_REVIEWED);
        requiredReviewed.add(Status.STATUS_DELETE_REVIEWED);
        counts.put("REQREVIEWED", support.getCountsByRequestorAndStatuses(userName, requiredReviewed));

        counts.put("REQREJECTED", support.getCountsByRequestorAndStatus(userName, Status.STATUS_REJECTED));
        counts.put("REQDELETED", support.getCountsByRequestorAndStatus(userName, Status.STATUS_DELETED));
        counts.put("REQMODIFIED", support.getCountsByRequestorAndStatus(userName, Status.STATUS_MODIFIED));
        counts.put("REQNOACTION", support.getCountsByRequestorAndStatus(userName, Status.STATUS_NEW));
        counts.put("TODOAPPROVE", support.getCountsByAssignedAndStatus(userName, Status.STATUS_REVIEWED, userName));
        List<String> toDoReview = new ArrayList<String>();
        toDoReview.add(Status.STATUS_NEW);
        toDoReview.add(Status.STATUS_MODIFIED);
        counts.put("TODOREVIEW", support.getCountsByAssignedAndStatuses(userName, toDoReview));
        counts.put("TODODELETE", support.getCountsByAssignedAndStatus(userName, Status.STATUS_DELETE_REVIEWED, userName));
        counts.put("UNASSIGNED", support.getCountsByAssignedAndStatus("UNASSIGNED", null));
        counts.put("TOTAPP", support.getCountsByStatus(Status.STATUS_APPROVED));
        counts.put("TOTREV",
                support.getCountsByStatus(Status.STATUS_REVIEWED) + support.getCountsByStatus(Status.STATUS_DELETE_REVIEWED));
        counts.put("TOTREJ", support.getCountsByStatus(Status.STATUS_REJECTED));
        counts.put("TOTDEL", support.getCountsByStatus(Status.STATUS_DELETED));
        counts.put("TOTMOD", support.getCountsByStatus(Status.STATUS_MODIFIED));
        counts.put("TOTNEW", support.getCountsByStatus(Status.STATUS_NEW));
        return counts;
    }

    /**
     * Construct a support class to populate request counts for a particular concept.
     * 
     * Note: this method will become abstract; it's not at this point so it doesn't break the build
     * 
     * @return the initialized helper class
     */
    protected abstract RequestCountSupport getRequestCountSupport();

    /**
     * Retrieves the Validation Results.
     * 
     * @param validator
     *            of type BusinessRulesValidation
     * @param crtValue
     *            object
     * @param prevValue
     *            object
     * @param action
     *            of type String
     * @return List typed as ValidationResults
     */
    private List<ValidationResult> getValidationResults(BusinessRulesValidation validator, Object crtValue, Object prevValue,
            String action) {
        if (crtValue == null) {
            String msg = "Attempting to validate null object";
            LOGGER.error(msg);
            throw new CustomizationServiceException(msg);
        }
        return validator.validate(crtValue, prevValue, action);
    }

    /**
     * Method Validate Action.
     * 
     * @param crtValue
     *            object
     * @param prevValue
     *            object
     * @param action
     *            String
     */
    private void validateAction(Object crtValue, Object prevValue, String action) {
        List<ValidationResult> validationResults = getValidationResults(genericDataValidation, crtValue, prevValue, action);
        BusinessRulesValidation v = getDataValidation(metadata);
        if (v != null) {
            validationResults.addAll(getValidationResults(v, crtValue, prevValue, action));
        }
        ArrayList<String> errorMessages = new ArrayList<String>();
        for (ValidationResult r : validationResults) {
            errorMessages.addAll(r.getMessages());
        }
        if (!errorMessages.isEmpty()) {
            throw new ValidationException(errorMessages);
        }
    }

    /**
     * Method No Status Change.
     * 
     * @param crtValue
     *            object
     * @param prevValue
     *            object
     * @param action
     *            of type String
     * @return boolean true if no messages.
     */
    private boolean noStatusChange(Object crtValue, Object prevValue, String action) {
        List<ValidationResult> validationResults =
                getValidationResults(genericStatusChangeValidation, crtValue, prevValue, action);

        for (ValidationResult r : validationResults) {
            if (!r.getMessages().isEmpty()) {
                return false;
            }
        }
        return true;
    }

    /**
     * Override to provide concept specific data validation rules.
     * 
     * @param meta
     *            the concept metadata
     * @return a validator
     */
    protected abstract BusinessRulesValidation getDataValidation(ConceptMetadata meta);

    /**
     * Pre-process the user query to handle inclusion/exclusion of historical records.
     * 
     * 
     * @param query
     *            the query to process
     * @param custom
     *            boolean - true if custom
     * @return the processed query
     */
    protected ConceptQuery preProcessQuery(ConceptQuery query, boolean custom) {
        ConceptQuery preProcessed = query.copy();
        // eliminate fields that are not supported by the query type (custom or FDB)
        QueryConditionVisitor visitor = new QueryTypeVisitor(custom);
        preProcessed.accept(visitor);

        if (custom && !query.includeHistory()) {
            // restrict to just current records
            visitor = new CurrentRecordsVisitor();
            preProcessed.accept(visitor);
        }
        return preProcessed;
    }

    /**
     * Retrieves the State Machine Factory.
     * 
     * @return PecsStateMachineFactory
     */
    public final PecsStateMachineFactory getStateMachineFactory() {
        return stateMachineFactory;
    }

    /**
     * Sets the State Machine Factory.
     * 
     * @param stateMachineFactory
     *            of type PecsStateMachineFactory
     */
    public final void setStateMachineFactory(PecsStateMachineFactory stateMachineFactory) {
        this.stateMachineFactory = stateMachineFactory;
    }

    /**
     * Sets MetadataService.
     * 
     * @param metadataService
     *            of type MetadataService
     */
    public final void setMetadataService(MetadataService metadataService) {
        this.metadataService = metadataService;
    }

    /**
     * Retrieves the MetadataService.
     * 
     * @return MetadataService
     */
    public final MetadataService getMetadataService() {
        return metadataService;
    }

    /**
     * Method executeQuery.
     * 
     * @param hql
     *            of type String
     * @param custom
     *            of type boolean, true if custom.
     * @param maxResults
     *            - int
     * @return List
     */
    @SuppressWarnings("unchecked")
    protected final List<Object[]> executeQuery(String hql, boolean custom, int maxResults) {
        HibernateDAO<?> dao = getDao(custom);
        org.hibernate.Query query = dao.getSession().createQuery(hql);
        if (maxResults > 0) {
            query.setFirstResult(0).setMaxResults(maxResults);
        }
        return query.list();
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.service.PecsService#getAllowableActions(gov.va.med.pharmacy.ct.domain.staging.PecsDTO,
     * gov.va.med.pharmacy.ct.domain.user.UserProfile)
     */
    @Override
    @Transactional(readOnly = true)
    public final Set<String> getAllowableActions(D record, UserProfile user) {
        Customizable<C> concept = findPersistentObject(record);
        PecsStateMachine sm = getStateMachineFactory().getStateMachine();
        configureStateMachine(sm, concept, user);
        try {
            return sm.getAllowableActions();
        } catch (Exception e) {
            String message = "Failed to determine allowable actions for " + concept;
            LOGGER.error(message, e);
            throw new CustomizationServiceException(message, e);
        }
    }

    /**
     * Retrieves the AllowableActions.
     * 
     * @param concept
     *            - Customizable
     * @param user
     *            - UserProfile
     * @return Set
     */
    @Transactional(readOnly = true)
    public final Set<String> getAllowableActions(Customizable<C> concept, UserProfile user) {
        PecsStateMachine sm = getStateMachineFactory().getStateMachine();
        configureStateMachine(sm, concept, user);
        try {
            return sm.getAllowableActions();
        } catch (Exception e) {
            String message = "Failed to determine allowable actions for " + concept;
            LOGGER.error(message, e);
            throw new CustomizationServiceException(message, e);
        }
    }

    /**
     * Method findPersistentObject.
     * 
     * @param record
     *            -
     * @return Customizable
     */
    protected abstract Customizable<C> findPersistentObject(D record);

    /**
     * Getter for Formatter.
     * 
     * @return the formatter
     */
    public final ExcelFormatter<D> getFormatter() {
        return formatter;
    }

    /**
     * Gets the terminology service.
     * 
     * @return the terminology service
     */
    public final TerminologyService getTerminologyService() {
        return terminologyService;
    }

    /**
     * Sets the terminology service.
     * 
     * @param terminologyService
     *            the new terminology service
     */
    public final void setTerminologyService(TerminologyService terminologyService) {
        this.terminologyService = terminologyService;
    }

    /**
     * Setter for formatter.
     * 
     * @param formatter
     *            the formatter to set
     */
    public final void setFormatter(ExcelFormatter<D> formatter) {
        this.formatter = formatter;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * gov.va.med.pharmacy.pecs.service.PecsService#getRequiredHistory(gov.va.med.pharmacy.pecs.criteria.ConceptHistoryCriteria)
     */
    @Override
    @Transactional(readOnly = true)
    public final Map<String, List<FieldHistoryLineItem>> getRequiredHistory(ConceptHistoryCriteria criteria) {

        List<ConceptField> fields = getMetadata().getFields();

        Map<String, List<FieldHistoryLineItem>> historyMap = new HashMap<String, List<FieldHistoryLineItem>>();

        for (ConceptField conceptField : fields) {

            if (conceptField.isRequired()) {

                List<FieldHistoryLineItem> fieldHistoryLineItems = getHistoryForField(criteria, conceptField.getName());

                if (!fieldHistoryLineItems.isEmpty()) {

                    historyMap.put(conceptField.getName(), fieldHistoryLineItems);
                }
            }

        }

        return historyMap;
    }

    /**
     * Method to retrieve History for the field.
     * 
     * @param criteria
     *            - the criteria to use in search
     * @param fieldName
     *            - the field name
     * @return FieldHistoryLineItem
     */
    @SuppressWarnings({ "rawtypes", "unchecked" })
    @Transactional(readOnly = true)
    private final List<FieldHistoryLineItem> getHistoryForField(ConceptHistoryCriteria criteria, String fieldName) {

        ConceptField conceptField = getMetadata().getField(fieldName);

        List<FieldHistoryLineItem> fieldHistoryLineItems = new ArrayList<FieldHistoryLineItem>();

        List<AbstractVersion> list = (List<AbstractVersion>) getCustomHistoryForField(criteria);

        if (list.size() < 1) {
            return Collections.emptyList();
        }

        if (isBaseLineStatus(list.get(0).getStatus())) {

            return Collections.emptyList();
        }

        Object retainedFieldValue = new Object();
        int listSize = list.size();
        int i = 1;

        for (AbstractVersion versionVa : list) {
            Object fieldValue = conceptField.getPersistentValue(true, versionVa);

            if (!retainedFieldValue.equals(fieldValue)) {
                FieldHistoryLineItem fieldHistoryLineItem = populateFieldHistoryLineItem(versionVa, conceptField);
                fieldHistoryLineItems.add(fieldHistoryLineItem);
                retainedFieldValue = fieldValue;
            }

            if (Status.STATUS_NEW.equals(versionVa.getStatus().getName()) && (listSize == i)) {

                FieldHistoryLineItem fdbfieldHistoryLineItem =
                        getFDBHistoryforField(criteria, conceptField, retainedFieldValue);
                if (fdbfieldHistoryLineItem != null) {
                    fieldHistoryLineItems.add(fdbfieldHistoryLineItem);
                }

            }

            if (isBaseLineStatus(versionVa.getStatus())) {
                break;

            }

            i++;
        }

        if (fieldHistoryLineItems.size() == 1) {
            return Collections.emptyList();
        }

        return fieldHistoryLineItems;
    }

    /**
     * Determine field value.
     * 
     * @param conceptField
     *            the concept field
     * @param versionVa
     *            the version va
     * @return the object
     */
    private Object determineFieldValue(ConceptField conceptField, AbstractVersion<?> versionVa) {

        Object code = conceptField.getPersistentValue(true, versionVa);
        String codeString = null;
        if (code instanceof Long) {
            codeString = code.toString();
        } else {
            codeString = (String) code;
        }
        return getTerminologyService().getByCode(conceptField.getLookupType(), codeString).getUiLabel();

    }

    /**
     * Checks if is base line status.
     * 
     * @param <T>
     *            - child of AbstractVersion
     * @param status
     *            -
     * @return boolean - true if Status is approved or delete reviewed
     */
    private <T> boolean isBaseLineStatus(Status status) {
        if (status != null) {
            return Status.STATUS_APPROVED.equals(status.getName()) || Status.STATUS_DELETED.equals(status.getName());
        }
        return false;

    }

    /**
     * Method to populate the DTO FieldHistoryLineItem from the Concept(s) Custom version object.
     * 
     * @param versionVa
     *            - child of AbstractVersion
     * @param conceptField
     *            the concept field
     * @return FieldHistoryLineItem
     */
    @SuppressWarnings("rawtypes")
    protected final FieldHistoryLineItem populateFieldHistoryLineItem(AbstractVersion versionVa, ConceptField conceptField) {

        Object fieldValue = null;
        if (conceptField.isLookup()) {

            fieldValue = determineFieldValue(conceptField, versionVa);

        } else {

            fieldValue = conceptField.getPersistentValue(true, versionVa);
        }

        FieldHistoryLineItem fieldHistoryLineItem = new FieldHistoryLineItem();
        fieldHistoryLineItem.setDate(versionVa.getDate());
        fieldHistoryLineItem.setUser(versionVa.getActionBy());
        fieldHistoryLineItem.setValue(fieldValue.toString());
        fieldHistoryLineItem.setId(Long.toString(versionVa.getVersionId()));
        fieldHistoryLineItem.setStatus(versionVa.getStatus().getName());
        return fieldHistoryLineItem;
    }

    /**
     * Method to override in the extension classes.
     * 
     * @param criteria
     *            - the criteria to use in search.
     * 
     * @return List<AbstractVersion>
     */

    @SuppressWarnings("rawtypes")
    protected abstract List<? extends AbstractVersion> getCustomHistoryForField(ConceptHistoryCriteria criteria);

    /**
     * Method to process FDB History for get history for field.
     * 
     * @param criteria
     *            - the criteria to use in search
     * @param conceptField
     *            -
     * @param retainedFieldValue
     *            -
     * @return FieldHistoryLineItem populated with FDB information
     */
    protected abstract FieldHistoryLineItem getFDBHistoryforField(ConceptHistoryCriteria criteria, ConceptField conceptField,
            Object retainedFieldValue);

    /**
     * Method to create a new user comment.
     * 
     * @param fdbId
     *            the fdb id
     * @param comment
     *            the IUserComment
     * @param bindingResult
     *            BindingResult object for errors
     */
    @Override
    @Transactional
    public final void createUserComment(Object fdbId, UserComment comment, BindingResult bindingResult) {

        try {
            ConceptXref xref = findConceptXref(fdbId);
            if (xref == null) {
                xref = createConceptXref(fdbId, comment);
            } else {
                xref.addUserComment(comment);
            }
            persistConceptXref(xref);

        } catch (RuntimeException re) {
            bindingResult.addError(new ObjectError("PecsService", re.getMessage()));
        }

    }

    /**
     * Method to create a new user comment.
     * 
     * @param fdbId
     *            the fdb id
     * @return the concept xref
     */
    @Transactional
    protected final ConceptXref findConceptXref(Object fdbId) {

        try {
            ConceptXref xref = findXrefByFDBId(fdbId);
            return xref;
        } catch (RuntimeException re) {
            String message = "Failed to retrieve ConceptXref";
            LOGGER.error(message, re);
            throw new CustomizationServiceException(message, re);
        }
    }

    /**
     * Persists the concept xref.
     * 
     * @param xref
     *            the xref
     */
    @Transactional
    protected final void persistConceptXref(ConceptXref xref) {

        ConceptXrefDAO dao = getConceptXrefDAO();
        try {
            dao.save(xref);
        } catch (RuntimeException re) {
            String message = "Failed to persist ConceptXref";
            LOGGER.error(message, re);
            throw new CustomizationServiceException(message, re);
        }

    }

    /**
     * Creates the concept xref for customization.
     * 
     * @param fdbId
     *            the fdbId
     * @return the concept xref
     */
    protected abstract ConceptXref createConceptXrefForCustomization(Object fdbId);

    /**
     * Creates the concept xref.
     * 
     * @param fdbId
     *            the fdb id
     * @param comment
     *            the comment
     * @return the concept xref
     */
    public abstract ConceptXref createConceptXref(Object fdbId, UserComment comment);

    /**
     * This method gets the proper fdb cross reference. This should be implemented by the individual services. If the cross
     * reference does not exist the method needs to return a new instance of the cross reference
     * 
     * @param fdbId
     *            the fdb id
     * @return the concept xref
     */
    public abstract ConceptXref findXrefByFDBId(Object fdbId);

    /**
     * Returns the ConceptQueryFactory implementation currently in use.
     * 
     * @return ConceptQueryFactory
     */
    public ConceptQueryFactory getConceptQueryFactory() {
        return conceptQueryFactory;
    }

    /**
     * Sets the ConceptQueryFactory implementation to use.
     * 
     * @param conceptQueryFactory
     *            an implementation of ConceptQueryFactory
     */
    public void setConceptQueryFactory(ConceptQueryFactory conceptQueryFactory) {
        this.conceptQueryFactory = conceptQueryFactory;
    }
    
    /**
     * Gets the FDB DAO that will provide the FDB persistence object of this concept.
     * @return
     */
    protected abstract FdbHibernateDAOImpl<?> getFdbDao();

}
