package gov.va.med.pharmacy.pecs.service.impl;

import gov.va.med.pharmacy.pecs.customupdate.BaseUpdateFileSupport;
import gov.va.med.pharmacy.pecs.customupdate.CtVersionVa;
import gov.va.med.pharmacy.pecs.customupdate.CustomUpdateDAO;
import gov.va.med.pharmacy.pecs.customupdate.CustomUpdateFileVa;
import gov.va.med.pharmacy.pecs.customupdate.CustomUpdateRecordCounts;
import gov.va.med.pharmacy.pecs.customupdate.DoseRangeUpdateFileSupport;
import gov.va.med.pharmacy.pecs.customupdate.DrugDrugInteractionStringsUpdateFileSupport;
import gov.va.med.pharmacy.pecs.customupdate.DrugDrugInteractionUpdateFileSupport;
import gov.va.med.pharmacy.pecs.customupdate.DrugPairUpdateFileSupport;
import gov.va.med.pharmacy.pecs.customupdate.DuplicateTherapyUpdateFileSupport;
import gov.va.med.pharmacy.pecs.customupdate.FdbVersion;
import gov.va.med.pharmacy.pecs.customupdate.FdbVersionDAO;
import gov.va.med.pharmacy.pecs.customupdate.MonographUpdateFileSupport;
import gov.va.med.pharmacy.pecs.customupdate.RecordCounts;
import gov.va.med.pharmacy.pecs.domain.impl.va.MonographPairDAO;
import gov.va.med.pharmacy.pecs.exception.CustomizationServiceException;
import gov.va.med.pharmacy.pecs.exception.FTPException;
import gov.va.med.pharmacy.pecs.net.FileTransferCommands;
import gov.va.med.pharmacy.pecs.net.FileTransferFactory;
import gov.va.med.pharmacy.pecs.net.FtpClient;
import gov.va.med.pharmacy.pecs.service.CustomUpdateService;
import gov.va.med.pharmacy.pecs.service.util.MessageHelper;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Properties;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import javax.annotation.Resource;

import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * Update File service provides routines that enable user to: Create an update file record in the database that includes all
 * 'active/approved' data as of that date Retrieve a previously created update file.
 */
@Service
public final class CustomUpdateServiceImpl implements CustomUpdateService, InitializingBean {

    // private static final String PECS_SERVICE_DRUGPAIR_SAVE_ERROR = "pecs.service.drugpair.save.error";
    /** The Constant PECS_FILE_PATH_ERROR. */
    private static final String PECS_FILE_PATH_ERROR = "pecs.service.customupdateservice.generate.filepath.error";

    /** The Constant LOGGER. */
    private static final Logger LOGGER = Logger.getLogger(CustomUpdateServiceImpl.class);

    /** The Constant FIELD_SEPARATOR. */
    private static final String FIELD_SEPARATOR = "|";

    /** The Constant VERSION_KEY. */
    private static final Long VERSION_KEY = new Long(1);

    /** The Constant NEW_LINE_STR. */
    private static final String NEW_LINE_STR = "\r\n";

    /** The Constant ZIP_FILE_PREFIX. */
    private static final String ZIP_FILE_PREFIX = "CstmUpdFile";

    /** The Constant ZIP_FILE_EXTENSION. */
    private static final String ZIP_FILE_EXTENSION = ".ZIP";

    /** The Constant ZIP_FILE_DELIMITER. */
    private static final String ZIP_FILE_DELIMITER = "_";

    /** The Constant FIELDDEFREC. */
    private static final String FIELDDEFREC =
            "F|7|1|versionkey|2|dbversion|3|buildversion|4|frequency|5|issuedate|6|versioncomment|7|dbtype" + NEW_LINE_STR;
    /** String for Control file name. */
    public static final String CONTROL_FILE_NAME = "FDBUPDCONTROL.DAT";
    /** String for file counts file name. */
    public static final String FILE_COUNTS_FILE_NAME = "FILECOUNTS.DAT";
    /** String for incremental file extension. */
    public static final String INCREMENTAL_FILE_EXTENSION = ".UPD";
    /** String for Full Update File Extension. */
    public static final String FULL_UPDATE_FILE_EXTENSION = ".TXT";
    /** String for Version File Name. */
    public static final String VERSION_FILE_NAME = "CTVERSION";
    /** String for Proddefinition file name. */
    public static final String PRODDEFINITION_FILE_NAME = "PRODDEFINITION.XML";
    /** String for Export file dir property. */
    public static final String EXPORT_FILE_DIR_PROPERTY = "export.file.dir";
    /** String for FDB File Dir Property. */
    // public static final String FDB_FILE_DIR_PROPERTY = "fdb.file.dir";
    /** String for Export File Server Property. */
    public static final String EXPORT_FILE_SERVER_PROPERTY = "export.file.server";
    /** String for export user name property. */
    public static final String EXPORT_USER_NAME_PROPERTY = "export.user.name";
    /** String for export password property. */
    public static final String EXPORT_PASSWORD_PROPERTY = "export.password";

    /** The Constant CT_BUILDVERSION. */
    private static final String CT_BUILDVERSION = "1";

    /** The Constant DB_TYPE. */
    private static final String DB_TYPE = "USA";

    /** The Constant CT_DBVERSION. */
    private static final String CT_DBVERSION = "3.3";

    /** The Constant CT_FREQUENCY. */
    private static final String CT_FREQUENCY = "W";

    /** The Constant FDB_VERSIONKEY. */
    private static final long FDB_VERSIONKEY = 1L;

    /** The monograph update file support. */
    @Resource
    private MonographUpdateFileSupport monographUpdateFileSupport;

    /** The duplicate therapy update file support. */
    @Resource
    private DuplicateTherapyUpdateFileSupport duplicateTherapyUpdateFileSupport;

    /** The dose range update file support. */
    @Resource
    private DoseRangeUpdateFileSupport doseRangeUpdateFileSupport;

    /** The drug drug interaction strings update file support. */
    @Resource
    private DrugDrugInteractionStringsUpdateFileSupport drugDrugInteractionStringsUpdateFileSupport;

    /** The drug drug interaction update file support. */
    @Resource
    private DrugDrugInteractionUpdateFileSupport drugDrugInteractionUpdateFileSupport;

    /** The drug pair update file support. */
    @Resource
    private DrugPairUpdateFileSupport drugPairUpdateFileSupport;

    /** The custom update dao. */
    @Resource
    private CustomUpdateDAO customUpdateDAO;

    /** The fdb version dao. */
    @Resource
    private FdbVersionDAO fdbVersionDAO;

    /** The monograph pair dao. */
    @Resource
    private MonographPairDAO monographPairDAO;

    /** The export file properties. */
    @Resource(name = "exportFileProperties")
    private Properties exportFileProperties = null;

    /** The message helper. */
    @Resource
    private MessageHelper messageHelper;

    /** The file transfer factory. */
    @Resource
    private FileTransferFactory fileTransferFactory;

    /**
     * Get File Name.
     * 
     * @param tableName
     *            - String
     * @param isFull
     *            - boolean
     * @return fileName.toString()
     */
    private String getFileName(String tableName, boolean isFull) {
        StringBuffer fileName = new StringBuffer(tableName);
        if (isFull) {
            fileName.append(FULL_UPDATE_FILE_EXTENSION);
        } else {
            fileName.append(INCREMENTAL_FILE_EXTENSION);
        }
        return fileName.toString();
    }

    /**
     * Create a string that can be used in the H record of the data files. It used used to properly sequence incremental
     * updates, by making sure that the issue date of the previous update in the target database (table CT_VERSION) is the
     * expected one, based on the version history maintained in PECS.
     * 
     * @param previousVersion
     *            - CtVersionVa
     * @return a properly formatted string that can be used in the V record
     */
    private String getPreviousVersionStr(CtVersionVa previousVersion) {
        if (previousVersion == null) {
            return null;
        }
        String versionStr =
                previousVersion.getCtDbversion() + FIELD_SEPARATOR + previousVersion.getCtFrequency() + FIELD_SEPARATOR
                        + formatDateCustomUpdateFileHeader(previousVersion.getCtIssuedate()) + FIELD_SEPARATOR
                        + previousVersion.getCtDbtype();

        return versionStr;
    }

    /**
     * Format date custom update file header.
     * 
     * @param date
     *            the date
     * @return the string
     */
    private String formatDateCustomUpdateFileHeader(Date date) {
        DateFormat dateFormatted = new SimpleDateFormat("yyyyMMdd");
        return dateFormatted.format(date);
    }

    /**
     * This method creates a ZipEntry object with the name specified in the fileName parameter, and appends it to the output
     * stream.
     * 
     * @param fileName
     *            String name of the new ZipEntry object
     * @param zout
     *            The output stream that a new ZipEntry object will be added to.
     * @return Output stream with a new ZipEntry object.
     */
    private ZipOutputStream createZipEntry(String fileName, ZipOutputStream zout) {
        if (fileName == null || "".equals(fileName) || zout == null)
            throw new CustomizationServiceException(
                    "A required parameter is null or undefined for the creation of a zip file entry. ");

        try {
            zout.putNextEntry(new ZipEntry(fileName));
        } catch (IOException ioe) {
            String msg = "An IO exception has occured while attempting to add a zip entry";
            LOGGER.error(msg, ioe);
            throw new CustomizationServiceException(msg, ioe);
        }

        return zout;
    }

    /**
     * Create the UPD Control file as defined by FDB
     * 
     * The control file must be named fdbupdcontrol.dat. Records in the control file must appear in the following sequence: 1
     * Version Control Record (record type V) 1 Field Definition Record (record type F) 1 Primary Key Definition Record (record
     * type P) 1 Beginning Version Change Record (record type B) 1 Ending Version Change Record (record type E)
     * 
     * Sample Entry: V|ct_version|4|dbversion|previous database version|3.2|frequency|previous database
     * frequency|W|issuedate|previous database issue date|20071126|dbtype|previous database type|USA F|7|1|versionkey
     * |2|dbversion|3|buildversion|4|frequency|5|issuedate|6|versioncomment |7|dbtype P|1|1
     * B|1|2|'0'|4|'X'|5|'00000000'|6|'Incremental update in progress'|7|'X' E|1|2|'3.2'|4|'W'|5|'20071203'|6|'Weekly custom
     * data issued 20071203, fdb production date 20071120'|7|'USA'
     * 
     * @param currentVersion
     *            CTVersionVA
     * @param previousVersion
     *            CTVersionVA
     * @param zout
     *            - ZipOutputStream
     * 
     */
    private void createUpdateControlFile(CtVersionVa currentVersion, CtVersionVa previousVersion, ZipOutputStream zout) {
        try {
            String versionControlRec =
                    "V|ct_version|4|dbversion|previous database version|" + previousVersion.getCtDbversion()
                            + "|frequency|previous database frequency|" + previousVersion.getCtFrequency()
                            + "|issuedate|previous database issue date|"
                            + formatDateCustomUpdateFileHeader(previousVersion.getCtIssuedate())
                            + "|dbtype|previous database type|" + previousVersion.getCtDbtype() + NEW_LINE_STR;
            String primaryKeyDefRec = "P|1|1" + NEW_LINE_STR;
            String begVersionChangeRec = "B|1|2|'0'|4|'X'|5|'00000000'|6|'Incremental update in progress'|7|'X'" + NEW_LINE_STR;
            String endVersionChangeRec =
                    "E|1|2|'" + currentVersion.getCtDbversion() + "'|4|'" + currentVersion.getCtFrequency() + "'|5|'"
                            + formatDateCustomUpdateFileHeader(currentVersion.getCtIssuedate()) + "'|6|'"
                            + currentVersion.getCtVersioncomment() + "'|7|'" + currentVersion.getCtDbtype() + "'"
                            + NEW_LINE_STR;

            zout.putNextEntry(new ZipEntry(CONTROL_FILE_NAME));
            zout.write(versionControlRec.getBytes(), 0, versionControlRec.length());
            zout.write(FIELDDEFREC.getBytes(), 0, FIELDDEFREC.length());
            zout.write(primaryKeyDefRec.getBytes(), 0, primaryKeyDefRec.length());
            zout.write(begVersionChangeRec.getBytes(), 0, begVersionChangeRec.length());
            zout.write(endVersionChangeRec.getBytes(), 0, endVersionChangeRec.length());

            zout.closeEntry();
            return;
        } catch (Exception e) {
            String msg = "Failed to create control file for custom tables";
            LOGGER.error(msg, e);
            throw new CustomizationServiceException(msg, e);
        }
    }

    /**
     * Used during the generation of a Full Custom Update file to create the CtVersion.txt and the filecounts.dat files used by
     * the FDB Updater tool.
     * 
     * @param currVersion
     *            the curr version
     * @param zout
     *            the zout
     * @param counts
     *            the counts
     */
    //@formatter:off
    private void createFileCountsFile(CtVersionVa currVersion, ZipOutputStream zout, 
            CustomUpdateRecordCounts counts) {
        //@formatter:on
        try {
            generateCtVersionFile(currVersion, zout);
            zout.putNextEntry(new ZipEntry(FILE_COUNTS_FILE_NAME));
            zout.write(counts.getFormatedDuplicateTherapyTotal(DuplicateTherapyUpdateFileSupport.DUPLICATE_THERAPY_FILE_NAME,
                    FIELD_SEPARATOR, NEW_LINE_STR).getBytes());
            zout.write(counts.getFormatedDrugDrugInteractionStringsTotal(
                    DrugDrugInteractionStringsUpdateFileSupport.DDIS_FILE_NAME, FIELD_SEPARATOR, NEW_LINE_STR).getBytes());
            zout.write(counts.getFormatedDrugDrugInteractionTotal(DrugDrugInteractionUpdateFileSupport.DDI_FILE_NAME,
                    FIELD_SEPARATOR, NEW_LINE_STR).getBytes());
            zout.write(counts.getFormatedDrugPairsTotal(DrugPairUpdateFileSupport.DP_FILE_NAME, FIELD_SEPARATOR, NEW_LINE_STR)
                    .getBytes());
            zout.write(counts.getFormatedDoseRangeTotal(DoseRangeUpdateFileSupport.DOSE_RANGE_FILE_NAME, FIELD_SEPARATOR,
                    NEW_LINE_STR).getBytes());
            zout.write(counts.getFormatedProfessionalMonographTotal(MonographUpdateFileSupport.MONOGRAPH_FILE_NAME,
                    FIELD_SEPARATOR, NEW_LINE_STR).getBytes());
            zout.write(counts.getCtVersionUpdateRecord(VERSION_FILE_NAME, FIELD_SEPARATOR, NEW_LINE_STR).getBytes());

            zout.closeEntry();
            return;
        } catch (Exception e) {
            LOGGER.error("Failed to create file counts file due to: ", e);
            throw new CustomizationServiceException("Failed to create file counts file for custom tables. ", e);
        }
    }

    /**
     * Generates the CtVersion.txt file included in a Full Custom Update extract.
     * 
     * @param ctVersion
     *            the ct version
     * @param zout
     *            the zout
     */
    private void generateCtVersionFile(CtVersionVa ctVersion, ZipOutputStream zout) {

        final StringBuffer dataRec = new StringBuffer();
        dataRec.append("1");
        dataRec.append(FIELD_SEPARATOR);
        dataRec.append(ctVersion.getCtDbversion());
        dataRec.append(FIELD_SEPARATOR);
        dataRec.append(ctVersion.getCtBuildversion());
        dataRec.append(FIELD_SEPARATOR);
        dataRec.append(ctVersion.getCtFrequency());
        dataRec.append(FIELD_SEPARATOR);
        dataRec.append(formatDateCustomUpdateFileHeader(ctVersion.getCtIssuedate()));
        dataRec.append(FIELD_SEPARATOR);
        dataRec.append(ctVersion.getCtVersioncomment());
        dataRec.append(FIELD_SEPARATOR);
        dataRec.append(ctVersion.getCtDbtype());
        dataRec.append(NEW_LINE_STR);

        try {
            zout.putNextEntry(new ZipEntry(getFileName(VERSION_FILE_NAME, true)));
            zout.write(dataRec.toString().getBytes());
            zout.closeEntry();
        } catch (IOException ioe) {
            String msg = "Failed creating " + VERSION_FILE_NAME;
            LOGGER.error(msg, ioe);
            throw new CustomizationServiceException(msg, ioe);
        }
    }

    /**
     * Creates file in the zip output stream, and copies contents of proddefinition.xml to the stream.
     * 
     * @param zout
     *            The output stream to copy the proddefinition.xml to.
     */
    private void loadScriptFiles(ZipOutputStream zout) {
        try {
            byte[] buffer = new byte[4096];
            InputStream in = this.getClass().getClassLoader().getResourceAsStream("xml/PRODDEFINITION.XML");
            int bytes = 0;
            zout.putNextEntry(new ZipEntry(PRODDEFINITION_FILE_NAME));
            while ((bytes = in.read(buffer, 0, buffer.length)) > 0) {
                zout.write(buffer, 0, bytes);
            }

            in.close();
            zout.closeEntry();
        } catch (Exception ex) {
            String msg = "Failed to create proddefinition file";
            LOGGER.error(msg, ex);
            throw new CustomizationServiceException(msg, ex);
        }
    }

    /**
     * This method generates ADD, CHANGE, and DELETE records for the Order Check type and appends the records to the output
     * stream parameter. If the isFull boolean parameter is true, records for a Full update are generated. If false, records for
     * an incremental update are generated.
     * 
     * @param zout
     *            The output stream that ADD, CHANGE, and DELETE records will be added to.
     * @param versionStr
     *            The FDB database update version.
     * @param isFull
     *            Boolean indicating whether a full or incremental update is requested.
     * @param fileDate
     *            the file date
     * @param counts
     *            the counts
     */
    public void generateFileMonograph(ZipOutputStream zout, String versionStr, boolean isFull, Date fileDate,
            CustomUpdateRecordCounts counts) {

        MonographUpdateFileSupport helper = getMonographUpdateFileSupport();
        generateUpdateFile(helper, zout, versionStr, isFull, fileDate, counts.getProfessionalMonographCounts());
        return;
    }

    /**
     * Generate file duplicate therapy.
     * 
     * @param zout
     *            the zout
     * @param versionStr
     *            the version str
     * @param isFull
     *            the is full
     * @param fileDate
     *            the file date
     * @param counts
     *            the counts
     */
    //@formatter:off
    public void generateFileDuplicateTherapy(ZipOutputStream zout, String versionStr, boolean isFull, Date fileDate, 
            CustomUpdateRecordCounts counts)
    {
       // @formatter:on
        DuplicateTherapyUpdateFileSupport helper = getDuplicateTherapyUpdateFileSupport();
        generateUpdateFile(helper, zout, versionStr, isFull, fileDate, counts.getDuplicateTherapyCounts());
        return;
    }

    /**
     * Gets the all record counts.
     * 
     * @return the all record counts
     */
    public CustomUpdateRecordCounts getAllRecordCounts() {
        CustomUpdateRecordCounts counts = new CustomUpdateRecordCounts();
        getDuplicateTherapyRecordCounts(counts);
        getDoseRangeRecordCounts(counts);
        getDrugPairRecordCounts(counts);
        getDrugDrugInteractionRecordCounts(counts);
        getDrugDrugInteractionStringRecordCounts(counts);
        getProfessionalMonographStringRecordCounts(counts);
        return counts;

    }

    /**
     * Gets the duplicate therapy record counts.
     * 
     * @param counts
     *            the counts
     * @return the duplicate therapy record counts
     */
    public void getDuplicateTherapyRecordCounts(CustomUpdateRecordCounts counts) {
        DuplicateTherapyUpdateFileSupport helper = getDuplicateTherapyUpdateFileSupport();
        counts.setDuplicateTherapyCounts(helper.getAllRecordCounts());
        return;
    }

    /**
     * Gets the dose range record counts.
     * 
     * @param counts
     *            the counts
     */
    public void getDoseRangeRecordCounts(CustomUpdateRecordCounts counts) {
        DoseRangeUpdateFileSupport helper = getDoseRangeUpdateFileSupport();
        counts.setDoseRangeCounts(helper.getAllRecordCounts());
        return;
    }

    /**
     * Gets the drug pair record counts.
     * 
     * @param counts
     *            the counts
     */
    public void getDrugPairRecordCounts(CustomUpdateRecordCounts counts) {
        DrugPairUpdateFileSupport helper = getDrugPairUpdateFileSupport();
        counts.setDrugPairCounts(helper.getAllRecordCounts());
        return;
    }

    /**
     * Gets the drug drug interaction record counts.
     * 
     * @param counts
     *            the counts
     */
    public void getDrugDrugInteractionRecordCounts(CustomUpdateRecordCounts counts) {
        DrugDrugInteractionUpdateFileSupport helper = getDrugDrugInteractionUpdateFileSupport();
        counts.setDrugDrugInteractionCounts(helper.getAllRecordCounts());
        return;
    }

    /**
     * Gets the drug drug interaction string record counts.
     * 
     * @param counts
     *            the counts
     */
    public void getDrugDrugInteractionStringRecordCounts(CustomUpdateRecordCounts counts) {
        DrugDrugInteractionStringsUpdateFileSupport helper = getDrugDrugInteractionStringsUpdateFileSupport();
        counts.setDrugDrugInteractionStringsCounts(helper.getAllRecordCounts());
        return;
    }

    /**
     * Gets the professional monograph string record counts.
     * 
     * @param counts
     *            the counts
     */
    public void getProfessionalMonographStringRecordCounts(CustomUpdateRecordCounts counts) {
        MonographUpdateFileSupport helper = getMonographUpdateFileSupport();
        counts.setProfessionalMonographCounts(helper.getAllRecordCounts());
        return;
    }

    /**
     * Generate file dose range.
     * 
     * @param zout
     *            the zout
     * @param versionStr
     *            the version str
     * @param isFull
     *            the is full
     * @param fileDate
     *            the file date
     * @param counts
     *            the counts
     */
    public void generateFileDoseRange(ZipOutputStream zout, String versionStr, boolean isFull, Date fileDate,
            CustomUpdateRecordCounts counts) {
        DoseRangeUpdateFileSupport helper = getDoseRangeUpdateFileSupport();
        generateUpdateFile(helper, zout, versionStr, isFull, fileDate, counts.getDoseRangeCounts());
        return;
    }

    /**
     * Generate file ddimstrings.
     * 
     * @param zout
     *            the zout
     * @param versionStr
     *            the version str
     * @param isFull
     *            the is full
     * @param fileDate
     *            the file date
     * @param counts
     *            the counts
     */
    public void generateFileDdimstrings(ZipOutputStream zout, String versionStr, boolean isFull, Date fileDate,
            CustomUpdateRecordCounts counts) {
        DrugDrugInteractionStringsUpdateFileSupport helper = getDrugDrugInteractionStringsUpdateFileSupport();
        generateUpdateFile(helper, zout, versionStr, isFull, fileDate, counts.getDrugDrugInteractionStringsCounts());
        return;
    }

    /**
     * Generate file drug drug interaction.
     * 
     * @param zout
     *            the DDI zout
     * @param versionStr
     *            the DDI version str
     * @param isFull
     *            the DDI isFull
     * @param fileDate
     *            the DDI file date
     * @param counts
     *            the DDI counts
     */
    public void generateFileDrugDrugInteraction(ZipOutputStream zout, String versionStr, boolean isFull, Date fileDate,
            CustomUpdateRecordCounts counts) {
        DrugDrugInteractionUpdateFileSupport helper = getDrugDrugInteractionUpdateFileSupport();
        generateUpdateFile(helper, zout, versionStr, isFull, fileDate, counts.getDrugDrugInteractionCounts());

        return;
    }

    /**
     * Generate file drug pair.
     * 
     * @param zout
     *            the drug pair zout
     * @param versionStr
     *            the drug pair version str
     * @param isFull
     *            the drug pair isFull
     * @param fileDate
     *            the drug pair file date
     * @param counts
     *            the drug pair counts
     */
    private void generateFileDrugPair(ZipOutputStream zout, String versionStr, boolean isFull, Date fileDate,
            CustomUpdateRecordCounts counts) {
        DrugPairUpdateFileSupport helper = getDrugPairUpdateFileSupport();
        generateUpdateFile(helper, zout, versionStr, isFull, fileDate, counts.getDrugPairCounts());
        return;
    }

    /**
     * Generate update file.
     * 
     * @param helper
     *            the helper
     * @param zout
     *            the zout
     * @param versionStr
     *            the version str
     * @param isFull
     *            the is full
     * @param fileDate
     *            the file date
     * @param recordCounts
     *            the record counts
     */
    private void generateUpdateFile(BaseUpdateFileSupport helper, ZipOutputStream zout, String versionStr, boolean isFull,
            Date fileDate, RecordCounts recordCounts) {

        zout = createZipEntry(getFileName(helper.getUpdateFileName(), isFull), zout);
        if (!isFull) {
            zout = helper.generateHeaderRecord(zout, recordCounts, versionStr);
        }

        if ((!isFull && recordCounts.getTotalIncrementalCounts() == 0)) {
            try {
                zout.closeEntry();
                return;
            } catch (IOException ioe) {
                String msg = "Unable to close zip entry";
                LOGGER.error(msg, ioe);
                throw new CustomizationServiceException(msg, ioe);
            }
        }

        try {
            if (!isFull) {
                zout.write((helper.generateFieldDefRecord() + NEW_LINE_STR).getBytes());
                zout.write((helper.generatePrimaryKeyRecord() + NEW_LINE_STR).getBytes());
                helper.generateRecordsAllStatuses(fileDate, recordCounts, zout);
                zout.closeEntry();

            } else {
                File file = helper.generateFullRecords();
                helper.generateFullRecords(zout, file);
            }

        } catch (IOException ioex) {
            String msg = "Unable to generate custom update records";
            LOGGER.error(msg, ioex);
            throw new CustomizationServiceException(msg, ioex);
        }
        return;
    }

    /**
     * Creates the update file.
     * 
     * @param createdBy
     *            the created by
     * @param isFull
     *            the is full
     * @param counts
     *            the counts
     * @return the custom update file va
     */
    private CustomUpdateFileVa createUpdateFiles(String createdBy, boolean isFull, CustomUpdateRecordCounts counts) {
        try {

            CtVersionVa previousVersion = customUpdateDAO.findMaxId();
            if (previousVersion == null && !isFull) {
                return null;
            }

            ByteArrayOutputStream dest = new ByteArrayOutputStream();
            ZipOutputStream zout = new ZipOutputStream(dest);
            Date fileDate = new Date();

            generateFilesAllConcepts(isFull, getPreviousVersionStr(previousVersion), zout, fileDate, counts);

            CtVersionVa ctVersionVa = new CtVersionVa();
            Long ctVersionKey = populateAndPersistCurrentVersion(fileDate, ctVersionVa);
            populatedAndPersistUpdatedVersion(createdBy, isFull, ctVersionVa, ctVersionKey);

            if (isFull) {
                createFileCountsFile(ctVersionVa, zout, counts);
            } else {
                createUpdateControlFile(ctVersionVa, previousVersion, zout);
            }

            loadScriptFiles(zout);
            zout.close();

            populatedAndPersistCustomUpdateFileVa(createdBy, dest, fileDate, ctVersionVa, ctVersionKey);

            return getCustomUpdateDAO().findById(ctVersionKey);
        } catch (Exception e) {
            String msg = "Failed to create customization update file";
            LOGGER.error(msg, e);
            throw new CustomizationServiceException(msg, e);
        }
    }

    /**
     * Populated and persist custom update file va.
     * 
     * @param createdBy
     *            the created by
     * @param dest
     *            the dest
     * @param fileDate
     *            the file date
     * @param ctVersionVa
     *            the ct version va
     * @param ctVersionKey
     *            the ct version key
     */
    private void populatedAndPersistCustomUpdateFileVa(String createdBy, ByteArrayOutputStream dest, Date fileDate,
            CtVersionVa ctVersionVa, Long ctVersionKey) {
        CustomUpdateFileVa customUpdateFileVa =
                new CustomUpdateFileVa(ctVersionKey.longValue(), fileDate, getVersionAsString(ctVersionVa, ctVersionKey),
                        createdBy, createZipFileName(fileDate, getVersionAsString(ctVersionVa, ctVersionKey)),
                        dest.toByteArray());

        getCustomUpdateDAO().save(customUpdateFileVa);
        // Does this logic need to flush the session or is the flush in the save method above take care of it?
        getCustomUpdateDAO().getSession().flush();
    }

    /**
     * Populated and persist updated version.
     * 
     * @param createdBy
     *            the created by
     * @param isFull
     *            the is full
     * @param ctVersionVa
     *            the ct version va
     * @param ctVersionKey
     *            the ct version key
     */
    private void
            populatedAndPersistUpdatedVersion(String createdBy, boolean isFull, CtVersionVa ctVersionVa, Long ctVersionKey) {
        if (isFull) {
            ctVersionVa.setCtVersioncomment("Full Update File Version: " + getVersionAsString(ctVersionVa, ctVersionKey)
                    + ", Created by: " + createdBy);
        } else {
            ctVersionVa.setCtVersioncomment("Incremental Update File Version: " + getVersionAsString(ctVersionVa, ctVersionKey)
                    + ", Created by: " + createdBy);
        }

        customUpdateDAO.save(ctVersionVa);
    }

    /**
     * Populate and persist current version.
     * 
     * @param fileDate
     *            the file date
     * @param ctVersionVa
     *            the ct version va
     * @return the long
     */
    private Long populateAndPersistCurrentVersion(Date fileDate, CtVersionVa ctVersionVa) {
        ctVersionVa.setCtIssuedate(fileDate);
        ctVersionVa.setCtBuildversion(CT_BUILDVERSION);
        ctVersionVa.setCtDbtype(DB_TYPE);
        ctVersionVa.setCtDbversion(CT_DBVERSION);
        ctVersionVa.setCtFrequency(CT_FREQUENCY);
        ctVersionVa.setFdbVersionkey(FDB_VERSIONKEY);
        return (Long) customUpdateDAO.save(ctVersionVa);
    }

    /**
     * Gets the version as string.
     * 
     * @param ctVersionVa
     *            the ct version va
     * @param ctVersionKey
     *            the ct version key
     * @return the version as string
     */
    private String getVersionAsString(CtVersionVa ctVersionVa, Long ctVersionKey) {
        return ctVersionVa.getCtDbversion() + "." + ctVersionKey.toString();
    }

    /**
     * Creates the zip file name.
     * 
     * @param fileDate
     *            the file date
     * @param updateVersionStr
     *            the update version str
     * @return the string
     */
    private String createZipFileName(Date fileDate, String updateVersionStr) {
        StringBuffer sb = new StringBuffer();
        sb.append(ZIP_FILE_PREFIX);
        sb.append(ZIP_FILE_DELIMITER);
        sb.append(updateVersionStr);
        sb.append(ZIP_FILE_DELIMITER);
        sb.append(formatDateAndTime(fileDate));
        sb.append(ZIP_FILE_EXTENSION);
        return sb.toString();
    }

    /**
     * Generate file all concepts.
     * 
     * @param isFull
     *            the is full
     * @param versionStr
     *            the version str
     * @param zout
     *            the zout
     * @param fileDate
     *            the file date
     * @param counts
     *            the counts
     */
    private void generateFilesAllConcepts(boolean isFull, String versionStr, ZipOutputStream zout, Date fileDate,
            CustomUpdateRecordCounts counts) {
        generateFileDuplicateTherapy(zout, versionStr, isFull, fileDate, counts);
        generateFileDrugPair(zout, versionStr, isFull, fileDate, counts);
        generateFileDdimstrings(zout, versionStr, isFull, fileDate, counts);
        generateFileDrugDrugInteraction(zout, versionStr, isFull, fileDate, counts);
        generateFileDoseRange(zout, versionStr, isFull, fileDate, counts);
        generateFileMonograph(zout, versionStr, isFull, fileDate, counts);
    }

    /**
     * Format time.
     * 
     * @param fileDate
     *            the file date
     * @return the string
     */
    private String formatTime(Date fileDate) {
        SimpleDateFormat timeFormat = new SimpleDateFormat("HHmmss");
        String tmStr = timeFormat.format(fileDate);
        return tmStr;
    }

    /**
     * Format date.
     * 
     * @param fileDate
     *            the file date
     * @return the string
     */
    private String formatDate(Date fileDate) {
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyyMMdd");
        String dtStr = dateFormat.format(fileDate);
        return dtStr;
    }

    /**
     * Format date and time.
     * 
     * @param fileDate
     *            the file date
     * @return the string
     */
    private String formatDateAndTime(Date fileDate) {

        String dateTime = formatDate(fileDate) + formatTime(fileDate);
        return dateTime;
    }

    /**
     * Retrieve the current fdb version.
     * 
     * @return FdbVersion
     */
    @Transactional(readOnly = true)
    public FdbVersion getFdbVersion() {
        return fdbVersionDAO.findById(VERSION_KEY);
    }

    /**
     * * Retrieve update file based on version If version is 0 create a new update file for current data and return the newly
     * created file Return a zip file that contains all files needed for executing the FDB Updater tool.
     * 
     * @param filepath
     *            : The file path that will be used for generating the file
     * 
     * @param createdBy
     *            : User name
     * 
     */
    @Transactional(timeout = 600)
    public void generateCustomUpdateFiles(String filepath, String createdBy) {
        if (StringUtils.isBlank(filepath)) {
            String message = getMessageHelper().getFormattedMessage(PECS_FILE_PATH_ERROR, null);
            throw new CustomizationServiceException(message);
        }

        CustomUpdateRecordCounts counts = getAllRecordCounts();

        CustomUpdateFileVa incrementalFile = createUpdateFiles(createdBy, false, counts);
        if (incrementalFile != null) {
            sendFileViaFTP(incrementalFile);
        }

        CustomUpdateFileVa updateFile = createUpdateFiles(createdBy, true, counts);
        if (updateFile == null) {
            throw new CustomizationServiceException("The first update generated by the system must me a full one.");
        } else {
            sendFileViaFTP(updateFile);
        }
        return;
    }

    /**
     * Sends the custom update file via FTP to the server specified in the exportfile.properties file.
     * 
     * @param updateFile
     *            the update file
     */
    private void sendFileViaFTP(CustomUpdateFileVa updateFile) {
        FileTransferCommands ftp = fileTransferFactory.createFtp();
        if (exportFileProperties != null) {
            try {
                final String exportServerName = exportFileProperties.getProperty(EXPORT_FILE_SERVER_PROPERTY);
                final String exportDirName = exportFileProperties.getProperty(EXPORT_FILE_DIR_PROPERTY);
                final String exportUserName = exportFileProperties.getProperty(EXPORT_USER_NAME_PROPERTY);
                if (exportServerName != null && exportDirName != null && exportUserName != null) {
                    try {
                        ftp.connectAndLogin(exportServerName, exportUserName, exportUserName);
                        ftp.changeWorkingDirectory(exportDirName);
                        ftp.setFileType(FtpClient.BINARY_FILE_TYPE);
                        if (ftp.storeFile(updateFile.getZipFileName(), updateFile.getZipFileContentBlob().getBinaryStream())) {
                            if (LOGGER.isInfoEnabled()) {
                                LOGGER.info("Successfully transferred " + updateFile.getZipFileName() + " to server "
                                        + exportServerName);
                            }
                        }
                    } catch (Exception e) {
                        String message = "Error sending file to server: " + exportServerName;
                        LOGGER.error("Error creating server side file: " + updateFile.getZipFileName(), e);
                        FTPException fe = new FTPException(message, e, exportServerName);
                        fe.setDirectoryName(exportDirName);
                        fe.setUserName(exportUserName);
                        throw fe;
                    } finally {
                        ftp.logout();
                    }
                } else {
                    StringBuffer message = populateExportFileErrorMessage(exportServerName, exportDirName, exportUserName);
                    throw new CustomizationServiceException(message.toString());
                }
            } catch (IOException ioe) {
                LOGGER.error("Cannot retrieve property " + EXPORT_FILE_DIR_PROPERTY);
            }
        }
    }

    /**
     * Populate export file error message.
     * 
     * @param exportServerName
     *            the export server name
     * @param exportDirName
     *            the export dir name
     * @param exportUserName
     *            the export user name
     * @return the string buffer
     */
    private StringBuffer populateExportFileErrorMessage(final String exportServerName, final String exportDirName,
            final String exportUserName) {
        StringBuffer message = new StringBuffer();
        if (exportServerName == null) {
            message.append("Cannot retrieve property " + EXPORT_FILE_SERVER_PROPERTY);
        }
        if (exportDirName == null) {
            if (message.length() > 0) {
                message.append("\n");
            }
            message.append("Cannot retrieve property " + EXPORT_FILE_DIR_PROPERTY);
        }
        if (exportUserName == null) {
            if (message.length() > 0) {
                message.append("\n");
            }
            message.append("Cannot retrieve property " + EXPORT_USER_NAME_PROPERTY);
        }
        return message;
    }

    /**
     * Method to find the update file.
     * 
     * @param customUpdateFileVaId
     *            the custom update file va id
     * @return CustomUpdateFileVa - the custom update file
     * @see gov.va.med.pharmacy.pecs.service.CustomUpdateService#findUpdateFile(java.lang.Long)
     */
    @Transactional(readOnly = true)
    public CustomUpdateFileVa findUpdateFile(Long customUpdateFileVaId) {
        return customUpdateDAO.findById(customUpdateFileVaId);

    }

    /**
     * Gets the custom update dao.
     * 
     * @return the custom update dao
     */
    public CustomUpdateDAO getCustomUpdateDAO() {
        return customUpdateDAO;
    }

    /**
     * Sets the custom update dao.
     * 
     * @param dao
     *            the new custom update dao
     */
    public void setCustomUpdateDAO(final CustomUpdateDAO dao) {
        this.customUpdateDAO = dao;
    }

    /**
     * Gets the fdb version dao.
     * 
     * @return the fdb version dao
     */
    public FdbVersionDAO getFdbVersionDAO() {
        return fdbVersionDAO;
    }

    /**
     * Sets the fdb version dao.
     * 
     * @param fdbVersionDAO
     *            the new fdb version dao
     */
    public void setFdbVersionDAO(final FdbVersionDAO fdbVersionDAO) {
        this.fdbVersionDAO = fdbVersionDAO;
    }

    /**
     * Gets the export file properties.
     * 
     * @return the export file properties
     */
    public Properties getExportFileProperties() {
        return exportFileProperties;
    }

    /**
     * Sets the export file properties.
     * 
     * @param exportFileProperties
     *            the new export file properties
     */
    public void setExportFileProperties(Properties exportFileProperties) {
        this.exportFileProperties = exportFileProperties;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.springframework.beans.factory.InitializingBean#afterPropertiesSet()
     */
    @Override
    public void afterPropertiesSet() throws Exception {

        if (monographUpdateFileSupport == null) {
            throw new Exception("MonographUpdateFileSupport has not been injected in to " + this.getClass().getName());
        }

        if (duplicateTherapyUpdateFileSupport == null) {
            throw new Exception("DuplicateTherapyUpdateFileSupport has not been injected in to " + this.getClass().getName());
        }

        if (doseRangeUpdateFileSupport == null) {
            throw new Exception("DoseRangeUpdateFileSupport has not been injected in to " + this.getClass().getName());
        }

        if (drugDrugInteractionStringsUpdateFileSupport == null) {
            throw new Exception("DrugDrugInteractionStringsUpdateFileSupport has not been injected in to "
                    + this.getClass().getName());
        }

        if (drugDrugInteractionUpdateFileSupport == null) {
            throw new Exception("DrugDrugInteractionUpdateFileSupport has not been injected in to  "
                    + this.getClass().getName());
        }

        if (drugPairUpdateFileSupport == null) {
            throw new Exception("DrugPairUpdateFileSupport has not been injected in to " + this.getClass().getName());
        }

        if (customUpdateDAO == null) {
            throw new Exception("CustomUpdateDAO has not been injected in to " + this.getClass().getName());
        }
        if (fdbVersionDAO == null) {
            throw new Exception("FdbVersionDAO has not been injected in to " + this.getClass().getName());
        }

        if (monographPairDAO == null) {
            throw new Exception("MonographPairDAO has not been injected in to " + this.getClass().getName());
        }
        if (exportFileProperties == null) {
            throw new Exception("ExportFileProperties has not been injected in to " + this.getClass().getName());
        }
        if (fileTransferFactory == null) {
            throw new Exception("The FileTransferFactory has not been injected in to " + this.getClass().getName());
        }
    }

    /**
     * Gets the monograph update file support.
     * 
     * @return the monograph update file support
     */
    public MonographUpdateFileSupport getMonographUpdateFileSupport() {
        return monographUpdateFileSupport;
    }

    /**
     * Sets the monograph update file support.
     * 
     * @param monographUpdateFileSupport
     *            the new monograph update file support
     */
    public void setMonographUpdateFileSupport(MonographUpdateFileSupport monographUpdateFileSupport) {
        this.monographUpdateFileSupport = monographUpdateFileSupport;
    }

    /**
     * Gets the duplicate therapy update file support.
     * 
     * @return the duplicate therapy update file support
     */
    public DuplicateTherapyUpdateFileSupport getDuplicateTherapyUpdateFileSupport() {
        return duplicateTherapyUpdateFileSupport;
    }

    /**
     * Sets the duplicate therapy update file support.
     * 
     * @param duplicateTherapyUpdateFileSupport
     *            the new duplicate therapy update file support
     */
    public void setDuplicateTherapyUpdateFileSupport(DuplicateTherapyUpdateFileSupport duplicateTherapyUpdateFileSupport) {
        this.duplicateTherapyUpdateFileSupport = duplicateTherapyUpdateFileSupport;
    }

    /**
     * Gets the dose range update file support.
     * 
     * @return the dose range update file support
     */
    public DoseRangeUpdateFileSupport getDoseRangeUpdateFileSupport() {
        return doseRangeUpdateFileSupport;
    }

    /**
     * Sets the dose range update file support.
     * 
     * @param doseRangeUpdateFileSupport
     *            the new dose range update file support
     */
    public void setDoseRangeUpdateFileSupport(DoseRangeUpdateFileSupport doseRangeUpdateFileSupport) {
        this.doseRangeUpdateFileSupport = doseRangeUpdateFileSupport;
    }

    /**
     * Gets the drug drug interaction strings update file support.
     * 
     * @return the drug drug interaction strings update file support
     */
    public DrugDrugInteractionStringsUpdateFileSupport getDrugDrugInteractionStringsUpdateFileSupport() {
        return drugDrugInteractionStringsUpdateFileSupport;
    }

    /**
     * Sets the drug drug interaction strings update file support.
     * 
     * @param drugDrugInteractionStringsUpdateFileSupport
     *            the new drug drug interaction strings update file support
     */
    public void setDrugDrugInteractionStringsUpdateFileSupport(
            DrugDrugInteractionStringsUpdateFileSupport drugDrugInteractionStringsUpdateFileSupport) {
        this.drugDrugInteractionStringsUpdateFileSupport = drugDrugInteractionStringsUpdateFileSupport;
    }

    /**
     * Gets the drug drug interaction update file support.
     * 
     * @return the drug drug interaction update file support
     */
    public DrugDrugInteractionUpdateFileSupport getDrugDrugInteractionUpdateFileSupport() {
        return drugDrugInteractionUpdateFileSupport;
    }

    /**
     * Sets the drug drug interaction update file support.
     * 
     * @param drugDrugInteractionUpdateFileSupport
     *            the new drug drug interaction update file support
     */
    public void setDrugDrugInteractionUpdateFileSupport(
            DrugDrugInteractionUpdateFileSupport drugDrugInteractionUpdateFileSupport) {
        this.drugDrugInteractionUpdateFileSupport = drugDrugInteractionUpdateFileSupport;
    }

    /**
     * Gets the drug pair update file support.
     * 
     * @return the drug pair update file support
     */
    public DrugPairUpdateFileSupport getDrugPairUpdateFileSupport() {
        return drugPairUpdateFileSupport;
    }

    /**
     * Sets the drug pair update file support.
     * 
     * @param drugPairUpdateFileSupport
     *            the new drug pair update file support
     */
    public void setDrugPairUpdateFileSupport(DrugPairUpdateFileSupport drugPairUpdateFileSupport) {
        this.drugPairUpdateFileSupport = drugPairUpdateFileSupport;
    }

    /**
     * Gets the message helper.
     * 
     * @return the message helper
     */
    public MessageHelper getMessageHelper() {
        return messageHelper;
    }

    /**
     * Sets the message helper.
     * 
     * @param messageHelper
     *            the new message helper
     */
    public void setMessageHelper(MessageHelper messageHelper) {
        this.messageHelper = messageHelper;
    }

    /**
     * Sets the file transfer factory.
     * 
     * @param fileTransferFactory
     *            the new file transfer factory
     */
    public void setFileTransferFactory(FileTransferFactory fileTransferFactory) {
        this.fileTransferFactory = fileTransferFactory;
    }

    /**
     * Retrives a list of years that we have custom update files.
     * 
     * @return the list
     */
    @Override
    @Transactional
    public List<Date> findAllYears() {
        return getCustomUpdateDAO().findAllYears();
    }

    /**
     * Returns the List of Months base off of the Year Requested.
     * 
     * @param year
     *            the year
     * @return the list
     */
    @Override
    @Transactional
    public List<Date> findAllMonthsOfUpdatesByYears(String year) {
        return customUpdateDAO.findMonthsByYear(year);
    }

    /**
     * Returns List of Custom Update Files sorted by Year and Month.
     * 
     * @param year
     *            the year
     * @param month
     *            the month
     * @return the custom update file by month andyear
     */
    @Override
    @Transactional
    public List<CtVersionVa> getCustomUpdateFileByMonthAndyear(String year, String month) {
        return customUpdateDAO.findFilesByMonthsAndYear(year, month);
    }

    /**
     * Retrieves the list of all the CustomUpdateVersion objects.
     * 
     * @return the list
     */
    @Transactional(readOnly = true)
    public List<CtVersionVa> findAllCtVersionVas() {
        return customUpdateDAO.findAllOrderByIssueDateDesc();
    }
}
