package gov.va.med.pharmacy.pecs.service.impl;

import gov.va.med.pharmacy.pecs.customupdate.FdbVersion;
import gov.va.med.pharmacy.pecs.domain.impl.va.UserProfile;
import gov.va.med.pharmacy.pecs.persistence.DrugPairDAOImpl;
import gov.va.med.pharmacy.pecs.service.CustomUpdateService;
import gov.va.med.pharmacy.pecs.service.DeletedRoutedGenericService;

import javax.annotation.Resource;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * Handles the processing of finding Drug Pairs associated with Routed Generics that have
 * been deleted from FDB, and deletes those Drug Pairs.
 * 
 * @author vhaisaduffg
 */
@Service
public class DeletedRoutedGenericServiceImpl implements DeletedRoutedGenericService, InitializingBean {

    private static final Log LOGGER = LogFactory.getLog(DeletedRoutedGenericServiceImpl.class);
    
    @Resource
    private DrugPairDAOImpl drugPairDao;
    @Resource
    private CustomUpdateService customUpdateService;
    
    private String issueDate = null;
    
    private static final String DELETION_USER = "SYSTEM";
    private static final String DELETION_COMMENT = "FDB Deleted on ";

    /**
     * Handles the processing of finding Drug Pairs associated with Routed Generics that have
     * been deleted from FDB, and deletes those Drug Pairs.
     * 
     * @param profile the UserProfile of the logged in user
     */
    @Override
    @Transactional
    public final void process(UserProfile profile) {
        // Null out the issue date so that if this service has not been restarted, the new
        // issue date will be found.
        issueDate = null;
        
        deleteDrugPairs(profile, generateComment());
    }

    /**
     * Deletes the custom Drug Pairs that are associated with FDB Routed Generics that
     * no longer exist.
     * 
     * @param profile the UserProfile of the logged in user
     * @param comment the comment to add to the deleted Drug Pairs
     * @return void
     */
    private void deleteDrugPairs(UserProfile profile, String comment) {
        drugPairDao.deleteDrugPairsWithDeletedRoutedGeneric(profile.getUsername(), DELETION_USER, comment, DELETION_COMMENT);
    }
    
    /**
     * Generates the comment that should be placed in to the Drug Pair.
     * 
     * @return String
     */
    private String generateComment() {
        StringBuilder sb = new StringBuilder();
        
        sb.append(DELETION_COMMENT);
        sb.append(findIssueDate());
        
        return sb.toString();
    }
    
    /**
     * Returns the latest FDB Issue Date.
     * 
     * @return String
     */
    private String findIssueDate() {
        if ( issueDate == null ) {
            FdbVersion fdbVersion = customUpdateService.getFdbVersion();
            issueDate = fdbVersion.getIssuedate();
        }
        return issueDate;
    }
    /**
     * Sets the drugPairDao.
     *
     * @param drugPairDao the drugPairDao to set
     */
    public final void setDrugPairDao(DrugPairDAOImpl drugPairDao) {
        this.drugPairDao = drugPairDao;
    }

    /**
     * Sets the customUpdateService.
     *
     * @param customUpdateService the customUpdateService to set
     */
    public final void setCustomUpdateService(CustomUpdateService customUpdateService) {
        this.customUpdateService = customUpdateService;
    }

    /**
     * Used to check to make sure all Spring Beans have been injected.
     * 
     * @throws Exception if a bean has not been injected
     */
    @Override
    public final void afterPropertiesSet() throws Exception {
        if ( drugPairDao == null ) {
            throw new Exception("DrugPairDAOImpl has not been injected in to DeletedRoutedGenericsServiceImpl.");
        }
        if ( customUpdateService == null ) {
            throw new Exception("CustomUpdateService has not been injected in to DeletedRoutedGenericsServiceImpl.");
        }
    }

}
