package gov.va.med.pharmacy.pecs.service.impl;

import gov.va.med.pharmacy.pecs.criteria.ConceptHistoryCriteria;
import gov.va.med.pharmacy.pecs.domain.DoseRange;
import gov.va.med.pharmacy.pecs.domain.UserComment;
import gov.va.med.pharmacy.pecs.domain.Version;
import gov.va.med.pharmacy.pecs.domain.impl.fdb.FdbDosing;
import gov.va.med.pharmacy.pecs.domain.impl.fdb.FdbDosingDAO;
import gov.va.med.pharmacy.pecs.domain.impl.lookup.DoseRoute;
import gov.va.med.pharmacy.pecs.domain.impl.lookup.DoseType;
import gov.va.med.pharmacy.pecs.domain.impl.va.ConceptXref;
import gov.va.med.pharmacy.pecs.domain.impl.va.DoseRangeKey;
import gov.va.med.pharmacy.pecs.domain.impl.va.DoseRangeVA;
import gov.va.med.pharmacy.pecs.domain.impl.va.DoseRangeVersionVa;
import gov.va.med.pharmacy.pecs.domain.impl.va.DoseRangeXref;
import gov.va.med.pharmacy.pecs.dto.DoseRangeDTO;
import gov.va.med.pharmacy.pecs.exception.CustomizationServiceException;
import gov.va.med.pharmacy.pecs.metadata.ConceptField;
import gov.va.med.pharmacy.pecs.metadata.ConceptMetadata;
import gov.va.med.pharmacy.pecs.persistence.DoseRangeVaDAO;
import gov.va.med.pharmacy.pecs.persistence.DoseRangeVersionVaDAO;
import gov.va.med.pharmacy.pecs.persistence.FdbHibernateDAOImpl;
import gov.va.med.pharmacy.pecs.persistence.HibernateDAO;
import gov.va.med.pharmacy.pecs.persistence.fdb.DrugConceptDao;
import gov.va.med.pharmacy.pecs.query.ConceptQuery;
import gov.va.med.pharmacy.pecs.query.DoseRangeVisitor;
import gov.va.med.pharmacy.pecs.query.Page;
import gov.va.med.pharmacy.pecs.rules.AlwaysFireTrigger;
import gov.va.med.pharmacy.pecs.rules.BusinessRulesValidation;
import gov.va.med.pharmacy.pecs.rules.BusinessRulesValidationImpl;
import gov.va.med.pharmacy.pecs.rules.ReferenceTextFieldRule;
import gov.va.med.pharmacy.pecs.service.DoseRangeService;
import gov.va.med.pharmacy.pecs.service.RequestCountSupport;
import gov.va.med.pharmacy.pecs.service.dto.FieldHistoryLineItem;
import gov.va.med.pharmacy.pecs.state.Status;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.BindingResult;
import org.springframework.validation.ObjectError;

/**
 * An implementation of the Dose Range Service interface.
 * 
 * @author vhaisaduffg
 * 
 */
@Service
public final class DoseRangeServiceImpl extends AbstractPecsService<DoseRangeDTO, DoseRange> implements DoseRangeService,
        InitializingBean {

    private static final Logger LOGGER = Logger.getLogger(DoseRangeServiceImpl.class);

    @Resource
    private DoseRangeVersionVaDAO doseRangeVersionVaDAO; // this is for VA --- version tables
    @Resource
    private DoseRangeVaDAO doseRangeVaDAO; // this is for VA --- version tables
    @Resource
    private FdbDosingDAO fdbDosingDAO; // this is for FDB database
    @Resource
    private DrugConceptDao drugConceptDao;
    private Map<String, DoseRoute> doseRouteMap;
    private Map<String, DoseType> doseTypeMap;

    /*
     * to modify the version record.
     */
    @Override
    @Transactional
    public DoseRangeDTO applyAction(String action, DoseRangeDTO doseRangeDTO, BindingResult err) {
        if (doseRangeDTO.getDoseTypeDesc() != null) {
            doseRangeDTO.setDoseTypeDesc(null);
        }
        if (doseRangeDTO.getDoseRouteDesc() != null) {
            doseRangeDTO.setDoseRouteDesc(null);
        }
        DoseRangeVA dr = findPersistentObject(doseRangeDTO);
        DoseRangeVersionVa v = dr.getCurrentVersion();
        DoseRangeDTO crtDoseRange = createDoseRangeDTO(v);
        boolean stateChange = doApplyAction(dr, doseRangeDTO, crtDoseRange, action, doseRangeDTO.getActionUser(), err);
        // set action by only when the state machine was invoked,
        // but not when a roll-back took place
        if (stateChange && !dr.wasRolledBack()) {
            dr.setActionBy(doseRangeDTO.getActionBy());
        }
        doseRangeDTO.setExportDate(null);
        return err.hasErrors() ? doseRangeDTO : populateAndPersistDoseRange(dr, doseRangeDTO, dr.wasRolledBack());

    }

    /**
     * Attempts to locate a custom Dose Range record. If the Dose Range Identifier is 0, an exception is thrown.
     * 
     * @param doseRangeDTO
     *            a DoseRangeDTO object with the Dose Range Id populate.
     * @return DoseRangeVA
     */
    @Override
    protected DoseRangeVA findPersistentObject(DoseRangeDTO doseRangeDTO) {
        long doseRangeId = doseRangeDTO.getDoseRangeId(); // dwd - not getting correct doseRangeId
        if (doseRangeId == 0) {
            LOGGER.error("Attempt to retrieve persistent dose range with id of 0");
            throw new CustomizationServiceException("Invalid doserange - id cannot be 0");
        }
        DoseRangeVA m = doseRangeVaDAO.findById(doseRangeId);
        return m;
    }

    /**
     * Populates the DoseRangeVA object with information from the DoseRangeDTO object and then persists the DoseRangeVA object.
     * 
     * @param dest
     *            DoseRangeVA
     * @param source
     *            DoseRangeDTO
     * @param partial
     *            if false, populates all information.
     * @return DoseRangeDTO
     */
    private DoseRangeDTO populateAndPersistDoseRange(DoseRangeVA dest, DoseRangeDTO source, boolean partial) {
        if (!partial) {
            // get rid of this, rely on BusinessRulesValidation

            dest.setAgeHighInDays(source.getAgehighindays());
            dest.setAgeLowInDays(source.getAgelowindays());
            dest.setReferenceText(source.getReferenceText());
            dest.setConceptid(source.getConceptid());
            dest.setConceptType(source.getConcepttype());
            dest.setDoseRoute(doseRouteMap.get(source.getDoserouteid()));
            dest.setDoseType(doseTypeMap.get(source.getDosetypeid()));
            dest.setCategory(source.getCategory());
            dest.setFdbdx(source.getFdbdx());
            dest.setLowEliminationHalfLife(source.getLoweliminationhalflife());
            dest.setHighEliminationHalfLife(source.getHigheliminationhalflife());
            dest.setFrequencyLow(source.getFrequencylow());
            dest.setFrequencyHigh(source.getFrequencyhigh());
            dest.setDurationLow(source.getDurationlow());
            dest.setDurationHigh(source.getDurationhigh());
            dest.setMaxDuration(source.getMaxduration());
            dest.setRenalImpairmentIndicator(source.getRenalimpairmentind());
            dest.setWeightImpairmentIndicator(source.getWeightrequiredind());
            dest.setBsaRequiredIndicator(source.getBsarequiredind());
            dest.setHepaticImpairmentIndicator(source.getHepaticimpairmentind());
            dest.setWarningCode(source.getWarningcode());
            dest.setDoseLow(source.getDoselow());
            dest.setDoseRateLow(source.getDoseratelow());
            dest.setDoseRateHigh(source.getDoseratehigh());
            dest.setDoseFormRateLow(source.getDoseformratelow());
            dest.setDoseFormRateHigh(source.getDoseformratehigh());
            dest.setMaxSingleDoseRate(source.getMaxsingledoserate());
            dest.setMaxSingleDoseFormRate(source.getMaxsingledoseformrate());
            dest.setMaxDailyDoseRate(source.getMaxdailydoserate());
            dest.setMaxDailyDoseFormRate(source.getMaxdailydoseformrate());
            dest.setDxid(source.getDxid());
            dest.setCrclThreshold(source.getCrclthreshhold());
            dest.setAgeLowInDays(source.getAgelowindays());
            dest.setAgeHighInDays(source.getAgehighindays());
            dest.setDoseLowUnits(source.getDoselowunits()); // dwd hardcode needs to be resolved
            dest.setDoseHighUnits(source.getDosehighunits());
            dest.setDoseFormLowUnits(source.getDoseformlowunits());
            dest.setDoseFormHighUnits(source.getDoseformhighunits());
            dest.setMaxSingleDoseUnits(source.getMaxsingledoseunits());
            dest.setMaxSingleDoseFormUnits(source.getMaxsingledoseformunits());
            dest.setMaxDailyDoseUnits(source.getMaxdailydoseunits());
            dest.setMaxDailyDoseFormUnits(source.getMaxdailydoseformunits());
            dest.setMaxLifeTimeDoseUnits(source.getMaxlifetimedoseunits());
            dest.setMaxLifeTimeDoseFormUnits(source.getMaxlifetimedoseformunits());
            dest.setCrclThresholdUnits(source.getCrclthreshholdunits());
            dest.setDoseRateLowUnits(source.getDoseratelowunits());
            dest.setDoseRateHighUnits(source.getDoseratehighunits());
            dest.setDoseFormRateLowUnits(source.getDoseformratelowunits());
            dest.setDoseFormRateHighUnits(source.getDoseformratehighunits());
            dest.setMaxSingleDoseRateUnits(source.getMaxsingledoserateunits());
            dest.setMaxSingleDoseFormRateUnits(source.getMaxsingledoseformrateunits());
            dest.setMaxDailyDoseRateUnits(source.getMaxdailydoserateunits());
            dest.setMaxDailyDoseFormRateUnits(source.getMaxdailydoseformrateunits());
            dest.setDoseFormLow(source.getDoseformlow());
            dest.setDoseHigh(source.getDosehigh());
            dest.setDoseFormHigh(source.getDoseformhigh());
            dest.setMaxSingleDose(source.getMaxsingledose());
            dest.setMaxSingleDoseForm(source.getMaxsingledoseform());
            dest.setMaxDailyDose(source.getMaxdailydose());
            dest.setMaxDailyDoseForm(source.getMaxdailydoseform());
            dest.setMaxLifeTimeDose(source.getMaxlifetimedose());
            dest.setMaxLifeTimeDoseForm(source.getMaxlifetimedoseform());
            dest.setHalfLifeUnits(getTerminologyService().getHalfLifeUnitByCode(source.getHalflifeunits()));

            // shahid 10-15-2012
            dest.setMaxSingleNTEDose(source.getMaxSingleNTEDose());
            dest.setMaxSingleNTEDoseUnit(source.getMaxSingleNTEDoseUnit());
            dest.setMaxSingleNTEDoseForm(source.getMaxSingleNTEDoseForm());
            dest.setMaxSingleNTEDoseFormUnit(source.getMaxSingleNTEDoseFormUnit());

            // Version attributes
            dest.setAssignedUser(source.getAssignedTo());
        }

        dest.setAuditUser(source.getActionBy());
        dest.setComment(source.getCurrentActionReason());
        dest.setModifiedDate(new Date());
        // DoseRangeKey doseRangeKey = populateDosingId(source);

        // dest.setConceptXref(source.getConceptXref());

        doseRangeVersionVaDAO.save(dest);
        return createDoseRangeDTO(dest.getCurrentVersion());
    }

    /**
     * @param source
     *            the va object
     * @return a dose range key
     */
    private DoseRangeKey populateDosingId(DoseRangeDTO source) {
        DoseRangeKey doseRangeKey = new DoseRangeKey();
        doseRangeKey.setAgeHighInDays(source.getAgehighindays());
        doseRangeKey.setAgeLowInDays(source.getAgelowindays());
        doseRangeKey.setConceptId(source.getConceptid());
        doseRangeKey.setConceptType(source.getConcepttype());
        doseRangeKey.setDoseRouteId(source.getDoserouteid());
        doseRangeKey.setDoseTypeId(source.getDosetypeid());
        doseRangeKey.setFdbdx(source.getFdbdx());
        doseRangeKey.setHitType(source.getHitType());
        return doseRangeKey;
    }

    /**
     * Create a DTO out of a domain object. The data is coming from database and going to browser.
     * 
     * @param source
     *            the Dose Range Version serving as a source
     * @return the populated DTO
     */
    private DoseRangeDTO createDoseRangeDTO(DoseRangeVersionVa source) {
        DoseRangeDTO dto = new DoseRangeDTO();
        // dwd test hard code setting to true
        dto.setDoseRangeId(source.getVersionedObject().getId());

        dto.setCurrent(source.isCurrent());
        dto.setConceptid(source.getConceptid());
        dto.setFdbCustomDosingVaId(source.getVersionId());
        dto.setDoserouteid(source.getDoseRoute().getCode());
        dto.setDoseRouteDesc(source.getDoseRoute().getName());
        dto.setDosetypeid(source.getDoseType().getCode());
        dto.setDoseTypeDesc(source.getDoseType().getName());
        dto.setAgelowindays(source.getAgeLowInDays());
        dto.setAgehighindays(source.getAgeHighInDays());
        dto.setDoseformlow(source.getDoseFormLow());
        dto.setDoseformlowunits(source.getDoseFormLowUnits());
        dto.setDoseformhigh(source.getDoseFormHigh());
        dto.setDoseformhighunits(source.getDoseFormHighUnits());
        dto.setMaxsingledose(source.getMaxSingleDose());
        dto.setMaxsingledoseunits(source.getMaxSingleDoseUnits());
        dto.setMaxsingledoseform(source.getMaxSingleDoseForm());
        dto.setMaxsingledoseformunits(source.getMaxSingleDoseFormUnits());
        dto.setFdbdx(source.getFdbdx());
        dto.setActionBy(source.getActionBy());
        dto.setExportDate(source.getExportDate());
        dto.setStatus(source.getStatus().getName());
        dto.setAssignedTo(source.getAssignedUser());
        dto.setRequestor(source.getVersionedObject().getRequestor());
        // NOTE: this is history --- call special method to pull all history.
        dto.setActionReason(source.getCommentHistory()); // it will pull all history
        dto.setActionDate(source.getDate());
        dto.setReferenceText(source.getReferenceText());
        dto.setCategory(source.getCategory());
        dto.setConcepttype(source.getConceptType());
        dto.setConceptDescription(source.getConceptDescription());
        dto.setDoselow(source.getDoseLow());
        dto.setDoselowunits(source.getDoseLowUnits());
        dto.setDosehigh(source.getDoseHigh());
        dto.setDosehighunits(source.getDoseHighUnits());
        dto.setMaxdailydose(source.getMaxDailyDose());
        dto.setMaxdailydoseunits(source.getMaxDailyDoseUnits());
        dto.setMaxdailydoseform(source.getMaxDailyDoseForm());
        dto.setMaxdailydoseformunits(source.getMaxDailyDoseFormUnits());
        dto.setMaxlifetimedose(source.getMaxLifeTimeDose());
        dto.setMaxlifetimedoseunits(source.getMaxLifeTimeDoseUnits());
        dto.setMaxlifetimedoseform(source.getMaxLifeTimeDoseForm());
        dto.setMaxlifetimedoseformunits(source.getMaxLifeTimeDoseFormUnits());
        dto.setLoweliminationhalflife(source.getLowEliminationHalfLife());
        dto.setHigheliminationhalflife(source.getHighEliminationHalfLife());
        if (source.getHalfLifeUnits() != null) {
            dto.setHalflifeunits(source.getHalfLifeUnits().getCode());
        }
        dto.setFrequencylow(source.getFrequencyLow());
        dto.setFrequencyhigh(source.getFrequencyHigh());
        dto.setDurationlow(source.getDurationLow());
        dto.setDurationhigh(source.getDurationHigh());
        dto.setMaxduration(source.getMaxDuration());
        dto.setHepaticimpairmentind(source.getHepaticImpairmentIndicator());
        dto.setRenalimpairmentind(source.getRenalImpairmentIndicator());
        dto.setCrclthreshhold(source.getCrclThreshold());
        dto.setCrclthreshholdunits(source.getCrclThresholdUnits());
        dto.setWeightrequiredind(source.getWeightImpairmentIndicator());
        dto.setBsarequiredind(source.getBsaRequiredIndicator());
        dto.setDoseratelow(source.getDoseRateLow());
        dto.setDoseratelowunits(source.getDoseRateLowUnits());
        dto.setDoseratehigh(source.getDoseRateHigh());
        dto.setDoseratehighunits(source.getDoseRateHighUnits());
        dto.setDoseformratelow(source.getDoseFormRateLow());
        dto.setDoseformratelowunits(source.getDoseFormRateLowUnits());
        dto.setDoseformratehigh(source.getDoseFormRateHigh());
        dto.setDoseformratehighunits(source.getDoseFormRateHighUnits());
        dto.setMaxsingledoserate(source.getMaxSingleDoseRate());
        dto.setMaxsingledoserateunits(source.getMaxSingleDoseRateUnits());
        dto.setMaxsingledoseformrate(source.getMaxSingleDoseFormRate());
        dto.setMaxsingledoseformrateunits(source.getMaxSingleDoseFormRateUnits());
        dto.setMaxdailydoserate(source.getMaxDailyDoseRate());
        dto.setMaxdailydoserateunits(source.getMaxDailyDoseRateUnits());
        dto.setMaxdailydoseformrate(source.getMaxDailyDoseFormRate());
        dto.setMaxdailydoseformrateunits(source.getMaxDailyDoseFormRateUnits());
        dto.setDxid(source.getDxid());
        dto.setFdbId(source.getFdbdx());
        dto.setMasterRecordId(source.getVersionedObject().getId());
        dto.setMaxSingleNTEDose(source.getMaxSingleNTEDose());
        dto.setMaxSingleNTEDoseUnit(source.getMaxSingleNTEDoseUnit());
        dto.setMaxSingleNTEDoseForm(source.getMaxSingleNTEDoseForm());
        dto.setMaxSingleNTEDoseFormUnit(source.getMaxSingleNTEDoseFormUnit());
        dto.setUserCommentHistory(source.getVersionedObject().getUserCommentHistory());
        dto.setConceptXref(source.getVersionedObject().getConceptXref());

        return dto;
    }

    /**
     * Convert from FdbDosing to DoseRangeDTO.
     * 
     * @param fdbList
     *            list of FdbDosing objects
     * @return a list of DoseRangeDTO objects
     */
    @SuppressWarnings("unchecked")
    @Override
    protected List<DoseRangeDTO> convertFdbToCustom(List<?> fdbList) {
        List<DoseRangeDTO> doseRangeDTO = new ArrayList<DoseRangeDTO>();
        for (FdbDosing o : (List<FdbDosing>) fdbList) {
            DoseRangeDTO dto = convertFdbToCustom(o);

            doseRangeDTO.add(dto);
        }
        return doseRangeDTO;
    }

    /**
     * Converts a FdbDosing object to a DoseRangeDTO object.
     * 
     * @param fdbDosing
     *            FdbDosing
     * @return DoseRangeDTO
     */
    private DoseRangeDTO convertFdbToCustom(FdbDosing fdbDosing) {
        DoseRangeDTO dto = new DoseRangeDTO();
        dto.setConceptid(fdbDosing.getId().getConceptid());
        dto.setDoserouteid(fdbDosing.getId().getDoseroute().getDoserouteid());
        dto.setDoseRouteDesc(fdbDosing.getId().getDoseroute().getDescription1());
        dto.setDosetypeid(fdbDosing.getId().getDosetype().getDosetypeid());
        dto.setDoseTypeDesc(fdbDosing.getId().getDosetype().getDescription1());
        dto.setAgelowindays(fdbDosing.getId().getAgelowindays());
        dto.setAgehighindays(fdbDosing.getId().getAgehighindays());
        dto.setFdbdx(fdbDosing.getId().getFdbdx());
        // shahid added 3-6-2011 to fix concept type not showing on the browser detail page first time.
        dto.setConcepttype(fdbDosing.getId().getConcepttype());
        dto.setConceptDescription(fdbDosing.getConceptDescription());
        dto.setConceptXref(fdbDosing.getConceptXref());
        dto.setDoselow(fdbDosing.getDoselow());
        dto.setDoselowunits(fdbDosing.getDoselowunits());
        dto.setDosehigh(fdbDosing.getDosehigh());
        dto.setDosehighunits(fdbDosing.getDosehighunits());
        dto.setDoseformlow(fdbDosing.getDoseformlow());
        dto.setDoseformlowunits(fdbDosing.getDoseformlowunits());
        dto.setDoseformhigh(fdbDosing.getDoseformhigh());
        dto.setDoseformhighunits(fdbDosing.getDoseformhighunits());
        dto.setMaxsingledose(fdbDosing.getMaxsingledose());
        dto.setMaxsingledoseunits(fdbDosing.getMaxsingledoseunits());
        dto.setMaxsingledoseform(fdbDosing.getMaxsingledoseform());
        dto.setMaxsingledoseformunits(fdbDosing.getMaxsingledoseformunits());
        dto.setMaxdailydose(fdbDosing.getMaxdailydose());
        dto.setMaxdailydoseunits(fdbDosing.getMaxdailydoseunits());
        dto.setMaxdailydoseform(fdbDosing.getMaxdailydoseform());
        dto.setMaxdailydoseformunits(fdbDosing.getMaxdailydoseformunits());
        dto.setMaxlifetimedose(fdbDosing.getMaxlifetimedose());
        dto.setMaxlifetimedoseunits(fdbDosing.getMaxlifetimedoseformunits());
        dto.setMaxlifetimedoseform(fdbDosing.getMaxlifetimedoseform());
        dto.setMaxlifetimedoseformunits(fdbDosing.getMaxlifetimedoseunits());
        dto.setLoweliminationhalflife(fdbDosing.getLoweliminationhalflife());
        dto.setHigheliminationhalflife(fdbDosing.getHigheliminationhalflife());
        dto.setHalflifeunits(fdbDosing.getHalflifeunits());
        dto.setFrequencylow(fdbDosing.getFrequencylow());
        dto.setFrequencyhigh(fdbDosing.getFrequencyhigh());
        dto.setDurationlow(fdbDosing.getDurationlow());
        dto.setDurationhigh(fdbDosing.getDurationhigh());
        dto.setMaxduration(fdbDosing.getMaxduration());
        dto.setHepaticimpairmentind(fdbDosing.getHepaticimpairmentind());
        dto.setRenalimpairmentind(fdbDosing.getRenalimpairmentind());
        dto.setCrclthreshhold(fdbDosing.getCrclthreshhold());
        dto.setCrclthreshholdunits(fdbDosing.getCrclthreshholdunits());
        dto.setWeightrequiredind(fdbDosing.getWeightrequiredind());
        dto.setBsarequiredind(fdbDosing.getBsarequiredind());
        dto.setDoseratelow(fdbDosing.getDoseratelow());
        dto.setDoseratelowunits(fdbDosing.getDoseratelowunits());
        dto.setDoseratehigh(fdbDosing.getDoseratehigh());
        dto.setDoseratehighunits(fdbDosing.getDoseratehighunits());
        dto.setDoseformratelow(fdbDosing.getDoseformratelow());
        dto.setDoseformratelowunits(fdbDosing.getDoseformratelowunits());
        dto.setDoseformratehigh(fdbDosing.getDoseformratehigh());
        dto.setDoseformratehighunits(fdbDosing.getDoseformratehighunits());
        dto.setMaxsingledoserate(fdbDosing.getMaxsingledoserate());
        dto.setMaxsingledoserateunits(fdbDosing.getMaxsingledoserateunits());
        dto.setMaxsingledoseformrate(fdbDosing.getMaxsingledoseformrate());
        dto.setMaxsingledoseformrateunits(fdbDosing.getMaxsingledoseformrateunits());
        dto.setMaxdailydoserate(fdbDosing.getMaxdailydoserate());
        dto.setMaxdailydoserateunits(fdbDosing.getMaxdailydoserateunits());
        dto.setMaxdailydoseformrate(fdbDosing.getMaxdailydoseformrate());
        dto.setMaxdailydoseformrateunits(fdbDosing.getMaxdailydoseformrateunits());
        dto.setDxid(fdbDosing.getDxid());

        // shahid 10-15-2012
        dto.setMaxSingleNTEDose(fdbDosing.getMaxSingleNTEDose());
        dto.setMaxSingleNTEDoseUnit(fdbDosing.getMaxSingleNTEDoseUnit());
        dto.setMaxSingleNTEDoseForm(fdbDosing.getMaxSingleNTEDoseForm());
        dto.setMaxSingleNTEDoseFormUnit(fdbDosing.getMaxSingleNTEDoseFormUnit());

        dto.setHitType(fdbDosing.getHitType());

        return dto;
    }

    /**
     * Creates a list of DTOs from the Dose Range Version list.
     * 
     * @param list
     *            DoseRangeVersionVa list
     * @return a list of DoseRangeDTO object.
     */
    private List<DoseRangeDTO> createDoseRangeDTOs(List<DoseRangeVersionVa> list) {
        ArrayList<DoseRangeDTO> res = new ArrayList<DoseRangeDTO>();
        for (DoseRangeVersionVa v : list) {
            DoseRangeDTO dto = createDoseRangeDTO(v);
            res.add(dto);
        }
        return res;
    }

    /**
     * Retrieves a Dose Range Version by it's identifier and returns it in a DoseRangeDTO object.
     * 
     * @param versionId
     *            the Dose Range Version identifier
     * @return DoseRangeVersionVa
     */
    @Transactional(readOnly = true)
    public List<DoseRangeVersionVa> getStagingDataHistoryByVersionId(Long versionId) {
        List<DoseRangeVersionVa> history = new ArrayList<DoseRangeVersionVa>();

        DoseRangeVersionVa doseRangeVersionVa = getDoseRangeVersionVaDAO().findById(versionId);

        if (doseRangeVersionVa != null) {
            DoseRangeVA doseRangeVA = doseRangeVersionVa.getDoseRangeVa();
            List<Version<DoseRange>> doseRangeHistory = doseRangeVA.getVersionHistory();
            for (Version<DoseRange> version : doseRangeHistory) {

                DoseRangeVersionVa historyVersionVa = (DoseRangeVersionVa) version;
                history.add((historyVersionVa));
            }
        } else {

            LOGGER.info("no history found");

        }

        Collections.reverse(history);

        return history;
    }

    /**
     * Retrieve custom monographs from staging based on values specified in the query.
     * 
     * @param query
     *            the query to execute
     * @param page
     *            the starting page number for the result set
     * @return Page
     * @throws CustomizationServiceException
     */
    @Override
    @SuppressWarnings("unchecked")
    @Transactional
    public Page getStagingData(String query, int page) {
        if (LOGGER.is   ugEnabled()) {
            LOGGER.debug("View custom dose ranges");
        }
        if (query == null || query.equals("")) {
            throw new IllegalArgumentException("required query xml string is null");
        }
        if (page < 1) {
            throw new IllegalArgumentException("required page parameter cannot be less than one");
        }

        Page retpage = doseRangeVersionVaDAO.findByQueryByPage(query, page);
        List<DoseRangeDTO> customDoseRanges = new ArrayList<DoseRangeDTO>();
        for (DoseRangeVersionVa v : (List<DoseRangeVersionVa>) retpage.getResults()) {
            DoseRangeDTO dto = createDoseRangeDTO(v);
            customDoseRanges.add(dto);
        }
        retpage.setResults(customDoseRanges);
        return retpage;
    }

    /**
     * Retrieves the Dose Range Versions for the passed in Concept Identifier, Dose Route Code and Dose Type Code. The result
     * set is returned as a list of DTOs.
     * 
     * @param conceptId
     *            Concept Identifier
     * @param doseRouteCode
     *            Dose Route Code
     * @param doseTypeCode
     *            Dose Type Code
     * @return a list of DoseRangeDTO objects
     */
    @Override
    @Transactional(readOnly = true)
    public List<DoseRangeDTO> getStagingData(Long conceptId, String doseRouteCode, String doseTypeCode) {
        return createDoseRangeDTOs(doseRangeVersionVaDAO.find(conceptId, doseRouteCode, doseTypeCode));
    }

    /**
     * Retrieves a unique DoseRangeDTO object from the database and returns it. If no Dose Ranges or more than 1 Dose Range is
     * found, a null value is returned.
     * 
     * @param conceptId
     *            Concept Identifier
     * @param conceptType
     *            Concept Type
     * @param ageLowInDays
     *            Age Low In Days
     * @param ageHighInDays
     *            Age High In Days
     * @param doseRouteId
     *            Dose Route Identifier
     * @param doseTypeId
     *            Dose Type Identifier
     * @param fdbdx
     *            FDBDX
     * @return DoseRangeDTO
     */
    @Override
    @Transactional
    public DoseRangeDTO getStagingData(Long conceptId, Long conceptType, Long ageLowInDays, Long ageHighInDays,
            String doseRouteId, String doseTypeId, String fdbdx) {
        List<DoseRangeVersionVa> doseRanges =
                doseRangeVersionVaDAO.find(conceptId, conceptType, ageLowInDays, ageHighInDays, doseRouteId, doseTypeId, fdbdx);
        if (doseRanges.size() == 1) {
            return createDoseRangeDTO(doseRanges.get(0));
        }
        return null;

    }

    /**
     * Retrieves a Dose Range Version by it's identifier and returns it in a DoseRangeDTO object.
     * 
     * @param versionId
     *            the Dose Range Version identifier
     * @return DoseRangeDTO
     */
    @Override
    @Transactional(readOnly = true)
    public DoseRangeDTO getStagingDataByVersionId(Long versionId) {
        return createDoseRangeDTO(doseRangeVersionVaDAO.findById(versionId));
    }

    /**
     * This method will find and return the current version for the version Id you pass in.
     * 
     * @param versionId
     *            the historic version id
     * 
     * @return the DTO for the current <code>DoseRangeVersionVA</code>
     */
    @Override
    @Transactional(readOnly = true)
    public DoseRangeDTO getCurrentStagingDataByVersionId(Long versionId) {
        DoseRangeVersionVa doseRangeVersionVa = doseRangeVersionVaDAO.findById(versionId);
        DoseRangeVA doseRangeVA = doseRangeVersionVa.getDoseRangeVa();
        return createDoseRangeDTO(doseRangeVA.getCurrentVersion());
    }

    /**
     * Retrieve custom doseranges from staging based on values specified in the query.
     * 
     * @param query
     *            the query to execute
     * @return List<DoseRangeDTO>
     * @throws CustomizationServiceException
     */
    @Transactional
    @SuppressWarnings("unchecked")
    @Override
    public List<DoseRangeDTO> doGetStagingData(ConceptQuery query) {
        if (LOGGER.is   ugEnabled()) {
            LOGGER.debug("View staging custom doseranges");
        }

        List<DoseRangeVersionVa> customDoseRanges = null;
        if (query == null) {
            customDoseRanges = doseRangeVersionVaDAO.findAll();
        }

        customDoseRanges = (List<DoseRangeVersionVa>) doseRangeVersionVaDAO.findByQuery(query, true); // true for staging and
                                                                                                      // false for FDB
        List<DoseRangeDTO> res = createDoseRangeDTOs(customDoseRanges);
        return res;
    }

    @Override
    @Transactional
    public DoseRangeDTO getFdbData(DoseRangeKey key) {
        DoseRangeDTO dto = null;
        FdbDosing doseRange = fdbDosingDAO.find(key);
        if (doseRange != null) {
            dto = convertFdbToCustom(doseRange);
        }

        return dto;
    }

    @Override
    @Transactional
    public FdbDosing getFdbDosing(DoseRangeKey key) {

        return fdbDosingDAO.find(key);
    }

    /**
     * Creates a new version record.
     * 
     * @param record
     *            the new version to save.
     * @param err
     *            a BindingResult for passing errors back to the UI
     * @return DoseRangeDTO
     */
    @Override
    @Transactional
    public DoseRangeDTO createNewRecord(DoseRangeDTO record, BindingResult err) {
        if (record == null)
            throw new IllegalArgumentException("required dosing object is null");

        long doseRangeId = record.getFdbCustomDosingVaId();

        if (doseRangeId != 0) {
            LOGGER.error("New doseRange shouldn't have its id set");
            throw new CustomizationServiceException("Invalid doseRange - id cannot be different from 0");
        }
        if (LOGGER.is   ugEnabled())
            LOGGER.debug("Creating new dosing record in Service Impl layer");

        if (!validateNewRecord(record, err)) {
            return record;
        }

        // Do this after the record has been validated so that we know Concept Id and Concept Type
        // are populated.
        if (getConceptDescription(record.getConceptid(), record.getConcepttype()) == null) {
            err.addError(new ObjectError("DoseRangeDTO",
                    "Cannot customize. Invalid Concept Type/Concept ID Number combination."));
            return record;
        }
        DoseRangeXref xref = (DoseRangeXref) record.getConceptXref();

        if (xref != null) {
            xref = (DoseRangeXref) createConceptXrefForCustomization(xref.getDoseRangeKey());

        }

        DoseRangeVA dest = DoseRangeVA.createDoseRange();
        dest.setConceptXref(xref);
        dest.setStatus(getStateMachineFactory().getStatusByCode(Status.STATUS_NEW));
        dest.setActionBy(record.getActionBy());
        return populateAndPersistDoseRange(dest, record, false);
    }

    /**
     * Additional validations needed in this concept.
     * 
     * @param meta
     *            ConceptMetadata
     * @return BusinessRulesValidation
     */
    @Override
    protected BusinessRulesValidation getDataValidation(ConceptMetadata meta) {
        // all required fields must be present
        BusinessRulesValidationImpl validator = new BusinessRulesValidationImpl(meta);
        validator.registerRule(new ReferenceTextFieldRule(DoseRangeDTO.REFERENCE_TEXT), new AlwaysFireTrigger());
        return validator;
    }

    /**
     * Retrieves a RequestCountSupport object for the Dose Range concept.
     * 
     * @return RequestCountSupport
     */
    @Override
    protected RequestCountSupport getRequestCountSupport() {
        return new GenericRequestCountSupport(DoseRangeVA.class, doseRangeVaDAO);
    }

    /**
     * Returns the DoseRangeVaDAO object in use.
     * 
     * @return DoseRangeVaDAO
     */
    public DoseRangeVaDAO getDoseRangeVaDAO() {
        return doseRangeVaDAO;
    }

    /**
     * Sets the DoseRangeVaDAO object to use.
     * 
     * @param doseRangeVaDAO
     *            DoseRangeVaDAO
     */
    public void setDoseRangeVaDAO(DoseRangeVaDAO doseRangeVaDAO) {
        this.doseRangeVaDAO = doseRangeVaDAO;
    }

    /**
     * Returns the DoseRangeVersionVaDAO object in use.
     * 
     * @return DoseRangeVersionVaDAO
     */
    public DoseRangeVersionVaDAO getDoseRangeVersionVaDAO() {
        return doseRangeVersionVaDAO;
    }

    /**
     * Sets the DoseRangeVersionVaDAO object to use.
     * 
     * @param doseRangeVersionVaDAO
     *            DoseRangeVersionVaDAO
     */
    public void setDoseRangeVersionVaDAO(DoseRangeVersionVaDAO doseRangeVersionVaDAO) {
        this.doseRangeVersionVaDAO = doseRangeVersionVaDAO;
    }

    /**
     * Returns the FdbDosingDAO object in use.
     * 
     * @return FdbDosingDAO
     */
    public FdbDosingDAO getFdbDosingDAO() {
        return fdbDosingDAO;
    }

    /**
     * Sets the FdbDosingDAO object to use.
     * 
     * @param fdbDosingDAO
     *            FdbDosingDAO
     */
    public void setFdbDosingDAO(FdbDosingDAO fdbDosingDAO) {
        this.fdbDosingDAO = fdbDosingDAO;
    }

    /**
     * Retrieves the metadata for the Dose Range.
     * 
     * @return ConceptMetadata
     */
    @Override
    @Transactional
    public ConceptMetadata getMetadata() {
        return getMetadataService().getMetadata(DoseRangeDTO.class);
    }

    /**
     * Returns the DAO supporting either Custom or FDB Dose Range based upon the custom parameter.
     * 
     * @param custom
     *            if true, return DAO supporting Custom Dose Range. If false, return FDB.
     * @return HibernateDAO
     */
    @Override
    public HibernateDAO<?> getDao(boolean custom) {
        // return custom? doseRangeVaDAO: fdbDosingDAO;
        // shahid 4-17-2011 to fix export to excel issue
        return custom ? doseRangeVersionVaDAO : fdbDosingDAO;

    }

    /**
     * Allows the DrugConceptDao to be set.
     * 
     * @param dao
     *            DrugConceptDao
     */
    public void setDrugConceptDao(DrugConceptDao dao) {
        drugConceptDao = dao;
    }

    /**
     * Allows the class to determine if all injectable beans have been set.
     * 
     * @throws Exception
     *             if an injectable bean has not been set
     */
    @Override
    public void afterPropertiesSet() throws Exception {
        // cache dose routes and dose types
        // that should be safe, since those values can only be changed through
        // a SQL script and to run that we'd probably take the application down
        super.afterPropertiesSet();
        if (drugConceptDao == null) {
            throw new Exception("DrugConceptDao has not been injected in to DoseRangeServiceImpl.");
        }
        doseTypeMap = new HashMap<String, DoseType>();
        doseRouteMap = new HashMap<String, DoseRoute>();

        List<DoseType> typeList = super.getTerminologyService().getDoseTypes();
        for (DoseType model : typeList) {
            doseTypeMap.put(model.getCode(), model);
        }
        List<DoseRoute> routeList = super.getTerminologyService().getDoseRoutes();
        for (DoseRoute model : routeList) {
            doseRouteMap.put(model.getCode(), model);
        }

    }

    /**
     * Retrieves the Concept Description for the Concept Identifier/Concept Type pair. A null value is returned if either of the
     * arguments are null or if the Concept Description cannot be found in FDB.
     * 
     * @param conceptId
     *            Concept Identifier
     * @param conceptType
     *            Concept Type
     * @return String
     */
    @Override
    public String getConceptDescription(Long conceptId, Long conceptType) {
        if (conceptId == null || conceptType == null) {
            return null;
        }
        return drugConceptDao.getDrugConceptDescription(conceptId, conceptType);
    }

    /**
     * Pre-process the user query to account for the Concept Id Description that may be included in the criteria. This method is
     * used for both Custom and FDB queries.
     * 
     * @param query
     *            the query to process
     * @param custom
     *            - boolean true when custom
     * @return the processed query
     */
    @Override
    protected ConceptQuery preProcessQuery(ConceptQuery query, boolean custom) {
        ConceptQuery preProcessed = query.copy();
        DoseRangeVisitor visitor = new DoseRangeVisitor(getMetadataService(), drugConceptDao);
        preProcessed.accept(visitor);
        return super.preProcessQuery(preProcessed, custom);
    }

    @SuppressWarnings({ "rawtypes", "unchecked" })
    @Override
    protected List getCustomHistoryForField(ConceptHistoryCriteria criteria) {

        List<DoseRangeVersionVa> historyList = getStagingDataHistoryByVersionId(criteria.getVersionId());

        if (historyList.size() < 1) {
            return new ArrayList();
        }
        criteria.setConceptXref(historyList.get(0).getVersionedObject().getConceptXref());

        return historyList;
    }

    @Override
    protected FieldHistoryLineItem getFDBHistoryforField(ConceptHistoryCriteria criteria, ConceptField conceptField,
            Object retainedFieldValue) {
        FieldHistoryLineItem fdbFieldHistoryLineItem = null;

        DoseRangeXref xref = (DoseRangeXref) criteria.getConceptXref();
        if (xref == null) {
            return fdbFieldHistoryLineItem;
        }

        FdbDosing fdbDosing = getFdbDosing(xref.getDoseRangeKey());
        if (fdbDosing != null) {
            Object fdbFieldValue = conceptField.getPersistentValue(false, fdbDosing);

            if (!fdbFieldValue.equals(retainedFieldValue)) {
                retainedFieldValue = fdbFieldValue;
                fdbFieldHistoryLineItem = populateFieldHistoryLineItemFDB(criteria, conceptField, fdbDosing);

            }
        }

        return fdbFieldHistoryLineItem;

    }

    public final FieldHistoryLineItem populateFieldHistoryLineItemFDB(ConceptHistoryCriteria criteria,
            ConceptField conceptField, FdbDosing fdbDosing) {
        FieldHistoryLineItem fdbFieldHistoryLineItem = new FieldHistoryLineItem();

        Object fieldValue = null;

        if (conceptField.isLookup()) {

            fieldValue = determineFdbFieldValue(conceptField, fdbDosing);

        } else {

            fieldValue = conceptField.getPersistentValue(false, fdbDosing);

        }

        fdbFieldHistoryLineItem.setUser("FDB");

        fdbFieldHistoryLineItem.setId(criteria.getConceptXref().toString());

        fdbFieldHistoryLineItem.setStatus("FDB");
        fdbFieldHistoryLineItem.setValue(fieldValue.toString());
        return fdbFieldHistoryLineItem;
    }

    /**
     * Determine fdb field value.
     * 
     * @param conceptField
     *            the concept field
     * @param fdbDosing
     *            the fdb dosing
     * @return the object
     */
    private Object determineFdbFieldValue(ConceptField conceptField, FdbDosing fdbDosing) {

        return super.getTerminologyService()
                .getByCode(conceptField.getLookupType(), conceptField.getPersistentValue(false, fdbDosing).toString())
                .getUiLabel();

    }

    /**
     * Retrieves the history data for a Duplicate Therapy for inclusion on the history report.
     * 
     * @param versionId
     *            the version id of the Duplicate Therapy Version being viewed
     * @return List<DoseRangeDTO>
     */
    @Override
    @Transactional
    public List<DoseRangeDTO> getVersionHistoryReportData(Long versionId) {
        List<DoseRangeDTO> versionHistory = new ArrayList<DoseRangeDTO>(0);
        try {

            DoseRangeVersionVa doseRangeVersionVa = doseRangeVersionVaDAO.findById(versionId);
            DoseRangeVA version = doseRangeVersionVa.getDoseRangeVa();

            getVersionHistory(version, versionHistory);
            if (version.getConceptXref() != null) {
                DoseRangeXref xref = (DoseRangeXref) version.getConceptXref();
                DoseRangeDTO fdbDTO = convertFdbToCustom(fdbDosingDAO.find(xref.getDoseRangeKey()));
                fdbDTO.setFdbId("0");
                versionHistory.add(fdbDTO);
            }

        } catch (Exception e) {
            LOGGER.error(e.getMessage(), e);
        }
        return versionHistory;
    }

    /**
     * A method to loop through the version history of a Dose Range and create DoseRangeDTO objects from that data.
     * 
     * @param doseRangeVa
     *            the Dose Range VA selected by the user
     * @param versionHistory
     *            the List to place the DoseRangeDTO objects in
     */
    private void getVersionHistory(DoseRangeVA doseRangeVa, List<DoseRangeDTO> versionHistory) {
        List<Version<DoseRange>> versions = doseRangeVa.getVersionHistory();
        for (Version<DoseRange> version : versions) {
            versionHistory.add(createDoseRangeDTOWithComment((DoseRangeVersionVa) version));
        }
        Collections.reverse(versionHistory);
    }

    /**
     * Create a Dose Range DTO with Comment included.
     * 
     * @param version
     *            - DoseRangeVersionVa
     * @return DoseRangeDTO - the dose range DTO
     */
    private DoseRangeDTO createDoseRangeDTOWithComment(DoseRangeVersionVa version) {
        DoseRangeDTO dto = createDoseRangeDTO(version);
        dto.setCurrentActionReason(version.getComment());
        return dto;
    }

    @Override
    public ConceptXref createConceptXref(Object key, UserComment comment) {
        DoseRangeXref xref = new DoseRangeXref();
        xref.setDoseRangeKey((DoseRangeKey) key);
        xref.addUserComment(comment);
        return xref;
    }

    @Override
    @Transactional
    public ConceptXref findXrefByFDBId(Object fdbId) {
        return getConceptXrefDAO().findDoseRangexRefByFDBId((DoseRangeKey) fdbId);
    }

    @Override
    @Transactional
    protected ConceptXref createConceptXrefForCustomization(Object fdbId) {

        // find xref if it already exists.
        DoseRangeXref xref = (DoseRangeXref) findXrefByFDBId(fdbId);
        // if it does not exist then create a new one set the id and save it.
        if (xref == null) {
            xref = new DoseRangeXref();
            DoseRangeKey id = (DoseRangeKey) fdbId;
            xref.setDoseRangeKey(id);
            persistConceptXref(xref);
        }

        return xref;
    }

    @Override
    @Transactional
    public List<DoseRangeDTO> findCustomByXref(DoseRangeKey key) {
        // Must send in DoseRangeKey with FDB Record values
        List<DoseRangeVersionVa> customRecords = getConceptXrefDAO().findCustomByXref(key);
        return createDoseRangeDTOs(customRecords);
    }

    /* (non-Javadoc)
     * @see gov.va.med.pharmacy.pecs.service.impl.AbstractPecsService#getFdbDao()
     */
    @Override
    protected FdbHibernateDAOImpl<?> getFdbDao() {
        return getFdbDosingDAO();
    }

}
