package gov.va.med.pharmacy.pecs.service.impl;

import gov.va.med.pharmacy.pecs.criteria.ConceptHistoryCriteria;
import gov.va.med.pharmacy.pecs.domain.DrugDrugInteraction;
import gov.va.med.pharmacy.pecs.domain.UserComment;
import gov.va.med.pharmacy.pecs.domain.Version;
import gov.va.med.pharmacy.pecs.domain.impl.fdb.FdbDdiminteraction;
import gov.va.med.pharmacy.pecs.domain.impl.fdb.FdbDrugDrugInteractionDAO;
import gov.va.med.pharmacy.pecs.domain.impl.va.AbstractCustomizable;
import gov.va.med.pharmacy.pecs.domain.impl.va.ConceptXref;
import gov.va.med.pharmacy.pecs.domain.impl.va.DdiminteractionReferenceVa;
import gov.va.med.pharmacy.pecs.domain.impl.va.DrugDrugInteractionVa;
import gov.va.med.pharmacy.pecs.domain.impl.va.DrugDrugInteractionVersionVa;
import gov.va.med.pharmacy.pecs.domain.impl.va.DrugDrugInteractionXref;
import gov.va.med.pharmacy.pecs.dto.DrugDrugInteractionDTO;
import gov.va.med.pharmacy.pecs.exception.CustomizationServiceException;
import gov.va.med.pharmacy.pecs.metadata.ConceptField;
import gov.va.med.pharmacy.pecs.metadata.ConceptMetadata;
import gov.va.med.pharmacy.pecs.persistence.DrugDrugInteractionDAOImpl;
import gov.va.med.pharmacy.pecs.persistence.FdbHibernateDAOImpl;
import gov.va.med.pharmacy.pecs.persistence.GenericDAO;
import gov.va.med.pharmacy.pecs.persistence.HibernateDAO;
import gov.va.med.pharmacy.pecs.query.ConceptQuery;
import gov.va.med.pharmacy.pecs.query.DrugDrugInteractionVisitor;
import gov.va.med.pharmacy.pecs.query.Page;
import gov.va.med.pharmacy.pecs.rules.AlwaysFireTrigger;
import gov.va.med.pharmacy.pecs.rules.BusinessRulesValidation;
import gov.va.med.pharmacy.pecs.rules.BusinessRulesValidationImpl;
import gov.va.med.pharmacy.pecs.rules.LargeTextFieldRule;
import gov.va.med.pharmacy.pecs.rules.RegexRule;
import gov.va.med.pharmacy.pecs.service.DrugDrugInteractionService;
import gov.va.med.pharmacy.pecs.service.RequestCountSupport;
import gov.va.med.pharmacy.pecs.service.dto.FieldHistoryLineItem;
import gov.va.med.pharmacy.pecs.state.Status;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;

import javax.annotation.Resource;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.BindingResult;

/**
 * Service to access the Drug Drug Interaction data.
 * 
 * @author vhaisacarusr
 * 
 */
@Service
public final class DrugDrugInteractionServiceImpl extends AbstractPecsService<DrugDrugInteractionDTO, DrugDrugInteraction>
        implements DrugDrugInteractionService, InitializingBean {

    /** The Constant LOGGER. */
    private static final Log LOGGER = LogFactory.getLog(DrugDrugInteractionServiceImpl.class);

    /** The drug drug interaction dao. */
    @Resource
    private DrugDrugInteractionDAOImpl drugDrugInteractionDAO;

    /** The generic dao. */
    @Resource
    private GenericDAO genericDAO;

    /** The fdb drug drug interaction dao. */
    @Resource
    private FdbDrugDrugInteractionDAO fdbDrugDrugInteractionDAO;

    @Transactional(readOnly = true)
    @Override
    public DrugDrugInteractionDTO getStagingData(Long identifier) {

        DrugDrugInteractionVersionVa ddiv = drugDrugInteractionDAO.findVersionById(identifier);
        return createDTO(ddiv);
    }

    /**
     * Gets the staging data history by version id.
     * 
     * @param criteria
     *            the criteria
     * @return the staging data history by version id
     */
    @Transactional(readOnly = true)
    public List<DrugDrugInteractionVersionVa> getStagingDataHistoryByVersionId(ConceptHistoryCriteria criteria) {

        List<DrugDrugInteractionVersionVa> history = new ArrayList<DrugDrugInteractionVersionVa>();

        if (criteria.getVersionId() != 0) {
            DrugDrugInteractionVersionVa versionVa = drugDrugInteractionDAO.findVersionById(criteria.getVersionId());
            if (versionVa != null) {

                criteria.setId(versionVa.getFdbId());

                List<Version<DrugDrugInteraction>> historyList = versionVa.getVersionHistory();

                for (Version<DrugDrugInteraction> version : historyList) {

                    DrugDrugInteractionVersionVa historyVersionVa = (DrugDrugInteractionVersionVa) version;
                    history.add((historyVersionVa));
                }
                Collections.reverse(history);
            }
        }

        return history;

    }

    @Transactional(readOnly = true)
    @Override
    public List<DrugDrugInteractionDTO> getStagingByFdbId(Long fdbId) {
        return createCurrentDTOs(drugDrugInteractionDAO.findDDIByFdbId(fdbId));
    }

    @Override
    @Transactional(readOnly = true)
    public List<DrugDrugInteractionDTO> getStagingListByInteractionId(Long interactionId) {
        return createCurrentDTOs(drugDrugInteractionDAO.findDDIListByInteractionID(interactionId));
    }

    @Transactional(readOnly = true)
    @Override
    public DrugDrugInteractionDTO getStagingByInteractionId(Long interactionId) {
        return createDTO(drugDrugInteractionDAO.findDDIByInteractionId(interactionId).getCurrentVersion());
    }

    @Transactional(readOnly = true)
    @Override
    public DrugDrugInteractionVa getDDIByInteractionId(Long interactionId) {
        return drugDrugInteractionDAO.findDDIByInteractionId(interactionId);
    }

    @Transactional(readOnly = true)
    @SuppressWarnings("unchecked")
    @Override
    public List<DrugDrugInteractionDTO> doGetStagingData(ConceptQuery query) {
        List<DrugDrugInteractionVersionVa> ddis =
                (query == null ? drugDrugInteractionDAO.findAll(DrugDrugInteractionVersionVa.class)
                        : (List<DrugDrugInteractionVersionVa>) drugDrugInteractionDAO.findByQuery(query, true));
        return createDDIDTOs(ddis);
    }

    @Transactional(readOnly = true)
    @Override
    public Page getStagingData(String query, int page) {
        if (LOGGER.is   ugEnabled()) {
            LOGGER.debug("View custom Drug-Drug Interactions");
        }
        if (query == null || query.equals("")) {
            throw new IllegalArgumentException("required query xml string is null");
        }
        if (page < 1) {
            throw new IllegalArgumentException("required page parameter cannot be less than one");
        }

        Page retPage = drugDrugInteractionDAO.findByQueryByPage(query, page);
        @SuppressWarnings("unchecked")
        List<DrugDrugInteractionDTO> ddis = createDDIDTOs(retPage.getResults());
        retPage.setResults(ddis);
        return retPage;
    }

    @Transactional(readOnly = true)
    @Override
    public DrugDrugInteractionDTO findFdbById(Long id) {
        FdbDdiminteraction fdbDdi = getFdbDrugDrugInteractionDAO().findById(id);
        return createDTO(fdbDdi);
    }

    @Override
    @Transactional
    public DrugDrugInteractionDTO createNewRecord(DrugDrugInteractionDTO dto, BindingResult result) {
        if (dto.getInteractionid() != 0) {
            LOGGER.error("New Drug-Drug Interaction shouldn't have its Interaction Id set");
            throw new CustomizationServiceException("Invalid Interaction Id cannot be different from 0");
        }

        if (!validateNewRecord(dto, result)) {
            return dto;
        }

        DrugDrugInteractionVa ddi = checkForDuplicate(dto);
        if (ddi == null) {
            ddi = DrugDrugInteractionVa.create();
            ddi.setStatus(getStateMachineFactory().getStatusByCode(Status.STATUS_NEW));
            ddi.setActionBy(dto.getActionBy());
            return populateAndPersist(ddi, dto, false);
        } else {
            // return existing Drug-Drug Interaction for display
            DrugDrugInteractionDTO duplicateDTO = createDTO(ddi.getCurrentVersion());
            duplicateDTO.setDuplicate(true);
            duplicateDTO.setFdbId(dto.getFdbId());
            duplicateDTO.setReverseFdbId(dto.getReverseFdbId());
            return duplicateDTO;
        }
    }

    /**
     * Check for duplicate.
     * 
     * @param record
     *            the record to be checked
     * @return the duplicate if there is one <code>null</code> if there is not
     */
    private DrugDrugInteractionVa checkForDuplicate(DrugDrugInteractionDTO record) {
        if (record.getFdbId() == 0L) {
            return null;
        }

        // This is a throw away object since the rules to generate the reverse Description are in the
        // DrugDrugInteractionVersionVa class.
        DrugDrugInteractionVa ddi = new DrugDrugInteractionVa(true);
        ddi.setDescription(record.getDescription1());

        List<DrugDrugInteractionVa> duplicates =
                drugDrugInteractionDAO.checkForDuplicate(record.getFdbId(), record.getReverseFdbId(),
                        record.getSeveritylevelcode());
        if (duplicates != null && duplicates.size() > 0) {
            for (DrugDrugInteractionVa duplicate : duplicates) {
                if (duplicate.getInteractionId() != record.getInteractionid()) {
                    return duplicate;
                }
            }
        }
        return null;
    }

    @Override
    @Transactional
    public DrugDrugInteractionDTO applyAction(String action, DrugDrugInteractionDTO record, BindingResult err) {
        DrugDrugInteractionVa duplicateDDI = checkForDuplicate(record);
        if (duplicateDDI == null) {
            DrugDrugInteractionVa ddi = findPersistentObject(record);
            DrugDrugInteractionVersionVa v = ddi.getCurrentVersion();
            DrugDrugInteractionDTO oldData = createDTO(v);

            boolean stateChange = doApplyAction(ddi, record, oldData, action, record.getActionUser(), err);
            // set action by only when the state machine was invoked,
            // but not when a roll-back took place
            if (stateChange && !ddi.wasRolledBack()) {
                ddi.setActionBy(record.getActionBy());
            }

            record.setExportDate(null);
            return err.hasErrors() ? record : populateAndPersist(ddi, record, ddi.wasRolledBack());
        } else {
            DrugDrugInteractionDTO data = createDTO(duplicateDDI.getCurrentVersion());
            data.setDuplicate(true);
            return data;
        }
    }

    @Override
    protected RequestCountSupport getRequestCountSupport() {
        return new GenericRequestCountSupport(DrugDrugInteractionVa.class, drugDrugInteractionDAO);
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        if (getTerminologyService() == null) {
            throw new Exception("terminologyService must be set in DrugDrugInteractionServiceImpl.");
        }
        if (drugDrugInteractionDAO == null) {
            throw new Exception("drugDrugInteractionDAO must be set in DrugDrugInteractionServiceImpl.");
        }
        if (genericDAO == null) {
            throw new Exception("GenericDAO must be set in DrugDrugInteractionServiceImpl.");
        }
        if (getFdbDrugDrugInteractionDAO() == null) {
            throw new Exception("FdbDrugDrugInteractionDAO must be set in DrugDrugInteractionServiceImpl.");
        }
        super.afterPropertiesSet();
    }

    /**
     * Takes the data from a DrugDrugInteractionDTO class and copies the data to a DrugDrugInteractionVa and
     * DrugDrugInteractionVersionVa class. These classes are then persisted. A DrugDrugInteractionDTO class is returned with the
     * populated identifiers from the persistent store.
     * 
     * @param dest
     *            the destination object
     * @param source
     *            the source object
     * @param partial
     *            true if only a partial population is necessary
     * 
     * @return DrugDrugInteractionDTO
     */
    private DrugDrugInteractionDTO
            populateAndPersist(DrugDrugInteractionVa dest, DrugDrugInteractionDTO source, boolean partial) {
        dest.setRequestor(source.getRequestor());
        dest.setFdbId(source.getFdbId() == 0 ? null : source.getFdbId());
        dest.setReverseFdbId(source.getReverseFdbId() == 0 ? null : source.getReverseFdbId());
        // FDB new calls this method. Export Date does not need to be copied over
        // because it is null for a FDB record, but it would not hurt if we need to copy it over because it is null.
        // IF VA approved and exported we do not want to copy over the export date.
        if (!partial) {
            dest.setClinicalEffect1(getTerminologyService().getClinicalEffectByCode(source.getClinicaleffectcode1()));
            dest.setClinicalEffect2(getTerminologyService().getClinicalEffectByCode(source.getClinicaleffectcode2()));
            dest.setDescription(source.getDescription1());
            if (source.getMonographid() == null) {
                source.setMonographid(0L);
            }
            dest.setMonographId(source.getMonographid() == 0L ? null : source.getMonographid());
            dest.setSeverityLevel(getTerminologyService().getSeverityLevelByCode(source.getSeveritylevelcode()));
            if (source.getDdiminteractionReferenceVa() != null) {
                final DdiminteractionReferenceVa refVa = source.getDdiminteractionReferenceVa();

                dest.setAidsGuideline(refVa.getReferenceTextVaByAidsGuidelines());
                dest.setAidsGuidelineText(refVa.getAidsGuidelinesText());
                dest.setDiFactDocumentation(refVa.getReferenceTextVaByDiFactsDocumentation());
                dest.setDiFactNumber(refVa.getReferenceTextVaByDiFactsNumber());
                dest.setDiFactOnset(refVa.getReferenceTextVaByDiFactsOnset());
                dest.setDiFactSeverity(refVa.getReferenceTextVaByDiFactsSeverity());
                dest.setDiFactText(refVa.getDiFactsText());
                dest.setEdiNumber(refVa.getReferenceTextVaByEdiNumber());
                dest.setEdiText(refVa.getEdiText());
                dest.setInteractionSource(refVa.getReferenceTextVaByInteractionSource());
                dest.setInteractionType(refVa.getReferenceTextVaByInteractionType());
                dest.setLevelOfEvidence(refVa.getReferenceTextVaByLevelOfEvidence());
                dest.setMedlineHit(refVa.getReferenceTextVaByMedlineHits());
                dest.setMedlineText(refVa.getMedlineText());
                dest.setMicromedexOnset(refVa.getReferenceTextVaByMicromedexOnset());
                dest.setMicromedexSeverity(refVa.getReferenceTextVaByMicromedexSeverity());
                dest.setMicromedexSubstantiation(refVa.getReferenceTextVaByMicromedexSubstantiation());
                dest.setMicromedexText(refVa.getMicromedexText());
                dest.setPackageInsert(refVa.getReferenceTextVaByPackageInsert());
                dest.setPackageInsertText(refVa.getPackageInsertText());
                dest.setPbmCriteria(refVa.getReferenceTextVaByPbmCriteria());
                dest.setPbmCriteriaText(refVa.getPbmCriteriaText());
                dest.setGroupDiscussion(refVa.getGroupDiscussion());

                dest.setAssignedUser(source.getAssignedTo());
            }
        }

        dest.setAuditUser(source.getActionBy());
        dest.setComment(source.getCurrentActionReason());

        dest.setModifiedDate(new Date());

        dest.setConceptXref(createConceptXrefForCustomization(source.getFdbId()));

        drugDrugInteractionDAO.save(dest);
        return createDTO(dest.getCurrentVersion());
    }

    /**
     * Takes a DrugDrugInteractionVersionVa object and copies the data to a DrugDrugInteractionDTO object.
     * 
     * @param incomingSource
     *            the object from the database
     * @return DrugDrugInteractionDTO
     */
    @SuppressWarnings("unchecked")
    private DrugDrugInteractionDTO createDTO(Version<DrugDrugInteraction> incomingSource) {
        DrugDrugInteractionDTO dto = new DrugDrugInteractionDTO();
        DrugDrugInteractionVersionVa source = (DrugDrugInteractionVersionVa) incomingSource;
        if (dto.getDdiminteractionReferenceVa() == null) {
            dto.setDdiminteractionReferenceVa(new DdiminteractionReferenceVa());
        }
        final DdiminteractionReferenceVa referenceVa = dto.getDdiminteractionReferenceVa();

        dto.setActionBy(source.getActionBy());
        dto.setActionReason(source.getCommentHistory());

        if (source.getAidsGuideline() != null) {
            referenceVa.setReferenceTextVaByAidsGuidelines(source.getAidsGuideline());
        }
        referenceVa.setAidsGuidelinesText(source.getAidsGuidelineText());
        dto.setAssignedTo(source.getAssignedUser());
        if (source.getClinicalEffect1() != null) {
            dto.setClinicaleffectcode1(source.getClinicalEffect1().getCode());
        }
        if (source.getClinicalEffect2() != null) {
            dto.setClinicaleffectcode2(source.getClinicalEffect2().getCode());
        }
        dto.setActionDate(source.getDate());
        dto.setExportDate(source.getExportDate());
        dto.setDdiminteractionVaId(source.getVersionId());
        dto.setDescription1(source.getDescription());
        if (source.getDiFactDocumentation() != null) {
            referenceVa.setReferenceTextVaByDiFactsDocumentation(source.getDiFactDocumentation());
        }
        referenceVa.setReferenceTextVaByDiFactsNumber(source.getDiFactNumber());
        referenceVa.setReferenceTextVaByDiFactsOnset(source.getDiFactOnset());
        referenceVa.setReferenceTextVaByDiFactsSeverity(source.getDiFactSeverity());
        referenceVa.setDiFactsText(source.getDiFactText());
        referenceVa.setReferenceTextVaByEdiNumber(source.getEdiNumber());
        referenceVa.setEdiText(source.getEdiText());
        if (source.getFdbId() != null) {
            dto.setFdbId(source.getFdbId().longValue());
            dto.setReverseFdbId(source.getReverseFdbId().longValue());
        }
        dto.setInteractionid(source.getInteractionId());
        referenceVa.setReferenceTextVaByInteractionSource(source.getInteractionSource());
        referenceVa.setReferenceTextVaByInteractionType(source.getInteractionType());
        referenceVa.setReferenceTextVaByLevelOfEvidence(source.getLevelOfEvidence());
        referenceVa.setReferenceTextVaByMedlineHits(source.getMedlineHit());
        referenceVa.setMedlineText(source.getMedlineText());
        referenceVa.setReferenceTextVaByMicromedexOnset(source.getMicromedexOnset());
        referenceVa.setReferenceTextVaByMicromedexSeverity(source.getMicromedexSeverity());
        referenceVa.setReferenceTextVaByMicromedexSubstantiation(source.getMicromedexSubstantiation());
        referenceVa.setMicromedexText(source.getMicromedexText());
        if (source.getMonographId() != null) {
            dto.setMonographid(source.getMonographId().longValue());
        }
        referenceVa.setReferenceTextVaByPackageInsert(source.getPackageInsert());
        referenceVa.setPackageInsertText(source.getPackageInsertText());
        referenceVa.setReferenceTextVaByPbmCriteria(source.getPbmCriteria());
        referenceVa.setPbmCriteriaText(source.getPbmCriteriaText());
        referenceVa.setGroupDiscussion(source.getGroupDiscussion());

        dto.setRequestor(((DrugDrugInteractionVa) source.getVersionedObject()).getRequestor());
        dto.setSeqno(1L);
        if (source.getSeverityLevel() != null) {
            dto.setSeveritylevelcode(source.getSeverityLevel().getCode());
        }
        dto.setStatus(source.getStatus().getName());
        dto.setUicategory1("VA");
        dto.setReadOnly(incomingSource.isModifiable());
        dto.setCurrent(source.isCurrent());
        dto.setMasterRecordId(source.getVersionedObject().getId());
        dto.setConceptXref(((AbstractCustomizable<DrugDrugInteraction>) source.getVersionedObject()).getConceptXref());
        dto.setUserCommentHistory(source.getVersionedObject().getUserCommentHistory());

        return dto;
    }

    /**
     * Creates the dto.
     * 
     * @param source
     *            the source object from the database
     * @return a populated DTO of type <code>DrugDrugInteractionDTO</code>
     */
    private DrugDrugInteractionDTO createDTO(FdbDdiminteraction source) {
        // Pre-populate custom models with fields from FDB
        DrugDrugInteractionDTO dest = new DrugDrugInteractionDTO();
        dest.setFdbId(source.getInteractionid());
        dest.setReverseFdbId(source.getReverseFdbId());
        dest.setInteractionid(source.getInteractionid());
        dest.setDescription1(source.getDescription1());
        dest.setMonographid(source.getMonographid());
        dest.setSeveritylevelcode(source.getSeveritylevelcode());
        dest.setClinicaleffectcode1(source.getClinicaleffectcode1());
        dest.setClinicaleffectcode2(source.getClinicaleffectcode2());
        dest.setUserCommentHistory(source.getUserCommentHistory());
        dest.setDocAnimalStudyInd(source.getDocanimalstudyind());
        dest.setDocCaseRptInd(source.getDoccaserptind());
        dest.setDocHumanTrialInd(source.getDochumantrialind());
        dest.setDocMfgInd(source.getDocmfgind());
        dest.setDocMtgAbstractInd(source.getDocmtgabstractind());
        dest.setDocRvwArticleInd(source.getDocrvwarticleind());
        dest.setEdiPageRef(source.getEdipageref());
        dest.setConceptXref(source.getConceptXref());
        dest.setCurrent(true);

        return dest;
    }

    @Override
    protected List<DrugDrugInteractionDTO> convertFdbToCustom(List<?> fdbObjects) {
        List<DrugDrugInteractionDTO> res = new ArrayList<DrugDrugInteractionDTO>();
        for (Object obj : fdbObjects) {
            res.add(createDTO((FdbDdiminteraction) obj));
        }
        return res;
    }

    /**
     * Creates the current dt os.
     * 
     * @param interactions
     *            a list of <code>DrugDrugInteractionVa</code>'s
     * @return a list of <code>DrugDrugInteractionDTO</code>. Never <code>null</code>.
     */
    private List<DrugDrugInteractionDTO> createCurrentDTOs(List<DrugDrugInteractionVa> interactions) {
        List<DrugDrugInteractionDTO> versions = new ArrayList<DrugDrugInteractionDTO>();
        for (DrugDrugInteractionVa ddi : interactions) {
            versions.add(createDTO(ddi.getCurrentVersion()));
        }
        return versions;
    }

    /**
     * Creates the ddidt os.
     * 
     * @param list
     *            a list of <code>DrugDrugInteractionVersionVa</code>
     * @return a list of <code>DrugDrugInteractionDTO</code>. Never <code>null</code>.
     */
    private List<DrugDrugInteractionDTO> createDDIDTOs(List<DrugDrugInteractionVersionVa> list) {
        List<DrugDrugInteractionDTO> res = new ArrayList<DrugDrugInteractionDTO>();
        for (DrugDrugInteractionVersionVa ddi : list) {
            res.add(createDTO(ddi));
        }
        return res;
    }

    @Override
    protected DrugDrugInteractionVa findPersistentObject(DrugDrugInteractionDTO record) {
        long interactionId = record.getInteractionid();
        if (interactionId == 0) {
            LOGGER.error("Attempt to retrieve persistent Drug-Drug Interaction with id of 0");
            throw new CustomizationServiceException("Invalid Drug-Drug Interaction - id cannot be 0");
        }
        DrugDrugInteractionVa ddi = drugDrugInteractionDAO.findDDIByInteractionId(interactionId);
        return ddi;
    }

    @Override
    protected BusinessRulesValidation getDataValidation(ConceptMetadata meta) {
        // all required fields must be present
        BusinessRulesValidationImpl validator = new BusinessRulesValidationImpl(meta);
        validator.registerRule(new RegexRule(DrugDrugInteractionDTO.DESCRIPTION1, "\\w*[^/]*/[^/]*\\w*[^/]*",
                "it must contain two drug names separated by a forward slash /"), new AlwaysFireTrigger());
        validator.registerRule(new RegexRule(DrugDrugInteractionDTO.DESCRIPTION1, ".{0,60}",
                "it can have at most 60 characters"), new AlwaysFireTrigger());

        // Text field validators
        validator.registerRule(new LargeTextFieldRule(DdiminteractionReferenceVa.EDI_TEXT,
                "it can have at most 4000 characters"), new AlwaysFireTrigger());
        validator.registerRule(new LargeTextFieldRule(DdiminteractionReferenceVa.DI_FACTS_TEXT,
                "it can have at most 4000 characters"), new AlwaysFireTrigger());
        validator.registerRule(new LargeTextFieldRule(DdiminteractionReferenceVa.MICROMEDEX_TEXT,
                "it can have at most 4000 characters"), new AlwaysFireTrigger());
        validator.registerRule(new LargeTextFieldRule(DdiminteractionReferenceVa.MEDLINE_TEXT,
                "it can have at most 4000 characters"), new AlwaysFireTrigger());
        validator.registerRule(new LargeTextFieldRule(DdiminteractionReferenceVa.PACKAGE_INSERT_TEXT,
                "it can have at most 4000 characters"), new AlwaysFireTrigger());
        validator.registerRule(new LargeTextFieldRule(DdiminteractionReferenceVa.PBM_CRITERIA_TEXT,
                "it can have at most 4000 characters"), new AlwaysFireTrigger());
        validator.registerRule(new LargeTextFieldRule(DdiminteractionReferenceVa.AIDS_GUIDELINES_TEXT,
                "it can have at most 4000 characters"), new AlwaysFireTrigger());
        validator.registerRule(new LargeTextFieldRule(DdiminteractionReferenceVa.GROUP_DISCUSSION,
                "it can have at most 4000 characters"), new AlwaysFireTrigger());

        return validator;
    }

    @SuppressWarnings("rawtypes")
    @Transactional(readOnly = true)
    @Override
    public List getInteractionDescriptions() {
        return drugDrugInteractionDAO.getInteractionDescriptions();
    }

    @Override
    public HibernateDAO<?> getDao(boolean custom) {
        return custom ? drugDrugInteractionDAO : genericDAO;
    }

    @Override
    @Transactional(readOnly = true)
    public ConceptMetadata getMetadata() {
        return getMetadataService().getMetadata(DrugDrugInteractionDTO.class);
    }

    /**
     * Pre-process the user query to account for the lack of stored reverse drugs interactions.
     * 
     * @param query
     *            the query to process
     * @param custom
     *            true if the record is custom
     * @return the processed query
     */
    @Override
    protected ConceptQuery preProcessQuery(ConceptQuery query, boolean custom) {
        if (custom) {
            ConceptQuery preProcessed = query.copy();
            DrugDrugInteractionVisitor visitor = new DrugDrugInteractionVisitor();
            preProcessed.accept(visitor);
            return super.preProcessQuery(preProcessed, true);
        } else {
            return super.preProcessQuery(query, false);
        }
    }

    /**
     * Gets the drugDrugInteractionDAO.
     * 
     * @return the drugDrugInteractionDAO
     */
    public final DrugDrugInteractionDAOImpl getDrugDrugInteractionDAO() {
        return drugDrugInteractionDAO;
    }

    /**
     * Sets the drugDrugInteractionDAO.
     * 
     * @param drugDrugInteractionDAO
     *            the drugDrugInteractionDAO to set
     */
    public final void setDrugDrugInteractionDAO(DrugDrugInteractionDAOImpl drugDrugInteractionDAO) {
        this.drugDrugInteractionDAO = drugDrugInteractionDAO;
    }

    /**
     * Gets the genericDAO.
     * 
     * @return the genericDAO
     */
    public final GenericDAO getGenericDAO() {
        return genericDAO;
    }

    /**
     * Sets the genericDAO.
     * 
     * @param genericDAO
     *            the genericDAO to set
     */
    public final void setGenericDAO(GenericDAO genericDAO) {
        this.genericDAO = genericDAO;
    }

    /**
     * Gets the fdbDrugDrugInteractionDAO.
     * 
     * @return the fdbDrugDrugInteractionDAO
     */
    public FdbDrugDrugInteractionDAO getFdbDrugDrugInteractionDAO() {
        return fdbDrugDrugInteractionDAO;
    }

    /**
     * Sets the fdbDrugDrugInteractionDAO.
     * 
     * @param fdbDrugDrugInteractionDAO
     *            the fdbDrugDrugInteractionDAO to set
     */
    public void setFdbDrugDrugInteractionDAO(FdbDrugDrugInteractionDAO fdbDrugDrugInteractionDAO) {
        this.fdbDrugDrugInteractionDAO = fdbDrugDrugInteractionDAO;
    }

    @SuppressWarnings({ "rawtypes", "unchecked" })
    @Override
    protected List getCustomHistoryForField(ConceptHistoryCriteria criteria) {

        List<DrugDrugInteractionVersionVa> historyList = getStagingDataHistoryByVersionId(criteria);

        if (historyList.size() < 1) {
            return new ArrayList();
        }
        return historyList;
    }

    @Override
    protected FieldHistoryLineItem getFDBHistoryforField(ConceptHistoryCriteria criteria, ConceptField conceptField,
            Object retainedFieldValue) {

        FieldHistoryLineItem fdbFieldHistoryLineItem = null;
        FdbDdiminteraction fdbDdiminteraction = null;

        if (criteria != null) {
            if (criteria.getId() == null || criteria.getId() == 0) {
                return fdbFieldHistoryLineItem;
            }
            fdbDdiminteraction = getFdbDrugDrugInteractionDAO().findById(criteria.getId());
        }

        if (fdbDdiminteraction != null) {
            Object fdbFieldValue = conceptField.getPersistentValue(false, fdbDdiminteraction);

            if (!fdbFieldValue.equals(retainedFieldValue)) {
                retainedFieldValue = fdbFieldValue;
                fdbFieldHistoryLineItem = populateFieldHistoryLineItemFDB(criteria, conceptField, fdbDdiminteraction);

            }
        }

        return fdbFieldHistoryLineItem;
    }

    /**
     * Populate field history line item fdb.
     * 
     * @param criteria
     *            the criteria
     * @param conceptField
     *            the concept field
     * @param fdbDdiminteraction
     *            the fdb ddiminteraction
     * @return the field history line item
     */
    private FieldHistoryLineItem populateFieldHistoryLineItemFDB(ConceptHistoryCriteria criteria, ConceptField conceptField,
            FdbDdiminteraction fdbDdiminteraction) {
        Object fieldValue = null;

        if (conceptField.isLookup()) {

            fieldValue = determineFdbFieldValue(conceptField, fdbDdiminteraction);

        } else {

            fieldValue = conceptField.getPersistentValue(false, fdbDdiminteraction);

        }

        FieldHistoryLineItem fdbFieldHistoryLineItem = new FieldHistoryLineItem();
        fdbFieldHistoryLineItem.setUser("FDB");
        if (criteria.getId() != null) {
            fdbFieldHistoryLineItem.setId((criteria.getId().toString()));
        }
        if (criteria.getVersionId() != null) {
            fdbFieldHistoryLineItem.setId((criteria.getVersionId().toString()));
        }

        fdbFieldHistoryLineItem.setStatus("FDB");
        fdbFieldHistoryLineItem.setValue(fieldValue.toString());
        return fdbFieldHistoryLineItem;

    }

    /**
     * Determine fdb field value.
     * 
     * @param conceptField
     *            the concept field
     * @param fdbDrugDrugInteraction
     *            the fdb drug drug interaction
     * @return the object
     */
    private Object determineFdbFieldValue(ConceptField conceptField, FdbDdiminteraction fdbDrugDrugInteraction) {

        return super
                .getTerminologyService()
                .getByCode(conceptField.getLookupType(),
                        (String) conceptField.getPersistentValue(false, fdbDrugDrugInteraction)).getUiLabel();

    }

    @Override
    @Transactional(readOnly = true)
    /**
     * Retrieves all of the versions for a DDI based upon the passed in versionId number.  That DDI Version is retrieved,
     * and the version history of the DDI is then retrieved, converted to DTOs and placed in to a List.  If the Custom DDI
     * is associated with a FDB DDI, the FDB DDI is retrieved and placed at the end of the List.
     * 
     * @param versionId the identifier of the DDI Version
     * @return List
     */
    public List<DrugDrugInteractionDTO> findReportHistoryData(Long versionId) {
        List<DrugDrugInteractionDTO> versionHistory = new ArrayList<DrugDrugInteractionDTO>(0);
        try {
            DrugDrugInteractionVersionVa version = drugDrugInteractionDAO.findVersionById(versionId);
            getVersionHistory(version, versionHistory);
            if (version.getFdbId() != 0) {
                DrugDrugInteractionDTO fdbDTO = createDTO(fdbDrugDrugInteractionDAO.findById(version.getFdbId()));
                fdbDTO.setFdbId(0L);
                versionHistory.add(fdbDTO);
            }
        } catch (Exception e) {
            LOGGER.error(e.getMessage(), e);
        }
        return versionHistory;
    }

    /**
     * A method to loop through the version history of a DDI DTO and create DrugDrugInteractionDTO objects from that data.
     * 
     * @param ddimVersion
     *            the Drug-Drug Interaction Version selected by the user
     * @param versionHistory
     *            the List to place the DrugDrugInteractionDTO objects in
     */
    private void getVersionHistory(DrugDrugInteractionVersionVa ddimVersion, List<DrugDrugInteractionDTO> versionHistory) {
        List<Version<DrugDrugInteraction>> versions = ddimVersion.getVersionedObject().getVersionHistory();
        for (Version<DrugDrugInteraction> version : versions) {
            DrugDrugInteractionDTO interaction = createDTO(version);
            interaction.setCurrentActionReason(version.getComment());
            versionHistory.add(interaction);
        }
        Collections.reverse(versionHistory);
    }

    @Override
    public ConceptXref findXrefByFDBId(Object fdbId) {

        if (fdbId instanceof String) {
            return getConceptXrefDAO().findDDIxRefByFDBId(Long.parseLong((String) fdbId));
        }

        return getConceptXrefDAO().findDDIxRefByFDBId((Long) fdbId);

    }

    /**
     * Created the concept xref for customization.
     * 
     * @param fdbId
     *            the fdb id
     * @return xref
     */
    @Override
    protected ConceptXref createConceptXrefForCustomization(Object fdbId) {
        // find xref if it already exists.
        DrugDrugInteractionXref xref = (DrugDrugInteractionXref) findXrefByFDBId(fdbId);
        // if it does not exist then create a new one set the id and save it.
        if (xref == null) {
            xref = new DrugDrugInteractionXref();
            xref.setInteractionId((Long) fdbId);
            persistConceptXref(xref);
        }

        return xref;
    }

    /**
     * Method to create a conceptXref with a comment.
     * 
     * @param interactionId
     *            of type Object
     * @param comment
     *            the comment
     * @return ConceptXref
     */
    @Override
    public ConceptXref createConceptXref(Object interactionId, UserComment comment) {

        DrugDrugInteractionXref xref = new DrugDrugInteractionXref();
        xref.setInteractionId(Long.parseLong((String) interactionId));
        xref.addUserComment(comment);
        return xref;
    }

    /* (non-Javadoc)
     * @see gov.va.med.pharmacy.pecs.service.impl.AbstractPecsService#getFdbDao()
     */
    @Override
    protected FdbHibernateDAOImpl<?> getFdbDao() {
        return getFdbDrugDrugInteractionDAO();
    }
    
}
