package gov.va.med.pharmacy.pecs.service.impl;

import gov.va.med.pharmacy.pecs.criteria.ConceptHistoryCriteria;
import gov.va.med.pharmacy.pecs.domain.DrugPair;
import gov.va.med.pharmacy.pecs.domain.RoutedGeneric;
import gov.va.med.pharmacy.pecs.domain.UserComment;
import gov.va.med.pharmacy.pecs.domain.impl.fdb.FdbDdimdruglink;
import gov.va.med.pharmacy.pecs.domain.impl.fdb.FdbDrugPairDAO;
import gov.va.med.pharmacy.pecs.domain.impl.fdb.RoutedGenericFdb;
import gov.va.med.pharmacy.pecs.domain.impl.fdb.RoutedGenericFdbLite;
import gov.va.med.pharmacy.pecs.domain.impl.va.ConceptXref;
import gov.va.med.pharmacy.pecs.domain.impl.va.DrugDrugInteractionVa;
import gov.va.med.pharmacy.pecs.domain.impl.va.DrugPairVa;
import gov.va.med.pharmacy.pecs.domain.impl.va.DrugPairVersionVa;
import gov.va.med.pharmacy.pecs.domain.impl.va.DrugPairXref;
import gov.va.med.pharmacy.pecs.domain.impl.va.UserProfile;
import gov.va.med.pharmacy.pecs.dto.DrugDrugInteractionDTO;
import gov.va.med.pharmacy.pecs.dto.DrugPairDTO;
import gov.va.med.pharmacy.pecs.metadata.ConceptField;
import gov.va.med.pharmacy.pecs.metadata.ConceptMetadata;
import gov.va.med.pharmacy.pecs.persistence.DrugPairDAOImpl;
import gov.va.med.pharmacy.pecs.persistence.FdbHibernateDAOImpl;
import gov.va.med.pharmacy.pecs.persistence.HibernateDAO;
import gov.va.med.pharmacy.pecs.query.ConceptQuery;
import gov.va.med.pharmacy.pecs.query.DrugPairVisitor;
import gov.va.med.pharmacy.pecs.query.Page;
import gov.va.med.pharmacy.pecs.rules.AlwaysFireTrigger;
import gov.va.med.pharmacy.pecs.rules.BusinessRulesValidation;
import gov.va.med.pharmacy.pecs.rules.BusinessRulesValidationImpl;
import gov.va.med.pharmacy.pecs.rules.ReferenceTextFieldRule;
import gov.va.med.pharmacy.pecs.service.DrugDrugInteractionService;
import gov.va.med.pharmacy.pecs.service.DrugPairService;
import gov.va.med.pharmacy.pecs.service.RequestCountSupport;
import gov.va.med.pharmacy.pecs.service.dto.FieldHistoryLineItem;
import gov.va.med.pharmacy.pecs.service.util.MessageHelper;
import gov.va.med.pharmacy.pecs.state.Status;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.BindingResult;

/**
 * A service that handles all of the manipulation of Drug Pairs.
 * 
 * @author VHAISACARUSR
 * 
 */
@Service
public class DrugPairServiceImpl extends AbstractPecsService<DrugPairDTO, DrugPair> implements DrugPairService {

    /** The Constant LOGGER. */
    private static final Log LOGGER = LogFactory.getLog(DrugPairRequestCountSupport.class);

    /** The dp dao. */
    @Resource
    private DrugPairDAOImpl dpDAO;

    /** The fdb dp dao. */
    @Resource
    private FdbDrugPairDAO fdbDpDAO;

    /** The ddi service. */
    @Resource
    private DrugDrugInteractionService ddiService;

    /** The message helper. */
    @Resource
    private MessageHelper messageHelper;

    /** Use to Calculate the Reverse FDBID of a DrugPair. */
    private static final int FDB_REVERSE = 32000;

    @Override
    @SuppressWarnings("unchecked")
    protected final List<DrugPairDTO> doGetStagingData(ConceptQuery query) {
        List<DrugPairVersionVa> drugPairs =
                (query == null ? getDpDAO().findAll(DrugPairVersionVa.class) : (List<DrugPairVersionVa>) getDpDAO()
                        .findByQuery(query, true));
        List<DrugPairDTO> res = createDrugPairDTOs(drugPairs);

        return res;
    }

    @Override
    @SuppressWarnings("unchecked")
    @Transactional(readOnly = true)
    public final Page getStagingData(String query, int page) {
        if (StringUtils.isBlank(query)) {
            throw new IllegalArgumentException("required query xml string is null");
        }
        if (page < 1) {
            throw new IllegalArgumentException("required page parameter cannot be less than one");
        }
        Page retpage = getDpDAO().findByQueryByPage(query, page);
        List<DrugPairDTO> res = createDrugPairDTOs(retpage.getResults());
        retpage.setResults(res);

        return retpage;
    }

    @Override
    @Transactional(readOnly = true)
    public final List<DrugPairDTO> getStagingData(Long interactionId) {
        return createCurrentDrugPairDTOs(getDpDAO().findDrugPairsByInteractionId(interactionId));
    }

    @Override
    @Transactional(readOnly = true)
    public final DrugPairDTO getFdbData(Long interactionId, Long rtgenId1, Long rtgenId2) {
        return convertFdbToCustom(getFdbDpDAO().findById(interactionId, rtgenId1, rtgenId2));
    }

    @Override
    @Transactional(readOnly = true)
    public final List<DrugPairDTO> getFdbData(Long interactionId) {
        return convertFdbToCustom(getFdbDpDAO().findByInteractionId(interactionId));
    }

    @Override
    @Transactional(readOnly = true)
    public final List<DrugPairDTO> getFilteredFdbData(Long fdbId) {
        List<DrugPairDTO> drugPairsForFdbInteraction = new ArrayList<DrugPairDTO>();
        List<DrugPairDTO> drugPairsForOtherInteractions = new ArrayList<DrugPairDTO>();
        drugPairsForFdbInteraction = convertFdbToCustom(getFdbDpDAO().findByInteractionId(fdbId));
        List<DrugDrugInteractionDTO> ddiForFdbInteraction = getDdiService().getStagingByFdbId(fdbId);

        for (DrugDrugInteractionDTO drugDrugInteractionDTO : ddiForFdbInteraction) {
            drugPairsForOtherInteractions.addAll(getAllDrugPairsForInteraction(drugDrugInteractionDTO.getInteractionid()));
        }

        for (DrugPairDTO drugPairDTO : drugPairsForOtherInteractions) {
            drugPairsForFdbInteraction.remove(drugPairDTO);
        }

        return drugPairsForFdbInteraction;
    }

    /**
     * Creates the DTOs for a list of drug pairs.
     * 
     * @param pairs
     *            the drug pairs from the database
     * @return a list of <code>DrugPairDTO</code>
     */
    private List<DrugPairDTO> createDrugPairDTOs(List<DrugPairVersionVa> pairs) {

        ArrayList<DrugPairDTO> res = new ArrayList<DrugPairDTO>();
        for (DrugPairVersionVa dp : pairs) {
            DrugPairDTO dto = createDrugPairDTO(dp, false);
            res.add(dto);
        }
        return res;
    }

    /**
     * Creates DTOs for the current versions of drug pairs passed in.
     * 
     * @param pairs
     *            the drug pairs from the database
     * @return a list of <code>DrugPairDTO</code>
     */
    private List<DrugPairDTO> createCurrentDrugPairDTOs(List<DrugPairVa> pairs) {

        ArrayList<DrugPairDTO> res = new ArrayList<DrugPairDTO>();
        for (DrugPairVa dp : pairs) {
            DrugPairDTO dto = createDrugPairDTO(dp.getCurrentVersion(), false);
            res.add(dto);
        }
        return res;
    }

    @Override
    protected final List<DrugPairDTO> convertFdbToCustom(List<?> fdbObjects) {
        List<DrugPairDTO> drugPairDTO = new ArrayList<DrugPairDTO>();
        for (Object d : fdbObjects) {
            DrugPairDTO dto = convertFdbToCustom(d);

            drugPairDTO.add(dto);
        }
        return drugPairDTO;
    }

    /**
     * Convert fdb to custom.
     * 
     * @param drugPair
     *            the drug pair
     * @return DrugPairDTO - converted FDB
     */
    private DrugPairDTO convertFdbToCustom(Object drugPair) {
        DrugPairDTO dto = new DrugPairDTO();
        FdbDdimdruglink fdbRec = (FdbDdimdruglink) drugPair;
        dto.setInteractionid(fdbRec.getId().getInteractionid());
        dto.setInteractionidDesc(fdbRec.getInteractionidDesc());
        dto.setRtgenid1(fdbRec.getId().getRtgenid1());
        dto.setRtgenid1Desc(fdbRec.getRtgenid1Desc());
        dto.setRtgenid2(fdbRec.getId().getRtgenid2());
        dto.setRtgenid2Desc(fdbRec.getRtgenid2Desc());
        dto.setSeveritylevelcode(fdbRec.getSeveritylevelcode());
        dto.setSeveritylevelcodeDesc(fdbRec.getSeveritylevelcodeDesc());
        return dto;
    }

    @Override
    @Transactional
    public final DrugPairDTO createNewRecord(DrugPairDTO drugPairDTO, BindingResult bindingResult) {

        // validations that only need the current interaction to determine if incoming drug pair is validate for further
        // processing.
        if (!validateNewDrugPairCurrentInteraction(drugPairDTO, bindingResult))
            return drugPairDTO;

        List<DrugDrugInteractionDTO> relatedInteractions = findRelatedInteractions(drugPairDTO);
        // validations that need the current interaction as well as related interactions to determine if the incoming drug pair
        // is validate for further processing.
        if (!validateNewDrugPairRelatedInteractions(drugPairDTO, relatedInteractions, bindingResult)) {
            return drugPairDTO;
        }

        // either reactive or create the drug pair.
        return createOrReactivateDrugPair(drugPairDTO, relatedInteractions, bindingResult);

    }

    /**
     * Creates OR reactive drug pair. Drug Pair does not currently exist in a non deleted/rejected status on any related
     * interaction.
     * 
     * @param drugPairDTO
     *            the drug pair dto
     * @param relatedInteractions
     *            the related interactions
     * @param bindingResult
     *            the binding result
     * @return the drug pair dto
     */
    private DrugPairDTO createOrReactivateDrugPair(DrugPairDTO drugPairDTO, List<DrugDrugInteractionDTO> relatedInteractions,
            BindingResult bindingResult) {
        // need to determine if it exists on this interaction in a rejected or deleted state in the requested order.
        for (DrugDrugInteractionDTO interaction : relatedInteractions) {
            if (interaction.getInteractionid() == drugPairDTO.getInteractionid()) {
                DrugPairVa relatedDrugPair = findDrugPair(drugPairDTO.getRtgenid1(), drugPairDTO.getRtgenid2(), interaction);
                if (relatedDrugPair != null) {
                    return reactivateOldDrugPair(drugPairDTO, bindingResult, relatedDrugPair);
                }
            }
        }

        // Old logic - investigate why binding results was not used further down in the logic
        DrugPairVa drugPairVa = createDrugPairVa(drugPairDTO);
        return populateAndPersistDrugPair(drugPairVa, drugPairDTO, false);
    }

    /**
     * Find related interactions method determines if related interactions are from an FDB Interaction or from the create from
     * Blank form process and returns the appropriate list.
     * 
     * @param drugPairDTO
     *            the drug pair dto
     * @return the list of interactions
     */
    private List<DrugDrugInteractionDTO> findRelatedInteractions(DrugPairDTO drugPairDTO) {

        List<DrugDrugInteractionDTO> relatedInteractions = null;

        if (isInteractionCreatedFromBlank(drugPairDTO)) {
            relatedInteractions = findRelatedInteractionsFromBlank(drugPairDTO);
        } else {
            relatedInteractions = findRelatedInteractionsFromFDB(drugPairDTO);
        }

        return relatedInteractions;
    }

    /**
     * Find related interactions when the interaction was created from open blank form process.
     * 
     * There will only be one interaction in the list - The current interaction.
     * 
     * @param drugPairDTO
     *            the drug pair dto
     * @return the list
     */
    private List<DrugDrugInteractionDTO> findRelatedInteractionsFromBlank(DrugPairDTO drugPairDTO) {
        ArrayList<DrugDrugInteractionDTO> relatedInteractions = new ArrayList<DrugDrugInteractionDTO>();
        relatedInteractions.add(getDdiService().getStagingByInteractionId(drugPairDTO.getInteractionid()));
        return relatedInteractions;

    }

    /**
     * Find related interactions by fdbid. Related interactions are all VA custom interactions created from the same FDB
     * interaction.
     * 
     * @param drugPairDTO
     *            the drug pair dto
     * @return the list
     */
    private List<DrugDrugInteractionDTO> findRelatedInteractionsFromFDB(DrugPairDTO drugPairDTO) {
        List<DrugDrugInteractionDTO> relatedInteractions;
        DrugDrugInteractionDTO vaInteractionForRecord =
                getDdiService().getStagingByInteractionId(drugPairDTO.getInteractionid());
        relatedInteractions = getDdiService().getStagingByFdbId(vaInteractionForRecord.getFdbId());
        return relatedInteractions;
    }

    /**
     * Checks if the interaction is created from blank.
     * 
     * @param drugPairDTO
     *            the drug pair dto
     * @return true, if is interaction created from blank
     */
    private boolean isInteractionCreatedFromBlank(DrugPairDTO drugPairDTO) {

        DrugPairVa drugPairVA = findDrugPair(drugPairDTO);
        if (drugPairVA != null && drugPairVA.getInteraction() != null) {
            if (drugPairVA.getInteraction().getFdbId() == null) {
                return true;
            }
        }

        return false;
    }

    /**
     * Validate new drug pair related interactions which includes the current interaction. Related interactions are custom
     * interactions created from the same FDB Interaction. (This does not include the FDB Interaction on the customizations)
     * 
     * Check each related interaction for the same drug pair in both forward and reverse order. If it already exists on any
     * related interaction and is not in a rejected or deleted status stop processing and return false.
     * 
     * 
     * @param newDrugPairDTO
     *            the new drug pair dto
     * @param relatedInteractions
     *            the related interactions
     * @param bindingResult
     *            the binding result
     * @return true, if successful
     */
    private boolean validateNewDrugPairRelatedInteractions(DrugPairDTO newDrugPairDTO,
            List<DrugDrugInteractionDTO> relatedInteractions, BindingResult bindingResult) {

        DrugPairVa relatedDrugPair = null;
        DrugPairVa relatedDrugPairReverse = null;

        for (DrugDrugInteractionDTO interaction : relatedInteractions) {

            relatedDrugPair = findDrugPair(newDrugPairDTO.getRtgenid1(), newDrugPairDTO.getRtgenid2(), interaction);

            relatedDrugPairReverse = findDrugPair(newDrugPairDTO.getRtgenid2(), newDrugPairDTO.getRtgenid1(), interaction);

            if (relatedDrugPair != null) {
                if (!isRejectedOrDeleted(relatedDrugPair)) {
                    createDuplicateMessage(newDrugPairDTO, bindingResult, interaction, relatedDrugPair);
                    return false;
                }
            }

            if (relatedDrugPairReverse != null) {
                if (!isRejectedOrDeleted(relatedDrugPairReverse)) {
                    createReverseDuplicateMessage(newDrugPairDTO, bindingResult, interaction, relatedDrugPairReverse);
                    return false;
                }
            }
        }

        return true;
    }

    /**
     * Find drug pair is a convenience method.
     * 
     * @param routedGenericA
     *            the routed generic a
     * @param routedGenericB
     *            the routed generic b
     * @param interaction
     *            the interaction
     * @return the drug pair va
     */
    private DrugPairVa findDrugPair(Long routedGenericA, Long routedGenericB, DrugDrugInteractionDTO interaction) {
        return getDpDAO().findDrugPair(routedGenericA, routedGenericB, interaction.getInteractionid());
    }

    /**
     * Validate new drug pair current interaction.
     * 
     * @param record
     *            the record
     * @param bindingResult
     *            the result
     * @return true, if successful
     */
    private final boolean validateNewDrugPairCurrentInteraction(DrugPairDTO record, BindingResult bindingResult) {

        // checks that the two drugs are different
        if (record.getRtgenid1() != null) {
            if (record.getRtgenid1().equals(record.getRtgenid2())) {
                createDrugsSameMessage(bindingResult);
                return false;
            }
        }

        if (!validateNewRecord(record, bindingResult)) {
            return false;
        }

        // check if this interaction customization has drug pair in reverse order already exists
        DrugPairVa reverseDrugPairRecord = findReverseDrugPair(record);
        if (reverseDrugPairRecord != null && !isRejectedOrDeleted(reverseDrugPairRecord)) {
            createDuplicateReverseMessage(bindingResult, reverseDrugPairRecord);
            return false;
        }
        return true;
    }

    /**
     * Creates the drugs same message.
     * 
     * @param bindingResult
     *            the binding result
     */
    private void createDrugsSameMessage(BindingResult bindingResult) {
        messageHelper.addMessageToBindingResult("dpcustomization", "pecs.service.drugpair.create.rtgen1.equals.rtgen2", null,
                bindingResult);
    }

    /**
     * Creates the duplicate reverse message.
     * 
     * @param result
     *            the result
     * @param reverseDrugPairRecord
     *            the reverse drug pair record
     */
    private void createDuplicateReverseMessage(BindingResult result, DrugPairVa reverseDrugPairRecord) {
        messageHelper.addMessageToBindingResult("dpcustomization", "pecs.service.drugpair.create.duplicate.reverse",
                new Object[] { reverseDrugPairRecord.getRoutedGeneric1().getDescription(),
                        reverseDrugPairRecord.getRoutedGeneric2().getDescription() }, result);
        return;
    }

    /**
     * Creates the reverse duplicate message.
     * 
     * @param record
     *            the record
     * @param result
     *            the result
     * @param interaction
     *            the interaction
     * @param drugPair
     *            the reverse drug pair
     * @return the drug pair dto
     */
    private DrugPairDTO createReverseDuplicateMessage(DrugPairDTO record, BindingResult result,
            DrugDrugInteractionDTO interaction, DrugPairVa drugPair) {
        messageHelper.addMessageToBindingResult(
                "dpcustomization",
                "pecs.service.drugpair.create.duplicate.reverse.another.interaction",
                new Object[] { drugPair.getRoutedGeneric1().getDescription(), drugPair.getRoutedGeneric2().getDescription(),
                        String.valueOf(interaction.getInteractionid()), String.valueOf(interaction.getInteractionid()),
                        String.valueOf(record.getInteractionid()) }, result);
        return record;
    }

    /**
     * Creates the drug pair va.
     * 
     * @param record
     *            the record
     * @return the drug pair va
     */
    private DrugPairVa createDrugPairVa(DrugPairDTO record) {
        RoutedGeneric rtGen1 = getDpDAO().findById(record.getRtgenid1(), RoutedGenericFdb.class);
        RoutedGeneric rtGen2 = getDpDAO().findById(record.getRtgenid2(), RoutedGenericFdb.class);
        DrugDrugInteractionVa ddi = ddiService.getDDIByInteractionId(record.getInteractionid());

        DrugPairVa dp = DrugPairVa.createDrugPair(rtGen1, rtGen2, ddi);
        dp.setStatus(getStateMachineFactory().getStatusByCode(Status.STATUS_NEW));
        dp.setActionBy(record.getActionBy());
        return dp;
    }

    /**
     * Find reverse drug pair.
     * 
     * @param record
     *            the record
     * @return the drug pair va
     */
    private DrugPairVa findReverseDrugPair(DrugPairDTO record) {
        DrugPairVa dpr = getDpDAO().findDrugPair(record.getRtgenid2(), record.getRtgenid1(), record.getInteractionid());
        return dpr;
    }

    /**
     * Find drug pair.
     * 
     * @param record
     *            the record
     * @return the drug pair va
     */
    private DrugPairVa findDrugPair(DrugPairDTO record) {
        DrugPairVa dpr = getDpDAO().findDrugPair(record.getRtgenid1(), record.getRtgenid2(), record.getInteractionid());
        return dpr;
    }

    /**
     * Creates the duplicate message.
     * 
     * @param record
     *            the drug pair that was passed in to the service
     * @param bindingResult
     *            the binding result
     * @param interaction
     *            the DDI
     * @param drugPair
     *            the drug pair that was found as a duplicate
     * @return the record that was passed in.
     */
    private DrugPairDTO createDuplicateMessage(DrugPairDTO record, BindingResult bindingResult,
            DrugDrugInteractionDTO interaction, DrugPairVa drugPair) {
        if (interaction.getInteractionid() != record.getInteractionid()) {
            createDuplicateMessageDifferentInteraction(record, bindingResult, interaction, drugPair);
        } else {
            createDuplicateMessageSameInteraction(drugPair, bindingResult);
        }
        return record;
    }

    /**
     * Creates the duplicate message different interaction.
     * 
     * @param record
     *            the record
     * @param result
     *            the result
     * @param interaction
     *            the interaction
     * @param drugPair
     *            the drug pair
     */
    private void createDuplicateMessageDifferentInteraction(DrugPairDTO record, BindingResult result,
            DrugDrugInteractionDTO interaction, DrugPairVa drugPair) {
        messageHelper.addMessageToBindingResult("dpcustomization",
                "pecs.service.drugpair.create.drugpair.second.interaction.error", new Object[] {
                        drugPair.getRoutedGeneric1().getDescription(), drugPair.getRoutedGeneric2().getDescription(),
                        ((Long) interaction.getInteractionid()).toString(), ((Long) interaction.getInteractionid()).toString(),
                        ((Long) record.getInteractionid()).toString() }, result);
    }

    /**
     * Creates the duplicate message same interaction.
     * 
     * @param drugPair
     *            the drug pair
     * @param result
     *            the result
     */
    private void createDuplicateMessageSameInteraction(DrugPairVa drugPair, BindingResult result) {
        messageHelper.addMessageToBindingResult("dpcustomization", "pecs.service.drugpair.create.duplicate.drugpair.error",
                new Object[] { drugPair.getRoutedGeneric1().getDescription(), drugPair.getRoutedGeneric2().getDescription() },
                result);
    }

    /**
     * Checks if is rejected or deleted.
     * 
     * @param dp
     *            the drug pair
     * @return true if the drug pair is deleted or rejected
     */
    private final boolean isRejectedOrDeleted(DrugPairVa dp) {
        LOGGER.debug(dp.getStatus().getName());

        return dp.getStatus().getName().equals(Status.STATUS_DELETED)
                || dp.getStatus().getName().equals(Status.STATUS_REJECTED);
    }

    /**
     * Adds back a Drug Pair that has been Deleted or Rejected.
     * 
     * @param record
     *            the record to be added
     * @param result
     *            the result
     * @param dp
     *            The old drug pair
     * @return the new persisted drug pair
     */
    private DrugPairDTO reactivateOldDrugPair(DrugPairDTO record, BindingResult result, DrugPairVa dp) {

        // reactivate the customization record; since the DTO that came in
        // as an argument is not fully populated, use the previously persisted data
        DrugPairDTO populatedRecord = createDrugPairDTO(dp.getCurrentVersion(), true);
        populatedRecord.setActionUser(record.getActionUser());
        populatedRecord.setAssignedTo(record.getAssignedTo());
        populatedRecord.setCurrentActionReason(record.getCurrentActionReason());
        // this field is not persisted, but it exists in the DTO;
        // we modify it to force state change
        populatedRecord.setUicategory6("force state change");
        // validateNewRecord(populatedRecord);
        return applyAction("Modify", populatedRecord, result);

    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.service.DrugPairService#findDrugPairsForFDBInteraction(java.lang.Long, java.lang.Long,
     * java.lang.Long)
     */
    @Override
    @Transactional
    public final List<DrugPairDTO> findDrugPairsForFDBInteraction(Long routedGeneric1, Long routedGeneric2, Long fdbId) {
        return createCurrentDrugPairDTOs(dpDAO.findDrugPairsByFdb(routedGeneric1, routedGeneric2, fdbId));
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.service.DrugPairService#lookupDpDesc(long, long, long)
     */
    @Override
    @Transactional
    public final String[] lookupDpDesc(long rtgenid1, long rtgenid2, long interactionid) {
        DrugPairVa dp = getDpDAO().findDrugPair(rtgenid1, rtgenid2, interactionid);
        String[] dpDescriptions = new String[2];
        if (dp != null) {
            dpDescriptions[0] = dp.getRoutedGeneric1().getDescription();
            dpDescriptions[1] = dp.getRoutedGeneric2().getDescription();

        }
        return dpDescriptions;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.service.DrugPairService#createNewRecords(java.util.List,
     * org.springframework.validation.BindingResult)
     */
    @Override
    @Transactional
    public final void createNewRecords(List<DrugPairDTO> records, BindingResult bindingResult) {
        for (DrugPairDTO ddim : records) {
            createNewRecord(ddim, bindingResult);

        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.service.PecsService#applyAction(java.lang.String, gov.va.med.pharmacy.pecs.dto.PecsDTO,
     * org.springframework.validation.BindingResult)
     */
    @Override
    @Transactional
    public final DrugPairDTO applyAction(String action, DrugPairDTO record, BindingResult err) {
        // fixed
        return applyAction(action, record, false, err);
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.service.DrugPairService#applyAction(java.lang.String, java.util.List,
     * org.springframework.validation.BindingResult)
     */
    @Override
    @Transactional
    public final void applyAction(String action, List<DrugPairDTO> records, BindingResult err) {

        // fixed
        long start = 0;
        if (LOGGER.is   ugEnabled()) {
            start = System.currentTimeMillis();
        }
        if (records == null) {
            return;
        }
        for (DrugPairDTO ddim : records) {
            ddim.setCurrentActionReason("System generated message for action " + action);
            applyAction(action, ddim, true, err);
        }
        if (LOGGER.is   ugEnabled()) {
            long end = System.currentTimeMillis();
            LOGGER.debug("Applying action " + action + " to " + records.size() + " drug pairs took " + (end - start)
                    + " milliseconds");
        }
    }

    /**
     * Apply action.
     * 
     * @param action
     *            the action
     * @param drugPairDTO
     *            the drug pair dto
     * @param batch
     *            the batch
     * @param result
     *            the result
     * @return the drug pair dto
     */
    private DrugPairDTO applyAction(String action, DrugPairDTO drugPairDTO, boolean batch, BindingResult result) {
        DrugPairVa dp = findPersistentObject(drugPairDTO);
        DrugPairVersionVa v = dp.getCurrentVersion();
        DrugPairDTO crt = createDrugPairDTO(v, false);

        boolean stateChange = doApplyAction(dp, drugPairDTO, crt, action, drugPairDTO.getActionUser(), result);
        // set action by only when the state machine was invoked,
        // but not when a roll-back took place
        if (stateChange && !dp.wasRolledBack()) {
            dp.setActionBy(drugPairDTO.getActionBy());
        }

        drugPairDTO.setExportDate(null);
        return result.hasErrors() ? drugPairDTO : populateAndPersistDrugPair(dp, drugPairDTO, batch);
    }

    /**
     * Populate and persist drug pair.
     * 
     * @param dest
     *            the dest
     * @param source
     *            the source
     * @param batch
     *            the batch
     * @return the drug pair dto
     */
    private DrugPairDTO populateAndPersistDrugPair(DrugPairVa dest, DrugPairDTO source, boolean batch) {
        dest.setAuditUser(source.getActionBy());
        dest.setModifiedDate(new Date());
        dest.setComment(source.getCurrentActionReason());
        // if (!batch) {
        if (!dest.wasRolledBack()) {
            dest.setAssignedUser(source.getAssignedTo());
            // The reference text is not passed in the source object when modifying drug pairs
            if (dest.getReferenceText() == null) {
                dest.setReferenceText(source.getReferenceText());
            }
        }
        getDpDAO().save(dest);

        return batch ? null : createDrugPairDTO(dest.getCurrentVersion(), true);
    }

    @Override
    protected final DrugPairVa findPersistentObject(DrugPairDTO drugPairDTO) {
        return findDrugPair(drugPairDTO);
    }

    @Override
    protected final RequestCountSupport getRequestCountSupport() {
        return new DrugPairRequestCountSupport(dpDAO);
    }

    @Override
    @Transactional
    public final List<Long> getInteractionsWithWork(String userName, List<String> status, String actionUser) {
        List<Long> interactions = new ArrayList<Long>();
        if (userName != null && userName.equals("UNASSIGNED")) {
            String query =
                    "select distinct interaction.interactionId  from DrugDrugInteractionVa as interaction "
                            + "inner join interaction.drugPairs as pair "
                            + "where pair.exportManager.currentVersion.assignedUser = '"
                            + userName
                            + "' "
                            + " and pair.exportManager.currentVersion.status.name not in ('"
                            + Status.STATUS_APPROVED
                            + "','"
                            + Status.STATUS_DELETED
                            + "','"
                            + Status.STATUS_REJECTED
                            + "') "
                            + (actionUser == null ? "" : " and pair.exportManager.currentVersion.actionBy != '" + actionUser
                                    + "' ") + " and interaction.exportManager.currentVersion.status.name = '"
                            + Status.STATUS_APPROVED + "'";
            interactions = getDpDAO().findByQuery(query, Long.class);
            LOGGER.info("query for " + userName + " " + status + " " + actionUser + " = " + interactions.size());
        } else {
            String statusDelimited = createDelimitedString(status);
            String query =
                    "select distinct interaction.interactionId from DrugDrugInteractionVa as interaction "
                            + "inner join interaction.drugPairs as pair "
                            + "where pair.exportManager.currentVersion.assignedUser"
                            + (userName == null ? " is null " : " = '" + userName + "' ")
                            + (statusDelimited == null ? "" : " and pair.exportManager.currentVersion.status.name in ("
                                    + statusDelimited + ")")
                            + (actionUser == null ? "" : " and pair.exportManager.currentVersion.actionBy != '" + actionUser
                                    + "' ") + " and interaction.exportManager.currentVersion.status.name = '"
                            + Status.STATUS_APPROVED + "'";
            LOGGER.info("query for " + userName + " " + status + " " + actionUser + " = " + interactions.size());
            interactions = getDpDAO().findByQuery(query, Long.class);
        }
        return interactions;
    }

    /**
     * Takes a list of strings and creates a comma delimited string.
     * 
     * @param status
     *            a list of strings
     * @return a comma delimited string
     */
    private String createDelimitedString(List<String> status) {
        String statusDelimited = null;
        for (String string : status) {
            if (statusDelimited == null) {
                statusDelimited = "'" + string + "'";
            } else {
                statusDelimited = statusDelimited + ", '" + string + "'";
            }

        }
        return statusDelimited;
    }

    @Override
    @Transactional
    public final ConceptMetadata getMetadata() {
        return getMetadataService().getMetadata(DrugPairDTO.class);
    }

    @Override
    @Transactional
    public final DrugPairDTO getCustomModel(long id) {
        DrugPairVersionVa v = getDpDAO().findById(id, DrugPairVersionVa.class);
        LOGGER.debug("getCustomModel drugPairVersionVa " + v);
        return v == null ? null : createDrugPairDTO(v, true);
    }

    /**
     * Gets the dp dao.
     * 
     * @return the dp dao
     */
    public final DrugPairDAOImpl getDpDAO() {
        return dpDAO;
    }

    /**
     * Sets the dp dao.
     * 
     * @param dpDAO
     *            the new dp dao
     */
    public final void setDpDAO(DrugPairDAOImpl dpDAO) {
        this.dpDAO = dpDAO;
    }

    /**
     * Gets the ddi service.
     * 
     * @return the ddi service
     */
    public final DrugDrugInteractionService getDdiService() {
        return ddiService;
    }

    /**
     * Sets the ddi service.
     * 
     * @param ddiService
     *            the new ddi service
     */
    public final void setDdiService(DrugDrugInteractionService ddiService) {
        this.ddiService = ddiService;
    }

    /**
     * Pre-process the user query to account for the lack of stored reverse drugs pairs and interactions.
     * 
     * @param query
     *            the query to process
     * @param custom
     *            true if custom
     * @return the processed query
     */
    @Override
    protected final ConceptQuery preProcessQuery(ConceptQuery query, boolean custom) {
        if (custom) {
            ConceptQuery preProcessed = query.copy();
            DrugPairVisitor visitor = new DrugPairVisitor();
            preProcessed.accept(visitor);
            return super.preProcessQuery(preProcessed, true);
        } else {
            return super.preProcessQuery(query, false);
        }
    }

    /**
     * Create a DTO from a DP version.
     * 
     * @param source
     *            the version object
     * @param full
     *            indicates whether to fully populate the DTO (for display on the details page) or not (e.g. when displaying in
     *            query results or on the DP customization page); this is meant as an optimization, to cut down on the time it
     *            takes to return large result sets
     * @return the DTO
     */
    private DrugPairDTO createDrugPairDTO(DrugPairVersionVa source, boolean full) {
        DrugPairDTO dto = new DrugPairDTO();
        dto.setFdbCustomDdimVaId(source.getVersionId());
        dto.setActionBy(source.getActionBy());
        dto.setActionDate(source.getDate());
        dto.setExportDate(source.getExportDate());
        dto.setAssignedTo(source.getAssignedUser());
        dto.setCurrent(source.isCurrent());
        dto.setInteractionid(source.getInteraction().getInteractionId());
        dto.setInteractionidDesc(source.getInteraction().getDescription());
        Long fdbId = source.getInteraction().getFdbId();
        dto.setFdbId(fdbId == null ? 0 : fdbId);
        Long reverseFdbId = source.getInteraction().getReverseFdbId();
        dto.setReverseFdbId(reverseFdbId == null ? 0 : reverseFdbId);
        dto.setStatus(source.getStatus().getName());
        dto.setReferenceText(source.getReferenceText());
        dto.setRequestor(source.getVersionedObject().getRequestor());
        RoutedGeneric rtgen = source.getRoutedGeneric1();
        if (rtgen != null) {
            dto.setRtgenid1(rtgen.getIdentifier());
            dto.setRtgenid1Desc(rtgen.getDescription());
        }
        rtgen = source.getRoutedGeneric2();
        if (rtgen != null) {
            dto.setRtgenid2(rtgen.getIdentifier());
            dto.setRtgenid2Desc(rtgen.getDescription());
        }
        dto.setSeveritylevelcode(source.getInteraction().getSeverityLevel().getCode());
        dto.setSeveritylevelcodeDesc(source.getInteraction().getSeverityLevel().getDescription());
        dto.setDrugPairId(source.getVersionedObject().getId());
        // this is an expensive operation, as it has to look at the entire version history
        if (full) {
            dto.setActionReason(source.getCommentHistory());
        }

        return dto;
    }

    @Override
    @Transactional
    public final List<RoutedGenericFdb> getRoutedGenericDrugs() {
        return getDpDAO().getRoutedGenericDrugs();
    }

    @Override
    @Transactional
    public final List<RoutedGenericFdbLite> getRoutedGenericDrugsLite() {
        return getDpDAO().getRoutedGenericDrugsLite();
    }    
    
    @Override
    @Transactional
    public final DrugPairDTO getCurrentDrugPair(long rtgenId1, long rtgenId2, long interactionId) {
        DrugPairVa dp = getDpDAO().findDrugPair(rtgenId1, rtgenId2, interactionId);
        return dp == null ? null : createDrugPairDTO(dp.getCurrentVersion(), true);
    }

    @Override
    @Transactional
    public final DrugPairDTO getCurrentDrugPairByFdb(long rtgenId1, long rtgenId2, long fdbId) {
        DrugPairVa dp = getDpDAO().findDrugPairByFdb(rtgenId1, rtgenId2, fdbId);
        if (dp == null) {
            dp = getDpDAO().findDrugPairByFdb(rtgenId2, rtgenId1, reverseFdbId(fdbId));
        }
        return dp == null ? null : createDrugPairDTO(dp.getCurrentVersion(), true);
    }

    /**
     * Reverse fdb id.
     * 
     * @param fdbId
     *            the fdb id
     * @return the long
     */
    private long reverseFdbId(long fdbId) {
        return FDB_REVERSE - fdbId;
    }

    /**
     * This method is invoked from the customization page. For it, we add to the DTOs all allowable actions (needed for the
     * display).
     * 
     * @param interactionId
     *            - the interaction id
     * @param user
     *            - the user profile
     * @return list of drug pairs
     */
    @Override
    @Transactional(readOnly = true)
    public final List<DrugPairDTO> getDrugPairsForInteraction(long interactionId, UserProfile user) {
        long start = System.currentTimeMillis();
        List<DrugPairVa> pairs = getDpDAO().findDrugPairsByInteractionId(interactionId);
        ArrayList<DrugPairDTO> res = new ArrayList<DrugPairDTO>();
        for (DrugPairVa dp : pairs) {
            DrugPairDTO dto = createDrugPairDTO(dp.getCurrentVersion(), false);
            dto.setAllowableActions(getAllowableActions(dp, user));
            res.add(dto);
        }
        if (LOGGER.is   ugEnabled()) {
            long end = System.currentTimeMillis();
            LOGGER.debug("Getting drugs for interaction " + interactionId + " took " + (end - start) + " milliseconds");
        }
        return res;
    }

    @Override
    @Transactional(readOnly = true)
    public final List<DrugPairDTO> getAllDrugPairsForInteraction(long interactionid) {
        ArrayList<DrugPairDTO> res = new ArrayList<DrugPairDTO>();
        List<DrugPairVa> drugPairs = getDpDAO().findDrugPairsByInteractionId(interactionid);
        for (DrugPairVa dp : drugPairs) {
            DrugPairDTO dto = createDrugPairDTO(dp.getCurrentVersion(), false);
            res.add(dto);
        }
        return res;
    }

    @Override
    @Transactional(readOnly = true)
    public final Map<String, Long> getDrugPairsStatusCountsByInteractionId(long interactionId) {
        Map<String, Long> drugPairsStatusCounts = getDpDAO().findDrugPairsStatusCountsByInteractionId(interactionId);
        return drugPairsStatusCounts;
    }

    @Override
    @Transactional(readOnly = true)
    public final boolean hasFdbDrugPairsOnOtherInteraction(long fdbId) {
        if (fdbId == 0) {
            return false;
        }
        List<DrugDrugInteractionDTO> interactions = ddiService.getStagingByFdbId(fdbId);
        for (DrugDrugInteractionDTO interaction : interactions) {
            List<DrugPairDTO> pairs = getAllDrugPairsForInteraction(interaction.getInteractionid());
            if (!pairs.isEmpty()) {
                return true;
            }
        }
        return false;
    }

    /**
     * Additional validations needed in this concept.
     * 
     * @param meta
     *            ConceptMetadata
     * @return BusinessRulesValidation
     */
    @Override
    protected final BusinessRulesValidation getDataValidation(ConceptMetadata meta) {
        // all required fields must be present
        BusinessRulesValidationImpl validator = new BusinessRulesValidationImpl(meta);
        validator.registerRule(new ReferenceTextFieldRule(DrugPairDTO.REFERENCE_TEXT), new AlwaysFireTrigger());
        return validator;
    }

    /**
     * Gets the fdb dp dao.
     * 
     * @return the fdb dp dao
     */
    public final FdbDrugPairDAO getFdbDpDAO() {
        return fdbDpDAO;
    }

    /**
     * Sets the fdb dp dao.
     * 
     * @param fdbDpDAO
     *            the new fdb dp dao
     */
    public final void setFdbDpDAO(FdbDrugPairDAO fdbDpDAO) {
        this.fdbDpDAO = fdbDpDAO;
    }

    @Override
    public final HibernateDAO<?> getDao(boolean custom) {
        return custom ? dpDAO : fdbDpDAO;
    }

    @SuppressWarnings({ "unchecked", "rawtypes" })
    @Override
    protected final List getCustomHistoryForField(ConceptHistoryCriteria criteria) {
        // Drug Pair does not utilize history for Fields
        return null;
    }

    @Override
    protected final FieldHistoryLineItem getFDBHistoryforField(ConceptHistoryCriteria criteria, ConceptField conceptField,
            Object retainedFieldValue) {

        return null;
    }

    @Override
    @Transactional(readOnly = true)
    public final List<DrugPairDTO> getVaDrugPairsByRoutedGenericId(long routedGenId) {
        ArrayList<DrugPairDTO> resultList = new ArrayList<DrugPairDTO>();
        List<DrugPairVa> drugPairs = dpDAO.findDrugPairsByRoutedGenericId(routedGenId);

        for (DrugPairVa drugPair : drugPairs) {
            DrugPairDTO dto = createDrugPairDTO(drugPair.getCurrentVersion(), false);
            resultList.add(dto);

        }
        return resultList;
    }

    @Override
    @Transactional(readOnly = true)
    public final List<DrugPairDTO> getFDBDrugPairsByRoutedGenericId(long routedGenId) {
        ArrayList<DrugPairDTO> resultList = new ArrayList<DrugPairDTO>();
        List<FdbDdimdruglink> drugPairs = getFdbDpDAO().findByDrugId(routedGenId);
        for (FdbDdimdruglink drugPair : drugPairs) {
            DrugPairDTO dto = convertFdbToCustom(drugPair);
            resultList.add(dto);
        }
        return resultList;
    }

    @Override
    public final ConceptXref createConceptXref(Object key, UserComment comment) {
        return new DrugPairXref();
    }

    @Override
    public final ConceptXref findXrefByFDBId(Object fdbId) {
        // Drug Pair does not utilize concept cross reference
        return null;
    }

    @Override
    protected final ConceptXref createConceptXrefForCustomization(Object fdbId) {
        // Drug Pair does not utilize concept cross reference
        return null;
    }

    /**
     * Gets the message helper.
     * 
     * @return the message helper
     */
    public final MessageHelper getMessageHelper() {
        return messageHelper;
    }

    /**
     * Sets the message helper.
     * 
     * @param messageHelper
     *            the new message helper
     */
    public final void setMessageHelper(MessageHelper messageHelper) {
        this.messageHelper = messageHelper;
    }

    /* (non-Javadoc)
     * @see gov.va.med.pharmacy.pecs.service.impl.AbstractPecsService#getFdbDao()
     */
    @Override
    protected FdbHibernateDAOImpl<?> getFdbDao() {
        return getFdbDpDAO();
    }
    
    
}
