package gov.va.med.pharmacy.pecs.service.impl;

import gov.va.med.pharmacy.pecs.criteria.ConceptHistoryCriteria;
import gov.va.med.pharmacy.pecs.domain.DuplicateTherapy;
import gov.va.med.pharmacy.pecs.domain.UserComment;
import gov.va.med.pharmacy.pecs.domain.Version;
import gov.va.med.pharmacy.pecs.domain.impl.fdb.FdbDuplicatetherapy;
import gov.va.med.pharmacy.pecs.domain.impl.fdb.FdbDuplicatetherapyDAO;
import gov.va.med.pharmacy.pecs.domain.impl.va.ConceptXref;
import gov.va.med.pharmacy.pecs.domain.impl.va.DuplicateTherapyVa;
import gov.va.med.pharmacy.pecs.domain.impl.va.DuplicateTherapyVersionVa;
import gov.va.med.pharmacy.pecs.domain.impl.va.DuplicateTherapyXref;
import gov.va.med.pharmacy.pecs.dto.DuplicateTherapyDTO;
import gov.va.med.pharmacy.pecs.exception.CustomizationServiceException;
import gov.va.med.pharmacy.pecs.metadata.ConceptField;
import gov.va.med.pharmacy.pecs.metadata.ConceptMetadata;
import gov.va.med.pharmacy.pecs.persistence.DuplicateTherapyVersionVaDAO;
import gov.va.med.pharmacy.pecs.persistence.FdbHibernateDAOImpl;
import gov.va.med.pharmacy.pecs.persistence.HibernateDAO;
import gov.va.med.pharmacy.pecs.query.ConceptQuery;
import gov.va.med.pharmacy.pecs.query.Page;
import gov.va.med.pharmacy.pecs.rules.AlwaysFireTrigger;
import gov.va.med.pharmacy.pecs.rules.BusinessRulesValidation;
import gov.va.med.pharmacy.pecs.rules.BusinessRulesValidationImpl;
import gov.va.med.pharmacy.pecs.rules.ReferenceTextFieldRule;
import gov.va.med.pharmacy.pecs.rules.RegexRule;
import gov.va.med.pharmacy.pecs.service.DuplicateTherapyService;
import gov.va.med.pharmacy.pecs.service.RequestCountSupport;
import gov.va.med.pharmacy.pecs.service.dto.FieldHistoryLineItem;
import gov.va.med.pharmacy.pecs.state.Status;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.List;

import javax.annotation.Resource;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.BindingResult;

/**
 * 
 * This class represents a service for accessing the Duplicate therapy.
 * 
 * @author vhaisacarusr
 * 
 */
@Service
public final class DuplicateTherapyServiceImpl extends AbstractPecsService<DuplicateTherapyDTO, DuplicateTherapy> implements
        DuplicateTherapyService, InitializingBean {

    private static Logger LOGGER = Logger.getLogger(DuplicateTherapyServiceImpl.class);
    @Resource
    private FdbDuplicatetherapyDAO fdbDuplicatetherapyDAO;
    @Resource
    private DuplicateTherapyVersionVaDAO customDuplicateTherapyDAO;

    private static final Comparator<DuplicateTherapyDTO> REPORT_HISTORY_COMPARATOR = new Comparator<DuplicateTherapyDTO>() {
        @Override
        public int compare(DuplicateTherapyDTO o1, DuplicateTherapyDTO o2) {
            return o1.getActionDate().compareTo(o2.getActionDate());
        }
    };

    @Override
    @Transactional
    public DuplicateTherapyDTO applyAction(String action, DuplicateTherapyDTO duplicateTherapyDTO, BindingResult err) {
        DuplicateTherapyVa dupVa = findPersistentObject(duplicateTherapyDTO);
        DuplicateTherapyVersionVa currentVersion = dupVa.getCurrentVersion();
        DuplicateTherapyDTO crtDtcat = createDuplicateTherapyDTO(currentVersion);
        boolean stateChange =
                doApplyAction(dupVa, duplicateTherapyDTO, crtDtcat, action, duplicateTherapyDTO.getActionUser(), err);
        // set action by only when the state machine was invoked,
        // but not when a roll-back took place
        if (stateChange && !dupVa.wasRolledBack()) {
            dupVa.setActionBy(duplicateTherapyDTO.getActionBy());
        }
        duplicateTherapyDTO.setExportDate(null);
        duplicateTherapyDTO.setConceptXref(dupVa.getConceptXref());
        return err.hasErrors() ? duplicateTherapyDTO : populateAndPersistDuplicateTherapy(dupVa, duplicateTherapyDTO,
                dupVa.wasRolledBack());

    }

    @Override
    @Transactional
    public DuplicateTherapyDTO createNewRecord(DuplicateTherapyDTO record, BindingResult err) {
        if (record == null)
            throw new IllegalArgumentException("required DT object is null");

        long customDTCId = record.getFdbCustomDtcatVaId();

        if (customDTCId != 0) {
            LOGGER.error("New DT shouldn't have its id set");
            throw new CustomizationServiceException("Invalid DT - id cannot be different from 0");
        }
        if (LOGGER.is   ugEnabled())
            LOGGER.debug("Creating new DT record in Service Impl layer");

        // validateNewRecord(record);
        if (!validateNewRecord(record, err)) {
            return record;
        }
        DuplicateTherapyVa dtcatVa = DuplicateTherapyVa.createDuplicateTherapy();
        dtcatVa.setStatus(getStateMachineFactory().getStatusByCode(Status.STATUS_NEW));
        dtcatVa.setActionBy(record.getActionBy());

        DuplicateTherapyXref xref = (DuplicateTherapyXref) createConceptXrefForCustomization(record.getDtcid());
        record.setConceptXref(xref);

        return populateAndPersistDuplicateTherapy(dtcatVa, record, dtcatVa.wasRolledBack());
    }

    /**
     * @param dest
     *            the destination <code>DuplicateTherapyVa</code>
     * @param source
     *            the source <code>DuplicateTherapyDTO</code>
     * @param partial
     *            true if you only need to populate the date user and comment
     * @return the persisted <code>DuplicateTherapyDTO</code>
     */
    private DuplicateTherapyDTO populateAndPersistDuplicateTherapy(DuplicateTherapyVa dest, DuplicateTherapyDTO source,
            boolean partial) {
        source.setCategory("VA");
        if (!partial) {
            dest.setCategory(source.getCategory());
            dest.setCustomString(source.getCustomstring());

            dest.setCustomDupAllowance(getTerminologyService().getDuplicateAllowanceByCode(source.getCustomdupallowance()));
            // dest.setCustomDupAllowance(source.getCustomdupallowance());
            dest.setCustomDupAllowanceInd(source.getCustomdupallowind());

            dest.setDtcid(source.getDtcid());
            dest.setReferenceText(source.getReferenceText());
            dest.setAssignedUser(source.getAssignedTo());
        }
        dest.setComment(source.getCurrentActionReason());
        dest.setAuditUser(source.getActionBy());
        dest.setModifiedDate(new Date());

        dest.setConceptXref(source.getConceptXref());

        getCustomDuplicateTherapyDAO().save(dest);
        return createDuplicateTherapyDTO(dest.getCurrentVersion());
    }

    /**
     * Create a DTO out of a domain object.
     * 
     * @param cv
     *            the <code>DuplicateTherapy</code> serving as a source
     * @return a populated DTO of type <code>DuplicateTherapyDTO</code>
     */
    private DuplicateTherapyDTO createDuplicateTherapyDTO(DuplicateTherapyVersionVa cv) {
        if (cv == null) {
            return null;
        }
        DuplicateTherapyDTO dto = new DuplicateTherapyDTO();
        dto.setActionBy(cv.getActionBy());
        dto.setCurrent(cv.isCurrent());
        dto.setFdbCustomDtcatVaId(cv.getVersionId());
        dto.setCustomstring(cv.getCustomString());
        dto.setCustomdupallowance(Long.valueOf(cv.getCustomDupAllowance().getCode()));
        dto.setCustomdupallowind(cv.getCustomDupAllowanceInd());
        dto.setDtcid(cv.getVersionedObject().getDtcid());
        dto.setReferenceText(cv.getReferenceText());
        dto.setAssignedTo(cv.getAssignedUser());
        dto.setActionDate(cv.getDate());
        dto.setExportDate(cv.getExportDate());
        dto.setDuplicatetherapyid(cv.getVersionedObject().getId());
        dto.setStatus(cv.getStatus().getName());
        dto.setRequestor(cv.getVersionedObject().getRequestor());
        dto.setActionReason(cv.getCommentHistory());
        dto.setConceptXref(cv.getVersionedObject().getConceptXref());
        dto.setMasterRecordId(cv.getVersionedObject().getId());
        return dto;
    }

    /**
     * Create a DTO out of a domain object with the Comment included. This is mainly for reporting purposes.
     * 
     * @param cv
     *            the <code>DuplicateTherapy</code> serving as a source
     * @return a populated DTO of type <code>DuplicateTherapyDTO</code>
     */
    private DuplicateTherapyDTO createDuplicateTherapyDTOWithComment(DuplicateTherapyVersionVa cv) {
        DuplicateTherapyDTO dto = createDuplicateTherapyDTO(cv);
        dto.setCurrentActionReason(cv.getComment());
        return dto;
    }

    /**
     * @param versions
     *            a list of <code>DuplicateTherapyVersionVa</code>
     * @return a list of populated DTO's of type <code>DuplicateTherapyDTO</code>
     */
    private List<DuplicateTherapyDTO> createDuplicateTherapyDTOs(List<DuplicateTherapyVersionVa> versions) {
        List<DuplicateTherapyDTO> list = new ArrayList<DuplicateTherapyDTO>();
        for (DuplicateTherapyVersionVa version : versions) {
            list.add(createDuplicateTherapyDTO(version));
        }
        return list;
    }

    @Transactional
    @Override
    public DuplicateTherapyDTO getFdbDataByDtcid(Long dtcid) {
        return convertFdbToCustom(getFdbDuplicatetherapyDAO().findById(dtcid));
    }

    @Override
    protected List<DuplicateTherapyDTO> convertFdbToCustom(List<?> fdbList) {
        List<DuplicateTherapyDTO> customList = new ArrayList<DuplicateTherapyDTO>();
        for (Object obj : fdbList) {
            DuplicateTherapyDTO dto = convertFdbToCustom(obj);
            customList.add(dto);
        }
        return customList;
    }

    /**
     * @param obj
     *            the uncast <code>FdbDuplicatetherapy</code> to be converted
     * @return a populated DTO of type <code>DuplicateTherapyDTO</code>
     */
    private DuplicateTherapyDTO convertFdbToCustom(Object obj) {
        FdbDuplicatetherapy dtcat = (FdbDuplicatetherapy) obj;
        DuplicateTherapyDTO dto = new DuplicateTherapyDTO();
        dto.setDtcid(dtcat.getDtcid());
        dto.setCustomdupallowance(dtcat.getDuplicationallowance());
        dto.setCustomstring(dtcat.getDescription1());
        dto.setConceptXref(dtcat.getConceptXref());
        return dto;
    }

    @Override
    @Transactional
    public ConceptMetadata getMetadata() {
        return getMetadataService().getMetadata(DuplicateTherapyDTO.class);
    }

    /**
     * Gets the metadata fdb.
     * 
     * @return the metadata fdb
     */
    @Transactional
    public ConceptMetadata getMetadataFDB() {
        return getMetadataService().getMetadata(FdbDuplicatetherapy.class);
    }

    /**
     * Retrieve Duplicate Therapy from staging based on values specified in the query.
     * 
     * @param query
     *            the query to be run
     * @return List<DuplicateTherapyDTO>. Never <code>null</code>
     * @throws CustomizationServiceException
     */
    @Transactional
    @SuppressWarnings("unchecked")
    @Override
    public List<DuplicateTherapyDTO> doGetStagingData(ConceptQuery query) {
        if (LOGGER.is   ugEnabled()) {
            LOGGER.debug("View staging custom monographs");
        }

        List<?> dtcats = null;
        if (query == null) {
            dtcats = getCustomDuplicateTherapyDAO().findAll();
        }

        dtcats = getCustomDuplicateTherapyDAO().findByQuery(query, true); // true for staging and false for FDB
        List<DuplicateTherapyDTO> res = new ArrayList<DuplicateTherapyDTO>();

        for (DuplicateTherapyVersionVa v : (List<DuplicateTherapyVersionVa>) dtcats) {
            DuplicateTherapyDTO dto = createDuplicateTherapyDTO(v);
            res.add(dto);
        }
        return res;
    }

    /**
     * Retrieve Duplicate Therapy from staging based on values specified in the query.
     * 
     * @param query
     *            the query to be run
     * @param page
     *            the page to return
     * @throws CustomizationServiceException
     * 
     * @return a <code>Page</code> object
     */
    @SuppressWarnings("unchecked")
    @Transactional
    @Override
    public Page getStagingData(String query, int page) {
        if (LOGGER.is   ugEnabled()) {
            LOGGER.debug("View Duplicate Therapy");
        }
        if (query == null || query.equals("")) {
            throw new IllegalArgumentException("required query xml string is null");
        }
        if (page < 1) {
            throw new IllegalArgumentException("required page parameter cannot be less than one");
        }
        Page retpage = new Page(null, page, page);
        retpage = getCustomDuplicateTherapyDAO().findByQueryByPage(query, page);

        List<DuplicateTherapyDTO> dtcats = new ArrayList<DuplicateTherapyDTO>();
        for (DuplicateTherapyVersionVa v : (List<DuplicateTherapyVersionVa>) retpage.getResults()) {
            DuplicateTherapyDTO dto = createDuplicateTherapyDTO(v);
            dtcats.add(dto);
        }
        retpage.setResults(dtcats);

        return retpage;
    }

    @Transactional
    @Override
    public List<DuplicateTherapyDTO> getStagingData(Long dtcId, String customString, Long customDupAllowance) {
        return createDuplicateTherapyDTOs(getCustomDuplicateTherapyDAO().find(dtcId, customString, customDupAllowance));
    }

    @Transactional
    @Override
    public List<DuplicateTherapyDTO> getStagingData(Long dtcId) {
        return createDuplicateTherapyDTOs(getCustomDuplicateTherapyDAO().find(dtcId));
    }

    @Transactional
    @Override
    public DuplicateTherapyDTO getStagingDataByVersionId(Long versionId) {
        return createDuplicateTherapyDTO(getCustomDuplicateTherapyDAO().findById(versionId));
    }

    @Override
    protected RequestCountSupport getRequestCountSupport() {
        return new GenericRequestCountSupport(DuplicateTherapyVa.class, getCustomDuplicateTherapyDAO());
    }

    @Override
    protected DuplicateTherapyVa findPersistentObject(DuplicateTherapyDTO duplicateTherapyDTO) {
        long duplicateTherapyId = duplicateTherapyDTO.getDuplicatetherapyid();
        if (duplicateTherapyId == 0) {
            LOGGER.error("Attempt to retrieve persistent Duplicate Therapy with id of 0");
            throw new CustomizationServiceException("Invalid Duplicate Therapy Object - id cannot be 0");
        }

        DuplicateTherapyVa dtcat = getCustomDuplicateTherapyDAO().findById(duplicateTherapyId, DuplicateTherapyVa.class);
        return dtcat;
    }

    @SuppressWarnings({ "rawtypes", "unchecked" })
    @Override
    protected List getCustomHistoryForField(ConceptHistoryCriteria criteria) {

        List<DuplicateTherapyVersionVa> list = getCustomDuplicateTherapyDAO().find(criteria.getId());

        if (list.size() < 1) {
            return new ArrayList();
        }
        return list;
    }

    @Override
    protected FieldHistoryLineItem getFDBHistoryforField(ConceptHistoryCriteria criteria, ConceptField conceptField,
            Object retainedFieldValue) {

        FdbDuplicatetherapy fdbDuplicateTherapy = getFdbDuplicatetherapyDAO().findById(criteria.getId());

        FieldHistoryLineItem fdbFieldHistoryLineItem = null;
        if (fdbDuplicateTherapy != null) {
            Object fdbFieldValue = conceptField.getPersistentValue(false, fdbDuplicateTherapy);
            if (!fdbFieldValue.equals(retainedFieldValue)) {
                retainedFieldValue = fdbFieldValue;
                fdbFieldHistoryLineItem = populateFieldHistoryLineItemFDB(criteria, conceptField, fdbDuplicateTherapy);
            }
        }

        return fdbFieldHistoryLineItem;
    }

    /**
     * Method to populate the DTO FieldHistoryLineItem from the Concept(s) FDB version object.
     * 
     * @param criteria
     *            - the criteria to use in search
     * @param conceptField
     *            the concept field
     * @param fdbDuplicateTherapy
     *            the fdb duplicate therapy
     * @return FieldHistoryLineItem
     */
    public final FieldHistoryLineItem populateFieldHistoryLineItemFDB(ConceptHistoryCriteria criteria,
            ConceptField conceptField, FdbDuplicatetherapy fdbDuplicateTherapy) {
        FieldHistoryLineItem fdbFieldHistoryLineItem = new FieldHistoryLineItem();

        Object fieldValue = null;

        if (conceptField.isLookup()) {

            fieldValue = determineFdbFieldValue(conceptField, fdbDuplicateTherapy);

        } else {

            fieldValue = conceptField.getPersistentValue(false, fdbDuplicateTherapy);

        }

        fdbFieldHistoryLineItem.setUser("FDB");
        if (criteria.getId() != null) {
            fdbFieldHistoryLineItem.setId((criteria.getId().toString()));
        }
        if (criteria.getVersionId() != null) {
            fdbFieldHistoryLineItem.setId((criteria.getVersionId().toString()));
        }

        fdbFieldHistoryLineItem.setStatus("FDB");
        fdbFieldHistoryLineItem.setValue(fieldValue.toString());
        return fdbFieldHistoryLineItem;
    }

    /**
     * Determine fdb field value.
     * 
     * @param conceptField
     *            the concept field
     * @param fdbDuplicateTherapy
     *            the fdb duplicate therapy
     * @return the object
     */
    private Object determineFdbFieldValue(ConceptField conceptField, FdbDuplicatetherapy fdbDuplicateTherapy) {

        return super
                .getTerminologyService()
                .getByCode(conceptField.getLookupType(), conceptField.getPersistentValue(false, fdbDuplicateTherapy).toString())
                .getUiLabel();

    }

    // shahid 9-20-2011
    @Override
    protected BusinessRulesValidation getDataValidation(ConceptMetadata meta) {
        // all required fields must be present
        BusinessRulesValidationImpl validator = new BusinessRulesValidationImpl(meta);
        validator.registerRule(
                new RegexRule(DuplicateTherapyDTO.DESCRIPTION, ".{0,255}", "it can have at most 255 characters"),
                new AlwaysFireTrigger());
        validator.registerRule(new ReferenceTextFieldRule(DuplicateTherapyDTO.REFERENCE_TEXT), new AlwaysFireTrigger());
        return validator;
    }

    @Override
    public HibernateDAO<?> getDao(boolean custom) {
        return custom ? getCustomDuplicateTherapyDAO() : getFdbDuplicatetherapyDAO();
    }

    /**
     * Gets the customDuplicateTherapyDAO.
     * 
     * @return the customDuplicateTherapyDAO
     */
    public DuplicateTherapyVersionVaDAO getCustomDuplicateTherapyDAO() {
        return customDuplicateTherapyDAO;
    }

    /**
     * Sets the customDuplicateTherapyDAO.
     * 
     * @param customDuplicateTherapyDAO
     *            the customDuplicateTherapyDAO to set
     */
    public void setCustomDuplicateTherapyDAO(DuplicateTherapyVersionVaDAO customDuplicateTherapyDAO) {
        this.customDuplicateTherapyDAO = customDuplicateTherapyDAO;
    }

    /**
     * Gets the fdbDuplicatetherapyDAO.
     * 
     * @return the fdbDuplicatetherapyDAO
     */
    public FdbDuplicatetherapyDAO getFdbDuplicatetherapyDAO() {
        return fdbDuplicatetherapyDAO;
    }

    /**
     * Sets the fdbDuplicatetherapyDAO.
     * 
     * @param fdbDuplicatetherapyDAO
     *            the fdbDuplicatetherapyDAO to set
     */
    public void setFdbDuplicatetherapyDAO(FdbDuplicatetherapyDAO fdbDuplicatetherapyDAO) {
        this.fdbDuplicatetherapyDAO = fdbDuplicatetherapyDAO;
    }

    /**
     * Retrieves the history data for a Duplicate Therapy for inclusion on the history report.
     * 
     * @param versionId
     *            the version id of the Duplicate Therapy Version being viewed
     * @return List<DuplicateTherapyDTO>
     */
    @Override
    @Transactional
    public List<DuplicateTherapyDTO> getVersionHistoryReportData(Long versionId) {
        List<DuplicateTherapyDTO> versionHistory = new ArrayList<DuplicateTherapyDTO>(0);
        try {
            DuplicateTherapyVersionVa version = customDuplicateTherapyDAO.findById(versionId);
            getVersionHistory(version, versionHistory);
            versionHistory.add(convertFdbToCustom(fdbDuplicatetherapyDAO.findById(version.getDuplicateTherapy().getDtcid())));
        } catch (Exception e) {
            LOGGER.error(e.getMessage(), e);
        }
        return versionHistory;
    }

    /**
     * A method to loop through the version history of a Duplicate Therapy and create DuplicateTherapyDTO objects from that
     * data.
     * 
     * @param dupTherapyVersion
     *            the Duplicate Therapy Version selected by the user
     * @param versionHistory
     *            the List to place the DuplicateTherapyDTO objects in
     */
    private void getVersionHistory(DuplicateTherapyVersionVa dupTherapyVersion, List<DuplicateTherapyDTO> versionHistory) {
        List<Version<DuplicateTherapy>> versions = dupTherapyVersion.getDuplicateTherapy().getVersionHistory();
        for (Version<DuplicateTherapy> version : versions) {
            versionHistory.add(createDuplicateTherapyDTOWithComment((DuplicateTherapyVersionVa) version));
        }
        Collections.reverse(versionHistory);
    }

    @Override
    public ConceptXref findXrefByFDBId(Object fdbId) {
        if (fdbId instanceof String) {
            return getConceptXrefDAO().findDTxRefByFDBId(Long.parseLong((String) fdbId));
        }

        return getConceptXrefDAO().findDTxRefByFDBId((Long) fdbId);

    }

    @Override
    protected ConceptXref createConceptXrefForCustomization(Object fdbId) {
        // find xref if it already exists.
        DuplicateTherapyXref xref = (DuplicateTherapyXref) findXrefByFDBId(fdbId);
        // if it does not exist then create a new one set the id and save it.
        if (xref == null) {
            xref = new DuplicateTherapyXref();
            xref.setDtcId((Long) fdbId);
            persistConceptXref(xref);
        }

        return xref;

    }

    @Override
    public ConceptXref createConceptXref(Object dtcId, UserComment comment) {
        DuplicateTherapyXref xref = new DuplicateTherapyXref();
        xref.setDtcId(Long.parseLong((String) dtcId));
        xref.addUserComment(comment);
        return xref;
    }

    /* (non-Javadoc)
     * @see gov.va.med.pharmacy.pecs.service.impl.AbstractPecsService#getFdbDao()
     */
    @Override
    protected FdbHibernateDAOImpl<?> getFdbDao() {
        return getFdbDuplicatetherapyDAO();
    }
    
    
}
