package gov.va.med.pharmacy.pecs.service.impl;

import gov.va.med.pharmacy.pecs.criteria.ConceptHistoryCriteria;
import gov.va.med.pharmacy.pecs.domain.Monograph;
import gov.va.med.pharmacy.pecs.domain.MonographSection.SectionCode;
import gov.va.med.pharmacy.pecs.domain.UserComment;
import gov.va.med.pharmacy.pecs.domain.Version;
import gov.va.med.pharmacy.pecs.domain.impl.fdb.FdbMonographDdimDAO;
import gov.va.med.pharmacy.pecs.domain.impl.fdb.FdbMonographDdimLite;
import gov.va.med.pharmacy.pecs.domain.impl.va.ConceptXref;
import gov.va.med.pharmacy.pecs.domain.impl.va.MonographKey;
import gov.va.med.pharmacy.pecs.domain.impl.va.MonographVersion;
import gov.va.med.pharmacy.pecs.domain.impl.va.MonographVersionDAO;
import gov.va.med.pharmacy.pecs.domain.impl.va.MonographXref;
import gov.va.med.pharmacy.pecs.domain.impl.va.ProfessionalDrugDrugMonographVa;
import gov.va.med.pharmacy.pecs.domain.impl.va.ProfessionalDrugDrugMonographVaDAO;
import gov.va.med.pharmacy.pecs.dto.MonographDTO;
import gov.va.med.pharmacy.pecs.exception.CustomizationServiceException;
import gov.va.med.pharmacy.pecs.exception.DuplicateConceptException;
import gov.va.med.pharmacy.pecs.metadata.ConceptField;
import gov.va.med.pharmacy.pecs.metadata.ConceptMetadata;
import gov.va.med.pharmacy.pecs.persistence.FdbHibernateDAOImpl;
import gov.va.med.pharmacy.pecs.persistence.HibernateDAO;
import gov.va.med.pharmacy.pecs.query.ConceptQuery;
import gov.va.med.pharmacy.pecs.query.Page;
import gov.va.med.pharmacy.pecs.rules.AlwaysFireTrigger;
import gov.va.med.pharmacy.pecs.rules.BusinessRulesValidation;
import gov.va.med.pharmacy.pecs.rules.BusinessRulesValidationImpl;
import gov.va.med.pharmacy.pecs.rules.ReferenceTextFieldRule;
import gov.va.med.pharmacy.pecs.service.MonographService;
import gov.va.med.pharmacy.pecs.service.RequestCountSupport;
import gov.va.med.pharmacy.pecs.service.dto.FieldHistoryLineItem;
import gov.va.med.pharmacy.pecs.state.Status;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.List;

import javax.annotation.Resource;

import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.BindingResult;

/**
 * MonogrpahServieImpl entity.
 * 
 * @author DOMAINOVERSD
 * @version Pecs 3.0
 * 
 * 
 */
@Service
public final class MonographServiceImpl extends AbstractPecsService<MonographDTO, Monograph> implements MonographService {

    /** The Constant FDB_P. */
    private static final String FDB_P = "FDB-P";

    /** Private Logger gets Logger from MonographServiceImpl.class */
    private static final Logger LOGGER = Logger.getLogger(MonographServiceImpl.class);

    /** Final List Array of strings helps bridge between collection api's and array api's. */
    private static final List<String> VERSIONCODES = Arrays.asList(new String[] { new ProfessionalDrugDrugMonographVa()
            .getVersionCode() });

    /** Private String Object. */
    private static final String MONO_TITLE_HEADER = "MONOGRAPH TITLE:  ";

    /** Private ProfessionalDrugDrugMonographVaDAO Object. */
    @Resource
    private ProfessionalDrugDrugMonographVaDAO monographVaDAO;

    /** Private FdbMonographDdimDAO Object. */
    @Resource
    private FdbMonographDdimDAO monographFdbDAO;

    /** Private MonographVersionDAO Object. */
    @Resource
    private MonographVersionDAO monographVersionDAO;

    /**
     * Gets the MonographVaDAO.
     * 
     * @return private monographVaDAO
     * */
    public final ProfessionalDrugDrugMonographVaDAO getMonographVaDAO() {
        return monographVaDAO;
    }

    /**
     * Set's the MonographVaDAO so outside classes and methods can access the private object.
     * 
     * @param monographVaDAO
     *            - ProfessionalDrugDrugMonographVaDAO
     * */
    public final void setMonographVaDAO(ProfessionalDrugDrugMonographVaDAO monographVaDAO) {
        this.monographVaDAO = monographVaDAO;
    }

    /**
     * Set's the MonographVersionDAP so outside classes and methods can access the private object.
     * 
     * @param monographVersionDAO
     *            object
     */
    public final void setMonographVersionDAO(MonographVersionDAO monographVersionDAO) {
        this.monographVersionDAO = monographVersionDAO;
    }

    /**
     * Allows outside classes and Methods to be able to call the Object MonographVersionDAO.
     * 
     * @return monographVersionDAO object
     */
    public final MonographVersionDAO getMonographVersionDAO() {
        return monographVersionDAO;
    }

    /**
     * Allows Outside class and methods to be able to call the Object MonographFdbDAO.
     * 
     * @param monographFdbDAO
     *            object
     * */
    public final void setMonographFdbDAO(FdbMonographDdimDAO monographFdbDAO) {
        this.monographFdbDAO = monographFdbDAO;
    }

    /**
     * Allows Outside class and methods to be able to call the Object MonographFdbDAO.
     * 
     * @return monographFdbDAO object
     * */
    public final FdbMonographDdimDAO getMonographFdbDAO() {
        return monographFdbDAO;
    }

    /**
     * Creates a New Monograph Va Record and Checks to see if it has a valid Monograph ID.
     * 
     * @param dto
     *            the monograph Id number being created
     * @param result
     *            Error Recorder
     * @return populatedAndPresistMonograph
     * */
    @Override
    @Transactional
    public final MonographDTO createNewRecord(MonographDTO dto, BindingResult result) {
        long monographId = dto.getMonographid();
        if (monographId != 0) {
            LOGGER.error("New monograph shouldn't have its id set");
            throw new CustomizationServiceException("Invalid monograph - id cannot be different from 0");
        }

        if (!validateNewRecord(dto, result)) {
            return dto;
        }

        ProfessionalDrugDrugMonographVa m = ProfessionalDrugDrugMonographVa.createMonograph();
        m.setStatus(getStateMachineFactory().getStatusByCode(Status.STATUS_NEW));
        m.setActionBy(dto.getActionBy());

        MonographXref xref = (MonographXref) dto.getConceptXref();

        if (xref != null) {
            xref = (MonographXref) createConceptXrefForCustomization(xref.getMonographKey());

        }

        dto.setConceptXref(xref);
        return populateAndPersistMonograph(m, dto, false);
    }

    /**
     * Changes the State of MonographDTO Object.
     * 
     * @param action
     *            String used in Change
     * @param record
     *            Monograph Object
     * @param result
     *            Error recorder
     * @return err returned if error is found
     * */

    @Override
    @Transactional
    public final MonographDTO applyAction(String action, MonographDTO record, BindingResult result) {
        ProfessionalDrugDrugMonographVa m = findPersistentObject(record);
        MonographVersion v = m.getCurrentVersion();
        MonographDTO crtMonograph = createMonographDTO(v);

        boolean stateChange = doApplyAction(m, record, crtMonograph, action, record.getActionUser(), result);

        if (record.getFdbId() != 0) {
            MonographXref xref = (MonographXref) findXrefByFDBId(record.getFdbId());
            record.setConceptXref(xref);
        }

        if (stateChange && !m.wasRolledBack()) {
            m.setActionBy(record.getActionBy());
        }

        record.setExportDate(null);
        return result.hasErrors() ? record : populateAndPersistMonograph(m, record, m.wasRolledBack());
    }

    /**
     * Finds the presistedObject by MonographVa Id.
     * 
     * @param monograph
     *            Object passed into the method to search for
     * @return m ProfessionalDrugDrugMonographVa found by Id
     * */
    @Override
    protected final ProfessionalDrugDrugMonographVa findPersistentObject(MonographDTO monograph) {
        long monographId = monograph.getMonographid();
        if (monographId == 0) {
            LOGGER.error("Attempt to retrieve persistent monograph with id of 0");
            throw new CustomizationServiceException("Invalid monograph - id cannot be 0");
        }
        ProfessionalDrugDrugMonographVa m = monographVaDAO.findById(monographId);
        return m;
    }

    /**
     * Populate a monograph from a DTO and persist it.
     * 
     * @param dest
     *            the monograph that needs to be populated
     * @param source
     *            the DTO the data comes from
     * @param partial
     *            a flag indicating whether the monograph should only be partially populated; that would be the case if the
     *            monograph has just been rolled back (thus most of its fields are already populated with data from the
     *            exportable version)
     * @return a DTO reflecting the data of the persisted domain object
     */
    //@formatter:off
    private MonographDTO populateAndPersistMonograph(ProfessionalDrugDrugMonographVa dest,MonographDTO source,boolean partial) {
        //@formatter:on
        if (!partial) {
            source.setMonographTitle(source.getMonographTitle().trim());

            if (!source.getMonographTitle().equals(dest.getTitle())) {
                if (existsMonographWithTitle(source.getMonographTitle())) {
                    LOGGER.error("Specified title is not unique: " + source.getMonographTitle());
                    throw new DuplicateConceptException("Custom monograph title '" + source.getMonographTitle()
                            + "' is not unique.");
                }
            }

            dest.setCustomizedMonographId(source.getFdbId());
            dest.setClinicalEffects(source.getClinicalEffects());
            dest.setDisclaimer(source.getDisclaimer());
            dest.setDiscussion(source.getDiscussion());
            dest.setMechanismOfAction(source.getMechanismOfAction());
            dest.setPatientManagement(source.getPatientManagement());
            dest.setPredisposingFactors(source.getPredisposingFactors());
            dest.setReferences(source.getReference());
            dest.setSeverityLevel(source.getSeverityLevel());
            dest.setTitle(source.getMonographTitle());

            dest.setAssignedUser(source.getAssignedTo());
            dest.setReferenceText(source.getReferenceText());
        }

        dest.setAuditUser(source.getActionBy());
        dest.setComment(source.getCurrentActionReason());
        dest.setModifiedDate(new Date());

        dest.setConceptXref(source.getConceptXref());

        monographVaDAO.save(dest);
        return createMonographDTO(dest.getCurrentVersion());
    }

    /**
     * Create a DTO out of a domain object.
     * 
     * @param source
     *            the monograph serving as a source
     * 
     * @return the populated DTO
     */
    private MonographDTO createMonographDTO(MonographVersion source) {
        MonographDTO dto = new MonographDTO();
        dto.setCurrent(source.isCurrent());
        dto.setFdbCustomMonographVaId(source.getVersionId());

        dto.setVersioncode(source.getVersionCode());
        dto.setCategorycode(source.getCategoryCode());
        dto.setActionBy(source.getActionBy());
        dto.setStatus(source.getStatus().getName());
        dto.setAssignedTo(source.getAssignedUser());
        dto.setRequestor(source.getVersionedObject().getRequestor());
        dto.setReferenceText(source.getReferenceText());

        dto.setActionReason(source.getCommentHistory());

        dto.setActionDate(source.getDate());
        dto.setExportDate(source.getExportDate());
        dto.setMonographid(source.getVersionedObject().getId());
        dto.setFdbId(source.getVersionedObject().getCustomizedMonographId());
        dto.setClinicalEffects(source.getSection(SectionCode.CLINICAL_EFFECTS).getText());
        dto.setDisclaimer(source.getSection(SectionCode.DISCLAIMER).getText());
        dto.setDiscussion(source.getSection(SectionCode.DISCUSSION).getText());
        dto.setMechanismOfAction(source.getSection(SectionCode.MECHANISM_OF_ACTION).getText());
        dto.setPatientManagement(source.getSection(SectionCode.PATIENT_MANAGEMENT).getText());
        dto.setPredisposingFactors(source.getSection(SectionCode.PREDISPOSING_FACTORS).getText());
        dto.setReference(source.getSection(SectionCode.REFERENCE).getText());
        dto.setSeverityLevel(source.getSection(SectionCode.SEVERITY_LEVEL).getText());
        String t = source.getTitle();
        dto.setMonographTitle(t);
        String displayTitle = t + " - " + source.getVersionedObject().getId();
        dto.setMonographDisplayTitle(displayTitle + " (custom)");
        dto.setMasterRecordId(source.getVersionedObject().getId());

        dto.setConceptXref(source.getVersionedObject().getConceptXref());
        return dto;
    }

    /**
     * Stores the DTO monographs in an Array List.
     * 
     * @param list
     *            basic List array type
     * @return res list object that the DTO are stored in
     */
    private List<MonographDTO> createMonographDTOs(List<MonographVersion> list) {
        ArrayList<MonographDTO> res = new ArrayList<MonographDTO>();
        for (MonographVersion v : list) {
            MonographDTO dto = createMonographDTO(v);
            res.add(dto);
        }
        return res;
    }

    /**
     * Checks to see the Monograph Exist with the current title.
     * 
     * @param title
     *            String Object passed in to see if it exist
     * 
     * @return mongraphVaDAO if it exist
     */
    @Override
    public final boolean existsMonographWithTitle(String title) {

        if (title != null && !title.equals("")) {
            if (title.indexOf("'") > 0) {

                title = title.replace("'", "''");
            }
            if (title.indexOf(".") > 0) {

                title = title.replace(".", " ");
            }
        }

        return monographVaDAO.existsMonographWithTitle(title);
    }

    /**
     * Prints out all the Monographs.
     * 
     * @return res List of monographs to be displayed
     */
    @Override
    @Transactional(readOnly = true)
    public final List<MonographDTO> getAllMonographs() {
        List<MonographDTO> res = createMonographDTOsLite(monographVaDAO.getApprovedMonographs(), true);
        res.addAll(createMonographDTOsLite(monographFdbDAO.getAllMonographs(), false));

        return res;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.service.MonographService#getAllActiveMonographs()
     */
    @Override
    @Transactional(readOnly = true)
    public final List<MonographDTO> getAllActiveMonographs() {
        List<MonographDTO> res = createMonographDTOsLite(monographVaDAO.getApprovedMonographs(), true);
        res.addAll(createMonographDTOsLite(monographFdbDAO.getAllMonographs(), false));

        return res;
    }

    /**
     * Create a list of monograph DTOs that have only the monograph id and title populated.
     * 
     * @param monographs
     *            a list of [monograph id, title] values
     * @param custom
     *            - boolean
     * @return the list of partially populated DTOs
     */
    private List<MonographDTO> createMonographDTOsLite(List<Object[]> monographs, boolean custom) {
        ArrayList<MonographDTO> res = new ArrayList<MonographDTO>();

        HashMap<Long, String> fragments = new HashMap<Long, String>();
        for (Object[] values : monographs) {
            MonographDTO dto = new MonographDTO();
            dto.setMonographid((Long) values[0]);
            String title = (String) values[1];
            if (!custom) {
                if (title.startsWith(MONO_TITLE_HEADER)) {
                    title = title.substring(MONO_TITLE_HEADER.length());
                } else {
                    String titleFragment = fragments.get(values[0]);
                    if (titleFragment == null) {
                        titleFragment = (String) values[1];
                    } else {
                        titleFragment = titleFragment + values[1];
                    }
                    fragments.put((Long) values[0], titleFragment);
                    continue;
                }
            }
            dto.setMonographTitle(title);
            dto.setDisplayTitle(custom);
            res.add(dto);
        }
        if (!custom) {
            for (MonographDTO m : res) {
                String titleFragment = fragments.get(m.getMonographid());
                if (titleFragment != null) {
                    m.setMonographTitle(m.getMonographTitle() + titleFragment);
                }
                m.setDisplayTitle(custom);
            }
        }
        return res;
    }

    /**
     * Retrieve custom monographs from staging based on values specified in the query.
     * 
     * @param query
     *            - String
     * @param page
     *            - int
     * @return Page
     */
    @Override
    @SuppressWarnings("unchecked")
    @Transactional(readOnly = true)
    public final Page getStagingData(String query, int page) {
        if (LOGGER.is   ugEnabled()) {
            LOGGER.debug("View custom monographs");
        }
        if (StringUtils.isBlank(query)) {
            throw new IllegalArgumentException("required query xml string is null");
        }
        if (page < 1) {
            throw new IllegalArgumentException("required page parameter cannot be less than one");
        }

        Page retpage = monographVersionDAO.findByQueryByPage(query, page);
        List<MonographDTO> customMonographs = createMonographDTOs(retpage.getResults());
        retpage.setResults(customMonographs);
        return retpage;
    }

    /**
     * Retrieves a Monograph version by it's identifier and returns it in a MonographVersion object.
     * 
     * @param criteria
     *            the criteria.id is used to find VA history
     * @return DoseRangeVersionVa
     */
    @Transactional(readOnly = true)
    public final List<MonographVersion> getStagingDataHistoryByVersionId(ConceptHistoryCriteria criteria) {
        List<MonographVersion> history = new ArrayList<MonographVersion>();

        MonographVersion monographVersion = getMonographVersionDAO().findById(criteria.getVersionId());

        if (monographVersion != null && monographVersion.getVersionedObject() != null) {

            criteria.setId(monographVersion.getVersionedObject().getCustomizedMonographId());

            List<Version<Monograph>> monographHistoryList = monographVersion.getVersionedObject().getVersionHistory();

            for (Version<Monograph> version : monographHistoryList) {

                MonographVersion historyVersion = (MonographVersion) version;

                history.add((historyVersion));
            }
            Collections.reverse(history);

        } else {

            LOGGER.info("no history found");

        }
        return history;
    }

    /**
     * Retrieve custom monographs from staging based on values specified in the query.
     * 
     * @param query
     *            - ConceptQuery
     * @return List<MonographVa>
     */
    @Override
    @SuppressWarnings("unchecked")
    public final List<MonographDTO> doGetStagingData(ConceptQuery query) {
        if (LOGGER.is   ugEnabled()) {
            LOGGER.debug("View staging custom monographs");
        }

        List<MonographVersion> customMonographs = null;
        customMonographs =
                (query == null ? monographVersionDAO.findAll() : (List<MonographVersion>) monographVersionDAO.findByQuery(
                        query, true));
        List<MonographDTO> res = createMonographDTOs(customMonographs);

        return res;
    }

    /**
     * Makes it possible for other classes and methods to call this object.
     * 
     * @return GenericRequestCountSupport(ProfessionalDrugDrugMonographVa.class, monographVaDAO)
     */
    @Override
    protected final RequestCountSupport getRequestCountSupport() {
        return new GenericRequestCountSupport(ProfessionalDrugDrugMonographVa.class, monographVaDAO);
    }

    /**
     * Gets the version Codes.
     * 
     * @return ArrayList<String>(versionCodes)
     */
    @Override
    @Transactional(readOnly = true)
    public final List<String> getVersionCodes() {
        return new ArrayList<String>(VERSIONCODES);
    }

    /**
     * Finds a FDB Monograph by its Id.
     * 
     * @param id
     *            long data object passed into the method
     * @return v if not null then create aDTO out of a domain object else return null
     */
    @Override
    @Transactional(readOnly = true)
    public final MonographDTO findMonographByFDBId(long id) {
        MonographVersion v = monographVaDAO.findMonographByFDBId(id);
        return v != null ? createMonographDTO(v) : null;
    }

    /**
     * Get the DAO.
     * 
     * @param custom
     *            boolean object passed into the class to check and return
     * @return custom return monographVersionDAO or monographFdbDAO
     */
    @Override
    public final HibernateDAO<?> getDao(boolean custom) {
        return custom ? monographVersionDAO : monographFdbDAO;
    }

    /**
     * ConceptMetadata getter to so other classes and methods can use this object.
     * 
     * @return getMetadataService().getMetadata(MonographVa.class)
     * 
     */
    @Override
    @Transactional
    public final ConceptMetadata getMetadata() {
        return getMetadataService().getMetadata(MonographDTO.class);
    }

    /**
     * Create monograph DTOs from FDB persistent records. This is only used for query results and has the baked-in assumption
     * that monograph query results only contain the id and title.
     * 
     * @param fdbObjects
     *            List<?> fdbobjects
     * @return createMonographDTOsLite(list, false)
     */
    @Override
    protected final List<MonographDTO> convertFdbToCustom(List<?> fdbObjects) {
        if (fdbObjects == null) {
            return Collections.emptyList();
        }

        List<Object[]> list = new ArrayList<Object[]>();

        for (Object obj : fdbObjects) {
            FdbMonographDdimLite fdbM = (FdbMonographDdimLite) obj;
            Object[] fields = new Object[2];
            fields[0] = fdbM.getMonographid();
            fields[1] = fdbM.getLinetext();
            list.add(fields);
        }

        return createMonographDTOsLite(list, false);
    }

    /**
     * Find Monograph by the version Id.
     * 
     * @param id
     *            version Id to search for and return
     * @return createMonographDTO(v)
     */
    @Override
    @Transactional(readOnly = true)
    public final MonographDTO findMonographByVersionId(long id) {
        MonographVersion v = monographVersionDAO.findById(id);
        return createMonographDTO(v);
    }

    /**
     * Find Monograph by version ID.
     * 
     * @param id
     *            version Id to search for and return
     * @return createMonographDTO(v)
     */
    @Override
    @Transactional(readOnly = true)
    public final MonographDTO findMonographById(long id) {
        MonographVersion v = monographVersionDAO.findById(id, ProfessionalDrugDrugMonographVa.class).getCurrentVersion();
        return createMonographDTO(v);
    }

    /**
     * find the FDB Monograph by id.
     * 
     * @param id
     *            the id of the monograph that you are searching for
     * @return FDB Monograph
     */
    @Override
    @Transactional(readOnly = true)
    public final MonographDTO findFdbMonographById(long id) {
        return MonographDTO.createFromFdb(monographFdbDAO.getMonographLines(id));
    }

    /**
     * Additional validations needed in this concept.
     * 
     * @param meta
     *            ConceptMetadata
     * @return BusinessRulesValidation
     */
    @Override
    protected final BusinessRulesValidation getDataValidation(ConceptMetadata meta) {
        BusinessRulesValidationImpl validator = new BusinessRulesValidationImpl(meta);
        validator.registerRule(new ReferenceTextFieldRule(MonographDTO.REFERENCE_TEXT), new AlwaysFireTrigger());
        return validator;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * gov.va.med.pharmacy.pecs.service.impl.AbstractPecsService#getCustomHistoryForField(gov.va.med.pharmacy.pecs.criteria.
     * ConceptHistoryCriteria)
     */
    @SuppressWarnings({ "rawtypes", "unchecked" })
    @Override
    protected final List getCustomHistoryForField(ConceptHistoryCriteria criteria) {

        List<MonographVersion> list = getStagingDataHistoryByVersionId(criteria);

        if (list.size() < 1) {
            return new ArrayList();
        }
        return list;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.service.impl.AbstractPecsService#getFDBHistoryforField(gov.va.med.pharmacy.pecs.criteria.
     * ConceptHistoryCriteria, gov.va.med.pharmacy.pecs.metadata.ConceptField, java.lang.Object)
     */
    @Override
    protected final FieldHistoryLineItem getFDBHistoryforField(ConceptHistoryCriteria criteria, ConceptField conceptField,
            Object retainedFieldValue) {

        FieldHistoryLineItem fdbFieldHistoryLineItem = null;
        Object fdbFieldValue = null;

        if (criteria != null && criteria.getId() != null) {

            if (criteria.getId() == 0) {
                return fdbFieldHistoryLineItem;
            }

            MonographDTO monograph = findFdbMonographById(criteria.getId());

            if (conceptField != null && conceptField.getName() != null) {
                if (conceptField.getName().equals("TITLE")) {
                    fdbFieldValue = monograph.getMonographTitle();
                }
                if (conceptField.getName().equals("CLINICAL_EFFECTS")) {
                    fdbFieldValue = monograph.getClinicalEffects();
                }
                if (conceptField.getName().equals("SEVERITY_LEVEL")) {
                    fdbFieldValue = monograph.getSeverityLevel();
                }
            }
        }

        if (retainedFieldValue != null) {

            if (!fdbFieldValue.equals(retainedFieldValue)) {

                fdbFieldHistoryLineItem = populateFieldHistoryLineItemFDB(criteria, fdbFieldValue);
                retainedFieldValue = fdbFieldValue;

            }
        }
        return fdbFieldHistoryLineItem;
    }

    /**
     * Method to populate the DTO FieldHistoryLineItem from the Concept(s) FDB version object.
     * 
     * @param criteria
     *            - the criteria to use in search
     * @param fieldValue
     *            - fieldValue
     * @return FieldHistoryLineItem
     */
    public final FieldHistoryLineItem populateFieldHistoryLineItemFDB(ConceptHistoryCriteria criteria, Object fieldValue) {
        FieldHistoryLineItem fdbFieldHistoryLineItem = new FieldHistoryLineItem();
        fdbFieldHistoryLineItem.setUser("FDB");
        if (criteria.getId() != null) {
            fdbFieldHistoryLineItem.setId((criteria.getId().toString()));
        }
        if (criteria.getVersionId() != null) {
            fdbFieldHistoryLineItem.setId((criteria.getVersionId().toString()));
        }

        fdbFieldHistoryLineItem.setStatus("FDB");
        fdbFieldHistoryLineItem.setValue(fieldValue.toString());
        return fdbFieldHistoryLineItem;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.service.MonographService#getVersionHistoryReportData(java.lang.Long)
     */
    @Override
    @Transactional
    public List<MonographDTO> getVersionHistoryReportData(Long versionId) {
        List<MonographDTO> versionHistory = new ArrayList<MonographDTO>(0);
        try {
            MonographVersion version = monographVersionDAO.findById(versionId);
            getVersionHistory(version, versionHistory);
            getFdbMonograph(versionHistory, version);
        } catch (Exception e) {
            LOGGER.error(e.getMessage(), e);
        }

        return versionHistory;
    }

    /**
     * Gets the fdb monograph.
     * 
     * @param versionHistory
     *            List<MonographDTO>
     * @param version
     *            MonographVersion
     * @return the fdb monograph
     */
    private void getFdbMonograph(List<MonographDTO> versionHistory, MonographVersion version) {
        Long fdbCustomizedMonographId = version.getVersionedObject().getCustomizedMonographId();
        if (fdbCustomizedMonographId != 0) {
            versionHistory.add(findFdbMonographById(fdbCustomizedMonographId));
        }
    }

    /**
     * Get the version history and adds the Comment column to the report.
     * 
     * @param monographVersion
     *            monographVersion
     * @param versionHistory
     *            - List<Monograph>
     * @return the version history
     */
    private void getVersionHistory(MonographVersion monographVersion, List<MonographDTO> versionHistory) {
        List<Version<Monograph>> versions = monographVersion.getVersionedObject().getVersionHistory();

        for (Version<Monograph> version : versions) {
            versionHistory.add(createMonographDTOWithComment((MonographVersion) version));
        }
        Collections.reverse(versionHistory);
    }

    /**
     * Create a MonographDTOWith Comment adds the History Report Comment to the DTO Object.
     * 
     * @param version
     *            MonographVersion
     * @return dto
     */
    private MonographDTO createMonographDTOWithComment(MonographVersion version) {
        MonographDTO dto = createMonographDTO(version);
        dto.setCurrentActionReason(version.getComment());
        return dto;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.service.impl.AbstractPecsService#createConceptXref(java.lang.Object,
     * gov.va.med.pharmacy.pecs.domain.UserComment)
     */
    @Override
    public ConceptXref createConceptXref(Object fdbId, UserComment comment) {
        MonographXref xref = new MonographXref();
        MonographDTO monograph = findFdbMonographById(Long.parseLong((String) fdbId));
        MonographKey key = populateMonographId(monograph);
        xref.setMonographKey(key);
        xref.addUserComment(comment);
        return xref;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.service.impl.AbstractPecsService#findXrefByFDBId(java.lang.Object)
     */
    @Override
    public ConceptXref findXrefByFDBId(Object fdbId) {

        MonographDTO monograph;
        MonographKey key = new MonographKey();

        if (fdbId instanceof String) {
            monograph = findFdbMonographById(Long.parseLong((String) fdbId));
            key = populateMonographId(monograph);
        } else if (fdbId instanceof Long) {
            monograph = findFdbMonographById((Long) fdbId);
            key = populateMonographId(monograph);
        } else if (fdbId instanceof MonographKey) {
            key = (MonographKey) fdbId;
        }

        return getConceptXrefDAO().findMonographxRefByFDBId(key);
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.service.impl.AbstractPecsService#createConceptXrefForCustomization(java.lang.Object)
     */
    @Override
    protected ConceptXref createConceptXrefForCustomization(Object monographKey) {
        // find xref if it already exists.
        MonographXref xref = (MonographXref) findXrefByFDBId(monographKey);
        // if it does not exist then create a new one set the id and save it.
        if (xref == null) {
            xref = new MonographXref();
            xref.setMonographKey((MonographKey) monographKey);
            persistConceptXref(xref);
        }

        return xref;
    }

    /**
     * Populate monograph id.
     * 
     * @param source
     *            the va object
     * @return a dose range key
     */
    @Override
    public MonographKey populateMonographId(MonographDTO source) {
        MonographKey monographKey = new MonographKey();
        monographKey.setLanguageId(source.getLanguageid());
        if (source.getFdbId() == 0) {
            monographKey.setMonographId(source.getMonographid());
        } else {
            monographKey.setMonographId(source.getFdbId());
        }

        monographKey.setSequenceNumber(1L);
        monographKey.setVersionCode(FDB_P);
        return monographKey;
    }

    /* (non-Javadoc)
     * @see gov.va.med.pharmacy.pecs.service.impl.AbstractPecsService#getFdbDao()
     */
    @Override
    protected FdbHibernateDAOImpl<?> getFdbDao() {
        return getMonographFdbDAO();
    }        
}
