package gov.va.med.pharmacy.pecs.service.impl;

import gov.va.med.pharmacy.pecs.domain.impl.va.SavedQueryVa;
import gov.va.med.pharmacy.pecs.domain.impl.va.UserProfile;
import gov.va.med.pharmacy.pecs.exception.CustomizationServiceException;
import gov.va.med.pharmacy.pecs.persistence.SavedQueryDAO;
import gov.va.med.pharmacy.pecs.service.SavedQueryService;

import java.util.Date;
import java.util.List;

import javax.annotation.Resource;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * The Class SavedQueryServiceImpl.
 */
@Service
public class SavedQueryServiceImpl implements SavedQueryService, InitializingBean {

    /** The logger. */
    private final Log logger = LogFactory.getLog(SavedQueryServiceImpl.class);

    /** The saved dao. */
    @Resource
    private SavedQueryDAO savedDao;

    
 
    /**
     * Save method.
     * @param profile
     *     the profile
     * @param concept
     *     the concept
     * @param query
     *     the query
     * @param queryName
     *     the query name
     *     
     */
    @Transactional
    public void save(UserProfile profile, String concept, String query, String queryName) {

        boolean enabled = true;
        SavedQueryVa savedQuery = new SavedQueryVa();
        savedQuery.setCreatedBy(profile.getUsername());
        savedQuery.setSavedQueryName(queryName);
        savedQuery.setQuerySql(query.trim());
        savedQuery.setQueryIsEnabled(true);
        savedQuery.setQueryTableName(concept);
        savedQuery.setCreatedDate(new Date());

        SavedQueryVa existingQuery = checkForExisting(savedQuery);

        if (existingQuery == null) {
            savedDao.save(savedQuery);
        } else {
            // if the query belongs to this user, just update
            existingQuery.setQueryIsEnabled(enabled);
            existingQuery.setCreatedDate(new Date());
            existingQuery.setQuerySql(query.trim());
            existingQuery.setSavedQueryName(savedQuery.getSavedQueryName());
            savedDao.merge(existingQuery);
        }
    }

    /**
     * Check for existing.
     * 
     * @param savedQuery
     *            the saved query
     * @return the saved query va
     */
    private SavedQueryVa checkForExisting(SavedQueryVa savedQuery) {
        // Determine if this query exists by sql string and table, regardless of who created the query.
        // This will accommodate the cases where user is executing Other's query, as well as
        // their own.
        List<SavedQueryVa> sqLst =
                savedDao.findBySqlTableUser(savedQuery.getQuerySql(),
                        savedQuery.getQueryTableName(), savedQuery.getCreatedBy());
        if (sqLst != null && sqLst.size() == 1) {
            return sqLst.get(0);
        }

        sqLst = savedDao.findByName(savedQuery.getSavedQueryName(), savedQuery.getCreatedBy());
        if (sqLst != null && sqLst.size() == 1) {
            return sqLst.get(0);
        }
        // If more than one query exists, return null since we don't know which one to update.
        return null;
    }

    /**
     * Deletes the saved query.
     * 
     * @param profile
     *     the profile
     * @param queryId
     *     the query id
     * @throws CustomizationServiceException
     *     the customization service exception
     */
    @Transactional
    public void deleteSavedQuery(UserProfile profile, Long queryId) throws CustomizationServiceException {
        if (logger.is   ugEnabled())
            logger.debug("Deleting ExecutedQuery association to SavedQuery query record");

        if (profile == null || queryId == null)
            throw new IllegalArgumentException("required profile, concept or queryId is null");

        // Determine if this query exists by id
        SavedQueryVa sqInDb = savedDao.findById(queryId);
        if (sqInDb == null)
            throw new CustomizationServiceException("Requested saved query does not exist by id: " + queryId);

        savedDao.delete(sqInDb);
    }

    
    /**
     * The VA saved query.
     * 
     * @param id
     *      the id
     * @throws CustomizationServiceException
     *      the customization service exception
     * @return the saved dao by id
     */
    @Transactional(readOnly = true)
    public SavedQueryVa findSavedQueryById(Long id) throws CustomizationServiceException {
        return savedDao.findById(id);
    }

  
    /**
     * Finds saved query for others by table.
     * 
     * @param tableName
     *      the table name
     * @param user
     *      the user
     * @throws CustomizationServiceException
     *      the customization service exception
     * @return the saved dao
     */
    @Transactional(readOnly = true)
    public List<SavedQueryVa> findSavedQueryForOthersByTable(String tableName, String user)
            throws CustomizationServiceException {
        return savedDao.findForOthersByTable(tableName, user);
    }

   
    /**
     * Finds saved query by user id and table.
     * 
     * @param userId
     *      the user id
     * @param tableName
     *      the table name
     * @throws CustomizationServiceException
     *      the customization service exception
     * @return the saved dao
     */
    @Transactional(readOnly = true)
    public List<SavedQueryVa> findSavedQueryByUserIdAndTable(String userId, String tableName)
            throws CustomizationServiceException {
        return savedDao.findByTableUser(tableName, userId);
    }

    /**
     * Gets the saved dao.
     * 
     * @return the saved dao
     */
    public SavedQueryDAO getSavedDao() {
        return savedDao;
    }

    /**
     * Sets the saved dao.
     * 
     * @param savedDao
     *            the new saved dao
     */
    public void setSavedDao(SavedQueryDAO savedDao) {
        this.savedDao = savedDao;
    }

    
    /**
     * The after properties set.
     * 
     * @throws Exception
     *      the Exception
     */
    public void afterPropertiesSet() throws Exception {
        if (savedDao == null) {
            throw new Exception("SavedQueryDAO has not been injected into SavedQueryServiceImpl.");
        }
    }
}
