package gov.va.med.pharmacy.pecs.service.impl;

import gov.va.med.authentication.kernel.LoginUserInfoVO;
import gov.va.med.pharmacy.pecs.domain.impl.va.CtUsersVa;
import gov.va.med.pharmacy.pecs.domain.impl.va.UserProfile;
import gov.va.med.pharmacy.pecs.exception.CustomizationServiceException;
import gov.va.med.pharmacy.pecs.persistence.UserDAO;
import gov.va.med.pharmacy.pecs.service.UserService;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * The Class UserServiceImpl.
 */
@Service
public class UserServiceImpl implements UserService, InitializingBean {

    /** The Constant logger. */
    private static final Log logger = LogFactory.getLog(UserService.class);
    // private CtUsersVaDAO ctUsersVaDAO = null;
    /** The user dao. */
    @Resource
    private UserDAO userDAO = null;

    /**
     * This method creates a new record into the CT_USERS_VA table, given information entered into the GUI. NOTE: there are
     * currently NO validation checks (except trim and uppercase), as to the validity of the information entered against the
     * VistA system. This is due to the unavailability of an API via KAAJEE to perform validation checks.
     * 
     * @param ctUsersVa
     *            The model object containing information received from the GUI.
     * @return CtUsersVa object that has had record ID and date information populated.
     * @throws CustomizationServiceException
     *             the customization service exception
     */
    @Transactional
    public CtUsersVa createRecord(CtUsersVa ctUsersVa) throws CustomizationServiceException {
        if (logger.is   ugEnabled())
            logger.debug("Creating new User record in Service Impl layer");

        if (ctUsersVa == null)
            throw new IllegalArgumentException("required user object is null");

        // Check the users table for existing record
        CtUsersVa user = userDAO.findByUserName(ctUsersVa.getUsername().toUpperCase());
        if (user != null) {
            // If there is an existing user, return the record
            user.setUserEnabled(true);
            user.setModifiedDate(new Date());
            userDAO.saveOrUpdate(user);
            return user;
        }

        // Create user
        CtUsersVa newCtUsersVa = null;
        try {
            Date now = new Date();
            ctUsersVa.setCreatedDate(now);
            ctUsersVa.setModifiedDate(now);
            ctUsersVa.setUserEnabled(true);
            Long rowid = (Long) userDAO.save(ctUsersVa);
            newCtUsersVa = userDAO.findById(rowid);
        } catch (Exception ex) {
            logger.error("Failed to create new user record due to: " + ex.getMessage(), ex);
            throw new CustomizationServiceException("Failed to create new user record", ex);
        }

        return newCtUsersVa;
    }

    /**
     * This method receives the KAAJEE LoginUserInfoVO authentication object and creates the custom tables user profile object,
     * including the determiniation and setting of the user role(s).
     * 
     * @param request
     *            HTTP session request object.
     * @return Custom tables user profile object.
     * @throws CustomizationServiceException
     *             the customization service exception
     */
    @Transactional
    public UserProfile authenticate(HttpServletRequest request) throws CustomizationServiceException {
        HttpSession session = request.getSession(false);
        LoginUserInfoVO userLoginInfo = (LoginUserInfoVO) session.getAttribute(LoginUserInfoVO.SESSION_KEY);
        if (userLoginInfo == null) {
            logger.error("KAAJEE Login User Info object could not be retrieved from the session.");
            throw new CustomizationServiceException("KAAJEE Login User Info object could not be retrieved from the session");
        }

        if (logger.is   ugEnabled())
            logger.debug("UserService received KAAJEE user login info object for: " + userLoginInfo.getUserNameDisplay());

        // Load property info
        UserProfile profile = new UserProfile();
        profile.setUserVpid(userLoginInfo.getUserVpid());
        profile.setUserDuz(userLoginInfo.getUserDuz());
        profile.setUserName01(userLoginInfo.getUserName01());
        profile.setUsername(userLoginInfo.getUserFirstName() + "_" + userLoginInfo.getUserLastName());
        profile.setUserNameDisplay(userLoginInfo.getUserNameDisplay());
        profile.setUserLastName(userLoginInfo.getUserLastName());
        profile.setUserFirstName(userLoginInfo.getUserFirstName());
        profile.setUserMiddleName(userLoginInfo.getUserMiddleName());
        profile.setUserPrefix(userLoginInfo.getUserPrefix());
        profile.setUserSuffix(userLoginInfo.getUserSuffix());
        profile.setUserDegree(userLoginInfo.getUserDegree());
        profile.setLoginStationNumber(userLoginInfo.getLoginStationNumber());
        profile.setUserParentAdministrativeFacilityStationNumber(userLoginInfo
                .getUserParentAdministrativeFacilityStationNumber());
        profile.setUserParentComputerSystemStationNumber(userLoginInfo.getUserParentComputerSystemStationNumber());
        profile.setPermittedDivisions(userLoginInfo.getPermittedNewPersonFileDivisions());
        profile.setCfDivisions(userLoginInfo.getLoginDivisionVistaProviderDivisions());

        // Load role info
        if (request.isUserInRole(UserProfile.ROLE_REQUESTOR))
            profile.setRole(UserProfile.ROLE_REQUESTOR);
        if (request.isUserInRole(UserProfile.ROLE_ADMIN))
            profile.setRole(UserProfile.ROLE_ADMIN);
        if (request.isUserInRole(UserProfile.ROLE_APPROVER))
            profile.setRole(UserProfile.ROLE_APPROVER);
        if (request.isUserInRole(UserProfile.ROLE_REL_MANAGER))
            profile.setRole(UserProfile.ROLE_REL_MANAGER);

        if (profile.getRoles().size() == 0) {
            logger.error("KAAJEE user: " + userLoginInfo.getUserNameDisplay() + " is not assigned any role(s)");
            throw new CustomizationServiceException("KAAJEE user: " + userLoginInfo.getUserNameDisplay()
                    + " is not assigned any role(s)");
        }

        return profile;
    }

    /**
     * This method loads a List of the users that have been entered in the CT_USERS_VA table. These users are in the APPROVER
     * role in the VistA system for the Custom Tables application.
     * 
     * @return List<CtUsersVa> of users that are entered in the CT_USERS_VA table.
     * @throws CustomizationServiceException
     *             the customization service exception
     */
    @Transactional
    public List<CtUsersVa> findApprovers() throws CustomizationServiceException {
        return userDAO.findByRole("APPROVER", Boolean.TRUE);
    }

    /**
     * This method performs a data update of an existing record with information that has been received from the GUI.
     * 
     * @param ctUsersVa
     *            The model object containing information received from the GUI.
     * @return CtUsersVa object that has had record ID and date information populated and updated.
     * @throws CustomizationServiceException
     *             the customization service exception
     */
    @Transactional
    public CtUsersVa modifyRecord(CtUsersVa ctUsersVa) throws CustomizationServiceException {
        if (logger.is   ugEnabled())
            logger.debug("Modifying a user record in Service Impl layer");

        if (ctUsersVa == null)
            throw new IllegalArgumentException("required user parameter is null");

        if (ctUsersVa.getCtUsersVaId() == 0)
            throw new IllegalArgumentException("User object is not valid.  The record ID cannot be determined.");

        CtUsersVa ctUserInDb = userDAO.findById(ctUsersVa.getCtUsersVaId());
        if (ctUserInDb == null) {
            logger.error("User object record does not exist in database for id: " + ctUsersVa.getCtUsersVaId());
            throw new CustomizationServiceException("User record does not exist in database for id: "
                    + ctUsersVa.getCtUsersVaId());
        }

        try {
            if (logger.is   ugEnabled())
                logger.debug("Attempting to modify user with id: " + ctUsersVa.getCtUsersVaId() + " and name: "
                        + ctUsersVa.getUsername());

            // Update database record against passed in information
            if ((ctUsersVa.getUsername() != null) && (ctUsersVa.getUsername().length() > 0))
                ctUserInDb.setUsername(ctUsersVa.getUsername());

            ctUserInDb.setModifiedDate(new Date());
            ctUserInDb.setUserEnabled(ctUsersVa.getUserEnabled());
            userDAO.save(ctUserInDb);
        } catch (Exception ex) {
            logger.error("Failed to modify user record due to: " + ex.getMessage(), ex);
            throw new CustomizationServiceException("Failed to modify user record", ex);
        }

        return ctUserInDb;
    }

    /**
     * This method is a convenience method for batch updates of CtUsersVa records for changes received from the GUI.
     * 
     * @param records
     *            List of model objects to update.
     * @return List<CtUsersVa> containing the updated records.
     * @throws CustomizationServiceException
     *             the customization service exception
     */
    @Transactional
    public List<CtUsersVa> modifyRecords(List<CtUsersVa> records) throws CustomizationServiceException {
        if (logger.is   ugEnabled())
            logger.debug("Batch modifying user records in Service Impl layer");

        if (records == null || records.size() == 0)
            throw new IllegalArgumentException("required user parameter is null");

        ArrayList<CtUsersVa> updatedModels = new ArrayList<CtUsersVa>();
        CtUsersVa updatedModel = null;
        for (CtUsersVa record : records) {
            try {
                updatedModel = modifyRecord(record);
                if (updatedModel != null)
                    updatedModels.add(updatedModel);
            } catch (Exception ex) {
                logger.error("Failed to update a user record due to: " + ex.getMessage(), ex);
            }
        }

        return updatedModels;
    }

    /**
     * Gets the user dao.
     * 
     * @return the user dao
     */
    public UserDAO getUserDAO() {
        return userDAO;
    }

    /**
     * Sets the user dao.
     * 
     * @param userDAO
     *            the new user dao
     */
    public void setUserDAO(UserDAO userDAO) {
        this.userDAO = userDAO;
    }

    /**
     * The after properties set.
     * 
     * @throws Exception
     *      the exception
     */
    public void afterPropertiesSet() throws Exception {
        if (userDAO == null) {
            throw new Exception("The UserDAO has not been injected to UserServiceImpl.");
        }
    }
}
