package gov.va.med.pharmacy.pecs.service.reports.impl;

import gov.va.med.pharmacy.pecs.domain.impl.fdb.FdbDdiminteraction;
import gov.va.med.pharmacy.pecs.domain.impl.va.DrugDrugInteractionVersionVa;
import gov.va.med.pharmacy.pecs.metadata.Concept;
import gov.va.med.pharmacy.pecs.metadata.ConceptField;
import gov.va.med.pharmacy.pecs.metadata.ConceptMetadata;
import gov.va.med.pharmacy.pecs.service.reports.IReport;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.Query;
import org.springframework.stereotype.Service;

/**
 * The Class DdiminteractionTotalCustomizationReport.
 * 
 * @author rick.bloyer
 */
@Service
public class DdiminteractionTotalCustomizationReport extends TotalCustomizationReportBase implements IReport {

    /** The Constant logger. */
    private static final Log logger = LogFactory.getLog(DdiminteractionTotalCustomizationReport.class);

    /** The default record batch size. */
    private static int DEFAULT_RECORD_BATCH_SIZE = 500;

    /** The module file name. */
    private static String MODULE_FILE_NAME = "Ddiminteraction_Total_Customization_Report";

    /** The combined hql. */
    private static String COMBINED_HQL = " from DrugDrugInteractionVersionVa t1 where t1.status.name = 'Approved' "
            + "and t1.versionedObject.exportManager.currentVersion = t1 order by "
            + "t1.versionedObject.interactionId, t1.severityLevel.code desc";

    /**
     * Default constructor.
     */
    public DdiminteractionTotalCustomizationReport() {
        super();
        fileName = MODULE_FILE_NAME;
        recordBatchSize = DEFAULT_RECORD_BATCH_SIZE;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.service.reports.impl.TotalCustomizationReportBase#findData()
     */
    @Override
    public List<List<Object>> findData() {
        List<List<Object>> reportableData = new ArrayList<List<Object>>();

        // Get required column names, comma delimited
        List<ConceptField> customColumns = getReportColumns(true);
        List<ConceptField> fdbColumns = getReportColumns(false);

        reportableData.add(getHeader(customColumns, fdbColumns));

        List<DrugDrugInteractionVersionVa> data = getReportableData();

        if (data.size() > 0) {
            for (DrugDrugInteractionVersionVa dataRow : data) {
                reportableData.add(getData(customColumns, fdbColumns, dataRow));
            }
        } else {
            reportableData.add(Arrays.asList(noDataFound));
        }
        return reportableData;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.service.reports.impl.TotalCustomizationReportBase#getFileName()
     */
    @Override
    public String getFileName() {
        return MODULE_FILE_NAME;
    }

    /**
     * Retrieves a list of ConceptField objects for the concept that is passed in. This method returns only items that are
     * marked as "isReportable" and returns them in the columnDisplayOrder (ascending) in the database.
     * 
     * @param custom
     *            the custom
     * @return the report columns
     */
    protected List<ConceptField> getReportColumns(boolean custom) {
        ConceptMetadata meta = findMetadata(Concept.DRUG_DRUG_INTERACTION.getConceptClass());
        return meta.getReportFields(custom);
    }

    /**
     * Gets the reportable data.
     * 
     * @return the reportable data
     */
    @SuppressWarnings("unchecked")
    private List<DrugDrugInteractionVersionVa> getReportableData() {
        Query query = getStagingSession().createQuery(COMBINED_HQL);
        return query.list();
    }

    /**
     * Gets the data.
     * 
     * @param customColumns
     *            the custom columns
     * @param fdbColumns
     *            the fdb columns
     * @param data
     *            the data
     * @return the data
     */
    private List<Object> getData(List<ConceptField> customColumns, List<ConceptField> fdbColumns,
            DrugDrugInteractionVersionVa data) {
        List<Object> dataValues;

        dataValues = getCustomData(customColumns, data);
        if (data.getFdbId() != null) {
            getAssociatedFdb(dataValues, fdbColumns, data.getFdbId());
        }

        return dataValues;
    }

    /**
     * Gets the custom data.
     * 
     * @param customColumns
     *            the custom columns
     * @param data
     *            the data
     * @return the custom data
     */
    private List<Object> getCustomData(List<ConceptField> customColumns, DrugDrugInteractionVersionVa data) {
        List<Object> dataList = new ArrayList<Object>();
        for (ConceptField column : customColumns) {
            dataList.add(column.getPersistentValue(true, data));
        }
        return dataList;
    }

    /**
     * Gets the associated fdb.
     * 
     * @param dataValues
     *            the data values
     * @param fdbColumns
     *            the fdb columns
     * @param fdbInteractionId
     *            the fdb interaction id
     * @return the associated fdb
     */
    private void getAssociatedFdb(List<Object> dataValues, List<ConceptField> fdbColumns, Long fdbInteractionId) {
        FdbDdiminteraction fdbDdim = getFdb(fdbInteractionId);
        if (fdbDdim != null) {
            for (ConceptField column : fdbColumns) {
                dataValues.add(column.getPersistentValue(false, fdbDdim));
            }
        }
    }

    /**
     * Gets the fdb.
     * 
     * @param fdbInteractionId
     *            the fdb interaction id
     * @return the fdb
     */
    private FdbDdiminteraction getFdb(Long fdbInteractionId) {
        String hql = "from FdbDdiminteraction where interactionid = ?";
        Query query = getStagingSession().createQuery(hql);
        query.setParameter(0, fdbInteractionId);
        @SuppressWarnings("unchecked")
        List<FdbDdiminteraction> list = query.list();
        if (list.size() == 1) {
            return list.get(0);
        }

        return null;
    }

}
