package gov.va.med.pharmacy.pecs.service.reports.impl;

import gov.va.med.pharmacy.pecs.domain.impl.fdb.FdbDosing;
import gov.va.med.pharmacy.pecs.domain.impl.va.DoseRangeKey;
import gov.va.med.pharmacy.pecs.domain.impl.va.DoseRangeVersionVa;
import gov.va.med.pharmacy.pecs.domain.impl.va.DoseRangeXref;
import gov.va.med.pharmacy.pecs.metadata.Concept;
import gov.va.med.pharmacy.pecs.metadata.ConceptField;
import gov.va.med.pharmacy.pecs.metadata.ConceptMetadata;
import gov.va.med.pharmacy.pecs.service.DoseRangeService;
import gov.va.med.pharmacy.pecs.service.reports.IReport;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.annotation.Resource;

import org.hibernate.Query;
import org.springframework.stereotype.Service;

/**
 * The Class DosingTotalCustomizationReport.
 * 
 */
@Service
public final class DosingTotalCustomizationReport extends TotalCustomizationReportBase implements IReport {

    @Resource
    private DoseRangeService doseRangeService;

    private static int DEFAULT_RECORD_BATCH_SIZE = 200;
    private static String MODULE_FILE_NAME = "Dosing_Total_Customization_Report";

    private static String MODULE_FROM_HQL = " from gov.va.med.pharmacy.pecs.domain.impl.va.DoseRangeVersionVa t1 "
            + "where t1.status.name = 'Approved' AND " + "t1.versionedObject.exportManager.currentVersion = t1 "
            + "ORDER BY t1.conceptid, t1.ageLowInDays, t1.ageHighInDays, t1.doseRoute.code, t1.doseType.code, t1.fdbdx desc";

    /**
     * Default constructor.
     */
    public DosingTotalCustomizationReport() {
        super();
        fileName = MODULE_FILE_NAME;
        recordBatchSize = DEFAULT_RECORD_BATCH_SIZE;
    }

    @Override
    public List<List<Object>> findData() {
        List<List<Object>> reportableData = new ArrayList<List<Object>>();

        // Get required column names, comma delimited
        List<ConceptField> customColumns = getReportColumns(true);
        List<ConceptField> fdbColumns = getReportColumns(false);

        reportableData.add(getHeader(customColumns, fdbColumns));

        List<DoseRangeVersionVa> data = getReportableData();

        if (data.size() > 0) {
            for (DoseRangeVersionVa dataRow : data) {
                reportableData.add(getData(customColumns, fdbColumns, dataRow));
            }
        } else {
            reportableData.add(Arrays.asList(noDataFound));
        }
        return reportableData;
    }

    @Override
    public String getFileName() {
        return MODULE_FILE_NAME;
    }

    /**
     * Retrieves a list of ConceptField objects for the concept that is passed in. This method returns only items that are
     * marked as "isReportable" and returns them in the columnDisplayOrder (ascending) in the database.
     * 
     * @param custom
     *            - boolean
     * @return List<ConceptField> -
     */
    protected List<ConceptField> getReportColumns(boolean custom) {
        ConceptMetadata meta = findMetadata(Concept.DOSE_RANGE.getConceptClass());
        return meta.getReportFields(custom);
    }

    /**
     * Gets the reportable data.
     * 
     * @return the reportable data
     */
    @SuppressWarnings("unchecked")
    private List<DoseRangeVersionVa> getReportableData() {
        Query query = getStagingSession().createQuery(MODULE_FROM_HQL);
        return query.list();
    }

    /**
     * Gets the data.
     * 
     * @param customColumns
     *            the custom columns
     * @param fdbColumns
     *            the fdb columns
     * @param data
     *            the data
     * @return the data
     */
    private List<Object> getData(List<ConceptField> customColumns, List<ConceptField> fdbColumns, DoseRangeVersionVa data) {
        List<Object> dataValues;

        dataValues = getCustomData(customColumns, data);
        getAssociatedFdb(dataValues, fdbColumns, data);

        return dataValues;
    }

    /**
     * Gets the custom data.
     * 
     * @param customColumns
     *            the custom columns
     * @param data
     *            the data
     * @return the custom data
     */
    private List<Object> getCustomData(List<ConceptField> customColumns, DoseRangeVersionVa data) {
        List<Object> dataList = new ArrayList<Object>();
        for (ConceptField column : customColumns) {
            dataList.add(column.getPersistentValue(true, data));
        }
        return dataList;
    }

    /**
     * Gets the associated fdb.
     * 
     * @param dataValues
     *            the data values
     * @param fdbColumns
     *            the fdb columns
     * @param data
     *            the data
     */
    private void getAssociatedFdb(List<Object> dataValues, List<ConceptField> fdbColumns, DoseRangeVersionVa data) {
        FdbDosing fdbDdim = getFdbDosing(data);
        if (fdbDdim != null) {
            for (ConceptField column : fdbColumns) {
                dataValues.add(column.getPersistentValue(false, fdbDdim));
            }
        }
    }

    /**
     * Gets the fdb dosing.
     * 
     * @param doseRangeVersionVa
     *            the dose range version va
     * @return the fdb dosing
     */
    private FdbDosing getFdbDosing(DoseRangeVersionVa doseRangeVersionVa) {

        if (doseRangeVersionVa == null || doseRangeVersionVa.getDoseRangeVa() == null
                || doseRangeVersionVa.getDoseRangeVa().getConceptXref() == null) {
            return null;

        }
        DoseRangeKey key = ((DoseRangeXref) doseRangeVersionVa.getDoseRangeVa().getConceptXref()).getDoseRangeKey();

        return getDoseRangeService().getFdbDosing(key);

    }

    /**
     * Gets the dose range service.
     * 
     * @return the dose range service
     */
    public DoseRangeService getDoseRangeService() {
        return doseRangeService;
    }

    /**
     * Sets the dose range service.
     * 
     * @param doseRangeService
     *            the new dose range service
     */
    public void setDoseRangeService(DoseRangeService doseRangeService) {
        this.doseRangeService = doseRangeService;
    }
}
