package gov.va.med.pharmacy.pecs.service.reports.impl;

import gov.va.med.pharmacy.pecs.domain.impl.fdb.FdbDuplicatetherapy;
import gov.va.med.pharmacy.pecs.domain.impl.va.DuplicateTherapyVersionVa;
import gov.va.med.pharmacy.pecs.metadata.Concept;
import gov.va.med.pharmacy.pecs.metadata.ConceptField;
import gov.va.med.pharmacy.pecs.metadata.ConceptMetadata;
import gov.va.med.pharmacy.pecs.service.reports.IReport;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.hibernate.Query;
import org.springframework.stereotype.Service;

/**
 * The Class DtcatTotalCustomizationReport.
 * 
 * @author rick.bloyer
 */
@Service
public class DtcatTotalCustomizationReport extends TotalCustomizationReportBase implements IReport {

    /** The default record batch size. */
    private static int DEFAULT_RECORD_BATCH_SIZE = 500;

    /** The module file name. */
    private static String MODULE_FILE_NAME = "Dtcat_Total_Customization_Report";

    /** The module from sql. */
    private static String MODULE_FROM_SQL = " FROM DuplicateTherapyVersionVa t1, FdbDuplicatetherapy t2 WHERE "
            + " t1.versionedObject.dtcid = t2.dtcid "
            + "AND t1.versionedObject.exportManager.currentVersion = t1 and t1.status.name = 'Approved'";

    /**
     * Default constructor.
     */
    public DtcatTotalCustomizationReport() {
        super();
        fileName = MODULE_FILE_NAME;
        recordBatchSize = DEFAULT_RECORD_BATCH_SIZE;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.service.reports.impl.TotalCustomizationReportBase#findData()
     */
    @Override
    public List<List<Object>> findData() {
        List<List<Object>> reportableData = new ArrayList<List<Object>>();

        // Get required column names, comma delimited
        List<ConceptField> customColumns = getReportColumns(true);
        List<ConceptField> fdbColumns = getReportColumns(false);

        reportableData.add(getHeader(customColumns, fdbColumns));

        List<Object[]> data = getReportableData();

        if (data.size() > 0) {
            for (Object[] dataRow : data) {
                reportableData.add(getData(customColumns, fdbColumns, dataRow));
            }
        } else {
            reportableData.add(Arrays.asList(noDataFound));
        }
        return reportableData;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.service.reports.impl.TotalCustomizationReportBase#getFileName()
     */
    @Override
    public String getFileName() {
        return MODULE_FILE_NAME;
    }

    /**
     * Retrieves a list of ConceptField objects for the concept that is passed in. This method returns only items that are
     * marked as "isReportable" and returns them in the columnDisplayOrder (ascending) in the database.
     * 
     * @param custom
     *            the custom
     * @return the report columns
     */
    protected List<ConceptField> getReportColumns(boolean custom) {
        ConceptMetadata meta = findMetadata(Concept.DUPLICATE_THERAPY.getConceptClass());
        return meta.getReportFields(custom);
    }

    /**
     * Gets the reportable data.
     * 
     * @return the reportable data
     */
    @SuppressWarnings("unchecked")
    private List<Object[]> getReportableData() {
        Query query = getStagingSession().createQuery(MODULE_FROM_SQL);
        return query.list();
    }

    /**
     * Gets the data.
     * 
     * @param customColumns
     *            the custom columns
     * @param fdbColumns
     *            the fdb columns
     * @param data
     *            the data
     * @return the data
     */
    private List<Object> getData(List<ConceptField> customColumns, List<ConceptField> fdbColumns, Object[] data) {
        List<Object> dataValues;

        dataValues = getCustomData(customColumns, (DuplicateTherapyVersionVa) data[0]);
        getAssociatedFdb(dataValues, fdbColumns, (FdbDuplicatetherapy) data[1]);

        return dataValues;
    }

    /**
     * Gets the custom data.
     * 
     * @param customColumns
     *            the custom columns
     * @param data
     *            the data
     * @return the custom data
     */
    private List<Object> getCustomData(List<ConceptField> customColumns, DuplicateTherapyVersionVa data) {
        List<Object> dataList = new ArrayList<Object>();
        for (ConceptField column : customColumns) {
            dataList.add(column.getPersistentValue(true, data));
        }
        return dataList;
    }

    /**
     * Gets the associated fdb.
     * 
     * @param dataValues
     *            the data values
     * @param fdbColumns
     *            the fdb columns
     * @param fdbDuplicateTherapy
     *            the fdb duplicate therapy
     * @return the associated fdb
     */
    private void getAssociatedFdb(List<Object> dataValues, List<ConceptField> fdbColumns,
            FdbDuplicatetherapy fdbDuplicateTherapy) {
        if (fdbDuplicateTherapy != null) {
            for (ConceptField column : fdbColumns) {
                dataValues.add(column.getPersistentValue(false, fdbDuplicateTherapy));
            }
        }
    }
}
