package gov.va.med.pharmacy.pecs.service.reports.impl;

import gov.va.med.pharmacy.pecs.domain.impl.fdb.FdbMonographDdim;
import gov.va.med.pharmacy.pecs.domain.impl.va.MonographVersion;
import gov.va.med.pharmacy.pecs.metadata.Concept;
import gov.va.med.pharmacy.pecs.metadata.ConceptField;
import gov.va.med.pharmacy.pecs.metadata.ConceptMetadata;
import gov.va.med.pharmacy.pecs.service.reports.IReport;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.apache.log4j.Logger;
import org.hibernate.Query;
import org.springframework.stereotype.Service;

/**
 * The Class MonographTotalCustomizationReport.
 * 
 * @author rick.bloyer
 */
@Service
public class MonographTotalCustomizationReport extends TotalCustomizationReportBase implements IReport {

    /** The Constant logger. */
    private static final Logger logger = Logger.getLogger(MonographTotalCustomizationReport.class);

    /** The default record batch size. */
    private static int DEFAULT_RECORD_BATCH_SIZE = 500;

    /** The module file name. */
    private static String MODULE_FILE_NAME = "Monograph_Total_Customization_Report";

    /** The module from hql. */
    private static String MODULE_FROM_HQL = " from MonographVersion t1, FdbMonographDdim t2 where "
            + "t1.versionedObject.customizedMonographId = t2.id.monographid " + "and t1.status.name = 'Approved'"
            + " and t1.versionedObject.exportManager.currentVersion = t1 and t2.sectioncode='T' "
            + "order by t1.versionedObject.id desc";

    /** The va module from hql. */
    private static String VA_MODULE_FROM_HQL = " from MonographVersion t1 where t1.versionedObject.customizedMonographId = 0 "
            + "and t1.status.name = 'Approved' "
            + "and t1.versionedObject.exportManager.currentVersion = t1 order by t1.versionedObject.id desc";

    /**
     * Default constructor.
     */
    public MonographTotalCustomizationReport() {
        super();
        fileName = MODULE_FILE_NAME;
        recordBatchSize = DEFAULT_RECORD_BATCH_SIZE;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.service.reports.impl.TotalCustomizationReportBase#findData()
     */
    @Override
    public List<List<Object>> findData() {
        List<List<Object>> reportableData = new ArrayList<List<Object>>();
        boolean dataNotFound = false;

        // Get required column names, comma delimited
        List<ConceptField> customColumns = getReportColumns(true);
        List<ConceptField> fdbColumns = getReportColumns(false);

        reportableData.add(getHeader(customColumns, fdbColumns));

        List<Object[]> data = getOverrideData(MODULE_FROM_HQL);

        if (data.size() > 0) {
            for (Object[] dataRow : data) {
                reportableData.add(getData(customColumns, fdbColumns, dataRow));
            }
        } else {
            dataNotFound = true;
        }

        List<MonographVersion> data2 = getReportableData(VA_MODULE_FROM_HQL);

        if (data2.size() > 0) {
            for (MonographVersion dataRow : data2) {
                reportableData.add(getData(customColumns, fdbColumns, dataRow));
            }
        } else {
            if (dataNotFound) {
                reportableData.add(Arrays.asList(noDataFound));
            }
        }
        return reportableData;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.service.reports.impl.TotalCustomizationReportBase#getFileName()
     */
    @Override
    public String getFileName() {
        return MODULE_FILE_NAME;
    }

    /**
     * Retrieves a list of ConceptField objects for the concept that is passed in. This method returns only items that are
     * marked as "isReportable" and returns them in the columnDisplayOrder (ascending) in the database.
     * 
     * @param custom
     *            the custom
     * @return the report columns
     */
    protected List<ConceptField> getReportColumns(boolean custom) {
        ConceptMetadata meta = findMetadata(Concept.PROFESSIONAL_MONOGRAPH.getConceptClass());
        return meta.getReportFields(custom);
    }

    /**
     * Gets the override data.
     * 
     * @param hql
     *            the hql
     * @return the override data
     */
    @SuppressWarnings("unchecked")
    private List<Object[]> getOverrideData(String hql) {
        Query query = getStagingSession().createQuery(hql);
        return query.list();
    }

    /**
     * Gets the reportable data.
     * 
     * @param hql
     *            the hql
     * @return the reportable data
     */
    @SuppressWarnings("unchecked")
    private List<MonographVersion> getReportableData(String hql) {
        Query query = getStagingSession().createQuery(hql);
        return query.list();
    }

    /**
     * Gets the data.
     * 
     * @param customColumns
     *            the custom columns
     * @param fdbColumns
     *            the fdb columns
     * @param data
     *            the data
     * @return the data
     */
    private List<Object> getData(List<ConceptField> customColumns, List<ConceptField> fdbColumns, MonographVersion data) {
        List<Object> dataValues;

        dataValues = getCustomData(customColumns, data);

        return dataValues;
    }

    /**
     * Gets the data.
     * 
     * @param customColumns
     *            the custom columns
     * @param fdbColumns
     *            the fdb columns
     * @param data
     *            the data
     * @return the data
     */
    private List<Object> getData(List<ConceptField> customColumns, List<ConceptField> fdbColumns, Object[] data) {
        List<Object> dataValues;

        dataValues = getCustomData(customColumns, (MonographVersion) data[0]);
        getAssociatedFdb(dataValues, fdbColumns, (FdbMonographDdim) data[1]);

        return dataValues;
    }

    /**
     * Gets the custom data.
     * 
     * @param customColumns
     *            the custom columns
     * @param data
     *            the data
     * @return the custom data
     */
    private List<Object> getCustomData(List<ConceptField> customColumns, MonographVersion data) {
        List<Object> dataList = new ArrayList<Object>();
        for (ConceptField column : customColumns) {
            dataList.add(column.getPersistentValue(true, data));
        }
        return dataList;
    }

    /**
     * Gets the associated fdb.
     * 
     * @param dataValues
     *            the data values
     * @param fdbColumns
     *            the fdb columns
     * @param fdbMonograph
     *            the fdb monograph
     * @return the associated fdb
     */
    private void getAssociatedFdb(List<Object> dataValues, List<ConceptField> fdbColumns, FdbMonographDdim fdbMonograph) {
        if (fdbMonograph != null) {
            for (ConceptField column : fdbColumns) {
                dataValues.add(column.getPersistentValue(false, fdbMonograph));
            }
        }
    }
}
