package gov.va.med.pharmacy.pecs.service.reports.impl;

import gov.va.med.pharmacy.pecs.metadata.ConceptField;
import gov.va.med.pharmacy.pecs.metadata.ConceptMetadata;
import gov.va.med.pharmacy.pecs.service.MetadataService;
import gov.va.med.pharmacy.pecs.service.reports.IReport;

import java.util.ArrayList;
import java.util.List;

import javax.annotation.Resource;

import org.apache.log4j.Logger;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.springframework.beans.factory.InitializingBean;

/**
 * The Class TotalCustomizationReportBase.
 */
public abstract class TotalCustomizationReportBase implements IReport, InitializingBean {

    /** The Constant logger. */
    private static final Logger logger = Logger.getLogger(TotalCustomizationReportBase.class);

    /** The staging session factory. */
    @Resource(name = "pecsSessionFactory")
    protected SessionFactory stagingSessionFactory = null;

    /** The metadata service. */
    @Resource
    protected MetadataService metadataService;

    /** The default delimiter. */
    protected static String DEFAULT_DELIMITER = ",";

    /** The preformat delimiter. */
    protected static char PREFORMAT_DELIMITER = '|';

    /** The output delimiter. */
    protected static char OUTPUT_DELIMITER = '|';

    /** The default crlf. */
    protected static String DEFAULT_CRLF = "\r\n";

    /** The default catenator. */
    protected static String DEFAULT_CATENATOR = "||'|'||";

    /** The default file extension. */
    protected static String DEFAULT_FILE_EXTENSION = ".csv";

    /** The DEFAUL t_ tabl e1_ alias. */
    protected static String DEFAULT_TABLE1_ALIAS = "t1";

    /** The DEFAUL t_ tabl e2_ alias. */
    protected static String DEFAULT_TABLE2_ALIAS = "t2";

    /** The delimiter. */
    protected String delimiter = null;

    /** The file path. */
    protected String filePath = null;

    /** The file name. */
    protected String fileName = null;

    /** The file extension. */
    protected String fileExtension = null;

    /** The record batch size. */
    protected int recordBatchSize;

    /** The no data found. */
    protected final Object[] noDataFound = { "No data found." };

    /**
     * Instantiates a new total customization report base.
     */
    public TotalCustomizationReportBase() {
        delimiter = DEFAULT_DELIMITER;
        fileExtension = DEFAULT_FILE_EXTENSION;
        filePath = System.getProperty("java.io.tmpdir");
    }

    /**
     * Finds the data.
     * 
     * @return the list
     */
    public abstract List<List<Object>> findData();

    /**
     * Gets the file name.
     * 
     * @return the file name
     */
    public abstract String getFileName();

    /**
     * Gets the header.
     * 
     * @param customColumns
     *            the custom columns
     * @param fdbColumns
     *            the fdb columns
     * @return the header
     */
    protected List<Object> getHeader(List<ConceptField> customColumns, List<ConceptField> fdbColumns) {
        List<Object> headers = new ArrayList<Object>();
        for (ConceptField column : customColumns) {
            headers.add(column.getDisplayName());
        }
        for (ConceptField column : fdbColumns) {
            headers.add(column.getDisplayName());
        }
        return headers;
    }

    /**
     * Gets the staging session.
     * 
     * @return the staging session
     */
    protected Session getStagingSession() {
        return stagingSessionFactory.getCurrentSession();
    }

    /**
     * Sets the staging session factory.
     * 
     * @param stagingSessionFactory
     *            the new staging session factory
     */
    public void setStagingSessionFactory(SessionFactory stagingSessionFactory) {
        this.stagingSessionFactory = stagingSessionFactory;
    }

    /**
     * Sets the metadata service.
     * 
     * @param metadataService
     *            the new metadata service
     */
    public void setMetadataService(MetadataService metadataService) {
        this.metadataService = metadataService;
    }

    /**
     * Find metadata.
     * 
     * @param conceptClass
     *            the concept class
     * @return the concept metadata
     */
    public ConceptMetadata findMetadata(Class<?> conceptClass) {
        return metadataService.getMetadata(conceptClass);
    }

    /**
     * The after properties set.
     * 
     * @throws Exception
     *             the Exception
     */
    public void afterPropertiesSet() throws Exception {
        if (stagingSessionFactory == null) {
            throw new Exception("StagingSessionFactory has not been injected into TotalCustomizationReportBase.");
        }
        if (metadataService == null) {
            throw new Exception("MetadataService has not been injected into TotalCustomizationReportBase.");
        }
    }
}
