dojo.require("dojox.grid.EnhancedGrid");
dojo.require("dojox.grid.enhanced.plugins.IndirectSelection");

/**
 * Creates a new Dojo EnhancedGrid as a child of the parentContainer.
 * 
 * @param servlet -
 *            The servlet to use for AJAX calls. Usually this is 'pecs'.
 * @param parentContainer -
 *            The container that will be the parent for this grid. Usually can
 *            be a DIV.
 * @param conceptType -
 *            Should be either "VA" or "FDB" depending upon what data will
 *            display in the grid.
 * @param conceptName -
 *            The String representation of the concept in the
 *            gov.va.med.pharmacy.pecs.web.Concept Enum.
 * @param selectMode -
 *            The selectionMode for the grid. Can be one of four values: "none",
 *            "single", "multiple" or "extended".
 * @param dataSourceURL -
 *            The URL and query criteria for obtaining the data to display in
 *            the grid.
 * @param hasSelect -
 *            A boolean entry to indicate if the DTO to be displayed in this
 *            Grid has a 'Select' property.
 */
function dataGridInit(servlet, parentContainer, conceptType, conceptName,
		selectMode, dataSourceURL, hasSelect, noResultsMessage) {
	try {
		var gridData = buildDataSource("/ct/" + servlet + "/" + dataSourceURL);

		var gridId = parentContainer + 'Grid';

		var grid = dijit.byId(gridId);

		var noDataMessage = "Your Query Returned No results";
		if (noResultsMessage != null && noResultsMessage != "") {
			noDataMessage = noResultsMessage;
		}

		// If the DataGrid already exists, just clear any selected rows and
		// replace the store.
		if (grid != null) {
			if (grid.selection != null) {
				grid.selection.clear();
			}
			grid.setStore(gridData);
		} else {
			// DataGrid does not exist.
			var gridLayout = buildLayout(servlet, conceptType, conceptName,
					parentContainer, hasSelect);
			grid = new dojox.grid.DataGrid({
				id : gridId,
				clientSort : true,
				rowsPerPage : 100,
				keepRows : 300,
				showTitle : true,
				columnReordering : true,
				loadingMessage : "Query In Progress...",
				noDataMessage : noDataMessage,
				onFetchError : gridFetchError,
				selectionMode : selectMode,
				selectable : true,
				canSort : canSort,
				onKeyEvent : onKeyEvent
			}, document.createElement('div'));
			dojo.byId(parentContainer).appendChild(grid.domNode);
			
			grid.setStore(gridData);
			grid.setStructure(gridLayout);
			grid.startup();
			dojo.connect(grid, "onCellDblClick", function(event) {
				//console.log(event);
				//console.log(event.target);
			});
		}
	} catch (err) {
		var txt = "An error occured while building the dataGrid.  The error is: "
				+ err.message + ".";
		alert(txt);
	}

};

/**
 * Creates a new Dojo DataGrid as a child of the parentContainer.
 * 
 * @param servlet -
 *            The servlet to use for AJAX calls. Usually this is 'pecs'.
 * @param parentContainer -
 *            The container that will be the parent for this grid. Usually can
 *            be a DIV.
 * @param dataSourceURL -
 *            The URL and query criteria for obtaining the data to display in
 *            the grid.
 * @param columnHeader -
 *            The header value to display for the number of "Assigned" requests.
 */
function fdbDrugPairDataGridInit(servlet, parentContainer, dataSourceURL) {

	var gridData = buildJsonQueryRestStore("/ct/" + servlet + "/"
			+ dataSourceURL);

	var gridId = parentContainer + 'Grid';

	var grid = dijit.byId(gridId);

	// If the DataGrid already exists, just clear any selected rows and
	// replace the store.
	if (grid != null) {
		if (grid.selection != null) {
			grid.selection.clear();
		}
		grid.setStore(gridData);
	} else {
		var plugins = {
			/*
			 * pagination: { pageSizes: ["10", "25", "50", "100", "All"],
			 * description: "30%", sizeSwitch: "260px", pageStepper: "30em",
			 * gotoButton: true, maxPageStep: 7, position: "bottom" },
			 */
			indirectSelection : {
				headerSelector : false,
				styles : "text-align: center;"
			}
		};

		// DataGrid does not exist.
		var gridLayout = buildFdbPairLayout();
		grid = new dojox.grid.EnhancedGrid({
			id : gridId,
			showTitle : true,
			columnReordering : false,
			loadingMessage : "Query In Progress...",
			noDataMessage : "Your Query Returned No results",
			onFetchError : gridFetchError,
			selectable : true,
			selectionMode : "extended",
			canSort : function(index) {
				return true;
			},
			plugins : plugins
		}, document.createElement('div'));
		if (parentContainer == null) {

			return;
		}
		dojo.byId(parentContainer).appendChild(grid.domNode);
		grid.setStore(gridData);
		grid.setStructure(gridLayout);

		grid.startup();

	}

};

/**
 * Creates a new Dojo DataGrid as a child of the parentContainer.
 * 
 * @param servlet -
 *            The servlet to use for AJAX calls. Usually this is 'pecs'.
 * @param parentContainer -
 *            The container that will be the parent for this grid. Usually can
 *            be a DIV.
 * @param dataSourceURL -
 *            The URL and query criteria for obtaining the data to display in
 *            the grid.
 * @param columnHeader -
 *            The header value to display for the number of "Assigned" requests.
 */
function assignedDataGridInit(servlet, parentContainer, dataSourceURL,
		columnHeader) {
	try {
		var gridData = buildDataSource("/ct/" + servlet + "/" + dataSourceURL);

		var gridId = parentContainer + 'Grid';
		var jsGridId = "js" + gridId;

		var grid = dijit.byId(gridId);

		// If the DataGrid already exists, just clear any selected rows and
		// replace the store.
		if (grid != null) {
			if (grid.selection != null) {
				grid.selection.clear();
			}
			grid.setStore(gridData);
		} else {
			// DataGrid does not exist.
			var gridLayout = buildAssignedLayout(servlet, parentContainer,
					columnHeader);
			grid = new dojox.grid.DataGrid({
				id : gridId,
				showTitle : true,
				columnReordering : false,
				loadingMessage : "Query In Progress...",
				noDataMessage : "Your Query Returned No results",
				onFetchError : gridFetchError,
				selectionMode : 'single',
				selectable : true,
				sort : false,
				onKeyEvent : onKeyEvent
			}, document.createElement('div'));
			dojo.byId(parentContainer).appendChild(grid.domNode);

			while (gridData.length > 0) {
				if (gridData = null) {
					gridData == '&nbsp';
				}
			}

			grid.setStore(gridData);
			grid.setStructure(gridLayout);

			grid.startup();
		}
	} catch (err) {
		var txt = "An error occured while building the dataGrid.  The error is: "
				+ err.message + ".";
		alert(txt);
	}

};

/**
 * Creates a new Dojo DataGrid as a child of the parentContainer. This DataGrid
 * is used to show the number of Requests in PECS.
 * 
 * @param servlet -
 *            The servlet to use for AJAX calls. Usually this is 'pecs'.
 * @param parentContainer -
 *            The container that will be the parent for this grid. Usually can
 *            be a DIV.
 * @param dataSourceURL -
 *            The URL and query criteria for obtaining the data to display in
 *            the grid.
 */
function workListDataGridInit(servlet, parentContainer, dataSourceURL) {
	try {
		var gridData = buildDataSource("/ct/" + servlet + "/" + dataSourceURL);

		var gridId = parentContainer + 'Grid';
		

		var grid = dijit.byId(gridId);

		// If the DataGrid already exists, just clear any selected rows and
		// replace the store.
		if (grid != null) {
			if (grid.selection != null) {
				grid.selection.clear();
			}
			grid.setStore(gridData);
		} else {
			// DataGrid does not exist.
			var gridLayout = buildWorkListLayout(servlet, parentContainer);
			grid = new dojox.grid.DataGrid({
				id : gridId,
				showTitle : true,
				columnReordering : false,
				loadingMessage : "Query In Progress...",
				noDataMessage : "Your Query Returned No results",
				onFetchError : gridFetchError,
				selectionMode : 'single',
				selectable : true,
				sort : false,
				onKeyEvent : onKeyEvent
			}, document.createElement('div'));
			dojo.byId(parentContainer).appendChild(grid.domNode);
			grid.setStore(gridData);
			grid.setStructure(gridLayout);

			grid.startup();
		}
	} catch (err) {
		var txt = "An error occured while building the dataGrid.  The error is: "
				+ err.message + ".";
		alert(txt);
	}

};

function onKeyEvent(event) {
	// I don't know why having this function enables the Enter key to submit
	// links, but
	// for some reason it does.
	return false;
}

/**
 * Checks the current layout to see if the column being sorted is the 'Select'
 * column. If it is the 'Select', do not allow the sort.
 * 
 * @param index
 * @returns {Boolean}
 */
function canSort(index) {
	if (this.structure[Math.abs(index) - 1].field == 'select') {
		return false;
	}
	if (this.structure[Math.abs(index) - 1].field == 'conceptDescription') {
		return false;
	}
	if (this.structure[Math.abs(index) - 1].field == 'clinicaleffectcode2') {
		return false;
	}
	return true;
}



/**
 * Expects the "entry" to be a JavaScript object with a "value" and "label"
 * property. If there is no "value" for the anchor, just the "label" is
 * returned.
 * 
 * @param entry
 * @returns {String}
 */
function formatSelectable(entry) {
	if (entry["value"] == null || entry["value"] == "null"
			|| entry["value"] == "") {
		return entry["label"];
	}
	return "<a href=\"" + encodeURI(entry["value"]) + "\" >" + entry["label"]
			+ "</a>";
};

/**
 * Retrieves the FieldMetadata from the server for the tableClass and builds the
 * layout based upon the FieldMetadata for displaying the DataGrid.
 * 
 * @param servlet
 * @param conceptType -
 *            Should be either "VA" or "FDB" depending upon what data will
 *            display in the grid.
 * @param conceptName -
 *            The String representation of the concept in the
 *            gov.va.med.pharmacy.pecs.web.Concept Enum.
 * @param target -
 *            The target for error messages.
 * @param hasSelect -
 *            A boolean entry to indicate if the DTO to be displayed in this
 *            Grid has a 'Select' property.
 * @returns {Array}
 */
function buildLayout(servlet, conceptType, conceptName, target, hasSelect) {
	var layout = new Array();
	if (hasSelect) {
		var obj = new Object();
		obj["field"] = 'select';
		obj["name"] = 'Select';
		obj["width"] = "75px";
		obj["hidden"] = false;
		obj["formatter"] = formatSelectable;
		obj["dnd"] = false;
		layout.push(obj);
	}

	// The parameters to pass to xhrGet, the url, how to handle it, and the
	// callbacks.
	var xhrArgs = {
		url : "/ct/" + servlet + "/fieldMetadata/" + conceptType + "/"
				+ conceptName,
		handleAs : "json",
		sync : true,
		load : function(data) {
			dojo
					.forEach(
							data,
							function(entry, i) {
								var obj = new Object();
								obj["field"] = entry['propertyName'];
								obj["name"] = entry['displayName'];
								obj["hidden"] = (entry['includeInQueryResults']) ? false
										: true;
								obj["width"] = "200px";
								var dataTypeVal = entry['dataType'];
								if ("Date" == dataTypeVal) {
									obj["formatter"] = formatTimestamp;
								}
								else if ("Timestamp" == dataTypeVal) {
									obj["formatter"] = formatTimestamp;
								}
								else if(dojo.isIE){
									obj["formatter"] = formatIeGrid;
								}

								layout.push(obj);
							});
		},
		error : function(error) {
			dojo.byId(target).innerHTML = "An unexpected error occurred: "
					+ error.message;
		}
	};

	var formatMono = function(value, rowIndex) {
		var link = "/ct/tables/monograph/professional/CreateWizardLoadAction.html?recordId="
				+ value + "&buttonValue=Submit";
		return "<a href=\"" + link + "\" class=\"monograph\">" + value + "</a>";

	};
	var formatFdbId = function(value, rowIndex) {
		var link = "/ct/pecs/drugdrug/interaction/FDB/" + value;
		return "<a href=\"" + link + "\" >" + value + "</a>";

	};
	var formatInteractionId = function(value, rowIndex) {
		var link = "/ct/pecs/drugPairCustomization/show?interactionId=" + value;
		return "<a href=\"" + link + "\" >" + value + "</a>";

	};

	// Call xhrGet
	var deferred = dojo.xhrGet(xhrArgs);
	return layout;
}

/**
 * Builds the layout for a "Assigned" WorkList DataGrid.
 * 
 * @returns {Array}
 */
function buildFdbPairLayout() {

	var myStructure = [ /*
						 * { type : "dojox.grid._CheckBoxSelector" },
						 */
	[ {
		name : "key",
		field : "key",
		hidden : true
	},

	{
		name : "Routed Generic #1 Description",
		field : "rtgenid1Desc",
		width : "auto"
	},

	{

		name : "Routed Generic #2 Description",
		field : "rtgenid2Desc",
		width : "auto"
	} ]

	];

	return myStructure;

}

/**
 * Builds the layout for a "Assigned" WorkList DataGrid.
 * 
 * @returns {Array}
 */
function buildAssignedLayout(servlet, target, columnHeader) {
	var layout = new Array();

	var obj = new Object();
	obj["field"] = 'concept';
	obj["name"] = 'Concept';
	obj["width"] = "200px";
	layout.push(obj);

	obj = new Object();
	obj["field"] = 'assigned';
	obj["name"] = columnHeader;
	obj["width"] = '102px';
	obj["formatter"] = formatSelectable;
	layout.push(obj);

	return layout;
}

/**
 * Builds the layout for a Request WorkList DataGrid.
 * 
 * @returns {Array}
 */
function buildWorkListLayout(servlet, target) {
	var layout = new Array();

	var obj = new Object();
	obj["field"] = 'concept';
	obj["name"] = 'Concept';
	obj["width"] = "200px";
	layout.push(obj);

	obj = new Object();
	obj["field"] = 'newRequests';
	obj["name"] = 'New';
	obj["width"] = '75px';
	obj["formatter"] = formatSelectable;
	layout.push(obj);

	obj = new Object();
	obj["field"] = 'modified';
	obj["name"] = "Modified";
	obj["width"] = '75px';
	obj["formatter"] = formatSelectable;
	layout.push(obj);

	obj = new Object();
	obj["field"] = 'reviewed';
	obj["name"] = "Reviewed";
	obj["width"] = '75px';
	obj["formatter"] = formatSelectable;
	layout.push(obj);

	obj = new Object();
	obj["field"] = 'approved';
	obj["name"] = "Approved";
	obj["width"] = '75px';
	obj["formatter"] = formatSelectable;
	layout.push(obj);

	obj = new Object();
	obj["field"] = 'rejected';
	obj["name"] = "Rejected";
	obj["width"] = '75px';
	obj["formatter"] = formatSelectable;
	layout.push(obj);

	obj = new Object();
	obj["field"] = 'deleted';
	obj["name"] = "Deleted";
	obj["width"] = '75px';
	obj["formatter"] = formatSelectable;
	layout.push(obj);

	obj = new Object();
	obj["field"] = 'total';
	obj["name"] = "All";
	obj["width"] = '75px';
	obj["formatter"] = formatSelectable;
	layout.push(obj);

	return layout;
}

function dataSourceOnComplete(items, resultSet) {

}

function buildDataSource(dataSourceURL, query) {
	try {
		var dataSource = new dojox.data.JsonQueryRestStore({
			target : dataSourceURL,
			idAttribute : "key"
		});
		// dataSource.fetch({onComplete: dataSourceOnComplete});
		return dataSource;
	} catch (err) {
		alert(err.message);
	}
}
