dojo.require("dojox.data.ClientFilter");
dojo.require("dojox.grid.EnhancedGrid");

/**
 * Creates a new Dojo EnhancedGrid as a child of the parentContainer.
 * 
 * @param servlet -
 *            The servlet to use for AJAX calls. Usually this is 'pecs'.
 * @param parentContainer -
 *            The container that will be the parent for this grid. Usually can
 *            be a DIV.
 * @param conceptType -
 *            Should be either "VA" or "FDB" depending upon what data will
 *            display in the grid.
 * @param conceptName -
 *            The String representation of the concept in the
 *            gov.va.med.pharmacy.pecs.web.Concept Enum.
 * @param selectMode -
 *            The selectionMode for the grid. Can be one of four values: "none",
 *            "single", "multiple" or "extended".
 * @param dataSourceURL -
 *            The URL and query criteria for obtaining the data to display in
 *            the grid.
 * @param hasSelect -
 *            A boolean entry to indicate if the DTO to be displayed in this
 *            Grid has a 'Select' property.
 */
function selectableDataGrid(servlet, parentContainer, conceptType, conceptName, selectMode, dataSourceURL, hasSelect,
        noResultsMessage) {
    //console.log(dojo.version);
    try {
        var gridData = buildJsonQueryRestStore("/ct/" + servlet + "/" + dataSourceURL);

        var gridId = parentContainer + 'Grid';

        var grid = dijit.byId(gridId);

        var noDataMessage = "Either no drug pairs exist for this custom interaction or there are no drug pairs for the current Action Status filter.  Please update the Action Status filter or create new custom drug pair(s) for this interaction by clicking on Select Drug Pairs to add to the above VA Custom Interaction.";
        if (noResultsMessage != null && noResultsMessage != "") {
            noDataMessage = noResultsMessage;
        }
        var plugins = {
                /*pagination: {
                    pageSizes: ["10", "25", "50", "100", "All"],    // Array, custom the items per page button
                    description: "30%", // boolean, custom whether or not the description will be displayed
                    sizeSwitch: "260px",    // boolean, custom whether or not the page size switch will be displayed
                    pageStepper: "30em",    // boolean, custom whether or not the page step will be displayed
                    gotoButton: true,   // boolean, custom whether or not the goto page button will be displayed
                    maxPageStep: 7,     // Integer, custom how many page step will be displayed
                    position: "bottom"  // String, custom the position of the pagination bar
                },*/
                indirectSelection : {
                    headerSelector : false,
                    styles : "text-align: center;"
                }
               /* filter: {
                    itemsName: 'drug pairs',
                    closeFilterbarButton: true,
                    ruleCount: 8
                }*/
            };

        // If the DataGrid already exists, just clear any selected rows and
        // replace the store.
        if (grid != null) {
            if (grid.selection != null) {
                grid.selection.clear();
            }
            grid.setStore(gridData);
        } else {
            // DataGrid does not exist.
            var gridLayout = buildLayoutSelectable(servlet, conceptType, conceptName, parentContainer, hasSelect);
            grid = new dojox.grid.EnhancedGrid({
                id : gridId,
                showTitle : true,
                columnReordering : false,
                loadingMessage : "Query In Progress...",
                noDataMessage : noDataMessage,
                onFetchError : gridFetchError,
                selectable : true,
                rowsPerPage: 10000,
                selectionMode : "extended",
                queryOptions: {cache: true},
                canSort : function(index) {
                    return true;
                },
                plugins : plugins
            }, document.createElement('div'));
            dojo.byId(parentContainer).appendChild(grid.domNode);
            grid.setStore(gridData);
            grid.setStructure(gridLayout);

            grid.startup();
        
        }
    } catch (err) {
        var txt = "An error occured while building the dataGrid.  The error is: " + err.message + ".";
        alert(txt);
    }

};




/**
 * Retrieves the FieldMetadata from the server for the tableClass and builds the
 * layout based upon the FieldMetadata for displaying the DataGrid.
 * 
 * @param servlet
 * @param conceptType -
 *            Should be either "VA" or "FDB" depending upon what data will
 *            display in the grid.
 * @param conceptName -
 *            The String representation of the concept in the
 *            gov.va.med.pharmacy.pecs.web.Concept Enum.
 * @param target -
 *            The target for error messages.
 * @param hasSelect -
 *            A boolean entry to indicate if the DTO to be displayed in this
 *            Grid has a 'Select' property.
 * @returns {Array}
 */
function buildLayoutSelectable(servlet, conceptType, conceptName, target, hasSelect) {
    var layout = new Array();
    if (hasSelect) {
        var obj = new Object();
        obj["field"] = 'rtgenid1';
        obj["hidden"] = true;
        obj["dnd"] = false;
        layout.push(obj);

        var obj = new Object();
        obj["field"] = 'rtgenid2';
        obj["hidden"] = true;
        obj["dnd"] = false;
        layout.push(obj);

        /*
         * var obj = new Object(); obj["field"] = "_item.allowableActions";
         * obj["hidden"] = true; obj["dnd"] = false; layout.push(obj);
         */
    }
    // The parameters to pass to xhrGet, the url, how to handle it, and the
    // callbacks.
    var xhrArgs = {
        url : "/ct/" + servlet + "/fieldMetadata/" + conceptType + "/" + conceptName,
        handleAs : "json",
        sync : true,
        load : function(data) {
            dojo.forEach(data, function(entry, i) {
                var obj = new Object();
                obj["field"] = entry['propertyName'];
                obj["name"] = entry['displayName'];
                obj["hidden"] = (entry['includeInQueryResults']) ? false : true;
                obj["width"] = "200px";
                var dataTypeVal = entry['dataType'];
                if ("Date" == dataTypeVal) {
                    obj["formatter"] = formatTimestamp;
                }
                if ("Timestamp" == dataTypeVal) {
                    obj["formatter"] = formatTimestamp;
                }

                layout.push(obj);
            });
        },
        error : function(error) {
            dojo.byId(target).innerHTML = "An unexpected error occurred: " + error.message;
        }
    };

    // Call xhrGet
    dojo.xhrGet(xhrArgs);
    return layout;
}
    