package gov.va.med.pharmacy.pecs.web;

import gov.va.med.pharmacy.pecs.domain.impl.va.CtUsersVa;
import gov.va.med.pharmacy.pecs.domain.impl.va.DrugDrugInteractionVa;
import gov.va.med.pharmacy.pecs.domain.impl.va.UserProfile;
import gov.va.med.pharmacy.pecs.service.MetadataService;
import gov.va.med.pharmacy.pecs.service.UserService;
import gov.va.med.pharmacy.pecs.state.Status;
import gov.va.med.pharmacy.pecs.web.util.InformationMessageSupport;
import gov.va.med.pharmacy.pecs.web.util.WarningMessageSupport;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Set;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;

import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.propertyeditors.CustomDateEditor;
import org.springframework.context.MessageSource;
import org.springframework.stereotype.Controller;
import org.springframework.validation.BindingResult;
import org.springframework.validation.ObjectError;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.annotation.InitBinder;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.SessionAttributes;
import org.springframework.web.servlet.ModelAndView;

/**
 * The Class AbstractController.
 */
@Controller
@SessionAttributes(AbstractController.INFO_MESSAGE)
public abstract class AbstractController implements InitializingBean, InformationMessageSupport, WarningMessageSupport {
    
    /** The date formatter. */
    private final SimpleDateFormat dateFormatter = new SimpleDateFormat("yyyy-MM-dd hh:mm:ss");
    /** The messages. */
    @Resource(name = "messageSource")
    private MessageSource messages;

    /** The user service. */
    @Resource
    private UserService userService;

    /** The Constant SELECT_UNASSIGNED. */
    private static final String SELECT_UNASSIGNED = "UNASSIGNED";

    /** The Constant USERS. */
    private static final String USERS = "users";

    /** The info messages. */
    private List<String> infoMessages = new ArrayList<String>();

    /** The Constant INFO_MESSAGE. */
    public static final String INFO_MESSAGE = "informationMessages";

    /** The Constant WARNING_MESSAGE. */
    public static final String WARNING_MESSAGE = "warningMessages";
    
    /** The Constant CONFIGURABLE_CONCEPTS_ATTRIBUTE. */
    protected static final String CONFIGURABLE_CONCEPTS_ATTRIBUTE = "configurableConcepts";

    /** The warning messages. */
    private List<String> warningMessages = new ArrayList<String>();
    
    /** The metadata service. */
    @Resource
    private MetadataService metadataService;
    
    
    
   

    /**
     * Returns the Configuration Concepts.
     * 
     * @return List<String>
     */
    public final List<String> populateConfigurableConcepts() {
        return getMetadataService().findConcepts();
    }

    /**
     * Gets the user profile.
     * 
     * @param request
     *            the request
     * @return the user profile
     */
    protected final UserProfile getUserProfile(HttpServletRequest request) {
        return (UserProfile) request.getSession().getAttribute(IConstants.SESSION_USER);
    }

    /**
     * Sets the userprofile.
     * 
     * @param request
     *            the request
     * @param profile
     *            the profile
     */
    protected final void setUserprofile(HttpServletRequest request, UserProfile profile) {
        request.getSession().setAttribute(IConstants.SESSION_USER, profile);
    }

    /**
     * Searches for the specified key in a properties file and returns the value associated with the key. No parameter
     * substitution is perform on the value.
     * 
     * @param key
     *            the key
     * @return the string
     */
    public final String findMessage(String key) {
        return findMessage(key, null);
    }

    /**
     * Searches for the specified key in a properties file and returns the value associated with the key. Parameter substitution
     * is performed.
     * 
     * @param key
     *            the key
     * @param values
     *            the values
     * @return the string
     */
    public final String findMessage(String key, Object[] values) {
        String defaultMessage = "The key " + key + " was not found.";
        return messages.getMessage(key, values, defaultMessage, null);

    }

    /**
     * Allows a message to be added for display to the user. The method {@link #AbstractController.findMessage(String)} is first
     * tried which allows ResourceBundle keys to be passed in and the message will be obtained from the ResourceBundle. If a
     * message is not found in a ResourceBundle, the value passed is used as the actual message.
     * 
     * @param message
     *            the message
     */
    @Override
    public final void addInformationMessage(String message) {
        addInformationMessage(message, null);
    }

    /**
     * Allows a message to be added for display to the user. The method
     * 
     * @param message
     *            the message
     * @param values
     *            the values {@link #AbstractController.findMessage(String, Object[])} is first tried which allows
     *            ResourceBundle keys to be passed in and the message will be obtained from the ResourceBundle. The values are
     *            passed to customize the message. If a message is not found in a ResourceBundle, the value passed is used as
     *            the actual message.
     */
    @Override
    public final void addInformationMessage(String message, Object[] values) {
        String msg = findMessage(message, values);
        if (msg == null || msg.endsWith("not found.")) {
            msg = message;
        }
        if (!infoMessages.contains(msg)) {
            infoMessages.add(msg);
        }

    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.web.util.InformationMessageSupport#getInformationMessages()
     */
    @Override
    @ModelAttribute(INFO_MESSAGE)
    public final List<String> getInformationMessages() {
        return infoMessages;
    }

    /**
     * retrieves the appropriate users and puts them in a list to be displayed in a drop down on the page.
     * 
     * @return a list of strings never <code>null</code>
     */
    @ModelAttribute(USERS)
    public final List<String> getAdminUsers() {

        List<CtUsersVa> userList = userService.findApprovers();
        ArrayList<String> orderedList = new ArrayList<String>();
        orderedList.add(SELECT_UNASSIGNED);
        for (CtUsersVa model : userList) {
            orderedList.add(model.getUsername());
        }

        return orderedList;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.web.util.InformationMessageSupport#clearInformationMessages()
     */
    @Override
    public final void clearInformationMessages() {
        infoMessages = new ArrayList<String>();
    }

    /**
     * Allows a Warning message to be added for display to the user. The method {@link #AbstractController.findMessage(String)}
     * is first tried which allows ResourceBundle keys to be passed in and the message will be obtained from the ResourceBundle.
     * If a message is not found in a ResourceBundle, the value passed is used as the actual message.
     * 
     * @param message
     *            the message
     */
    @Override
    public final void addWarningMessage(String message) {
        addWarningMessage(message, null);
    }

    /**
     * Allows a warning message to be added for display to the user. The method
     * 
     * @param message
     *            the message
     * @param values
     *            the values {@link #AbstractController.findMessage(String, Object[])} is first tried which allows
     *            ResourceBundle keys to be passed in and the message will be obtained from the ResourceBundle. The values are
     *            passed to customize the message. If a message is not found in a ResourceBundle, the value passed is used as
     *            the actual message.
     */
    @Override
    public final void addWarningMessage(String message, Object[] values) {
        String msg = findMessage(message, values);
        if (msg == null || msg.endsWith("not found.")) {
            msg = message;
        }
        warningMessages.add(msg);
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.web.util.WarningMessageSupport#getWarningMessages()
     */
    @Override
    @ModelAttribute(WARNING_MESSAGE)
    public final List<String> getWarningMessages() {
        return warningMessages;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.web.util.WarningMessageSupport#clearWarningMessages()
     */
    @Override
    public final void clearWarningMessages() {
        warningMessages = new ArrayList<String>();
    }

    /**
     * Gets the messages.
     * 
     * @return the messages
     */
    public final MessageSource getMessages() {
        return messages;
    }

    /**
     * Sets the messages.
     * 
     * @param messages
     *            the new messages
     */
    public final void setMessages(MessageSource messages) {
        this.messages = messages;
    }

    /**
     * Gets the userService.
     * 
     * @return the userService
     */
    public final UserService getUserService() {
        return userService;
    }

    /**
     * Sets the userService.
     * 
     * @param userService
     *            the userService to set
     */
    public final void setUserService(UserService userService) {
        this.userService = userService;
    }

    /* (non-Javadoc)
     * @see org.springframework.beans.factory.InitializingBean#afterPropertiesSet()
     */
    @Override
    public void afterPropertiesSet() throws Exception {
        if (messages == null) {
            throw new Exception("The property MessageSource in AbstractController has not been set.");
        }
    }

    /**
     * Inits the binder.
     * 
     * @param binder
     *            the binder
     */
    @InitBinder
    public final void initBinder(WebDataBinder binder) {
        CustomDateEditor editor = new CustomDateEditor(dateFormatter, true);
        binder.registerCustomEditor(Date.class, editor);
    }

    /**
     * Creates a list of error strings to be displayed on the page. If no errors occur an empty list will be returned.
     * 
     * @param result
     *            the binding result with errors.
     * @return a list of error strings
     */
    protected final List<String> convertResultToErrors(BindingResult result) {
        List<ObjectError> errors = result.getAllErrors();
        List<String> returnObject = new ArrayList<String>();
        for (ObjectError objectError : errors) {
            returnObject.add(objectError.getDefaultMessage());
        }
        return returnObject;
    }

    /**
     * Validates that the Current Role has permission to Customize the FDB or Va Record.
     * 
     * @param profile
     *            - Role Logged into the system
     * @param actions
     *            the actions
     * @param view
     *            - Model and View being populated
     * @param id
     *            long
     */
    public final void permissionValidation(UserProfile profile, Set<String> actions, ModelAndView view, Long id) {

        if (profile.getRoles().containsKey(UserProfile.ROLE_REQUESTOR)
                || profile.getRoles().containsKey(UserProfile.ROLE_APPROVER)) {
            if (id == 0) {
                view.addObject("allowableActions", Collections.singleton("Customize"));
            } else {
                view.addObject("allowableActions", actions);
            }
        }

    }

    /**
     * Format action.
     * 
     * @param request
     *            the request
     * @return the string
     */
    protected final String formatAction(HttpServletRequest request) {
        return formatAction(request.getParameter("action"));
    }

    /**
     * Format action.
     * 
     * @param action
     *            the action
     * @return the string
     */
    protected final String formatAction(String action) {
        action = action.replace(' ', '_');
        action = action.replace("_For_", "_for_");
        action = action.replace("_As_", "_as_");
        return action;
    }

    /**
     * Check interaction status.
     * 
     * @param drugDrugInteractionVa
     *            the drug drug interaction va
     * @param result
     *            the result
     */
    protected final void checkInteractionStatus(DrugDrugInteractionVa drugDrugInteractionVa, BindingResult result) {

        // make sure the DDI isn't in a DELETED status
        if (Status.STATUS_DELETED.equals(drugDrugInteractionVa.getStatus().getName())
                || Status.STATUS_DELETE_REVIEWED.equals(drugDrugInteractionVa.getStatus().getName())) {
            String message =
                    findMessage("ddi.message.deleted.interaction.drugpair.save.error", new Object[] { drugDrugInteractionVa
                            .getStatus().getName() });
            result.addError(new ObjectError("dpcustomization", message));
        }

    }
    
    /**
     * Gets the metadata service.
     *
     * @return the metadata service
     */
    public final MetadataService getMetadataService() {
        return metadataService;
    }

    /**
     * Sets the metadata service.
     *
     * @param metadataService the new metadata service
     */
    public final void setMetadataService(MetadataService metadataService) {
        this.metadataService = metadataService;
    }
}
