package gov.va.med.pharmacy.pecs.web.auth;

import gov.va.med.pharmacy.pecs.domain.impl.va.UserProfile;
import gov.va.med.pharmacy.pecs.dto.DrugPairDTO;
import gov.va.med.pharmacy.pecs.dto.PecsDTO;
import gov.va.med.pharmacy.pecs.metadata.Concept;
import gov.va.med.pharmacy.pecs.metadata.ConceptMetadata;
import gov.va.med.pharmacy.pecs.query.ConceptQuery;
import gov.va.med.pharmacy.pecs.query.ConditionOperator;
import gov.va.med.pharmacy.pecs.query.Conjunction;
import gov.va.med.pharmacy.pecs.query.FieldCondition;
import gov.va.med.pharmacy.pecs.query.OperatorFactory;
import gov.va.med.pharmacy.pecs.query.QueryCondition;
import gov.va.med.pharmacy.pecs.service.DoseRangeService;
import gov.va.med.pharmacy.pecs.service.DrugDrugInteractionService;
import gov.va.med.pharmacy.pecs.service.DrugPairService;
import gov.va.med.pharmacy.pecs.service.DuplicateTherapyService;
import gov.va.med.pharmacy.pecs.service.MetadataService;
import gov.va.med.pharmacy.pecs.service.MonographService;
import gov.va.med.pharmacy.pecs.state.Status;
import gov.va.med.pharmacy.pecs.util.LabelValue;
import gov.va.med.pharmacy.pecs.web.AbstractController;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;

import org.springframework.beans.factory.InitializingBean;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

/**
 * A Controller class that handles the AJAX retrieval of the Requests and Assigned To for the WorkList/Home page.
 * 
 * @author vhaisaduffg
 * 
 */
@Controller
@RequestMapping(value = "requests")
public final class WorkListRequestsController extends AbstractController implements InitializingBean {

    /** The ddi service. */
    @Resource
    private DrugDrugInteractionService ddiService;

    /** The drug pair service. */
    @Resource
    private DrugPairService drugPairService;

    /** The dose range service. */
    @Resource
    private DoseRangeService doseRangeService;

    /** The dt cat service. */
    @Resource
    private DuplicateTherapyService dtCatService;

    /** The monograph service. */
    @Resource
    private MonographService monographService;

    /** The metadata service. */
    @Resource
    private MetadataService metadataService;

    /**
     * Populates the My Requests section of the home page.
     * 
     * @param request
     *            the HttpServletRequest
     * @param json
     *            the input query
     * @return a list of <code>RequestWorkListModel</code>
     */
    @RequestMapping(value = "requested")
    @ResponseBody
    public List<RequestWorkListModel> populateMyRequests(HttpServletRequest request, @RequestParam("json") String json) {
        List<RequestWorkListModel> myRequests = new ArrayList<RequestWorkListModel>();
        List<RequestWorkListModel> allRequests = new ArrayList<RequestWorkListModel>();

        UserProfile profile = getUserProfile(request);

        fillRequestCounts(Concept.DRUG_DRUG_INTERACTION.name(), ddiService.getStagingDataCounts(profile.getUsername()),
                profile, myRequests, allRequests);
        fillRequestCounts(Concept.PROFESSIONAL_MONOGRAPH.name(), monographService.getStagingDataCounts(profile.getUsername()),
                profile, myRequests, allRequests);
        fillRequestCounts(Concept.DUPLICATE_THERAPY.name(), dtCatService.getStagingDataCounts(profile.getUsername()), profile,
                myRequests, allRequests);
        fillRequestCounts(Concept.DOSE_RANGE.name(), doseRangeService.getStagingDataCounts(profile.getUsername()), profile,
                myRequests, allRequests);

        convertKeyToDisplayName(myRequests);
        return myRequests;
    }

    /**
     * Populate my review.
     * 
     * @param request
     *            the request
     * @param json
     *            the json
     * @return the list
     */
    @RequestMapping(value = "review")
    @ResponseBody
    public List<AssignedWorkListModel> populateMyReview(HttpServletRequest request, @RequestParam("json") String json) {
        List<AssignedWorkListModel> myReviews = new ArrayList<AssignedWorkListModel>();

        UserProfile profile = getUserProfile(request);

        fillReviewCounts(Concept.DRUG_DRUG_INTERACTION.name(), ddiService.getStagingDataCounts(profile.getUsername()), profile,
                myReviews);
        fillReviewCounts(Concept.PROFESSIONAL_MONOGRAPH.name(), monographService.getStagingDataCounts(profile.getUsername()),
                profile, myReviews);
        fillReviewCounts(Concept.DUPLICATE_THERAPY.name(), dtCatService.getStagingDataCounts(profile.getUsername()), profile,
                myReviews);
        fillReviewCounts(Concept.DOSE_RANGE.name(), doseRangeService.getStagingDataCounts(profile.getUsername()), profile,
                myReviews);
        fillReviewCountsDrugPairs(Concept.DRUG_PAIR.name(), drugPairService.getStagingDataCounts(profile.getUsername()),
                profile, myReviews);

        convertConceptKeyToDisplayName(myReviews);
        return myReviews;
    }

    /**
     * Populate my approve.
     * 
     * @param request
     *            the request
     * @param json
     *            the json
     * @return the list
     */
    @RequestMapping(value = "approve")
    @ResponseBody
    public List<AssignedWorkListModel> populateMyApprove(HttpServletRequest request, @RequestParam("json") String json) {
        List<AssignedWorkListModel> myApprove = new ArrayList<AssignedWorkListModel>();

        UserProfile profile = getUserProfile(request);

        fillApproveCounts(Concept.DRUG_DRUG_INTERACTION.name(), ddiService.getStagingDataCounts(profile.getUsername()),
                profile, myApprove);
        fillApproveCounts(Concept.PROFESSIONAL_MONOGRAPH.name(), monographService.getStagingDataCounts(profile.getUsername()),
                profile, myApprove);
        fillApproveCounts(Concept.DUPLICATE_THERAPY.name(), dtCatService.getStagingDataCounts(profile.getUsername()), profile,
                myApprove);
        fillApproveCounts(Concept.DOSE_RANGE.name(), doseRangeService.getStagingDataCounts(profile.getUsername()), profile,
                myApprove);
        fillApproveCountsDrugPairs(Concept.DRUG_PAIR.name(), drugPairService.getStagingDataCounts(profile.getUsername()),
                profile, myApprove);

        convertConceptKeyToDisplayName(myApprove);
        return myApprove;
    }

    /**
     * Populate my delete.
     * 
     * @param request
     *            the request
     * @param json
     *            the json
     * @return the list
     */
    @RequestMapping(value = "delete")
    @ResponseBody
    public List<AssignedWorkListModel> populateMyDelete(HttpServletRequest request, @RequestParam("json") String json) {
        List<AssignedWorkListModel> myDelete = new ArrayList<AssignedWorkListModel>();

        UserProfile profile = getUserProfile(request);

        fillDeleteCounts(Concept.DRUG_DRUG_INTERACTION.name(), ddiService.getStagingDataCounts(profile.getUsername()), profile,
                myDelete);
        fillDeleteCounts(Concept.PROFESSIONAL_MONOGRAPH.name(), monographService.getStagingDataCounts(profile.getUsername()),
                profile, myDelete);
        fillDeleteCounts(Concept.DUPLICATE_THERAPY.name(), dtCatService.getStagingDataCounts(profile.getUsername()), profile,
                myDelete);
        fillDeleteCounts(Concept.DOSE_RANGE.name(), doseRangeService.getStagingDataCounts(profile.getUsername()), profile,
                myDelete);
        fillDeleteCountsDrugPairs(drugPairService.getStagingDataCounts(profile.getUsername()), profile, myDelete);
        convertConceptKeyToDisplayName(myDelete);
        return myDelete;
    }

    /**
     * Populate unassigned.
     * 
     * @param request
     *            the request
     * @param json
     *            the json
     * @return the list
     */
    @RequestMapping(value = "unassigned")
    @ResponseBody
    public List<AssignedWorkListModel> populateUnassigned(HttpServletRequest request, @RequestParam("json") String json) {
        List<AssignedWorkListModel> unassigned = new ArrayList<AssignedWorkListModel>();

        UserProfile profile = getUserProfile(request);

        fillUnassignedCounts(Concept.DRUG_DRUG_INTERACTION.name(), ddiService.getStagingDataCounts(profile.getUsername()),
                profile, unassigned);
        fillUnassignedCounts(Concept.PROFESSIONAL_MONOGRAPH.name(),
                monographService.getStagingDataCounts(profile.getUsername()), profile, unassigned);
        fillUnassignedCounts(Concept.DUPLICATE_THERAPY.name(), dtCatService.getStagingDataCounts(profile.getUsername()),
                profile, unassigned);
        fillUnassignedCounts(Concept.DOSE_RANGE.name(), doseRangeService.getStagingDataCounts(profile.getUsername()), profile,
                unassigned);
        fillUnassignedCountsDrugPairs(drugPairService.getStagingDataCounts(profile.getUsername()), profile, unassigned);
        convertConceptKeyToDisplayName(unassigned);

        return unassigned;
    }

    /**
     * Populate all requests.
     * 
     * @param request
     *            the request
     * @param json
     *            the json
     * @return the list
     */
    @RequestMapping(value = "allRequests")
    @ResponseBody
    public List<RequestWorkListModel> populateAllRequests(HttpServletRequest request, @RequestParam("json") String json) {
        List<RequestWorkListModel> allRequests = new ArrayList<RequestWorkListModel>();

        UserProfile profile = getUserProfile(request);

        fillAllCounts(Concept.DRUG_DRUG_INTERACTION.name(), ddiService.getStagingDataCounts(profile.getUsername()), profile,
                allRequests);
        fillAllCounts(Concept.PROFESSIONAL_MONOGRAPH.name(), monographService.getStagingDataCounts(profile.getUsername()),
                profile, allRequests);
        fillAllCounts(Concept.DUPLICATE_THERAPY.name(), dtCatService.getStagingDataCounts(profile.getUsername()), profile,
                allRequests);
        fillAllCounts(Concept.DOSE_RANGE.name(), doseRangeService.getStagingDataCounts(profile.getUsername()), profile,
                allRequests);

        convertKeyToDisplayName(allRequests);

        return allRequests;
    }

    /**
     * Fill request and todo counts of each module for the user of this session.
     * 
     * @param tableName
     *            Name of the modules for which the counts are being filled
     * @param counts
     *            Map consisting of the various counts for the module
     * @param profile
     *            the profile
     * @param myRequests
     *            the my requests
     * @param allRequests
     *            the all requests
     */
    private void fillRequestCounts(String tableName, Map<String, Integer> counts, UserProfile profile,
            List<RequestWorkListModel> myRequests, List<RequestWorkListModel> allRequests) {
        myRequests.add(getRequestCounts(tableName, counts, profile));
    }

    /**
     * Fill todo counts of each module for the user of this session.
     * 
     * @param tableName
     *            Name of the modules for which the counts are being filled
     * @param counts
     *            Map consisting of the various counts for the module
     * @param profile
     *            the profile
     * @param myReview
     *            the my review
     */
    private void fillReviewCounts(String tableName, Map<String, Integer> counts, UserProfile profile,
            List<AssignedWorkListModel> myReview) {
        myReview.add(getCounts(tableName, counts, profile, "TODOREVIEW", Status.STATUS_REVIEWED));
    }

    /**
     * Fill review counts drug pairs.
     * 
     * @param tableName
     *            the table name
     * @param counts
     *            the counts
     * @param profile
     *            the profile
     * @param myReview
     *            the my review
     */
    private void fillReviewCountsDrugPairs(String tableName, Map<String, Integer> counts, UserProfile profile,
            List<AssignedWorkListModel> myReview) {
        myReview.add(getDPCounts(counts, profile, "TODOREVIEW", Status.STATUS_REVIEWED));
    }

    /**
     * Fill todo counts of each module for the user of this session.
     * 
     * @param tableName
     *            Name of the modules for which the counts are being filled
     * @param counts
     *            Map consisting of the various counts for the module
     * @param profile
     *            the profile
     * @param myApprove
     *            the my approve
     */
    private void fillApproveCounts(String tableName, Map<String, Integer> counts, UserProfile profile,
            List<AssignedWorkListModel> myApprove) {
        myApprove.add(getCounts(tableName, counts, profile, "TODOAPPROVE", Status.STATUS_APPROVED));
    }

    /**
     * Fill approve counts drug pairs.
     * 
     * @param tableName
     *            the table name
     * @param counts
     *            the counts
     * @param profile
     *            the profile
     * @param myApprove
     *            the my approve
     */
    private void fillApproveCountsDrugPairs(String tableName, Map<String, Integer> counts, UserProfile profile,
            List<AssignedWorkListModel> myApprove) {
        myApprove.add(getDPCounts(counts, profile, "TODOAPPROVE", Status.STATUS_APPROVED));
    }

    /**
     * Fill todo counts of each module for the user of this session.
     * 
     * @param tableName
     *            Name of the modules for which the counts are being filled
     * @param counts
     *            Map consisting of the various counts for the module
     * @param profile
     *            the profile
     * @param myDelete
     *            the my delete
     */
    private void fillDeleteCounts(String tableName, Map<String, Integer> counts, UserProfile profile,
            List<AssignedWorkListModel> myDelete) {
        myDelete.add(getCounts(tableName, counts, profile, "TODODELETE", Status.STATUS_DELETED));
    }

    /**
     * Fill delete counts drug pairs.
     * 
     * @param counts
     *            the counts
     * @param profile
     *            the profile
     * @param myDelete
     *            the my delete
     */
    private void fillDeleteCountsDrugPairs(Map<String, Integer> counts, UserProfile profile,
            List<AssignedWorkListModel> myDelete) {
        myDelete.add(getDPCounts(counts, profile, "TODODELETE", Status.STATUS_DELETED));

    }

    /**
     * Fill todo counts of each module for the user of this session.
     * 
     * @param tableName
     *            Name of the modules for which the counts are being filled
     * @param counts
     *            Map consisting of the various counts for the module
     * @param profile
     *            the profile
     * @param unassigned
     *            the unassigned
     */
    private void fillUnassignedCounts(String tableName, Map<String, Integer> counts, UserProfile profile,
            List<AssignedWorkListModel> unassigned) {
        UserProfile unassignedProfile = new UserProfile();
        unassignedProfile.setUsername("UNASSIGNED");
        unassigned.add(getUnassignedCounts(tableName, counts, unassignedProfile, "UNASSIGNED", null));
    }

    /**
     * Fill unassigned counts drug pairs.
     * 
     * @param counts
     *            the counts
     * @param profile
     *            the profile
     * @param unassigned
     *            the unassigned
     */
    private void fillUnassignedCountsDrugPairs(Map<String, Integer> counts, UserProfile profile,
            List<AssignedWorkListModel> unassigned) {
        UserProfile unassignedProfile = new UserProfile();
        unassignedProfile.setUsername("UNASSIGNED");
        unassigned.add(getDPCounts(counts, unassignedProfile, "UNASSIGNED", null));

    }

    /**
     * Fill todo counts of each module for the user of this session.
     * 
     * @param tableName
     *            Name of the modules for which the counts are being filled
     * @param counts
     *            Map consisting of the various counts for the module
     * @param profile
     *            the profile
     * @param allRequests
     *            the all requests
     */
    private void fillAllCounts(String tableName, Map<String, Integer> counts, UserProfile profile,
            List<RequestWorkListModel> allRequests) {
        allRequests.add(getAllCounts(tableName, counts, profile));
    }

    /**
     * Get request counts as string for home page display.
     * 
     * @param conceptName
     *            the concept name
     * @param counts
     *            Map consisting of the various counts
     * @param profile
     *            the profile
     * @return the request counts
     */
    private RequestWorkListModel getRequestCounts(String conceptName, Map<String, Integer> counts, UserProfile profile) {
        RequestWorkListModel summary = new RequestWorkListModel();
        summary.setConcept(conceptName);
        summary.setNewRequests(createRequestLink(conceptName, Status.STATUS_NEW, profile, counts.get("REQNOACTION")));
        summary.setModified(createRequestLink(conceptName, Status.STATUS_MODIFIED, profile, counts.get("REQMODIFIED")));
        summary.setReviewed(createRequestLink(conceptName, Status.STATUS_REVIEWED, profile, counts.get("REQREVIEWED")));
        summary.setApproved(createRequestLink(conceptName, Status.STATUS_APPROVED, profile, counts.get("REQAPPROVED")));
        summary.setRejected(createRequestLink(conceptName, Status.STATUS_REJECTED, profile, counts.get("REQREJECTED")));
        summary.setDeleted(createRequestLink(conceptName, Status.STATUS_DELETED, profile, counts.get("REQDELETED")));
        summary.setTotal(createRequestLink(conceptName, null, profile, counts.get("REQTOTAL")));

        return summary;
    }

    /**
     * Get request counts as string for home page display.
     * 
     * @param conceptName
     *            the concept name
     * @param counts
     *            Map consisting of the various counts
     * @param profile
     *            the profile
     * @param countKey
     *            the count key
     * @param status
     *            the status
     * @return the counts
     */
    private AssignedWorkListModel getCounts(String conceptName, Map<String, Integer> counts, UserProfile profile,
            String countKey, String status) {
        AssignedWorkListModel summary = new AssignedWorkListModel();
        summary.setConcept(conceptName);
        summary.setAssigned(createAssignedToLink(conceptName, status, profile, counts.get(countKey)));
        return summary;
    }

//@formatter:off
    /**
 * Gets the dP counts.
 *
 * @param counts the counts
 * @param profile the profile
 * @param countKey the count key
 * @param status the status
 * @return the dP counts
 */
private AssignedWorkListModel getDPCounts(Map<String, Integer> counts,
            UserProfile profile, String countKey, String status) {
//@formatter:on
        AssignedWorkListModel summary = new AssignedWorkListModel();
        summary.setConcept(Concept.DRUG_PAIR.toString());
        summary.setAssigned(createDrugPairLink(status, profile, counts.get(countKey)));
        return summary;
    }

    /**
     * Get request counts as string for home page display.
     * 
     * @param conceptName
     *            the concept name
     * @param counts
     *            Map consisting of the various counts
     * @param profile
     *            the profile
     * @param countKey
     *            the count key
     * @param status
     *            the status
     * @return the unassigned counts
     */
    private AssignedWorkListModel getUnassignedCounts(String conceptName, Map<String, Integer> counts, UserProfile profile,
            String countKey, String status) {
        AssignedWorkListModel summary = new AssignedWorkListModel();
        summary.setConcept(conceptName);
        summary.setAssigned(createUnassignedToLink(conceptName, status, profile, counts.get(countKey)));
        return summary;
    }

    /**
     * Get request counts as string for home page display.
     * 
     * @param conceptName
     *            the concept name
     * @param counts
     *            Map consisting of the various counts
     * @param profile
     *            the profile
     * @return the all counts
     */
    private RequestWorkListModel getAllCounts(String conceptName, Map<String, Integer> counts, UserProfile profile) {
        RequestWorkListModel summary = new RequestWorkListModel();
        summary.setConcept(conceptName);
        summary.setNewRequests(createRequestLink(conceptName, Status.STATUS_NEW, null, counts.get("TOTNEW")));
        summary.setModified(createRequestLink(conceptName, Status.STATUS_MODIFIED, null, counts.get("TOTMOD")));
        summary.setReviewed(createRequestLink(conceptName, Status.STATUS_REVIEWED, null, counts.get("TOTREV")));
        summary.setApproved(createRequestLink(conceptName, Status.STATUS_APPROVED, null, counts.get("TOTAPP")));
        summary.setRejected(createRequestLink(conceptName, Status.STATUS_REJECTED, null, counts.get("TOTREJ")));
        summary.setDeleted(createRequestLink(conceptName, Status.STATUS_DELETED, null, counts.get("TOTDEL")));
        Integer totalCount =
                counts.get("TOTNEW") + counts.get("TOTMOD") + counts.get("TOTREV") + counts.get("TOTAPP")
                        + counts.get("TOTREJ") + counts.get("TOTDEL");
        summary.setTotal(new LabelValue("" + totalCount, null));
        return summary;
    }

    /**
     * Creates the link.
     * 
     * @param conceptName
     *            the concept name
     * @param count
     *            the count
     * @param queryCriteria
     *            the query criteria
     * @return the label value
     */
    private LabelValue createLink(String conceptName, Integer count, String queryCriteria) {
        LabelValue lv = new LabelValue();
        lv.setLabel(count.toString());
        if (count > 0 && queryCriteria != null && queryCriteria.length() > 0) {
            StringBuffer sb = new StringBuffer();
            sb.append("/ct/pecs/customization/query/VA/");
            sb.append(conceptName);
            sb.append("?json=");
            sb.append(queryCriteria);
            sb.append("&history=false");
            lv.setValue(sb.toString());
        }
        return lv;
    }

    /**
     * Creates the request link.
     * 
     * @param conceptName
     *            the concept name
     * @param status
     *            the status
     * @param profile
     *            the profile
     * @param count
     *            the count
     * @return the label value
     */
    private LabelValue createRequestLink(String conceptName, String status, UserProfile profile, Integer count) {
        return createLink(conceptName, count, buildRequestorQueryCriteria(conceptName, profile, status));
    }

    /**
     * Creates the unassigned to link.
     * 
     * @param conceptName
     *            the concept name
     * @param status
     *            the status
     * @param profile
     *            the profile
     * @param count
     *            the count
     * @return the label value
     */
    private LabelValue createUnassignedToLink(String conceptName, String status, UserProfile profile, Integer count) {
        return createLink(conceptName, count, buildUnassignedQueryCriteria(conceptName));
    }

    /**
     * Creates the assigned to link.
     * 
     * @param conceptName
     *            the concept name
     * @param status
     *            the status
     * @param profile
     *            the profile
     * @param count
     *            the count
     * @return the label value
     */
    private LabelValue createAssignedToLink(String conceptName, String status, UserProfile profile, Integer count) {
        return createLink(conceptName, count, buildAssignedQueryCriteria(conceptName, profile, status));
    }

    /**
     * Builds the requestor query criteria.
     * 
     * @param conceptName
     *            the concept name
     * @param profile
     *            the profile
     * @param status
     *            the status
     * @return the string
     */
    private String buildRequestorQueryCriteria(String conceptName, UserProfile profile, String status) {
        return buildQueryCriteria(conceptName, PecsDTO.REQUESTOR, profile, status);
    }

    /**
     * Builds the assigned query criteria.
     * 
     * @param conceptName
     *            the concept name
     * @param profile
     *            the profile
     * @param status
     *            the status
     * @return the string
     */
    private String buildAssignedQueryCriteria(String conceptName, UserProfile profile, String status) {
        return buildQueryCriteria(conceptName, PecsDTO.ASSIGNED_TO, profile, status);
    }

    /**
     * Creates the drug pair link.
     * 
     * @param status
     *            the status
     * @param profile
     *            the profile
     * @param count
     *            the count
     * @return the label value
     */
    private LabelValue createDrugPairLink(String status, UserProfile profile, Integer count) {
        return createLink(Concept.DRUG_DRUG_INTERACTION.toString(), count, buildQueryCriteriaForDP(profile, status));
    }

    /**
     * Builds the query criteria for dp.
     * 
     * @param profile
     *            the profile
     * @param status
     *            the status
     * @return the string
     */
    private String buildQueryCriteriaForDP(UserProfile profile, String status) {

        List<String> statuses = new ArrayList<String>();
        if (Status.STATUS_REVIEWED.equals(status)) {
            statuses.add(Status.STATUS_NEW);
            statuses.add(Status.STATUS_MODIFIED);
        } else if (Status.STATUS_APPROVED.equals(status)) {
            statuses.add(Status.STATUS_REVIEWED);
        } else if (Status.STATUS_DELETED.equals(status)) {
            statuses.add(Status.STATUS_DELETE_REVIEWED);
        } else {
            statuses.add(status);
        }

        List<Long> interactions = drugPairService.getInteractionsWithWork(profile.getUsername(), statuses, null);
        if (!interactions.isEmpty()) {

            ConceptMetadata meta = ddiService.getMetadata();
            if (meta == null) {
                return null;
            }
            ConceptQuery query = new ConceptQuery(meta);
            QueryCondition cond;

            cond =
                    new FieldCondition(meta.getField(DrugPairDTO.INTERACTIONID), OperatorFactory.getInstance().getOperator(
                            ConditionOperator.IN), interactions);
            query.addCondition(cond, Conjunction.OR);
            return query.toJSON();
        }
        return "";
    }

    /**
     * Builds the query criteria.
     * 
     * @param conceptName
     *            the concept name
     * @param attribute
     *            the attribute
     * @param profile
     *            the profile
     * @param status
     *            the status
     * @return the string
     */
    private String buildQueryCriteria(String conceptName, String attribute, UserProfile profile, String status) {
        ConceptMetadata meta = findConceptMetadata(conceptName);
        if (meta == null) {
            return null;
        }
        ConceptQuery query = new ConceptQuery(meta);
        QueryCondition cond;

        if (profile != null) {
            cond =
                    new FieldCondition(meta.getField(attribute), OperatorFactory.getInstance()
                            .getOperator(ConditionOperator.EQ), profile.getUsername());
            query.addCondition(cond, Conjunction.AND);
        }

        if (status != null) {
            if (attribute.equals(PecsDTO.REQUESTOR) && status.equals(Status.STATUS_REVIEWED)) {
                List<String> statusValues = new ArrayList<String>();
                statusValues.add(Status.STATUS_REVIEWED);
                statusValues.add(Status.STATUS_DELETE_REVIEWED);
                cond =
                        new FieldCondition(meta.getField(PecsDTO.ACTION_STATUS), OperatorFactory.getInstance().getOperator(
                                ConditionOperator.IN), statusValues);
            } else if (attribute.equals(PecsDTO.ASSIGNED_TO) && status.equals(Status.STATUS_REVIEWED)) {
                List<String> statusValues = new ArrayList<String>();
                statusValues.add(Status.STATUS_NEW);
                statusValues.add(Status.STATUS_MODIFIED);
                cond =
                        new FieldCondition(meta.getField(PecsDTO.ACTION_STATUS), OperatorFactory.getInstance().getOperator(
                                ConditionOperator.IN), statusValues);
            } else if (attribute.equals(PecsDTO.ASSIGNED_TO) && status.equals(Status.STATUS_APPROVED)) {
                cond =
                        new FieldCondition(meta.getField(PecsDTO.ACTION_BY), OperatorFactory.getInstance().getOperator(
                                ConditionOperator.NEQ), profile.getUsername());
                query.addCondition(cond, Conjunction.AND);
                cond =
                        new FieldCondition(meta.getField(PecsDTO.ACTION_STATUS), OperatorFactory.getInstance().getOperator(
                                ConditionOperator.EQ), Status.STATUS_REVIEWED);
            } else if (attribute.equals(PecsDTO.ASSIGNED_TO) && status.equals(Status.STATUS_DELETED)) {
                cond =
                        new FieldCondition(meta.getField(PecsDTO.ACTION_BY), OperatorFactory.getInstance().getOperator(
                                ConditionOperator.NEQ), profile.getUsername());
                query.addCondition(cond, Conjunction.AND);
                cond =
                        new FieldCondition(meta.getField(PecsDTO.ACTION_STATUS), OperatorFactory.getInstance().getOperator(
                                ConditionOperator.EQ), Status.STATUS_DELETE_REVIEWED);
            } else {
                cond =
                        new FieldCondition(meta.getField(PecsDTO.ACTION_STATUS), OperatorFactory.getInstance().getOperator(
                                ConditionOperator.EQ), status);
            }
            query.addCondition(cond, Conjunction.AND);
        }

        return query.toJSON();
    }

    /**
     * Builds the unassigned query criteria.
     * 
     * @param conceptName
     *            the concept name
     * @return the string
     */
    private String buildUnassignedQueryCriteria(String conceptName) {
        ConceptMetadata meta = findConceptMetadata(conceptName);
        if (meta == null) {
            return null;
        }
        ConceptQuery query = new ConceptQuery(meta);

        QueryCondition cond =
                new FieldCondition(meta.getField(PecsDTO.ASSIGNED_TO), OperatorFactory.getInstance().getOperator(
                        ConditionOperator.EQ), "UNASSIGNED");
        query.addCondition(cond, Conjunction.AND);

        cond =
                new FieldCondition(meta.getField(PecsDTO.ACTION_STATUS), OperatorFactory.getInstance().getOperator(
                        ConditionOperator.NEQ), Status.STATUS_APPROVED);
        query.addCondition(cond, Conjunction.AND);

        cond =
                new FieldCondition(meta.getField(PecsDTO.ACTION_STATUS), OperatorFactory.getInstance().getOperator(
                        ConditionOperator.NEQ), Status.STATUS_DELETED);
        query.addCondition(cond, Conjunction.AND);

        cond =
                new FieldCondition(meta.getField(PecsDTO.ACTION_STATUS), OperatorFactory.getInstance().getOperator(
                        ConditionOperator.NEQ), Status.STATUS_REJECTED);
        query.addCondition(cond, Conjunction.AND);

        return query.toJSON();
    }

    /**
     * Find concept metadata.
     * 
     * @param conceptName
     *            the concept name
     * @return the concept metadata
     */
    private ConceptMetadata findConceptMetadata(String conceptName) {
        return metadataService.getMetadata(Concept.fromString(conceptName).getConceptClass());
    }

    /**
     * Convert concept key to display name.
     * 
     * @param list
     *            the list
     */
    private void convertConceptKeyToDisplayName(List<AssignedWorkListModel> list) {
        for (AssignedWorkListModel item : list) {
            String conceptName = item.getConcept();
            if (Concept.DRUG_DRUG_INTERACTION.name().equals(conceptName)) {
                item.setConcept("Drug-Drug Interaction");
            } else if (Concept.DOSE_RANGE.name().equals(conceptName)) {
                item.setConcept("Dose Range");
            } else if (Concept.PROFESSIONAL_MONOGRAPH.name().equals(conceptName)) {
                item.setConcept("Professional Monograph");
            } else if (Concept.DUPLICATE_THERAPY.name().equals(conceptName)) {
                item.setConcept("Duplicate Therapy");
            } else if (Concept.DOSE_RANGE.name().equals(conceptName)) {
                item.setConcept("Duplicate Therapy");
            } else if (Concept.DRUG_PAIR.name().equals(conceptName)) {
                item.setConcept("Approved Drug Drug Interactions With Pending Drug Pairs");
            }
        }
    }

    /**
     * Convert key to display name.
     * 
     * @param list
     *            the list
     */
    private void convertKeyToDisplayName(List<RequestWorkListModel> list) {
        for (RequestWorkListModel item : list) {
            String conceptName = item.getConcept();
            if (Concept.DRUG_DRUG_INTERACTION.name().equals(conceptName)) {
                item.setConcept("Drug-Drug Interaction");
            } else if (Concept.DOSE_RANGE.name().equals(conceptName)) {
                item.setConcept("Dose Range");
            } else if (Concept.PROFESSIONAL_MONOGRAPH.name().equals(conceptName)) {
                item.setConcept("Professional Monograph");
            } else if (Concept.DUPLICATE_THERAPY.name().equals(conceptName)) {
                item.setConcept("Duplicate Therapy");
            } else if (Concept.DRUG_PAIR.name().equals(conceptName)) {
                item.setConcept("Approved Drug Drug Interactions With Pending Drug Pairs");
            }
        }
    }

    /**
     * Sets the ddi service.
     * 
     * @param ddiService
     *            the new ddi service
     */
    public void setDdiService(DrugDrugInteractionService ddiService) {
        this.ddiService = ddiService;
    }

    /**
     * Gets the drug pair service.
     * 
     * @return the drugPairService
     */
    public DrugPairService getDrugPairService() {
        return drugPairService;
    }

    /**
     * Sets the drug pair service.
     * 
     * @param drugPairService
     *            the drugPairService to set
     */
    public void setDrugPairService(DrugPairService drugPairService) {
        this.drugPairService = drugPairService;
    }

    /**
     * Sets the dose range service.
     * 
     * @param doseRangeService
     *            the new dose range service
     */
    public void setDoseRangeService(DoseRangeService doseRangeService) {
        this.doseRangeService = doseRangeService;
    }

    /**
     * Sets the dt cat service.
     * 
     * @param dtCatService
     *            the new dt cat service
     */
    public void setDtCatService(DuplicateTherapyService dtCatService) {
        this.dtCatService = dtCatService;
    }

    /**
     * Sets the monograph service.
     * 
     * @param monographService
     *            the new monograph service
     */
    public void setMonographService(MonographService monographService) {
        this.monographService = monographService;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.web.AbstractController#afterPropertiesSet()
     */
    @Override
    public void afterPropertiesSet() throws Exception {
        if (ddiService == null) {
            throw new Exception("The DrugDrugInteractionService has not been injected in to WorkListRequestsController.");
        }
        if (doseRangeService == null) {
            throw new Exception("The DoseRangeService has not been injected in to WorkListRequestsController.");
        }
        if (dtCatService == null) {
            throw new Exception("The DtCatTherapyService has not been injected in to WorkListRequestsController.");
        }
        if (monographService == null) {
            throw new Exception("The MonographService has not been injected in to WorkListRequestsController.");
        }
        if (metadataService == null) {
            throw new Exception("The MetadataService has not been injected in to WorkListRequestsController.");
        }
    }
}
