package gov.va.med.pharmacy.pecs.web.customization;

import gov.va.med.pharmacy.pecs.domain.impl.va.SavedQueryVa;
import gov.va.med.pharmacy.pecs.dto.PecsDTO;
import gov.va.med.pharmacy.pecs.exception.CustomizationServiceException;
import gov.va.med.pharmacy.pecs.metadata.Concept;
import gov.va.med.pharmacy.pecs.metadata.ConceptField;
import gov.va.med.pharmacy.pecs.metadata.ConceptMetadata;
import gov.va.med.pharmacy.pecs.query.ConceptQuery;
import gov.va.med.pharmacy.pecs.query.ConceptQueryFactory;
import gov.va.med.pharmacy.pecs.query.DoJoGridSuport;
import gov.va.med.pharmacy.pecs.query.Page;
import gov.va.med.pharmacy.pecs.service.DoseRangeService;
import gov.va.med.pharmacy.pecs.service.DrugDrugInteractionService;
import gov.va.med.pharmacy.pecs.service.DrugPairService;
import gov.va.med.pharmacy.pecs.service.DuplicateTherapyService;
import gov.va.med.pharmacy.pecs.service.MetadataService;
import gov.va.med.pharmacy.pecs.service.MonographService;
import gov.va.med.pharmacy.pecs.service.PecsService;
import gov.va.med.pharmacy.pecs.service.SavedQueryService;
import gov.va.med.pharmacy.pecs.web.AbstractController;
import gov.va.med.pharmacy.pecs.web.RecordType;

import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.stereotype.Controller;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.SessionAttributes;
import org.springframework.web.servlet.ModelAndView;

/**
 * The Controller class used to process the User Settings page in PECS.
 */
@SessionAttributes({ "queryCriteria" })
@Controller
@RequestMapping(value = "/customization")
public class CustomizationQueryController extends AbstractController implements InitializingBean {

    /** The logger. */
    private final Log logger = LogFactory.getLog(CustomizationQueryController.class);

    /** The drug drug interaction service. */
    @Resource
    private DrugDrugInteractionService drugDrugInteractionService;

    /** The drug pair service. */
    @Resource
    private DrugPairService drugPairService;

    /** The dt cat service. */
    @Resource
    private DuplicateTherapyService dtCatService;

    /** The monograph service. */
    @Resource
    private MonographService monographService;

    /** The metadata service. */
    @Resource
    private MetadataService metadataService;

    /** The dose range service. */
    @Resource
    private DoseRangeService doseRangeService;

    /** The saved query service. */
    @Resource
    private SavedQueryService savedQueryService;

    /** The factory. */
    @Resource(name = "jsonConceptQueryFactory")
    private ConceptQueryFactory factory;

    /** The service map. */
    private Map<String, PecsService<?>> serviceMap;

    /** The Constant TILES_VIEW. */
    private final static String TILES_VIEW = "pecs.customizationquery";

    /** The Constant TILES_QUERYBUILDER_VIEW. */
    private final static String TILES_QUERYBUILDER_VIEW = "pecs.customizationquery.builder";

    /** The Constant SAVED_QUERY_VIEW. */
    private static final String SAVED_QUERY_VIEW = "pecs.customizationquery.manager";

    /**
     * Handles the initial display of the Advanced Customization/Query page.
     * 
     * @return the Tiles view to display.
     **/
    @RequestMapping(method = RequestMethod.GET)
    public String get() {
        return TILES_VIEW;
    }

    /**
     * Returns a CustomizationQueryModel object.
     * 
     * @return the customization query model
     */
    @ModelAttribute("customizationQueryModel")
    public CustomizationQueryModel findCustomizationQueryModel() {
        CustomizationQueryModel customizationQueryModel = new CustomizationQueryModel();
        return customizationQueryModel;
    }

    /**
     * Populate form.
     * 
     * @return the concept record form
     */
    @ModelAttribute(value = "queryCriteria")
    public ConceptRecordForm populateForm() {
        return new ConceptRecordForm();
    }

    /**
     * Update config clumns.
     * 
     * @param conceptRecordForm
     *            the concept record form
     * @param result
     *            the result
     * @return the model and view
     */
    @RequestMapping(value = "/criteria")
    public ModelAndView updateConfigClumns(@ModelAttribute(value = "queryCriteria") ConceptRecordForm conceptRecordForm,
            BindingResult result) {

        String concept = conceptRecordForm.getConceptSelection();
        ModelAndView modelAndView = new ModelAndView(TILES_QUERYBUILDER_VIEW);
        RecordType type = RecordType.fromString(conceptRecordForm.getRecordSelection());
        Boolean custom = null;
        switch (type) {
        case FDB:
            custom = false;
            break;
        case VA:
            custom = true;
            break;
        }
        List<ConceptField> columns = getConfigColumns(Concept.fromString(concept).getConceptClass(), custom);

        if (columns == null || columns.size() == 0) {
            final String msg = "No metadata found for " + concept + ".";
            logger.error(msg);
            throw new CustomizationServiceException(msg);
        }

        modelAndView.addObject("columns", columns);

        return modelAndView;

    }

    /**
     * Query from url.
     * 
     * @param recordType
     *            the record type
     * @param concept
     *            the concept
     * @param json
     *            the json
     * @param history
     *            the history
     * @param response
     *            the response
     * @return the string
     */
    @RequestMapping(value = "/query/{recordType}/{concept}", method = RequestMethod.GET)
    public String queryFromUrl(@PathVariable(value = "recordType") String recordType,
            @PathVariable(value = "concept") String concept, @RequestParam(value = "json") String json, @RequestParam(
                    value = "history") boolean history, HttpServletResponse response) {

        return TILES_VIEW;
    }

    /**
     * Submit query.
     * 
     * @param recordType
     *            the record type
     * @param concept
     *            the concept
     * @param json
     *            the json
     * @param history
     *            the history
     * @param response
     *            the response
     * @param request
     *            the request
     * @return the list
     */
    @SuppressWarnings("unchecked")
    @RequestMapping(
            value = "/query/{recordType}/{concept}", method = RequestMethod.GET, headers = "Content-Type=application/json")
    @ResponseBody
    public List<PecsDTO> submitQuery(@PathVariable(value = "recordType") String recordType,
            @PathVariable(value = "concept") String concept, @RequestParam(value = "json") String json, @RequestParam(
                    value = "history") boolean history, HttpServletResponse response, HttpServletRequest request) {
        if (logger.is   ugEnabled()) {
            logger.debug("Query for concept " + concept + " (" + recordType + "): " + json);
        }
        boolean va = false;
        if ("VA".equals(recordType)) {
            va = true;
        } else if ("FDB".equals(recordType)) {
            va = false;
        } else {
            throw new IllegalArgumentException("Invalid record type: " + recordType);
        }
        HttpSession session = request.getSession();
        ConceptRecordForm conceptRecordForm = (ConceptRecordForm) session.getAttribute("queryCriteria");
        conceptRecordForm.setConceptSelection(concept);
        conceptRecordForm.setRecordSelection(recordType);

        Page page;        
        DoJoGridSuport support = new DoJoGridSuport(json);
        if (support.isParsed()) {
            PecsService<?> service = getService(concept);
            ConceptQuery query = factory.createQuery(service.getMetadata(), support.getJson());
            query.setIncludeHistory(history);
            query.setPaged(true);
            if (support.hasSorting()) {
                ConceptField field = service.getMetadata().getFieldByProperty(support.getSortColumn());
                query.setSortFields(Arrays.asList(new ConceptField[] { field }));
                query.setSortOrder(support.sortAsc());
            } else {
                ConceptField field = service.getMetadata().getField(PecsDTO.ACTION_DATE);
                query.setSortFields(Arrays.asList(new ConceptField[] { field }));
                query.setSortOrder(false);
            }                       
            
            if (va) {
                
                page = service.getStagingData(query, support.getPageNumber());
            } else {

                page = service.getFdbData(query, support.getPageNumber());
            }                        
            
            //DOJO standard is to set Content-Range with "items x-y/totcount", but dropping the "items " makes both 
            //IE and FF behave properly. 
            response.setHeader("Content-Range", support.getStartRow() + "-" + support.getEndRow() + "/" 
                    + page.getQuerySize());

            return page.getResults();            
        }

        return Collections.emptyList();               
    }

    /**
     * Load query.
     * 
     * @param queryId
     *            the query id
     * @return the saved query va
     */
    @RequestMapping(value = "/query/load/{queryId}")
    @ResponseBody
    public SavedQueryVa loadQuery(@PathVariable(value = "queryId") Long queryId) {
        return savedQueryService.findSavedQueryById(queryId);
    }

    /**
     * Save query.
     * 
     * @param recordType
     *            the record type
     * @param concept
     *            the concept
     * @param query
     *            the query
     * @param queryName
     *            the query name
     * @param request
     *            the request
     */
    @RequestMapping(value = "/query/save/{recordType}/{concept}")
    @ResponseBody
    public void saveQuery(@PathVariable(value = "recordType") String recordType,
            @PathVariable(value = "concept") String concept, @RequestParam("query") String query,
            @RequestParam("queryName") String queryName, HttpServletRequest request) {
        savedQueryService.save(getUserProfile(request),
                getTableName(Concept.fromString(concept), RecordType.fromString(recordType)), query, queryName);
    }

    /**
     * Delete query.
     * 
     * @param queryId
     *            the query id
     * @param request
     *            the request
     */
    @RequestMapping(value = "/query/delete/{queryId}")
    @ResponseBody
    public void deleteQuery(@PathVariable("queryId") Long queryId, HttpServletRequest request) {
        savedQueryService.deleteSavedQuery(getUserProfile(request), queryId);
    }

    /**
     * Gets the config columns.
     * 
     * @param clazz
     *            the clazz
     * @param custom
     *            the custom
     * @return the config columns
     */
    private List<ConceptField> getConfigColumns(Class<?> clazz, Boolean custom) {
        ConceptMetadata metadata = metadataService.getMetadata(clazz);
        List<ConceptField> fieldList = custom == null ? metadata.getQueryFields() : metadata.getQueryFields(custom);

        return fieldList;
    }

    /**
     * Gets the my queries.
     * 
     * @param request
     *            the request
     * @param recordType
     *            the record type
     * @param tableName
     *            the table name
     * @param command
     *            the command
     * @return the my queries
     */
    @RequestMapping(value = "/myQueries/{recordType}/{tableName}")
    public ModelAndView getMyQueries(HttpServletRequest request, @PathVariable(value = "recordType") String recordType,
            @PathVariable(value = "tableName") String tableName, @ModelAttribute(value = "command") QueryForm command) {
        String userName = getUserProfile(request).getUsername();
        List<SavedQueryVa> savedQueriesVas =
                getSavedQueryService().findSavedQueryByUserIdAndTable(userName,
                        getTableName(Concept.fromString(tableName), RecordType.fromString(recordType)));
        command.setRecordType(recordType);

        ModelAndView mav = new ModelAndView(SAVED_QUERY_VIEW);
        mav.addObject("savedQueries", savedQueriesVas);
        mav.addObject("savedQueryType", "mine");
        return mav;
    }

    /**
     * Gets the others queries.
     * 
     * @param request
     *            the request
     * @param recordType
     *            the record type
     * @param tableName
     *            the table name
     * @param command
     *            the command
     * @return the others queries
     */
    @RequestMapping(value = "/othersQueries/{recordType}/{tableName}")
    public ModelAndView getOthersQueries(HttpServletRequest request, @PathVariable(value = "recordType") String recordType,
            @PathVariable(value = "tableName") String tableName, @ModelAttribute(value = "command") QueryForm command) {
        String userName = getUserProfile(request).getUsername();
        List<SavedQueryVa> otherSavedSearches =
                getSavedQueryService().findSavedQueryForOthersByTable(
                        getTableName(Concept.fromString(tableName), RecordType.fromString(recordType)), userName);

        ModelAndView mav = new ModelAndView(SAVED_QUERY_VIEW);
        mav.addObject("savedQueries", otherSavedSearches);
        mav.addObject("savedQueryType", "others");
        return mav;
    }

    /**
     * Gets the saved query.
     * 
     * @param id
     *            the id
     * @return the saved query
     */
    @RequestMapping(value = "/savedQuery", method = RequestMethod.POST)
    public String getSavedQuery(@PathVariable(value = "id") long id) {
        SavedQueryVa va = savedQueryService.findSavedQueryById(id);

        return va.getQuerySql();
    }

    /**
     * Gets the table name.
     * 
     * @param concept
     *            the concept
     * @param recordType
     *            the record type
     * @return the table name
     */
    private String getTableName(Concept concept, RecordType recordType) {
        if (concept == Concept.DRUG_DRUG_INTERACTION) {
            if (recordType == RecordType.FDB) {
                return "FDB_DDIMINTERACTION";
            } else {
                return "PECS_DDI_VERSION_VA";
            }
        } else if (concept == Concept.DRUG_PAIR) {
            if (recordType == RecordType.FDB) {
                return "FDB_DDIMDRUGLINK";
            } else {
                return "PECS_DRUG_PAIR_VERSION_VA";
            }
        } else if (concept == Concept.DOSE_RANGE) {
            if (recordType == RecordType.FDB) {
                return "FDB_DOSING";
            } else {
                return "PECS_DOSE_RANGE_VERSION_VA";
            }
        } else if (concept == Concept.DUPLICATE_THERAPY) {
            if (recordType == RecordType.FDB) {
                return "FDB_DUPLTHERAPY";
            } else {
                return "PECS_DUPL_THERAPY_VERSION_VA";
            }
        } else if (concept == Concept.PROFESSIONAL_MONOGRAPH) {
            if (recordType == RecordType.FDB) {
                return "FDB_DDIM_MONOGRAPH";
            } else {
                return "PECS_MONOGRAPH_VA";
            }
        }
        return null;
    }

    /**
     * Gets the service.
     * 
     * @param concept
     *            the concept
     * @return the service
     */
    private PecsService<?> getService(String concept) {
        return serviceMap.get(concept);
    }

    // Spring Injector methods
    /**
     * Gets the drug drug interaction service.
     * 
     * @return the drug drug interaction service
     */
    public DrugDrugInteractionService getDrugDrugInteractionService() {
        return drugDrugInteractionService;
    }

    /**
     * Sets the drug drug interaction service.
     * 
     * @param drugDrugInteractionService
     *            the new drug drug interaction service
     */
    public void setDrugDrugInteractionService(DrugDrugInteractionService drugDrugInteractionService) {
        this.drugDrugInteractionService = drugDrugInteractionService;
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.web.AbstractController#afterPropertiesSet()
     */
    @Override
    public void afterPropertiesSet() throws Exception {
        super.afterPropertiesSet();
        if (drugDrugInteractionService == null) {
            throw new Exception("The property drugDrugInteractionService in CustomizationQueryController has not been set.");
        }
        if (drugPairService == null) {
            throw new Exception("The property drugPairService in CustomizationQueryController has not been set.");
        }
        if (getMetadataService() == null) {
            throw new Exception("The property configService in CustomizationQueryController has not been set.");
        }
        serviceMap = new HashMap<String, PecsService<?>>();
        serviceMap.put(Concept.DRUG_DRUG_INTERACTION.toString(), drugDrugInteractionService);
        serviceMap.put(Concept.DRUG_PAIR.toString(), drugPairService);
        serviceMap.put(Concept.DUPLICATE_THERAPY.toString(), dtCatService);
        serviceMap.put(Concept.PROFESSIONAL_MONOGRAPH.toString(), monographService);
        serviceMap.put(Concept.DOSE_RANGE.toString(), getDoseRangeService());
    }

  

    /**
     * Sets the saved query service.
     * 
     * @param savedQueryService
     *            the new saved query service
     */
    public void setSavedQueryService(SavedQueryService savedQueryService) {
        this.savedQueryService = savedQueryService;
    }

    /**
     * Gets the saved query service.
     * 
     * @return the saved query service
     */
    public SavedQueryService getSavedQueryService() {
        return savedQueryService;
    }

    /**
     * Sets the drug pair service.
     * 
     * @param service
     *            the new drug pair service
     */
    public void setDrugPairService(DrugPairService service) {
        this.drugPairService = service;
    }

    /**
     * Gets the drug pair service.
     * 
     * @return the drug pair service
     */
    public DrugPairService getDrugPairService() {
        return drugPairService;
    }

    /**
     * Sets the concept query factory.
     * 
     * @param factory
     *            the new concept query factory
     */
    public void setConceptQueryFactory(ConceptQueryFactory factory) {
        this.factory = factory;
    }

    /**
     * Gets the concept query factory.
     * 
     * @return the concept query factory
     */
    public ConceptQueryFactory getConceptQueryFactory() {
        return factory;
    }

    /**
     * Sets the dt cat service1.
     * 
     * @param newDtCatService
     *            the new dt cat service1
     */
    public void setDtCatService1(DuplicateTherapyService newDtCatService) {
        this.dtCatService = newDtCatService;
    }

    /**
     * Gets the dt cat service.
     * 
     * @return the dt cat service
     */
    public DuplicateTherapyService getDtCatService() {
        return dtCatService;
    }

    /**
     * Sets the monograph service.
     * 
     * @param monographService
     *            the new monograph service
     */
    public void setMonographService(MonographService monographService) {
        this.monographService = monographService;
    }

    /**
     * Gets the monograph service.
     * 
     * @return the monograph service
     */
    public MonographService getMonographService() {
        return monographService;
    }

    /**
     * Sets the dose range service.
     * 
     * @param doseRangeService
     *            the new dose range service
     */
    public void setDoseRangeService(DoseRangeService doseRangeService) {
        this.doseRangeService = doseRangeService;
    }

    /**
     * Gets the dose range service.
     * 
     * @return the dose range service
     */
    public DoseRangeService getDoseRangeService() {
        return doseRangeService;
    }

}
