package gov.va.med.pharmacy.pecs.web.detail;

import gov.va.med.pharmacy.pecs.domain.UserComment;
import gov.va.med.pharmacy.pecs.domain.impl.va.DoseRangeKey;
import gov.va.med.pharmacy.pecs.domain.impl.va.DoseRangeUserComment;
import gov.va.med.pharmacy.pecs.domain.impl.va.DrugDrugInteractionUserComment;
import gov.va.med.pharmacy.pecs.domain.impl.va.DuplicateTherapyUserComment;
import gov.va.med.pharmacy.pecs.domain.impl.va.MonographUserComment;
import gov.va.med.pharmacy.pecs.metadata.Concept;
import gov.va.med.pharmacy.pecs.service.DoseRangeService;
import gov.va.med.pharmacy.pecs.service.DrugDrugInteractionService;
import gov.va.med.pharmacy.pecs.service.DuplicateTherapyService;
import gov.va.med.pharmacy.pecs.service.MonographService;
import gov.va.med.pharmacy.pecs.web.AbstractController;
import gov.va.med.pharmacy.pecs.web.detail.doserange.DoseRangeCheckRecordKeyHelper;
import gov.va.med.pharmacy.pecs.web.detail.drugdrug.UserCommentForm;

import java.util.Calendar;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;

import org.springframework.stereotype.Controller;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.servlet.ModelAndView;

/**
 * the controller that handles comments for the FDB records.
 * 
 * @author VHAISACARUSR
 * 
 */
@Controller
public final class FDBCommentController extends AbstractController {
    private static final String COMMENT_VIEW = "pecs.detail.comment";

    @Resource
    private DrugDrugInteractionService drugDrugInteractionService;

    @Resource
    private DuplicateTherapyService dtCatTherapyService;

    @Resource
    private MonographService monographService;

    @Resource
    private DoseRangeService doseRangeService;

    /**
     * Gets and populates the JSP and form for the FDB comments.
     * 
     * @param fdbId
     *            the id passed in from the page
     * @param commentForm
     *            the form to populate and display
     * @param bindingResult
     *            errors if the exist
     * 
     * @param concept
     *            concept
     * @return the JSP fragment with the form
     */
    @RequestMapping(value = "/comment/{concept}/{fdbId:.+}", method = RequestMethod.GET)
    public ModelAndView displayComment(@PathVariable("fdbId") Object fdbId, @PathVariable("concept") String concept,
            @ModelAttribute("commentForm") UserCommentForm commentForm, BindingResult bindingResult) {
        commentForm.setFdbId(fdbId);
        commentForm.setConcept(concept);
        return new ModelAndView(COMMENT_VIEW);
    }

    /**
     * Saves the FDB comment.
     * 
     * @param request
     *            the HTTP request
     * @param commentForm
     *            the form to populate and display
     * @param bindingResult
     *            errors if the exist
     * @return errors if there are any
     */
    @RequestMapping(value = "/comment", method = RequestMethod.POST)
    @ResponseBody
    public AjaxMessageResponse saveComment(HttpServletRequest request,
            @ModelAttribute("commentForm") UserCommentForm commentForm, BindingResult bindingResult) {

        saveComment(commentForm, getUserProfile(request).getUsername(), bindingResult);
        AjaxMessageResponse response = new AjaxMessageResponse();
        if (bindingResult.hasErrors()) {
            bindingResult.getAllErrors();
            response.setErrorMessages(convertResultToErrors(bindingResult));

        } else {
            response.setSuccessMessage("Succesfully added comment");
        }
        return response;

    }

    /**
     * @param userName
     *            the user who is saving the comment
     * @param commentForm
     *            the form received from the JSP
     * @param userComment
     *            the DTO to send back
     */
    private void populateFDBComment(String userName, UserCommentForm commentForm, UserComment userComment) {

        userComment.setComment(commentForm.getComment());
        userComment.setUserName(userName);
        userComment.setCommentDate(Calendar.getInstance().getTime());

    }

    /**
     * Save comment.
     * 
     * @param commentForm
     *            the form returned from the JSP
     * @param userName
     *            the name of the user saving the FDB comment
     * @param bindingResult
     *            the errors if there are any
     */
    private void saveComment(UserCommentForm commentForm, String userName, BindingResult bindingResult) {
        String concept = commentForm.getConcept();
        UserComment userComment = null;
        switch (Concept.fromString(concept)) {
        case DRUG_DRUG_INTERACTION:
            userComment = new DrugDrugInteractionUserComment();
            populateFDBComment(userName, commentForm, userComment);
            getDrugDrugInteractionService().createUserComment(commentForm.getFdbId(), userComment, bindingResult);
            break;
        case DUPLICATE_THERAPY:
            userComment = new DuplicateTherapyUserComment();
            populateFDBComment(userName, commentForm, userComment);
            getDtCatTherapyService().createUserComment(commentForm.getFdbId(), userComment, bindingResult);
            break;
        case PROFESSIONAL_MONOGRAPH:
            userComment = new MonographUserComment();
            populateFDBComment(userName, commentForm, userComment);
            getMonographService().createUserComment(commentForm.getFdbId(), userComment, bindingResult);
            break;
        case DOSE_RANGE:
            userComment = new DoseRangeUserComment();
            populateFDBComment(userName, commentForm, userComment);
            DoseRangeKey doseRangeKey = new DoseRangeCheckRecordKeyHelper((String) commentForm.getFdbId()).createDoseRangeKey();
            getDoseRangeService().createUserComment(doseRangeKey, userComment, bindingResult);
            break;
        default:
            break;
        }
    }

    /**
     * Gets the drug drug interaction service.
     * 
     * @return the drugDrugInteractionService
     */
    public DrugDrugInteractionService getDrugDrugInteractionService() {
        return drugDrugInteractionService;
    }

    /**
     * Sets the drug drug interaction service.
     * 
     * @param drugDrugInteractionService
     *            the drugDrugInteractionService to set
     */
    public void setDrugDrugInteractionService(DrugDrugInteractionService drugDrugInteractionService) {
        this.drugDrugInteractionService = drugDrugInteractionService;
    }

    /**
     * Gets the dtCatTherapyService.
     * 
     * @return the dtCatTherapyService
     */
    public DuplicateTherapyService getDtCatTherapyService() {
        return dtCatTherapyService;
    }

    /**
     * Sets the dtCatTherapyService.
     * 
     * @param dtCatTherapyService
     *            the dtCatTherapyService to set
     */
    public void setDtCatTherapyService(DuplicateTherapyService dtCatTherapyService) {
        this.dtCatTherapyService = dtCatTherapyService;
    }

    /**
     * Gets the monographService.
     * 
     * @return the monographService
     */
    public MonographService getMonographService() {
        return monographService;
    }

    /**
     * Sets the monographService.
     * 
     * @param monographService
     *            the monographService to set
     */
    public void setMonographService(MonographService monographService) {
        this.monographService = monographService;
    }

    /**
     * Gets the dose range service.
     * 
     * @return the dose range service
     */
    public DoseRangeService getDoseRangeService() {
        return doseRangeService;
    }

    /**
     * Sets the dose range service.
     * 
     * @param doseRangeService
     *            the new dose range service
     */
    public void setDoseRangeService(DoseRangeService doseRangeService) {
        this.doseRangeService = doseRangeService;
    }

}
