package gov.va.med.pharmacy.pecs.web.detail;

import gov.va.med.pharmacy.pecs.lock.ConceptRecordId;
import gov.va.med.pharmacy.pecs.lock.Lock;
import gov.va.med.pharmacy.pecs.service.RecordLockingService;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;

/**
 * 
 * This is a helper class to consolidate the code used for locking the concept records.
 * 
 * @author vhaisacarusr
 * 
 */
@Configuration
public class RecordLockingHelper {
    @Autowired
    private RecordLockingService recordLockingService;

    /**
     * Checks to see if the user can edit a record.
     * 
     * @param recordType
     *            either FDB or VA use the <code>RecordType</code> Enum
     * @param conceptName
     *            Name of the concept. use the <code>ConceptType</code> Enum
     * @param id
     *            the id of the master record. For FDB records this should be the FDB id
     * @param userName
     *            the user name from the <code>Profile</code>
     * @return a message if there is an error and an empty string if the lock was a success.
     */
    public final String checkEdit(String recordType, String conceptName, String id, String userName) {

        ConceptRecordId conceptRecordId = new ConceptRecordId(conceptName, id, recordType);
        boolean isLocked = this.isLocked(conceptRecordId);
        Lock lock = null;
        if (isLocked) {
            lock = getRecordLockingService().getLockingInformation(conceptRecordId);
            return "The record is being edited by user " + lock.getUser() + " and is unavailable for editing";
        }
        lock = new Lock();
        lock.setConceptRecordId(conceptRecordId);
        lock.setDateLocked();
        lock.setUser(userName);
        getRecordLockingService().lockRecord(lock);
        return "";
    }

    /**
     * Cancels the edit and deletes the lock on the object.
     * 
     * @param recordType
     *            either FDB or VA use the <code>RecordType</code> Enum
     * @param conceptName
     *            Name of the concept. use the <code>ConceptType</code> Enum
     * @param id
     *            the id of the master record. For FDB records this should be the FDB id
     * @param userName
     *            the user name from the <code>Profile</code>
     * @return a message if there is an error and an empty string if the lock was a success.
     */
    public final String cancelEdit(String recordType, String conceptName, String id, String userName) {

        ConceptRecordId conceptRecordId = new ConceptRecordId(conceptName, id, recordType);
        Lock lock = getRecordLockingService().getLockingInformation(conceptRecordId);
        if (lock != null && userName.equals(lock.getUser())) {
            deleteLock(conceptRecordId);
        }
        return "";
    }

    /**
     * Checks to see if the record is locked.
     * 
     * @param conceptRecordId
     *            the record id to be checked
     * @return true if the record is locked false otherwise
     */
    public final boolean isLocked(ConceptRecordId conceptRecordId) {
        return recordLockingService.isLocked(conceptRecordId);
    }

    /**
     * Deletes the lock for the specified record Id.
     * 
     * @param conceptRecordId
     *            the record id to be deleted
     */
    public final void deleteLock(ConceptRecordId conceptRecordId) {
        Lock lock = getRecordLockingService().getLockingInformation(conceptRecordId);
        getRecordLockingService().deleteLock(lock);

    }

    /**
     * Extends the lock time on a record.
     * 
     * @param recordType
     *            either FDB or VA use the <code>RecordType</code> Enum
     * @param conceptName
     *            Name of the concept. use the <code>ConceptType</code> Enum
     * @param id
     *            the id of the master record. For FDB records this should be the FDB id
     * @param userName
     *            the user name from the <code>Profile</code>
     */
    public final void extendLock(String recordType, String conceptName, String id, String userName) {
        ConceptRecordId conceptRecordId = new ConceptRecordId(conceptName, id, recordType);
        boolean isLocked = this.isLocked(conceptRecordId);
        Lock lock = new Lock();
        if (isLocked) {
            lock = getRecordLockingService().getLockingInformation(conceptRecordId);
            if (lock.getUser().equals(userName)) {
                this.deleteLock(conceptRecordId);
                lock.setDateLocked();
                getRecordLockingService().lockRecord(lock);
            }
        }
    }

    /**
     * Gets the record locking service.
     * 
     * @return the recordLockingService
     */
    public final RecordLockingService getRecordLockingService() {
        return recordLockingService;
    }

    /**
     * Sets the record locking service.
     * 
     * @param recordLockingService
     *            the recordLockingService to set
     */
    public final void setRecordLockingService(RecordLockingService recordLockingService) {
        this.recordLockingService = recordLockingService;
    }
}
