package gov.va.med.pharmacy.pecs.web.detail.doserange;

import gov.va.med.pharmacy.pecs.domain.impl.va.DoseRangeKey;
import gov.va.med.pharmacy.pecs.exception.CustomizationServiceException;
import gov.va.med.pharmacy.pecs.web.util.BaseRecordKeyHelper;

import org.apache.log4j.Logger;

/**
 * A helper class to deal with the pipe delimited Dose Range key from the web.
 * 
 * @author VHAISACARUSR
 * 
 */
public final class DoseRangeCheckRecordKeyHelper {
    private static final Logger LOG = Logger.getLogger(DoseRangeCheckRecordKeyHelper.class);
    private static final int CONCEPT_ID = 0;
    private static final int CONCEPT_TYPE = 1;
    private static final int AGE_LOW_IN_DAYS = 2;
    private static final int AGE_HIGH_IN_DAYS = 3;

    private static final int DOSE_ROUTE_ID = 4;
    private static final int DOSE_TYPE_ID = 5;
    private static final int FDBDX = 6;
    private static final int HIT_TYPE = 7;

    private long conceptId;
    private long conceptType;
    private long ageLowDays;
    private long ageHighDays;
    private String doseRouteId;
    private String doseTypeId;
    private String fdbdx;
    private long hitType;

    /**
     * Parses the record key.
     * 
     * example 12|6|0|29|064|01|999|1
     * 
     * @param recordKey
     *            the pipe delimited key
     */
    public DoseRangeCheckRecordKeyHelper(String recordKey) {
        if (recordKey == null || recordKey.equals(""))
            throw new IllegalArgumentException("Record ID parameter is null or could not be determined from: " + recordKey);

        if (LOG.is   ugEnabled())
            LOG.debug("Parsing recordkey: " + recordKey);

        try {
            String[] result = recordKey.split("\\" + BaseRecordKeyHelper.WIZARD_RECORDID_DELIMITER);
            if (result.length > 0) {

                setConceptId(Long.parseLong(result[CONCEPT_ID]));
                setConceptType(Long.parseLong(result[CONCEPT_TYPE]));
                setAgeLowDays(Long.parseLong(result[AGE_LOW_IN_DAYS]));
                setAgeHighDays(Long.parseLong(result[AGE_HIGH_IN_DAYS]));
                setDoseRouteId(result[DOSE_ROUTE_ID]);
                setDoseTypeId(result[DOSE_TYPE_ID]);
                setFdbdx(result[FDBDX]);
                setHitType(Long.parseLong(result[HIT_TYPE]));
            }
        } catch (Exception ex) {
            LOG.error("Unable to parse record id: " + recordKey, ex);
            throw new CustomizationServiceException("Unable to parse record key", ex);
        }

    }

    /**
     * A method to turn the key back to a pipe delimited string to create a URL.
     * 
     * @param key
     *            the dose range key
     * @return a pipe delimited string
     */
    public static final String createUrlFromKey(DoseRangeKey key) {
        return key.getConceptId() + "|" + key.getConceptType() + "|" + key.getAgeLowInDays() + "|" + key.getAgeHighInDays()
                + "|" + key.getDoseRouteId() + "|" + key.getDoseTypeId() + "|" + key.getFdbdx() + "|" + key.getHitType();
    }

    /**
     * Gets the conceptId.
     * 
     * @return the conceptId
     */
    public long getConceptId() {
        return conceptId;
    }

    /**
     * Sets the conceptId.
     * 
     * @param conceptId
     *            the conceptId to set
     */
    public void setConceptId(long conceptId) {
        this.conceptId = conceptId;
    }

    /**
     * Gets the conceptType.
     * 
     * @return the conceptType
     */
    public long getConceptType() {
        return conceptType;
    }

    /**
     * Sets the conceptType.
     * 
     * @param conceptType
     *            the conceptType to set
     */
    public void setConceptType(long conceptType) {
        this.conceptType = conceptType;
    }

    /**
     * Gets the ageLowDays.
     * 
     * @return the ageLowDays
     */
    public long getAgeLowDays() {
        return ageLowDays;
    }

    /**
     * Sets the ageLowDays.
     * 
     * @param ageLowDays
     *            the ageLowDays to set
     */
    public void setAgeLowDays(long ageLowDays) {
        this.ageLowDays = ageLowDays;
    }

    /**
     * Gets the ageHighDays.
     * 
     * @return the ageHighDays
     */
    public long getAgeHighDays() {
        return ageHighDays;
    }

    /**
     * Sets the ageHighDays.
     * 
     * @param ageHighDays
     *            the ageHighDays to set
     */
    public void setAgeHighDays(long ageHighDays) {
        this.ageHighDays = ageHighDays;
    }

    /**
     * Gets the doseRouteId.
     * 
     * @return the doseRouteId
     */
    public String getDoseRouteId() {
        return doseRouteId;
    }

    /**
     * Sets the doseRouteId.
     * 
     * @param doseRouteId
     *            the doseRouteId to set
     */
    public void setDoseRouteId(String doseRouteId) {
        this.doseRouteId = doseRouteId;
    }

    /**
     * Gets the doseTypeId.
     * 
     * @return the doseTypeId
     */
    public String getDoseTypeId() {
        return doseTypeId;
    }

    /**
     * Sets the doseTypeId.
     * 
     * @param doseTypeId
     *            the doseTypeId to set
     */
    public void setDoseTypeId(String doseTypeId) {
        this.doseTypeId = doseTypeId;
    }

    /**
     * Gets the fdbdx.
     * 
     * @return the fdbdx
     */
    public String getFdbdx() {
        return fdbdx;
    }

    /**
     * Sets the fdbdx.
     * 
     * @param fdbdx
     *            the fdbdx to set
     */
    public void setFdbdx(String fdbdx) {
        this.fdbdx = fdbdx;
    }

    /**
     * Gets the hitType.
     * 
     * @return the hitType
     */
    public long getHitType() {
        return hitType;
    }

    /**
     * Sets the hitType.
     * 
     * @param hitType
     *            the hitType to set
     */
    public void setHitType(long hitType) {
        this.hitType = hitType;
    }

    /**
     * Creates a dose range key and returns it.
     * 
     * @return a populated dose range key
     */
    public DoseRangeKey createDoseRangeKey() {
        DoseRangeKey doseRangeKey = new DoseRangeKey();
        doseRangeKey.setAgeHighInDays(getAgeHighDays());
        doseRangeKey.setAgeLowInDays(getAgeLowDays());
        doseRangeKey.setConceptId(getConceptId());
        doseRangeKey.setConceptType(getConceptType());
        doseRangeKey.setDoseRouteId(getDoseRouteId());
        doseRangeKey.setDoseTypeId(getDoseTypeId());

        doseRangeKey.setFdbdx(getFdbdx());
        doseRangeKey.setHitType(getHitType());
        return doseRangeKey;
    }

}
