package gov.va.med.pharmacy.pecs.web.detail.doserange;

import gov.va.med.pharmacy.pecs.domain.impl.lookup.DoseRoute;
import gov.va.med.pharmacy.pecs.domain.impl.lookup.DoseType;
import gov.va.med.pharmacy.pecs.domain.impl.lookup.DoseUnitType;
import gov.va.med.pharmacy.pecs.domain.impl.lookup.HalfLifeUnit;
import gov.va.med.pharmacy.pecs.domain.impl.va.DoseRangeKey;
import gov.va.med.pharmacy.pecs.domain.impl.va.DoseRangeXref;
import gov.va.med.pharmacy.pecs.domain.impl.va.UserProfile;
import gov.va.med.pharmacy.pecs.dto.DoseRangeDTO;
import gov.va.med.pharmacy.pecs.exception.CustomizationReportException;
import gov.va.med.pharmacy.pecs.exception.CustomizationServiceException;
import gov.va.med.pharmacy.pecs.metadata.Concept;
import gov.va.med.pharmacy.pecs.metadata.ConceptField;
import gov.va.med.pharmacy.pecs.metadata.ConceptMetadata;
import gov.va.med.pharmacy.pecs.persistence.fdb.DrugConceptDao;
import gov.va.med.pharmacy.pecs.service.DoseRangeService;
import gov.va.med.pharmacy.pecs.service.TerminologyService;
import gov.va.med.pharmacy.pecs.state.Status;
import gov.va.med.pharmacy.pecs.web.AbstractController;
import gov.va.med.pharmacy.pecs.web.RecordType;
import gov.va.med.pharmacy.pecs.web.detail.LockingResult;
import gov.va.med.pharmacy.pecs.web.detail.RecordLockingHelper;
import gov.va.med.pharmacy.pecs.web.view.ReportHistoryXLSXViewer;
import gov.va.med.pharmacy.pecs.web.view.document.AbstractXLSXViewer;

import java.io.IOException;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.stereotype.Controller;
import org.springframework.validation.BindingResult;
import org.springframework.validation.ObjectError;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.view.RedirectView;

/**
 * The controller class for customizing a Dose Range concept.
 * 
 * @author vhaisaduffg
 * 
 */
@Controller
@RequestMapping(value = "/doseRangeCustomization")
public final class DoseRangeCustomizationController extends AbstractController implements InitializingBean {

    /** The Constant SELECT_UNASSIGNED. */
    private static final String SELECT_UNASSIGNED = "UNASSIGNED";

    /** The logger. */
    private final Log logger = LogFactory.getLog(DoseRangeCustomizationController.class);

    /** The Constant CONCEPT_TYPE. */
    private static final String CONCEPT_TYPE = "concepttype";

    /** The Constant CONCEPT_ID. */
    private static final String CONCEPT_ID = "conceptid";

    /** The Constant HISTORY_REPORT. */
    private static final String HISTORY_REPORT = "HISTORY_REPORT";

    /** The Constant DEFAULT_CONCEPT_TYPE. */
    private static final long DEFAULT_CONCEPT_TYPE = 6;

    /** The Constant DEFAULT_FDBDX. */
    private static final String DEFAULT_FDBDX = "999";

    /** The Constant DEFAULT_DXID. */
    private static final String DEFAULT_DXID = "4892";

    /** The Constant DEFAULT_CATEGORY. */
    private static final String DEFAULT_CATEGORY = "VA";

    /** The Constant DOUBLE_FORMAT. */
    private static final NumberFormat DOUBLE_FORMAT = new DecimalFormat("############.########");

    /** The Constant DOUBLE_PRECISION. */
    private static final Integer DOUBLE_PRECISION = 10;

    /** The Constant LONG_PRECISION. */
    private static final Integer LONG_PRECISION = 6;

    /** The Constant SCALE. */
    private static final Integer SCALE = 6; // The number of digits after the decimal point

    /** The Constant REDIRECT_URL. */
    private static final String REDIRECT_URL = "/pecs/doseRangeCustomization/VA/";

    /** The dose range service. */
    @Resource
    private DoseRangeService doseRangeService;

    /** The terminology service. */
    @Resource
    private TerminologyService terminologyService;

    /** The record locking helper. */
    @Resource
    private RecordLockingHelper recordLockingHelper;

    /** The Constant VIEW. */
    private static final String VIEW = "pecs.doserangecustomization.page";

    /**
     * Execute query.
     * 
     * @param recordId
     *            the record id
     * @param response
     *            the response
     * @return the model and view
     * @throws CustomizationReportException
     *             the customization report exception
     */
    @RequestMapping(value = "/history/VA/{recordId:.+}", method = RequestMethod.GET)
    public ModelAndView executeQuery(@PathVariable(value = "recordId") String recordId, HttpServletResponse response)
            throws CustomizationReportException {
        Map<String, Object> model = new HashMap<String, Object>();
        model.put(ReportHistoryXLSXViewer.METADATA_KEY, Concept.DOSE_RANGE);
        model.put(ReportHistoryXLSXViewer.DATA_KEY, doseRangeService.getVersionHistoryReportData(Long.parseLong(recordId)));
        model.put(AbstractXLSXViewer.EXCEL_FILE_NAME_KEY, Concept.DOSE_RANGE);
        model.put(AbstractXLSXViewer.EXCEL_FILE_TYPE_KEY, HISTORY_REPORT);
        return new ModelAndView(ReportHistoryXLSXViewer.REPORT_HISTORY_VIEW, model);

    }

    /**
     * Requests This will load the detail page for staging query result only.
     * 
     * @param recordId
     *            the identifier of the record
     * @param request
     *            HttpServletRequest
     * @return ModelAndView
     */
    @RequestMapping(value = "/VA/{recordId:.+}", method = RequestMethod.GET)
    public final ModelAndView load(@PathVariable(value = "recordId") String recordId, HttpServletRequest request) {

        DoseRangeDTO doseRangeDTO = new DoseRangeDTO();
        try {
            doseRangeDTO = retrieveCustomModelByRecordId(recordId);
            if (doseRangeDTO.getConceptXref() != null) {
                DoseRangeKey key = ((DoseRangeXref) doseRangeDTO.getConceptXref()).getDoseRangeKey();
                DoseRangeDTO fdbRecord = doseRangeService.getFdbData(key);
                if (fdbRecord != null) {
                    String url =
                            DoseRangeCheckRecordKeyHelper.createUrlFromKey(((DoseRangeXref) doseRangeDTO.getConceptXref())
                                    .getDoseRangeKey());
                    this.addInformationMessage("<a href='../../doseRangeCustomization/FDB/" + url
                            + "' >Corresponding FDB Record</a>");
                } else {
                    this.addInformationMessage("Corresponding FDB record not found");
                }
            }
        } catch (CustomizationServiceException ex) {
            logger.error("Unable to display Dose Range detail page ", ex);

        }

        doseRangeDTO.setReadOnlyRecord(!doseRangeDTO.isCurrent());

        return setupForDisplay(getUserProfile(request), doseRangeDTO, false);
    }

    /**
     * This will update the state machine for ctstaging detail page.
     * 
     * @param recordId
     *            the record identifier
     * @param recordType
     *            the record type
     * @param request
     *            the HTTP request
     * @param response
     *            the HTTP response
     * @param doseRangeDTO
     *            the data
     * @param bindingResult
     *            object for returning errors
     * @return ModelAndView
     * @throws IOException
     *             when an error is encountered writing to a file
     */
    @RequestMapping(value = "/{recordType}/{recordId:.+}", method = RequestMethod.POST)
    public final ModelAndView performAction(@PathVariable(value = "recordId") String recordId, @PathVariable(
            value = "recordType") String recordType, HttpServletRequest request, HttpServletResponse response,
            @Valid DoseRangeDTO doseRangeDTO, BindingResult bindingResult) throws IOException {
        UserProfile profile = getUserProfile(request);
        ModelAndView modelAndView = new ModelAndView(VIEW);
        if (bindingResult.hasErrors()) {
            modelAndView = setupForDisplay(profile, doseRangeDTO, false);
            return modelAndView;
        }
        validate(doseRangeDTO, bindingResult);
        if (bindingResult.hasErrors()) {
            modelAndView = setupForDisplay(profile, doseRangeDTO, false);
            return modelAndView;
        }

        String action = formatAction(request);

        DoseRangeDTO doseRangeDTOExistingData = new DoseRangeDTO();

        if (RecordType.FDB.equals(RecordType.fromString(recordType))) {
            DoseRangeCheckRecordKeyHelper helper = new DoseRangeCheckRecordKeyHelper(recordId);

            DoseRangeXref fdbReference = new DoseRangeXref();

            fdbReference.setDoseRangeKey(helper.createDoseRangeKey());
            doseRangeDTO.setConceptXref(fdbReference);

            doseRangeDTO = save(recordId, request, doseRangeDTO, action, bindingResult);

            if (bindingResult.hasErrors()) {
                List<DoseRangeDTO> customRecords = doseRangeService.findCustomByXref(helper.createDoseRangeKey());
                modelAndView = setupForDisplay(profile, doseRangeDTO, false);
                modelAndView.addObject("showCommentButton", showCommentButton(customRecords));
                return modelAndView;
            }
            cancelLock("FDB", doseRangeDTO.getFdbId(), request);
        } else {
            try {
                List<DoseRangeDTO> fdbModelList =
                        retrieveCustomModels(doseRangeDTO.getConceptid(), doseRangeDTO.getDoserouteid(),
                                doseRangeDTO.getDosetypeid());
                if (fdbModelList != null && fdbModelList.size() > 0) {
                    int i = 0;
                    boolean b = false;
                    for (DoseRangeDTO model : fdbModelList) {
                        i = i + 1;
                        if (!model.generateRecordId().equals(doseRangeDTO.generateRecordId())) {
                            if (model.isCurrent()) {
                                logger.debug("value of current before if " + doseRangeDTO.isCurrent());
                                boolean valid =
                                        !validateAgeRange(doseRangeDTO.getAgelowindays(), doseRangeDTO.getAgehighindays(),
                                                model.getAgelowindays(), model.getAgehighindays());
                                if (valid) {
                                    logger.debug("value of current inside if" + doseRangeDTO.isCurrent());

                                    b = true;
                                }
                            }
                        }
                    }
                    if (b) {
                        bindingResult.addError(new ObjectError("doseRangeDTO", findMessage("dosing.wizard.age.overlap")));
                    }
                }
                doseRangeDTOExistingData = retrieveCustomModelByRecordId(recordId);
                doseRangeDTO.setCategory("VA");
                doseRangeDTO.setActionBy(profile.getUsername());
                doseRangeDTO.setActionUser(profile);
                if (profile.isApprover() && doseRangeDTO.getAssignedTo() == null) {
                    doseRangeDTO.setAssignedTo(profile.getUsername());
                }

                if (doseRangeDTO.getAssignedTo() == null || doseRangeDTO.getAssignedTo().equals("")) {
                    doseRangeDTO.setAssignedTo(SELECT_UNASSIGNED);
                }
                String prevStatus = doseRangeDTO.getStatus();
                doseRangeDTO = doseRangeService.applyAction(action, doseRangeDTO, bindingResult);

                if (bindingResult.hasErrors()) {
                    modelAndView = setupForDisplay(profile, doseRangeDTO, false);
                    modelAndView.addObject("showCommentButton", false);
                    return modelAndView;
                } else {
                    String newStatus = doseRangeDTO.getStatus();
                    if (newStatus.equals("Modified")) {
                        if (newStatus.equals(prevStatus)) {
                            boolean fdbFieldChanged = false;
                            if (doseRangeDTOExistingData.getAgelowindays().equals(doseRangeDTO.getAgelowindays())) {
                                fdbFieldChanged = true;
                            }
                            if (doseRangeDTOExistingData.getAgehighindays().equals(doseRangeDTO.getAgehighindays())) {
                                fdbFieldChanged = true;
                            }
                            if (fdbFieldChanged) {
                                String message =
                                        getInformationalMessageDisplay(action, newStatus, doseRangeDTOExistingData.getStatus());
                                addInformationMessage(message);
                            }
                        } else {

                            addInformationMessage(getInformationalMessageDisplay(action, newStatus,
                                    doseRangeDTOExistingData.getStatus()));
                        }
                    } else {
                        if (!action.equals("Modify")) {
                            addInformationMessage(getInformationalMessageDisplay(action, newStatus,
                                    doseRangeDTOExistingData.getStatus()));
                        }
                    }
                }
            } catch (CustomizationServiceException ex) {

                logger.error("Unable to display Dose Range detail page ", ex);
                logger.error("Unable to save ddim models: ", ex);
                bindingResult.addError(new ObjectError("doseRangeDTO", findMessage("us.exception.create")));
            }
        }
        // THIS PART WILL BE CALLED AFTER UPDATE THE DATA.
        // repeated code from load method... need to put in a single
        ModelAndView view = new ModelAndView(VIEW);

        view = setupForDisplay(getUserProfile(request), doseRangeDTO, false);
        view.addObject("showCommentButton", false);
        if (!bindingResult.hasErrors()) {
            if (RecordType.FDB.equals(RecordType.fromString(recordType))) {
                cancelLock("FDB", recordId, request);
            } else {
                cancelLock("VA", String.valueOf(doseRangeDTO.getFdbCustomDosingVaId()), request);
            }
        }

        RedirectView redirectView = new RedirectView(REDIRECT_URL + doseRangeDTO.getFdbCustomDosingVaId(), true, false, false);
        view.setView(redirectView);
        return view;

    }

    /**
     * Requests This will load the detail page for FDB query result only. Before loading it will check to see if there is an
     * existing record.
     * 
     * @param recordId
     *            the identifier of the record
     * @param request
     *            HttpServletRequest
     * @return ModelAndView
     */
    @RequestMapping(value = "/FDB/{recordId:.+}", method = RequestMethod.GET)
    public final ModelAndView loadFDB(@PathVariable(value = "recordId") String recordId, HttpServletRequest request) {
        DoseRangeDTO doseRangeDTO = null;
        ModelAndView view = new ModelAndView(VIEW);
        UserProfile profile = getUserProfile(request);
        if (logger.is   ugEnabled())
            logger.debug("Received RecordKey from ActionContext: " + recordId);

        try {
            DoseRangeCheckRecordKeyHelper helper = new DoseRangeCheckRecordKeyHelper(recordId);

            // Determine if there is an existing INFLIGHT customization
            // for PECS 2.1, the value of concept type will be set to 6.
            List<DoseRangeDTO> customRecords = doseRangeService.findCustomByXref(helper.createDoseRangeKey());
            for (DoseRangeDTO customDosing : customRecords) {
                String msg =
                        "Existing VA Custom Record with Action Status of " + customDosing.getStatus()
                                + " created on: <a href='../../doseRangeCustomization/VA/"
                                + customDosing.getFdbCustomDosingVaId() + "' >" + customDosing.getActionDate() + "</a>";

                addInformationMessage(msg);

            }
            doseRangeDTO = doseRangeService.getFdbData(helper.createDoseRangeKey());

            doseRangeDTO.setActionBy(profile.getUsername());
            doseRangeDTO.setRequestor(profile.getUsername());
            doseRangeDTO.setActionDate(new Date());

            view = setupForDisplay(getUserProfile(request), doseRangeDTO, false);
            view.addObject("showCommentButton", showCommentButton(customRecords));

        } catch (CustomizationServiceException ex) {
            logger.error("Unable to load dose range models: ", ex);

        }
        return view;

    }

    /**
     * Requests This will load the detail page for FDB query result only. Before loading it will check to see if there is an
     * existing record.
     * 
     * @param request
     *            HttpServletRequest
     * @return ModelAndView
     */
    @RequestMapping(value = "/blank", method = RequestMethod.GET)
    public final ModelAndView blank(HttpServletRequest request) {

        DoseRangeDTO doseRangeDTO = new DoseRangeDTO();
        UserProfile profile = new UserProfile();
        try {

            profile = getUserProfile(request);
            doseRangeDTO = new DoseRangeDTO();
            doseRangeDTO.setActionBy(profile.getUsername());
            doseRangeDTO.setRequestor(profile.getUsername());
            doseRangeDTO.setActionDate(new Date());

        } catch (CustomizationServiceException ex) {
            logger.error("Unable to load dose range models: ", ex);
        }

        return setupForDisplay(profile, doseRangeDTO, true);

    }

    /**
     * Post processing for a Open From Blank request.
     * 
     * @param request
     *            HTTP request
     * @param response
     *            HttpServletResponse
     * @param doseRangeDTO
     *            the data
     * @param bindingResult
     *            an object for errors
     * @return ModelAndView
     * @throws IOException
     *             problems with sending redirect
     */
    @RequestMapping(value = "/blank", method = RequestMethod.POST)
    public final ModelAndView blank(HttpServletRequest request, HttpServletResponse response, @Valid DoseRangeDTO doseRangeDTO,
            BindingResult bindingResult) throws IOException {
        doseRangeDTO = save(null, request, doseRangeDTO, "Customize", bindingResult);
        UserProfile profile = getUserProfile(request);
        if (bindingResult.hasErrors()) {
            return setupForDisplay(profile, doseRangeDTO, true);
        }

        ModelAndView mav = new ModelAndView(VIEW);
        response.sendRedirect(request.getContextPath() + request.getServletPath() + "/doseRangeCustomization/VA/"
                + doseRangeDTO.getFdbCustomDosingVaId());
        mav.setViewName("redirect:" + VIEW);
        return mav;
    }

    /**
     * Locks a Dose Range record.
     * 
     * @param recordType
     *            the type of record being locked
     * @param recordId
     *            the identifier of the record to lock
     * @param request
     *            HttpServletRequest
     * @return LockingResult
     */
    @RequestMapping(value = "edit/{recordType}/{recordId:.+}", method = RequestMethod.GET)
    @ResponseBody
    public final LockingResult lockRecord(@PathVariable(value = "recordType") String recordType, @PathVariable(
            value = "recordId") String recordId, HttpServletRequest request) {
        DoseRangeDTO doseRangeDTO = null;
        LockingResult result = new LockingResult();
        if (RecordType.VA.equals(RecordType.fromString(recordType))) {
            doseRangeDTO = doseRangeService.getStagingDataByVersionId(Long.parseLong(recordId));

            if (!doseRangeDTO.isCurrent()) {
                result.setMessage(findMessage("detail.message.notcurrent"));
                DoseRangeDTO current = doseRangeService.getCurrentStagingDataByVersionId(Long.parseLong(recordId));

                result.setCurrent(false);
                result.setUrl(current.getSelect().getValue());
                return result;
            }
            recordId = String.valueOf(doseRangeDTO.getMasterRecordId());
        }

        String message =
                getRecordLockingHelper().checkEdit(recordType, Concept.DOSE_RANGE.toString(), recordId,
                        getUserProfile(request).getUsername());
        result.setMessage(message);
        logger.warn(message);
        return result;
    }

    /**
     * Cancels a lock on a record.
     * 
     * @param recordType
     *            type of record
     * @param recordId
     *            the identifier of the record
     * @param request
     *            HttpServletRequest
     * @return String
     */
    @RequestMapping(value = "cancel/{recordType}/{recordId:.+}", method = RequestMethod.GET)
    @ResponseBody
    public final String cancelLock(@PathVariable(value = "recordType") String recordType,
            @PathVariable(value = "recordId") String recordId, HttpServletRequest request) {

        DoseRangeDTO doseRangeDTO = null;

        if (RecordType.VA.equals(RecordType.fromString(recordType))) {
            doseRangeDTO = doseRangeService.getStagingDataByVersionId(Long.parseLong(recordId));
            recordId = String.valueOf(doseRangeDTO.getMasterRecordId());
        }

        return getRecordLockingHelper().cancelEdit(recordType, Concept.DOSE_RANGE.toString(), recordId,
                getUserProfile(request).getUsername());

    }

    /**
     * Extends a lock on a record.
     * 
     * @param recordType
     *            type of record
     * @param recordId
     *            the identifier of the record
     * @param request
     *            HttpServletRequest
     * @return String
     */
    @RequestMapping(value = "extend/{recordType}/{recordId:.+}", method = RequestMethod.GET)
    @ResponseBody
    public final LockingResult extendLock(@PathVariable(value = "recordType") String recordType, @PathVariable(
            value = "recordId") String recordId, HttpServletRequest request) {
        DoseRangeDTO doseRangeDTO = null;
        LockingResult result = new LockingResult();
        if (RecordType.VA.equals(RecordType.fromString(recordType))) {
            doseRangeDTO = doseRangeService.getStagingDataByVersionId(Long.parseLong(recordId));
            if (!doseRangeDTO.isCurrent()) {
                DoseRangeDTO current = doseRangeService.getCurrentStagingDataByVersionId(Long.parseLong(recordId));
                result.setCurrent(false);
                result.setUrl(current.getSelect().getValue());
                return result;
            }

            recordId = String.valueOf(doseRangeDTO.getMasterRecordId());
        }

        getRecordLockingHelper().extendLock(recordType, Concept.DOSE_RANGE.toString(), recordId,
                getUserProfile(request).getUsername());

        return result;
    }

    /**
     * This is called from performAction method to save FDB record to staging database.
     * 
     * @param recordId
     *            identifier for the record
     * @param request
     *            HttpServletRequest
     * @param doseRangeDTO
     *            doseRangeDTO object
     * @param action
     *            the action being performed
     * @param bindingResult
     *            BindingResult object
     * @return doseRangeDTO
     */
    private final DoseRangeDTO save(String recordId, HttpServletRequest request, DoseRangeDTO doseRangeDTO, String action,
            BindingResult bindingResult) {

        doseRangeDTO.setCategory("VA");

        // begin block 1 of 2

        try {
            // Validate that required fields are populated
            long newAgeLow = (doseRangeDTO.getAgelowindays() != null ? doseRangeDTO.getAgelowindays() : 0L);
            long newAgeHigh = (doseRangeDTO.getAgehighindays() != null ? doseRangeDTO.getAgehighindays() : 0L);

            // Retrieve any existing custom models, and validate age overlap
            List<DoseRangeDTO> fdbModelList =
                    retrieveCustomModels(doseRangeDTO.getConceptid(), doseRangeDTO.getDoserouteid(),
                            doseRangeDTO.getDosetypeid());
            if (fdbModelList != null && fdbModelList.size() > 0) {
                boolean isValidAgeRange = false;
                for (DoseRangeDTO model : fdbModelList) {
                    if (model.isCurrent()) {
                        if (!validateAgeRange(newAgeLow, newAgeHigh, model.getAgelowindays(), model.getAgehighindays())) {
                            isValidAgeRange = true;
                        }
                    }
                }

                if (isValidAgeRange) {
                    bindingResult.addError(new ObjectError("doseRangeDTO", findMessage("dosing.wizard.age.overlap")));
                    return doseRangeDTO;
                }
            }
        } catch (CustomizationServiceException ex) {
            logger.error("Unable to display Dose Range detail page ", ex);
            logger.error("Unable to save ddim models: ", ex);
            bindingResult.addError(new ObjectError("doseRangeDTO", findMessage("us.exception.create")));
        }
        // end block 1 of 2
        // begin block 2 of 2
        try {
            // Re-Populate default, and uncustomized and undisplayed values
            // -Update model with computed GUI form fields
            UserProfile profile = getUserProfile(request);
            if (profile.isApprover() && doseRangeDTO.getAssignedTo() == null)
                doseRangeDTO.setAssignedTo(profile.getUsername());

            if (doseRangeDTO.getAssignedTo() == null || doseRangeDTO.getAssignedTo().equals(""))
                doseRangeDTO.setAssignedTo(SELECT_UNASSIGNED);

            doseRangeDTO.setActionBy(profile.getUsername());
            doseRangeDTO.setRequestor(profile.getUsername());

            doseRangeDTO.setConcepttype(DEFAULT_CONCEPT_TYPE);
            doseRangeDTO.setCategory(DEFAULT_CATEGORY);
            if (doseRangeDTO.getFdbdx() == null || doseRangeDTO.getFdbdx().equals(""))
                doseRangeDTO.setFdbdx(DEFAULT_FDBDX);

            doseRangeDTO.setDxid(DEFAULT_DXID);
            doseRangeDTO = doseRangeService.createNewRecord(doseRangeDTO, bindingResult);
            if (bindingResult.hasErrors()) {
                return doseRangeDTO;// do not know if it is correct
                                    // way to return.
            } else {
                doseRangeDTO.setCurrentActionReason("");
                String message = getInformationalMessageDisplay(action, "", "");
                addInformationMessage(message);
            }

        } catch (CustomizationServiceException ex) {
            logger.error("Unable to save dose range models: ", ex);
            displayError(findMessage("error.save") + " (" + ex.getMessage() + ")", doseRangeDTO, bindingResult);
            bindingResult.addError(new ObjectError("doseRangeDTO", findMessage("us.exception.create")));
        }
        return doseRangeDTO;

    }

    /**
     * Displays an error message to the user.
     * 
     * @param error
     *            message to display
     * @param doseRangeDTO
     *            doseRangeDTO object
     * @param result
     *            BindingResult
     * @return ModelAndView
     */
    public final ModelAndView displayError(String error, DoseRangeDTO doseRangeDTO, BindingResult result) {
        result.addError(new ObjectError("doseRangeDTO", error));
        ModelAndView view = new ModelAndView(VIEW);
        view.addObject("doseRangeDTO", doseRangeDTO);
        return view;
    }

    /**
     * Attempts to retrieve and return the Description of the concept that the Concept Type and Concept Identifier fields point
     * to.
     * 
     * @param conceptType
     *            the concept type
     * @param conceptId
     *            the concept identifier
     * @return String
     */
    @RequestMapping(value = "/find/conceptdescription", method = RequestMethod.GET)
    @ResponseBody
    public final String findConceptDescription(@RequestParam(CONCEPT_TYPE) String conceptType,
            @RequestParam(CONCEPT_ID) String conceptId) {
        String returnValue = null;

        if (conceptType == null || conceptId == null) {
            return null;
        } else {
            final Long cncptType = new Long(conceptType);
            final Long cncptId = new Long(conceptId);

            returnValue = doseRangeService.getConceptDescription(cncptId, cncptType);
        }
        return returnValue;
    }

    /**
     * Retrieves a customized Dose Range.
     * 
     * @param conceptId
     *            identifier of the Drug
     * @param conceptType
     *            type of Drug
     * @param ageLowInDays
     *            Age in days Low
     * @param ageHighInDays
     *            Age in days High
     * @param doseRouteId
     *            identifier of the Dose Route
     * @param doseTypeId
     *            identifier of the Dose Type
     * @param fdbdx
     *            FDBDX
     * @return doseRangeDTO
     */
    protected final DoseRangeDTO retrieveCustomModelByKey(long conceptId, long conceptType, long ageLowInDays,
            long ageHighInDays, String doseRouteId, String doseTypeId, String fdbdx) {
        return doseRangeService.getStagingData(conceptId, conceptType, ageLowInDays, ageHighInDays, doseRouteId, doseTypeId,
                fdbdx);
    }

    /**
     * Retrieves a Custom Dose Range by it's identifier.
     * 
     * @param recordId
     *            the identifier
     * @return doseRangeDTO
     */
    private DoseRangeDTO retrieveCustomModelByRecordId(String recordId) {
        return doseRangeService.getStagingDataByVersionId(Long.valueOf(recordId));
    }

    /**
     * Retrieves the user, lookup values and common data for displaying the Dose Range.
     * 
     * @param profile
     *            UserProfile
     * @param doseRangeDTO
     *            doseRangeDTO
     * @param allowEditableConceptId
     *            true if the Dose Range will be editable
     * @return ModelAndView
     */
    private ModelAndView setupForDisplay(UserProfile profile, DoseRangeDTO doseRangeDTO, boolean allowEditableConceptId) {

        ModelAndView view = new ModelAndView(VIEW);

        ConceptMetadata conceptMetadata = doseRangeService.getMetadata();
        Map<String, ConceptField> fieldMap = conceptMetadata.getFieldMap();

        // Concept Id should be editable when opening a blank Dose Range. At all other times it should be read-only.
        if (allowEditableConceptId) {
            ConceptField conceptId = fieldMap.get("CONCEPTID");
            conceptId.setOverride(ConceptField.EDITABLE, true);
        }

        // Dose Range should not be editable if it is not current, or the Concept Type is not a DispensableGeneric.
        doseRangeDTO.setReadOnlyRecord(!doseRangeDTO.isCurrent()
                || doseRangeDTO.getConcepttype() != DrugConceptDao.DISPENSABLE_GENERIC_TYPE);

        view.addObject("doseRangeDTO", doseRangeDTO);
        view.addObject("fieldMap", fieldMap);

        // display list of request assigned to.

        List<DoseUnitType> doseUnits = terminologyService.getDoseUnitTypeData();
        view.addObject("units", doseUnits);

        List<DoseRoute> doseRoutes = terminologyService.getDoseRoutes();
        view.addObject("routes", doseRoutes);

        List<DoseType> doseTypes = terminologyService.getDoseTypes();
        view.addObject("types", doseTypes);

        List<HalfLifeUnit> halfLifeUnits = terminologyService.getHalfLifeUnits();
        view.addObject("halfLifeUnits", halfLifeUnits);

        List<String> userList = this.getAdminUsers();
        view.addObject("userList", userList);

        view.addObject("allowableActions", Collections.emptySet());

        // interrogating the state machine for allowable actions makes sense
        // if we're not displaying the record in read-only mode and it's not
        // and FDB record
        Set<String> actions = Collections.emptySet();
        if (!doseRangeDTO.isReadOnlyRecord()) {
            if (doseRangeDTO.getFdbCustomDosingVaId() != 0) {
                actions = doseRangeService.getAllowableActions(doseRangeDTO, profile);
            }
            permissionValidation(profile, actions, view, doseRangeDTO.getFdbCustomDosingVaId());
        }
        return view;
    }

    /**
     * Retrieves one or more custom Dose Range based on the parameters.
     * 
     * @param conceptId
     *            identifier of the drug
     * @param doseRouteId
     *            identifier of the Dose Route
     * @param doseTypeId
     *            identifier of the Dose Type
     * @return List<doseRangeDTO>
     */
    private List<DoseRangeDTO> retrieveCustomModels(Long conceptId, String doseRouteId, String doseTypeId) {
        return doseRangeService.getStagingData(conceptId, doseRouteId, doseTypeId);
    }

    /**
     * Validates the Age Range.
     * 
     * @param newAgeLow
     *            new Age in days low
     * @param newAgeHigh
     *            new Age in days high
     * @param custAgeLow
     *            customized age in days low
     * @param custAgeHigh
     *            customized age in days high
     * @return true if the new ages can be used, false otherwise
     */
    private boolean validateAgeRange(long newAgeLow, long newAgeHigh, long custAgeLow, long custAgeHigh) {
        if (logger.is   ugEnabled())
            logger.debug("Validate age range received newagelow: " + newAgeLow + " newagehigh: " + newAgeHigh + " custagelow: "
                    + custAgeLow + " custagehigh: " + custAgeHigh);

        if (newAgeLow == 0 && newAgeHigh == 0)
            return false;

        if (newAgeLow < newAgeHigh) {
            if (newAgeHigh < custAgeLow)
                return true;

            if (newAgeLow > custAgeHigh)
                return true;
        }

        return false;
    }

    /**
     * Validates the data in the incoming doseRangeDTO object. Errors are placed in the BindingResult object.
     * 
     * @param doseRangeDTO
     *            the data to validate
     * @param bindingResult
     *            holds errors that are found
     * @return BindingResult
     */
    private BindingResult validate(DoseRangeDTO doseRangeDTO, BindingResult bindingResult) {
        if (isLongValidAndNonZero(doseRangeDTO.getConceptid(), false) != DoseRangeValidation.VALID) {
            bindingResult.addError(new ObjectError("doseRangeDTO", findMessage("Digits.doseRangeDTO.conceptid")));
        }
        
        validateAgeInDaysRelationship(doseRangeDTO, bindingResult);
        validateDoubleUnitType(doseRangeDTO.getDoselow(), doseRangeDTO.getDoselowunits(),
                "invalid.decimal.doseRangeDTO.doselow", "invalid.unit.doseRangeDTO.doselow", bindingResult);
        validateDoubleUnitType(doseRangeDTO.getDosehigh(), doseRangeDTO.getDosehighunits(),
                "invalid.decimal.doseRangeDTO.dosehigh", "invalid.unit.doseRangeDTO.dosehigh", bindingResult);
        validateDoubleUnitType(doseRangeDTO.getDoseformlow(), doseRangeDTO.getDoseformlowunits(),
                "invalid.decimal.doseRangeDTO.doseformlow", "invalid.unit.doseRangeDTO.doseformlow", bindingResult);
        validateDoubleUnitType(doseRangeDTO.getDoseformhigh(), doseRangeDTO.getDoseformhighunits(),
                "invalid.decimal.doseRangeDTO.doseformhigh", "invalid.unit.doseRangeDTO.doseformhigh", bindingResult);

        // ///////////////5-11
        if (doseRangeDTO.getFrequencylow() != null && doseRangeDTO.getFrequencylow().hashCode() > 0) {
            if (isDecimalValid(doseRangeDTO.getFrequencylow()) == DoseRangeValidation.INVALID) {
                bindingResult
                        .addError(new ObjectError("doseRangeDTO", findMessage("invalid.decimal.doseRangeDTO.frequencylow")));
            }
        }
        if (doseRangeDTO.getFrequencyhigh() != null && doseRangeDTO.getFrequencyhigh().hashCode() > 0) {
            if (isDecimalValid(doseRangeDTO.getFrequencyhigh()) == DoseRangeValidation.INVALID) {
                bindingResult.addError(new ObjectError("doseRangeDTO",
                        findMessage("invalid.decimal.doseRangeDTO.frequencyhigh")));
            }
        }

        // shahid 10-23-2012
        validateDoubleUnitType(doseRangeDTO.getMaxSingleNTEDose(), doseRangeDTO.getMaxSingleNTEDoseUnit(),
                "invalid.decimal.doseRangeDTO.maxSingleNTEDose", "invalid.unit.doseRangeDTO.maxSingleNTEDose", bindingResult);

        validateDoubleUnitType(doseRangeDTO.getMaxSingleNTEDoseForm(), doseRangeDTO.getMaxSingleNTEDoseFormUnit(),
                "invalid.decimal.doseRangeDTO.maxSingleNTEDoseForm", "invalid.unit.doseRangeDTO.maxSingleNTEDoseForm",
                bindingResult);

        validateDoubleUnitType(doseRangeDTO.getMaxsingledose(), doseRangeDTO.getMaxsingledoseunits(),
                "invalid.decimal.doseRangeDTO.maxsingledose", "invalid.unit.doseRangeDTO.maxsingledose", bindingResult);
        validateDoubleUnitType(doseRangeDTO.getMaxsingledoseform(), doseRangeDTO.getMaxsingledoseformunits(),
                "invalid.decimal.doseRangeDTO.maxsingledoseform", "invalid.unit.doseRangeDTO.maxsingledoseform", bindingResult);
        // ////////////////12-15
        validateDoubleUnitType(doseRangeDTO.getMaxdailydose(), doseRangeDTO.getMaxdailydoseunits(),
                "invalid.decimal.doseRangeDTO.maxdailydose", "invalid.unit.doseRangeDTO.maxdailydose", bindingResult);
        validateDoubleUnitType(doseRangeDTO.getMaxdailydoseform(), doseRangeDTO.getMaxdailydoseformunits(),
                "invalid.decimal.doseRangeDTO.maxdailydoseform", "invalid.unit.doseRangeDTO.maxdailydoseform", bindingResult);
        validateDoubleUnitType(doseRangeDTO.getMaxlifetimedose(), doseRangeDTO.getMaxlifetimedoseunits(),
                "invalid.decimal.doseRangeDTO.maxlifetimedose", "invalid.unit.doseRangeDTO.maxlifetimedose", bindingResult);
        validateDoubleUnitType(doseRangeDTO.getMaxlifetimedoseform(), doseRangeDTO.getMaxlifetimedoseformunits(),
                "invalid.decimal.doseRangeDTO.maxlifetimedoseform", "invalid.unit.doseRangeDTO.maxlifetimedoseform",
                bindingResult);
        validateDoubleUnitType(doseRangeDTO.getDoseratelow(), doseRangeDTO.getDoseratelowunits(),
                "invalid.decimal.doseRangeDTO.doseratelow", "invalid.unit.doseRangeDTO.doseratelow", bindingResult);
        validateDoubleUnitType(doseRangeDTO.getDoseratehigh(), doseRangeDTO.getDoseratehighunits(),
                "invalid.decimal.doseRangeDTO.doseratehigh", "invalid.unit.doseRangeDTO.doseratehigh", bindingResult);
        validateDoubleUnitType(doseRangeDTO.getDoseformratelow(), doseRangeDTO.getDoseformratelowunits(),
                "invalid.decimal.doseRangeDTO.doseformratelow", "invalid.unit.doseRangeDTO.doseformratelow", bindingResult);
        validateDoubleUnitType(doseRangeDTO.getDoseformratehigh(), doseRangeDTO.getDoseformratehighunits(),
                "invalid.decimal.doseRangeDTO.doseformratehigh", "invalid.unit.doseRangeDTO.doseformratehigh", bindingResult);
        validateDoubleUnitType(doseRangeDTO.getMaxsingledoserate(), doseRangeDTO.getMaxsingledoserateunits(),
                "invalid.decimal.doseRangeDTO.maxsingledoserate", "invalid.unit.doseRangeDTO.maxsingledoserate", bindingResult);
        validateDoubleUnitType(doseRangeDTO.getMaxsingledoseformrate(), doseRangeDTO.getMaxsingledoseformrateunits(),
                "invalid.decimal.doseRangeDTO.maxsingledoseformrate", "invalid.unit.doseRangeDTO.maxsingledoseformrate",
                bindingResult);
        validateDoubleUnitType(doseRangeDTO.getMaxdailydoserate(), doseRangeDTO.getMaxdailydoserateunits(),
                "invalid.decimal.doseRangeDTO.maxdailydoserate", "invalid.unit.doseRangeDTO.maxdailydoserate", bindingResult);
        validateDoubleUnitType(doseRangeDTO.getMaxdailydoseformrate(), doseRangeDTO.getMaxdailydoseformrateunits(),
                "invalid.decimal.doseRangeDTO.maxdailydoseformrate", "invalid.unit.doseRangeDTO.maxdailydoseformrate",
                bindingResult);

        validateLongUnitType(doseRangeDTO.getCrclthreshhold(), doseRangeDTO.getCrclthreshholdunits(),
                "Digits.doseRangeDTO.crclthreshhold", "invalid.unit.doseRangeDTO.crclthreshhold", bindingResult);

        // ///////////////27-30
        if (doseRangeDTO.getLoweliminationhalflife() != null && doseRangeDTO.getLoweliminationhalflife().hashCode() != 0) {
            if (isDecimalValid(doseRangeDTO.getLoweliminationhalflife()) == DoseRangeValidation.INVALID) {
                bindingResult.addError(new ObjectError("DoseRangeDTO",
                        findMessage("invalid.decimal.doseRangeDTO.loweliminationhalflife")));
            }
        }
        if (doseRangeDTO.getHigheliminationhalflife() != null && doseRangeDTO.getHigheliminationhalflife().hashCode() != 0) {
            if (isDecimalValid(doseRangeDTO.getHigheliminationhalflife()) == DoseRangeValidation.INVALID) {
                bindingResult.addError(new ObjectError("DoseRangeDTO",
                        findMessage("invalid.decimal.doseRangeDTO.higheliminationhalflife")));
            }
        }

        return bindingResult;

    }

    /**
     * Checks the Age In Days Low and Age In Days High against each other.  
     * If both are specified, then the age high in days cannot be less than the age low in days.  
     * 
     * @param doseRangeDTO
     *            data to validate
     * @param bindingResult
     *            BindingResult
     * @return BindingResult
     */
    private BindingResult validateAgeInDaysRelationship(DoseRangeDTO doseRangeDTO, BindingResult bindingResult) {
        
        if (doseRangeDTO.getAgelowindays() == null || doseRangeDTO.getAgehighindays() == null){
            return bindingResult;
        }
        if (doseRangeDTO.getAgelowindays() > doseRangeDTO.getAgehighindays()){
            String error = findMessage("dosing.wizard.age.greater");
            bindingResult.addError(new ObjectError("doseRangeDTO", error));
        }

        return bindingResult;
    }

    /**
     * Retrieves the Informational Message to display.
     * 
     * @param action
     *            the action being performed
     * @param newStatus
     *            the new status of the object
     * @param prevStat
     *            the previous status of the object
     * @return String
     */
    private String getInformationalMessageDisplay(String action, String newStatus, String prevStat) {
        String msg = "";
        if (action.equals("Modify")) {
            msg = "message.us.information.requestor.submitted";
        } else if (action.equals("Approve")) {
            msg = "message.us.information.approver.approve";
        } else if (action.equals("Delete")) {
            msg = "message.us.information.approver.delete";
        } else if (action.equals("Reject")) {
            if (newStatus.equals("Approved")) {
                if (prevStat.equals("Modified")) {
                    msg = "message.us.information.approver.reject4";
                } else if (prevStat.equals("Reviewed")) {
                    msg = "message.us.information.approver.reject6";
                } else {
                    msg = "message.us.information.approver.reject2";
                }
            } else if (newStatus.equals("Deleted")) {
                if (prevStat.equals("Modified")) {
                    msg = "message.us.information.approver.reject5";
                } else {
                    msg = "message.us.information.approver.reject3";
                }
            } else {
                msg = "message.us.information.approver.reject";
            }
        } else if (action.equals("Submit_as_Reviewed")) {
            msg = "message.us.information.approver.submittedAsReviewed";
        } else if (action.equals("Submit_for_Delete")) {
            msg = "message.us.information.approver.submitfordelete";
        } else if (action.equals("Customize")) {
            msg = "message.us.information.requestor.submitted";
        }
        return msg;
    }

    /**
     * Validates the Double Value and the Unit Type associated with that value.
     * 
     * @param value
     *            the value to validate
     * @param unitType
     *            the unit type to validate
     * @param valueErrorMessageKey
     *            the key of the error message if the value is not valid
     * @param unitTypeErrorMessageKey
     *            the key of the error message if the unit type is not valid
     * @param bindingResult
     *            BindingResult
     */
    private void validateDoubleUnitType(Double value, String unitType, String valueErrorMessageKey,
            String unitTypeErrorMessageKey, BindingResult bindingResult) {

        DoseRangeValidation valueIsValid = isDecimalValid(value);
        boolean unitTypeIsValid = isUnitTypeValid(unitType);

        if (valueIsValid == DoseRangeValidation.VALID) {
            if (!unitTypeIsValid) {
                bindingResult.addError(new ObjectError("doseRangeDTO", findMessage(unitTypeErrorMessageKey)));
            }
        } else if (valueIsValid == DoseRangeValidation.INVALID) {
            bindingResult.addError(new ObjectError("doseRangeDTO", findMessage(valueErrorMessageKey)));
        }
    }

    /**
     * Validates the Long Value and the Unit Type associated with that value.
     * 
     * @param value
     *            the value to validate
     * @param unitType
     *            the unit type to validate
     * @param valueErrorMessageKey
     *            the key of the error message if the value is not valid
     * @param unitTypeErrorMessageKey
     *            the key of the error message if the unit type is not valid
     * @param bindingResult
     *            BindingResult
     */
    private void validateLongUnitType(Long value, String unitType, String valueErrorMessageKey, String unitTypeErrorMessageKey,
            BindingResult bindingResult) {

        DoseRangeValidation valueIsValid = isLongValidAndNonZero(value, true);
        boolean unitTypeIsValid = isUnitTypeValid(unitType);

        if (valueIsValid == DoseRangeValidation.VALID) {
            if (!unitTypeIsValid) {
                bindingResult.addError(new ObjectError("doseRangeDTO", findMessage(unitTypeErrorMessageKey)));
            }
        } else if (valueIsValid == DoseRangeValidation.INVALID) {
            bindingResult.addError(new ObjectError("doseRangeDTO", findMessage(valueErrorMessageKey)));
        }
    }

    /**
     * Checks to make sure the Long value is valid. If validatePrecision is true, then the number of digits is also checked.
     * 
     * @param num
     *            the Long to validate
     * @param validatePrecision
     *            true to validate precision, false otherwise
     * @return true if valid, false otherwise
     */
    private DoseRangeValidation isLongValidAndNonZero(Long num, boolean validatePrecision) {
        if (num != null && num.hashCode() != 0) {
            if (validatePrecision && num.toString().length() > LONG_PRECISION) {
                return DoseRangeValidation.INVALID;
            }
            return DoseRangeValidation.VALID;
        }
        return DoseRangeValidation.NULL_OR_ZERO;
    }
    
    /**
     * This method validates the length of a double number including the total number of digits including the number of digits
     * after the decimal point.
     * 
     * @param num
     *            The number to validate
     * @return True if the decimal number is valid, false if not.
     */
    private DoseRangeValidation isDecimalValid(Double num) {
        if (num != null && num.hashCode() != 0) {
            // Must handle 10 digits, which will be converted to scientific notation
            // by struts
            String snum = DOUBLE_FORMAT.format(num);

            if (logger.is   ugEnabled())
                logger.debug("Found decimal index at: " + snum.indexOf('.') + " in string: " + snum);

            if (snum.indexOf('.') > -1) {
                String scale = snum.substring(snum.indexOf('.') + 1);
                if (logger.is   ugEnabled())
                    logger.debug("Found substring after decimal of: " + scale + " having length of: " + scale.length());

                if (scale.length() > SCALE)
                    return DoseRangeValidation.INVALID;

                // Check if number plus decimal is greater than precision
                if (snum.length() > DOUBLE_PRECISION + 1)
                    return DoseRangeValidation.INVALID;
            } else {
                // The number does not have a decimal component, and therefore
                // cannot have total digits larger than the precision value.
                if (snum.length() > DOUBLE_PRECISION)
                    return DoseRangeValidation.INVALID;
            }
            return DoseRangeValidation.VALID;
        }
        return DoseRangeValidation.NULL_OR_ZERO;
    }

    /**
     * Checks to make sure the Unit Type is not null and is not empty.
     * 
     * @param unitType
     *            String
     * @return true if not null and not empty, false otherwise
     */
    private boolean isUnitTypeValid(String unitType) {
        if (unitType != null && !StringUtils.isEmpty(unitType)) {
            return true;
        }
        return false;
    }

    /**
     * Sets the DoseRangeService to use.
     * 
     * @param doseRangeService
     *            DoseRangeService
     */
    public final void setDoseRangeService(DoseRangeService doseRangeService) {
        this.doseRangeService = doseRangeService;
    }

    /**
     * Called by Spring for initialization after the bean has been created.
     * 
     * @throws Exception
     *             if a problem is encountered
     */
    @Override
    public final void afterPropertiesSet() throws Exception {
        if (doseRangeService == null) {
            throw new Exception(findMessage("message.resource.not.set", new Object[] { "DoseRangeService" }));
        }
        if (recordLockingHelper == null) {
            throw new Exception(findMessage("message.resource.not.set", new Object[] { "RecordLockingHelper" }));
        }
    }

    /**
     * Retrieves the Record Locking Helper.
     * 
     * @return RecordLockingHelper
     */
    public final RecordLockingHelper getRecordLockingHelper() {
        return recordLockingHelper;
    }

    /**
     * Sets the Record Locking Helper.
     * 
     * @param recordLockingHelper
     *            the recordLockingHelper to set
     */
    public final void setRecordLockingHelper(RecordLockingHelper recordLockingHelper) {
        this.recordLockingHelper = recordLockingHelper;
    }

    /**
     * Instead of using just a boolean to tell if a value is valid or not, there are actually three states we are interested in.
     * If a value is Valid, Invalid, or if it is Null/Empty/ equal to 0. So we use this enumeration in the place of a boolean.
     * 
     * @author vhaisaduffg
     * 
     */
    private enum DoseRangeValidation {

        /** The null or zero. */
        NULL_OR_ZERO,
        /** The invalid. */
        INVALID,
        /** The valid. */
        VALID
    }

    /**
     * Show comment button.
     * 
     * @param list
     *            the list
     * @return true, if successful
     */
    public boolean showCommentButton(List<DoseRangeDTO> list) {
        boolean showButton = true;
        if (list != null && list.size() > 0) {
            for (DoseRangeDTO doseRangeDTO : list) {
                if (!doseRangeDTO.getStatus().equalsIgnoreCase(Status.STATUS_DELETED)) {
                    showButton = false;
                    break;
                }
            }
        }
        return showButton;
    }
}
