/**
 *
 */
package gov.va.med.pharmacy.pecs.web.detail.drugpair;

import gov.va.med.pharmacy.pecs.domain.impl.va.UserProfile;
import gov.va.med.pharmacy.pecs.dto.DrugPairDTO;
import gov.va.med.pharmacy.pecs.web.RecordType;

import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.SessionAttributes;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.view.RedirectView;

/**
 * 
 * Controller for the 508 complaint drug pair customization page.
 * 
 * @author vhaisacarusr 
 * 
 */

@Controller
@RequestMapping(value = "/drugPairCustomization508")
@SessionAttributes({ "genericDrugPairsForm", "fdbDrugPairsForm" })
public class DrugPairCustomization508 extends AbstractDrugPairController {
    private static final String VIEW = "pecs.dp.customization.508";
    private static final String URL = "/ct/pecs/drugPairCustomization508/interaction/";

    /**
     * Displays the Drug Pair Customization page for an interaction ID.
     * 
     * @param interactionId
     *            The Interaction we are assigning the drug pairs to
     * @param modelMap
     *            the model object that holds the forms stored in the session
     * @param request
     *            the HttpServletRequest
     * @return a <code>ModelAndView</code> object
     */

    @RequestMapping(value = "/interaction/{interactionId}", method = RequestMethod.GET)
    public final ModelAndView displayDrugPairPage(@PathVariable(value = "interactionId") Long interactionId, ModelMap modelMap,
            HttpServletRequest request) {
        ModelAndView modelAndView = new ModelAndView(VIEW);
        modelAndView.addAllObjects(modelMap);
        UserProfile profile = getUserProfile(request);
        prepareModelAndViewForDisplay(interactionId, modelAndView, profile);

        return modelAndView;
    }

    /**
     * 
     * Adds Generic drug pairs to the interaction.
     * 
     * @param genericDrugPairsForm
     *            the form that holds the generic drug pairs
     * @param bindingResult
     *            holds the errors returned from the service
     * @param request
     *            the HTTP request
     * @return a model and view object
     */
    @RequestMapping(value = "/save/genericDrugPairs", method = RequestMethod.POST)
    public final ModelAndView addGenericDrugPairs(HttpServletRequest request,
            @ModelAttribute("genericDrugPairsForm") GenericDrugPairsForm genericDrugPairsForm, BindingResult bindingResult) {
        ModelAndView modelAndView = new ModelAndView(VIEW);

        validateInteractionStatus(genericDrugPairsForm.getInteractionId(), bindingResult);
        if (bindingResult.hasErrors()) {
            modelAndView.addObject("genericDrugPairsForm", genericDrugPairsForm);
            prepareModelAndViewForDisplay(genericDrugPairsForm.getInteractionId(), modelAndView, getUserProfile(request));
            return modelAndView;
        }

        DrugPairDTO genericDrugPair =
                createNewDrugPair(genericDrugPairsForm.getRoutedGenericId1(), genericDrugPairsForm.getRoutedGenericId2(),
                        genericDrugPairsForm.getInteractionId(), getUserProfile(request), genericDrugPairsForm.getAssignedTo(),
                        genericDrugPairsForm.getCurrentActionReason(), genericDrugPairsForm.getReferenceText());

        getDrugPairService().createNewRecord(genericDrugPair, bindingResult);

        if (bindingResult.hasErrors()) {
            modelAndView.addObject("genericDrugPairsForm", genericDrugPairsForm);
            prepareModelAndViewForDisplay(genericDrugPairsForm.getInteractionId(), modelAndView, getUserProfile(request));
            return modelAndView;
        }

        RedirectView redirectView = new RedirectView(URL + genericDrugPair.getInteractionid(), false, false, false);
        modelAndView.setView(redirectView);
        cancelLock(RecordType.VA.toString(), genericDrugPairsForm.getInteractionId(), getUserProfile(request));
        return modelAndView;
    }

    /**
     * Adds the FDB drug pairs to the interaction that are supplied in the form.
     * 
     * @param fdbDrugPairsForm
     *            the form which holds the drug pairs
     * @param bindingResult
     *            holds the errors from the service
     * @param request
     *            the HTTP request
     * @return a model and view object
     */
    @RequestMapping(value = "/save/fdbDrugPairs", method = RequestMethod.POST)
    public final ModelAndView addFdbDrugPairs(HttpServletRequest request,
            @ModelAttribute("fdbDrugPairsForm") FdbDrugPairsForm fdbDrugPairsForm, BindingResult bindingResult) {
        ModelAndView modelAndView = new ModelAndView(VIEW);

        validateInteractionStatus(fdbDrugPairsForm.getInteractionId(), bindingResult);
        if (bindingResult.hasErrors()) {
            modelAndView.addObject("fdbDrugPairsForm", fdbDrugPairsForm);
            prepareModelAndViewForDisplay(fdbDrugPairsForm.getInteractionId(), modelAndView, getUserProfile(request));
            return modelAndView;
        }

        List<DrugPairDTO> fdbDrugPairs = convertFormToDrugPairs(getUserProfile(request), fdbDrugPairsForm);
        getDrugPairService().createNewRecords(fdbDrugPairs, bindingResult);
        if (bindingResult.hasErrors()) {
            modelAndView.addObject("fdbDrugPairsForm", fdbDrugPairsForm);
            prepareModelAndViewForDisplay(fdbDrugPairsForm.getInteractionId(), modelAndView, getUserProfile(request));
            return modelAndView;
        }
        RedirectView view = new RedirectView(URL + fdbDrugPairsForm.getInteractionId(), false, false, false);
        modelAndView.setView(view);
        cancelLock(RecordType.VA.toString(), fdbDrugPairsForm.getInteractionId(), getUserProfile(request));
        return modelAndView;
    }

    /**
     * This method processes the form submitted by the user to apply an action to custom drug pairs.
     * 
     * @param drugPairModificationForm
     *            the for that holds the custom drug pairs and pertinent information
     * @param bindingResult
     *            Holds the errors after processing
     * @param request
     *            the incoming HTTP request
     * @param action
     *            the action to be applied
     * @return the model and view
     */
    @RequestMapping(value = "/save/modifyDrugPairs", method = RequestMethod.POST)
    public final ModelAndView modifyDrugPairs(HttpServletRequest request, @RequestParam("wizardButton") String action,
            @ModelAttribute("drugPairModificationForm") DrugPairModificationForm drugPairModificationForm,
            BindingResult bindingResult) {
        ModelAndView modelAndView = new ModelAndView(VIEW);

        action = formatAction(action);
        if (!isRejectOrDeleteAction(action)) {
            validateInteractionStatus(drugPairModificationForm.getInteractionId(), bindingResult);
        }
        if (bindingResult.hasErrors()) {
            modelAndView.addObject("drugPairModificationForm", drugPairModificationForm);
            prepareModelAndViewForDisplay(drugPairModificationForm.getInteractionId(), modelAndView, getUserProfile(request));
            return modelAndView;
        }

        List<DrugPairDTO> drugPairs = createModifiableDrugPairs(drugPairModificationForm, getUserProfile(request), action);
        getDrugPairService().applyAction(action, drugPairs, bindingResult);
        if (bindingResult.hasErrors()) {
            prepareModelAndViewForDisplay(drugPairModificationForm.getInteractionId(), modelAndView, getUserProfile(request));

            return modelAndView;
        }
        RedirectView view = new RedirectView(URL + drugPairModificationForm.getInteractionId(), false, false, false);
        modelAndView.setView(view);
        cancelLock(RecordType.VA.toString(), drugPairModificationForm.getInteractionId(), getUserProfile(request));
        return modelAndView;
    }

}
