package gov.va.med.pharmacy.pecs.web.detail.drugpair;

import gov.va.med.pharmacy.pecs.domain.impl.va.DrugDrugInteractionVa;
import gov.va.med.pharmacy.pecs.domain.impl.va.UserProfile;
import gov.va.med.pharmacy.pecs.dto.DrugDrugInteractionDTO;
import gov.va.med.pharmacy.pecs.dto.DrugPairDTO;
import gov.va.med.pharmacy.pecs.web.detail.AjaxMessageResponse;
import gov.va.med.pharmacy.pecs.web.detail.LockingResult;

import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.InitializingBean;
import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.SessionAttributes;
import org.springframework.web.servlet.ModelAndView;

/**
 * 
 * The controller class for the drug pair customization page.
 * 
 * @author vhaisacarusr 
 * 
 */
@Controller
@RequestMapping(value = "/drugPairCustomization")
@SessionAttributes({ "routedGenericsForm", "fdbDrugPairsForm" })
public class DrugPairCustomizationController extends AbstractDrugPairController implements InitializingBean {

    /** The Constant VIEW. */
    private static final String VIEW = "pecs.drugpaircustomization";

    /**
     * This method prepares for the loading of the drug pair customization page.
     * 
     * @param request
     *            the incoming request
     * @param interactionId
     *            the interaction id for the VA drug drug interaction
     * @param modelMap
     *            the model object that holds the forms stored in the session
     * @return a model and view for the page
     */
    @RequestMapping(value = "/interaction/{interactionId}", method = RequestMethod.GET)
    public final ModelAndView load(HttpServletRequest request, @PathVariable("interactionId") Long interactionId,
            ModelMap modelMap) {
        ModelAndView modelAndView = new ModelAndView(VIEW);
        modelAndView.addAllObjects(modelMap);
        prepareModelAndViewForDisplay(interactionId, modelAndView, getUserProfile(request));
        return modelAndView;
    }

    /**
     * Persists the FDB drug pairs selection.
     * 
     * @param request
     *            the incoming request
     * @param fdbDrugPairsForm
     *            the HTML form passed in
     * @param bindingResult
     *            the binding result for any errors
     * @return some object. not sure what yet
     */
    @ResponseBody
    @RequestMapping(value = "/fdbDrugPairs", method = RequestMethod.POST)
    public final AjaxMessageResponse saveFdbDrugPairs(HttpServletRequest request,
            @ModelAttribute("fdbDrugPairsForm") FdbDrugPairsForm fdbDrugPairsForm, BindingResult bindingResult) {
        AjaxMessageResponse ajaxResponse = new AjaxMessageResponse();

        validateInteractionStatus(fdbDrugPairsForm.getInteractionId(), bindingResult);

        if (bindingResult.hasErrors()) {
            ajaxResponse.setErrorMessages(convertResultToErrors(bindingResult));
            return ajaxResponse;
        }

        List<DrugPairDTO> drugPairs = convertFormToDrugPairs(getUserProfile(request), fdbDrugPairsForm);

        getDrugPairService().createNewRecords(drugPairs, bindingResult);

        if (bindingResult.hasErrors()) {
            ajaxResponse.setErrorMessages(convertResultToErrors(bindingResult));
        } else {
            ajaxResponse.setSuccessMessage("Successfully Saved " + drugPairs.size() + " Drug Pairs");
            cancelLock(RECORD_TYPE, fdbDrugPairsForm.getInteractionId(), request);
        }
        return ajaxResponse;
    }

    /**
     * Retrieves a list of routed generic drugs and returns it as the response body as JSON.
     * 
     * @return a list of routed generic drugs
     */
    @RequestMapping(value = "/routedGenerics", method = RequestMethod.GET)
    @ResponseBody
    public final ItemFileReadStore retrieveRoutedGenerics() {

        return new ItemFileReadStore(getDrugPairService().getRoutedGenericDrugsLite());
    }

    /**
     * Retrievers a list of routed generic drugs and returns it as the response body as JSON.
     * 
     * @param request
     *            the HTTP request
     * @param routedGenericsForm
     *            the form that holds the routed generic
     * @param bindingResult
     *            the binding result with errors.
     * @return a model and view for the page
     */
    @ResponseBody
    @RequestMapping(value = "/routedGenerics", method = RequestMethod.POST)
    public final AjaxMessageResponse saveRoutedGenerics(HttpServletRequest request,
            @ModelAttribute("routedGenericsForm") RoutedGenericsForm routedGenericsForm, BindingResult bindingResult) {
        AjaxMessageResponse ajaxResponse = new AjaxMessageResponse();

        validateInteractionStatus(routedGenericsForm.getInteractionId(), bindingResult);

        if (bindingResult.hasErrors()) {
            ajaxResponse.setErrorMessages(convertResultToErrors(bindingResult));
            return ajaxResponse;
        }

        UserProfile profile = getUserProfile(request);
        List<DrugPairDTO> drugPairs = new ArrayList<DrugPairDTO>();

        for (Long id : routedGenericsForm.getRoutedGeneric1()) {
            for (Long id2 : routedGenericsForm.getRoutedGeneric2()) {
                DrugPairDTO drugPair =
                        createNewDrugPair(id, id2, routedGenericsForm.getInteractionId(), profile,
                                routedGenericsForm.getAssignedTo(), routedGenericsForm.getCurrentActionReason(),
                                routedGenericsForm.getReferenceText());
                drugPairs.add(drugPair);

            }
        }
        getDrugPairService().createNewRecords(drugPairs, bindingResult);
        if (bindingResult.hasErrors()) {
            extendLock(RECORD_TYPE, routedGenericsForm.getInteractionId(), request);
            // request.setAttribute("drugPairSource", "generic");
            ajaxResponse.setErrorMessages(convertResultToErrors(bindingResult));
            if (drugPairs.size() != bindingResult.getErrorCount()) {
                cancelLock("VA", routedGenericsForm.getInteractionId(), request);
                ajaxResponse.setSuccessMessage("Added " + (drugPairs.size() - bindingResult.getErrorCount()) + " drug pairs");
            }
            return ajaxResponse;

        }
        cancelLock("VA", routedGenericsForm.getInteractionId(), request);
        ajaxResponse.setSuccessMessage("Added " + drugPairs.size() + " drug pairs");
        return ajaxResponse;
    }

    /**
     * Attempts to change the state of the custom drug pairs selected on the page.
     * 
     * @param request
     *            the incoming request
     * @param modelMap
     *            holds the forms stored in the session
     * @param drugPairModificationForm
     *            The form that was submitted by the user
     * @param bindingResult
     *            hold any errors found during processing
     * @param action
     *            the action to be performed
     * @return the model and view
     */
    @ResponseBody
    @RequestMapping(value = "/save/modifyDrugPairs", method = RequestMethod.POST)
    public final AjaxMessageResponse performAction(HttpServletRequest request, ModelMap modelMap,
            @ModelAttribute DrugPairModificationForm drugPairModificationForm, BindingResult bindingResult,
            @RequestParam("wizardButton") String action) {

        AjaxMessageResponse ajaxResponse = new AjaxMessageResponse();

        action = formatAction(action);

        if (!isRejectOrDeleteAction(action)) {
            validateInteractionStatus(drugPairModificationForm.getInteractionId(), bindingResult);
        }

        if (bindingResult.hasErrors()) {
            ajaxResponse.setErrorMessages(convertResultToErrors(bindingResult));
            return ajaxResponse;
        }

        List<DrugPairDTO> modifiedDdims = null;
        DrugDrugInteractionVa interaction =
                getDrugDrugInteractionService().getDDIByInteractionId(drugPairModificationForm.getInteractionId());
        modifiedDdims =
                populateModifiableDrugPairs(drugPairModificationForm, getUserProfile(request), interaction.getSeverityLevel()
                        .getCode(), action);

        getDrugPairService().applyAction(action, modifiedDdims, bindingResult);

        if (!bindingResult.hasErrors()) {
            ajaxResponse.setSuccessMessage("Successfully Saved " + modifiedDdims.size() + " Drug Pairs");
            cancelLock("VA", drugPairModificationForm.getInteractionId(), request);
        } else {
            ajaxResponse.setErrorMessages(convertResultToErrors(bindingResult));
            extendLock("VA", drugPairModificationForm.getInteractionId(), request);
        }

        return ajaxResponse;
    }

    /**
     * Locks the Drug Drug interaction.
     * 
     * @param recordType
     *            VA or FDB
     * @param recordId
     *            the id of the record
     * @param request
     *            the incoming request used to get the user profile
     * @return <code>LockingResult</code> holds the locking information for the record
     */
    @RequestMapping(value = "/edit/{recordType}/{recordId}", method = RequestMethod.GET)
    @ResponseBody
    public final LockingResult lockRecord(@PathVariable(value = "recordType") String recordType, @PathVariable(
            value = "recordId") Long recordId, HttpServletRequest request) {
        UserProfile userProfile = getUserProfile(request);
        return lockRecord(recordType, recordId, userProfile);

    }

    /**
     * Cancels the lock on the interaction.
     * 
     * @param recordType
     *            VA or FDB
     * @param recordId
     *            the id of the record
     * @param request
     *            the incoming request used to get the user profile
     * @return error message if there is an error the empty string if there is none
     */
    @RequestMapping(value = "/cancel/{recordType}/{recordId}", method = RequestMethod.GET)
    @ResponseBody
    public final String cancelLock(@PathVariable(value = "recordType") String recordType,
            @PathVariable(value = "recordId") Long recordId, HttpServletRequest request) {
        UserProfile userProfile = getUserProfile(request);
        String response = cancelLock(recordType, recordId, userProfile);

        return response;
    }

    /**
     * Extends the lock on the interaction.
     * 
     * @param recordType
     *            VA or FDB
     * @param recordId
     *            the id of the record
     * @param request
     *            the incoming request used to get the user profile
     * @return error message if there is an error the empty string if there is none
     */
    @RequestMapping(value = "/extend/{recordType}/{recordId}", method = RequestMethod.GET)
    @ResponseBody
    public final String extendLock(@PathVariable(value = "recordType") String recordType,
            @PathVariable(value = "recordId") Long recordId, HttpServletRequest request) {
        UserProfile userProfile = getUserProfile(request);
        extendLock(recordType, recordId, userProfile);
        return "";
    }

    /**
     * Populates the DataGrid with the drug pairs.
     * 
     * @param interactionId
     *            the custom interaction id
     * @param json
     *            not used there is no paging
     * @return a list of FDB drug pairs to be displayed in the grid
     */
    @RequestMapping(value = "/FDB/drugpairs/{interactionId}", method = RequestMethod.GET)
    @ResponseBody
    public final List<DrugPairsGridBean> retrieveFDBDrugPairs(@PathVariable(value = "interactionId") Long interactionId,
            @RequestParam("json") String json) {

        DrugDrugInteractionDTO interaction = getDrugDrugInteractionService().getStagingByInteractionId(interactionId);
        List<DrugPairDTO> fdbDrugPairs = getDrugPairService().getFilteredFdbData(interaction.getFdbId());
        List<DrugPairsGridBean> drugPairRows = new ArrayList<DrugPairsGridBean>();
        for (DrugPairDTO drugpair : fdbDrugPairs) {
            DrugPairsGridBean drugPairsGridBean =
                    new DrugPairsGridBean(drugpair.getInteractionid(), drugpair.getRtgenid1(), drugpair.getRtgenid2(),
                            drugpair.getRtgenid1Desc(), drugpair.getRtgenid2Desc());
            drugPairRows.add(drugPairsGridBean);
        }
        FdbDrugPairSortSupport sortSupport = new FdbDrugPairSortSupport(drugPairRows, json);

        return sortSupport.sort();
    }

    /**
     * Populates the DataGrid with the drug pairs.
     * 
     * @param interactionId
     *            the custom interaction id
     * @param json
     *            not used there is no paging
     * @param request
     *            the incoming request used to get the user profile
     * @param response
     *            the response
     * @return a list of FDB drug pairs to be displayed in the grid
     */
    @RequestMapping(value = "/VA/drugpairs/{interactionId}", method = RequestMethod.GET)
    @ResponseBody
    public final List<DrugPairDTO> retrieveVADrugPairs(@PathVariable(value = "interactionId") Long interactionId,
            @RequestParam("json") String json, HttpServletRequest request, HttpServletResponse response) {

        List<DrugPairDTO> drugPairList =
                getDrugPairService().getDrugPairsForInteraction(interactionId, getUserProfile(request));
        String range = "items 0-" + (drugPairList.size() - 1) + "/" + drugPairList.size();
        response.setHeader("Content-Range", range);
        return drugPairList;

    }
}
