package gov.va.med.pharmacy.pecs.web.detail.drugpair;

import gov.va.med.pharmacy.pecs.domain.impl.fdb.RoutedGenericFdbLite;
import gov.va.med.pharmacy.pecs.domain.impl.va.UserProfile;
import gov.va.med.pharmacy.pecs.dto.DrugDrugInteractionDTO;
import gov.va.med.pharmacy.pecs.dto.DrugPairDTO;
import gov.va.med.pharmacy.pecs.exception.CustomizationServiceException;
import gov.va.med.pharmacy.pecs.exception.ValidationException;
import gov.va.med.pharmacy.pecs.metadata.ConceptField;
import gov.va.med.pharmacy.pecs.metadata.ConceptMetadata;
import gov.va.med.pharmacy.pecs.service.DrugDrugInteractionService;
import gov.va.med.pharmacy.pecs.service.DrugPairService;
import gov.va.med.pharmacy.pecs.service.MetadataService;
import gov.va.med.pharmacy.pecs.state.Status;
import gov.va.med.pharmacy.pecs.web.AbstractController;
import gov.va.med.pharmacy.pecs.web.detail.drugdrug.LabelValue;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.stereotype.Controller;
import org.springframework.validation.BindingResult;
import org.springframework.validation.ObjectError;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.servlet.ModelAndView;

/**
 * Controller to display the drug pair detail page.
 * 
 * @author vhaisacarusr
 * 
 */
@Controller
@RequestMapping(value = "/drugPairDetail")
public final class DrugPairDetailController extends AbstractController {

    /** The Constant LOGGER. */
    private static final Log LOGGER = LogFactory.getLog(DrugPairDetailController.class);

    /** The Constant VIEW. */
    private static final String VIEW = "pecs.drugpairdetail.page";

    /** The Constant VIEW. */
    private static final String FDB_VIEW = "pecs.fdbdrugpairdetail.page";

    /** The drug pair service. */
    @Resource
    private DrugPairService drugPairService;

    /** The ddi service. */
    @Resource
    private DrugDrugInteractionService ddiService;

    /** The metadata service. */
    @Resource
    private MetadataService metadataService;

    /**
     * Displays the FDB record for the given FDB record ID if there is an active Custom Record for that FDB Record you will be
     * forwarded to that record.
     * 
     * @param recordId
     *            the record ID for the FDB drug pair
     * @param request
     *            the HTTP Request
     * @param response
     *            the HTTP Response
     * @return a model and view for the page
     */
    @RequestMapping(value = "/FDB/{recordId}", method = RequestMethod.GET)
    public ModelAndView displayDrugPair(@PathVariable(value = "recordId") String recordId, HttpServletRequest request,
            HttpServletResponse response) {
        ModelAndView mav = new ModelAndView(FDB_VIEW);

        DrugPairDTO activeCustomDrugPairDTO = null;
        DrugPairKeyHelper helper = new DrugPairKeyHelper(recordId);
        Long fdbInteractionId = helper.getInteractionId();
        DrugPairDTO fdbDrugPairDTO =
                getDrugPairService().getFdbData(fdbInteractionId, helper.getRoutedGenericId1(), helper.getRoutedGenericId2());

        List<DrugPairDTO> customDrugPairs =
                getDrugPairService().findDrugPairsForFDBInteraction(helper.getRoutedGenericId1(), helper.getRoutedGenericId2(),
                        fdbInteractionId);

        if (customDrugPairs.size() > 0) {
            determineMessagesForDeletedOrRejectedDrugPairs(customDrugPairs);

            activeCustomDrugPairDTO = determineActiveCustomDrugPair(customDrugPairs);

            if (activeCustomDrugPairDTO != null) {
                addMessageDPOnOtherDDI(activeCustomDrugPairDTO);
            }
            addMessageDPPreviouslyCustomized();
            
        } else {
            List<DrugDrugInteractionDTO> customInteractions = ddiService.getStagingByFdbId(fdbInteractionId);
            if (customInteractions != null && customInteractions.size() > 0) {
                addMessageDPNeverCustomized(customInteractions);
            } else {
                addMessageDDINeverCustomized(fdbInteractionId, fdbDrugPairDTO);
                mav.addObject("recordIdAll", recordId);
                mav.addObject("useOpenButtons", true);
            }
        }

        setupFDBForDisplay(request, fdbDrugPairDTO, mav);
        return mav;
    }

    /**
     * Finds the DrugPairDTO in the specified list that is not deleted or rejected, or returns null if none found.
     * 
     * @param pairs
     *            List of custom DrugPairDTOs to search
     * @return The Active Custom DrugPairDTO or null if none in the specified list are active
     */
    private DrugPairDTO determineActiveCustomDrugPair(List<DrugPairDTO> pairs) {
        DrugPairDTO drugPairDTO = null;
        for (DrugPairDTO customDrugPair : pairs) {
            // if not deleted or rejected, then it is the current one, since
            // only one active is allowed at a time.
            if (!customDrugPair.getStatus().equals(Status.STATUS_REJECTED)
                    && !customDrugPair.getStatus().equals(Status.STATUS_DELETED)) {
                drugPairDTO = customDrugPair;
            }
        }
        return drugPairDTO;
    }

    /**
     * Adds informational messages for custom drug pairs which are deleted or rejected.
     * 
     * @param pairs
     *            the pairs
     */
    private void determineMessagesForDeletedOrRejectedDrugPairs(List<DrugPairDTO> pairs) {
        for (DrugPairDTO customDrugPair : pairs) {
            if (customDrugPair.getStatus().equals(Status.STATUS_REJECTED)
                    || customDrugPair.getStatus().equals(Status.STATUS_DELETED)) {
                addMessageDPOnOtherDDI(customDrugPair);
               
            }
        }
    }

    /**
     * Adds an informational messages for a custom drug pair which is on another interaction.
     * 
     * @param customDrugPair
     *            a custom drug pair that is on another interaction
     */
    private void addMessageDPOnOtherDDI(DrugPairDTO customDrugPair) {
        String msg =
                "The selected drug pair is also associated with VA Custom Interaction "
                        + buildDDILinkFromDrugPair(customDrugPair) + " with severity level "
                        + customDrugPair.getSeveritylevelcode() + " and in the " + customDrugPair.getStatus()
                        + " action status.";
        addInformationMessage(msg);
    }

    /**
     * Builds a link to the interaction based on the drug pair specified.
     * 
     * @param drugPairDTO - the drug pair dto used to create the hyperlink.
     * @return String - the string containing the hyperlink.
     */
    private String buildDDILinkFromDrugPair(DrugPairDTO drugPairDTO) {
        String url = "../../drugdrug/interactionid/" + drugPairDTO.getInteractionid();
        String display = drugPairDTO.getInteractionid() + " - " + drugPairDTO.getInteractionidDesc();
        return "<a href='" + url + " ' >" + display + "</a>";
    }

    /**
     * Build a hyperlink for a drug drug interaction using the drug drug interaction DTO.
     * 
     * @param drugDrugInteractionDTO
     *            - the drug drug interaction dto.
     * @return String - the string containing the hyperlink
     */
    private String buildDDILinkFromInteraction(DrugDrugInteractionDTO drugDrugInteractionDTO) {
        String url = "../../drugdrug/interactionid/" + drugDrugInteractionDTO.getInteractionid();
        String display = drugDrugInteractionDTO.getInteractionid() + " - " + drugDrugInteractionDTO.getDescription1();
        return "<a href='" + url + " ' >" + display + "</a>";
    }

    /**
     * Adds a message for when the Drug Drug Interaction has never been customized.
     * 
     * @param fdbInteractionId
     *            - the interaction id used to build the message
     * @param drugPairDTO
     *            - the Drug Pair Dto used to build the message.
     */
    private void addMessageDDINeverCustomized(Long fdbInteractionId, DrugPairDTO drugPairDTO) {
        String message =
                "The Drug-Drug interaction '" + fdbInteractionId + " - " + drugPairDTO.getInteractionidDesc()
                        + "' has not been customized.";
        addInformationMessage(message);
        message = "You must customize the Drug-Drug interaction prior to customizing the Drug-Drug pair.";
        addInformationMessage(message);
        message = "To customize this Drug-Drug interaction click the Open Button below.";
        addInformationMessage(message);
    }

    /**
     * Adds a message when the Drug Pair has never been customized.
     * 
     * @param interactionList
     *            - the list of interactions
     */
    private void addMessageDPNeverCustomized(List<DrugDrugInteractionDTO> interactionList) {
        addInformationMessage("The selected Drug Pair has not been customized.");
        
        addInformationMessage("The FDB Drug-Drug Interaction is associated with "
                + "the following VA Custom Drug-Drug Interaction(s): ");
        for (DrugDrugInteractionDTO customDdim : interactionList) {
            StringBuffer inf = new StringBuffer();
            inf.append("Interaction:  ");
            inf.append(buildDDILinkFromInteraction(customDdim));
            inf.append(" - Severity code: ");
            inf.append(customDdim.getSeveritylevelcode());
            addInformationMessage(inf.toString());
        }

        addInformationMessage("Customization of this Drug Pair can only be done through the "
                + "VA custom Drug-Drug Interaction detail page.");
    }
    
    /**
     * Add a message for when the Drug Pair has previously been customized.
     * 
     */
    private void addMessageDPPreviouslyCustomized (){
        
    addInformationMessage( "Further customization or deletion of this drug pair can only be done " 
            + "through the VA custom Drug-Drug Interaction detail page.");
    }
   

    /**
     * Builds the rtgen list.
     * 
     * @return the list
     */
    private List<LabelValue> buildRtgenList() {
        List<RoutedGenericFdbLite> rtgens = getDrugPairService().getRoutedGenericDrugsLite();
        rtgens.get(0).getDescription();
        List<String> rtgenIdDesc = new ArrayList<String>();
        for (int i = 0; i < rtgens.size(); i++) {
            rtgenIdDesc.add(Long.toString(rtgens.get(i).getIdentifier()));
            rtgenIdDesc.add(rtgens.get(i).getDescription());
        }
        List<LabelValue> drugList = new ArrayList<LabelValue>();
        for (int i = 0; i < (rtgenIdDesc.size() - 1); ++i) {
            LabelValue labelValue = new LabelValue(rtgenIdDesc.get(i + 1), rtgenIdDesc.get(i));
            drugList.add(labelValue);
            i++;
        }
        return drugList;
    }

    /**
     * Gets the va interactions.
     * 
     * @param drugPairDTO
     *            the drug pair dto
     * @return the va interactions
     */
    private List<LabelValue> getVaInteractions(DrugPairDTO drugPairDTO) {
        List<DrugDrugInteractionDTO> purelyCustomInteractions =
                ddiService.getStagingListByInteractionId(drugPairDTO.getInteractionid());
        List<String> interactionDescs = new ArrayList<String>();
        for (int i = 0; i < purelyCustomInteractions.size(); i++) {
            interactionDescs.add(Long.toString(purelyCustomInteractions.get(i).getInteractionid()));
            interactionDescs.add(Long.toString(purelyCustomInteractions.get(i).getInteractionid()) + " - "
                    + purelyCustomInteractions.get(i).getDescription1());
        }
        List<LabelValue> interactionList = new ArrayList<LabelValue>();
        for (int i = 0; i < (interactionDescs.size() - 1); ++i) {
            LabelValue labelValue = new LabelValue(interactionDescs.get(i + 1), interactionDescs.get(i));
            interactionList.add(labelValue);
            i++;
        }
        return interactionList;
    }

    /**
     * Load va.
     * 
     * @param recordId
     *            the record id
     * @param request
     *            the request
     * @param response
     *            the response
     * @return the model and view
     */
    @RequestMapping(value = "/VA/{recordId}", method = RequestMethod.GET)
    public ModelAndView loadVa(@PathVariable(value = "recordId") String recordId, HttpServletRequest request,
            HttpServletResponse response) {
        if (LOGGER.is   ugEnabled()) {
            LOGGER.debug("DrugPairDetailController loadVa(): recordid = " + recordId);
        }
        ModelAndView mav = new ModelAndView(VIEW);

        DrugPairDTO drugPairDTO = new DrugPairDTO();
        try {
            if (recordId != null && !"".equals(recordId)) {
                drugPairDTO = getDrugPairService().getCustomModel(Long.valueOf(recordId));
            }
        } catch (Exception e) {
            LOGGER.error("DrugPairDetailController loadVa(): exception = " + e);
        }
        mav = setupForDisplay(request, drugPairDTO, mav);
        return mav;
    }

    /**
     * Save.
     * 
     * @param recordId
     *            the record id
     * @param request
     *            the request
     * @param drugPairDTO
     *            the drug pair dto
     * @param bindingResult
     *            the binding result
     * @return the drug pair dto
     */
    public DrugPairDTO save(String recordId, HttpServletRequest request, DrugPairDTO drugPairDTO, BindingResult bindingResult) {

        try {
            UserProfile profile = getUserProfile(request);
            drugPairDTO.setRequestor(profile.getUsername());
            drugPairDTO.setActionBy(profile.getUsername());
            if (profile.isApprover() && drugPairDTO.getAssignedTo() == null)
                drugPairDTO.setAssignedTo(profile.getUsername());
            if (drugPairDTO.getAssignedTo() == null || drugPairDTO.getAssignedTo().equals(""))
                drugPairDTO.setAssignedTo("UNASSIGNED");
            drugPairDTO = getDrugPairService().createNewRecord(drugPairDTO, bindingResult);
            drugPairDTO.setCurrentActionReason("");
            String message =
                    "This custom  record has been successfully submitted and will " + "be reviewed at the national level.";
            if (!bindingResult.hasErrors()) {
                addInformationMessage(message);
            }
        } catch (ValidationException ex) {
            LOGGER.error("Data validation error", ex);
            bindingResult.addError(new ObjectError("drugPairDTO", findMessage("us.exception.create")));
        } catch (CustomizationServiceException ex) {
            LOGGER.error("Unable to save ddim models: ", ex);
            bindingResult.addError(new ObjectError("drugPairDTO", findMessage("error.save")));
        }
        return drugPairDTO;
    }

    /**
     * Setup for display.
     * 
     * @param request
     *            the request
     * @param drugPairDTO
     *            the drug pair dto
     * @param view
     *            the view
     * @return the model and view
     */
    private ModelAndView setupForDisplay(HttpServletRequest request, DrugPairDTO drugPairDTO, ModelAndView view) {
        ConceptMetadata conceptMetadata = getDrugPairService().getMetadata();
        Map<String, ConceptField> fieldMap = conceptMetadata.getFieldMap();
        view.addObject("disableInteractionId", true);
        view.addObject("disableRtgenId1", true);
        view.addObject("disableRtgenId2", true);
        view.addObject("drugPairDTO", drugPairDTO);
        view.addObject("fieldMap", fieldMap);
        List<LabelValue> drugList = buildRtgenList();
        view.addObject("drugs", drugList);

        List<String> userList = getAdminUsers();
        view.addObject("users", userList);
        view.addObject("interactionList", getVaInteractions(drugPairDTO));
        view.addObject("allowableActions", Collections.emptySet());
        view.addObject("showDetail", "true");

        if (!drugPairDTO.isReadOnlyRecord()) {
            if (drugPairDTO.getFdbCustomDdimVaId() != null && drugPairDTO.getFdbCustomDdimVaId() != 0) {
                try {
                    view.addObject("allowableActions",
                            getDrugPairService().getAllowableActions(drugPairDTO, getUserProfile(request)));
                } catch (Exception e) {
                    LOGGER.error("Cannot determine allowable actions", e);
                    findMessage("ddim.wizard.required.recordid");
                }
            } else {

                view.addObject("allowableActions", Collections.singleton("Customize"));
            }
        }
        if (!drugPairDTO.isReadOnlyRecord()) {
            String msg =
                    "Further customization or deletion of this drug pair can only be done through the"
                            + " VA custom Drug-Drug Interaction detail page.";
            addInformationMessage(msg);
        }
        return view;
    }

    /**
     * Setup for display.
     * 
     * @param request
     *            the request
     * @param drugPairDTO
     *            the drug pair dto
     * @param view
     *            the view
     * @return the model and view
     */
    private ModelAndView setupFDBForDisplay(HttpServletRequest request, DrugPairDTO drugPairDTO, ModelAndView view) {
        ConceptMetadata conceptMetadata = getDrugPairService().getMetadata();
        Map<String, ConceptField> fieldMap = conceptMetadata.getFieldMap();
        view.addObject("fieldMap", fieldMap);
        view.addObject("drugPairDTO", drugPairDTO);
        view.addObject("allowableActions", Collections.emptySet());
        view.addObject("showDetail", "true");
        return view;
    }

   

    /**
     * Gets the drugPairService.
     * 
     * @return the drugPairService
     */
    public DrugPairService getDrugPairService() {
        return drugPairService;
    }

    /**
     * Sets the drugPairService.
     * 
     * @param drugPairService
     *            the drugPairService to set
     */
    public void setDrugPairService(DrugPairService drugPairService) {
        this.drugPairService = drugPairService;
    }

}
