package gov.va.med.pharmacy.pecs.web.detail.duplicatetherapy;

import gov.va.med.pharmacy.pecs.domain.impl.lookup.DuplicateAllowance;
import gov.va.med.pharmacy.pecs.domain.impl.va.UserProfile;
import gov.va.med.pharmacy.pecs.dto.DuplicateTherapyDTO;
import gov.va.med.pharmacy.pecs.dto.PecsDTO;
import gov.va.med.pharmacy.pecs.exception.CustomizationReportException;
import gov.va.med.pharmacy.pecs.exception.CustomizationServiceException;
import gov.va.med.pharmacy.pecs.metadata.Concept;
import gov.va.med.pharmacy.pecs.metadata.ConceptField;
import gov.va.med.pharmacy.pecs.metadata.ConceptMetadata;
import gov.va.med.pharmacy.pecs.service.DuplicateTherapyService;
import gov.va.med.pharmacy.pecs.service.MetadataService;
import gov.va.med.pharmacy.pecs.service.TerminologyService;
import gov.va.med.pharmacy.pecs.state.Status;
import gov.va.med.pharmacy.pecs.web.AbstractController;
import gov.va.med.pharmacy.pecs.web.IConstants;
import gov.va.med.pharmacy.pecs.web.RecordType;
import gov.va.med.pharmacy.pecs.web.detail.LockingResult;
import gov.va.med.pharmacy.pecs.web.detail.RecordLockingHelper;
import gov.va.med.pharmacy.pecs.web.view.ReportHistoryXLSXViewer;
import gov.va.med.pharmacy.pecs.web.view.document.AbstractXLSXViewer;

import java.io.IOException;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.stereotype.Controller;
import org.springframework.validation.BindingResult;
import org.springframework.validation.ObjectError;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.View;
import org.springframework.web.servlet.view.RedirectView;

/**
 * Controller to handle Duplicate Therapy UI interactions.
 * 
 * @author vhaisaduffg
 * 
 */
@Controller
@RequestMapping(value = "/duplicateTherapyCustomization")
public final class DuplicateTherapyCustomizationController extends AbstractController implements InitializingBean {

    private static final Log LOGGER = LogFactory.getLog(DuplicateTherapyCustomizationController.class);
    private static final String HISTORY_REPORT = "HISTORY_REPORT";
    private static final String VIEW = "pecs.duplicatetherapycustomization.page";

    @Resource
    private DuplicateTherapyService dtCatService = null;
    @Resource
    private MetadataService metadataService;
    @Resource
    private TerminologyService terminologyService;

    @Resource
    private RecordLockingHelper recordLockingHelper;

    /**
     * Executes a query for the history report.
     * 
     * @param recordId
     *            the id of the current record
     * @param response
     *            the http response
     * @return a the report in the model and view
     * @throws CustomizationReportException
     *             if an exception occurs
     */
    @RequestMapping(value = "/history/VA/{recordId}", method = RequestMethod.GET)
    public ModelAndView executeQuery(@PathVariable(value = "recordId") String recordId, HttpServletResponse response)
            throws CustomizationReportException {
        Map<String, Object> model = new HashMap<String, Object>();
        model.put(ReportHistoryXLSXViewer.METADATA_KEY, Concept.DUPLICATE_THERAPY);
        model.put(ReportHistoryXLSXViewer.DATA_KEY, dtCatService.getVersionHistoryReportData(Long.parseLong(recordId)));
        model.put(AbstractXLSXViewer.EXCEL_FILE_NAME_KEY, Concept.DUPLICATE_THERAPY);
        model.put(AbstractXLSXViewer.EXCEL_FILE_TYPE_KEY, HISTORY_REPORT);
        return new ModelAndView(ReportHistoryXLSXViewer.REPORT_HISTORY_VIEW, model);

    }

    /**
     * This will load the detail page for staging query result only using the DTCID.
     * 
     * @param dtcId
     *            the DTCID
     * @param request
     *            HttpServletRequest
     * @param response
     *            HttpServletResponse
     * @return ModelAndView
     * @throws IOException
     *             if an error occurs sending the redirect
     */
    @RequestMapping(value = "/dtcid/VA/{dtcId}", method = RequestMethod.GET)
    public final ModelAndView displayVAByDTCID(@PathVariable(value = "dtcId") String dtcId, HttpServletRequest request,
            HttpServletResponse response) throws IOException {

        DuplicateTherapyDTO duplicateTherapyDTO = new DuplicateTherapyDTO();

        try {
            duplicateTherapyDTO = retrieveCustomModelByKey(Long.parseLong(dtcId));
        } catch (CustomizationServiceException ex) {
            findMessage("error.load.duplicateTherapy");
            LOGGER.error("Unable to display Duplicate Therapy detail page ", ex);
        }

        ModelAndView view = setupForDisplay(getUserProfile(request), duplicateTherapyDTO);
        response.sendRedirect(request.getContextPath() + request.getServletPath() + "/duplicateTherapyCustomization/VA/"
                + duplicateTherapyDTO.getFdbCustomDtcatVaId());
        view.setViewName("redirect:" + VIEW);
        return view;
    }

    /**
     * Requests This will load the detail page for staging query result only.
     * 
     * @param recordId
     *            the version identifier of the DT to display
     * @param request
     *            HttpServletRequest
     * @return ModelAndView
     */
    @RequestMapping(value = "/VA/{recordId}", method = RequestMethod.GET)
    public final ModelAndView load(@PathVariable(value = "recordId") String recordId, HttpServletRequest request) {
        DuplicateTherapyDTO duplicateTherapyDTO = new DuplicateTherapyDTO();

        try {

            duplicateTherapyDTO = retrieveCustomModelByRecordId(Long.parseLong(recordId));

        } catch (CustomizationServiceException ex) {
            findMessage("error.load.duplicateTherapy");
            LOGGER.error("Unable to display Duplicate Therapy detail page ", ex);

        }

        duplicateTherapyDTO.setReadOnlyRecord(!duplicateTherapyDTO.isCurrent());

        return setupForDisplay(getUserProfile(request), duplicateTherapyDTO);
    }

    /**
     * This will update the Duplicate Therapy.
     * 
     * @param recordType
     *            the type of record
     * @param recordId
     *            the record/version identifier
     * @param request
     *            HttpServletRequest
     * @param response
     *            HttpServletResponse
     * @param duplicateTherapyDTO
     *            the data to update
     * @param bindingResult
     *            BindingResult
     * @return ModelAndView
     * @throws IOException
     *             if a problem occurs sending the redirect
     */
    @RequestMapping(value = "/{recordType}/{recordId}", method = RequestMethod.POST)
    public final ModelAndView performAction(@PathVariable(value = "recordType") String recordType, @PathVariable(
            value = "recordId") String recordId, HttpServletRequest request, HttpServletResponse response,
            @Valid DuplicateTherapyDTO duplicateTherapyDTO, BindingResult bindingResult) throws IOException {

        String action = formatAction(request);

        UserProfile profile = getUserProfile(request);

        // get the existing data from database for compare for informational
        // message for modify.
        DuplicateTherapyDTO duplicateTherapyDTOExistingData = new DuplicateTherapyDTO();

        try {
            duplicateTherapyDTOExistingData = retrieveCustomModelByRecordId(Long.parseLong(recordId));
            LOGGER.debug("Shahid >>>>> value of duplicate therapy id inside load method: "
                    + duplicateTherapyDTO.getDuplicatetherapyid());

        } catch (CustomizationServiceException ex) {
            findMessage("error.load.duplicateTherapy");
            LOGGER.error("Unable to display Duplicate Therapy detail page ", ex);
        }

        if (RecordType.FDB.equals(RecordType.fromString(recordType))) {

            duplicateTherapyDTO = save(profile, duplicateTherapyDTO, action, duplicateTherapyDTOExistingData, bindingResult);
            if (bindingResult.hasErrors()) {
                return setupForDisplay(profile, duplicateTherapyDTO);
            }
        } else {
            try {
                duplicateTherapyDTO.setActionBy(profile.getUsername());
                duplicateTherapyDTO.setActionUser(profile);

                if (profile.isApprover() && duplicateTherapyDTO.getAssignedTo() == null) {
                    duplicateTherapyDTO.setAssignedTo(profile.getUsername());
                }

                if (duplicateTherapyDTO.getAssignedTo() == null || duplicateTherapyDTO.getAssignedTo().equals("")) {
                    duplicateTherapyDTO.setAssignedTo(IConstants.SELECT_UNASSIGNED);
                }

                String prevStatus = duplicateTherapyDTO.getStatus();
                duplicateTherapyDTO = dtCatService.applyAction(action, duplicateTherapyDTO, bindingResult);

                if (bindingResult.hasErrors()) {
                    return setupForDisplay(profile, duplicateTherapyDTO);
                }
                String newStatus = duplicateTherapyDTO.getStatus();
                if ("Modified".equals(newStatus)) {
                    if (newStatus.equals(prevStatus)) {
                        boolean fdbFieldChanged = false;
                        if (!duplicateTherapyDTOExistingData.getCustomstring().equals(duplicateTherapyDTO.getCustomstring())) {
                            fdbFieldChanged = true;
                        }
                        if (!duplicateTherapyDTOExistingData.getCustomdupallowance().equals(
                                duplicateTherapyDTO.getCustomdupallowance())) {
                            fdbFieldChanged = true;
                        }
                        if (fdbFieldChanged) {
                            String message =
                                    getInformationalMessageDisplay(action, newStatus,
                                            duplicateTherapyDTOExistingData.getStatus());
                            addInformationMessage(message);
                        }
                    } else {
                        String message =
                                getInformationalMessageDisplay(action, newStatus, duplicateTherapyDTOExistingData.getStatus());
                        addInformationMessage(message);
                    }

                } else {
                    if (!"Modify".equals(action)) {
                        String message =
                                getInformationalMessageDisplay(action, newStatus, duplicateTherapyDTOExistingData.getStatus());
                        addInformationMessage(message);
                    }
                }

            } catch (CustomizationServiceException ex) {
                LOGGER.error("Unable to display Duplicate Therapy detail page ", ex);
                bindingResult.addError(new ObjectError("duplicateTherapyDTO", findMessage("us.exception.create")));
            }

        }

        ModelAndView modelAndView = setupForDisplay(getUserProfile(request), duplicateTherapyDTO);
        if (!bindingResult.hasErrors()) {
            if (RecordType.FDB.equals(RecordType.fromString(recordType))) {
                cancelLock("FDB", recordId, request);
            } else {
                cancelLock("VA", String.valueOf(duplicateTherapyDTO.getFdbCustomDtcatVaId()), request);

            }

        }
        String url =
                request.getContextPath() + request.getServletPath() + "/duplicateTherapyCustomization/VA/"
                        + duplicateTherapyDTO.getFdbCustomDtcatVaId();
        View view = new RedirectView(url, false, false, false);
        modelAndView.setView(view);
        return modelAndView;

    }

    /**
     * Requests This will load the detail page for FDB query result only. Before loading it will check to see if there is an
     * existing record.
     * 
     * @param recordId
     *            the DTCID of the FDB Duplicate Therapy
     * @param request
     *            HttpServletRequest
     * @return ModelAndView
     */
    @RequestMapping(value = "/FDB/{recordId}", method = RequestMethod.GET)
    public final ModelAndView loadFDB(@PathVariable(value = "recordId") String recordId, HttpServletRequest request) {

        DuplicateTherapyDTO fdbModel = new DuplicateTherapyDTO();
        DuplicateTherapyDTO duplicateTherapyDTO = new DuplicateTherapyDTO();

        try {
            fdbModel = retrieveFdbReferenceModels(Long.parseLong(recordId));
            duplicateTherapyDTO = retrieveCustomModelByKey(Long.parseLong(recordId));

            UserProfile profile = getUserProfile(request);
            fdbModel.setActionBy(profile.getUsername());
            fdbModel.setRequestor(profile.getUsername());
            fdbModel.setActionDate(new Date());

            if (duplicateTherapyDTO != null) {
                fdbModel.setVaExists(false);
                fdbModel.setCurrent(false);

                Object[] arg =
// @formatter:off
                        { String.valueOf(duplicateTherapyDTO.getFdbCustomDtcatVaId()), 
                        String.valueOf(duplicateTherapyDTO.getDtcid()) };
                        //   @formatter:on
                addInformationMessage("message.us.dt.information.vaexist", arg);
            }

        } catch (CustomizationServiceException ex) {
            LOGGER.error("Unable to load duplicate therapy models: ", ex);

        }
        return setupForDisplay(getUserProfile(request), fdbModel);
    }

    /**
     * Locks a DT record.
     * 
     * @param recordType
     *            the record type
     * @param recordId
     *            the identifier of the record
     * @param request
     *            HttpServletRequest
     * @return LockingResult
     */
    @RequestMapping(value = "edit/{recordType}/{recordId}", method = RequestMethod.GET)
    @ResponseBody
    public final LockingResult lockRecord(@PathVariable(value = "recordType") String recordType, @PathVariable(
            value = "recordId") String recordId, HttpServletRequest request) {
        LockingResult result = new LockingResult();
        DuplicateTherapyDTO customDtcatVa = null;
        if (RecordType.VA.equals(RecordType.fromString(recordType))) {

            customDtcatVa = dtCatService.getStagingDataByVersionId(Long.parseLong(recordId));
            if (!customDtcatVa.isCurrent()) {

                DuplicateTherapyDTO current = this.retrieveCustomModelByKey(customDtcatVa.getDtcid());

                result.setCurrent(false);
                result.setMessage(findMessage("detail.message.notcurrent"));
                result.setUrl(current.getSelect().getValue());
                return result;
            }
            recordId = String.valueOf(customDtcatVa.getMasterRecordId());
        } else {

            customDtcatVa = dtCatService.getFdbDataByDtcid(Long.parseLong(recordId));

            DuplicateTherapyDTO current = this.retrieveCustomModelByKey(customDtcatVa.getDtcid());
            if (current != null) {
                result.setCurrent(false);
                result.setMessage(findMessage("detail.message.notcurrent"));
                result.setUrl(current.getSelect().getValue());
                return result;
            }
        }
        String message =
                recordLockingHelper.checkEdit(recordType, Concept.DUPLICATE_THERAPY.toString(), recordId,
                        getUserProfile(request).getUsername());

        result.setMessage(message);
        return result;
    }

    /**
     * Cancels the locking of a DT record.
     * 
     * @param recordType
     *            the type of record
     * @param recordId
     *            the identifier of the record
     * @param request
     *            HttpServletRequest
     * @return String
     */
    @RequestMapping(value = "cancel/{recordType}/{recordId}", method = RequestMethod.GET)
    @ResponseBody
    public final String cancelLock(@PathVariable(value = "recordType") String recordType,
            @PathVariable(value = "recordId") String recordId, HttpServletRequest request) {
        DuplicateTherapyDTO customDtcatVa = null;

        if (RecordType.VA.equals(RecordType.fromString(recordType))) {
            customDtcatVa = dtCatService.getStagingDataByVersionId(Long.parseLong(recordId));
            recordId = String.valueOf(customDtcatVa.getMasterRecordId());
        }

        return recordLockingHelper.cancelEdit(recordType, Concept.DUPLICATE_THERAPY.toString(), recordId,
                getUserProfile(request).getUsername());

    }

    /**
     * Extends the time for a locked DT record.
     * 
     * @param recordType
     *            the type of record
     * @param recordId
     *            the identifier of the record
     * @param request
     *            HttpServletRequest
     * @return String
     */
    @RequestMapping(value = "extend/{recordType}/{recordId}", method = RequestMethod.GET)
    @ResponseBody
    public final LockingResult extendLock(@PathVariable(value = "recordType") String recordType, @PathVariable(
            value = "recordId") String recordId, HttpServletRequest request) {
        LockingResult result = new LockingResult();
        DuplicateTherapyDTO customDtcatVa = null;
        if (RecordType.VA.equals(RecordType.fromString(recordType))) {
            customDtcatVa = dtCatService.getStagingDataByVersionId(Long.parseLong(recordId));
            if (!customDtcatVa.isCurrent()) {
                DuplicateTherapyDTO current = this.retrieveCustomModelByKey(customDtcatVa.getDtcid());
                result.setCurrent(false);
                result.setUrl(current.getSelect().getValue());
                return result;
            }

            recordId = String.valueOf(customDtcatVa.getMasterRecordId());
        } else {
            recordId = String.valueOf(Long.parseLong(recordId));
        }
        recordLockingHelper.extendLock(recordType, Concept.DUPLICATE_THERAPY.toString(), recordId, getUserProfile(request)
                .getUsername());

        return result;
    }

    /**
     * Called from performAction method to save FDB record to staging database.
     * 
     * @param profile
     *            UserProfile
     * @param duplicateTherapyDTO
     *            DuplicateTherapyDTO
     * @param action
     *            String
     * @param duplicateTherapyDTOExistingData
     *            DuplicateTherapyDTO
     * @param bindingResult
     *            BindingResult
     * @return DuplicateTherapyDTO
     */
    private DuplicateTherapyDTO save(UserProfile profile, DuplicateTherapyDTO duplicateTherapyDTO, String action,
            DuplicateTherapyDTO duplicateTherapyDTOExistingData, BindingResult bindingResult) {

        if (LOGGER.is   ugEnabled())
            LOGGER.debug("GUI 'Save', received duplicateTherapyDTO object with dtcid: " + duplicateTherapyDTO.getDtcid()
                    + " and custom string: " + duplicateTherapyDTO.getCustomstring());

        try {
            if (profile.isApprover() && duplicateTherapyDTO.getAssignedTo() == null)
                duplicateTherapyDTO.setAssignedTo(profile.getUsername());

            if (duplicateTherapyDTO.getAssignedTo() == null || duplicateTherapyDTO.getAssignedTo().equals(""))
                duplicateTherapyDTO.setAssignedTo(IConstants.SELECT_UNASSIGNED);

            duplicateTherapyDTO.setActionBy(profile.getUsername());
            duplicateTherapyDTO.setRequestor(profile.getUsername());

            List<DuplicateTherapyDTO> models =
                    retrieveCustomModels(duplicateTherapyDTO.getDtcid(), duplicateTherapyDTO.getCustomstring(),
                            duplicateTherapyDTO.getCustomdupallowance());
            if (LOGGER.is   ugEnabled())
                LOGGER.debug("Got back existing models object: " + models);

            duplicateTherapyDTO = dtCatService.createNewRecord(duplicateTherapyDTO, bindingResult);

            if (!bindingResult.hasErrors()) {
                duplicateTherapyDTO.setCurrentActionReason("");
                String message = getInformationalMessageDisplay(action, "", "");
                addInformationMessage(message);
            }
        } catch (CustomizationServiceException ex) {
            LOGGER.error("Unable to save duplicate therapy models: ", ex);
            bindingResult.addError(new ObjectError("duplicateTherapyDTO", findMessage("us.exception.create")));
        }
        return duplicateTherapyDTO;

    }

    /**
     * Retrieves a custom Duplicate Therapy by the DTCID.
     * 
     * @param dtcid
     *            The identifier of the DT
     * @return DuplicateTherapyDTO
     */
    private DuplicateTherapyDTO retrieveCustomModelByKey(long dtcid) {
        DuplicateTherapyDTO dtcatVa = null;
        List<DuplicateTherapyDTO> models = dtCatService.getStagingData(dtcid);

        if (models != null && models.size() > 0) {
            dtcatVa = models.get(0);
            if (LOGGER.is   ugEnabled())
                LOGGER.debug("duplicate therapy record exists for id: " + dtcatVa.getDtcid());
        }
        return dtcatVa;
    }

    /**
     * Retrieves FDB DT by DTCID.
     * 
     * @param dtcid
     *            The identifier of the DT
     * @return DuplicateTherapyDTO
     */
    private DuplicateTherapyDTO retrieveFdbReferenceModels(long dtcid) {
        return dtCatService.getFdbDataByDtcid(dtcid);
    }

    /**
     * Retrieves a custom DT by the version identifier.
     * 
     * @param recordId
     *            the version identifier
     * @return DuplicateTherapyDTO
     */
    private DuplicateTherapyDTO retrieveCustomModelByRecordId(long recordId) {
        return dtCatService.getStagingDataByVersionId(recordId);
    }

    /**
     * Configures the ModelAndView for displaying to the user.
     * 
     * @param profile
     *            UserProfile
     * @param duplicateTherapyDTO
     *            DuplicateTherapyDTO
     * @return ModelAndView
     */
    private ModelAndView setupForDisplay(UserProfile profile, DuplicateTherapyDTO duplicateTherapyDTO) {
        ModelAndView view = new ModelAndView(VIEW);
        ConceptMetadata conceptMetadata = getMetadataService().getMetadata(DuplicateTherapyDTO.class);
        Map<String, ConceptField> fieldMap = conceptMetadata.getFieldMap();

        view.addObject("duplicateTherapyDTO", duplicateTherapyDTO);
        view.addObject("fieldMap", fieldMap);
        List<DuplicateAllowance> dupAllowance = terminologyService.getDuplicateAllowances();
        view.addObject("customDuplicateAllowanceList", dupAllowance);
        List<String> userList = getAdminUsers();
        view.addObject("userList", userList);

        view.addObject("allowableActions", Collections.emptySet());

        Set<String> actions = Collections.emptySet();
        duplicateTherapyDTO.getDtcid();
        duplicateTherapyDTO.getFdbCustomDtcatVaId();
        if (!duplicateTherapyDTO.isReadOnlyRecord()) {
            if (duplicateTherapyDTO.getFdbCustomDtcatVaId() != 0) {
                actions = dtCatService.getAllowableActions(duplicateTherapyDTO, profile);
            }
            permissionValidation(profile, actions, view, duplicateTherapyDTO.getFdbCustomDtcatVaId());
        }

        List<DuplicateTherapyDTO> vaList = dtCatService.getStagingData(duplicateTherapyDTO.getDtcid());
        if (duplicateTherapyDTO.getFdbCustomDtcatVaId() == 0) {
            showCommentButtonDuplicateTherapy(vaList, view);
        }

        return view;
    }

    /**
     * Show comment button.
     * 
     * @param vaInteractions
     *            list from ctstaging for va records by first data bank identification.
     * @param mav
     *            the Model and View.
     */
    public final void showCommentButtonDuplicateTherapy(List<?> vaInteractions, ModelAndView mav) {
        boolean showCommentButton = false;
        if (vaInteractions != null && vaInteractions.size() > 0) {
            Object interaction = vaInteractions.get(0);

            if (((PecsDTO) interaction).getStatus().equalsIgnoreCase(Status.STATUS_DELETED)) {
                showCommentButton = true;
            }
        } else {
            showCommentButton = true;
        }
        mav.addObject("showCommentButton", showCommentButton);
    }

    /**
     * Retrieves a list of custom DT using the supplied parameters.
     * 
     * @param dtcid
     *            the identifier of the DT
     * @param customString
     *            the customString value
     * @param customDupAllowance
     *            the Duplicate Allowance value
     * @return List<DuplicateTherapyDTO>
     */
    private List<DuplicateTherapyDTO> retrieveCustomModels(long dtcid, String customString, long customDupAllowance) {
        if (LOGGER.is   ugEnabled())
            LOGGER.debug("Attempting to retrive any existing custom models for dtcid: " + dtcid + " and custom string"
                    + customString);

        List<DuplicateTherapyDTO> models = dtCatService.getStagingData(dtcid, customString, customDupAllowance);
        if (models != null && models.size() > 0)
            return models;

        return null;
    }

    /**
     * Sets the DtCatTherapyService.
     * 
     * @param dtCatService
     *            DtCatTherapyService
     */
    public final void setDtCatService(DuplicateTherapyService dtCatService) {
        this.dtCatService = dtCatService;
    }

    /**
     * Called by Spring so that the class can check to see if all beans have been injected and to perform initialization.
     * 
     * @throws Exception
     *             if a problem was found
     */
    @Override
    public final void afterPropertiesSet() throws Exception {
        if (dtCatService == null) {
            throw new Exception(findMessage("message.resource.not.set", new Object[] { "dtCatService" }));
        }
        if (metadataService == null) {
            throw new Exception(findMessage("message.resource.not.set", new Object[] { "MetadataService" }));
        }
    }

    /**
     * Returns an message to display to the user.
     * 
     * @param action
     *            the action performed by the user
     * @param newStatus
     *            the new status of the DT
     * @param prevStat
     *            the previous status of the DT
     * @return String
     */
    private String getInformationalMessageDisplay(String action, String newStatus, String prevStat) {
        String msg = "";
        if (action.equals("Modify")) {
            msg = "message.us.information.requestor.submitted";
        } else if (action.equals("Approve")) {
            msg = "message.us.information.approver.approve";
        } else if (action.equals("Delete")) {
            msg = "message.us.information.approver.delete";
        } else if (action.equals("Reject")) {
            if (newStatus.equals("Approved")) {
                if (prevStat.equals("Modified")) {
                    msg = "message.us.information.approver.reject4";
                } else if (prevStat.equals("Reviewed")) {
                    msg = "message.us.information.approver.reject6";
                } else {
                    msg = "message.us.information.approver.reject2";
                }

            } else if (newStatus.equals("Deleted")) {
                if (prevStat.equals("Modified")) {
                    msg = "message.us.information.approver.reject5";
                } else {
                    msg = "message.us.information.approver.reject3";
                }
            } else {
                msg = "message.us.information.approver.reject";
            }
        } else if (action.equals("Submit_as_Reviewed")) {
            msg = "message.us.information.approver.submittedAsReviewed";
        } else if (action.equals("Submit_for_Delete")) {
            msg = "message.us.information.approver.submitfordelete";
        } else if (action.equals("Customize")) {
            msg = "message.us.information.requestor.submitted";
        }
        return msg;
    }

  

    /**
     * Returns the RecordLockingHelper.
     * 
     * @return the recordLockingHelper
     */
    public final RecordLockingHelper getRecordLockingHelper() {
        return recordLockingHelper;
    }

    /**
     * Sets the RecordLockingHelper.
     * 
     * @param recordLockingHelper
     *            the recordLockingHelper to set
     */
    public final void setRecordLockingHelper(RecordLockingHelper recordLockingHelper) {
        this.recordLockingHelper = recordLockingHelper;
    }

    /**
     * Gets a record from a DTCID.
     * 
     * @param id
     *            the dtcid
     * @return a DuplicateTherapyDTO
     */
    public final DuplicateTherapyDTO getRecordByDTCID(long id) {
        return dtCatService.getFdbDataByDtcid(id);
    }

}
