package gov.va.med.pharmacy.pecs.web.easy;

import gov.va.med.pharmacy.pecs.domain.impl.easy.DoseCheck;
import gov.va.med.pharmacy.pecs.domain.impl.easy.DosingInfo;
import gov.va.med.pharmacy.pecs.domain.impl.easy.DrugCheckResult;
import gov.va.med.pharmacy.pecs.domain.impl.easy.DrugIdentifier;
import gov.va.med.pharmacy.pecs.domain.impl.easy.DrugIdentifierType;
import gov.va.med.pharmacy.pecs.domain.impl.easy.PecsDrugInfo;
import gov.va.med.pharmacy.pecs.domain.impl.lookup.DoseRoute;
import gov.va.med.pharmacy.pecs.domain.impl.lookup.DoseType;
import gov.va.med.pharmacy.pecs.domain.impl.lookup.DoseUnitType;
import gov.va.med.pharmacy.pecs.service.easy.EasySearchService;
import gov.va.med.pharmacy.pecs.util.LabelValue;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.annotation.Resource;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

/**
 * This is the controller that takes request for searching drugs.
 * 
 * @author vhaisacarusr
 * 
 */
@Controller
@RequestMapping(value = "drugSearch")
public final class DrugSearchController implements InitializingBean {

    /** The Constant LOGGER. */
    private static final Log LOGGER = LogFactory.getLog(DrugSearchController.class);

    /** The easy search service. */
    @Resource
    private EasySearchService easySearchService;

    /**
     * Find drugs.
     * 
     * @param criteria
     *            the criteria
     * @return the collection
     */
    @RequestMapping(value = "findDrugs")
    @ResponseBody
    public Collection<PecsDrugInfo> findDrugs(@RequestParam("criteria") String criteria) {
        return easySearchService.searchDrugs(criteria);

    }

    /**
     * Find drugs dose.
     * 
     * @param criteria
     *            the criteria
     * @return the drug check result
     */
    @RequestMapping(value = "findDrugsDose")
    @ResponseBody
    public DrugCheckResult<DosingInfo> findDrugsDose(@RequestParam("criteria") String criteria) {

        DoseCheck doseCheck = new DoseCheck();
        return easySearchService.checkDosing(doseCheck);

    }

    /**
     * This method takes a comma and pipe delimited string like the one below parses it and returns a list of label value pairs.
     * 
     * GCN_SEQNO|4387,GCN_SEQNO|4386
     * 
     * @param drugs
     *            the list of drugs
     * @return a label value pair list as JSON
     */
    @RequestMapping(value = "findDrugsByIds")
    @ResponseBody
    public ArrayList<LabelValue> findDrugsByIds(@RequestParam("drugs") String drugs) {
        ArrayList<LabelValue> labels = new ArrayList<LabelValue>();
        String[] drugList = drugs.split("\\,");
        for (String element : drugList) {
            LabelValue label = new LabelValue();
            String[] drug = element.split("\\|");
            PecsDrugInfo drugInfo = null;
            try {
                drugInfo = easySearchService.getDrug(new DrugIdentifier(drug[1], DrugIdentifierType.fromString(drug[0])));
                label.setLabel(drugInfo.getUiLabel());
                label.setValue(element);
                labels.add(label);

            } catch (Exception e) {
                LOGGER.info(e);
            }

        }

        return labels;
    }

    /**
     * Submit drugs dose.
     * 
     * @param criteria
     *            the criteria
     * @return the drug check result
     */
    @RequestMapping(value = "submitDrugsDose")
    @ResponseBody
    public DrugCheckResult<DosingInfo> submitDrugsDose(@RequestParam("criteria") String criteria) {
        DoseCheck doseCheck = new DoseCheck();
        doseCheck.setDrugId(new DrugIdentifier(39191, DrugIdentifierType.GCN_SEQNO));
        doseCheck.setSingleDoseAmount(17);
        doseCheck.setDoseUnit("CAPSULE");
        doseCheck.setDoseRateUnit("DAY");
        doseCheck.setDoseType("MAINTENANCE");
        doseCheck.setDoseRoute("ORAL");
        doseCheck.setDuration(3);
        doseCheck.setDurationRate("DAY");
        doseCheck.setFrequency("BID");

        doseCheck.setAgeInDays(10000);
        doseCheck.setWeightInKg(60);
        // doseCheck.setHeightInCm(165);
        doseCheck.setBsaInSqM(0.392);

        DrugCheckResult<DosingInfo> res = easySearchService.checkDosing(doseCheck);
        return res;

    }

    /**
     * Drug for dose range checked.
     * 
     * @param criteria
     *            the criteria
     * @return the pecs drug info
     */
    @RequestMapping(value = "drugForDoseRangeChecked")
    @ResponseBody
    public PecsDrugInfo drugForDoseRangeChecked(@RequestParam("criteria") String criteria) {
        DrugIdentifier di = DrugIdentifierKeyHelper.createDrugIdentifier(criteria);
        PecsDrugInfo drugInfo = easySearchService.getDrug(di);
        return drugInfo;
    }

    /**
     * Retrieves the Dose Units for the id/type provided.
     * 
     * @param id
     *            the id
     * @param type
     *            the type
     * @return List<DoseUnitType>
     */
    @RequestMapping(value = "getDoseUnits/{id}/{type}")
    @ResponseBody
    public List<DoseUnitType> getDoseUnits(@PathVariable("id") String id, @PathVariable("type") String type) {

        DrugIdentifier drugIndentifier = new DrugIdentifier(id, DrugIdentifierType.fromString(type));
        List<DoseUnitType> doseUnits = null;
        try {
            doseUnits = easySearchService.getDoseUnits(drugIndentifier);
        } catch (Exception e) {
            LOGGER.info(e);
        }
        return doseUnits;
    }

    /**
     * Retrieves the Dose Routes for the provided id/type.
     * 
     * @param id
     *            the id
     * @param type
     *            the type
     * @return List<DoseRoute>
     */
    @RequestMapping(value = "getDoseRoutes/{id}/{type}")
    @ResponseBody
    public List<DoseRoute> getDoseRoutes(@PathVariable("id") String id, @PathVariable("type") String type) {
        DrugIdentifier drugIndentifier = new DrugIdentifier(id, DrugIdentifierType.fromString(type));
        List<DoseRoute> doseRoutes = null;
        try {
            doseRoutes = easySearchService.getDoseRoutes(drugIndentifier);
        } catch (Exception e) {
            LOGGER.info(e);
        }
        return doseRoutes;
    }

    /**
     * Retrieves the Dose Types for the id/type provided.
     * 
     * @param id
     *            the id
     * @param type
     *            the type
     * @return List<DoseType>
     */
    @RequestMapping(value = "getDoseTypes/{id}/{type}")
    @ResponseBody
    public List<DoseType> getDoseTypes(@PathVariable("id") String id, @PathVariable("type") String type) {
        DrugIdentifier drugIndentifier = new DrugIdentifier(id, DrugIdentifierType.fromString(type));
        List<DoseType> doseTypes = null;
        try {
            doseTypes = easySearchService.getDoseTypes(drugIndentifier);

        } catch (Exception e) {
            LOGGER.info(e);
        }
        return doseTypes;

    }

    /**
     * Submit drugs dose range bsa.
     * 
     * @param age
     *            the age
     * @param height
     *            the height
     * @param weight
     *            the weight
     * @return the string
     */
    @RequestMapping(value = "submitDrugsDoseRangeBSA")
    @ResponseBody
    public String submitDrugsDoseRangeBSA(@RequestParam("age") String age, @RequestParam("height") String height,
            @RequestParam("weight") String weight) {

        // var answer = Math.pow(form.kg.value,0.425) * Math.pow(form.cm.value,0.725) * 0.007184;

        Float floatHeight = new Float(height);
        Float floatWeight = new Float(weight);

        double BSA = Math.pow(floatWeight, 0.425) * Math.pow(floatHeight, 0.725) * 0.007184;

        return String.valueOf(BSA);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.springframework.beans.factory.InitializingBean#afterPropertiesSet()
     */
    @Override
    public void afterPropertiesSet() throws Exception {
        if (easySearchService == null) {
            throw new Exception("The EasySearchService has not been injected in to EasySearchController.");
        }

    }

}
