package gov.va.med.pharmacy.pecs.web.easy;

import gov.va.med.pharmacy.pecs.domain.impl.easy.DoseCheck;
import gov.va.med.pharmacy.pecs.domain.impl.easy.DosingInfo;
import gov.va.med.pharmacy.pecs.domain.impl.easy.DrugCheckResult;
import gov.va.med.pharmacy.pecs.domain.impl.easy.DrugDrugInteractionInfo;
import gov.va.med.pharmacy.pecs.domain.impl.easy.DrugIdentifier;
import gov.va.med.pharmacy.pecs.domain.impl.easy.DrugIdentifierType;
import gov.va.med.pharmacy.pecs.domain.impl.easy.DuplicateTherapyInfo;
import gov.va.med.pharmacy.pecs.domain.impl.easy.PecsDrugInfo;
import gov.va.med.pharmacy.pecs.domain.impl.lookup.DoseRoute;
import gov.va.med.pharmacy.pecs.domain.impl.lookup.DoseType;
import gov.va.med.pharmacy.pecs.domain.impl.lookup.DoseUnitType;
import gov.va.med.pharmacy.pecs.domain.impl.lookup.SeverityLevel;
import gov.va.med.pharmacy.pecs.service.TerminologyService;
import gov.va.med.pharmacy.pecs.service.easy.EasySearchService;
import gov.va.med.pharmacy.pecs.web.AbstractController;

import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.codehaus.jackson.map.DeserializationConfig;
import org.codehaus.jackson.map.ObjectMapper;
import org.codehaus.jackson.map.SerializationConfig;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.stereotype.Controller;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.servlet.ModelAndView;

/**
 * 
 * The controller for easy search.
 * 
 * @author VHAISACARUSR
 * 
 */

@Controller
@RequestMapping(value = "/easySearch")
public final class EasySearchController extends AbstractController implements InitializingBean {

    /**
     * 
     */
    private static final int DAYS_IN_YEAR = 365;
    private static final String TILES_VIEW = "pecs.easysearch";
    private static final String TILES_RESULTS_VIEW = "pecs.es.results";
    private static final String TILES_DR_RESULTS_VIEW = "pecs.es.dr.results";

    private static final String DDIDT_SELECTION_CODE = "DDIDT";
    private static final String DOSER_SELECTION_CODE = "DOSER";

    private static final String CHAR_ENCODING_SCHEMA = "ISO-8859-1";

    private static final String ALL_SEVERITIES = "Display_Severity_Levels_all";

    private final Log logger = LogFactory.getLog(EasySearchController.class);
    private static final double METRIC = 2.20462262185;

    @Resource
    private TerminologyService terminologyService;
    @Resource
    private EasySearchService easySearchService;

    private ObjectMapper objectMapper;

    /**
     * Displays the easy search page.
     * 
     * @param request
     *            the HttpServletReqest
     * @return a model and view object
     */
    @RequestMapping(method = RequestMethod.GET)
    public ModelAndView get(HttpServletRequest request) {
        ModelAndView view = new ModelAndView(TILES_VIEW);
        EasySearchModel model = convertJsonToModel(request.getParameter("easySearchModel"));
        if (model == null) {
            model = new EasySearchModel();
        } else {
            if (DDIDT_SELECTION_CODE.equals(model.getConceptSelection())) {
                Collection<DrugIdentifier> ids = buildRoutedDrugList(model.getProspectiveDrugList());
                model.setDrugsSearched(buildDrugsSearched(ids));
            }
            if (DOSER_SELECTION_CODE.equals(model.getConceptSelection())) {
                Collection<DrugIdentifier> ids = buildRoutedDrugList(model.getProspectiveDRDrugList());
                model.setDrugsDRSearched(buildDrugsSearched(ids));
                DrugIdentifier id =
                        new DrugIdentifier(model.getDrugIdentifierValue(), DrugIdentifierType.fromString(model
                                .getDrugIdentifierType()));
                List<DoseType> doseTypes = buildDoseTypes(id);
                model.setDrugDoseTypeList(doseTypes);
                List<DoseRoute> doseRoutes = buildDoseRoutes(id);
                model.setDrugDoseRouteList(doseRoutes);
                List<DoseUnitType> doseUnits = buildDoseUnits(id);
                model.setDrugDoseUnitList(doseUnits);
            }
        }
        model.setConceptSelectionList(getTerminologyService().getEasySearchSelectTypes());
        view.addObject("easySearchModel", model);
        return view;
    }

    /**
     * @param id
     *            the drug identifier
     * @return a list of DoseUnitTypes
     */
    private List<DoseUnitType> buildDoseUnits(DrugIdentifier id) {
        return getEasySearchService().getDoseUnits(id);
    }

    /**
     * @param id
     *            the drug identifier
     * @return a list of DoseRoutes
     */
    private List<DoseRoute> buildDoseRoutes(DrugIdentifier id) {
        return getEasySearchService().getDoseRoutes(id);
    }

    /**
     * @param id
     *            the drug identifier
     * @return list of DoseTypes
     */
    private List<DoseType> buildDoseTypes(DrugIdentifier id) {
        return getEasySearchService().getDoseTypes(id);
    }

    /**
     * Handles the post for the easy search page.
     * 
     * @param model
     *            holds the values needed for the page.
     * @param result
     *            the binding result. Holds any errors on the page.
     * @return a ModelAndView object
     * @throws Exception
     *             if an error occurs
     */
    @RequestMapping(method = RequestMethod.POST)
    public ModelAndView process(@ModelAttribute(value = "easySearchModel") @Valid EasySearchModel model, BindingResult result)
            throws Exception {
        if (result.hasErrors()) {
            ModelAndView view = new ModelAndView(TILES_VIEW);
            model.setConceptSelectionList(getTerminologyService().getEasySearchSelectTypes());
            view.addObject("easySearchModel", model);
            return view;
        }
        if (DDIDT_SELECTION_CODE.equals(model.getConceptSelection())) {
            ddidtSearch(model);
            ModelAndView resultsView = new ModelAndView(TILES_RESULTS_VIEW);
            resultsView.addObject("easySearchModel", model);
            resultsView.addObject("jsonBackQuery", getModelAsJson(model));
            return resultsView;
        }
        if (DOSER_SELECTION_CODE.equals(model.getConceptSelection())) {
            drSearch(model);
            ModelAndView resultsView = new ModelAndView(TILES_DR_RESULTS_VIEW);
            resultsView.addObject("easySearchModel", model);
            resultsView.addObject("jsonBackQuery", getModelAsJson(model));
            return resultsView;
        }
        return null;

    }

    /**
     * @param model
     *            the easy search model
     * @return the encoded URL
     */
    private String getModelAsJson(EasySearchModel model) {
        try {
            return URLEncoder.encode(getObjectMapper().writeValueAsString(model), CHAR_ENCODING_SCHEMA);
        } catch (Exception e) {
            logger.error(e.getMessage(), e);
            return "";
        }
    }

    /**
     * @param jsonString
     *            a JSON string
     * @return a model object
     */
    private EasySearchModel convertJsonToModel(String jsonString) {
        if (jsonString == null || jsonString.length() < 1) {
            return null;
        }
        try {
            return getObjectMapper().readValue(jsonString, EasySearchModel.class);
        } catch (Exception e) {
            logger.error(e.getMessage(), e);
            return null;
        }
    }

    /**
     * @return and object mapper
     */
    private ObjectMapper getObjectMapper() {
        if (objectMapper == null) {
            objectMapper = new ObjectMapper();
            objectMapper.configure(SerializationConfig.Feature.USE_ANNOTATIONS, true);
            objectMapper.configure(DeserializationConfig.Feature.USE_ANNOTATIONS, true);
        }
        return objectMapper;
    }

    /**
     * @param model
     *            the easy search model
     * @throws Exception
     *             if an error occurs
     */
    private void ddidtSearch(EasySearchModel model) throws Exception {
        Collection<DrugIdentifier> ids = buildRoutedDrugList(model.getProspectiveDrugList());
        model.setDrugsSearched(buildDrugsSearched(ids));
        if (model.isDdiSelected()) {
            searchForDrugDrugInteraction(ids, model);
        }
        if (model.isDtSelected()) {
            searchForDuplicateTherapy(ids, model);
        }
    }

    /**
     * @param model
     *            the easy search model
     * @throws Exception
     *             if an error occurs
     */
    private void drSearch(EasySearchModel model) throws Exception {
        DoseCheck doseCheck = new DoseCheck();

        doseCheck.setDrugId(new DrugIdentifier(model.getDrugIdentifierValue(), DrugIdentifierType.fromString(model
                .getDrugIdentifierType())));
        doseCheck.setSingleDoseAmount(Double.valueOf(model.getSingleDose()));
        doseCheck.setDoseUnit(model.getDoseUnitSelection());
        doseCheck.setDoseRateUnit(model.getDoseRateUnitSelection());
        doseCheck.setDoseType(model.getDoseTypeSelection());
        doseCheck.setDoseRoute(model.getDoseRouteSelection());
        doseCheck.setDuration(1);
        doseCheck.setDurationRate("DAY");
        doseCheck.setFrequency(model.getFrequency());
        long ageInYears = Long.parseLong(model.getDoseAgeDays());

        doseCheck.setAgeInDays(ageInYears * DAYS_IN_YEAR);
        double weight = Double.parseDouble(model.getDoseWeight());
        model.getDemographicWeightSelection();
        if ("LB".equalsIgnoreCase(model.getDemographicWeightSelection())) {
            weight = weight / METRIC;
        }
        doseCheck.setWeightInKg(weight);
        doseCheck.setBsaInSqM(Double.parseDouble(model.getDoseBSA()));

        DrugCheckResult<DosingInfo> checkDosing = getEasySearchService().checkDosing(doseCheck);
        if (!checkDosing.getChecks().isEmpty()) {
            model.setCustom(checkDosing.getChecks().iterator().next().isCustom());
        }
        model.setDrCheckResult(checkDosing);
        Collection<DrugIdentifier> ids = new ArrayList<DrugIdentifier>();
        ids.add(doseCheck.getDrugId());
        model.setDrugsDRSearched(buildDrugsSearched(ids));
        model.setDrDoseRangeSelection(model.getDrugsDRSearched().get(0).getUiLabel());
        model.setDoseUnitSelected(getTerminologyService().getDoseUnitTypeByCode(model.getDoseUnitSelection()));
    }

    /**
     * @param ids
     *            a list of drug identifiers
     * @param model
     *            the easy search model
     * @throws Exception
     *             if an error occurs
     */
    private void searchForDuplicateTherapy(Collection<DrugIdentifier> ids, EasySearchModel model) throws Exception {
        DrugCheckResult<DuplicateTherapyInfo> result = getEasySearchService().checkDuplicateTherapy(ids);
        model.setDuplicateTherapyCheckResult(result);
    }

    /**
     * @param ids
     *            a list of drug identifiers
     * @param model
     *            the easy search model
     * @throws Exception
     *             if an error occurs
     */
    private void searchForDrugDrugInteraction(Collection<DrugIdentifier> ids, EasySearchModel model) throws Exception {
        DrugCheckResult<DrugDrugInteractionInfo> result = getEasySearchService().checkDrugDrugInteraction(ids);
        if (!ALL_SEVERITIES.equals(model.getDdiSeverityLevel())) {
            List<SeverityLevel> vistaSeverities = getEasySearchService().getVistaSeverityLevels();
            for (Iterator<DrugDrugInteractionInfo> iter = result.getChecks().iterator(); iter.hasNext();) {
                if (!vistaSeverities.contains(iter.next().getSeverity())) {
                    iter.remove();
                }
            }
        }
        model.setDrugDrugCheckResults(result);
    }

    /**
     * @param drugIdentifiers
     *            a list of drug id's as strings
     * @return a collection of drug identifiers
     */
    private Collection<DrugIdentifier> buildRoutedDrugList(List<String> drugIdentifiers) {
        return buildDrugList(drugIdentifiers);
    }

    /**
     * @param ids
     *            a list of drug id's as strings
     * 
     * @return a collection of drug identifiers
     */
    private Collection<DrugIdentifier> buildDrugList(List<String> ids) {
        Collection<DrugIdentifier> idList = new ArrayList<DrugIdentifier>();
        if (ids == null) {
            return idList;
        }
        for (String id : ids) {
            idList.add(DrugIdentifierKeyHelper.createDrugIdentifier(id));
        }
        return idList;
    }

    /**
     * @param drugs
     *            a list of drug identifiers
     * @return a list of drug information
     */
    private List<PecsDrugInfo> buildDrugsSearched(Collection<DrugIdentifier> drugs) {
        List<PecsDrugInfo> drugsSearched = new ArrayList<PecsDrugInfo>();
        for (DrugIdentifier id : drugs) {
            try {
                drugsSearched.add(getEasySearchService().getDrug(id));
            } catch (Exception e) {
                logger.info("Received exception in EasySearchController while obtaining drugs searched: ", e);
            }
        }
        return drugsSearched;
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        super.afterPropertiesSet();
        if (getTerminologyService() == null) {
            throw new Exception("The TerminologyService has not been injected in to EasySearchController.");
        }
        if (getEasySearchService() == null) {
            throw new Exception("The EasySearchService has not been injected in to EasySearchController.");
        }
    }

    /**
     * Gets the terminologyService.
     * 
     * @return the terminologyService
     */
    public TerminologyService getTerminologyService() {
        return terminologyService;
    }

    /**
     * Sets the terminologyService.
     * 
     * @param terminologyService
     *            the terminologyService to set
     */
    public void setTerminologyService(TerminologyService terminologyService) {
        this.terminologyService = terminologyService;
    }

    /**
     * Gets the easySearchService.
     * 
     * @return the easySearchService
     */
    public EasySearchService getEasySearchService() {
        return easySearchService;
    }

    /**
     * Sets the easySearchService.
     * 
     * @param easySearchService
     *            the easySearchService to set
     */
    public void setEasySearchService(EasySearchService easySearchService) {
        this.easySearchService = easySearchService;
    }
}
