package gov.va.med.pharmacy.pecs.web.filter;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.util.zip.GZIPOutputStream;

import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.ServletOutputStream;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpServletResponseWrapper;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.web.filter.GenericFilterBean;

/**
 * GZIP Filter This filter will gzip the content of a response iff:
 * <ul>
 * <li>The filter is mapped to a matching path
 * <li>The response status code is >=200 and <300
 * <li>The content length is unknown or more than the minGzipSize initParameter or the minGzipSize is 0(default)
 * <li>No content-encoding is specified by the resource
 * </ul>
 * 
 * <p>
 * Compressing the content can greatly improve the network bandwidth usage, but at a cost of memory and CPU cycles.
 * </p>
 * 
 * This is an adaptation of Jetty's GZIP servlet.
 */
public final class GzipFilter extends GenericFilterBean {

    /** The Constant LOGGER. */
    private static final Log LOGGER = LogFactory.getLog(GzipFilter.class);

    /** The Constant DEFAULT_BUFFER_SIZE. */
    private static final int DEFAULT_BUFFER_SIZE = 8192;

    /** The buffer size. */
    private int bufferSize = DEFAULT_BUFFER_SIZE;

    /** The min gzip size. */
    private int minGzipSize = 0;

    // custom implementation of the doFilter method
    /*
     * (non-Javadoc)
     * 
     * @see javax.servlet.Filter#doFilter(javax.servlet.ServletRequest, javax.servlet.ServletResponse,
     * javax.servlet.FilterChain)
     */
    @Override
    public void doFilter(ServletRequest req, ServletResponse res, FilterChain chain) throws IOException, ServletException {
        LOGGER.debug("compressing file" + req.getLocalName());
        // make sure we are dealing with HTTP
        if (req instanceof HttpServletRequest) {
            HttpServletRequest request = (HttpServletRequest) req;
            HttpServletResponse response = (HttpServletResponse) res;
            // check for the HTTP header that
            // signifies GZIP support
            String ae = request.getHeader("accept-encoding");
            if (ae != null && ae.indexOf("gzip") != -1) {
                GZIPResponseWrapper wrappedResponse = new GZIPResponseWrapper(request, response);
                boolean exceptional = true;
                try {
                    chain.doFilter(req, wrappedResponse);
                    exceptional = false;
                } finally {
                    if (exceptional && !response.isCommitted()) {
                        wrappedResponse.resetBuffer();
                        wrappedResponse.noGzip();
                    } else {
                        wrappedResponse.finish();
                    }
                }
            } else {
                chain.doFilter(req, res);
            }
        }
    }

    /**
     * The Class GZIPResponseWrapper.
     */
    public final class GZIPResponseWrapper extends HttpServletResponseWrapper {

        /** The _request. */
        HttpServletRequest _request;

        /** The _no gzip. */
        boolean _noGzip;

        /** The _writer. */
        PrintWriter _writer;

        /** The _gz stream. */
        GzipStream _gzStream;

        /** The _content length. */
        long _contentLength = -1;

        /**
         * Instantiates a new gZIP response wrapper.
         * 
         * @param request
         *            the request
         * @param response
         *            the response
         */
        public GZIPResponseWrapper(HttpServletRequest request, HttpServletResponse response) {
            super(response);
            _request = request;
        }

        /*
         * (non-Javadoc)
         * 
         * @see javax.servlet.ServletResponseWrapper#setContentType(java.lang.String)
         */
        @Override
        public void setContentType(String ct) {
            super.setContentType(ct);
            int colon = ct.indexOf(";");
            if (colon > 0) {
                ct = ct.substring(0, colon);
            }

            if ((_gzStream == null || _gzStream._out == null) && ("application/gzip".equalsIgnoreCase(ct))) {
                noGzip();
            }
        }

        /*
         * (non-Javadoc)
         * 
         * @see javax.servlet.http.HttpServletResponseWrapper#setStatus(int, java.lang.String)
         */
        @Override
        public void setStatus(int sc, String sm) {
            super.setStatus(sc, sm);
            if (sc < 200 || sc >= 300) {
                noGzip();
            }
        }

        /*
         * (non-Javadoc)
         * 
         * @see javax.servlet.http.HttpServletResponseWrapper#setStatus(int)
         */
        @Override
        public void setStatus(int sc) {
            super.setStatus(sc);
            if (sc < 200 || sc >= 300)
                noGzip();
        }

        /*
         * (non-Javadoc)
         * 
         * @see javax.servlet.ServletResponseWrapper#setContentLength(int)
         */
        @Override
        public void setContentLength(int length) {
            _contentLength = length;
            if (_gzStream != null) {
                _gzStream.setContentLength(length);
            }
        }

        /*
         * (non-Javadoc)
         * 
         * @see javax.servlet.http.HttpServletResponseWrapper#setHeader(java.lang.String, java.lang.String)
         */
        @Override
        public void setHeader(String name, String value) {
            if ("content-length".equalsIgnoreCase(name)) {
                _contentLength = Long.parseLong(value);
                if (_gzStream != null) {
                    _gzStream.setContentLength(_contentLength);
                }
            } else if ("content-type".equalsIgnoreCase(name)) {
                setContentType(value);
            } else if ("content-encoding".equalsIgnoreCase(name)) {
                super.setHeader(name, value);
                if (!isCommitted()) {
                    noGzip();
                }
            } else
                super.setHeader(name, value);
        }

        /*
         * (non-Javadoc)
         * 
         * @see javax.servlet.http.HttpServletResponseWrapper#setIntHeader(java.lang.String, int)
         */
        @Override
        public void setIntHeader(String name, int value) {
            if ("content-length".equalsIgnoreCase(name)) {
                _contentLength = value;
                if (_gzStream != null) {
                    _gzStream.setContentLength(_contentLength);
                }
            } else {
                super.setIntHeader(name, value);
            }
        }

        /*
         * (non-Javadoc)
         * 
         * @see javax.servlet.ServletResponseWrapper#flushBuffer()
         */
        @Override
        public void flushBuffer() throws IOException {
            if (_writer != null) {
                _writer.flush();
            }
            if (_gzStream != null) {
                _gzStream.finish();
            } else {
                getResponse().flushBuffer();
            }
        }

        /*
         * (non-Javadoc)
         * 
         * @see javax.servlet.ServletResponseWrapper#reset()
         */
        @Override
        public void reset() {
            super.reset();
            if (_gzStream != null) {
                _gzStream.resetBuffer();
            }
            _writer = null;
            _gzStream = null;
            _noGzip = false;
            _contentLength = -1;
        }

        /*
         * (non-Javadoc)
         * 
         * @see javax.servlet.ServletResponseWrapper#resetBuffer()
         */
        @Override
        public void resetBuffer() {
            super.resetBuffer();
            if (_gzStream != null) {
                _gzStream.resetBuffer();
            }
            _writer = null;
            _gzStream = null;
        }

        /*
         * (non-Javadoc)
         * 
         * @see javax.servlet.http.HttpServletResponseWrapper#sendError(int, java.lang.String)
         */
        @Override
        public void sendError(int sc, String msg) throws IOException {
            resetBuffer();
            super.sendError(sc, msg);
        }

        /*
         * (non-Javadoc)
         * 
         * @see javax.servlet.http.HttpServletResponseWrapper#sendError(int)
         */
        @Override
        public void sendError(int sc) throws IOException {
            resetBuffer();
            super.sendError(sc);
        }

        /*
         * (non-Javadoc)
         * 
         * @see javax.servlet.http.HttpServletResponseWrapper#sendRedirect(java.lang.String)
         */
        @Override
        public void sendRedirect(String location) throws IOException {
            resetBuffer();
            super.sendRedirect(location);
        }

        /*
         * (non-Javadoc)
         * 
         * @see javax.servlet.ServletResponseWrapper#getOutputStream()
         */
        @Override
        public ServletOutputStream getOutputStream() throws IOException {
            if (_gzStream == null) {
                if (getResponse().isCommitted() || _noGzip) {
                    return getResponse().getOutputStream();
                }

                _gzStream =
                        newGzipStream(_request, (HttpServletResponse) getResponse(), _contentLength, bufferSize, minGzipSize);
            } else if (_writer != null) {
                throw new IllegalStateException("getWriter() called");
            }

            return _gzStream;
        }

        /*
         * (non-Javadoc)
         * 
         * @see javax.servlet.ServletResponseWrapper#getWriter()
         */
        @Override
        public PrintWriter getWriter() throws IOException {
            if (_writer == null) {
                if (_gzStream != null) {
                    throw new IllegalStateException("getOutputStream() called");
                }

                if (getResponse().isCommitted() || _noGzip) {
                    return getResponse().getWriter();
                }

                _gzStream =
                        newGzipStream(_request, (HttpServletResponse) getResponse(), _contentLength, bufferSize, minGzipSize);
                String encoding = getCharacterEncoding();
                _writer =
                        encoding == null ? new PrintWriter(_gzStream) : new PrintWriter(new OutputStreamWriter(_gzStream,
                                encoding));
            }
            return _writer;
        }

        /**
         * No gzip.
         */
        void noGzip() {
            _noGzip = true;
            if (_gzStream != null) {
                try {
                    _gzStream.doNotGzip();
                } catch (IOException e) {
                    throw new IllegalStateException();
                }
            }
        }

        /**
         * Finish.
         * 
         * @throws IOException
         *             Signals that an I/O exception has occurred.
         */
        void finish() throws IOException {
            if (_writer != null) {
                _writer.flush();
            }
            if (_gzStream != null) {
                _gzStream.finish();
            }
        }

        /**
         * New gzip stream.
         * 
         * @param request
         *            the request
         * @param response
         *            the response
         * @param contentLength
         *            the content length
         * @param bufferSize
         *            the buffer size
         * @param minGzipSize
         *            the min gzip size
         * @return the gzip stream
         * @throws IOException
         *             Signals that an I/O exception has occurred.
         */
        protected GzipStream newGzipStream(HttpServletRequest request, HttpServletResponse response, long contentLength,
                int bufferSize, int minGzipSize) throws IOException {
            return new GzipStream(request, response, contentLength, bufferSize, minGzipSize);
        }
    }

    /**
     * The Class GzipStream.
     */
    public static final class GzipStream extends ServletOutputStream {

        /** The _request. */
        protected HttpServletRequest _request;

        /** The _response. */
        protected HttpServletResponse _response;

        /** The _out. */
        protected OutputStream _out;

        /** The _b out. */
        protected ByteArrayOutputStream2 _bOut;

        /** The _gz out. */
        protected GZIPOutputStream _gzOut;

        /** The _closed. */
        protected boolean _closed;

        /** The _buffer size. */
        protected int _bufferSize;

        /** The _min gzip size. */
        protected int _minGzipSize;

        /** The _content length. */
        protected long _contentLength;

        /**
         * Instantiates a new gzip stream.
         * 
         * @param request
         *            the request
         * @param response
         *            the response
         * @param contentLength
         *            the content length
         * @param bufferSize
         *            the buffer size
         * @param minGzipSize
         *            the min gzip size
         * @throws IOException
         *             Signals that an I/O exception has occurred.
         */
        public GzipStream(
                HttpServletRequest request, HttpServletResponse response, long contentLength, int bufferSize, int minGzipSize)
                throws IOException {
            _request = request;
            _response = response;
            _contentLength = contentLength;
            _bufferSize = bufferSize;
            _minGzipSize = minGzipSize;
            if (minGzipSize == 0) {
                doGzip();
            }
        }

        /**
         * Reset buffer.
         */
        public void resetBuffer() {
            _closed = false;
            _out = null;
            _bOut = null;
            if (_gzOut != null && !_response.isCommitted()) {
                _response.setHeader("Content-Encoding", null);
            }
            _gzOut = null;
        }

        /**
         * Sets the content length.
         * 
         * @param length
         *            the new content length
         */
        public void setContentLength(long length) {
            _contentLength = length;
        }

        /*
         * (non-Javadoc)
         * 
         * @see java.io.OutputStream#flush()
         */
        @Override
        public void flush() throws IOException {
            if (_out == null || _bOut != null) {
                if (_contentLength > 0 && _contentLength < _minGzipSize) {
                    doNotGzip();
                } else {
                    doGzip();
                }
            }

            _out.flush();
        }

        /*
         * (non-Javadoc)
         * 
         * @see java.io.OutputStream#close()
         */
        @Override
        public void close() throws IOException {
            if (_request.getAttribute("javax.servlet.include.request_uri") != null) {
                flush();
            } else {
                if (_bOut != null) {
                    if (_contentLength < 0) {
                        _contentLength = _bOut.getCount();
                    }
                    if (_contentLength < _minGzipSize) {
                        doNotGzip();
                    } else {
                        doGzip();
                    }
                } else if (_out == null) {
                    doNotGzip();
                }

                if (_gzOut != null) {
                    _gzOut.finish();
                }
                _out.close();
                _closed = true;
            }
        }

        /**
         * Finish.
         * 
         * @throws IOException
         *             Signals that an I/O exception has occurred.
         */
        public void finish() throws IOException {
            if (!_closed) {
                if (_out == null || _bOut != null) {
                    if (_contentLength > 0 && _contentLength < _minGzipSize) {
                        doNotGzip();
                    } else {
                        doGzip();
                    }
                }

                if (_gzOut != null) {
                    _gzOut.finish();
                }
            }
        }

        /*
         * (non-Javadoc)
         * 
         * @see java.io.OutputStream#write(int)
         */
        @Override
        public void write(int b) throws IOException {
            checkOut(1);
            _out.write(b);
        }

        /*
         * (non-Javadoc)
         * 
         * @see java.io.OutputStream#write(byte[])
         */
        @Override
        public void write(byte b[]) throws IOException {
            checkOut(b.length);
            _out.write(b);
        }

        /*
         * (non-Javadoc)
         * 
         * @see java.io.OutputStream#write(byte[], int, int)
         */
        @Override
        public void write(byte b[], int off, int len) throws IOException {
            checkOut(len);
            _out.write(b, off, len);
        }

        /**
         * Sets the content encoding gzip.
         * 
         * @return true, if successful
         */
        protected boolean setContentEncodingGzip() {
            _response.setHeader("Content-Encoding", "gzip");
            return _response.containsHeader("Content-Encoding");
        }

        /**
         * Do gzip.
         * 
         * @throws IOException
         *             Signals that an I/O exception has occurred.
         */
        public void doGzip() throws IOException {
            if (_gzOut == null) {
                if (_response.isCommitted()) {
                    throw new IllegalStateException();
                }

                if (setContentEncodingGzip()) {
                    _out = _gzOut = new GZIPOutputStream(_response.getOutputStream(), _bufferSize);

                    if (_bOut != null) {
                        _out.write(_bOut.getBuf(), 0, _bOut.getCount());
                        _bOut = null;
                    }
                } else {
                    doNotGzip();
                }
            }
        }

        /**
         * Do not gzip.
         * 
         * @throws IOException
         *             Signals that an I/O exception has occurred.
         */
        public void doNotGzip() throws IOException {
            if (_gzOut != null) {
                throw new IllegalStateException();
            }
            if (_out == null || _bOut != null) {
                _out = _response.getOutputStream();
                if (_contentLength >= 0) {
                    if (_contentLength < Integer.MAX_VALUE) {
                        _response.setContentLength((int) _contentLength);
                    } else {
                        _response.setHeader("Content-Length", Long.toString(_contentLength));
                    }
                }

                if (_bOut != null) {
                    _out.write(_bOut.getBuf(), 0, _bOut.getCount());
                }
                _bOut = null;
            }
        }

        /**
         * Check out.
         * 
         * @param length
         *            the length
         * @throws IOException
         *             Signals that an I/O exception has occurred.
         */
        private void checkOut(int length) throws IOException {
            if (_closed) {
                new Throwable().printStackTrace();
                throw new IOException("CLOSED");
            }

            if (_out == null) {
                if (_response.isCommitted() || (_contentLength >= 0 && _contentLength < _minGzipSize)) {
                    doNotGzip();
                } else if (length > _minGzipSize) {
                    doGzip();
                } else {
                    _out = _bOut = new ByteArrayOutputStream2(_bufferSize);
                }
            } else if (_bOut != null) {
                if (_response.isCommitted() || (_contentLength >= 0 && _contentLength < _minGzipSize)) {
                    doNotGzip();
                } else if (length >= (_bOut.size() - _bOut.getCount())) {
                    doGzip();
                }
            }
        }
    }

    /**
     * The Class ByteArrayOutputStream2.
     */
    public static final class ByteArrayOutputStream2 extends ByteArrayOutputStream {

        /**
         * Instantiates a new byte array output stream2.
         */
        public ByteArrayOutputStream2() {
            super();
        }

        /**
         * Instantiates a new byte array output stream2.
         * 
         * @param size
         *            the size
         */
        public ByteArrayOutputStream2(int size) {
            super(size);
        }

        /**
         * Gets the buf.
         * 
         * @return the buf
         */
        public byte[] getBuf() {
            return buf;
        }

        /**
         * Gets the count.
         * 
         * @return the count
         */
        public int getCount() {
            return count;
        }

        /**
         * Sets the count.
         * 
         * @param count
         *            the new count
         */
        public void setCount(int count) {
            this.count = count;
        }

        /**
         * Reset.
         * 
         * @param minSize
         *            the min size
         */
        public void reset(int minSize) {
            reset();
            if (buf.length < minSize) {
                buf = new byte[minSize];
            }
        }

        /**
         * Write unchecked.
         * 
         * @param b
         *            the b
         */
        public void writeUnchecked(int b) {
            buf[count++] = (byte) b;
        }

    }

    /**
     * Gets the buffer size.
     * 
     * @return the buffer size
     */
    public int getBufferSize() {
        return bufferSize;
    }

    /**
     * Sets the buffer size.
     * 
     * @param bufferSize
     *            the new buffer size
     */
    public void setBufferSize(int bufferSize) {
        this.bufferSize = bufferSize;
    }

    /**
     * Gets the min gzip size.
     * 
     * @return the min gzip size
     */
    public int getMinGzipSize() {
        return minGzipSize;
    }

    /**
     * Sets the min gzip size.
     * 
     * @param minGzipSize
     *            the new min gzip size
     */
    public void setMinGzipSize(int minGzipSize) {
        this.minGzipSize = minGzipSize;
    }
}
