/**
 * 
 */
package gov.va.med.pharmacy.pecs.web.history;

import gov.va.med.pharmacy.pecs.criteria.ConceptHistoryCriteria;
import gov.va.med.pharmacy.pecs.service.DoseRangeService;
import gov.va.med.pharmacy.pecs.service.DrugDrugInteractionService;
import gov.va.med.pharmacy.pecs.service.DuplicateTherapyService;
import gov.va.med.pharmacy.pecs.service.MonographService;
import gov.va.med.pharmacy.pecs.service.dto.FieldHistoryLineItem;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;

import org.springframework.beans.factory.InitializingBean;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;

/**
 * Accesses the services and returns the history for a record.
 * 
 * @author vhaisacarusr
 * 
 */
@Controller
@RequestMapping("history")
public final class HistoryController implements InitializingBean {

    @Resource
    private DuplicateTherapyService dtCatTherapyService;

    @Resource
    private DoseRangeService doseRangeService;

    @Resource
    private DrugDrugInteractionService drugDrugInteractionService;

    @Resource
    private MonographService monographService;

    /**
     * Gets the history of the required history for duplicate therapy.
     * 
     * @param recordId
     *            the id of the record to compare
     * @return a list of field history items as the response body
     */
    @RequestMapping("DUPLICATE_THERAPY/{recordId}")
    @ResponseBody
    public Map<String, List<FieldHistoryLineItem>> retrieveRequiredDuplicateTherapyHistory(
            @PathVariable(value = "recordId") Long recordId) {
        Map<String, List<FieldHistoryLineItem>> history = new HashMap<String, List<FieldHistoryLineItem>>();
        ConceptHistoryCriteria conceptHistoryCriteria = new ConceptHistoryCriteria();
        conceptHistoryCriteria.setId(recordId);
        history = dtCatTherapyService.getRequiredHistory(conceptHistoryCriteria);
        return history;
    }

    /**
     * Gets the history of the required history for DDI.
     * 
     * @param recordId
     *            the id of the record to compare
     * @return a list of field history items as the response body
     */
    @RequestMapping("DRUG_DRUG_INTERACTION/{recordId}")
    @ResponseBody
    public Map<String, List<FieldHistoryLineItem>> retrieveRequiredDrugDrugInteraction(
            @PathVariable(value = "recordId") Long recordId) {
        Map<String, List<FieldHistoryLineItem>> history = new HashMap<String, List<FieldHistoryLineItem>>();
        ConceptHistoryCriteria conceptHistoryCriteria = new ConceptHistoryCriteria();
        conceptHistoryCriteria.setVersionId(recordId);
        history = drugDrugInteractionService.getRequiredHistory(conceptHistoryCriteria);
        return history;
    }

    /**
     * Gets the history of the required history for dose range.
     * 
     * @param recordId
     *            the id of the record to compare
     * @return a list of field history items as the response body
     */
    @RequestMapping("DOSE_RANGE/{recordId}")
    @ResponseBody
    //@formatter:off
    public Map<String, List<FieldHistoryLineItem>> retrieveRequiredDoseRange(
            @PathVariable(value = "recordId") Long recordId) {
    //@formatter:on
        Map<String, List<FieldHistoryLineItem>> history = new HashMap<String, List<FieldHistoryLineItem>>();
        ConceptHistoryCriteria conceptHistoryCriteria = new ConceptHistoryCriteria();
        conceptHistoryCriteria.setVersionId(recordId);
        history = doseRangeService.getRequiredHistory(conceptHistoryCriteria);
        return history;
    }

    /**
     * Gets the history of the required history for Professional Monograph.
     * 
     * @param recordId
     *            the id of the record to compare
     * @return a list of field history items as the response body
     */
    @RequestMapping("PROFESSIONAL_MONOGRAPH/{recordId}")
    @ResponseBody
    public Map<String, List<FieldHistoryLineItem>> retrieveRequiredProfessionalMonograph(
            @PathVariable(value = "recordId") Long recordId) {
        Map<String, List<FieldHistoryLineItem>> history = new HashMap<String, List<FieldHistoryLineItem>>();
        ConceptHistoryCriteria conceptHistoryCriteria = new ConceptHistoryCriteria();
        conceptHistoryCriteria.setVersionId(recordId);
        history = monographService.getRequiredHistory(conceptHistoryCriteria);
        return history;
    }

    /**
     * Gets the dtCatTherapyService.
     * 
     * @return the dtCatTherapyService
     */
    public DuplicateTherapyService getDtCatTherapyService() {
        return dtCatTherapyService;
    }

    /**
     * Sets the dtCatTherapyService.
     * 
     * @param dtCatTherapyService
     *            the dtCatTherapyService to set
     */
    public void setDtCatTherapyService(DuplicateTherapyService dtCatTherapyService) {
        this.dtCatTherapyService = dtCatTherapyService;
    }

    /**
     * Gets the doseRangeService.
     * 
     * @return the doseRangeService
     */
    public DoseRangeService getDoseRangeService() {
        return doseRangeService;
    }

    /**
     * Sets the doseRangeService.
     * 
     * @param doseRangeService
     *            the doseRangeService to set
     */
    public void setDoseRangeService(DoseRangeService doseRangeService) {
        this.doseRangeService = doseRangeService;
    }

    /**
     * Gets the drugDrugInteractionService.
     * 
     * @return the drugDrugInteractionService
     */
    public DrugDrugInteractionService getDrugDrugInteractionService() {
        return drugDrugInteractionService;
    }

    /**
     * Sets the drugDrugInteractionService.
     * 
     * @param drugDrugInteractionService
     *            the drugDrugInteractionService to set
     */
    public void setDrugDrugInteractionService(DrugDrugInteractionService drugDrugInteractionService) {
        this.drugDrugInteractionService = drugDrugInteractionService;
    }

    /**
     * Gets the monographService.
     * 
     * @return the monographService
     */
    public MonographService getMonographService() {
        return monographService;
    }

    /**
     * Sets the monographService.
     * 
     * @param monographService
     *            the monographService to set
     */
    public void setMonographService(MonographService monographService) {
        this.monographService = monographService;
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        if (dtCatTherapyService == null) {
            throw new Exception("duplicate therapy service not initialzed");
        }

    }

}
