package gov.va.med.pharmacy.pecs.web.lookup.drugpair;

import gov.va.med.pharmacy.pecs.domain.impl.lookup.SeverityLevel;
import gov.va.med.pharmacy.pecs.dto.DrugPairDTO;
import gov.va.med.pharmacy.pecs.dto.PecsDTO;
import gov.va.med.pharmacy.pecs.metadata.ConceptField;
import gov.va.med.pharmacy.pecs.query.ConceptQuery;
import gov.va.med.pharmacy.pecs.query.ConceptQueryFactory;
import gov.va.med.pharmacy.pecs.query.DoJoGridSuport;
import gov.va.med.pharmacy.pecs.query.Page;
import gov.va.med.pharmacy.pecs.service.DrugPairService;
import gov.va.med.pharmacy.pecs.service.TerminologyService;
import gov.va.med.pharmacy.pecs.web.AbstractController;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

/**
 * The Class DrugPairLookupController.
 */
@Controller
@RequestMapping(value = "/drugPairLookup")
public class DrugPairLookupController extends AbstractController implements InitializingBean {

    /** The logger. */
    private Log logger = LogFactory.getLog(DrugPairLookupController.class);

    /** The terminology service. */
    @Resource
    private TerminologyService terminologyService;

    /** The drug pair service. */
    @Resource
    private DrugPairService drugPairService;

    /** The factory. */
    @Resource(name = "jsonConceptQueryFactory")
    private ConceptQueryFactory factory;

    /** The tiles view. */
    private final String TILES_VIEW = "pecs.drugpairlookup";

    /** The criteria. */
    private final String CRITERIA = "drugPairLookupCriteria";

    /** The va query. */
    private final String VA_QUERY = "stagingQuery";

    /** The fdb query. */
    private final String FDB_QUERY = "fdbQuery";

    /** The criteria. */
    private DrugPairLookupCriteria criteria = new DrugPairLookupCriteria();

    /**
     * Populate severity level codes.
     * 
     * @return the list
     */
    @ModelAttribute("severityLevels")
    public List<SeverityLevel> populateSeverityLevelCodes() {
        List<SeverityLevel> levels = new ArrayList<SeverityLevel>();
        List<SeverityLevel> tempLevels = terminologyService.getSeverityLevels();
        if (tempLevels.get(0).getCode() != null) {
            levels.add(new SeverityLevel());
            levels.addAll(tempLevels);
        }
        return levels;
    }

    /**
     * Populate criteria.
     * 
     * @return the drug pair lookup criteria
     */
    @ModelAttribute(CRITERIA)
    public DrugPairLookupCriteria populateCriteria() {
        return criteria;
    }

    /**
     * Populate staging data.
     * 
     * @param json
     *            the json
     * @param response
     *            the Servlet Response   
     * @return the list
     * @throws Exception
     *             the exception
     */
    @RequestMapping(value = VA_QUERY)
    @ResponseBody
    public List<DrugPairDTO> populateStagingData(@RequestParam("json") String json, HttpServletResponse response) 
            throws Exception {
        return executeQuery(json, "VA", response);
    }

    /**
     * Populate fdb data.
     * 
     * @param json
     *            the json
     * @param response
     *            the Servlet Response
     * @return the list
     * @throws Exception
     *             the exception
     */
    @RequestMapping(value = FDB_QUERY)
    @ResponseBody
    public List<DrugPairDTO> populateFdbData(@RequestParam("json") String json, HttpServletResponse response) 
            throws Exception {
        return executeQuery(json, "FDB", response);
    }

    /**
     * Display.
     * 
     * @param request
     *            the request        
     * @return the string
     */
    @RequestMapping(method = RequestMethod.GET)
    public String display(HttpServletRequest request) {
        return TILES_VIEW;
    }

    /**
     * Execute query.
     * 
     * @param json
     *            the json
     * @param recordType
     *            the record type
     * @param response
     *            the Servlet Response
     * @return the list
     */
    @SuppressWarnings("unchecked")
    private List<DrugPairDTO> executeQuery(String json, String recordType, HttpServletResponse response) {
        
        logger.info(json);

        boolean va = false;
        if ("VA".equals(recordType)) {
            va = true;
        } else if ("FDB".equals(recordType)) {
            va = false;
        } else {
            throw new IllegalArgumentException("Invalid record type: " + recordType);
        }
        
        Page page;
        DoJoGridSuport support = new DoJoGridSuport(json);
        if (support.isParsed()) {
            ConceptQuery query = factory.createQuery(drugPairService.getMetadata(), support.getJson());
            query.setPaged(true);
            if (support.hasSorting()) {
                ConceptField field = drugPairService.getMetadata().getFieldByProperty(support.getSortColumn());
                query.setSortFields(Arrays.asList(new ConceptField[] { field }));
                query.setSortOrder(support.sortAsc());
            } else {
                ConceptField field = drugPairService.getMetadata().getField(PecsDTO.ACTION_DATE);
                query.setSortFields(Arrays.asList(new ConceptField[] { field }));
                query.setSortOrder(false);
            }
            if (va) {
                page = drugPairService.getStagingData(query, support.getPageNumber());
            } else {
                page = drugPairService.getFdbData(query, support.getPageNumber());
            }
            response.setHeader("Content-Range", "items=" + support.getStartRow() + "-" + support.getEndRow() + "/" 
                    + page.getQuerySize());            
            
            return page.getResults();
        }
        return Collections.emptyList();
    }

    /**
     * Sets the drug pair service.
     * 
     * @param drugPairService
     *            the new drug pair service
     */
    public void setdrugPairService(DrugPairService drugPairService) {
        this.drugPairService = drugPairService;
    }

    /**
     * Sets the concept query factory.
     * 
     * @param factory
     *            the new concept query factory
     */
    public void setConceptQueryFactory(ConceptQueryFactory factory) {
        this.factory = factory;
    }

    /**
     * Gets the terminology service.
     * 
     * @return the terminology service
     */
    public TerminologyService getTerminologyService() {
        return terminologyService;
    }

    /**
     * Sets the terminology service.
     * 
     * @param terminologyService
     *            the new terminology service
     */
    public void setTerminologyService(TerminologyService terminologyService) {
        this.terminologyService = terminologyService;
    }

    /**
     * The after properties set .
     * 
     * @throws Exception
     *    the Exception
     */
    public void afterPropertiesSet() throws Exception {
        if (terminologyService == null) {
            throw new Exception(findMessage("message.resource.not.set", new Object[] { "TerminologyService" }));
        }
        if (drugPairService == null) {
            throw new Exception(findMessage("message.resource.not.set", new Object[] { "drugPairService" }));
        }
        if (factory == null) {
            throw new Exception(findMessage("message.resource.not.set", new Object[] { "ConceptQueryFactory" }));
        }
    }
}
