package gov.va.med.pharmacy.pecs.web.monograph.professional;

import gov.va.med.pharmacy.pecs.domain.impl.va.MonographKey;
import gov.va.med.pharmacy.pecs.domain.impl.va.MonographXref;
import gov.va.med.pharmacy.pecs.domain.impl.va.UserProfile;
import gov.va.med.pharmacy.pecs.dto.MonographDTO;
import gov.va.med.pharmacy.pecs.exception.CustomizationReportException;
import gov.va.med.pharmacy.pecs.exception.CustomizationServiceException;
import gov.va.med.pharmacy.pecs.exception.DuplicateConceptException;
import gov.va.med.pharmacy.pecs.exception.ValidationException;
import gov.va.med.pharmacy.pecs.metadata.Concept;
import gov.va.med.pharmacy.pecs.metadata.ConceptField;
import gov.va.med.pharmacy.pecs.metadata.ConceptMetadata;
import gov.va.med.pharmacy.pecs.rules.InternalFieldsRule;
import gov.va.med.pharmacy.pecs.rules.ValidationResult;
import gov.va.med.pharmacy.pecs.service.MonographService;
import gov.va.med.pharmacy.pecs.service.UserService;
import gov.va.med.pharmacy.pecs.state.Status;
import gov.va.med.pharmacy.pecs.web.AbstractController;
import gov.va.med.pharmacy.pecs.web.RecordType;
import gov.va.med.pharmacy.pecs.web.detail.LockingResult;
import gov.va.med.pharmacy.pecs.web.detail.RecordLockingHelper;
import gov.va.med.pharmacy.pecs.web.view.ReportHistoryXLSXViewer;
import gov.va.med.pharmacy.pecs.web.view.document.AbstractXLSXViewer;

import java.io.IOException;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.stereotype.Controller;
import org.springframework.validation.BindingResult;
import org.springframework.validation.ObjectError;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.View;
import org.springframework.web.servlet.view.RedirectView;

/**
 * The controller for the professional monograph page.
 * 
 * @author vhaisacarusr
 * 
 */
@Controller
@RequestMapping(value = "/monograph/professional")
public final class ProfessionalMonographDetailController extends AbstractController implements InitializingBean {

    /** The Constant FDB_RECORD_CUTOFF. */
    private static final int FDB_RECORD_CUTOFF = 150000;

    /** The user service. */
    @Resource
    private UserService userService;

    /** The monograph service. */
    @Resource
    private MonographService monographService;

    /** The record locking helper. */
    @Resource
    private RecordLockingHelper recordLockingHelper;

    /** The Constant VIEW. */
    private static final String VIEW = "pecs.monographdetail";

    /** The Constant LOGGER. */
    private static final Log LOGGER = LogFactory.getLog(ProfessionalMonographDetailController.class);

    /** The read only record. */
    private boolean readOnlyRecord;

    /** The Constant SELECT_UNASSIGNED. */
    private static final String SELECT_UNASSIGNED = "UNASSIGNED";

    /** The Constant HISTORY_REPORT. */
    private static final String HISTORY_REPORT = "HISTORY_REPORT";

    /**
     * This Method is the mapping for the History Report Generator. It is used to generate the Excel Report.
     * 
     * @param recordId
     *            the record id
     * @param response
     *            the response
     * @return Model and View for a History VA Professional Monograph.
     * @throws CustomizationReportException
     *             the customization report exception
     */
    @RequestMapping(value = "/history/VA/{recordId}", method = RequestMethod.GET)
    public ModelAndView executeQuery(@PathVariable(value = "recordId") String recordId, HttpServletResponse response)
            throws CustomizationReportException {
        Map<String, Object> model = new HashMap<String, Object>();
        model.put(ReportHistoryXLSXViewer.METADATA_KEY, Concept.PROFESSIONAL_MONOGRAPH);
        model.put(ReportHistoryXLSXViewer.DATA_KEY, monographService.getVersionHistoryReportData(Long.parseLong(recordId)));
        model.put(AbstractXLSXViewer.EXCEL_FILE_NAME_KEY, Concept.PROFESSIONAL_MONOGRAPH);
        model.put(AbstractXLSXViewer.EXCEL_FILE_TYPE_KEY, HISTORY_REPORT);
        return new ModelAndView(ReportHistoryXLSXViewer.REPORT_HISTORY_VIEW, model);

    }

    /**
     * This method takes a request and outputs the response for a blank form so the user can create a Professional Monograph
     * from scratch.
     * 
     * @return a ModelAndView object for a blank form
     */
    @RequestMapping(value = "/blank", method = RequestMethod.GET)
    public ModelAndView loadBlank() {
        MonographDTO monographDTO = new MonographDTO();
        List<String> users = getAdminUsers();
        ConceptMetadata conceptMetadata = monographService.getMetadata();
        Map<String, ConceptField> fieldMap = conceptMetadata.getFieldMap();
        ModelAndView mav = new ModelAndView(VIEW);
        mav.addObject("fieldMap", fieldMap);
        mav.addObject("showVaFields", false);
        mav.addObject("allowableActions", Collections.singleton("Customize"));
        mav.addObject("users", users);
        mav.addObject("monographDTO", monographDTO);
        return mav;
    }

    /**
     * This method takes a request and outputs the response for a blank form so the user can create a Professional Monograph
     * from scratch.
     * 
     * @param monographDTO
     *            the monograph object passed in from the form
     * @param bindingResult
     *            holds errors if applicable
     * @param request
     *            the HTTP servlet request
     * @return a ModelAndView object for a blank form
     */
    @RequestMapping(value = "/blank", method = RequestMethod.POST)
    public ModelAndView saveBlank(@ModelAttribute("monographDTO") MonographDTO monographDTO, BindingResult bindingResult,
            HttpServletRequest request) {

        monographDTO = save(monographDTO.getRecordId(), request, monographDTO, bindingResult);

        ModelAndView mav = setUpForDisplay(monographDTO, request);

        if (bindingResult.hasErrors()) {
            return mav;
        }

        String url = "VA/" + monographDTO.getFdbCustomMonographVaId();

        RedirectView redirectView = new RedirectView(url, true, false, false);
        mav.setView(redirectView);
        return mav;

    }

    /**
     * Load the monograph given the monograph ID.
     * 
     * @param monographId
     *            the ID of the monograph to load
     * @param request
     *            the HTTP request
     * @param response
     *            the HTTP response
     * @return a model and view to display the page
     */
    @RequestMapping(value = "/monographid/{monographId}", method = RequestMethod.GET)
    public ModelAndView loadFromMonographId(@PathVariable(value = "monographId") Long monographId, HttpServletRequest request,
            HttpServletResponse response) {
        MonographDTO monographDTO = null;
        if (monographId < FDB_RECORD_CUTOFF) {
            monographDTO = monographService.findFdbMonographById(monographId);
            String url =
                    request.getContextPath() + request.getServletPath() + "/monograph/professional/FDB/"
                            + monographDTO.getMonographid();
            try {
                response.sendRedirect(url);
                return new ModelAndView();
            } catch (IOException e) {
                CustomizationServiceException cse = new CustomizationServiceException(e);
                LOGGER.error("an error occured when trying ot redirect to " + url, cse);
                throw cse;
            }

        }
        monographDTO = monographService.findMonographById(monographId);
        String url =
                request.getContextPath() + request.getServletPath() + "/monograph/professional/VA/"
                        + monographDTO.getFdbCustomMonographVaId();
        try {
            response.sendRedirect(url);
            return new ModelAndView();
        } catch (IOException e) {
            CustomizationServiceException cse = new CustomizationServiceException(e);
            LOGGER.error("an error occured when trying ot redirect to " + url, cse);
            throw cse;
        }

    }

    /**
     * Load va.
     * 
     * @param recordId
     *            the record id
     * @param request
     *            the request
     * @param response
     *            the response
     * @return the model and view
     */
    @RequestMapping(value = "/VA/{recordId}", method = RequestMethod.GET)
    public ModelAndView loadVa(@PathVariable(value = "recordId") String recordId, HttpServletRequest request,
            HttpServletResponse response) {
        if (LOGGER.is   ugEnabled())
            LOGGER.debug("Received from ActionContext - monographid: " + recordId);
        MonographDTO monographDTO = new MonographDTO();
        ModelAndView mav = new ModelAndView(VIEW);
        mav.addObject("showVaFields", true);

        ConceptMetadata conceptMetadata = monographService.getMetadata();
        Map<String, ConceptField> fieldMap = conceptMetadata.getFieldMap();
        fieldMap.size();
        if (recordId == null || recordId.equals("")) {
            findMessage("monograph.wizard.required.recordid");
        }
        long monographRecordId = 0;
        try {
            monographRecordId = Long.parseLong(recordId);
        } catch (NumberFormatException e) {
            findMessage("monograph.wizard.required.recordid");
        }
        try {
            // Retrieve customized model
            monographDTO = monographService.findMonographByVersionId(monographRecordId);

            monographDTO.setReadOnlyRecord(!monographDTO.isCurrent());

        } catch (CustomizationServiceException cex) {
            LOGGER.error("Unable to retrieve monograph version with id: " + monographRecordId);
            findMessage("monograph.notfound");
        }
        // Display result back to GUI

        mav.addObject("allowableActions", Collections.emptySet());
        mav.addObject("monographDTO", monographDTO);
        mav.addObject("fieldMap", fieldMap);
        List<String> users = getAdminUsers();
        mav.addObject("users", users);
        mav.addObject("showCommentButton", false);

        Set<String> actions = Collections.emptySet();
        if (!monographDTO.isReadOnlyRecord()) {
            if (monographDTO.isCurrent()) {
                if (monographDTO.getFdbCustomMonographVaId() != 0) {
                    actions = monographService.getAllowableActions(monographDTO, getUserProfile(request));
                }
                permissionValidation(getUserProfile(request), actions, mav, monographDTO.getFdbCustomMonographVaId());
            }

        }
        return mav;
    }

    /**
     * Loads an FDB record.
     * 
     * @param recordId
     *            the record ID
     * @param request
     *            the HTTPRequest
     * @param response
     *            the HTTPResponse
     * @return a model and view
     */
    @RequestMapping(value = "/FDB/{recordId}", method = RequestMethod.GET)
    public ModelAndView loadFdb(@PathVariable(value = "recordId") String recordId, HttpServletRequest request,
            HttpServletResponse response) {

        ModelAndView mav = new ModelAndView(VIEW);
        MonographDTO fdbMonograph = new MonographDTO();
        ConceptMetadata conceptMetadata = monographService.getMetadata();
        Map<String, ConceptField> fieldMap = conceptMetadata.getFieldMap();
        if (recordId == null || recordId.equals("")) {
            throw new IllegalArgumentException();
        }

        // See if there is a custom monograph already created for this fdb
        // monograph
        MonographDTO customMonographDTO = null;
        customMonographDTO = retrieveCustomModelByCustomizedMonograph(recordId);
        if (customMonographDTO != null) {
            // Navigate to the existing custom record instead of creating a
            // new one
            MonographDTO fdbMon = retrieveFdbMonograph(recordId);
//@formatter:off
                Object[] arg = { String.valueOf(customMonographDTO.getFdbCustomMonographVaId()), 
                        String.valueOf(customMonographDTO.getMonographid()) };
//@formatter:on
            addInformationMessage("message.wizard.information.vaexist", arg);

            fdbMon.setCurrent(false);

            List<String> users = getAdminUsers();
            mav.addObject("showVaFields", false);
            mav.addObject("monographDTO", fdbMon);
            mav.addObject("fieldMap", fieldMap);
            mav.addObject("users", users);
            mav.addObject("allowableActions", Collections.emptySet());
            mav.addObject("showCommentButton", showCommentButton(customMonographDTO));
            return mav;
        }

        fdbMonograph = retrieveFdbMonograph(recordId);
        if (fdbMonograph == null) {

            String inf = findMessage("monograph.wizard.fdb.notfound");
            addInformationMessage(inf);
            List<String> users = getAdminUsers();

            mav.addObject("showVaFields", false);
            mav.addObject("monographDTO", new MonographDTO());
            mav.addObject("fieldMap", fieldMap);
            mav.addObject("users", users);
            mav.addObject("users", userService.findApprovers());
            return mav;

        }

        // Display result back to GUI

        List<String> users = getAdminUsers();
        mav.addObject("showVaFields", false);
        mav.addObject("monographDTO", fdbMonograph);
        mav.addObject("fieldMap", fieldMap);
        mav.addObject("users", users);
        mav.addObject("allowableActions", Collections.emptySet());
        mav.addObject("showCommentButton", true);

        Set<String> actions = Collections.emptySet();
        if (!fdbMonograph.isReadOnlyRecord()) {
            if (fdbMonograph.isCurrent()) {
                if (fdbMonograph.getFdbCustomMonographVaId() != 0) {
                    actions = monographService.getAllowableActions(fdbMonograph, getUserProfile(request));
                }
                permissionValidation(getUserProfile(request), actions, mav, fdbMonograph.getFdbCustomMonographVaId());
            }
        }

        return mav;

    }

    /**
     * Perform action.
     * 
     * @param recordId
     *            the record id
     * @param recordType
     *            the record type
     * @param request
     *            the request
     * @param response
     *            the response
     * @param monographDTO
     *            the monograph dto
     * @param bindingResult
     *            the binding result
     * @return the model and view
     * @throws IOException
     *             Signals that an I/O exception has occurred.
     */
    @RequestMapping(value = "/{recordType}/{recordId}", method = RequestMethod.POST)
    public ModelAndView performAction(@PathVariable(value = "recordId") String recordId,
            @PathVariable(value = "recordType") String recordType, HttpServletRequest request, HttpServletResponse response,
            @ModelAttribute("monographDTO") MonographDTO monographDTO, BindingResult bindingResult) throws IOException {

        ModelAndView modelAndView = new ModelAndView(VIEW);
        modelAndView.addObject("showVaFields", true);
        String action = request.getParameter("action");
        action = action.replace(' ', '_');
        action = action.replace("For", "for");
        action = action.replace("As", "as");
        if (RecordType.FDB.equals(RecordType.fromString(recordType))) {

            monographDTO.setFdbId(Long.valueOf(recordId));

            MonographXref xref = new MonographXref();
            MonographKey key = monographService.populateMonographId(monographDTO);
            xref.setMonographKey(key);
            monographDTO.setConceptXref(xref);

            monographDTO = save(recordId, request, monographDTO, bindingResult);

            if (bindingResult.hasErrors()) {

                modelAndView = setUpForDisplay(monographDTO, request);
                modelAndView.addObject("showCommentButton", showCommentButton(monographDTO));
                return modelAndView;
            }
        } else {
            try {
                monographDTO.setFdbCustomMonographVaId(Long.valueOf(recordId));
                // Set action by
                processUserProfile(request, monographDTO);
                // Modify the customization
                String prevStatus = monographDTO.getStatus();
                monographDTO = monographService.applyAction(action, monographDTO, bindingResult);
                final MonographDTO oldMonograph = monographService.findMonographByVersionId(Long.valueOf(recordId));
                InternalFieldsRule fieldsRule = new InternalFieldsRule();
                fieldsRule.setMetadata(monographService.getMetadata());
                ValidationResult result = fieldsRule.validate(monographDTO, oldMonograph);
                boolean fdbChange = true;
                if (result.getMessages().isEmpty()) {
                    fdbChange = false;
                }
                if (bindingResult.hasErrors()) {
                    modelAndView.addObject("showCommentButton", false);
                    return setupForDisplay2(request, monographDTO, modelAndView);
                } else {
                    createMessages(monographDTO, action, prevStatus, fdbChange);
                }
                monographDTO.setCurrentActionReason("");

            } catch (ValidationException ex) {
                for (String m : ex.getMessages()) {
                    // this.addActionError(m);
                    LOGGER.debug(m);
                }
                LOGGER.error("Data validation error", ex);
            } catch (CustomizationServiceException ex) {
                // this.addActionError(getText("error.modify") + " " +
                // ex.getMessage());
                LOGGER.error("Unable to modify monograph models: ", ex);
            }
        }

        if (RecordType.FDB.equals(RecordType.fromString(recordType))) {
            recordId = String.valueOf(monographDTO.getFdbId());
        }
        cancelLock(recordType, recordId, request);

        String url =
                request.getContextPath() + request.getServletPath() + "/monograph/professional/VA/"
                        + monographDTO.getFdbCustomMonographVaId();
        View view = new RedirectView(url, false, false, false);
        modelAndView.setView(view);
        return modelAndView;
        // return setUpForDisplay(request);
    }

    /**
     * Process user profile.
     * 
     * @param request
     *            the request
     * @param monographDTO
     *            the monograph dto
     */
    private void processUserProfile(HttpServletRequest request, MonographDTO monographDTO) {
        UserProfile profile = getUserProfile(request);
        monographDTO.setRequestor(profile.getUsername());
        monographDTO.setActionBy(profile.getUsername());
        monographDTO.setActionUser(profile);

        // Set assigned to
        if (profile.isApprover() && monographDTO.getAssignedTo() == null) {
            monographDTO.setAssignedTo(profile.getUsername());
        }
        if (monographDTO.getAssignedTo() == null || monographDTO.getAssignedTo().equals("")) {
            monographDTO.setAssignedTo(SELECT_UNASSIGNED);
        }
    }

    /**
     * Creates the messages.
     * 
     * @param monographDTO
     *            the monograph object
     * @param action
     *            the action to be performed
     * @param prevStatus
     *            previous status
     * @param fdbChange
     *            if a FDB field was changed
     */
    private void createMessages(MonographDTO monographDTO, String action, String prevStatus, boolean fdbChange) {
        String message = "";
        if ("Delete Reviewed".equals(prevStatus) && "Reject".equals(action)) {
            message =
                    "This request for deletion has been rejected. "
                            + "The record has returned to previous action status (\'Approved\').";
        } else if ("Reviewed".equals(prevStatus) && "Reject".equals(action) && "Deleted".equals(monographDTO.getStatus())) {
            message =
                    "This reviewed record has been rejected. "
                            + "The record has returned to previous action status (\'Deleted\').";
        } else if ("Reviewed".equals(prevStatus) && "Reject".equals(action) && !"Rejected".equals(monographDTO.getStatus())) {
            message =
                    "This reviewed record has been rejected. "
                            + "The record has returned to previous action status (\'Approved\').";
        } else if ("Modified".equals(prevStatus) && "Reject".equals(action) && "Approved".equals(monographDTO.getStatus())) {
            message =
                    "This request for modification has been rejected. "
                            + "The record has returned to previous action status (\'Approved\').";
        } else if ("Modified".equals(prevStatus) && "Reject".equals(action) && "Deleted".equals(monographDTO.getStatus())) {
            message =
                    "This request for modification has been rejected. "
                            + "The record has returned to previous action status (\'Deleted\').";
        } else if ("Deleted".equals(prevStatus) && "Modify".equals(action) && "Modified".equals(monographDTO.getStatus())) {
            message = "This custom record has been successfully submitted and will be reviewed " + "at the national level.";
        } else if ("Approved".equals(prevStatus) && "Modify".equals(action) && "Modified".equals(monographDTO.getStatus())) {
            message = "This custom record has been successfully submitted and will be reviewed " + "at the national level.";
        } else if ("Modify".equals(action) && "Modified".equals(monographDTO.getStatus()) && fdbChange) {
            message = "This custom record has been successfully submitted and will be reviewed " + "at the national level.";
        } else {
            message = getInformationalMessageDisplay(action);
        }
        if (!"".equals(message)) {
            addInformationMessage(message);
        }
    }

    /**
     * Lock record.
     * 
     * @param recordType
     *            the record type
     * @param recordId
     *            the record id
     * @param request
     *            the request
     * @return the locking result
     */
    @RequestMapping(value = "edit/{recordType}/{recordId}", method = RequestMethod.GET)
    @ResponseBody
    public LockingResult lockRecord(@PathVariable(value = "recordType") String recordType,
            @PathVariable(value = "recordId") String recordId, HttpServletRequest request) {
        MonographDTO monographDTO = null;
        LockingResult result = new LockingResult();
        if (RecordType.VA.equals(RecordType.fromString(recordType))) {
            monographDTO = monographService.findMonographByVersionId(Long.parseLong(recordId));
            recordId = String.valueOf(monographDTO.getMasterRecordId());
            if (!monographDTO.isCurrent()) {
                result.setMessage(findMessage("detail.message.notcurrent"));
                MonographDTO current = monographService.findMonographById(monographDTO.getMonographid());
                result.setCurrent(false);
                result.setUrl(current.getSelect().getValue());
                return result;
            }
        }
        String message =
                recordLockingHelper.checkEdit(recordType, Concept.PROFESSIONAL_MONOGRAPH.toString(), recordId,
                        getUserProfile(request).getUsername());
        result.setMessage(message);

        return result;
    }

    /**
     * Cancel lock.
     * 
     * @param recordType
     *            the record type
     * @param recordId
     *            the record id
     * @param request
     *            the request
     * @return the string
     */
    @RequestMapping(value = "cancel/{recordType}/{recordId}", method = RequestMethod.GET)
    @ResponseBody
    public String cancelLock(@PathVariable(value = "recordType") String recordType,
            @PathVariable(value = "recordId") String recordId, HttpServletRequest request) {
        MonographDTO monographDTO = null;
        if (RecordType.VA.equals(RecordType.fromString(recordType))) {
            monographDTO = monographService.findMonographByVersionId(Long.parseLong(recordId));
            recordId = String.valueOf(monographDTO.getMasterRecordId());

        }

        return recordLockingHelper.cancelEdit(recordType, Concept.PROFESSIONAL_MONOGRAPH.toString(), recordId,
                getUserProfile(request).getUsername());

    }

    /**
     * Extend lock.
     * 
     * @param recordType
     *            the record type
     * @param recordId
     *            the record id
     * @param request
     *            the request
     * @return the locking result
     */
    @RequestMapping(value = "extend/{recordType}/{recordId}", method = RequestMethod.GET)
    @ResponseBody
    public LockingResult extendLock(@PathVariable(value = "recordType") String recordType,
            @PathVariable(value = "recordId") String recordId, HttpServletRequest request) {
        MonographDTO monographDTO = null;
        LockingResult result = new LockingResult();
        if (RecordType.VA.equals(RecordType.fromString(recordType))) {
            monographDTO = monographService.findMonographByVersionId(Long.parseLong(recordId));
            if (!monographDTO.isCurrent()) {
                MonographDTO current = monographService.findMonographById(monographDTO.getMonographid());
                result.setCurrent(false);
                result.setUrl(current.getSelect().getValue());
                return result;
            }

            recordId = String.valueOf(monographDTO.getMasterRecordId());

        } else {
            monographDTO = monographService.findFdbMonographById(Long.parseLong(recordId));
            recordId = String.valueOf(monographDTO.getFdbId());
        }

        recordLockingHelper.extendLock(recordType, Concept.PROFESSIONAL_MONOGRAPH.toString(), recordId, getUserProfile(request)
                .getUsername());

        return result;
    }

    //@formatter:off
    /**
     * Save.
     *
     * @param recordId the record id
     * @param request the request
     * @param monographDTO the monograph dto
     * @param bindingResult the binding result
     * @return the monograph dto
     */
    public MonographDTO save(String recordId,HttpServletRequest request,MonographDTO monographDTO,BindingResult bindingResult) {
        // @formatter:on
        if (LOGGER.is   ugEnabled())
            LOGGER.debug("GUI 'Save', received monographDTO object with monographid: " + monographDTO.getMonographid());
        try {
            // Validate object being saved.
            // Handle or delegate (to service layer or another action) business
            // rules:
            // Update Requestor
            UserProfile profile = getUserProfile(request);
            monographDTO.setRequestor(profile.getUsername());
            monographDTO.setActionBy(profile.getUsername());
            if (profile.isApprover() && monographDTO.getAssignedTo() == null)
                monographDTO.setAssignedTo(profile.getUsername());
            if (monographDTO.getAssignedTo() == null || monographDTO.getAssignedTo().equals(""))
                monographDTO.setAssignedTo(ProfessionalMonographDetailController.SELECT_UNASSIGNED);
            // Create new customization
            monographDTO = monographService.createNewRecord(monographDTO, bindingResult);
            monographDTO.setCurrentActionReason("");
            if (!bindingResult.hasErrors()) {
                String message =
                        "This custom record has been successfully submitted and will be reviewed at the national level.";
                addInformationMessage(message);
            }
        } catch (DuplicateConceptException e) {
            LOGGER.error("Duplicat concept", e);
            bindingResult.addError(new ObjectError("monographDTO", findMessage("message.us.information.pm.duplicate",
                    new Object[] { e.getMessage() })));

        } catch (ValidationException ex) {

            LOGGER.error("Data validation error", ex);
            bindingResult.addError(new ObjectError("monographDTO", findMessage("us.exception.create")));

        } catch (CustomizationServiceException ex) {
            // this.addActionError(getText("error.save") + " "+
            // ex.getMessage());
            LOGGER.error("Unable to save monograph models: ", ex);
            bindingResult.addError(new ObjectError("monographDTO", findMessage("us.exception.create")));
        }
        return monographDTO;
    }

    /**
     * Sets the up for display.
     * 
     * @param monographDTO
     *            the monograph dto
     * @param request
     *            the request
     * @return the model and view
     */
    private ModelAndView setUpForDisplay(MonographDTO monographDTO, HttpServletRequest request) {

        ModelAndView mav = new ModelAndView(VIEW);
        mav.addObject("showVaFields", false);
        try {

            setReadOnlyRecord(!monographDTO.isCurrent());
            ConceptMetadata conceptMetadata = monographService.getMetadata();
            Map<String, ConceptField> fieldMap = conceptMetadata.getFieldMap();
            mav.addObject("monographDTO", monographDTO);
            mav.addObject("fieldMap", fieldMap);
            // display list of request assigned to.
            List<String> users = getAdminUsers();
            mav.addObject("users", users);
            mav.addObject("allowableActions", Collections.emptySet());

            // interrogating the state machine for allowable actions makes sense
            // if we're not displaying the record in read-only mode and it's not
            // and FDB record
            if (!monographDTO.isReadOnlyRecord()) {
                if (monographDTO.getMonographid() != 0) {
                    try {
                        mav.addObject("allowableActions",
                                monographService.getAllowableActions(monographDTO, getUserProfile(request)));
                    } catch (Exception e) {
                        LOGGER.error("Cannot determine allowable actions", e);
                        findMessage("ddim.wizard.required.recordid");
                    }
                } else {
                    // must be a new record, so the only available action is
                    // Customize
                    mav.addObject("allowableActions", Collections.singleton("Customize"));
                }
            }

            return mav;
        } catch (CustomizationServiceException ex) {
            LOGGER.error("Unable to load monograph models: ", ex);
            findMessage("error.load");
        }
        return mav;
    }

    /**
     * Setup for display_2.
     * 
     * @param request
     *            the request
     * @param monographDTO
     *            the monograph dto
     * @param view
     *            the view
     * @return the model and view
     */
    private ModelAndView setupForDisplay2(HttpServletRequest request, MonographDTO monographDTO, ModelAndView view) {
        ConceptMetadata conceptMetadata = monographService.getMetadata();
        Map<String, ConceptField> fieldMap = conceptMetadata.getFieldMap();
        view.addObject("showVaFields", true);
        view.addObject("monographDTO", monographDTO);
        view.addObject("fieldMap", fieldMap);
        List<String> users = getAdminUsers();
        view.addObject("users", users);
        view.addObject("allowableActions", Collections.emptySet());

        // monographDTO.setFdbCustomMonographVaId(monographDTO.getFdbId());
        setReadOnlyRecord(!monographDTO.isCurrent());
        if (!isReadOnlyRecord()) {
            if (monographDTO.getFdbCustomMonographVaId() != 0) {
                try {
                    view.addObject("allowableActions",
                            monographService.getAllowableActions(monographDTO, getUserProfile(request)));
                } catch (Exception e) {
                    findMessage("error.columnlist.notfound");
                    LOGGER.error("Cannot determine allowable actions", e);
                }
            } else {
                // must be a new record, so the only available action is
                // Customize
                view.addObject("allowableActions", Collections.singleton("Customize"));
            }
        }
        return view;
    }

    /**
     * Retrieve custom model by customized monograph.
     * 
     * @param fdbid
     *            the fdbid
     * @return the monograph dto
     */
    private MonographDTO retrieveCustomModelByCustomizedMonograph(String fdbid) {
        return monographService.findMonographByFDBId(Long.parseLong(fdbid));
    }

    /**
     * Retrieve fdb monograph.
     * 
     * @param monographid
     *            the monographid
     * @return the monograph dto
     */
    private MonographDTO retrieveFdbMonograph(String monographid) {
        long id = Long.parseLong(monographid);
        return monographService.findFdbMonographById(id);
    }

    /**
     * Gets the informational message display.
     * 
     * @param action
     *            the action
     * @return the informational message display
     */
    private String getInformationalMessageDisplay(String action) {
        String msg = "";
        // if (action.equals("Modify")) {
        // msg = "message.us.information.modify";
        // } else if (action.equals("Approve")) {
        if (action.equals("Approve")) {
            msg = "message.us.information.approver.approve";
        } else if (action.equals("Delete")) {
            msg = "message.us.information.approver.delete";
        } else if (action.equals("Reject")) {
            msg = "message.us.information.approver.reject";
        } else if (action.equals("Submit_as_Reviewed")) {
            msg = "message.us.information.approver.submittedAsReviewed";
        } else if (action.equals("Submit_for_Delete")) {
            msg = "message.us.information.approver.submitfordelete";
        } else if (action.equals("Customize")) {
            msg = "message.us.information.requestor.submitted";
            // message.us.information.requestor.rejected
        }
        return msg;
    }

    /**
     * Checks if is read only record.
     * 
     * @return true, if is read only record
     */
    public boolean isReadOnlyRecord() {
        return readOnlyRecord;
    }

    /**
     * Sets the read only record.
     * 
     * @param readOnlyRecord
     *            the new read only record
     */
    public void setReadOnlyRecord(boolean readOnlyRecord) {
        this.readOnlyRecord = readOnlyRecord;
    }

    /**
     * Gets the record locking helper.
     * 
     * @return the recordLockingHelper
     */
    public RecordLockingHelper getRecordLockingHelper() {
        return recordLockingHelper;
    }

    /**
     * Sets the record locking helper.
     * 
     * @param recordLockingHelper
     *            the recordLockingHelper to set
     */
    public void setRecordLockingHelper(RecordLockingHelper recordLockingHelper) {
        this.recordLockingHelper = recordLockingHelper;
    }

    /**
     * Show comment button.
     * 
     * @param m
     *            the m
     * @return true, if successful
     */
    public boolean showCommentButton(MonographDTO m) {
        boolean showButton = true;

        if (m != null && m.getStatus() != null) {
            if (!m.getStatus().equalsIgnoreCase(Status.STATUS_DELETED)) {
                showButton = false;
            }
        }
        return showButton;
    }

}
