package gov.va.med.pharmacy.pecs.web.reports;

import gov.va.med.pharmacy.pecs.domain.impl.lookup.ComparisonReportConcept;
import gov.va.med.pharmacy.pecs.domain.impl.va.FdbUpdateReport;
import gov.va.med.pharmacy.pecs.exception.CustomizationReportException;
import gov.va.med.pharmacy.pecs.service.ReportService;
import gov.va.med.pharmacy.pecs.service.TerminologyService;
import gov.va.med.pharmacy.pecs.service.fdbreports.FdbUpdateReportService;
import gov.va.med.pharmacy.pecs.util.BlobHandler;
import gov.va.med.pharmacy.pecs.web.view.ReportXLSXViewer;
import gov.va.med.pharmacy.pecs.web.view.document.AbstractXLSXViewer;

import java.sql.Blob;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.stereotype.Controller;
import org.springframework.util.FileCopyUtils;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.servlet.ModelAndView;

/**
 * The Controller to handle Report requests.
 * 
 * @author vhaisaduffg
 * 
 */
@Controller
@RequestMapping(value = "/reports")
// public class ReportController extends AbstractDownloadableContentController
// implements InitializingBean {
public class ReportController implements InitializingBean {
    
    /** The logger. */
    private static Log logger = LogFactory.getLog(ReportController.class);

    /** The content type for an Excel response. */
    private static final String CONTENT_TYPE = "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet";
    // private static final String CONTENT_TYPE = "application/vnd.ms-excel";

    /** The Constant CONTENT_DISPOSITION_KEY. */
    private static final String CONTENT_DISPOSITION_KEY = "Content-Disposition";

    /** The blob handler. */
    @Resource
    private BlobHandler blobHandler;

    /** The report service. */
    @Resource
    private ReportService reportService;

    /** The fdb update report service. */
    @Resource
    private FdbUpdateReportService fdbUpdateReportService;

    /** The terminology service. */
    @Resource
    private TerminologyService terminologyService;

    /** The Constant TILES_VIEW. */
    private static final String TILES_VIEW = "pecs.reports";
    
    /** The Constant VIEW. */
    private static final String VIEW = "reportXlsxViewer";

    /** The Constant REPORT_MODEL_ATTRIBUTE. */
    private static final String REPORT_MODEL_ATTRIBUTE = "reportModel";
    
    /** The Constant COMPARISON_REPORT_CONCEPTS. */
    private static final String COMPARISON_REPORT_CONCEPTS = "comparisonReportConcepts";

    /**
     * Find report model.
     *
     * @return the report model
     */
    @ModelAttribute(REPORT_MODEL_ATTRIBUTE)
    public ReportModel findReportModel() {
        return new ReportModel();
    }

    /**
     * Returns a List of ComparisonReportConcepts for display on the UI.
     * 
     * @return List<ComparisonReportConcept>
     */
    private List<ComparisonReportConcept> populateComparisonReportConcepts() {
        return terminologyService.getComparisonReportConcepts();
    }

    /**
     * Get method used to display the Reports page.
     * 
     * @return ModelAndView
     */
    @RequestMapping(method = RequestMethod.GET)
    public ModelAndView displayReports() {
        ModelAndView view = new ModelAndView(TILES_VIEW);
        view.addObject(COMPARISON_REPORT_CONCEPTS, populateComparisonReportConcepts());
        return view;
    }

    /**
     * Used to execute and download a Customization report.
     * 
     * @param reportName
     *            the internal name of the report to generate
     * @return ModelAndView
     * @throws CustomizationReportException
     *             if an error occurs while generating the report
     */
    @RequestMapping(value = "/customreport/{reportName}", method = RequestMethod.GET)
    public ModelAndView downloadFile(@PathVariable("reportName") String reportName) throws CustomizationReportException {
        Map<String, Object> model = new HashMap<String, Object>();
        model.put(AbstractXLSXViewer.EXCEL_FILE_NAME_KEY, reportService.findFileName(reportName));
        model.put(ReportXLSXViewer.REPORT_DATA_KEY, reportService.findData(reportName));
        return new ModelAndView(VIEW, model);
    }

    /**
     * Retrieves and downloads a Comparison report to the user.
     * 
     * @param reportId
     *            the identifier of the report to download
     * @param response
     *            the HttpServletResponse where the report will be written
     * @throws Exception
     *             if an exception occurs while writing the report to the response
     */
    @RequestMapping(value = "/reportdisplay/{reportid}", method = RequestMethod.GET)
    public void downloadComparisonReport(@PathVariable("reportid") String reportId, HttpServletResponse response)
            throws Exception {
        FdbUpdateReport report = fdbUpdateReportService.getReport(new Long(reportId));
        if (report != null) {
            Blob content = report.getContent();
            response.setContentType(CONTENT_TYPE);
            response.setHeader(CONTENT_DISPOSITION_KEY, "attachment;filename=\"" + report.getFilename() + "\"");
            FileCopyUtils.copy(blobHandler.getStream(content), response.getOutputStream());
        }
    }

    // Spring injector methods

    /**
     * Sets the FdbUpdateReportService to use.
     * 
     * @param fdbUpdateReportService
     *            the FdbUpdateReportService
     */
    public final void setFdbUpdateReportService(FdbUpdateReportService fdbUpdateReportService) {
        this.fdbUpdateReportService = fdbUpdateReportService;
    }

    /**
     * Sets the ReportService to use.
     * 
     * @param reportService
     *            the ReportService
     */
    public final void setReportService(ReportService reportService) {
        this.reportService = reportService;
    }

    /**
     * Sets the TerminologyService to use.
     * 
     * @param service
     *            TerminologyService
     */
    public final void setTerminologyService(TerminologyService service) {
        terminologyService = service;
    }

    /**
     * Checks to make sure all beans that should be injected have been injected.
     * 
     * @throws Exception
     *             if a needed bean has not been injected
     */
    public final void afterPropertiesSet() throws Exception {
        if (blobHandler == null) {
            throw new Exception("BlobHandler has not been injected into ReportController.");
        }
        if (reportService == null) {
            throw new Exception("ReportService has not been injected into ReportController.");
        }
        if (fdbUpdateReportService == null) {
            throw new Exception("FdbUpdateReportService has not been injected into ReportController.");
        }
        if (terminologyService == null) {
            throw new Exception("TerminologyService has not been injected into ReportController.");
        }
    }
}
