package gov.va.med.pharmacy.pecs.web.settings;

import gov.va.med.pharmacy.pecs.exception.CustomizationServiceException;

import gov.va.med.pharmacy.pecs.web.AbstractController;

import java.util.ArrayList;


import javax.annotation.Resource;
import javax.validation.Valid;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.stereotype.Controller;
import org.springframework.validation.BindingResult;
import org.springframework.validation.ObjectError;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.view.RedirectView;

/**
 * The controller for the main Administration page.
 * 
 * @author vhaisaduffg
 *
 */
@Controller
@RequestMapping(value = "/administration")
public class SettingsController extends AbstractController implements InitializingBean {

    private final Log logger = LogFactory.getLog(SettingsController.class);


    /**
     * The main Administration view.
     */
    public static final String TILES_VIEW = "pecs.settings";
    private static final String TILES_TABLESETTINGS_VIEW = "pecs.tablesettings";
    private static final String CONCEPT_NAME_ATTRIBUTE = "conceptName";

    /**
     * Method used to display the Administration page.
     * 
     * @return ModelAndView
     */
    @RequestMapping(method = RequestMethod.GET)
    public final ModelAndView display() {
        ModelAndView view = new ModelAndView(TILES_VIEW);
        view.addObject(CONFIGURABLE_CONCEPTS_ATTRIBUTE, populateConfigurableConcepts());
        return view;
    }

    /**
     * Method used to display the Concept Customization page.
     * 
     * @param conceptName the name of the concept to customize
     * @return ModelAndView
     */
    @RequestMapping(value = "/conceptConfiguration/{conceptName}", method = RequestMethod.GET)
    public final ModelAndView displayTableSettings(@PathVariable("conceptName") String conceptName) {
        ModelAndView view = new ModelAndView(TILES_TABLESETTINGS_VIEW);
        view.addObject(CONCEPT_NAME_ATTRIBUTE, conceptName);
        TableSettings settings = new TableSettings();
        settings.setConfigurableTables((ArrayList<String>) populateConfigurableConcepts());
        settings.setTableName(conceptName);
        settings.setConfigColumns(getMetadataService().findByConcept(conceptName));
        view.addObject("tableSettings", settings);
        return view;
    }

    /**
     * Method to process and save any customizations for a concept.
     * 
     * @param conceptName the name of the concept
     * @param settings the new customizations
     * @param result a BindingResult for errors
     * @return Object could be a redirect or errors 
     */
    @RequestMapping(value = "/conceptConfiguration/{conceptName}", method = RequestMethod.POST)
    public final ModelAndView saveTableSettings(@PathVariable("conceptName") String conceptName,
            @ModelAttribute("tableSettings") @Valid TableSettings settings, 
            BindingResult result) {

        ModelAndView view = new ModelAndView(TILES_TABLESETTINGS_VIEW);

        if (result.hasErrors()) {
            ((ModelAndView) view).addObject("tableSettings", settings);
        } else {
            if (saveConfig(settings, result)) {
                addInformationMessage("tableSettings.save.success");
                RedirectView redirectView = new RedirectView(settings.getTableName(), true);
                view.setView(redirectView);
            } else {
                ((ModelAndView) view).addObject("tableSettings", settings);
            }
        }

        return view;
    }

    /**
     * Method used to actually save the new customization of a concept.
     * 
     * @param settings the new customizations
     * @param result a BindingResult for errors
     * @return Boolean
     */
    private boolean saveConfig(TableSettings settings, BindingResult result) {
        if (settings == null || settings.getConfigColumns() == null || settings.getConfigColumns().isEmpty()) {
            result.addError(new ObjectError("tableSettings", "The server failed to receive any configuration data to update."));
            return false;
        }

        boolean savedSuccessfully = false;

        try {
            // Save out values to db with new modified values
            getMetadataService().modifyConfiguration(settings.getConfigColumns());
            savedSuccessfully = true;

            if (logger.is   ugEnabled()) {
                logger.debug("Saved settings for concept: " + settings.getTableName());
            }
        } catch (CustomizationServiceException cse) {
            result.addError(new ObjectError("tableSettings", cse.getMessage()));
        }

        catch (Exception ex) {
            logger.error("Unable to save settings for concept '" + settings.getTableName() + "'", ex);
        }
        return savedSuccessfully;
    }

    /**
     * Method used by Spring to make sure all beans have been injected.
     * 
     * @throws Exception if a bean has not been injected.
     */
    @Override
    public final void afterPropertiesSet() throws Exception {
        if (getMetadataService() == null) {
            throw new Exception("The property MetadataService in SettingsController has not been set.");
        }
    }

}
