package gov.va.med.pharmacy.pecs.web.settings;

import gov.va.med.pharmacy.pecs.domain.impl.va.CtUsersVa;
import gov.va.med.pharmacy.pecs.exception.CustomizationServiceException;
import gov.va.med.pharmacy.pecs.service.UserService;
import gov.va.med.pharmacy.pecs.web.AbstractController;

import java.util.ArrayList;
import java.util.List;

import javax.annotation.Resource;
import javax.validation.Valid;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.stereotype.Controller;
import org.springframework.validation.BindingResult;
import org.springframework.validation.ObjectError;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;

/*
 * The Controller class used to process the User Settings page in PECS.
 */
/**
 * The Class UserSettingsController.
 */
@Controller
@RequestMapping(value = "/userSettings")
public class UserSettingsController extends AbstractController implements InitializingBean {

    /** The logger. */
    private final Log logger = LogFactory.getLog(UserSettingsController.class);

    /** The user service. */
    @Resource
    private UserService userService;

    /** The Constant TILES_VIEW. */
    private final static String TILES_VIEW = "pecs.usersettings";

    /** The Constant USER_MODEL. */
    private final static String USER_MODEL = "userSettingsModel";

    /*
     * Returns the currently list of enabled Approvers in the PECS.
     */
    /**
     * Find approvers.
     * 
     * @return the list
     */
    @ModelAttribute("userList")
    public List<CtUsersVa> findApprovers() {
        return userService.findApprovers();
    }

    /*
     * Returns an empty UserSettingsModel object.
     */
    /**
     * Find user settings model.
     * 
     * @return the user settings model
     */
    @ModelAttribute(USER_MODEL)
    public UserSettingsModel findUserSettingsModel() {
        return new UserSettingsModel();
    }

    /*
     * Handles the initial display of the User Settings page.
     * 
     * @return the Tiles view to display.
     */
    /**
     * Gets the.
     * 
     * @return the string
     */
    @RequestMapping(method = RequestMethod.GET)
    public String get() {
        return TILES_VIEW;
    }

    /*
     * Handles a POST method to the User Settings. The only time a POST should be made is when the user selects to Save their
     * changes.
     * 
     * @return either the Tiles view to display in case of errors, or a redirect to the same page in the case everything saved
     * correctly.
     */
    /**
     * Post.
     * 
     * @param userSettingsModel
     *            the user settings model
     * @param result
     *            the result
     * @return the string
     */
    @RequestMapping(method = RequestMethod.POST)
    // public ModelAndView process(@RequestParam(required=false,value="userDeletes") String userDeletes,
    // @RequestParam(required=false,value="addUserText") String addUserText){
            public
            String post(@ModelAttribute(USER_MODEL) @Valid UserSettingsModel userSettingsModel, BindingResult result) {
        if (result.hasErrors()) {
            return TILES_VIEW;
        }

        processForm(userSettingsModel, result);
        return "redirect:userSettings";
    }

    /*
     * Handles the processing of the data entered by the user.
     */
    /**
     * Process form.
     * 
     * @param userSettingsModel
     *            the user settings model
     * @param result
     *            the result
     */
    private void processForm(UserSettingsModel userSettingsModel, BindingResult result) {
        // Load deletes list and process
        String userDeletes = userSettingsModel.getUserDeletes();
        if (userDeletes != null && userDeletes.length() > 0) {
            if (logger.is   ugEnabled())
                logger.debug("Submitted list of approvers to delete: " + userDeletes);

            String[] deleteIds = userDeletes.split(",");
            try {
                ArrayList<CtUsersVa> records = new ArrayList<CtUsersVa>();
                for (String id : deleteIds) {
                    CtUsersVa ctUser = new CtUsersVa();
                    ctUser.setCtUsersVaId(Long.parseLong(id.trim()));
                    ctUser.setUserEnabled(false);
                    records.add(ctUser);
                }

                List<CtUsersVa> updatedUsers = null;
                if (records.size() > 0)
                    updatedUsers = userService.modifyRecords(records);

                if (updatedUsers != null && updatedUsers.size() > 0) {
                    // TODO Action messages
                    addInformationMessage("us.message.userDeleted", new Object[] { updatedUsers.size() });
                }

            } catch (CustomizationServiceException ex) {
                result.addError(new ObjectError(USER_MODEL, findMessage("us.exception.delete")));
                logger.error("Unable to delete approved user ", ex);
            }
        }

        String addUserText = userSettingsModel.getAddUserText();
        // Add approver user if text exists in add user text field
        if (addUserText != null && addUserText.length() > 0) {
            if (logger.is   ugEnabled())
                logger.debug("Submitted addUserText content: " + addUserText);

            try {
                String userName = addUserText.trim();
                userName = userName.toUpperCase();

                CtUsersVa newCtUser = new CtUsersVa();
                newCtUser.setUsername(userName);
                CtUsersVa newUser = userService.createRecord(newCtUser);
                // TODO Action messages
                addInformationMessage("us.message.userAdded", new Object[] { newUser.getUsername() });
            } catch (CustomizationServiceException ex) {
                // TODO Error Messages
                result.addError(new ObjectError(USER_MODEL, findMessage("us.exception.add")));
                logger.error("Unable to add approved user " + addUserText, ex);
            }
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.pecs.web.AbstractController#afterPropertiesSet()
     */
    @Override
    public void afterPropertiesSet() throws Exception {
        super.afterPropertiesSet();
        if (userService == null) {
            throw new Exception("The property UserService in UserSettingsController has not been set.");
        }
    }

}
