package gov.va.med.pharmacy.pecs.web.updates;

import gov.va.med.pharmacy.pecs.customupdate.CtVersionVa;
import gov.va.med.pharmacy.pecs.domain.impl.va.UserProfile;
import gov.va.med.pharmacy.pecs.exception.CustomizationServiceException;
import gov.va.med.pharmacy.pecs.exception.FTPException;
import gov.va.med.pharmacy.pecs.service.CustomUpdateService;
import gov.va.med.pharmacy.pecs.web.AbstractDownloadableContentController;

import java.util.Date;
import java.util.List;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.stereotype.Controller;
import org.springframework.validation.BindingResult;
import org.springframework.validation.ObjectError;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.servlet.ModelAndView;

/**
 * The Controller class for the Custom Updates page. The only functionality contained in this controller is:
 * 
 * 1) Display the Custom Updates page. 2) Create a new Custom Update File.
 * 
 * Downloading a Custom Update file has been moved to the FilesController.
 * 
 * @author vhaisaduffg
 * 
 */
@Controller
@RequestMapping(value = "/updates")
public final class UpdateController extends AbstractDownloadableContentController implements InitializingBean {

    /** The Constant LOGGER. */
    private static final Log LOGGER = LogFactory.getLog(UpdateController.class);

    /** The update file service. */
    @Resource
    private CustomUpdateService updateFileService;

    /** The Constant TILES_VIEW. */
    private static final String TILES_VIEW = "pecs.customupdates";

    /** The Constant JSP Fragment For Months. */
    private static final String MONTHS_VIEW = "pecs.months";

    /** The Constant JSP Fragment that holds the custom updates. */
    private static final String UPDATE_FILES_VIEW = "pecs.updatefiles";

    // Model Attributes
    /** The Constant VERSION_LIST. */
    public static final String VERSION_LIST = "ctVersions";

    /** The Constant CUSTOM_UPDATE_MODEL. */
    public static final String CUSTOM_UPDATE_MODEL = "customUpdateModel";

    /**
     * Gather months for year.
     * 
     * @param year
     *            the year
     * @param view
     *            the view
     * @return the model and view
     */
    @RequestMapping(value = "/year/{year}", method = RequestMethod.GET)
    public ModelAndView gatherMonthsForYear(@PathVariable(value = "year") String year, ModelAndView view) {
        view = new ModelAndView(MONTHS_VIEW);
        List<Date> months = updateFileService.findAllMonthsOfUpdatesByYears(year);
        view.addObject("year", year);
        view.addObject("months", months);
        return view;
    }

    /**
     * Gets the custom updates for month.
     * 
     * @param year
     *            the year
     * @param month
     *            the month
     * @param view
     *            the view
     * @return the custom updates for month
     */
    @RequestMapping(value = "/year/{year}/month/{month}")
    public ModelAndView getCustomUpdatesForMonth(@PathVariable(value = "year") String year,
            @PathVariable(value = "month") String month, ModelAndView view) {
        view = new ModelAndView(UPDATE_FILES_VIEW);
        List<CtVersionVa> customFiles = updateFileService.getCustomUpdateFileByMonthAndyear(year, month);

        view.addObject("customFiles", customFiles);
        view.addObject("month", month);
        view.addObject("year", year);
        return view;
    }

    /**
     * Returns a new CustomUpdateModel object.
     * 
     * @return the custom update model
     */
    @ModelAttribute(CUSTOM_UPDATE_MODEL)
    public CustomUpdateModel findCustomUpdateModel() {
        return new CustomUpdateModel();
    }

    /**
     * Returns the view that should be displayed when the user selects to view the Custom Updates.
     * 
     * @param view
     *            the view
     * @return the string
     */
    @RequestMapping(method = RequestMethod.GET)
    public ModelAndView display(ModelAndView view) {
        view = new ModelAndView(TILES_VIEW);
        List<Date> years = updateFileService.findAllYears();
        view.addObject("years", years);
        return view;
    }

    /**
     * Handles the process of creating a new Custom Update file.
     * 
     * @param request
     *            the request
     * @param customUpdateModel
     *            the custom update model
     * @param result
     *            the result
     * @return the string
     */
    @RequestMapping(method = RequestMethod.POST)
    public String createCustomUpdate(HttpServletRequest request,
            @ModelAttribute(CUSTOM_UPDATE_MODEL) CustomUpdateModel customUpdateModel, BindingResult result) {
        UserProfile profile = getUserProfile(request);
        final String tempDirPath = getTempDirPath(request);

        try {
            updateFileService.generateCustomUpdateFiles(tempDirPath, profile.getUsername());
        } catch (FTPException fe) {
            result.addError(new ObjectError(CUSTOM_UPDATE_MODEL, findMessage("cu.error.sending",
                    new Object[] { fe.getServer() })));
            return TILES_VIEW;
        } catch (CustomizationServiceException cse) {
            LOGGER.error(cse.getMessage(), cse);
            result.addError(new ObjectError(CUSTOM_UPDATE_MODEL, cse.getMessage()));
            return TILES_VIEW;
        }
        return "redirect:updates";
    }

    // Spring Injector methods
    /**
     * Gets the update file service.
     * 
     * @return the update file service
     */
    public CustomUpdateService getUpdateFileService() {
        return updateFileService;
    }

    /**
     * Sets the update file service.
     * 
     * @param updateFileService
     *            the new update file service
     */
    public void setUpdateFileService(CustomUpdateService updateFileService) {
        this.updateFileService = updateFileService;
    }

    /**
     * Method.
     * 
     * @throws Exception
     *             the exception
     * @see gov.va.med.pharmacy.pecs.web.AbstractController#afterPropertiesSet()
     */
    public void afterPropertiesSet() throws Exception {
        if (updateFileService == null) {
            throw new Exception("The property UpdateFileService in UpdateController has not been set.");
        }
    }
}
