package gov.va.med.pharmacy.pecs.web.view;

import gov.va.med.pharmacy.pecs.dto.PecsDTO;
import gov.va.med.pharmacy.pecs.metadata.ConceptField;
import gov.va.med.pharmacy.pecs.metadata.ConceptMetadata;
import gov.va.med.pharmacy.pecs.service.MetadataService;
import gov.va.med.pharmacy.pecs.web.queryresults.QueryResultsController;
import gov.va.med.pharmacy.pecs.web.view.document.AbstractXLSXViewer;

import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.context.MessageSource;

/**
 * A class used to generate Excel documents that are downloaded to a user's workstation off query results on the Advanced Query
 * tab.
 * 
 * @author vhaisaduffg
 * 
 */
public class QueryResultsXLSXViewer extends AbstractXLSXViewer implements InitializingBean {

    private static final Log LOGGER = LogFactory.getLog(QueryResultsXLSXViewer.class);

    @Resource
    private MetadataService metadataService;
    @Resource
    private MessageSource messageSource;

    private static final int MAX_EXPORTABLE_ROWS = 10000;
    private static final int MAX_EXPORTABLE_DOSE_RANGE_ROWS = 5000;
    private static final int DEBUG_REPORTABLE_ROWS = 1000;
    private static final int WARNING_MESSAGE_ROW = 5;
    private static final String DOSE_RANGE_CLASS = "gov.va.med.pharmacy.pecs.dto.DoseRangeDTO";
    private static final String CRITERIA_SHEET_NAME = "Criteria";
    private String versionActive;
    private String versionHistorical;

    /**
     * The method used to actually populate an Excel spreadsheet with data.
     * 
     * @param model
     *            the model Map
     * @param workbook
     *            the Excel workbook to complete
     * @param request
     *            in case we need locale etc. Shouldn't look at attributes.
     * @param response
     *            in case we need to set cookies. Shouldn't write to it.
     * @param styles
     *            the CellStyle map containing the styles in this workbook
     * @throws Exception
     *             if an error occurs
     */
    @Override
    protected final void buildExcelDocument(Map<String, Object> model, XSSFWorkbook workbook, HttpServletRequest request,
            HttpServletResponse response, Map<String, CellStyle> styles) throws Exception {

        versionActive = messageSource.getMessage("version.status.active", null, null);
        versionHistorical = messageSource.getMessage("version.status.historical", null, null);

        if (model.containsKey(CRITERIA_STRING)) {
            produceCriteriaSheet(model, workbook, styles);
        }
        Iterator<Entry<String, Object>> iter = model.entrySet().iterator();
        while (iter.hasNext()) {
            Entry<String, Object> entry = iter.next();

            createSheet(workbook, entry, styles);
        }
    }

    /**
     * Creates a worksheet in the workbook and populates that worksheet with data.
     * 
     * @param workbook
     *            the Excel workbook
     * @param entry
     *            the data from the model
     * @param styles
     *            the styles for the cells
     */
    private void createSheet(XSSFWorkbook workbook, Entry<String, Object> entry, Map<String, CellStyle> styles) {
        @SuppressWarnings({ "rawtypes" })
        List data = (List) entry.getValue();

        final CellStyle headerStyle = styles.get(HEADER_CELL_STYLE);
        final CellStyle defaultCellStyle = styles.get(DEFAULT_CELL_STYLE);

        if (data.isEmpty()){
            createWarningNoRowsSheet(workbook, defaultCellStyle);
            return;
        }
        
        String concept = data.get(0).getClass().getName();
        if (isGreaterThanMaxAllowed(concept, data.size())) {
            createWarningTooManyRowsSheet(workbook, data.size(), concept, defaultCellStyle);
        }
        Sheet sheet = workbook.createSheet(entry.getKey());
        int rowNum = 0;
        Row row = sheet.createRow(rowNum++);
        int colNum = 0;
        boolean custom = true;
        if (entry.getKey().startsWith(QueryResultsController.FDB_PREFIX)) {
            custom = false;
        }

        try {
            // header
            ConceptMetadata meta = metadataService.getMetadata(Class.forName(data.get(0).getClass().getName()));

            if (custom) {
                Cell cell = row.createCell(colNum++);
                cell.setCellValue(messageSource.getMessage("header.version.status", null, null));
                cell.setCellStyle(headerStyle);
            }

            for (ConceptField f : meta.getQueryFields(custom)) {
                Cell cell = row.createCell(colNum++);
                cell.setCellValue(f.getDisplayName());
                cell.setCellStyle(headerStyle);
            }
            sheet.createFreezePane(0, 1, 0, 1);

            if (LOGGER.is   ugEnabled()) {
                LOGGER.debug("Trying to write " + data.size() + " rows to spreadsheet.");
            }
            for (Object d : data) {
                row = sheet.createRow(rowNum++);
                colNum = 0;
                if (custom) {
                    String currentValue = "";
                    if (d instanceof PecsDTO) {
                        currentValue = ((PecsDTO) d).isCurrent() ? versionActive : versionHistorical;
                    }
                    Cell cell = row.createCell(colNum++);
                    setCellValue(cell, currentValue, defaultCellStyle);
                }
                for (ConceptField f : meta.getQueryFields(custom)) {
                    Cell cell = row.createCell(colNum++);
                    setCellValue(cell, f.getValue(d), defaultCellStyle);
                }
                if (isGreaterThanMaxAllowed(concept, rowNum)) {
                    break;
                }
                if (LOGGER.is   ugEnabled() && (rowNum % DEBUG_REPORTABLE_ROWS) == 0) {
                    LOGGER.debug("Placed " + rowNum + " rows in the spreadsheet.");
                }
            }

            // Size columns - This has to be done after the data is populated.
            autoSizeColumns(sheet, meta.getQueryFields(custom).size());
            workbook.setSheetOrder(entry.getKey(), 0);
        } catch (ClassNotFoundException cnfe) {
            LOGGER.error(cnfe.getMessage(), cnfe);
        }
    }

    /**
     * Adds a warning message to the criteria worksheet if there are data rows that are not displayed in the report. This
     * warning message was a way for us to limit OutOfMemoryException when a large number of Dose Range concepts were selected
     * for display.
     * 
     * @param workbook
     *            the Excel workbook
     * @param rowsInResultSet
     *            the number of row of data
     * @param concept
     *            the name of the Concept
     * @param defaultCellStyle
     *            the cell style to use
     */
    private void createWarningTooManyRowsSheet(XSSFWorkbook workbook, int rowsInResultSet, 
            String concept, CellStyle defaultCellStyle) {
        createWarningSheet(workbook,
                messageSource.getMessage("message.export.too.large",
                        new Object[] { rowsInResultSet,
                                (concept.equals(DOSE_RANGE_CLASS) ? MAX_EXPORTABLE_DOSE_RANGE_ROWS : MAX_EXPORTABLE_ROWS) },
                        null, null), defaultCellStyle); 
    }

    /**
     * Adds a warning message to the criteria worksheet if there are no data rows to display in the report. 
     * 
     * @param workbook
     *            the Excel workbook
     * @param defaultCellStyle
     *            the cell style to use
     */
    private void createWarningNoRowsSheet(XSSFWorkbook workbook, CellStyle defaultCellStyle) {
        createWarningSheet(workbook, messageSource.getMessage("message.export.no.rows", null,null), defaultCellStyle);         
    }    
    
    /**
     * Adds the specified warning message to the criteria worksheet. 
     * 
     * @param workbook
     *            the Excel workbook
     * @param message
     *            the message to display
     * @param defaultCellStyle
     *            the cell style to use
     */
    private void createWarningSheet(XSSFWorkbook workbook, String message, CellStyle defaultCellStyle) {
        Sheet sheet = workbook.getSheet(CRITERIA_SHEET_NAME);
        int rowNum = 0;
        if (sheet == null) {
            sheet = workbook.createSheet("PECS Message");
        } else {
            rowNum = WARNING_MESSAGE_ROW;
        }
        Row row = sheet.createRow(rowNum);
        Cell cell = row.createCell(0);
        setCellValue(cell, message, defaultCellStyle);
        autoSizeColumns(sheet, 1);          
    }       
    
    /**
     * Checks to see if the number of rows of data for a Concept is greater than the number allowed in the report.
     * 
     * @param concept
     *            the Concept
     * @param numberOfRows
     *            the number of rows of data
     * @return true if not all of the data rows will be in the report, otherwise false
     */
    private boolean isGreaterThanMaxAllowed(String concept, int numberOfRows) {
        if (concept.equals(DOSE_RANGE_CLASS) && numberOfRows > MAX_EXPORTABLE_DOSE_RANGE_ROWS) {
            return true;
        }
        if (numberOfRows > MAX_EXPORTABLE_ROWS) {
            return true;
        }
        return false;
    }

    /**
     * Creates the worksheet where the criteria used to generate the data on the report will be displayed.
     * 
     * @param model
     *            the model containing the data
     * @param workbook
     *            the Excel workbook
     * @param styles
     *            the cell styles
     */
    private void produceCriteriaSheet(Map<String, Object> model, XSSFWorkbook workbook, Map<String, CellStyle> styles) {
        try {
            Sheet sheet = workbook.createSheet(CRITERIA_SHEET_NAME);
            int rowNum = 0;
            int colNum = 0;

            Row row = sheet.createRow(rowNum++);
            Cell cell = row.createCell(colNum++);
            cell.setCellValue("Query Criteria");
            cell.setCellStyle(styles.get(HEADER_CELL_STYLE));
            cell = row.createCell(colNum++);

            sheet.createFreezePane(0, 1, 0, 1);

            row = sheet.createRow(rowNum++);
            cell = row.createCell(0);
            cell.setCellValue((String) model.get(CRITERIA_STRING));
            cell.setCellStyle(styles.get(DEFAULT_CELL_STYLE));
            autoSizeColumns(sheet, 1);

            model.remove(CRITERIA_STRING);
        } catch (Exception e) {
            LOGGER.error(e.getMessage(), e);
        }
    }

    /**
     * Sets the MessageSource object.
     * 
     * @param messageSource
     *            the MessageSource
     */
    public final void setMessageSource(MessageSource messageSource) {
        this.messageSource = messageSource;
    }

    /**
     * Sets the MetadataService object.
     * 
     * @param service
     *            the MetadataService
     */
    public final void setMetadataService(MetadataService service) {
        metadataService = service;
    }

    /**
     * A method called by Spring so that we can check to make sure everything has been injected that needs to be injected.
     * 
     * @throws Exception
     *             if some object has not been injected
     */
    @Override
    public final void afterPropertiesSet() throws Exception {
        if (metadataService == null) {
            throw new Exception("MetadataService has not been injected in to QueryResultsXLSXViewer.");
        }
        if (messageSource == null) {
            throw new Exception("MessageSource has not been injected in to QueryResultsXLSXViewer.");
        }
    }
}
