package gov.va.med.pharmacy.pecs.web.view;

import gov.va.med.pharmacy.pecs.dto.DrugPairDTO;
import gov.va.med.pharmacy.pecs.metadata.ConceptField;
import gov.va.med.pharmacy.pecs.metadata.ConceptMetadata;
import gov.va.med.pharmacy.pecs.service.MetadataService;
import gov.va.med.pharmacy.pecs.service.TerminologyService;
import gov.va.med.pharmacy.pecs.web.view.document.AbstractXLSXViewer;

import java.text.DecimalFormat;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.InitializingBean;

/**
 * A class used to create a Report History Excel spreadsheet and download to the user.
 * 
 * @author shahid sheikh -12-11-2012
 * 
 */
public final class ReportDrugInteractionsXLSXViewer extends AbstractXLSXViewer implements InitializingBean {

    // private static final Log LOGGER = LogFactory.getLog(ReportHistoryXLSXViewer.class);

    /** The metadata service. */
    @Resource
    private MetadataService metadataService;

    /** The terminology service. */
    private TerminologyService terminologyService;

    /** The Constant DRUG_INTERACTION_REPORT. */
    private static final String DRUG_INTERACTION_REPORT = "DRUG INTERACTION REPORT";

    /** The Constant SOURCE. */
    private static final String SOURCE = "Source";

    /** The Constant ROUTED_GENERIC_1. */
    private static final String ROUTED_GENERIC_1 = "Routed Generic #1";

    /** The Constant ROUTED_GENERIC_2. */
    private static final String ROUTED_GENERIC_2 = "Routed Generic #2";

    /** The Constant SEVERITY_LEVEL_CODE. */
    private static final String SEVERITY_LEVEL_CODE = "Severity Level Code";

    /** The Constant SEVERITY_LEVEL_DESC. */
    private static final String SEVERITY_LEVEL_DESC = "Severity Level Desc";

    /** The Constant INTERACTION_DESC. */
    private static final String INTERACTION_DESC = "Interaction Description";

    /** The Constant INTERACTION_ID. */
    private static final String INTERACTION_ID = "Interaction ID";

    /** The Constant CORRESPONDING_FDB_ID. */
    private static final String CORRESPONDING_FDB_ID = "Corresponding FDB Interaction ID";

    /** The Constant ACTION_DATE. */
    private static final String ACTION_DATE = "Action Date";

    /**
     * The view to return to get data to display using this viewer.
     */
    public static final String REPORT_DRUG_INTER_VIEW = "reportDrugInteractionsXLSXViewer";

    /**
     * The key in the model for the Concept that is being reported on.
     */
    public static final String METADATA_KEY = "metadataKey";
    /**
     * The key in the model for the Data that should be included in the report.
     */
    public static final String DATA_KEY = "dataKey";

    /** The Constant UPDATED_INDICATOR. */
    private static final String UPDATED_INDICATOR = " *";

    /** The Constant DECIMAL_FORMATTER. */
    private static final DecimalFormat DECIMAL_FORMATTER = new DecimalFormat("####0.######");

    /**
     * The main controller method for this class. This is the method that is responsible for the population of an Excel
     * spreadsheet with the reporting data that is needed.
     * 
     * @param model
     *            a Map that contains the data that should be placed in the report
     * @param workbook
     *            the Excel spreadsheet
     * @param request
     *            HttpServletRequest
     * @param response
     *            HttpServletResponse
     * @param styles
     *            a Map containing the CellStyle used in this workbook
     * @throws Exception
     *             if an error occurs while creating the spreadsheet.
     */

    @SuppressWarnings("unchecked")
    @Override
    protected void buildExcelDocument(Map<String, Object> model, XSSFWorkbook workbook, HttpServletRequest request,
            HttpServletResponse response, Map<String, CellStyle> styles) throws Exception {

        ConceptMetadata meta = null;
        List<ConceptField> reportHistoryFields = null;
        List<Object> data = null;

        data = (List<Object>) model.get(DATA_KEY);

        int x = 0;
        int numOfColumns = 8;
        Sheet sheet = workbook.createSheet(DRUG_INTERACTION_REPORT);

        addHeader(reportHistoryFields, sheet, styles.get(HEADER_CELL_STYLE));
        if (data.size() > 0) {
            int row = 1;
            row = addData(data, "VA", sheet, styles.get(DEFAULT_CELL_STYLE), row);
        } else {
            noDataFound(reportHistoryFields, sheet, styles.get(HEADER_CELL_STYLE));
        }
        while (numOfColumns != 0) {
            sheet.setColumnWidth(x, 6000);
            x++;
            numOfColumns--;
        }

    }

    /**
     * Loops through the reportHistoryFields and pulls the needed information in to the header of the report.
     * 
     * @param reportHistoryFields
     *            The metadata for the fields to be included in this report.
     * @param sheet
     *            The worksheet to populate.
     * @param style
     *            the CellStyle to use for these header cells
     */
    private void addHeader(List<ConceptField> reportHistoryFields, Sheet sheet, CellStyle style) {
        Row headerRow = sheet.createRow(0);
        int col = 0;

        // for (ConceptField field : reportHistoryFields) {
        // addHeaderCell(field, headerRow, col++, style);
        // }
        addHeaderCell(SOURCE, headerRow, col++, style);
        addHeaderCell(ROUTED_GENERIC_1, headerRow, col++, style);
        addHeaderCell(ROUTED_GENERIC_2, headerRow, col++, style);
        addHeaderCell(SEVERITY_LEVEL_CODE, headerRow, col++, style);
        addHeaderCell(INTERACTION_DESC, headerRow, col++, style);
        addHeaderCell(INTERACTION_ID, headerRow, col++, style);
        addHeaderCell(CORRESPONDING_FDB_ID, headerRow, col++, style);
        addHeaderCell(ACTION_DATE, headerRow, col++, style);

    }

    /**
     * No data found.
     * 
     * @param reportHistoryFields
     *            the report history fields
     * @param sheet
     *            the sheet
     * @param style
     *            the style
     */
    private void noDataFound(List<ConceptField> reportHistoryFields, Sheet sheet, CellStyle style) {
        Row headerRow = sheet.createRow(0);
        int col = 0;

        addHeaderCell("No matching data for drug report selection", headerRow, col++, style);

    }

    /**
     * Adds a Cell to the header and populates that cell with the Display Name from the ConceptField.
     * 
     * @param fieldName
     *            the field name
     * @param row
     *            a Row in a worksheet
     * @param colNum
     *            the column to populate
     * @param style
     *            the CellStyle to use for these header cells
     */
    private void addHeaderCell(String fieldName, Row row, int colNum, CellStyle style) {
        Cell cell = row.createCell(colNum);
        // cell.setCellValue(field.getDisplayName());
        cell.setCellValue(fieldName);
        cell.setCellStyle(style);
    }

    /**
     * Controls the adding of data rows to the worksheet.
     * 
     * @param data
     *            the data to include in the report
     * @param source
     *            the source
     * @param sheet
     *            the worksheet to populate
     * @param style
     *            the CellStyle to use for these data cells
     * @param row
     *            the row
     * @return the int
     */
    // private void addData(List<Object> data, List<ConceptField> reportHistoryFields, Sheet sheet, CellStyle style) {
    private int addData(List<Object> data, String source, Sheet sheet, CellStyle style, int row) {
        // int row = 1;

        for (Object dataRow : data) {
            DrugPairDTO drugPairDTO = (DrugPairDTO) dataRow;
            Row excelRow = sheet.createRow(row++);
            int col = 0;
            // Create from Blank might have a 0 for FDBID so this is not a good choice.
            if (drugPairDTO.getActionDate() == null) {
                addData("FDB", excelRow, col++, style);
            } else {
                addData("VA", excelRow, col++, style);
            }
            addData(drugPairDTO.getRtgenid1Desc(), excelRow, col++, style);
            addData(drugPairDTO.getRtgenid2Desc(), excelRow, col++, style);
            addData(drugPairDTO.getSeveritylevelcode() + "-" + drugPairDTO.getSeveritylevelcodeDesc(), excelRow, col++, style);
            addData(drugPairDTO.getInteractionidDesc(), excelRow, col++, style);
            addData(drugPairDTO.getInteractionid(), excelRow, col++, style);
            addData(drugPairDTO.getFdbId(), excelRow, col++, style);
            addData(drugPairDTO.getActionDate(), excelRow, col++, style);

        }
        return row;
    }

    /**
     * Creates a new cell and sets the value of that cell.
     * 
     * @param value
     *            the data to populate the new cell with
     * @param row
     *            a Row in the worksheet
     * @param col
     *            the column to populate
     * @param style
     *            the CellStyle to use for the data
     */
    private void addData(Object value, Row row, int col, CellStyle style) {
        Cell cell = row.createCell(col);
        setCellValue(cell, value, style);
    }

    /**
     * Creates a worksheet with an error message for the user.
     * 
     * @param sheet
     *            the worksheet to populate
     * @param errorMessage
     *            the error message to place on the worksheet
     * @param style
     *            the CellStyle to use for creating this error
     */
    private void createError(Sheet sheet, String errorMessage, CellStyle style) {
        Row row = sheet.createRow(0);
        int col = 0;
        addData(errorMessage, row, col, style);
    }

    /**
     * Retrieves the current String value for a cell.
     * 
     * @param sheet
     *            the worksheet
     * @param row
     *            the row number to the cell
     * @param col
     *            the column number of the cell
     * @return String
     */
    private String getNewValue(Sheet sheet, int row, int col) {
        Row sheetRow = sheet.getRow(row);
        return sheetRow.getCell(col - 1).getStringCellValue();
    }

    /**
     * Retrieves the current String value for a cell. If the cell has been updated, the UPDATE_INDICATOR is stripped from the
     * returned value.
     * 
     * @param sheet
     *            the worksheet
     * @param row
     *            the row number to the cell
     * @param col
     *            the column number to the cell
     * @return String
     */
    private String getOldValue(Sheet sheet, int row, int col) {

        Row sheetRow = sheet.getRow(row + 1);
        Cell cell = sheetRow.getCell(col - 1);
        String value;
        if (cell.getCellType() == Cell.CELL_TYPE_NUMERIC) {
            value = String.valueOf(cell.getNumericCellValue());
        } else {
            value = cell.getStringCellValue();
        }
        if (value.endsWith(UPDATED_INDICATOR)) {
            value = value.substring(0, value.length() - UPDATED_INDICATOR.length());
        }
        return value;
    }

    /**
     * Retrieves the current Numeric value for a cell. If the cell has been updated, the UPDATE_INDICATOR is stripped from the
     * returned value.
     * 
     * @param sheet
     *            the worksheet
     * @param row
     *            the row number to the cell
     * @param col
     *            the column number to the cell
     * @return Double
     */
    private Double getOldNumericValue(Sheet sheet, int row, int col) {
        Row sheetRow = sheet.getRow(row + 1);
        String value;
        try {
            value = sheetRow.getCell(col - 1).getStringCellValue();
        } catch (IllegalStateException ise) {
            value = "" + sheetRow.getCell(col - 1).getNumericCellValue();
        }
        if (value.endsWith(UPDATED_INDICATOR)) {
            value = value.substring(0, value.length() - UPDATED_INDICATOR.length());
        }
        if (value.length() == 0) {
            return null;
        }
        return Double.parseDouble(value);
    }

    /**
     * Appends the UPDATED_INDICATOR to the passed in value and set the new value and Font of a particular cell.
     * 
     * @param sheet
     *            the worksheet
     * @param row
     *            the row number to the cell
     * @param col
     *            the column number to the cell
     * @param value
     *            the value that should be set
     * @param style
     *            the CellStyle to use for updated data
     */
    private void setUpdated(Sheet sheet, int row, int col, String value, CellStyle style) {
        Row sheetRow = sheet.getRow(row);
        Cell cell = sheetRow.getCell(col - 1);
        cell.setCellValue(value + UPDATED_INDICATOR);
        cell.setCellStyle(style);
    }

    /**
     * Sets the metadataService.
     * 
     * @param service
     *            the metadataService to set
     */
    public void setMetadataService(MetadataService service) {
        metadataService = service;
    }

    /**
     * Called by the Spring Framework to make sure the injection succeeded.
     * 
     * @throws Exception
     *             if the injection was not successful
     */
    @Override
    public void afterPropertiesSet() throws Exception {
        if (metadataService == null) {
            throw new Exception("MetadataService has not been injected in to ReportHistoryXLSXViewer.");
        }
    }

    /**
     * Gets the terminology service.
     * 
     * @return the terminology service
     */
    public TerminologyService getTerminologyService() {
        return terminologyService;
    }

    /**
     * Sets the terminology service.
     * 
     * @param terminologyService
     *            the new terminology service
     */
    public void setTerminologyService(TerminologyService terminologyService) {
        this.terminologyService = terminologyService;
    }
}
