package gov.va.med.pharmacy.pecs.web.view;

import gov.va.med.pharmacy.pecs.domain.impl.lookup.SeverityLevel;
import gov.va.med.pharmacy.pecs.metadata.Concept;
import gov.va.med.pharmacy.pecs.metadata.ConceptField;
import gov.va.med.pharmacy.pecs.metadata.ConceptMetadata;
import gov.va.med.pharmacy.pecs.service.MetadataService;
import gov.va.med.pharmacy.pecs.service.TerminologyService;
import gov.va.med.pharmacy.pecs.web.view.document.AbstractXLSXViewer;

import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.InitializingBean;

/**
 * A class used to create a Report History Excel spreadsheet and download to the user.
 * 
 * @author vhaisaduffg
 * 
 */
public final class ReportHistoryXLSXViewer extends AbstractXLSXViewer implements InitializingBean {

    // private static final Log LOGGER = LogFactory.getLog(ReportHistoryXLSXViewer.class);

    /** The metadata service. */
    @Resource
    private MetadataService metadataService;

    /** The terminology service. */
    private TerminologyService terminologyService;

    /** The Constant HISTORY_REPORT. */
    private static final String HISTORY_REPORT = "HISTORY_REPORT";

    /**
     * The view to return to get data to display using this viewer.
     */
    public static final String REPORT_HISTORY_VIEW = "reportHistoryXLSXViewer";

    /**
     * The key in the model for the Concept that is being reported on.
     */
    public static final String METADATA_KEY = "metadataKey";
    /**
     * The key in the model for the Data that should be included in the report.
     */
    public static final String DATA_KEY = "dataKey";

    /** The Constant UPDATED_INDICATOR. */
    private static final String UPDATED_INDICATOR = " *";

    /** The Constant DECIMAL_FORMATTER. */
    private static final DecimalFormat DECIMAL_FORMATTER = new DecimalFormat("####0.######");

    /**
     * The main controller method for this class. This is the method that is responsible for the population of an Excel
     * spreadsheet with the reporting data that is needed.
     * 
     * @param model
     *            a Map that contains the data that should be placed in the report
     * @param workbook
     *            the Excel spreadsheet
     * @param request
     *            HttpServletRequest
     * @param response
     *            HttpServletResponse
     * @param styles
     *            a Map containing the CellStyle used in this workbook
     * @throws Exception
     *             if an error occurs while creating the spreadsheet.
     */
    @SuppressWarnings("unchecked")
    @Override
    protected void buildExcelDocument(Map<String, Object> model, XSSFWorkbook workbook, HttpServletRequest request,
            HttpServletResponse response, Map<String, CellStyle> styles) throws Exception {

        ConceptMetadata meta = null;
        List<ConceptField> reportHistoryFields = new ArrayList<ConceptField>();
        List<Object> data = null;

        if (model.containsKey(METADATA_KEY)) {
            Concept concept = (Concept) model.get(METADATA_KEY);
            meta = metadataService.getMetadata(concept.getConceptClass());
            if (meta != null) {
                reportHistoryFields = meta.getReportHistoryFields();
            }
        }

        if (model.containsKey(DATA_KEY)) {
            data = (List<Object>) model.get(DATA_KEY);
        }
        int x = 0;
        int numOfColumns = reportHistoryFields.size();
        Sheet sheet = workbook.createSheet(HISTORY_REPORT);

        if (meta == null) {
            createError(sheet, "No metadata was passed to the viewer.", styles.get(DEFAULT_CELL_STYLE));
        } else if (reportHistoryFields.isEmpty()) {
            createError(sheet, "No Report History metadata has been configured.", styles.get(DEFAULT_CELL_STYLE));
        } else if (data == null) {
            createError(sheet, "No data was passed to the viewer.", styles.get(DEFAULT_CELL_STYLE));
        } else {
            addHeader(reportHistoryFields, sheet, styles.get(HEADER_CELL_STYLE));
            addData(data, reportHistoryFields, sheet, styles.get(DEFAULT_CELL_STYLE));
            updatedField(sheet, reportHistoryFields, styles.get(UPDATED_CELL_STYLE));
            // autoSizeColumns(sheet, reportHistoryFields.size());
            while (numOfColumns != 0) {
                sheet.setColumnWidth(x, 6000);
                x++;
                numOfColumns--;
            }

        }
    }

    /**
     * Loops through the reportHistoryFields and pulls the needed information in to the header of the report.
     * 
     * @param reportHistoryFields
     *            The metadata for the fields to be included in this report.
     * @param sheet
     *            The worksheet to populate.
     * @param style
     *            the CellStyle to use for these header cells
     */
    private void addHeader(List<ConceptField> reportHistoryFields, Sheet sheet, CellStyle style) {
        Row headerRow = sheet.createRow(0);
        int col = 0;

        for (ConceptField field : reportHistoryFields) {
            addHeaderCell(field, headerRow, col++, style);
        }
    }

    /**
     * Adds a Cell to the header and populates that cell with the Display Name from the ConceptField.
     * 
     * @param field
     *            The ConceptField containing the heading information
     * @param row
     *            a Row in a worksheet
     * @param colNum
     *            the column to populate
     * @param style
     *            the CellStyle to use for these header cells
     */
    private void addHeaderCell(ConceptField field, Row row, int colNum, CellStyle style) {
        Cell cell = row.createCell(colNum);
        cell.setCellValue(field.getDisplayName());
        cell.setCellStyle(style);
    }

    /**
     * Controls the adding of data rows to the worksheet.
     * 
     * @param data
     *            the data to include in the report
     * @param reportHistoryFields
     *            the metadata for the fields to include
     * @param sheet
     *            the worksheet to populate
     * @param style
     *            the CellStyle to use for these data cells
     */
    private void addData(List<Object> data, List<ConceptField> reportHistoryFields, Sheet sheet, CellStyle style) {
        int row = 1;

        for (Object dataRow : data) {
            Row excelRow = sheet.createRow(row++);
            int col = 0;
            for (ConceptField field : reportHistoryFields) {

                if (field.getLookupType() == SeverityLevel.class) {

                    String fieldValue =
                            getTerminologyService().getByCode(field.getLookupType(), (String) field.getValue(dataRow))
                                    .getUiLabel();
                    addData(fieldValue, excelRow, col++, style);
                } else {
                    addData(field.getValue(dataRow), excelRow, col++, style);
                }

            }
        }
    }

    /**
     * Creates a new cell and sets the value of that cell.
     * 
     * @param value
     *            the data to populate the new cell with
     * @param row
     *            a Row in the worksheet
     * @param col
     *            the column to populate
     * @param style
     *            the CellStyle to use for the data
     */
    private void addData(Object value, Row row, int col, CellStyle style) {
        Cell cell = row.createCell(col);
        setCellValue(cell, value, style);
    }

    /**
     * Creates a worksheet with an error message for the user.
     * 
     * @param sheet
     *            the worksheet to populate
     * @param errorMessage
     *            the error message to place on the worksheet
     * @param style
     *            the CellStyle to use for creating this error
     */
    private void createError(Sheet sheet, String errorMessage, CellStyle style) {
        Row row = sheet.createRow(0);
        int col = 0;
        addData(errorMessage, row, col, style);
    }

    /**
     * Controls the processing of updated fields in this report.
     * 
     * @param sheet
     *            the worksheet in use
     * @param reportHistoryFields
     *            the metadata for the fields in the report
     * @param style
     *            the CellStyle to use for updated cells
     */
    private void updatedField(Sheet sheet, List<ConceptField> reportHistoryFields, CellStyle style) {
        int row = sheet.getLastRowNum() - 1;
        while (row > 0) {
            for (ConceptField field : reportHistoryFields) {
                if (field.isReportHistoryCompare()) {
                    Cell cell = sheet.getRow(row).getCell(field.getReportHistoryOrder() - 1);
                    switch (cell.getCellType()) {
                    case Cell.CELL_TYPE_NUMERIC:
                        Double newNumericValue = cell.getNumericCellValue();
                        Double oldNumericValue = getOldNumericValue(sheet, row, field.getReportHistoryOrder());
                        if (!newNumericValue.equals(oldNumericValue)) {
                            setUpdated(sheet, row, field.getReportHistoryOrder(), DECIMAL_FORMATTER.format(newNumericValue),
                                    style);
                        }
                        break;
                    case Cell.CELL_TYPE_STRING:
                        String newValue =
                                getNewValue(sheet, row, field.getReportHistoryOrder()).trim().replaceAll("[\r\n]", "");
                        String oldValue =
                                getOldValue(sheet, row, field.getReportHistoryOrder()).trim().replaceAll("[\r\n]", "");
                        if (!newValue.equals(oldValue)) {
                            setUpdated(sheet, row, field.getReportHistoryOrder(), newValue, style);
                        }
                        break;
                    default:
                        break;
                    }
                }
            }
            --row;
        }
    }

    /**
     * Retrieves the current String value for a cell.
     * 
     * @param sheet
     *            the worksheet
     * @param row
     *            the row number to the cell
     * @param col
     *            the column number of the cell
     * @return String
     */
    private String getNewValue(Sheet sheet, int row, int col) {
        Row sheetRow = sheet.getRow(row);
        return sheetRow.getCell(col - 1).getStringCellValue();
    }

    /**
     * Retrieves the current String value for a cell. If the cell has been updated, the UPDATE_INDICATOR is stripped from the
     * returned value.
     * 
     * @param sheet
     *            the worksheet
     * @param row
     *            the row number to the cell
     * @param col
     *            the column number to the cell
     * @return String
     */
    private String getOldValue(Sheet sheet, int row, int col) {

        Row sheetRow = sheet.getRow(row + 1);
        Cell cell = sheetRow.getCell(col - 1);
        String value;
        if (cell.getCellType() == Cell.CELL_TYPE_NUMERIC) {
            value = String.valueOf(cell.getNumericCellValue());
        } else {
            value = cell.getStringCellValue();
        }
        if (value.endsWith(UPDATED_INDICATOR)) {
            value = value.substring(0, value.length() - UPDATED_INDICATOR.length());
        }
        return value;
    }

    /**
     * Retrieves the current Numeric value for a cell. If the cell has been updated, the UPDATE_INDICATOR is stripped from the
     * returned value.
     * 
     * @param sheet
     *            the worksheet
     * @param row
     *            the row number to the cell
     * @param col
     *            the column number to the cell
     * @return Double
     */
    private Double getOldNumericValue(Sheet sheet, int row, int col) {
        Row sheetRow = sheet.getRow(row + 1);
        String value;
        try {
            value = sheetRow.getCell(col - 1).getStringCellValue();
        } catch (IllegalStateException ise) {
            value = "" + sheetRow.getCell(col - 1).getNumericCellValue();
        }
        if (value.endsWith(UPDATED_INDICATOR)) {
            value = value.substring(0, value.length() - UPDATED_INDICATOR.length());
        }
        if (value.length() == 0) {
            return null;
        }
        return Double.parseDouble(value);
    }

    /**
     * Appends the UPDATED_INDICATOR to the passed in value and set the new value and Font of a particular cell.
     * 
     * @param sheet
     *            the worksheet
     * @param row
     *            the row number to the cell
     * @param col
     *            the column number to the cell
     * @param value
     *            the value that should be set
     * @param style
     *            the CellStyle to use for updated data
     */
    private void setUpdated(Sheet sheet, int row, int col, String value, CellStyle style) {
        Row sheetRow = sheet.getRow(row);
        Cell cell = sheetRow.getCell(col - 1);
        cell.setCellValue(value + UPDATED_INDICATOR);
        cell.setCellStyle(style);
    }

    /**
     * Sets the metadataService.
     * 
     * @param service
     *            the metadataService to set
     */
    public void setMetadataService(MetadataService service) {
        metadataService = service;
    }

    /**
     * Called by the Spring Framework to make sure the injection succeeded.
     * 
     * @throws Exception
     *             if the injection was not successful
     */
    @Override
    public void afterPropertiesSet() throws Exception {
        if (metadataService == null) {
            throw new Exception("MetadataService has not been injected in to ReportHistoryXLSXViewer.");
        }
    }

    /**
     * Gets the terminology service.
     * 
     * @return the terminology service
     */
    public TerminologyService getTerminologyService() {
        return terminologyService;
    }

    /**
     * Sets the terminology service.
     * 
     * @param terminologyService
     *            the new terminology service
     */
    public void setTerminologyService(TerminologyService terminologyService) {
        this.terminologyService = terminologyService;
    }
}
