package gov.va.med.pharmacy.pecs.web.view;

import gov.va.med.pharmacy.pecs.web.view.document.AbstractXLSXViewer;

import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;

/**
 * A viewer class that takes the data from a Customization report and produces an Excel spreadsheet.
 * 
 * @author vhaisaduffg
 * 
 */
public final class ReportXLSXViewer extends AbstractXLSXViewer {

    private static final int DEBUG_REPORTABLE_ROWS = 1000;

    /**
     * The key in the model where the report data resides.
     */
    public static final String REPORT_DATA_KEY = "reportData";

    /**
     * Populates the Excel workbook with data.
     * 
     * @param model
     *            the model Map
     * @param workbook
     *            the Excel workbook to complete
     * @param request
     *            in case we need locale etc. Shouldn't look at attributes.
     * @param response
     *            in case we need to set cookies. Shouldn't write to it.
     * @param styles
     *            the CellStyle map containing the styles in this workbook
     * @throws Exception
     *             if an error occurs
     */
    @SuppressWarnings("unchecked")
    @Override
    protected void buildExcelDocument(Map<String, Object> model, XSSFWorkbook workbook, HttpServletRequest request,
            HttpServletResponse response, Map<String, CellStyle> styles) throws Exception {
        if (model.containsKey(REPORT_DATA_KEY)) {
            createSheet(workbook, (List<List<Object>>) model.get(REPORT_DATA_KEY), styles);
        }
    }

    /**
     * Creates a worksheet and populates the worksheet with data.
     * 
     * @param workbook
     *            the Excel workbook
     * @param data
     *            the report data from the model
     * @param styles
     *            the cell styles
     */
    private void createSheet(XSSFWorkbook workbook, List<List<Object>> data, Map<String, CellStyle> styles) {

        Sheet sheet = workbook.createSheet("Report Data");
        int rowNum = 0;
        CellStyle style = styles.get(HEADER_CELL_STYLE);

        if (logger.is   ugEnabled()) {
            logger.debug("Trying to write " + data.size() + " rows to spreadsheet.");
        }
        for (List<Object> d : data) {
            if (rowNum > 0) {
                style = styles.get(DEFAULT_CELL_STYLE);
            }
            Row row = sheet.createRow(rowNum++);
            int colNum = 0;
            for (Object field : d) {
                Cell cell = row.createCell(colNum++);
                setCellValue(cell, field, style);
            }
            if (logger.is   ugEnabled() && (rowNum % DEBUG_REPORTABLE_ROWS) == 0) {
                logger.debug("Placed " + rowNum + " rows in the spreadsheet.");
            }
        }

        sheet.createFreezePane(0, 1, 0, 1);
        // Size columns - This has to be done after the data is populated.
        int column = 0;

        // Row 0 should be the header
        int numberOfColumns = sheet.getRow(0).getLastCellNum();
        for (int i = 0; i < numberOfColumns; i++) {
            sheet.autoSizeColumn(column++);
        }
    }
}
