/*
 * Copyright 2002-2009 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package gov.va.med.pharmacy.pecs.web.view.document;

import java.util.Date;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.CreationHelper;
import org.apache.poi.ss.usermodel.IndexedColors;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.xssf.usermodel.XSSFCell;
import org.apache.poi.xssf.usermodel.XSSFFont;
import org.apache.poi.xssf.usermodel.XSSFRow;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.core.io.Resource;
import org.springframework.core.io.support.LocalizedResourceHelper;
import org.springframework.web.servlet.support.RequestContextUtils;
import org.springframework.web.servlet.view.AbstractView;

/**
 * This code was taken from the Spring Framework 3.0 AbstractExcelView. Modifications were made to allow for XLSX generation
 * instead of XLS.
 * 
 * Convenient superclass for Excel document views. Compatible with Apache POI 3.0 as well as 3.5, as of Spring 3.0.
 * 
 * <p>
 * Properties:
 * <ul>
 * <li>url (optional): The url of an existing Excel document to pick as a starting point. It is done without localization part
 * nor the ".xls" extension.
 * </ul>
 * 
 * <p>
 * The file will be searched with locations in the following order:
 * <ul>
 * <li>[url]_[language]_[country].xls
 * <li>[url]_[language].xls
 * <li>[url].xls
 * </ul>
 * 
 * <p>
 * For working with the workbook in the subclass, see <a href="http://jakarta.apache.org/poi/index.html">Jakarta's POI site</a>
 * 
 * <p>
 * As an example, you can try this snippet:
 * 
 * <pre class="code">
 * protected void buildExcelDocument(Map&lt;String, Object&gt; model, XSSFWorkbook workbook, HttpServletRequest request,
 *         HttpServletResponse response) {
 * 
 *     // Go to the first sheet.
 *     // getSheetAt: only if workbook is created from an existing document
 *     // XSSFSheet sheet = workbook.getSheetAt(0);
 *     XSSFSheet sheet = workbook.createSheet(&quot;Spring&quot;);
 *     sheet.setDefaultColumnWidth(12);
 * 
 *     // Write a text at A1.
 *     XSSFCell cell = getCell(sheet, 0, 0);
 *     setText(cell, &quot;Spring POI test&quot;);
 * 
 *     // Write the current date at A2.
 *     XSSFCellStyle dateStyle = workbook.createCellStyle();
 *     dateStyle.setDataFormat(XSSFDataFormat.getBuiltinFormat(&quot;m/d/yy&quot;));
 *     cell = getCell(sheet, 1, 0);
 *     cell.setCellValue(new Date());
 *     cell.setCellStyle(dateStyle);
 * 
 *     // Write a number at A3
 *     getCell(sheet, 2, 0).setCellValue(458);
 * 
 *     // Write a range of numbers.
 *     XSSFRow sheetRow = sheet.createRow(3);
 *     for (short i = 0; i &lt; 10; i++) {
 *         sheetRow.createCell(i).setCellValue(i * 10);
 *     }
 * }
 * </pre>
 * 
 * This class is similar to the org.springframework.web.servlet.view.document.AbstractPdfView class in usage style.
 * 
 * @author Jean-Pierre Pawlak
 * @author Juergen Hoeller
 * @see org.springframework.web.servlet.view.document.AbstractPdfView
 */
public abstract class AbstractXLSXViewer extends AbstractView {

    /** The content type for an Excel response. */
    private static final String CONTENT_TYPE = "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet";

    /** The extension to look for existing templates. */
    private static final String EXTENSION = ".xlsx";

    /**
     * If the file should be specifically named when it is download through the browser, this key can be used to set that file
     * name value in the model.
     */
    public static final String EXCEL_FILE_NAME_KEY = "excelFileNameKey";

    /** The Constant EXCEL_FILE_TYPE_KEY. */
    public static final String EXCEL_FILE_TYPE_KEY = "excelFileTypeKey";
    /**
     * If the report is an export of a query result, the query criteria that is used to generate the data can be set in the
     * model via this key. This will produce a worksheet in the workbook that contains the criteria used to produce the report.
     */
    public static final String CRITERIA_STRING = "criteriaString";

    /**
     * The default style for cells.
     */
    protected static final String DEFAULT_CELL_STYLE = "defaultCellStyle";
    /**
     * The cell style for cells in header row(s).
     */
    protected static final String HEADER_CELL_STYLE = "headerCellStyle";
    /**
     * The cell style for "updated" values.
     */
    protected static final String UPDATED_CELL_STYLE = "updatedCellStyle";

    /** The url. */
    private String url;

    /**
     * Default Constructor. Sets the content type of the view to "application/vnd.ms-excel".
     */
    public AbstractXLSXViewer() {
        setContentType(CONTENT_TYPE);
    }

    /**
     * Set the URL of the Excel workbook source, without localization part nor extension.
     * 
     * @param url
     *            if the workbook to be created should be created based on a template, pass the template workbook via this URL.
     */
    public final void setUrl(String url) {
        this.url = url;
    }

    /**
     * A Spring required method that determines if this viewer produces downloadable content or not.
     * 
     * @return true, this viewer generates downloadable content
     */
    @Override
    protected final boolean generatesDownloadContent() {
        return true;
    }

    /**
     * Creates the styles to be used in this workbook and returns them in a map. The styles being created are the *_CELL_STYLE
     * static variables defined above.
     * 
     * @param workbook
     *            the Excel workbook
     * @return Map<String,CellStyle>
     */
    private Map<String, CellStyle> createFonts(XSSFWorkbook workbook) {
        Map<String, CellStyle> styles = new HashMap<String, CellStyle>();
        CreationHelper createHelper = workbook.getCreationHelper();

        CellStyle defaultStyle = workbook.createCellStyle();
        defaultStyle.setWrapText(true);
        defaultStyle.setDataFormat(createHelper.createDataFormat().getFormat("yyyy-MM-dd HH:mm:ss"));
        styles.put(DEFAULT_CELL_STYLE, defaultStyle);

        CellStyle headerStyle = workbook.createCellStyle();
        XSSFFont headerFont = workbook.createFont();
        headerFont.setBold(true);
        headerStyle.cloneStyleFrom(defaultStyle);
        headerStyle.setFont(headerFont);
        styles.put(HEADER_CELL_STYLE, headerStyle);

        CellStyle updatedStyle = workbook.createCellStyle();
        updatedStyle.cloneStyleFrom(defaultStyle);
        XSSFFont updatedFont = workbook.createFont();
        updatedFont.setBold(true);
        updatedFont.setColor(IndexedColors.RED.getIndex());
        updatedStyle.setFont(updatedFont);
        styles.put(UPDATED_CELL_STYLE, updatedStyle);

        return styles;
    }

    /**
     * Renders the Excel view, given the specified model.
     * 
     * @param model
     *            a Map containing the data that will be used to produce the view
     * @param request
     *            HttpServletRequest
     * @param response
     *            HttpServletResponse
     * @throws Exception
     *             if an error is encountered
     */
    @Override
    protected final void renderMergedOutputModel(Map<String, Object> model, HttpServletRequest request,
            HttpServletResponse response) throws Exception {

        XSSFWorkbook workbook;
        if (this.url != null) {
            workbook = getTemplateSource(this.url, request);
        } else {
            workbook = new XSSFWorkbook();
            logger.debug("Created Excel Workbook from scratch");
        }

        if (model.containsKey(EXCEL_FILE_NAME_KEY)) {

            Object fileName = model.get(EXCEL_FILE_NAME_KEY);
            Object fileType = new String("");

            if (model.containsKey(EXCEL_FILE_TYPE_KEY)) {
                fileType = "_" + model.get(EXCEL_FILE_TYPE_KEY);
            }

            response.addHeader("Content-Disposition", "attachment; filename=\"" + fileName + fileType + EXTENSION + "\"");
            model.remove(EXCEL_FILE_NAME_KEY);
        }
        if (model.containsKey("concept")) {
            model.remove("concept");

        }
        if (model.containsKey("recordType")) {
            model.remove("recordType");

        }
        Map<String, CellStyle> styles = createFonts(workbook);
        buildExcelDocument(model, workbook, request, response, styles);

        workbook.setSelectedTab(0);

        // Set the content type.
        response.setContentType(getContentType());

        // Flush byte array to servlet output stream.
        ServletOutputStream out = response.getOutputStream();
        workbook.write(out);
        out.flush();
    }

    /**
     * Creates the workbook from an existing XLS document.
     * 
     * @param templateUrl
     *            the URL of the Excel template without localization part nor extension
     * @param request
     *            current HTTP request
     * @return the XSSFWorkbook
     * @throws Exception
     *             in case of failure
     */
    protected final XSSFWorkbook getTemplateSource(String templateUrl, HttpServletRequest request) throws Exception {
        LocalizedResourceHelper helper = new LocalizedResourceHelper(getApplicationContext());
        Locale userLocale = RequestContextUtils.getLocale(request);
        Resource inputFile = helper.findLocalizedResource(templateUrl, EXTENSION, userLocale);

        // Create the Excel document from the source.
        if (logger.is   ugEnabled()) {
            logger.debug("Loading Excel workbook from " + inputFile);
        }
        return new XSSFWorkbook(inputFile.getInputStream());
    }

    /**
     * Subclasses must implement this method to create an Excel HSSFWorkbook document, given the model.
     * 
     * @param model
     *            the model Map
     * @param workbook
     *            the Excel workbook to complete
     * @param request
     *            in case we need locale etc. Shouldn't look at attributes.
     * @param response
     *            in case we need to set cookies. Shouldn't write to it.
     * @param styles
     *            the CellStyle map containing the styles in this workbook
     * @throws Exception
     *             if an error occurs
     */
    protected abstract void buildExcelDocument(Map<String, Object> model, XSSFWorkbook workbook, HttpServletRequest request,
            HttpServletResponse response, Map<String, CellStyle> styles) throws Exception;

    /**
     * Convenient method to obtain the cell in the given sheet, row and column.
     * <p>
     * Creates the row and the cell if they still doesn't already exist. Thus, the column can be passed as an int, the method
     * making the needed downcasts.
     * 
     * @param sheet
     *            a sheet object. The first sheet is usually obtained by workbook.getSheetAt(0)
     * @param row
     *            thr row number
     * @param col
     *            the column number
     * @return the HSSFCell
     */
    protected final XSSFCell getCell(XSSFSheet sheet, int row, int col) {
        XSSFRow sheetRow = sheet.getRow(row);
        if (sheetRow == null) {
            sheetRow = sheet.createRow(row);
        }
        XSSFCell cell = sheetRow.getCell((short) col);
        if (cell == null) {
            cell = sheetRow.createCell((short) col);
        }
        return cell;
    }

    /**
     * Convenient method to set a String as text content in a cell.
     * 
     * @param cell
     *            the cell in which the text must be put
     * @param text
     *            the text to put in the cell
     */
    protected final void setText(XSSFCell cell, String text) {
        cell.setCellType(XSSFCell.CELL_TYPE_STRING);
        cell.setCellValue(text);
    }

    /**
     * A method that allows sub-classes to set the values and style of individual cells.
     * 
     * @param cell
     *            the Cell to set the value
     * @param val
     *            the value to set
     * @param cellStyle
     *            the style to use for the cell
     */
    protected final void setCellValue(Cell cell, Object val, CellStyle cellStyle) {
        if (val == null) {
            cell.setCellValue("");
        } else if (val instanceof Date) {
            cell.setCellValue((Date) val);
            cell.setCellStyle(cellStyle);
        } else if (val instanceof Number) {
            cell.setCellValue(((Number) val).doubleValue());
        } else {
            cell.setCellValue(val.toString());
            cell.setCellStyle(cellStyle);
        }
    }

    /**
     * Loops through all columns in the worksheet autosizing the column.
     * 
     * @param sheet
     *            the worksheet
     * @param numberOfColumns
     *            the number of columns on the worksheet
     */
    protected final void autoSizeColumns(Sheet sheet, int numberOfColumns) {
        for (int i = 0; i < numberOfColumns; i++) {
            sheet.autoSizeColumn(i);
        }

    }
}
