SET LINESIZE 1000
SET PAGESIZE 60
SPOOL PECS5_migration.log

SELECT 'Migration Started: ' || CURRENT_TIMESTAMP FROM DUAL;

SET ECHO ON

-- Compile PROCEDURE concept_export_migration that is used in the PECS 5 migration

CREATE OR REPLACE PROCEDURE concept_export_migration (
   source_table   IN VARCHAR2
  ,destination    IN VARCHAR2
  ,key_field      IN VARCHAR2)
IS
   id_var               NUMBER;
   exportable_id_var    NUMBER;
   exported_id_var      NUMBER;
   active_id_var        NUMBER;
   alter_cursor         INTEGER;
   source_cursor        INTEGER;
   destination_cursor   INTEGER;
   ignore               INTEGER;
   row_count            INTEGER := 0;
   sql_stmt             VARCHAR2 (150);
BEGIN
   -- Prepare a cursor to select from the source table:
   sql_stmt :=
         'SELECT '
      || key_field
      || ', exportable_version_id, exported_version_id, '
      || CASE
            WHEN (source_table IN ('CTSTAGING.PECS_DUPL_THERAPY_VA'
                                  ,'CTSTAGING.PECS_DOSE_RANGE_VA'
                                  ,'CTSTAGING.PECS_MONOGRAPH_VA'))
            THEN
               'CRT_VERSION_ID'
            WHEN (source_table = 'CTSTAGING.PECS_DRUG_PAIR_VA')
            THEN
               'CURRENT_VERSION_ID'
            ELSE
               'ACTIVE_VERSION_ID'
         END
      || ' FROM '
      || source_table;
   source_cursor := DBMS_SQL.open_cursor;
   DBMS_SQL.parse (source_cursor, sql_stmt, DBMS_SQL.native);
   DBMS_SQL.define_column (source_cursor, 1, id_var);
   DBMS_SQL.define_column (source_cursor, 2, exportable_id_var);
   DBMS_SQL.define_column (source_cursor, 3, exported_id_var);
   DBMS_SQL.define_column (source_cursor, 4, active_id_var);
   ignore := DBMS_SQL.execute (source_cursor);

   -- Prepare a cursor to insert into the destination table:
   destination_cursor := DBMS_SQL.open_cursor;
   DBMS_SQL.parse (
      destination_cursor
     ,   'INSERT INTO '
      || destination
      || ' VALUES (:id_bind, :exportable_bind, :exported_bind, :active_bind)'
     ,DBMS_SQL.native);

   -- Fetch a row from the source table and insert it into the destination table:
   LOOP
      IF DBMS_SQL.fetch_rows (source_cursor) > 0
      THEN
         -- get column values of the row
         DBMS_SQL.COLUMN_VALUE (source_cursor, 1, id_var);
         DBMS_SQL.COLUMN_VALUE (source_cursor, 2, exportable_id_var);
         DBMS_SQL.COLUMN_VALUE (source_cursor, 3, exported_id_var);
         DBMS_SQL.COLUMN_VALUE (source_cursor, 4, active_id_var);

         -- Bind the row into the cursor that inserts into the destination table. You
         -- could alter this example to require the use of dynamic SQL by inserting an
         -- if condition before the bind.
         DBMS_SQL.bind_variable (destination_cursor, ':id_bind', id_var);
         DBMS_SQL.bind_variable (destination_cursor
                                ,':exportable_bind'
                                ,exportable_id_var);
         DBMS_SQL.bind_variable (destination_cursor
                                ,':exported_bind'
                                ,exported_id_var);
         DBMS_SQL.bind_variable (destination_cursor
                                ,':active_bind'
                                ,active_id_var);
         ignore := DBMS_SQL.execute (destination_cursor);
         row_count := row_count + 1;
      ELSE
         -- No more rows to copy:
         EXIT;
      END IF;
   END LOOP;

   DBMS_OUTPUT.put_line (
         'Concept table created: '
      || destination
      || '.  Number of rows migrated: '
      || TO_CHAR (row_count));

   -- create cursor to remove column from destination table



   SELECT    'ALTER TABLE '
          || source_table
          || ' DROP ('
          || CASE
                WHEN (source_table IN ('CTSTAGING.PECS_DUPL_THERAPY_VA'
                                      ,'CTSTAGING.PECS_DOSE_RANGE_VA'
                                      ,'CTSTAGING.PECS_MONOGRAPH_VA'))
                THEN
                   'CRT_VERSION_ID'
                WHEN (source_table = 'CTSTAGING.PECS_DRUG_PAIR_VA')
                THEN
                   'CURRENT_VERSION_ID'
                ELSE
                   'ACTIVE_VERSION_ID'
             END
          || ', EXPORTED_VERSION_ID, EXPORTABLE_VERSION_ID) CASCADE CONSTRAINTS'
     INTO sql_stmt
     FROM DUAL;

   alter_cursor := DBMS_SQL.open_cursor;
   DBMS_SQL.parse (alter_cursor, sql_stmt, DBMS_SQL.native);
   ignore := DBMS_SQL.execute (alter_cursor);
   DBMS_OUTPUT.put_line (
         'Concept table modified, version columns removed from: '
      || source_table
      || '.');
   -- Commit and close all cursors:
   COMMIT;
   DBMS_SQL.close_cursor (source_cursor);
   DBMS_SQL.close_cursor (destination_cursor);
   DBMS_SQL.close_cursor (alter_cursor);
EXCEPTION
   WHEN OTHERS
   THEN
      IF DBMS_SQL.is_open (source_cursor)
      THEN
         DBMS_SQL.close_cursor (source_cursor);
      END IF;

      IF DBMS_SQL.is_open (destination_cursor)
      THEN
         DBMS_SQL.close_cursor (destination_cursor);
      END IF;

      RAISE;
END;
/

SHOW ERRORS


REM ###########################################################################
REM #  Description: Create PECS 5 database objects
REM #
REM #
REM #   Usage:
REM #   Input Parameters:
REM #   1) None
REM #   Output:
REM #   1) Terminal
REM #   Internal Script Parameters:
REM #   1) None
REM #
REM ###########################################################################
REM # Change History
REM #
REM # DATE       User Name         DESCRIPTION
REM # ---------- ----------------- -------------------------------------------
REM # 11/17/2013 Duncan Shelley    Created script
REM #
REM ##########################################################################

DECLARE
   -- CONSTANTS

   cc_schema_name   CONSTANT VARCHAR2 (35) := 'CTSTAGING';

   -- VARIABLES

   vc_object_name            VARCHAR2 (30);
   vc_sql_statement          VARCHAR2 (4000);
   vc_errm                   VARCHAR2 (64);
   vn_code                   NUMBER;
   vn_continue               NUMBER;

   vn_row_count              NUMBER;

   -----------------------------------------------------------------------------
   -- NAME:         check_for_pecs5
   -- TYPE:         Function
   -- DESCRIPTSION: Check to see if PECS 5 object already created
   -- INPUTS:       None
   -- OUTPUTS:      Number: (0 - good: >0 - error)
   -----------------------------------------------------------------------------
   FUNCTION check_for_pecs5
      RETURN NUMBER
   IS
   BEGIN
      SELECT COUNT (*)
        INTO vn_row_count
        FROM sys.dba_tables
       WHERE     owner = cc_schema_name
             AND table_name IN ('MONOGRAPH_COMMENT'
                               ,'FDB_MONOGRAPH_XREF'
                               ,'DUPLTHERAPY_COMMENT'
                               ,'FDB_DUPTHERAPY_XREF'
                               ,'DOSERANGE_COMMENT'
                               ,'FDB_DOSE_RANGE_XREF'
                               ,'FDB_DDIM_XREF'
                               ,'DDIMINTERACTION_COMMENT'
                               ,'PECS_CONTACT'
                               ,'PECS_DDI_EXPORT_VA'
                               ,'PECS_DRUG_PAIR_EXPORT_VA'
                               ,'PECS_DOSE_RANGE_EXPORT_VA'
                               ,'PECS_DUPL_THERAPY_EXPORT_VA'
                               ,'PECS_MONOGRAPH_EXPORT_VA');

      IF vn_row_count > 0
      THEN
         RETURN (1);
      END IF;

      RETURN (0);
   EXCEPTION
      WHEN OTHERS
      THEN
         vn_code := SQLCODE;
         vc_errm := SUBSTR (SQLERRM, 1, 64);
         DBMS_OUTPUT.put_line (
            'check_for_pecs5: Error code ' || vn_code || ': ' || vc_errm);
         sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
         RETURN (1);
   END check_for_pecs5;

   -----------------------------------------------------------------------------
   -- NAME:         create_fdb_comments_revised
   -- TYPE:         Function
   -- DESCRIPTSION: Create objects that were in Create_FDB_COMMENTS_Revised.sql
   -- INPUTS:       None
   -- OUTPUTS:      Number: (0 - good: >0 - error)
   -----------------------------------------------------------------------------
   FUNCTION create_fdb_comments_revised
      RETURN NUMBER
   IS
   BEGIN
      sys.DBMS_OUTPUT.put_line (CHR (10));
      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   START:  create_fdb_comments_revised'); -- Create sequences section -------------------------------------------------

      -- Create sequences section -------------------------------------------------
      vc_sql_statement :=
         'CREATE SEQUENCE CTSTAGING.DDIM_COMMENT_ID_SEQ START WITH 1 MAXVALUE 9999999999999999999999999999 MINVALUE 1 CACHE 20';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'CREATE SEQUENCE CTSTAGING.FDB_DDIM_REF_ID_SEQ START WITH 1 MAXVALUE 9999999999999999999999999999 MINVALUE 1 CACHE 20';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'CREATE SEQUENCE CTSTAGING.MONOGRAPH_COMMENT_ID_SEQ START WITH 1 MAXVALUE 9999999999999999999999999999 MINVALUE 1 CACHE 20';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'CREATE SEQUENCE CTSTAGING.FDB_MONOGRAPH_REF_ID_SEQ START WITH 1 MAXVALUE 9999999999999999999999999999 MINVALUE 1 CACHE 20';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'CREATE SEQUENCE CTSTAGING.DUPLTHERAPY_COMMENT_ID_SEQ START WITH 1 MAXVALUE 9999999999999999999999999999 MINVALUE 1 CACHE 20';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'CREATE SEQUENCE CTSTAGING.FDB_DUPLTHERAPY_REF_ID_SEQ START WITH 1 MAXVALUE 9999999999999999999999999999 MINVALUE 1 CACHE 20';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'CREATE SEQUENCE CTSTAGING.DOSEROUTE_COMMENT_ID_SEQ START WITH 1 MAXVALUE 9999999999999999999999999999 MINVALUE 1 CACHE 20';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'CREATE SEQUENCE CTSTAGING.FDB_DOSEROUTE_REF_ID_SEQ START WITH 1 MAXVALUE 9999999999999999999999999999 MINVALUE 1 CACHE 20';

      EXECUTE IMMEDIATE vc_sql_statement;

      -- Create tables section -------------------------------------------------

      -- Table CTSTAGING.DDIMINTERACTION_COMMENT

      vc_sql_statement :=
            'CREATE TABLE CTSTAGING.DDIMINTERACTION_COMMENT('
         || 'COMMENT_ID Integer NOT NULL,'
         || 'FDB_REFERENCE_ID Integer,'
         || 'COMMENT_TEXT Varchar2(4000 BYTE),'
         || 'COMMENT_DATE Timestamp(6), '
         || 'COMMENT_USER Varchar2(50 BYTE) NOT NULL) '
         || 'TABLESPACE CTSTAGING_DATA';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'CREATE UNIQUE INDEX CTSTAGING.XPKDDIMCOMMENTID ON CTSTAGING.DDIMINTERACTION_COMMENT (COMMENT_ID) TABLESPACE CTSTAGING_INDEX';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.DDIMINTERACTION_COMMENT ADD CONSTRAINT PKDDIMCOMMENTID PRIMARY KEY (COMMENT_ID) USING INDEX CTSTAGING.XPKDDIMCOMMENTID';

      EXECUTE IMMEDIATE vc_sql_statement;

      -- Table CTSTAGING.FDB_DDIM_XREF

      vc_sql_statement :=
            'CREATE TABLE CTSTAGING.FDB_DDIM_XREF( '
         || 'FDB_REFERENCE_ID Integer NOT NULL, '
         || 'INTERACTION_ID Integer ) '
         || 'TABLESPACE CTSTAGING_DATA';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'CREATE UNIQUE INDEX CTSTAGING.XPK_FDB_DDIM_XREF ON CTSTAGING.FDB_DDIM_XREF (FDB_REFERENCE_ID) TABLESPACE CTSTAGING_INDEX';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.FDB_DDIM_XREF ADD CONSTRAINT PK_FDB_DDIM_XREF PRIMARY KEY (FDB_REFERENCE_ID) USING INDEX CTSTAGING.XPK_FDB_DDIM_XREF';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'COMMENT ON COLUMN CTSTAGING.FDB_DDIM_XREF.INTERACTION_ID IS ''Interaction ID from FDB_DDIMINTERATION table ''';

      EXECUTE IMMEDIATE vc_sql_statement;

      -- Table CTSTAGING.FDB_DOSE_RANGE_XREF

      vc_sql_statement :=
            'CREATE TABLE CTSTAGING.FDB_DOSE_RANGE_XREF( '
         || 'FDB_REFERENCE_ID Integer NOT NULL, '
         || 'CONCEPTTYPE Integer, '
         || 'CONCEPTID Integer, '
         || 'AGELOWINDAYS Integer, '
         || 'AGEHIGHINDAYS Integer, '
         || 'DOSEROUTEID Varchar2(3 BYTE), '
         || 'DOSETYPEID Varchar2(2 BYTE), '
         || 'FDBDX Varchar2(9 BYTE), '
         || 'HITTYPE Integer ) '
         || 'TABLESPACE CTSTAGING_DATA';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'CREATE UNIQUE INDEX CTSTAGING.XPK_FDB_DOSE_XREF ON CTSTAGING.FDB_DOSE_RANGE_XREF (FDB_REFERENCE_ID) TABLESPACE CTSTAGING_INDEX';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.FDB_DOSE_RANGE_XREF ADD CONSTRAINT PK_FDB_DOSE_XREF PRIMARY KEY (FDB_REFERENCE_ID) USING INDEX CTSTAGING.XPK_FDB_DOSE_XREF';

      EXECUTE IMMEDIATE vc_sql_statement;

      -- Table CTSTAGING.DOSERANGE_COMMENT

      vc_sql_statement :=
            'CREATE TABLE CTSTAGING.DOSERANGE_COMMENT( '
         || 'COMMENT_ID Integer NOT NULL, '
         || 'FDB_REFERENCE_ID Integer, '
         || 'COMMENT_TEXT Varchar2(4000 BYTE), '
         || 'COMMENT_DATE Timestamp(6), '
         || 'COMMENT_USER Varchar2(50 BYTE) NOT NULL) '
         || ' TABLESPACE CTSTAGING_DATA';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'CREATE UNIQUE INDEX CTSTAGING.XPKDOSECOMMENTID ON CTSTAGING.DOSERANGE_COMMENT (COMMENT_ID) TABLESPACE CTSTAGING_INDEX';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.DOSERANGE_COMMENT ADD CONSTRAINT PKDOSECOMMENTID PRIMARY KEY (COMMENT_ID) USING INDEX CTSTAGING.XPKDOSECOMMENTID';

      EXECUTE IMMEDIATE vc_sql_statement;

      -- Table CTSTAGING.FDB_DUPTHERAPY_XREF

      vc_sql_statement :=
            'CREATE TABLE CTSTAGING.FDB_DUPTHERAPY_XREF( '
         || 'FDB_REFERENCE_ID Integer NOT NULL, '
         || 'DTCID Integer ) '
         || 'TABLESPACE CTSTAGING_DATA';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'CREATE UNIQUE INDEX CTSTAGING.XPK_FDB_DT_XREF ON CTSTAGING.FDB_DUPTHERAPY_XREF (FDB_REFERENCE_ID) TABLESPACE CTSTAGING_INDEX';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.FDB_DUPTHERAPY_XREF ADD CONSTRAINT PK_FDB_DT_XREF PRIMARY KEY (FDB_REFERENCE_ID) USING INDEX CTSTAGING.XPK_FDB_DT_XREF';

      EXECUTE IMMEDIATE vc_sql_statement;

      -- Table CTSTAGING.DUPLTHERAPY_COMMENT

      vc_sql_statement :=
            'CREATE TABLE CTSTAGING.DUPLTHERAPY_COMMENT( '
         || 'COMMENT_ID Integer NOT NULL, '
         || 'FDB_REFERENCE_ID Integer, '
         || 'COMMENT_TEXT Varchar2(4000 BYTE), '
         || 'COMMENT_DATE Timestamp(6), '
         || 'COMMENT_USER Varchar2(50 BYTE) NOT NULL) '
         || 'TABLESPACE CTSTAGING_DATA';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'CREATE UNIQUE INDEX CTSTAGING.XPKDTCOMMENTID ON CTSTAGING.DUPLTHERAPY_COMMENT (COMMENT_ID) TABLESPACE CTSTAGING_INDEX';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.DUPLTHERAPY_COMMENT ADD CONSTRAINT PKDTCOMMENTID PRIMARY KEY (COMMENT_ID) USING INDEX CTSTAGING.XPKDTCOMMENTID';

      EXECUTE IMMEDIATE vc_sql_statement;

      -- Table CTSTAGING.FDB_MONOGRAPH_XREF

      vc_sql_statement :=
            'CREATE TABLE CTSTAGING.FDB_MONOGRAPH_XREF( '
         || 'FDB_REFERENCE_ID Integer NOT NULL, '
         || 'LANGUAGEID Integer, '
         || 'VERSIONCODE Varchar2(10 ), '
         || 'MONOGRAPHID Integer, '
         || 'SEQUENCENUMBER Integer) '
         || 'TABLESPACE CTSTAGING_DATA';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'CREATE UNIQUE INDEX CTSTAGING.XPK_FDB_MONO_XREF ON CTSTAGING.FDB_MONOGRAPH_XREF (FDB_REFERENCE_ID) TABLESPACE CTSTAGING_INDEX';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.FDB_MONOGRAPH_XREF ADD CONSTRAINT PK_FDB_MONO_XREF PRIMARY KEY (FDB_REFERENCE_ID) USING INDEX CTSTAGING.XPK_FDB_MONO_XREF';

      EXECUTE IMMEDIATE vc_sql_statement;

      -- Table CTSTAGING.MONOGRAPH_COMMENT

      vc_sql_statement :=
            'CREATE TABLE CTSTAGING.MONOGRAPH_COMMENT( '
         || 'COMMENT_ID Integer NOT NULL, '
         || 'FDB_REFERENCE_ID Integer, '
         || 'COMMENT_TEXT Varchar2(4000 BYTE), '
         || 'COMMENT_DATE Timestamp(6), '
         || 'COMMENT_USER Varchar2(50 BYTE) NOT NULL) '
         || 'TABLESPACE CTSTAGING_DATA';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'CREATE UNIQUE INDEX CTSTAGING.XPKMONOCOMMENTID ON CTSTAGING.MONOGRAPH_COMMENT (COMMENT_ID) TABLESPACE CTSTAGING_INDEX';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.MONOGRAPH_COMMENT ADD CONSTRAINT PKMONOCOMMENTID PRIMARY KEY (COMMENT_ID) USING INDEX CTSTAGING.XPKMONOCOMMENTID';

      EXECUTE IMMEDIATE vc_sql_statement;

      -- Create synonyms section -------------------------------------------------

      vc_sql_statement :=
         'CREATE OR REPLACE PUBLIC SYNONYM DDIMINTERACTION_COMMENT FOR CTSTAGING.DDIMINTERACTION_COMMENT';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'CREATE OR REPLACE PUBLIC SYNONYM DDIM_COMMENT_ID_SEQ FOR CTSTAGING.DDIM_COMMENT_ID_SEQ';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'CREATE OR REPLACE PUBLIC SYNONYM FDB_DDIM_REF_ID_SEQ FOR CTSTAGING.FDB_DDIM_REF_ID_SEQ';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'CREATE OR REPLACE PUBLIC SYNONYM FDB_DDIM_XREF FOR CTSTAGING.FDB_DDIM_XREF';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'CREATE OR REPLACE PUBLIC SYNONYM FDB_DUPTHERAPY_XREF FOR CTSTAGING.FDB_DUPTHERAPY_XREF';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'CREATE OR REPLACE PUBLIC SYNONYM FDB_DOSE_RANGE_XREF FOR CTSTAGING.FDB_DOSE_RANGE_XREF';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'CREATE OR REPLACE PUBLIC SYNONYM FDB_MONOGRAPH_XREF FOR CTSTAGING.FDB_MONOGRAPH_XREF';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'CREATE OR REPLACE PUBLIC SYNONYM DOSERANGE_COMMENT FOR CTSTAGING.DOSERANGE_COMMENT';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'CREATE OR REPLACE PUBLIC SYNONYM DUPLTHERAPY_COMMENT FOR CTSTAGING.DUPLTHERAPY_COMMENT';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'CREATE OR REPLACE PUBLIC SYNONYM MONOGRAPH_COMMENT FOR CTSTAGING.MONOGRAPH_COMMENT';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'CREATE OR REPLACE PUBLIC SYNONYM DOSEROUTE_COMMENT_ID_SEQ FOR CTSTAGING.DOSEROUTE_COMMENT_ID_SEQ';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'CREATE OR REPLACE PUBLIC SYNONYM DUPLTHERAPY_COMMENT_ID_SEQ FOR CTSTAGING.DUPLTHERAPY_COMMENT_ID_SEQ';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'CREATE OR REPLACE PUBLIC SYNONYM FDB_DOSEROUTE_REF_ID_SEQ FOR CTSTAGING.FDB_DOSEROUTE_REF_ID_SEQ';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'CREATE OR REPLACE PUBLIC SYNONYM FDB_DUPLTHERAPY_REF_ID_SEQ FOR CTSTAGING.FDB_DUPLTHERAPY_REF_ID_SEQ';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'CREATE OR REPLACE PUBLIC SYNONYM FDB_MONOGRAPH_REF_ID_SEQ FOR CTSTAGING.FDB_MONOGRAPH_REF_ID_SEQ';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'CREATE OR REPLACE PUBLIC SYNONYM MONOGRAPH_COMMENT_ID_SEQ FOR CTSTAGING.MONOGRAPH_COMMENT_ID_SEQ';

      EXECUTE IMMEDIATE vc_sql_statement;

      -- Create relationships section -------------------------------------------------

      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.DDIMINTERACTION_COMMENT ADD CONSTRAINT FK_XREF_DDIMCOMMENT FOREIGN KEY (FDB_REFERENCE_ID) REFERENCES CTSTAGING.FDB_DDIM_XREF (FDB_REFERENCE_ID)';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.DOSERANGE_COMMENT ADD CONSTRAINT FK_XREF_DRCOMMENT FOREIGN KEY (FDB_REFERENCE_ID) REFERENCES CTSTAGING.FDB_DOSE_RANGE_XREF (FDB_REFERENCE_ID)';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.DUPLTHERAPY_COMMENT ADD CONSTRAINT FK_XREF_DTCOMMENT FOREIGN KEY (FDB_REFERENCE_ID) REFERENCES CTSTAGING.FDB_DUPTHERAPY_XREF (FDB_REFERENCE_ID)';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.MONOGRAPH_COMMENT ADD CONSTRAINT FK_XREF_MONOCOMMENT FOREIGN KEY (FDB_REFERENCE_ID) REFERENCES CTSTAGING.FDB_MONOGRAPH_XREF (FDB_REFERENCE_ID)';

      EXECUTE IMMEDIATE vc_sql_statement;

      -- Add concept table column changes section ---------------------------------------------------

      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.PECS_MONOGRAPH_VA ADD FDB_REFERENCE_ID Integer ';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.PECS_DOSE_RANGE_VA ADD FDB_REFERENCE_ID Integer';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.PECS_DUPL_THERAPY_VA ADD FDB_REFERENCE_ID Integer';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.PECS_DDI_VA ADD FDB_REFERENCE_ID Integer';

      EXECUTE IMMEDIATE vc_sql_statement;

      -- Create concept table relationships section -------------------------------------------------

      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.PECS_DDI_VA ADD CONSTRAINT FK_XREF_DDIM FOREIGN KEY (FDB_REFERENCE_ID) REFERENCES CTSTAGING.FDB_DDIM_XREF (FDB_REFERENCE_ID)';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.PECS_DUPL_THERAPY_VA ADD CONSTRAINT FK_XREF_DUPLTHERAPY FOREIGN KEY (FDB_REFERENCE_ID) REFERENCES CTSTAGING.FDB_DUPTHERAPY_XREF (FDB_REFERENCE_ID)';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.PECS_DOSE_RANGE_VA ADD CONSTRAINT FK_XREF_DOSERANGE FOREIGN KEY (FDB_REFERENCE_ID) REFERENCES CTSTAGING.FDB_DOSE_RANGE_XREF (FDB_REFERENCE_ID)';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.PECS_MONOGRAPH_VA ADD CONSTRAINT FK_XREF_MONOGRAPH FOREIGN KEY (FDB_REFERENCE_ID) REFERENCES CTSTAGING.FDB_MONOGRAPH_XREF (FDB_REFERENCE_ID)';

      EXECUTE IMMEDIATE vc_sql_statement;


      -- Grant permissions section -------------------------------------------------


      vc_sql_statement :=
         'GRANT SELECT ON CTSTAGING.DDIMINTERACTION_COMMENT TO CTSTAGING_READ_ONLY_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT DELETE ON CTSTAGING.DDIMINTERACTION_COMMENT TO CTSTAGING_UPDATE_USER_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT INSERT ON CTSTAGING.DDIMINTERACTION_COMMENT TO CTSTAGING_UPDATE_USER_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT UPDATE ON CTSTAGING.DDIMINTERACTION_COMMENT TO CTSTAGING_UPDATE_USER_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT SELECT ON CTSTAGING.DDIMINTERACTION_COMMENT TO CTSTAGING_UPDATE_USER_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT SELECT ON CTSTAGING.DDIMINTERACTION_COMMENT TO FDB_DIF_READ_ONLY_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT SELECT ON CTSTAGING.DOSERANGE_COMMENT TO CTSTAGING_READ_ONLY_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT DELETE ON CTSTAGING.DOSERANGE_COMMENT TO CTSTAGING_UPDATE_USER_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT INSERT ON CTSTAGING.DOSERANGE_COMMENT TO CTSTAGING_UPDATE_USER_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT UPDATE ON CTSTAGING.DOSERANGE_COMMENT TO CTSTAGING_UPDATE_USER_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT SELECT ON CTSTAGING.DOSERANGE_COMMENT TO CTSTAGING_UPDATE_USER_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT SELECT ON CTSTAGING.DOSERANGE_COMMENT TO FDB_DIF_READ_ONLY_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT SELECT ON CTSTAGING.DUPLTHERAPY_COMMENT TO CTSTAGING_READ_ONLY_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT DELETE ON CTSTAGING.DUPLTHERAPY_COMMENT TO CTSTAGING_UPDATE_USER_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT INSERT ON CTSTAGING.DUPLTHERAPY_COMMENT TO CTSTAGING_UPDATE_USER_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT UPDATE ON CTSTAGING.DUPLTHERAPY_COMMENT TO CTSTAGING_UPDATE_USER_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT SELECT ON CTSTAGING.DUPLTHERAPY_COMMENT TO CTSTAGING_UPDATE_USER_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT SELECT ON CTSTAGING.DUPLTHERAPY_COMMENT TO FDB_DIF_READ_ONLY_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT SELECT ON CTSTAGING.MONOGRAPH_COMMENT TO CTSTAGING_READ_ONLY_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT DELETE ON CTSTAGING.MONOGRAPH_COMMENT TO CTSTAGING_UPDATE_USER_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT INSERT ON CTSTAGING.MONOGRAPH_COMMENT TO CTSTAGING_UPDATE_USER_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT UPDATE ON CTSTAGING.MONOGRAPH_COMMENT TO CTSTAGING_UPDATE_USER_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT SELECT ON CTSTAGING.MONOGRAPH_COMMENT TO CTSTAGING_UPDATE_USER_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT SELECT ON CTSTAGING.MONOGRAPH_COMMENT TO FDB_DIF_READ_ONLY_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT SELECT ON CTSTAGING.FDB_DDIM_XREF TO CTSTAGING_READ_ONLY_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT DELETE ON CTSTAGING.FDB_DDIM_XREF TO CTSTAGING_UPDATE_USER_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT INSERT ON CTSTAGING.FDB_DDIM_XREF TO CTSTAGING_UPDATE_USER_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT UPDATE ON CTSTAGING.FDB_DDIM_XREF TO CTSTAGING_UPDATE_USER_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT SELECT ON CTSTAGING.FDB_DDIM_XREF TO CTSTAGING_UPDATE_USER_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT SELECT ON CTSTAGING.FDB_DDIM_XREF TO FDB_DIF_READ_ONLY_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;


      vc_sql_statement :=
         'GRANT SELECT ON CTSTAGING.FDB_DUPTHERAPY_XREF TO CTSTAGING_READ_ONLY_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT DELETE ON CTSTAGING.FDB_DUPTHERAPY_XREF TO CTSTAGING_UPDATE_USER_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT INSERT ON CTSTAGING.FDB_DUPTHERAPY_XREF TO CTSTAGING_UPDATE_USER_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT UPDATE ON CTSTAGING.FDB_DUPTHERAPY_XREF TO CTSTAGING_UPDATE_USER_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT SELECT ON CTSTAGING.FDB_DUPTHERAPY_XREF TO CTSTAGING_UPDATE_USER_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT SELECT ON CTSTAGING.FDB_DUPTHERAPY_XREF TO FDB_DIF_READ_ONLY_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;


      vc_sql_statement :=
         'GRANT SELECT ON CTSTAGING.FDB_DOSE_RANGE_XREF TO CTSTAGING_READ_ONLY_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT DELETE ON CTSTAGING.FDB_DOSE_RANGE_XREF TO CTSTAGING_UPDATE_USER_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT INSERT ON CTSTAGING.FDB_DOSE_RANGE_XREF TO CTSTAGING_UPDATE_USER_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT UPDATE ON CTSTAGING.FDB_DOSE_RANGE_XREF TO CTSTAGING_UPDATE_USER_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT SELECT ON CTSTAGING.FDB_DOSE_RANGE_XREF TO CTSTAGING_UPDATE_USER_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT SELECT ON CTSTAGING.FDB_DOSE_RANGE_XREF TO FDB_DIF_READ_ONLY_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT SELECT ON CTSTAGING.FDB_MONOGRAPH_XREF TO CTSTAGING_READ_ONLY_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT DELETE ON CTSTAGING.FDB_MONOGRAPH_XREF TO CTSTAGING_UPDATE_USER_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT INSERT ON CTSTAGING.FDB_MONOGRAPH_XREF TO CTSTAGING_UPDATE_USER_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT UPDATE ON CTSTAGING.FDB_MONOGRAPH_XREF TO CTSTAGING_UPDATE_USER_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT SELECT ON CTSTAGING.FDB_MONOGRAPH_XREF TO CTSTAGING_UPDATE_USER_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT SELECT ON CTSTAGING.FDB_MONOGRAPH_XREF TO FDB_DIF_READ_ONLY_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT SELECT ON CTSTAGING.FDB_DOSEROUTE_REF_ID_SEQ TO CTSTAGING_UPDATE_USER_ROLE ';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT SELECT ON CTSTAGING.DOSEROUTE_COMMENT_ID_SEQ TO CTSTAGING_UPDATE_USER_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT SELECT ON  CTSTAGING.FDB_DUPLTHERAPY_REF_ID_SEQ TO CTSTAGING_UPDATE_USER_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT SELECT ON  CTSTAGING.DUPLTHERAPY_COMMENT_ID_SEQ TO CTSTAGING_UPDATE_USER_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT SELECT ON  CTSTAGING.FDB_MONOGRAPH_REF_ID_SEQ TO CTSTAGING_UPDATE_USER_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT SELECT ON CTSTAGING.MONOGRAPH_COMMENT_ID_SEQ TO CTSTAGING_UPDATE_USER_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT SELECT ON CTSTAGING.FDB_DDIM_REF_ID_SEQ TO CTSTAGING_UPDATE_USER_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT SELECT ON CTSTAGING.DDIM_COMMENT_ID_SEQ TO CTSTAGING_UPDATE_USER_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   END:  create_fdb_comments_revised');
      RETURN (0);
   EXCEPTION
      WHEN OTHERS
      THEN
         vn_code := SQLCODE;
         vc_errm := SUBSTR (SQLERRM, 1, 64);
         sys.DBMS_OUTPUT.put_line (
               'create_fdb_comments_revised: Error code '
            || vn_code
            || ': '
            || vc_errm);
         sys.DBMS_OUTPUT.put_line (
            'create_fdb_comments_revised: last value for vc_sql_statement ');
         sys.DBMS_OUTPUT.put_line (
            'create_fdb_comments_revised: ' || vc_sql_statement);
         RETURN (1);
   END create_fdb_comments_revised;

   -----------------------------------------------------------------------------
   -- NAME:         Create_PECS_CONTACT
   -- TYPE:         Function
   -- DESCRIPTSION: Create objects that were in Create_PECS_CONTACT.sql
   -- INPUTS:       None
   -- OUTPUTS:      Number: (0 - good: >0 - error)
   -----------------------------------------------------------------------------
   FUNCTION create_pecs_contact
      RETURN NUMBER
   IS
   BEGIN
      sys.DBMS_OUTPUT.put_line (CHR (10));
      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   START:  Create_PECS_CONTACT'); -- Create sequences section -------------------------------------------------

      vc_sql_statement :=
         'CREATE SEQUENCE ctstaging.pecs_contact_id_seq START WITH 1 MAXVALUE 9999999999999999999999999999 MINVALUE 1 CACHE 20';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
            'CREATE TABLE ctstaging.pecs_contact '
         || '(pecs_contact_id        NUMBER (11) NOT NULL '
         || ',created_date           TIMESTAMP (6) '
         || ',created_by             VARCHAR2 (64 BYTE) '
         || ',pecs_contact_content   BLOB) '
         || 'LOB (pecs_contact_content) STORE AS '
         || '(TABLESPACE ctstaging_data '
         || 'ENABLE STORAGE IN ROW '
         || 'CHUNK 8192 '
         || 'RETENTION '
         || 'NOCACHE LOGGING '
         || 'STORAGE (INITIAL 64 K '
         || 'NEXT 1 M '
         || 'MINEXTENTS 1 '
         || 'MAXEXTENTS UNLIMITED '
         || 'PCTINCREASE 0 '
         || 'BUFFER_POOL DEFAULT '
         || 'FLASH_CACHE DEFAULT '
         || 'CELL_FLASH_CACHE DEFAULT)) '
         || 'TABLESPACE ctstaging_data '
         || 'RESULT_CACHE (MODE DEFAULT)';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
            'CREATE UNIQUE INDEX ctstaging.xpk_pecs_contact '
         || 'ON ctstaging.pecs_contact (pecs_contact_id) '
         || 'LOGGING '
         || 'TABLESPACE ctstaging_index';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'CREATE OR REPLACE PUBLIC SYNONYM pecs_contact FOR ctstaging.pecs_contact';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'CREATE OR REPLACE PUBLIC SYNONYM pecs_contact_id_seq FOR ctstaging.pecs_contact_id_seq';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
            'ALTER TABLE ctstaging.pecs_contact ADD ( '
         || 'CONSTRAINT pk_pecs_contact '
         || 'PRIMARY KEY '
         || '(pecs_contact_id) '
         || 'USING INDEX ctstaging.xpk_pecs_contact '
         || 'ENABLE VALIDATE)';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT SELECT ON ctstaging.pecs_contact TO ctstaging_read_only_role';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
            'GRANT DELETE, INSERT, UPDATE '
         || 'ON ctstaging.pecs_contact '
         || 'TO ctstaging_update_user_role';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT SELECT ON ctstaging.pecs_contact_id_seq TO ctstaging_update_user_role';

      EXECUTE IMMEDIATE vc_sql_statement;

      sys.DBMS_OUTPUT.put_line ('INFORMATION -   END:  Create_PECS_CONTACT');
      RETURN (0);
   EXCEPTION
      WHEN OTHERS
      THEN
         vn_code := SQLCODE;
         vc_errm := SUBSTR (SQLERRM, 1, 64);
         sys.DBMS_OUTPUT.put_line (
            'Create_PECS_CONTACT: Error code ' || vn_code || ': ' || vc_errm);
         sys.DBMS_OUTPUT.put_line (
            'Create_PECS_CONTACT: last value for vc_sql_statement ');
         sys.DBMS_OUTPUT.put_line (
            'Create_PECS_CONTACT: ' || vc_sql_statement);
         RETURN (1);
   END create_pecs_contact;

   -----------------------------------------------------------------------------
   -- NAME:         PECS_DDI_EXPORT_VA
   -- TYPE:         Function
   -- DESCRIPTSION: Create objects that were in PECS_DDI_EXPORT_VA.sql
   -- INPUTS:       None
   -- OUTPUTS:      Number: (0 - good: >0 - error)
   -----------------------------------------------------------------------------
   FUNCTION pecs_ddi_export_va
      RETURN NUMBER
   IS
   BEGIN
      sys.DBMS_OUTPUT.put_line (CHR (10));
      sys.DBMS_OUTPUT.put_line ('INFORMATION -   START:  PECS_DDI_EXPORT_VA'); -- Create sequences section -------------------------------------------------

      vc_sql_statement :=
            'CREATE TABLE CTSTAGING.PECS_DDI_EXPORT_VA '
         || '(PECS_DDI_VA_ID  NUMBER(11)             NOT NULL, '
         || 'EXPORTABLE_VERSION_ID  NUMBER(11), '
         || 'EXPORTED_VERSION_ID    NUMBER(11), '
         || 'ACTIVE_VERSION_ID      NUMBER(11)             NOT NULL) '
         || 'TABLESPACE CTSTAGING_DATA';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
            'CREATE UNIQUE INDEX CTSTAGING.XPK_PECS_DDI_EXPORT_VA ON CTSTAGING.PECS_DDI_EXPORT_VA '
         || '(PECS_DDI_VA_ID) '
         || 'LOGGING '
         || 'TABLESPACE CTSTAGING_INDEX';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'CREATE OR REPLACE PUBLIC SYNONYM PECS_DDI_EXPORT_VA FOR CTSTAGING.PECS_DDI_EXPORT_VA';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
            'ALTER TABLE CTSTAGING.PECS_DDI_EXPORT_VA ADD ( '
         || 'CONSTRAINT PK_PECS_DDI_EXPORT_VA '
         || 'PRIMARY KEY '
         || '(PECS_DDI_VA_ID) '
         || 'USING INDEX CTSTAGING.XPK_PECS_DDI_EXPORT_VA '
         || 'ENABLE VALIDATE)';

      EXECUTE IMMEDIATE vc_sql_statement;

      -- Add unique keys
      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.PECS_DDI_EXPORT_VA ADD CONSTRAINT UKDDIEXPORTABLEVERSIONID UNIQUE (EXPORTABLE_VERSION_ID)';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.PECS_DDI_EXPORT_VA ADD CONSTRAINT UKDDIEXPORTEDVERSIONID UNIQUE (EXPORTED_VERSION_ID)';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.PECS_DDI_EXPORT_VA ADD CONSTRAINT UKDDIACTIVEVERSIONID UNIQUE (ACTIVE_VERSION_ID)';

      -- Add foreign keys

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.PECS_DDI_EXPORT_VA ADD CONSTRAINT FK_DDI_EXPORT_VA_ID FOREIGN KEY (PECS_DDI_VA_ID) REFERENCES CTSTAGING.PECS_DDI_VA (PECS_DDI_VA_ID)';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.PECS_DDI_EXPORT_VA ADD CONSTRAINT FK_DDIV_EXPORTABLE_VERSION_ID FOREIGN KEY (EXPORTABLE_VERSION_ID) REFERENCES CTSTAGING.PECS_DDI_VERSION_VA (VERSION_ID)';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.PECS_DDI_EXPORT_VA ADD CONSTRAINT FK_DDIV_EXPORTED_VERSION_ID FOREIGN KEY (EXPORTED_VERSION_ID) REFERENCES CTSTAGING.PECS_DDI_VERSION_VA (VERSION_ID)';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.PECS_DDI_EXPORT_VA ADD CONSTRAINT FK_DDIV_ACTIVE_VERSION_ID FOREIGN KEY (ACTIVE_VERSION_ID) REFERENCES CTSTAGING.PECS_DDI_VERSION_VA (VERSION_ID)';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT SELECT ON CTSTAGING.PECS_DDI_EXPORT_VA TO CTSTAGING_READ_ONLY_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT DELETE, INSERT, UPDATE ON CTSTAGING.PECS_DDI_EXPORT_VA TO CTSTAGING_UPDATE_USER_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      sys.DBMS_OUTPUT.put_line ('INFORMATION -   END:  PECS_DDI_EXPORT_VA');
      RETURN (0);
   EXCEPTION
      WHEN OTHERS
      THEN
         vn_code := SQLCODE;
         vc_errm := SUBSTR (SQLERRM, 1, 64);
         sys.DBMS_OUTPUT.put_line (
            'PECS_DDI_EXPORT_VA: Error code ' || vn_code || ': ' || vc_errm);
         sys.DBMS_OUTPUT.put_line (
            'PECS_DDI_EXPORT_VA: last value for vc_sql_statement ');
         sys.DBMS_OUTPUT.put_line (
            'PECS_DDI_EXPORT_VA: ' || vc_sql_statement);
         RETURN (1);
   END pecs_ddi_export_va;

   -----------------------------------------------------------------------------
   -- NAME:         PECS_DP_EXPORT_VA
   -- TYPE:         Function
   -- DESCRIPTSION: Create objects that were in PECS_DP_EXPORT_VA.sql
   -- INPUTS:       None
   -- OUTPUTS:      Number: (0 - good: >0 - error)
   -----------------------------------------------------------------------------
   FUNCTION pecs_dp_export_va
      RETURN NUMBER
   IS
   BEGIN
      sys.DBMS_OUTPUT.put_line (CHR (10));
      sys.DBMS_OUTPUT.put_line ('INFORMATION -   START:  PECS_DP_EXPORT_VA'); -- Create sequences section -------------------------------------------------

      vc_sql_statement :=
            'CREATE TABLE CTSTAGING.PECS_DRUG_PAIR_EXPORT_VA '
         || '(  VA_DRUG_PAIR_ID  NUMBER(11)             NOT NULL, '
         || 'EXPORTABLE_VERSION_ID  NUMBER(11), '
         || 'EXPORTED_VERSION_ID    NUMBER(11), '
         || 'ACTIVE_VERSION_ID      NUMBER(11)             NOT NULL) '
         || 'TABLESPACE CTSTAGING_DATA';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
            'CREATE UNIQUE INDEX CTSTAGING.XPK_PECS_DP_EXPORT_VA ON CTSTAGING.PECS_DRUG_PAIR_EXPORT_VA '
         || '(VA_DRUG_PAIR_ID) '
         || 'LOGGING '
         || 'TABLESPACE CTSTAGING_INDEX';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'CREATE OR REPLACE PUBLIC SYNONYM PECS_DRUG_PAIR_EXPORT_VA FOR CTSTAGING.PECS_DRUG_PAIR_EXPORT_VA';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
            'ALTER TABLE CTSTAGING.PECS_DRUG_PAIR_EXPORT_VA ADD ( '
         || 'CONSTRAINT PK_PECS_DP_EXPORT_VA '
         || 'PRIMARY KEY '
         || '(VA_DRUG_PAIR_ID) '
         || 'USING INDEX CTSTAGING.XPK_PECS_DP_EXPORT_VA '
         || 'ENABLE VALIDATE)';

      EXECUTE IMMEDIATE vc_sql_statement;

      -- Add unique keys
      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.PECS_DRUG_PAIR_EXPORT_VA ADD CONSTRAINT UKDPEXPORTABLEVERSIONID UNIQUE (EXPORTABLE_VERSION_ID)';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.PECS_DRUG_PAIR_EXPORT_VA ADD CONSTRAINT UKDPEXPORTEDVERSIONID UNIQUE (EXPORTED_VERSION_ID)';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.PECS_DRUG_PAIR_EXPORT_VA ADD CONSTRAINT UKDPACTIVEVERSIONID UNIQUE (ACTIVE_VERSION_ID)';

      EXECUTE IMMEDIATE vc_sql_statement;

      -- Add foreign keys
      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.PECS_DRUG_PAIR_EXPORT_VA ADD CONSTRAINT FK_DP_EXPORT_VA_ID FOREIGN KEY (VA_DRUG_PAIR_ID) REFERENCES CTSTAGING.PECS_DRUG_PAIR_VA (VA_DRUG_PAIR_ID)';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.PECS_DRUG_PAIR_EXPORT_VA ADD CONSTRAINT FK_DPV_EXPORTABLE_VERSION_ID FOREIGN KEY (EXPORTABLE_VERSION_ID) REFERENCES CTSTAGING.PECS_DRUG_PAIR_VERSION_VA (VERSION_ID)';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.PECS_DRUG_PAIR_EXPORT_VA ADD CONSTRAINT FK_DPV_EXPORTED_VERSION_ID FOREIGN KEY (EXPORTED_VERSION_ID) REFERENCES CTSTAGING.PECS_DRUG_PAIR_VERSION_VA (VERSION_ID)';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.PECS_DRUG_PAIR_EXPORT_VA ADD CONSTRAINT FK_DPV_ACTIVE_VERSION_ID FOREIGN KEY (ACTIVE_VERSION_ID) REFERENCES CTSTAGING.PECS_DRUG_PAIR_VERSION_VA (VERSION_ID)';

      EXECUTE IMMEDIATE vc_sql_statement;


      vc_sql_statement :=
         'GRANT SELECT ON CTSTAGING.PECS_DRUG_PAIR_EXPORT_VA TO CTSTAGING_READ_ONLY_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT DELETE, INSERT, UPDATE, SELECT ON CTSTAGING.PECS_DRUG_PAIR_EXPORT_VA TO CTSTAGING_UPDATE_USER_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;


      sys.DBMS_OUTPUT.put_line ('INFORMATION -   END:  PECS_DP_EXPORT_VA');
      RETURN (0);
   EXCEPTION
      WHEN OTHERS
      THEN
         vn_code := SQLCODE;
         vc_errm := SUBSTR (SQLERRM, 1, 64);
         sys.DBMS_OUTPUT.put_line (
            'PECS_DP_EXPORT_VA: Error code ' || vn_code || ': ' || vc_errm);
         sys.DBMS_OUTPUT.put_line (
            'PECS_DP_EXPORT_VA: last value for vc_sql_statement ');
         sys.DBMS_OUTPUT.put_line ('PECS_DP_EXPORT_VA: ' || vc_sql_statement);
         RETURN (1);
   END pecs_dp_export_va;

   -----------------------------------------------------------------------------
   -- NAME:         PECS_DR_EXPORT_VA
   -- TYPE:         Function
   -- DESCRIPTSION: Create objects that were in PECS_DR_EXPORT_VA.sql
   -- INPUTS:       None
   -- OUTPUTS:      Number: (0 - good: >0 - error)
   -----------------------------------------------------------------------------
   FUNCTION pecs_dr_export_va
      RETURN NUMBER
   IS
   BEGIN
      sys.DBMS_OUTPUT.put_line (CHR (10));
      sys.DBMS_OUTPUT.put_line ('INFORMATION -   START:  PECS_DR_EXPORT_VA'); -- Create sequences section -------------------------------------------------

      vc_sql_statement :=
            'CREATE TABLE CTSTAGING.PECS_DOSE_RANGE_EXPORT_VA '
         || '(VA_DOSE_RANGE_ID  NUMBER(11)             NOT NULL, '
         || 'EXPORTABLE_VERSION_ID  NUMBER(11), '
         || 'EXPORTED_VERSION_ID    NUMBER(11), '
         || 'ACTIVE_VERSION_ID      NUMBER(11)             NOT NULL) '
         || 'TABLESPACE CTSTAGING_DATA';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
            'CREATE UNIQUE INDEX CTSTAGING.XPK_PECS_DOSE_RANGE_EXPORT_VA ON CTSTAGING.PECS_DOSE_RANGE_EXPORT_VA '
         || '(VA_DOSE_RANGE_ID) '
         || 'LOGGING '
         || 'TABLESPACE CTSTAGING_INDEX';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'CREATE OR REPLACE PUBLIC SYNONYM PECS_DOSE_RANGE_EXPORT_VA FOR CTSTAGING.PECS_DOSE_RANGE_EXPORT_VA';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
            'ALTER TABLE CTSTAGING.PECS_DOSE_RANGE_EXPORT_VA ADD ( '
         || 'CONSTRAINT PK_PECS_DOSE_RANGE_EXPORT_VA '
         || 'PRIMARY KEY '
         || '(VA_DOSE_RANGE_ID) '
         || 'USING INDEX CTSTAGING.XPK_PECS_DOSE_RANGE_EXPORT_VA '
         || 'ENABLE VALIDATE)';

      EXECUTE IMMEDIATE vc_sql_statement;

      -- Add unique keys
      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.PECS_DOSE_RANGE_EXPORT_VA ADD CONSTRAINT UKDREXPORTABLEVERSIONID UNIQUE (EXPORTABLE_VERSION_ID)';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.PECS_DOSE_RANGE_EXPORT_VA ADD CONSTRAINT UKDREXPORTEDVERSIONID UNIQUE (EXPORTED_VERSION_ID)';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.PECS_DOSE_RANGE_EXPORT_VA ADD CONSTRAINT UKDRACTIVEVERSIONID UNIQUE (ACTIVE_VERSION_ID)';

      -- Add foreign keys

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.PECS_DOSE_RANGE_EXPORT_VA ADD CONSTRAINT FK_DR_EXPORT_VA_ID FOREIGN KEY (VA_DOSE_RANGE_ID) REFERENCES CTSTAGING.PECS_DOSE_RANGE_VA (VA_DOSE_RANGE_ID)';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.PECS_DOSE_RANGE_EXPORT_VA ADD CONSTRAINT FK_DRV_EXPORTABLE_VERSION_ID FOREIGN KEY (EXPORTABLE_VERSION_ID) REFERENCES CTSTAGING.PECS_DOSE_RANGE_VERSION_VA (VERSION_ID)';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.PECS_DOSE_RANGE_EXPORT_VA ADD CONSTRAINT FK_DRV_EXPORTED_VERSION_ID FOREIGN KEY (EXPORTED_VERSION_ID) REFERENCES CTSTAGING.PECS_DOSE_RANGE_VERSION_VA (VERSION_ID)';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.PECS_DOSE_RANGE_EXPORT_VA ADD CONSTRAINT FK_DRV_ACTIVE_VERSION_ID FOREIGN KEY (ACTIVE_VERSION_ID) REFERENCES CTSTAGING.PECS_DOSE_RANGE_VERSION_VA (VERSION_ID)';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT SELECT ON CTSTAGING.PECS_DOSE_RANGE_EXPORT_VA TO CTSTAGING_READ_ONLY_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT DELETE, INSERT, UPDATE ON CTSTAGING.PECS_DOSE_RANGE_EXPORT_VA TO CTSTAGING_UPDATE_USER_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      sys.DBMS_OUTPUT.put_line ('INFORMATION -   END:  PECS_DR_EXPORT_VA');
      RETURN (0);
   EXCEPTION
      WHEN OTHERS
      THEN
         vn_code := SQLCODE;
         vc_errm := SUBSTR (SQLERRM, 1, 64);
         sys.DBMS_OUTPUT.put_line (
            'PECS_DR_EXPORT_VA: Error code ' || vn_code || ': ' || vc_errm);
         sys.DBMS_OUTPUT.put_line (
            'PECS_DR_EXPORT_VA: last value for vc_sql_statement ');
         sys.DBMS_OUTPUT.put_line ('PECS_DR_EXPORT_VA: ' || vc_sql_statement);
         RETURN (1);
   END pecs_dr_export_va;

   -----------------------------------------------------------------------------
   -- NAME:         PECS_DT_EXPORT_VA
   -- TYPE:         Function
   -- DESCRIPTSION: Create objects that were in PECS_DT_EXPORT_VA.sql
   -- INPUTS:       None
   -- OUTPUTS:      Number: (0 - good: >0 - error)
   -----------------------------------------------------------------------------
   FUNCTION pecs_dt_export_va
      RETURN NUMBER
   IS
   BEGIN
      sys.DBMS_OUTPUT.put_line (CHR (10));
      sys.DBMS_OUTPUT.put_line ('INFORMATION -   START:  PECS_DT_EXPORT_VA'); -- Create sequences section -------------------------------------------------

      vc_sql_statement :=
            'CREATE TABLE CTSTAGING.PECS_DUPL_THERAPY_EXPORT_VA '
         || '(VA_DUPL_THERAPY_ID  NUMBER(11)             NOT NULL, '
         || 'EXPORTABLE_VERSION_ID  NUMBER(11), '
         || 'EXPORTED_VERSION_ID    NUMBER(11), '
         || 'ACTIVE_VERSION_ID      NUMBER(11)             NOT NULL) '
         || 'TABLESPACE CTSTAGING_DATA';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
            'CREATE UNIQUE INDEX CTSTAGING.XPK_PECS_DT_EXPORT_VA ON CTSTAGING.PECS_DUPL_THERAPY_EXPORT_VA '
         || '(VA_DUPL_THERAPY_ID) '
         || 'LOGGING '
         || 'TABLESPACE CTSTAGING_INDEX';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'CREATE OR REPLACE PUBLIC SYNONYM PECS_DUPL_THERAPY_EXPORT_VA FOR CTSTAGING.PECS_DUPL_THERAPY_EXPORT_VA';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
            'ALTER TABLE CTSTAGING.PECS_DUPL_THERAPY_EXPORT_VA ADD ( '
         || 'CONSTRAINT PK_PECS_DT_EXPORT_VA '
         || 'PRIMARY KEY '
         || '(VA_DUPL_THERAPY_ID) '
         || 'USING INDEX CTSTAGING.XPK_PECS_DT_EXPORT_VA '
         || 'ENABLE VALIDATE)';

      EXECUTE IMMEDIATE vc_sql_statement;

      -- Add unique keys
      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.PECS_DUPL_THERAPY_EXPORT_VA ADD CONSTRAINT UKDTEXPORTABLEVERSIONID UNIQUE (EXPORTABLE_VERSION_ID)';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.PECS_DUPL_THERAPY_EXPORT_VA ADD CONSTRAINT UKDTEXPORTEDVERSIONID UNIQUE (EXPORTED_VERSION_ID)';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.PECS_DUPL_THERAPY_EXPORT_VA ADD CONSTRAINT UKDTACTIVEVERSIONID UNIQUE (ACTIVE_VERSION_ID)';

      EXECUTE IMMEDIATE vc_sql_statement;

      -- Add foreign keys
      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.PECS_DUPL_THERAPY_EXPORT_VA ADD CONSTRAINT FK_DT_EXPORT_VA_ID FOREIGN KEY (VA_DUPL_THERAPY_ID) REFERENCES CTSTAGING.PECS_DUPL_THERAPY_VA (VA_DUPL_THERAPY_ID)';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.PECS_DUPL_THERAPY_EXPORT_VA ADD CONSTRAINT FK_DTV_EXPORTABLE_VERSION_ID FOREIGN KEY (EXPORTABLE_VERSION_ID) REFERENCES CTSTAGING.PECS_DUPL_THERAPY_VERSION_VA (VERSION_ID)';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.PECS_DUPL_THERAPY_EXPORT_VA ADD CONSTRAINT FK_DTV_EXPORTED_VERSION_ID FOREIGN KEY (EXPORTED_VERSION_ID) REFERENCES CTSTAGING.PECS_DUPL_THERAPY_VERSION_VA (VERSION_ID)';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.PECS_DUPL_THERAPY_EXPORT_VA ADD CONSTRAINT FK_DTV_ACTIVE_VERSION_ID FOREIGN KEY (ACTIVE_VERSION_ID) REFERENCES CTSTAGING.PECS_DUPL_THERAPY_VERSION_VA (VERSION_ID)';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT SELECT ON CTSTAGING.PECS_DUPL_THERAPY_EXPORT_VA TO CTSTAGING_READ_ONLY_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT DELETE, INSERT, UPDATE ON CTSTAGING.PECS_DUPL_THERAPY_EXPORT_VA TO CTSTAGING_UPDATE_USER_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      sys.DBMS_OUTPUT.put_line ('INFORMATION -   END:  PECS_DT_EXPORT_VA');
      RETURN (0);
   EXCEPTION
      WHEN OTHERS
      THEN
         vn_code := SQLCODE;
         vc_errm := SUBSTR (SQLERRM, 1, 64);
         sys.DBMS_OUTPUT.put_line (
            'PECS_DT_EXPORT_VA: Error code ' || vn_code || ': ' || vc_errm);
         sys.DBMS_OUTPUT.put_line (
            'PECS_DT_EXPORT_VA: last value for vc_sql_statement ');
         sys.DBMS_OUTPUT.put_line ('PECS_DT_EXPORT_VA: ' || vc_sql_statement);
         RETURN (1);
   END pecs_dt_export_va;

   -----------------------------------------------------------------------------
   -- NAME:         PECS_MONO_EXPORT_VA
   -- TYPE:         Function
   -- DESCRIPTSION: Create objects that were in PECS_MONO_EXPORT_VA.sql
   -- INPUTS:       None
   -- OUTPUTS:      Number: (0 - good: >0 - error)
   -----------------------------------------------------------------------------
   FUNCTION pecs_mono_export_va
      RETURN NUMBER
   IS
   BEGIN
      sys.DBMS_OUTPUT.put_line (CHR (10));
      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   START:  PECS_MONO_EXPORT_VA'); -- Create sequences section -------------------------------------------------

      vc_sql_statement :=
            'CREATE TABLE CTSTAGING.PECS_MONOGRAPH_EXPORT_VA '
         || '(VA_MONOGRAPH_ID  NUMBER(11)             NOT NULL, '
         || 'EXPORTABLE_VERSION_ID  NUMBER(11), '
         || 'EXPORTED_VERSION_ID    NUMBER(11), '
         || 'ACTIVE_VERSION_ID      NUMBER(11)             NOT NULL) '
         || 'TABLESPACE CTSTAGING_DATA';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
            'CREATE UNIQUE INDEX CTSTAGING.XPK_PECS_MONO_EXPORT_VA ON CTSTAGING.PECS_MONOGRAPH_EXPORT_VA '
         || '(VA_MONOGRAPH_ID) '
         || 'LOGGING '
         || 'TABLESPACE CTSTAGING_INDEX';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'CREATE OR REPLACE PUBLIC SYNONYM PECS_MONOGRAPH_EXPORT_VA FOR CTSTAGING.PECS_MONOGRAPH_EXPORT_VA';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
            'ALTER TABLE CTSTAGING.PECS_MONOGRAPH_EXPORT_VA ADD ( '
         || 'CONSTRAINT PK_PECS_MONO_EXPORT_VA '
         || 'PRIMARY KEY '
         || '(VA_MONOGRAPH_ID) '
         || 'USING INDEX CTSTAGING.XPK_PECS_MONO_EXPORT_VA '
         || 'ENABLE VALIDATE)';

      EXECUTE IMMEDIATE vc_sql_statement;

      -- Add unique keys
      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.PECS_MONOGRAPH_EXPORT_VA ADD CONSTRAINT UKMNEXPORTABLEVERSIONID UNIQUE (EXPORTABLE_VERSION_ID)';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.PECS_MONOGRAPH_EXPORT_VA ADD CONSTRAINT UKMNEXPORTEDVERSIONID UNIQUE (EXPORTED_VERSION_ID)';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.PECS_MONOGRAPH_EXPORT_VA ADD CONSTRAINT UKMNACTIVEVERSIONID UNIQUE (ACTIVE_VERSION_ID)';

      EXECUTE IMMEDIATE vc_sql_statement;

      -- Add foreign keys
      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.PECS_MONOGRAPH_EXPORT_VA ADD CONSTRAINT FK_MONO_EXPORT_VA_ID FOREIGN KEY (VA_MONOGRAPH_ID) REFERENCES CTSTAGING.PECS_MONOGRAPH_VA (VA_MONOGRAPH_ID)';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.PECS_MONOGRAPH_EXPORT_VA ADD CONSTRAINT FK_MONOV_EXPORTABLE_VERSION_ID FOREIGN KEY (EXPORTABLE_VERSION_ID) REFERENCES CTSTAGING.PECS_MONOGRAPH_VERSION_VA (VERSION_ID)';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.PECS_MONOGRAPH_EXPORT_VA ADD CONSTRAINT FK_MONOV_EXPORTED_VERSION_ID FOREIGN KEY (EXPORTED_VERSION_ID) REFERENCES CTSTAGING.PECS_MONOGRAPH_VERSION_VA (VERSION_ID)';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.PECS_MONOGRAPH_EXPORT_VA ADD CONSTRAINT FK_MONOV_ACTIVE_VERSION_ID FOREIGN KEY (ACTIVE_VERSION_ID) REFERENCES CTSTAGING.PECS_MONOGRAPH_VERSION_VA (VERSION_ID)';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT SELECT ON CTSTAGING.PECS_MONOGRAPH_EXPORT_VA TO CTSTAGING_READ_ONLY_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT DELETE, INSERT, UPDATE, SELECT ON CTSTAGING.PECS_MONOGRAPH_EXPORT_VA TO CTSTAGING_UPDATE_USER_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      sys.DBMS_OUTPUT.put_line ('INFORMATION -   END:  PECS_MONO_EXPORT_VA');
      RETURN (0);
   EXCEPTION
      WHEN OTHERS
      THEN
         vn_code := SQLCODE;
         vc_errm := SUBSTR (SQLERRM, 1, 64);
         sys.DBMS_OUTPUT.put_line (
            'PECS_MONO_EXPORT_VA: Error code ' || vn_code || ': ' || vc_errm);
         sys.DBMS_OUTPUT.put_line (
            'PECS_MONO_EXPORT_VA: last value for vc_sql_statement ');
         sys.DBMS_OUTPUT.put_line (
            'PECS_MONO_EXPORT_VA: ' || vc_sql_statement);
         RETURN (1);
   END pecs_mono_export_va;

   -----------------------------------------------------------------------------
   -- NAME:         DUPLICATE_ALLOWANCE
   -- TYPE:         Function
   -- DESCRIPTSION: Create objects that were in DUPLICATE_ALLOWANCE.sql
   -- INPUTS:       None
   -- OUTPUTS:      Number: (0 - good: >0 - error)
   -----------------------------------------------------------------------------
   FUNCTION duplicate_allowance
      RETURN NUMBER
   IS
   BEGIN
      sys.DBMS_OUTPUT.put_line (CHR (10));
      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   START:  DUPLICATE_ALLOWANCE'); -- Create sequences section -------------------------------------------------

      vc_sql_statement :=
         'DROP TABLE CTSTAGING.DUPLICATE_ALLOWANCE CASCADE CONSTRAINTS';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
            'CREATE TABLE CTSTAGING.DUPLICATE_ALLOWANCE '
         || '(DUPLICATE_ALLOWANCE_ID  NUMBER(11)            NOT NULL, '
         || 'CODE                    INTEGER               NOT NULL, '
         || 'NAME                    VARCHAR2(80 BYTE), '
         || 'DESCRIPTION             VARCHAR2(250 BYTE), '
         || 'SORT_ORDER              NUMBER(6), '
         || 'ISACTIVE                VARCHAR2(1 BYTE)      NOT NULL, '
         || 'EFFECTIVEDATE           DATE, '
         || 'EXPIRATIONDATE          DATE, '
         || 'RECORD_CREATED_BY       VARCHAR2(30 BYTE)     NOT NULL, '
         || 'RECORD_CREATED_DATE     DATE                  NOT NULL, '
         || 'RECORD_MODIFIED_BY      VARCHAR2(30 BYTE)     NOT NULL, '
         || 'RECORD_MODIFIED_DATE    DATE                  NOT NULL) '
         || 'TABLESPACE CTSTAGING_DATA';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
            'CREATE UNIQUE INDEX CTSTAGING.XPKDUPLICATE_ALLOWANCE ON CTSTAGING.DUPLICATE_ALLOWANCE '
         || '(DUPLICATE_ALLOWANCE_ID) '
         || 'LOGGING '
         || 'TABLESPACE CTSTAGING_INDEX';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
            'CREATE OR REPLACE TRIGGER CTSTAGING.DUPLICATEALLOWANCE_TRIGGER  '
         || 'BEFORE INSERT ON CTSTAGING.DUPLICATE_ALLOWANCE '
         || 'REFERENCING NEW AS NEW '
         || 'FOR EACH ROW '
         || 'BEGIN '
         || 'IF (NVL(:NEW.DUPLICATE_ALLOWANCE_ID, 0) < 1) THEN '
         || 'SELECT REFTABLE_SEQ.nextval INTO :NEW.DUPLICATE_ALLOWANCE_ID FROM dual; '
         || 'END IF; '
         || 'END;';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'CREATE OR REPLACE PUBLIC SYNONYM DUPLICATE_ALLOWANCE FOR CTSTAGING.DUPLICATE_ALLOWANCE';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
            'ALTER TABLE CTSTAGING.DUPLICATE_ALLOWANCE ADD ( '
         || 'CONSTRAINT PK_DUPLICATE_ALLOWANCE '
         || 'PRIMARY KEY '
         || '(DUPLICATE_ALLOWANCE_ID) '
         || 'USING INDEX CTSTAGING.XPKDUPLICATE_ALLOWANCE '
         || 'ENABLE VALIDATE)';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT SELECT ON CTSTAGING.DUPLICATE_ALLOWANCE TO CTSTAGING_READ_ONLY_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'GRANT DELETE, INSERT, UPDATE ON CTSTAGING.DUPLICATE_ALLOWANCE TO CTSTAGING_UPDATE_USER_ROLE';

      EXECUTE IMMEDIATE vc_sql_statement;

      sys.DBMS_OUTPUT.put_line ('INFORMATION -   END:  DUPLICATE_ALLOWANCE');
      RETURN (0);
   EXCEPTION
      WHEN OTHERS
      THEN
         vn_code := SQLCODE;
         vc_errm := SUBSTR (SQLERRM, 1, 64);
         sys.DBMS_OUTPUT.put_line (
            'DUPLICATE_ALLOWANCE: Error code ' || vn_code || ': ' || vc_errm);
         sys.DBMS_OUTPUT.put_line (
            'DUPLICATE_ALLOWANCE: last value for vc_sql_statement ');
         sys.DBMS_OUTPUT.put_line (
            'DUPLICATE_ALLOWANCE: ' || vc_sql_statement);
         RETURN (1);
   END duplicate_allowance;

   -----------------------------------------------------------------------------
   -- NAME:         Create_DupAllow_Constraint
   -- TYPE:         Function
   -- DESCRIPTSION: Create objects that were in Create_DupAllow_Constraint.sql
   -- INPUTS:       None
   -- OUTPUTS:      Number: (0 - good: >0 - error)
   -----------------------------------------------------------------------------
   FUNCTION create_dupallow_constraint
      RETURN NUMBER
   IS
   BEGIN
      sys.DBMS_OUTPUT.put_line (CHR (10));
      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   START:  Create_DupAllow_Constraint'); -- Create sequences section -------------------------------------------------

      vc_sql_statement :=
            'ALTER TABLE CTSTAGING.PECS_DUPL_THERAPY_VERSION_VA ADD ( '
         || 'CONSTRAINT FK_DUPALLOW_DUPLTHERAPY  '
         || 'FOREIGN KEY (CUSTOM_DUP_ALLOWANCE_ID)  '
         || 'REFERENCES CTSTAGING.DUPLICATE_ALLOWANCE (DUPLICATE_ALLOWANCE_ID) '
         || 'ENABLE VALIDATE)';

      EXECUTE IMMEDIATE vc_sql_statement;


      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   END:  Create_DupAllow_Constraint');
      RETURN (0);
   EXCEPTION
      WHEN OTHERS
      THEN
         vn_code := SQLCODE;
         vc_errm := SUBSTR (SQLERRM, 1, 64);
         sys.DBMS_OUTPUT.put_line (
               'Create_DupAllow_Constraint: Error code '
            || vn_code
            || ': '
            || vc_errm);
         sys.DBMS_OUTPUT.put_line (
            'Create_DupAllow_Constraint: last value for vc_sql_statement ');
         sys.DBMS_OUTPUT.put_line (
            'Create_DupAllow_Constraint: ' || vc_sql_statement);
         RETURN (1);
   END create_dupallow_constraint;

   -----------------------------------------------------------------------------
   -- NAME:         DDI_CrossReference_Migration
   -- TYPE:         Function
   -- DESCRIPTSION: Create objects that were in DDI_CrossReference_Migration.sql
   -- INPUTS:       None
   -- OUTPUTS:      Number: (0 - good: >0 - error)
   -----------------------------------------------------------------------------
   FUNCTION ddi_crossreference_migration
      RETURN NUMBER
   IS
   BEGIN
      sys.DBMS_OUTPUT.put_line (CHR (10));
      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   START:  DDI_CrossReference_Migration');

      vc_sql_statement :=
            'INSERT INTO ctstaging.fdb_ddim_xref (fdb_reference_id, interaction_id) '
         || 'SELECT ctstaging.fdb_ddim_ref_id_seq.NEXTVAL, custom.fdb_id '
         || 'FROM (  SELECT DISTINCT ddi.fdb_id '
         || 'FROM ctstaging.pecs_ddi_va ddi '
         || 'INNER JOIN fdb_dif.fdb_ddiminteraction fdb '
         || 'ON fdb.interactionid = ddi.fdb_id '
         || 'WHERE NOT EXISTS '
         || '(SELECT * '
         || 'FROM ctstaging.fdb_ddim_xref xref '
         || 'WHERE xref.interaction_id = fdb.interactionid) '
         || 'ORDER BY ddi.fdb_id) custom';


      EXECUTE IMMEDIATE vc_sql_statement;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) inserted into ctstaging.fdb_ddim_xref : '
         || TO_CHAR (SQL%ROWCOUNT));

      COMMIT;

      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   END:  DDI_CrossReference_Migration');
      RETURN (0);
   EXCEPTION
      WHEN OTHERS
      THEN
         vn_code := SQLCODE;
         vc_errm := SUBSTR (SQLERRM, 1, 64);
         sys.DBMS_OUTPUT.put_line (
               'DDI_CrossReference_Migration: Error code '
            || vn_code
            || ': '
            || vc_errm);
         sys.DBMS_OUTPUT.put_line (
            'DDI_CrossReference_Migration: last value for vc_sql_statement ');
         sys.DBMS_OUTPUT.put_line (
            'DDI_CrossReference_Migration: ' || vc_sql_statement);
         RETURN (1);
   END ddi_crossreference_migration;

   -----------------------------------------------------------------------------
   -- NAME:         Create_DDI_XREF_Link
   -- TYPE:         Function
   -- DESCRIPTSION: Create objects that were in Create_DDI_XREF_Link.sql
   -- INPUTS:       None
   -- OUTPUTS:      Number: (0 - good: >0 - error)
   -----------------------------------------------------------------------------
   FUNCTION create_ddi_xref_link
      RETURN NUMBER
   IS
   BEGIN
      sys.DBMS_OUTPUT.put_line (CHR (10));
      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   START:  Create_DDI_XREF_Link');

      vc_sql_statement :=
            'UPDATE ctstaging.pecs_ddi_va ddi '
         || 'SET ddi.fdb_reference_id = '
         || '(SELECT xref.fdb_reference_id '
         || 'FROM ctstaging.fdb_ddim_xref xref '
         || 'WHERE xref.interaction_id = ddi.fdb_id) '
         || 'WHERE     ddi.fdb_reference_id IS NULL '
         || 'AND EXISTS '
         || '(SELECT * '
         || 'FROM ctstaging.fdb_ddim_xref subxref '
         || 'WHERE subxref.interaction_id = ddi.fdb_id)';

      EXECUTE IMMEDIATE vc_sql_statement;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) updated in ctstaging.pecs_ddi_va : '
         || TO_CHAR (SQL%ROWCOUNT));

      COMMIT;

      sys.DBMS_OUTPUT.put_line ('INFORMATION -   END:  Create_DDI_XREF_Link');
      RETURN (0);
   EXCEPTION
      WHEN OTHERS
      THEN
         vn_code := SQLCODE;
         vc_errm := SUBSTR (SQLERRM, 1, 64);
         sys.DBMS_OUTPUT.put_line (
            'Create_DDI_XREF_Link: Error code ' || vn_code || ': ' || vc_errm);
         sys.DBMS_OUTPUT.put_line (
            'Create_DDI_XREF_Link: last value for vc_sql_statement ');
         sys.DBMS_OUTPUT.put_line (
            'Create_DDI_XREF_Link: ' || vc_sql_statement);
         RETURN (1);
   END create_ddi_xref_link;

   -----------------------------------------------------------------------------
   -- NAME:         doserange_crossreference_migra
   -- TYPE:         Function
   -- DESCRIPTSION: Create objects that were in DoseRange_CrossReference_Migration.sql
   -- INPUTS:       None
   -- OUTPUTS:      Number: (0 - good: >0 - error)
   -----------------------------------------------------------------------------
   FUNCTION doserange_crossreference_migra
      RETURN NUMBER
   IS
   BEGIN
      sys.DBMS_OUTPUT.put_line (CHR (10));
      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   START:  doserange_crossreference_migra');

      vc_sql_statement :=
            'INSERT INTO ctstaging.fdb_dose_range_xref (fdb_reference_id '
         || ',concepttype '
         || ',conceptid '
         || ',agelowindays '
         || ',agehighindays '
         || ',doserouteid '
         || ',dosetypeid '
         || ',fdbdx '
         || ',hittype) '
         || 'SELECT ctstaging.fdb_doseroute_ref_id_seq.NEXTVAL '
         || ',concepttype '
         || ',conceptid '
         || ',agelowindays '
         || ',agehighindays '
         || ',routecode '
         || ',typecode '
         || ',fdbdx '
         || ',hittype '
         || 'FROM (  SELECT DISTINCT custom.concepttype '
         || ',custom.conceptid '
         || ',custom.agelowindays '
         || ',custom.agehighindays '
         || ',custom.routecode '
         || ',custom.typecode '
         || ',custom.fdbdx '
         || ',fdb.hittype '
         || 'FROM (  SELECT dr.va_dose_range_id '
         || ',dr.concepttype '
         || ',dr.conceptid '
         || ',dr.agelowindays '
         || ',dr.agehighindays '
         || ',droute.code AS routecode '
         || ',dtype.code AS typecode '
         || ',dr.fdbdx '
         || ',COUNT (*) cntrecs '
         || 'FROM ctstaging.pecs_dose_range_version_va dr '
         || 'INNER JOIN ctstaging.dose_route droute '
         || 'ON droute.dose_route_id = dr.dose_route_id '
         || 'INNER JOIN ctstaging.dose_type dtype '
         || 'ON dtype.dose_type_id = dr.dose_type_id '
         || 'WHERE dr.version_id = '
         || '(SELECT MIN (version_id) '
         || 'FROM ctstaging.pecs_dose_range_version_va subdr '
         || 'WHERE subdr.va_dose_range_id = '
         || 'dr.va_dose_range_id) '
         || 'GROUP BY dr.va_dose_range_id '
         || ',dr.concepttype '
         || ',dr.conceptid '
         || ',dr.agelowindays '
         || ',dr.agehighindays '
         || ',droute.code '
         || ',dtype.code '
         || ',dr.fdbdx) custom '
         || 'INNER JOIN fdb_dif.fdb_dosing fdb '
         || 'ON     custom.agehighindays = fdb.agehighindays '
         || 'AND custom.agelowindays = fdb.agelowindays '
         || 'AND custom.conceptid = fdb.conceptid '
         || 'AND custom.concepttype = fdb.concepttype '
         || 'AND custom.routecode = fdb.doserouteid '
         || 'AND custom.typecode = fdb.dosetypeid '
         || 'AND custom.fdbdx = fdb.fdbdx '
         || 'WHERE NOT EXISTS '
         || '(SELECT * '
         || 'FROM ctstaging.fdb_dose_range_xref xref '
         || 'WHERE     custom.agehighindays = '
         || 'xref.agehighindays '
         || 'AND custom.agelowindays = xref.agelowindays '
         || 'AND custom.conceptid = xref.conceptid '
         || 'AND custom.concepttype = xref.concepttype '
         || 'AND custom.routecode = xref.doserouteid '
         || 'AND custom.typecode = xref.dosetypeid '
         || 'AND custom.fdbdx = xref.fdbdx '
         || 'AND fdb.hittype = xref.hittype) '
         || 'ORDER BY custom.concepttype '
         || ',custom.conceptid '
         || ',custom.agelowindays '
         || ',custom.agehighindays '
         || ',custom.routecode '
         || ',custom.typecode '
         || ',custom.fdbdx '
         || ',fdb.hittype)';

      EXECUTE IMMEDIATE vc_sql_statement;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) inserted into ctstaging.fdb_dose_range_xref : '
         || TO_CHAR (SQL%ROWCOUNT));

      COMMIT;

      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   END:  doserange_crossreference_migra');
      RETURN (0);
   EXCEPTION
      WHEN OTHERS
      THEN
         vn_code := SQLCODE;
         vc_errm := SUBSTR (SQLERRM, 1, 64);
         sys.DBMS_OUTPUT.put_line (
               'doserange_crossreference_migra: Error code '
            || vn_code
            || ': '
            || vc_errm);
         sys.DBMS_OUTPUT.put_line (
            'doserange_crossreference_migra: last value for vc_sql_statement ');
         sys.DBMS_OUTPUT.put_line (
            'doserange_crossreference_migra: ' || vc_sql_statement);
         RETURN (1);
   END doserange_crossreference_migra;

   -----------------------------------------------------------------------------
   -- NAME:         Create_DoseRange_XREF_Link
   -- TYPE:         Function
   -- DESCRIPTSION: Create objects that were in Create_DoseRange_XREF_Link.sql
   -- INPUTS:       None
   -- OUTPUTS:      Number: (0 - good: >0 - error)
   -----------------------------------------------------------------------------
   FUNCTION create_doserange_xref_link
      RETURN NUMBER
   IS
   BEGIN
      sys.DBMS_OUTPUT.put_line (CHR (10));
      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   START:  Create_DoseRange_XREF_Link');

      vc_sql_statement :=
            'UPDATE ctstaging.pecs_dose_range_va dr '
         || 'SET dr.fdb_reference_id = '
         || '(SELECT DISTINCT xref.fdb_reference_id '
         || 'FROM ctstaging.pecs_dose_range_version_va drver '
         || 'INNER JOIN dose_route droute '
         || 'ON droute.dose_route_id = drver.dose_route_id '
         || 'INNER JOIN dose_type dtype '
         || 'ON dtype.dose_type_id = drver.dose_type_id '
         || 'INNER JOIN ctstaging.fdb_dose_range_xref xref '
         || 'ON     drver.agehighindays = xref.agehighindays '
         || 'AND drver.agelowindays = xref.agelowindays '
         || 'AND drver.conceptid = xref.conceptid '
         || 'AND drver.concepttype = xref.concepttype '
         || 'AND droute.code = xref.doserouteid '
         || 'AND dtype.code = xref.dosetypeid '
         || 'AND drver.fdbdx = xref.fdbdx '
         || 'WHERE     drver.va_dose_range_id = dr.va_dose_range_id '
         || 'AND drver.version_id = '
         || '(SELECT DISTINCT MIN (version_id) '
         || 'FROM ctstaging.pecs_dose_range_version_va subdr '
         || 'WHERE subdr.va_dose_range_id = '
         || 'drver.va_dose_range_id)) '
         || 'WHERE     dr.fdb_reference_id IS NULL '
         || 'AND EXISTS '
         || '(SELECT xref.fdb_reference_id '
         || ' FROM ctstaging.pecs_dose_range_version_va drver '
         || 'INNER JOIN dose_route droute '
         || 'ON droute.dose_route_id = drver.dose_route_id '
         || 'INNER JOIN dose_type dtype '
         || 'ON dtype.dose_type_id = drver.dose_type_id '
         || 'INNER JOIN ctstaging.fdb_dose_range_xref xref '
         || 'ON     drver.agehighindays = xref.agehighindays '
         || 'AND drver.agelowindays = xref.agelowindays '
         || 'AND drver.conceptid = xref.conceptid '
         || 'AND drver.concepttype = xref.concepttype '
         || 'AND droute.code = xref.doserouteid '
         || 'AND dtype.code = xref.dosetypeid '
         || 'AND drver.fdbdx = xref.fdbdx '
         || 'WHERE     drver.va_dose_range_id = dr.va_dose_range_id '
         || 'AND drver.version_id = '
         || '(SELECT DISTINCT MIN (version_id) '
         || 'FROM ctstaging.pecs_dose_range_version_va subdr '
         || 'WHERE subdr.va_dose_range_id = '
         || 'drver.va_dose_range_id))';

      EXECUTE IMMEDIATE vc_sql_statement;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) updated in ctstaging.pecs_dose_range_va : '
         || TO_CHAR (SQL%ROWCOUNT));

      COMMIT;

      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   END:  Create_DoseRange_XREF_Link');
      RETURN (0);
   EXCEPTION
      WHEN OTHERS
      THEN
         vn_code := SQLCODE;
         vc_errm := SUBSTR (SQLERRM, 1, 64);
         sys.DBMS_OUTPUT.put_line (
               'Create_DoseRange_XREF_Link: Error code '
            || vn_code
            || ': '
            || vc_errm);
         sys.DBMS_OUTPUT.put_line (
            'Create_DoseRange_XREF_Link: last value for vc_sql_statement ');
         sys.DBMS_OUTPUT.put_line (
            'Create_DoseRange_XREF_Link: ' || vc_sql_statement);
         RETURN (1);
   END create_doserange_xref_link;

   -----------------------------------------------------------------------------
   -- NAME:         DT_CrossReference_Migration
   -- TYPE:         Function
   -- DESCRIPTSION: Create objects that were in DT_CrossReference_Migration.sql
   -- INPUTS:       None
   -- OUTPUTS:      Number: (0 - good: >0 - error)
   -----------------------------------------------------------------------------
   FUNCTION dt_crossreference_migration
      RETURN NUMBER
   IS
   BEGIN
      sys.DBMS_OUTPUT.put_line (CHR (10));
      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   START:  DT_CrossReference_Migration');

      vc_sql_statement :=
            'INSERT INTO ctstaging.fdb_duptherapy_xref (fdb_reference_id, dtcid) '
         || 'SELECT ctstaging.fdb_dupltherapy_ref_id_seq.NEXTVAL, custom.dtcid '
         || 'FROM (  SELECT DISTINCT fdb.dtcid '
         || 'FROM ctstaging.pecs_dupl_therapy_va dt '
         || 'INNER JOIN fdb_dif.fdb_duplicatetherapy fdb '
         || 'ON fdb.dtcid = dt.dtcid '
         || 'WHERE NOT EXISTS '
         || '(SELECT * '
         || 'FROM ctstaging.fdb_duptherapy_xref xref '
         || 'WHERE xref.dtcid = fdb.dtcid) '
         || 'GROUP BY fdb.dtcid) custom';

      EXECUTE IMMEDIATE vc_sql_statement;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) inserted into ctstaging.fdb_duptherapy_xref : '
         || TO_CHAR (SQL%ROWCOUNT));

      COMMIT;

      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   END:  DT_CrossReference_Migration');
      RETURN (0);
   EXCEPTION
      WHEN OTHERS
      THEN
         vn_code := SQLCODE;
         vc_errm := SUBSTR (SQLERRM, 1, 64);
         sys.DBMS_OUTPUT.put_line (
               'DT_CrossReference_Migration: Error code '
            || vn_code
            || ': '
            || vc_errm);
         sys.DBMS_OUTPUT.put_line (
            'DT_CrossReference_Migration: last value for vc_sql_statement ');
         sys.DBMS_OUTPUT.put_line (
            'DT_CrossReference_Migration: ' || vc_sql_statement);
         RETURN (1);
   END dt_crossreference_migration;

   -----------------------------------------------------------------------------
   -- NAME:         Create_DT_XREF_Link
   -- TYPE:         Function
   -- DESCRIPTSION: Create objects that were in Create_DT_XREF_Link.sql
   -- INPUTS:       None
   -- OUTPUTS:      Number: (0 - good: >0 - error)
   -----------------------------------------------------------------------------
   FUNCTION create_dt_xref_link
      RETURN NUMBER
   IS
   BEGIN
      sys.DBMS_OUTPUT.put_line (CHR (10));
      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   START:  Create_DT_XREF_Link');

      vc_sql_statement :=
            'UPDATE ctstaging.pecs_dupl_therapy_va dt '
         || 'SET dt.fdb_reference_id = '
         || '(SELECT xref.fdb_reference_id '
         || 'FROM ctstaging.fdb_duptherapy_xref xref '
         || 'WHERE xref.dtcid = dt.dtcid) '
         || 'WHERE     dt.fdb_reference_id IS NULL '
         || 'AND EXISTS '
         || '(SELECT * '
         || 'FROM ctstaging.fdb_duptherapy_xref subxref '
         || 'WHERE subxref.dtcid = dt.dtcid)';

      EXECUTE IMMEDIATE vc_sql_statement;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) updated in ctstaging.pecs_dupl_therapy_va : '
         || TO_CHAR (SQL%ROWCOUNT));

      COMMIT;

      sys.DBMS_OUTPUT.put_line ('INFORMATION -   END:  Create_DT_XREF_Link');
      RETURN (0);
   EXCEPTION
      WHEN OTHERS
      THEN
         vn_code := SQLCODE;
         vc_errm := SUBSTR (SQLERRM, 1, 64);
         sys.DBMS_OUTPUT.put_line (
            'Create_DT_XREF_Link: Error code ' || vn_code || ': ' || vc_errm);
         sys.DBMS_OUTPUT.put_line (
            'Create_DT_XREF_Link: last value for vc_sql_statement ');
         sys.DBMS_OUTPUT.put_line (
            'Create_DT_XREF_Link: ' || vc_sql_statement);
         RETURN (1);
   END create_dt_xref_link;

   -----------------------------------------------------------------------------
   -- NAME:         MONO_CrossReference_Migration
   -- TYPE:         Function
   -- DESCRIPTSION: Create objects that were in MONO_CrossReference_Migration.sql
   -- INPUTS:       None
   -- OUTPUTS:      Number: (0 - good: >0 - error)
   -----------------------------------------------------------------------------
   FUNCTION mono_crossreference_migration
      RETURN NUMBER
   IS
   BEGIN
      sys.DBMS_OUTPUT.put_line (CHR (10));
      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   START:  MONO_CrossReference_Migration');

      vc_sql_statement :=
            'INSERT INTO ctstaging.fdb_monograph_xref (fdb_reference_id '
         || ',languageid '
         || ',monographid '
         || ',versioncode '
         || ',sequencenumber) '
         || 'SELECT ctstaging.fdb_monograph_ref_id_seq.NEXTVAL '
         || ',custom.languageid '
         || ',custom.monographid '
         || ',''FDB-P'' '
         || ',1 '
         || 'FROM (  SELECT fdb.monographid, fdb.languageid, COUNT (*) cntrecs '
         || 'FROM ctstaging.pecs_monograph_va mono '
         || 'INNER JOIN fdb_dif.fdb_monograph_ddim fdb '
         || 'ON     fdb.monographid = mono.monograph_id '
         || 'AND fdb.languageid = mono.language_id '
         || 'WHERE NOT EXISTS '
         || '(SELECT * '
         || 'FROM ctstaging.fdb_monograph_xref xref '
         || 'WHERE     fdb.monographid = xref.monographid '
         || 'AND fdb.languageid = xref.languageid) '
         || 'GROUP BY fdb.monographid, fdb.languageid '
         || 'ORDER BY fdb.monographid, fdb.languageid) custom';

      EXECUTE IMMEDIATE vc_sql_statement;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) inserted into ctstaging.fdb_monograph_xref : '
         || TO_CHAR (SQL%ROWCOUNT));

      COMMIT;

      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   END:  MONO_CrossReference_Migration');
      RETURN (0);
   EXCEPTION
      WHEN OTHERS
      THEN
         vn_code := SQLCODE;
         vc_errm := SUBSTR (SQLERRM, 1, 64);
         sys.DBMS_OUTPUT.put_line (
               'MONO_CrossReference_Migration: Error code '
            || vn_code
            || ': '
            || vc_errm);
         sys.DBMS_OUTPUT.put_line (
            'MONO_CrossReference_Migration: last value for vc_sql_statement ');
         sys.DBMS_OUTPUT.put_line (
            'MONO_CrossReference_Migration: ' || vc_sql_statement);
         RETURN (1);
   END mono_crossreference_migration;

   -----------------------------------------------------------------------------
   -- NAME:         Create_MONO_XREF_Link
   -- TYPE:         Function
   -- DESCRIPTSION: Create objects that were in Create_MONO_XREF_Link.sql
   -- INPUTS:       None
   -- OUTPUTS:      Number: (0 - good: >0 - error)
   -----------------------------------------------------------------------------
   FUNCTION create_mono_xref_link
      RETURN NUMBER
   IS
   BEGIN
      sys.DBMS_OUTPUT.put_line (CHR (10));
      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   START:  Create_MONO_XREF_Link');

      vc_sql_statement :=
            'UPDATE ctstaging.pecs_monograph_va mono '
         || 'SET mono.fdb_reference_id = '
         || '(SELECT xref.fdb_reference_id '
         || 'FROM ctstaging.fdb_monograph_xref xref '
         || ' WHERE     xref.languageid = mono.language_id '
         || 'AND xref.monographid = mono.monograph_id) '
         || 'WHERE     mono.fdb_reference_id IS NULL '
         || 'AND EXISTS '
         || '(SELECT * '
         || 'FROM ctstaging.fdb_monograph_xref subxref '
         || 'WHERE     subxref.languageid = mono.language_id '
         || 'AND subxref.monographid = mono.monograph_id)';

      EXECUTE IMMEDIATE vc_sql_statement;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) updated in ctstaging.pecs_monograph_va : '
         || TO_CHAR (SQL%ROWCOUNT));

      COMMIT;

      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   END:  Create_MONO_XREF_Link');
      RETURN (0);
   EXCEPTION
      WHEN OTHERS
      THEN
         vn_code := SQLCODE;
         vc_errm := SUBSTR (SQLERRM, 1, 64);
         sys.DBMS_OUTPUT.put_line (
               'Create_MONO_XREF_Link: Error code '
            || vn_code
            || ': '
            || vc_errm);
         sys.DBMS_OUTPUT.put_line (
            'Create_MONO_XREF_Link: last value for vc_sql_statement ');
         sys.DBMS_OUTPUT.put_line (
            'Create_MONO_XREF_Link: ' || vc_sql_statement);
         RETURN (1);
   END create_mono_xref_link;

   -----------------------------------------------------------------------------
   -- NAME:         FIELD_METADATA_DATA
   -- TYPE:         Function
   -- DESCRIPTSION: Create objects that were in FIELD_METADATA_DATA.sql
   -- INPUTS:       None
   -- OUTPUTS:      Number: (0 - good: >0 - error)
   -----------------------------------------------------------------------------
   FUNCTION field_metadata_data
      RETURN NUMBER
   IS
      vn_maxval         NUMBER;
      vc_concept_name   VARCHAR (200);
   BEGIN
      sys.DBMS_OUTPUT.put_line (CHR (10));
      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   START:  FIELD_METADATA_DATA');

      BEGIN
           SELECT concept_name, MAX (display_order) + 1 AS maxval
             INTO vc_concept_name, vn_maxval
             FROM ctstaging.field_metadata
            WHERE concept_name = 'FdbCustomDdiminteractionVa'
         GROUP BY concept_name;
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            vn_maxval := 1;
      END;

      IF vn_maxval IS NULL
      THEN
         vn_maxval := 1;
      END IF;

      vc_sql_statement :=
            'INSERT INTO ctstaging.field_metadata (field_id, concept_name, field_name, display_order, is_query, is_report, is_detail, display_name) '
         || 'SELECT ctstaging.fieldmeta_seq.nextval, ''FdbCustomDdiminteractionVa'', ''USER_COMMENT_HISTORY'', :newmaxval, ''N'', ''N'', ''Y'', ''re-Customization Comment History'' '
         || 'FROM dual';

      BEGIN
         EXECUTE IMMEDIATE vc_sql_statement USING vn_maxval;
      EXCEPTION
         WHEN OTHERS
         THEN
            vn_code := SQLCODE;
            vc_errm := SUBSTR (SQLERRM, 1, 64);
            sys.DBMS_OUTPUT.put_line (
                  'FIELD_METADATA_DATA: Error code '
               || vn_code
               || ': '
               || vc_errm);
            sys.DBMS_OUTPUT.put_line (
               'FIELD_METADATA_DATA: ' || vc_sql_statement);
      END;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) inserted into ctstaging.field_metadata : '
         || TO_CHAR (SQL%ROWCOUNT));

      BEGIN
           SELECT concept_name, MAX (display_order) + 1 AS maxval
             INTO vc_concept_name, vn_maxval
             FROM ctstaging.field_metadata
            WHERE concept_name = 'FdbCustomDdimVa'
         GROUP BY concept_name;
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            vn_maxval := 1;
      END;


      IF vn_maxval IS NULL
      THEN
         vn_maxval := 1;
      END IF;

      vc_sql_statement :=
            'INSERT INTO ctstaging.field_metadata (field_id, concept_name, field_name, display_order, is_query, is_report, is_detail, display_name) '
         || 'SELECT ctstaging.fieldmeta_seq.nextval, ''FdbCustomDdimVa'', ''USER_COMMENT_HISTORY'', :newmaxval, ''N'', ''N'', ''Y'', ''Pre-Customization Comment History'' '
         || 'FROM dual';

      BEGIN
         EXECUTE IMMEDIATE vc_sql_statement USING vn_maxval;
      EXCEPTION
         WHEN OTHERS
         THEN
            vn_code := SQLCODE;
            vc_errm := SUBSTR (SQLERRM, 1, 64);
            sys.DBMS_OUTPUT.put_line (
                  'FIELD_METADATA_DATA: Error code '
               || vn_code
               || ': '
               || vc_errm);
            sys.DBMS_OUTPUT.put_line (
               'FIELD_METADATA_DATA: ' || vc_sql_statement);
      END;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) inserted into ctstaging.field_metadata : '
         || TO_CHAR (SQL%ROWCOUNT));

      BEGIN
           SELECT concept_name, MAX (display_order) + 1 AS maxval
             INTO vc_concept_name, vn_maxval
             FROM ctstaging.field_metadata
            WHERE concept_name = 'FdbCustomDtcatVa'
         GROUP BY concept_name;
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            vn_maxval := 1;
      END;

      IF vn_maxval IS NULL
      THEN
         vn_maxval := 1;
      END IF;

      vc_sql_statement :=
            'INSERT INTO ctstaging.field_metadata (field_id, concept_name, field_name, display_order, is_query, is_report, is_detail, display_name) '
         || 'SELECT ctstaging.fieldmeta_seq.nextval, ''FdbCustomDtcatVa'', ''USER_COMMENT_HISTORY'', :newmaxval, ''N'', ''N'', ''Y'', ''Pre-Customization Comment History'' '
         || 'FROM dual';

      BEGIN
         EXECUTE IMMEDIATE vc_sql_statement USING vn_maxval;
      EXCEPTION
         WHEN OTHERS
         THEN
            vn_code := SQLCODE;
            vc_errm := SUBSTR (SQLERRM, 1, 64);
            sys.DBMS_OUTPUT.put_line (
                  'FIELD_METADATA_DATA: Error code '
               || vn_code
               || ': '
               || vc_errm);
            sys.DBMS_OUTPUT.put_line (
               'FIELD_METADATA_DATA: ' || vc_sql_statement);
      END;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) inserted into ctstaging.field_metadata : '
         || TO_CHAR (SQL%ROWCOUNT));


      BEGIN
           SELECT concept_name, MAX (display_order) + 1 AS maxval
             INTO vc_concept_name, vn_maxval
             FROM ctstaging.field_metadata
            WHERE concept_name = 'FdbCustomDosingVa'
         GROUP BY concept_name;
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            vn_maxval := 1;
      END;

      IF vn_maxval IS NULL
      THEN
         vn_maxval := 1;
      END IF;

      vc_sql_statement :=
            'INSERT INTO ctstaging.field_metadata (field_id, concept_name, field_name, display_order, is_query, is_report, is_detail, display_name) '
         || 'SELECT ctstaging.fieldmeta_seq.nextval, ''FdbCustomDosingVa'', ''USER_COMMENT_HISTORY'', :newmaxval, ''N'', ''N'', ''Y'', ''Pre-Customization Comment History'' '
         || 'FROM dual';

      BEGIN
         EXECUTE IMMEDIATE vc_sql_statement USING vn_maxval;
      EXCEPTION
         WHEN OTHERS
         THEN
            vn_code := SQLCODE;
            vc_errm := SUBSTR (SQLERRM, 1, 64);
            sys.DBMS_OUTPUT.put_line (
                  'FIELD_METADATA_DATA: Error code '
               || vn_code
               || ': '
               || vc_errm);
            sys.DBMS_OUTPUT.put_line (
               'FIELD_METADATA_DATA: ' || vc_sql_statement);
      END;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) inserted into ctstaging.field_metadata : '
         || TO_CHAR (SQL%ROWCOUNT));

      BEGIN
           SELECT concept_name, MAX (display_order) + 1 AS maxval
             INTO vc_concept_name, vn_maxval
             FROM ctstaging.field_metadata
            WHERE concept_name = 'MonographVa'
         GROUP BY concept_name;
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            vn_maxval := 1;
      END;

      IF vn_maxval IS NULL
      THEN
         vn_maxval := 1;
      END IF;

      vc_sql_statement :=
            'INSERT INTO ctstaging.field_metadata (field_id, concept_name, field_name, display_order, is_query, is_report, is_detail, display_name) '
         || 'SELECT ctstaging.fieldmeta_seq.nextval, ''MonographVa'', ''USER_COMMENT_HISTORY'', :newmaxval, ''N'', ''N'', ''Y'', ''Pre-Customization Comment History'' '
         || 'FROM dual';

      BEGIN
         EXECUTE IMMEDIATE vc_sql_statement USING vn_maxval;
      EXCEPTION
         WHEN OTHERS
         THEN
            vn_code := SQLCODE;
            vc_errm := SUBSTR (SQLERRM, 1, 64);
            sys.DBMS_OUTPUT.put_line (
                  'FIELD_METADATA_DATA: Error code '
               || vn_code
               || ': '
               || vc_errm);
            sys.DBMS_OUTPUT.put_line (
               'FIELD_METADATA_DATA: ' || vc_sql_statement);
      END;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) inserted into ctstaging.field_metadata : '
         || TO_CHAR (SQL%ROWCOUNT));

      COMMIT;

      sys.DBMS_OUTPUT.put_line ('INFORMATION -   END:  FIELD_METADATA_DATA');
      RETURN (0);
   EXCEPTION
      WHEN OTHERS
      THEN
         vn_code := SQLCODE;
         vc_errm := SUBSTR (SQLERRM, 1, 64);
         sys.DBMS_OUTPUT.put_line (
            'FIELD_METADATA_DATA: Error code ' || vn_code || ': ' || vc_errm);
         sys.DBMS_OUTPUT.put_line (
            'FIELD_METADATA_DATA: last value for vc_sql_statement ');
         sys.DBMS_OUTPUT.put_line (
            'FIELD_METADATA_DATA: ' || vc_sql_statement);
   END field_metadata_data;

   -----------------------------------------------------------------------------
   -- NAME:         FIELD_METADATA_ClassNameUpdate
   -- TYPE:         Function
   -- DESCRIPTSION: Create objects that were in FIELD_METADATA_ClassNameUpdate.sql
   -- INPUTS:       None
   -- OUTPUTS:      Number: (0 - good: >0 - error)
   -----------------------------------------------------------------------------
   FUNCTION field_metadata_classnameupdate
      RETURN NUMBER
   IS
   BEGIN
      sys.DBMS_OUTPUT.put_line (CHR (10));
      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   START:  FIELD_METADATA_ClassNameUpdate');

      vc_sql_statement :=
            'UPDATE field_metadata '
         || 'SET field_metadata.concept_name = ''MonographDTO'' '
         || 'WHERE field_metadata.concept_name = ''MonographVa'' ';


      BEGIN
         EXECUTE IMMEDIATE vc_sql_statement;
      EXCEPTION
         WHEN OTHERS
         THEN
            vn_code := SQLCODE;
            vc_errm := SUBSTR (SQLERRM, 1, 64);
            sys.DBMS_OUTPUT.put_line (
                  'field_metadata_classnameupdate: Error code '
               || vn_code
               || ': '
               || vc_errm);
            sys.DBMS_OUTPUT.put_line (
               'field_metadata_classnameupdate: ' || vc_sql_statement);
      END;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) updated in ctstaging.field_metadata : '
         || TO_CHAR (SQL%ROWCOUNT));

      vc_sql_statement :=
            'UPDATE field_metadata '
         || 'SET field_metadata.concept_name = ''DuplicateTherapyDTO'' '
         || 'WHERE field_metadata.concept_name = ''FdbCustomDtcatVa'' ';


      BEGIN
         EXECUTE IMMEDIATE vc_sql_statement;
      EXCEPTION
         WHEN OTHERS
         THEN
            vn_code := SQLCODE;
            vc_errm := SUBSTR (SQLERRM, 1, 64);
            sys.DBMS_OUTPUT.put_line (
                  'field_metadata_classnameupdate: Error code '
               || vn_code
               || ': '
               || vc_errm);
            sys.DBMS_OUTPUT.put_line (
               'field_metadata_classnameupdate: ' || vc_sql_statement);
      END;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) updated in ctstaging.field_metadata : '
         || TO_CHAR (SQL%ROWCOUNT));


      vc_sql_statement :=
            'UPDATE field_metadata '
         || 'SET field_metadata.concept_name = ''DoseRangeDTO'' '
         || 'WHERE field_metadata.concept_name = ''FdbCustomDosingVa'' ';


      BEGIN
         EXECUTE IMMEDIATE vc_sql_statement;
      EXCEPTION
         WHEN OTHERS
         THEN
            vn_code := SQLCODE;
            vc_errm := SUBSTR (SQLERRM, 1, 64);
            sys.DBMS_OUTPUT.put_line (
                  'field_metadata_classnameupdate: Error code '
               || vn_code
               || ': '
               || vc_errm);
            sys.DBMS_OUTPUT.put_line (
               'field_metadata_classnameupdate: ' || vc_sql_statement);
      END;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) updated in ctstaging.field_metadata : '
         || TO_CHAR (SQL%ROWCOUNT));


      vc_sql_statement :=
            'UPDATE field_metadata '
         || 'SET field_metadata.concept_name = ''DrugDrugInteractionDTO'' '
         || 'WHERE field_metadata.concept_name = ''FdbCustomDdiminteractionVa'' ';


      BEGIN
         EXECUTE IMMEDIATE vc_sql_statement;
      EXCEPTION
         WHEN OTHERS
         THEN
            vn_code := SQLCODE;
            vc_errm := SUBSTR (SQLERRM, 1, 64);
            sys.DBMS_OUTPUT.put_line (
                  'field_metadata_classnameupdate: Error code '
               || vn_code
               || ': '
               || vc_errm);
            sys.DBMS_OUTPUT.put_line (
               'field_metadata_classnameupdate: ' || vc_sql_statement);
      END;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) updated in ctstaging.field_metadata : '
         || TO_CHAR (SQL%ROWCOUNT));


      vc_sql_statement :=
            'UPDATE field_metadata '
         || 'SET field_metadata.concept_name = ''DrugPairDTO'' '
         || 'WHERE field_metadata.concept_name = ''FdbCustomDdimVa'' ';


      BEGIN
         EXECUTE IMMEDIATE vc_sql_statement;
      EXCEPTION
         WHEN OTHERS
         THEN
            vn_code := SQLCODE;
            vc_errm := SUBSTR (SQLERRM, 1, 64);
            sys.DBMS_OUTPUT.put_line (
                  'field_metadata_classnameupdate: Error code '
               || vn_code
               || ': '
               || vc_errm);
            sys.DBMS_OUTPUT.put_line (
               'field_metadata_classnameupdate: ' || vc_sql_statement);
      END;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) updated in ctstaging.field_metadata : '
         || TO_CHAR (SQL%ROWCOUNT));

      COMMIT;

      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   END:  FIELD_METADATA_ClassNameUpdate');
      RETURN (0);
   EXCEPTION
      WHEN OTHERS
      THEN
         vn_code := SQLCODE;
         vc_errm := SUBSTR (SQLERRM, 1, 64);
         sys.DBMS_OUTPUT.put_line (
               'FIELD_METADATA_ClassNameUpdate: Error code '
            || vn_code
            || ': '
            || vc_errm);
         sys.DBMS_OUTPUT.put_line (
            'FIELD_METADATA_ClassNameUpdate: last value for vc_sql_statement ');
         sys.DBMS_OUTPUT.put_line (
            'FIELD_METADATA_ClassNameUpdate: ' || vc_sql_statement);
   END field_metadata_classnameupdate;

   -----------------------------------------------------------------------------
   -- NAME:         COMPARISON_REPORT_CONCEPT_Dose
   -- TYPE:         Function
   -- DESCRIPTSION: Create objects that were in COMPARISON_REPORT_CONCEPT_DoseRange.sql
   -- INPUTS:       None
   -- OUTPUTS:      Number: (0 - good: >0 - error)
   -----------------------------------------------------------------------------
   FUNCTION comparison_report_concept_dose
      RETURN NUMBER
   IS
      vn_maxval   NUMBER;
   BEGIN
      sys.DBMS_OUTPUT.put_line (CHR (10));
      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   START:  COMPARISON_REPORT_CONCEPT_Dose');

      BEGIN
         SELECT MAX (comparison_report_concept_id) + 1 AS maxval
           INTO vn_maxval
           FROM ctstaging.comparison_report_concept;
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            vn_maxval := 1;
      END;

      IF vn_maxval IS NULL
      THEN
         vn_maxval := 1;
      END IF;

      vc_sql_statement :=
            'INSERT INTO ctstaging.comparison_report_concept (comparison_report_concept_id '
         || ' ,code '
         || ',name '
         || ',description '
         || ',sort_order '
         || ',effectivedate '
         || ',expirationdate '
         || ',record_created_by '
         || ',record_create_date '
         || ',record_modified_by '
         || ',record_modified_date) '
         || 'VALUES (:vn_maxval '
         || ',''DR'' '
         || ',''Dose Range'' '
         || ',NULL '
         || ',NULL '
         || ',CURRENT_DATE '
         || ',NULL '
         || ',''PECS Story 21120'' '
         || ',NULL '
         || ',NULL '
         || ',NULL)';


      BEGIN
         EXECUTE IMMEDIATE vc_sql_statement USING vn_maxval;
      EXCEPTION
         WHEN OTHERS
         THEN
            vn_code := SQLCODE;
            vc_errm := SUBSTR (SQLERRM, 1, 64);
            sys.DBMS_OUTPUT.put_line (
                  'field_metadata_classnameupdate: Error code '
               || vn_code
               || ': '
               || vc_errm);
            sys.DBMS_OUTPUT.put_line (
               'field_metadata_classnameupdate: ' || vc_sql_statement);
      END;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) inserted into ctstaging.comparison_report_concept : '
         || TO_CHAR (SQL%ROWCOUNT));

      COMMIT;

      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   END:  COMPARISON_REPORT_CONCEPT_Dose');
      RETURN (0);
   EXCEPTION
      WHEN OTHERS
      THEN
         vn_code := SQLCODE;
         vc_errm := SUBSTR (SQLERRM, 1, 64);
         sys.DBMS_OUTPUT.put_line (
               'COMPARISON_REPORT_CONCEPT_Dose: Error code '
            || vn_code
            || ': '
            || vc_errm);
         sys.DBMS_OUTPUT.put_line (
            'COMPARISON_REPORT_CONCEPT_Dose: last value for vc_sql_statement ');
         sys.DBMS_OUTPUT.put_line (
            'COMPARISON_REPORT_CONCEPT_Dose: ' || vc_sql_statement);
   END comparison_report_concept_dose;

   -----------------------------------------------------------------------------
   -- NAME:         DDI_EXPORT_DATE_COLUMN_ADD
   -- TYPE:         Function
   -- DESCRIPTSION: Create objects that were in DDI_EXPORT_DATE_COLUMN_ADD.sql
   -- INPUTS:       None
   -- OUTPUTS:      Number: (0 - good: >0 - error)
   -----------------------------------------------------------------------------
   FUNCTION ddi_export_date_column_add
      RETURN NUMBER
   IS
   BEGIN
      sys.DBMS_OUTPUT.put_line (CHR (10));
      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   START:  DDI_EXPORT_DATE_COLUMN_ADD');

      vc_sql_statement :=
         'ALTER TABLE ctstaging.pecs_ddi_version_va ADD export_date TIMESTAMP';

      EXECUTE IMMEDIATE vc_sql_statement;

      COMMIT;

      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   Successfully add column export_date to pecs_ddi_version_va');
      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   END:  DDI_EXPORT_DATE_COLUMN_ADD');
      RETURN (0);
   EXCEPTION
      WHEN OTHERS
      THEN
         vn_code := SQLCODE;
         vc_errm := SUBSTR (SQLERRM, 1, 64);
         sys.DBMS_OUTPUT.put_line (
               'DDI_EXPORT_DATE_COLUMN_ADD: Error code '
            || vn_code
            || ': '
            || vc_errm);
         sys.DBMS_OUTPUT.put_line (
            'DDI_EXPORT_DATE_COLUMN_ADD: last value for vc_sql_statement ');
         sys.DBMS_OUTPUT.put_line (
            'DDI_EXPORT_DATE_COLUMN_ADD: ' || vc_sql_statement);
         RETURN (1);
   END ddi_export_date_column_add;

   -----------------------------------------------------------------------------
   -- NAME:         DDI_EXPORT_DATE_FIELD_METADATA
   -- TYPE:         Function
   -- DESCRIPTSION: Create objects that were in DDI_EXPORT_DATE_FIELD_METADATA.sql
   -- INPUTS:       None
   -- OUTPUTS:      Number: (0 - good: >0 - error)
   -----------------------------------------------------------------------------
   FUNCTION ddi_export_date_field_metadata
      RETURN NUMBER
   IS
      vn_maxval   NUMBER;
   BEGIN
      sys.DBMS_OUTPUT.put_line (CHR (10));
      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   START:  DDI_EXPORT_DATE_FIELD_METADATA');

      BEGIN
         SELECT MAX (display_order) + 1 AS maxval
           INTO vn_maxval
           FROM ctstaging.field_metadata
          WHERE concept_name = 'DrugDrugInteractionDTO';
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            vn_maxval := 1;
      END;

      IF vn_maxval IS NULL
      THEN
         vn_maxval := 1;
      END IF;

      vc_sql_statement :=
            'INSERT INTO ctstaging.field_metadata (field_id '
         || ',concept_name '
         || ',field_name '
         || ',display_order '
         || ',is_query '
         || ',is_report '
         || ',is_detail '
         || ',display_name) '
         || 'SELECT ctstaging.fieldmeta_seq.NEXTVAL '
         || ',''DrugDrugInteractionDTO'' '
         || ',''EXPORT_DATE'' '
         || ',:newmaxval '
         || ',''Y'' '
         || ',''Y'' '
         || ',''Y'' '
         || ',''Export Date'' '
         || 'FROM DUAL';


      BEGIN
         EXECUTE IMMEDIATE vc_sql_statement USING vn_maxval;
      EXCEPTION
         WHEN OTHERS
         THEN
            vn_code := SQLCODE;
            vc_errm := SUBSTR (SQLERRM, 1, 64);
            sys.DBMS_OUTPUT.put_line (
                  'DDI_EXPORT_DATE_FIELD_METADATA: Error code '
               || vn_code
               || ': '
               || vc_errm);
            sys.DBMS_OUTPUT.put_line (
               'DDI_EXPORT_DATE_FIELD_METADATA: ' || vc_sql_statement);
      END;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) inserted into ctstaging.field_metadata : '
         || TO_CHAR (SQL%ROWCOUNT));

      COMMIT;

      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   END:  DDI_EXPORT_DATE_FIELD_METADATA');
      RETURN (0);
   EXCEPTION
      WHEN OTHERS
      THEN
         vn_code := SQLCODE;
         vc_errm := SUBSTR (SQLERRM, 1, 64);
         sys.DBMS_OUTPUT.put_line (
               'DDI_EXPORT_DATE_FIELD_METADATA: Error code '
            || vn_code
            || ': '
            || vc_errm);
         sys.DBMS_OUTPUT.put_line (
            'DDI_EXPORT_DATE_FIELD_METADATA: last value for vc_sql_statement ');
         sys.DBMS_OUTPUT.put_line (
            'DDI_EXPORT_DATE_FIELD_METADATA: ' || vc_sql_statement);
         RETURN (1);
   END ddi_export_date_field_metadata;

   -----------------------------------------------------------------------------
   -- NAME:         DR_EXPORT_DATE_COLUMN_ADD
   -- TYPE:         Function
   -- DESCRIPTSION: Create objects that were in DR_EXPORT_DATE_COLUMN_ADD.sql
   -- INPUTS:       None
   -- OUTPUTS:      Number: (0 - good: >0 - error)
   -----------------------------------------------------------------------------
   FUNCTION dr_export_date_column_add
      RETURN NUMBER
   IS
   BEGIN
      sys.DBMS_OUTPUT.put_line (CHR (10));
      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   START:  DR_EXPORT_DATE_COLUMN_ADD');

      vc_sql_statement :=
         'ALTER TABLE ctstaging.pecs_dose_range_version_va ADD export_date TIMESTAMP';

      EXECUTE IMMEDIATE vc_sql_statement;

      COMMIT;

      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   Successfully add column export_date to pecs_dose_range_version_va');
      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   END:  DR_EXPORT_DATE_COLUMN_ADD');
      RETURN (0);
   EXCEPTION
      WHEN OTHERS
      THEN
         vn_code := SQLCODE;
         vc_errm := SUBSTR (SQLERRM, 1, 64);
         sys.DBMS_OUTPUT.put_line (
               'DR_EXPORT_DATE_COLUMN_ADD: Error code '
            || vn_code
            || ': '
            || vc_errm);
         sys.DBMS_OUTPUT.put_line (
            'DR_EXPORT_DATE_COLUMN_ADD: last value for vc_sql_statement ');
         sys.DBMS_OUTPUT.put_line (
            'DR_EXPORT_DATE_COLUMN_ADD: ' || vc_sql_statement);
         RETURN (1);
   END dr_export_date_column_add;

   -----------------------------------------------------------------------------
   -- NAME:         DR_EXPORT_DATE_FIELD_METADATA
   -- TYPE:         Function
   -- DESCRIPTSION: Create objects that were in DR_EXPORT_DATE_FIELD_METADATA.sql
   -- INPUTS:       None
   -- OUTPUTS:      Number: (0 - good: >0 - error)
   -----------------------------------------------------------------------------
   FUNCTION dr_export_date_field_metadata
      RETURN NUMBER
   IS
      vn_maxval   NUMBER;
   BEGIN
      sys.DBMS_OUTPUT.put_line (CHR (10));
      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   START:  DR_EXPORT_DATE_FIELD_METADATA');

      BEGIN
         SELECT MAX (display_order) + 1 AS maxval
           INTO vn_maxval
           FROM ctstaging.field_metadata
          WHERE concept_name = 'DoseRangeDTO';
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            vn_maxval := 1;
      END;

      IF vn_maxval IS NULL
      THEN
         vn_maxval := 1;
      END IF;

      vc_sql_statement :=
            'INSERT INTO ctstaging.field_metadata (field_id '
         || ',concept_name '
         || ',field_name '
         || ',display_order '
         || ',is_query '
         || ',is_report '
         || ',is_detail '
         || ',display_name) '
         || 'SELECT ctstaging.fieldmeta_seq.NEXTVAL '
         || ',''DoseRangeDTO'' '
         || ',''EXPORT_DATE'' '
         || ',:newmaxval '
         || ',''Y'' '
         || ',''Y'' '
         || ',''Y'' '
         || ',''Export Date'' '
         || 'FROM DUAL';


      BEGIN
         EXECUTE IMMEDIATE vc_sql_statement USING vn_maxval;
      EXCEPTION
         WHEN OTHERS
         THEN
            vn_code := SQLCODE;
            vc_errm := SUBSTR (SQLERRM, 1, 64);
            sys.DBMS_OUTPUT.put_line (
                  'DR_EXPORT_DATE_FIELD_METADATA: Error code '
               || vn_code
               || ': '
               || vc_errm);
            sys.DBMS_OUTPUT.put_line (
               'DR_EXPORT_DATE_FIELD_METADATA: ' || vc_sql_statement);
      END;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) inserted into ctstaging.field_metadata : '
         || TO_CHAR (SQL%ROWCOUNT));

      COMMIT;

      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   END:  DR_EXPORT_DATE_FIELD_METADATA');
      RETURN (0);
   EXCEPTION
      WHEN OTHERS
      THEN
         vn_code := SQLCODE;
         vc_errm := SUBSTR (SQLERRM, 1, 64);
         sys.DBMS_OUTPUT.put_line (
               'DR_EXPORT_DATE_FIELD_METADATA: Error code '
            || vn_code
            || ': '
            || vc_errm);
         sys.DBMS_OUTPUT.put_line (
            'DR_EXPORT_DATE_FIELD_METADATA: last value for vc_sql_statement ');
         sys.DBMS_OUTPUT.put_line (
            'DR_EXPORT_DATE_FIELD_METADATA: ' || vc_sql_statement);
         RETURN (1);
   END dr_export_date_field_metadata;

   -----------------------------------------------------------------------------
   -- NAME:         DRUGPAIRS_EXPORT_DATE_COLUMN_A
   -- TYPE:         Function
   -- DESCRIPTSION: Create objects that were in DRUGPAIRS_EXPORT_DATE_COLUMN_ADD.sql
   -- INPUTS:       None
   -- OUTPUTS:      Number: (0 - good: >0 - error)
   -----------------------------------------------------------------------------
   FUNCTION drugpairs_export_date_column_a
      RETURN NUMBER
   IS
   BEGIN
      sys.DBMS_OUTPUT.put_line (CHR (10));
      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   START:  DRUGPAIRS_EXPORT_DATE_COLUMN_A');

      vc_sql_statement :=
         'ALTER TABLE ctstaging.pecs_drug_pair_version_va ADD export_date TIMESTAMP';

      EXECUTE IMMEDIATE vc_sql_statement;

      COMMIT;

      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   Successfully add column export_date to pecs_drug_pair_version_va');
      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   END:  DRUGPAIRS_EXPORT_DATE_COLUMN_A');
      RETURN (0);
   EXCEPTION
      WHEN OTHERS
      THEN
         vn_code := SQLCODE;
         vc_errm := SUBSTR (SQLERRM, 1, 64);
         sys.DBMS_OUTPUT.put_line (
               'DRUGPAIRS_EXPORT_DATE_COLUMN_A: Error code '
            || vn_code
            || ': '
            || vc_errm);
         sys.DBMS_OUTPUT.put_line (
            'DRUGPAIRS_EXPORT_DATE_COLUMN_A: last value for vc_sql_statement ');
         sys.DBMS_OUTPUT.put_line (
            'DRUGPAIRS_EXPORT_DATE_COLUMN_A: ' || vc_sql_statement);
         RETURN (1);
   END drugpairs_export_date_column_a;

   -----------------------------------------------------------------------------
   -- NAME:         DRUGPAIRS_EXPORT_DATE_FIELD_ME
   -- TYPE:         Function
   -- DESCRIPTSION: Create objects that were in DRUGPAIRS_EXPORT_DATE_FIELD_METADATA.sql
   -- INPUTS:       None
   -- OUTPUTS:      Number: (0 - good: >0 - error)
   -----------------------------------------------------------------------------
   FUNCTION drugpairs_export_date_field_me
      RETURN NUMBER
   IS
      vn_maxval   NUMBER;
   BEGIN
      sys.DBMS_OUTPUT.put_line (CHR (10));
      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   START:  DRUGPAIRS_EXPORT_DATE_FIELD_ME');

      BEGIN
         SELECT MAX (display_order) + 1 AS maxval
           INTO vn_maxval
           FROM ctstaging.field_metadata
          WHERE concept_name = 'DrugPairDTO';
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            vn_maxval := 1;
      END;

      IF vn_maxval IS NULL
      THEN
         vn_maxval := 1;
      END IF;

      vc_sql_statement :=
            'INSERT INTO ctstaging.field_metadata (field_id '
         || ',concept_name '
         || ',field_name '
         || ',display_order '
         || ',is_query '
         || ',is_report '
         || ',is_detail '
         || ',display_name) '
         || 'SELECT ctstaging.fieldmeta_seq.NEXTVAL '
         || ',''DrugPairDTO'' '
         || ',''EXPORT_DATE'' '
         || ',:newmaxval '
         || ',''Y'' '
         || ',''Y'' '
         || ',''Y'' '
         || ',''Export Date'' '
         || 'FROM DUAL';


      BEGIN
         EXECUTE IMMEDIATE vc_sql_statement USING vn_maxval;
      EXCEPTION
         WHEN OTHERS
         THEN
            vn_code := SQLCODE;
            vc_errm := SUBSTR (SQLERRM, 1, 64);
            sys.DBMS_OUTPUT.put_line (
                  'DRUGPAIRS_EXPORT_DATE_FIELD_ME: Error code '
               || vn_code
               || ': '
               || vc_errm);
            sys.DBMS_OUTPUT.put_line (
               'DRUGPAIRS_EXPORT_DATE_FIELD_ME: ' || vc_sql_statement);
      END;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) inserted into ctstaging.field_metadata : '
         || TO_CHAR (SQL%ROWCOUNT));

      COMMIT;

      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   END:  DRUGPAIRS_EXPORT_DATE_FIELD_ME');
      RETURN (0);
   EXCEPTION
      WHEN OTHERS
      THEN
         vn_code := SQLCODE;
         vc_errm := SUBSTR (SQLERRM, 1, 64);
         sys.DBMS_OUTPUT.put_line (
               'DRUGPAIRS_EXPORT_DATE_FIELD_ME: Error code '
            || vn_code
            || ': '
            || vc_errm);
         sys.DBMS_OUTPUT.put_line (
            'DRUGPAIRS_EXPORT_DATE_FIELD_ME: last value for vc_sql_statement ');
         sys.DBMS_OUTPUT.put_line (
            'DRUGPAIRS_EXPORT_DATE_FIELD_ME: ' || vc_sql_statement);
         RETURN (1);
   END drugpairs_export_date_field_me;

   -----------------------------------------------------------------------------
   -- NAME:         DT_EXPORT_DATE_COLUMN_ADD
   -- TYPE:         Function
   -- DESCRIPTSION: Create objects that were in DT_EXPORT_DATE_COLUMN_ADD.sql
   -- INPUTS:       None
   -- OUTPUTS:      Number: (0 - good: >0 - error)
   -----------------------------------------------------------------------------
   FUNCTION dt_export_date_column_add
      RETURN NUMBER
   IS
   BEGIN
      sys.DBMS_OUTPUT.put_line (CHR (10));
      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   START:  DT_EXPORT_DATE_COLUMN_ADD');

      vc_sql_statement :=
         'ALTER TABLE ctstaging.pecs_dupl_therapy_version_va ADD export_date TIMESTAMP';

      EXECUTE IMMEDIATE vc_sql_statement;

      COMMIT;

      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   Successfully add column export_date to pecs_dupl_therapy_version_va');
      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   END:  DT_EXPORT_DATE_COLUMN_ADD');
      RETURN (0);
   EXCEPTION
      WHEN OTHERS
      THEN
         vn_code := SQLCODE;
         vc_errm := SUBSTR (SQLERRM, 1, 64);
         sys.DBMS_OUTPUT.put_line (
               'DT_EXPORT_DATE_COLUMN_ADD: Error code '
            || vn_code
            || ': '
            || vc_errm);
         sys.DBMS_OUTPUT.put_line (
            'DT_EXPORT_DATE_COLUMN_ADD: last value for vc_sql_statement ');
         sys.DBMS_OUTPUT.put_line (
            'DT_EXPORT_DATE_COLUMN_ADD: ' || vc_sql_statement);
         RETURN (1);
   END dt_export_date_column_add;

   -----------------------------------------------------------------------------
   -- NAME:         DT_EXPORT_DATE_FIELD_METADATA
   -- TYPE:         Function
   -- DESCRIPTSION: Create objects that were in DT_EXPORT_DATE_FIELD_METADATA.sql
   -- INPUTS:       None
   -- OUTPUTS:      Number: (0 - good: >0 - error)
   -----------------------------------------------------------------------------
   FUNCTION dt_export_date_field_metadata
      RETURN NUMBER
   IS
      vn_maxval   NUMBER;
   BEGIN
      sys.DBMS_OUTPUT.put_line (CHR (10));
      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   START:  DT_EXPORT_DATE_FIELD_METADATA');

      BEGIN
         SELECT MAX (display_order) + 1 AS maxval
           INTO vn_maxval
           FROM ctstaging.field_metadata
          WHERE concept_name = 'DuplicateTherapyDTO';
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            vn_maxval := 1;
      END;

      IF vn_maxval IS NULL
      THEN
         vn_maxval := 1;
      END IF;

      vc_sql_statement :=
            'INSERT INTO ctstaging.field_metadata (field_id '
         || ',concept_name '
         || ',field_name '
         || ',display_order '
         || ',is_query '
         || ',is_report '
         || ',is_detail '
         || ',display_name) '
         || 'SELECT ctstaging.fieldmeta_seq.NEXTVAL '
         || ',''DuplicateTherapyDTO'' '
         || ',''EXPORT_DATE'' '
         || ',:newmaxval '
         || ',''Y'' '
         || ',''Y'' '
         || ',''Y'' '
         || ',''Export Date'' '
         || 'FROM DUAL';


      BEGIN
         EXECUTE IMMEDIATE vc_sql_statement USING vn_maxval;
      EXCEPTION
         WHEN OTHERS
         THEN
            vn_code := SQLCODE;
            vc_errm := SUBSTR (SQLERRM, 1, 64);
            sys.DBMS_OUTPUT.put_line (
                  'DT_EXPORT_DATE_FIELD_METADATA: Error code '
               || vn_code
               || ': '
               || vc_errm);
            sys.DBMS_OUTPUT.put_line (
               'DT_EXPORT_DATE_FIELD_METADATA: ' || vc_sql_statement);
      END;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) inserted into ctstaging.field_metadata : '
         || TO_CHAR (SQL%ROWCOUNT));

      COMMIT;

      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   END:  DT_EXPORT_DATE_FIELD_METADATA');
      RETURN (0);
   EXCEPTION
      WHEN OTHERS
      THEN
         vn_code := SQLCODE;
         vc_errm := SUBSTR (SQLERRM, 1, 64);
         sys.DBMS_OUTPUT.put_line (
               'DT_EXPORT_DATE_FIELD_METADATA: Error code '
            || vn_code
            || ': '
            || vc_errm);
         sys.DBMS_OUTPUT.put_line (
            'DT_EXPORT_DATE_FIELD_METADATA: last value for vc_sql_statement ');
         sys.DBMS_OUTPUT.put_line (
            'DT_EXPORT_DATE_FIELD_METADATA: ' || vc_sql_statement);
         RETURN (1);
   END dt_export_date_field_metadata;

   -----------------------------------------------------------------------------
   -- NAME:         PM_EXPORT_DATE_COLUMN_ADD
   -- TYPE:         Function
   -- DESCRIPTSION: Create objects that were in PM_EXPORT_DATE_COLUMN_ADD.sql
   -- INPUTS:       None
   -- OUTPUTS:      Number: (0 - good: >0 - error)
   -----------------------------------------------------------------------------
   FUNCTION pm_export_date_column_add
      RETURN NUMBER
   IS
   BEGIN
      sys.DBMS_OUTPUT.put_line (CHR (10));
      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   START:  PM_EXPORT_DATE_COLUMN_ADD');

      vc_sql_statement :=
         'ALTER TABLE ctstaging.pecs_monograph_version_va ADD export_date TIMESTAMP';

      EXECUTE IMMEDIATE vc_sql_statement;

      COMMIT;

      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   Successfully add column export_date to pecs_monograph_version_va');
      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   END:  PM_EXPORT_DATE_COLUMN_ADD');
      RETURN (0);
   EXCEPTION
      WHEN OTHERS
      THEN
         vn_code := SQLCODE;
         vc_errm := SUBSTR (SQLERRM, 1, 64);
         sys.DBMS_OUTPUT.put_line (
               'PM_EXPORT_DATE_COLUMN_ADD: Error code '
            || vn_code
            || ': '
            || vc_errm);
         sys.DBMS_OUTPUT.put_line (
            'PM_EXPORT_DATE_COLUMN_ADD: last value for vc_sql_statement ');
         sys.DBMS_OUTPUT.put_line (
            'PM_EXPORT_DATE_COLUMN_ADD: ' || vc_sql_statement);
         RETURN (1);
   END pm_export_date_column_add;

   -----------------------------------------------------------------------------
   -- NAME:         PM_EXPORT_DATE_FIELD_METADATA
   -- TYPE:         Function
   -- DESCRIPTSION: Create objects that were in PM_EXPORT_DATE_FIELD_METADATA.sql
   -- INPUTS:       None
   -- OUTPUTS:      Number: (0 - good: >0 - error)
   -----------------------------------------------------------------------------
   FUNCTION pm_export_date_field_metadata
      RETURN NUMBER
   IS
      vn_maxval   NUMBER;
   BEGIN
      sys.DBMS_OUTPUT.put_line (CHR (10));
      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   START:  PM_EXPORT_DATE_FIELD_METADATA');

      BEGIN
         SELECT MAX (display_order) + 1 AS maxval
           INTO vn_maxval
           FROM ctstaging.field_metadata
          WHERE concept_name = 'MonographDTO';
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            vn_maxval := 1;
      END;

      IF vn_maxval IS NULL
      THEN
         vn_maxval := 1;
      END IF;

      vc_sql_statement :=
            'INSERT INTO ctstaging.field_metadata (field_id '
         || ',concept_name '
         || ',field_name '
         || ',display_order '
         || ',is_query '
         || ',is_report '
         || ',is_detail '
         || ',display_name) '
         || 'SELECT ctstaging.fieldmeta_seq.NEXTVAL '
         || ',''MonographDTO'' '
         || ',''EXPORT_DATE'' '
         || ',:newmaxval '
         || ',''Y'' '
         || ',''Y'' '
         || ',''Y'' '
         || ',''Export Date'' '
         || 'FROM DUAL';


      BEGIN
         EXECUTE IMMEDIATE vc_sql_statement USING vn_maxval;
      EXCEPTION
         WHEN OTHERS
         THEN
            vn_code := SQLCODE;
            vc_errm := SUBSTR (SQLERRM, 1, 64);
            sys.DBMS_OUTPUT.put_line (
                  'PM_EXPORT_DATE_FIELD_METADATA: Error code '
               || vn_code
               || ': '
               || vc_errm);
            sys.DBMS_OUTPUT.put_line (
               'PM_EXPORT_DATE_FIELD_METADATA: ' || vc_sql_statement);
      END;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) inserted into ctstaging.field_metadata : '
         || TO_CHAR (SQL%ROWCOUNT));

      COMMIT;

      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   END:  PM_EXPORT_DATE_FIELD_METADATA');
      RETURN (0);
   EXCEPTION
      WHEN OTHERS
      THEN
         vn_code := SQLCODE;
         vc_errm := SUBSTR (SQLERRM, 1, 64);
         sys.DBMS_OUTPUT.put_line (
               'PM_EXPORT_DATE_FIELD_METADATA: Error code '
            || vn_code
            || ': '
            || vc_errm);
         sys.DBMS_OUTPUT.put_line (
            'PM_EXPORT_DATE_FIELD_METADATA: last value for vc_sql_statement ');
         sys.DBMS_OUTPUT.put_line (
            'PM_EXPORT_DATE_FIELD_METADATA: ' || vc_sql_statement);
         RETURN (1);
   END pm_export_date_field_metadata;

   -----------------------------------------------------------------------------
   -- NAME:         Migrate_DDI_Export_Date
   -- TYPE:         Function
   -- DESCRIPTSION: Create objects that were in Migrate_DDI_Export_Date.sql
   -- INPUTS:       None
   -- OUTPUTS:      Number: (0 - good: >0 - error)
   -----------------------------------------------------------------------------
   FUNCTION migrate_ddi_export_date
      RETURN NUMBER
   IS
   BEGIN
      sys.DBMS_OUTPUT.put_line (CHR (10));
      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   START:  Migrate_DDI_Export_Date');

      vc_sql_statement :=
            'UPDATE ctstaging.pecs_ddi_version_va mainddi '
         || 'SET (export_date) = '
         || '(SELECT maxdate '
         || 'FROM ctstaging.pecs_ddi_version_va ddi '
         || 'LEFT OUTER JOIN '
         || '(  SELECT subddi.pecs_ddi_va_id '
         || ',MAX (subddi.version_date) maxdate '
         || 'FROM ctstaging.pecs_ddi_version_va subddi '
         || 'WHERE subddi.status_id IN (5, 7) '
         || 'GROUP BY subddi.pecs_ddi_va_id '
         || 'ORDER BY subddi.pecs_ddi_va_id) source_ddi '
         || 'ON ddi.pecs_ddi_va_id = source_ddi.pecs_ddi_va_id '
         || 'WHERE     version_date = maxdate '
         || 'AND ddi.pecs_ddi_va_id = mainddi.pecs_ddi_va_id) '
         || 'WHERE (mainddi.pecs_ddi_va_id, mainddi.version_num) IN (SELECT ddi.pecs_ddi_va_id '
         || ',ddi.version_num '
         || 'FROM ctstaging.pecs_ddi_version_va ddi '
         || 'LEFT OUTER JOIN '
         || '(  SELECT subddi.pecs_ddi_va_id '
         || ',MAX ( '
         || 'subddi.version_date) '
         || 'maxdate '
         || 'FROM ctstaging.pecs_ddi_version_va subddi '
         || 'WHERE subddi.status_id IN (5,7) '
         || 'GROUP BY subddi.pecs_ddi_va_id '
         || 'ORDER BY subddi.pecs_ddi_va_id) '
         || 'source_ddi '
         || 'ON ddi.pecs_ddi_va_id = '
         || 'source_ddi.pecs_ddi_va_id '
         || 'WHERE version_date = '
         || 'maxdate)';


      EXECUTE IMMEDIATE vc_sql_statement;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) updated in ctstaging.pecs_ddi_version_va : '
         || TO_CHAR (SQL%ROWCOUNT));

      COMMIT;

      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   END:  Migrate_DDI_Export_Date');
      RETURN (0);
   EXCEPTION
      WHEN OTHERS
      THEN
         vn_code := SQLCODE;
         vc_errm := SUBSTR (SQLERRM, 1, 64);
         sys.DBMS_OUTPUT.put_line (
               'Migrate_DDI_Export_Date: Error code '
            || vn_code
            || ': '
            || vc_errm);
         sys.DBMS_OUTPUT.put_line (
            'Migrate_DDI_Export_Date: last value for vc_sql_statement ');
         sys.DBMS_OUTPUT.put_line (
            'Migrate_DDI_Export_Date: ' || vc_sql_statement);
         RETURN (1);
   END migrate_ddi_export_date;

   -----------------------------------------------------------------------------
   -- NAME:         Migrate_DP_Export_Date
   -- TYPE:         Function
   -- DESCRIPTSION: Create objects that were in Migrate_DP_Export_Date.sql
   -- INPUTS:       None
   -- OUTPUTS:      Number: (0 - good: >0 - error)
   -----------------------------------------------------------------------------
   FUNCTION migrate_dp_export_date
      RETURN NUMBER
   IS
   BEGIN
      sys.DBMS_OUTPUT.put_line (CHR (10));
      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   START:  Migrate_DP_Export_Date');

      vc_sql_statement :=
            'UPDATE ctstaging.pecs_drug_pair_version_va maindp '
         || 'SET (export_date) = '
         || '(SELECT DISTINCT maxdate '
         || 'FROM ctstaging.pecs_drug_pair_version_va dp '
         || 'LEFT OUTER JOIN '
         || '(  SELECT subdp.va_drug_pair_id '
         || ',MAX (subdp.version_date) maxdate '
         || 'FROM ctstaging.pecs_drug_pair_version_va subdp '
         || 'WHERE subdp.status_id IN (5, 7) '
         || 'GROUP BY subdp.va_drug_pair_id '
         || 'ORDER BY subdp.va_drug_pair_id) source_dp '
         || 'ON dp.va_drug_pair_id = source_dp.va_drug_pair_id '
         || 'WHERE     version_date = maxdate '
         || 'AND dp.va_drug_pair_id = maindp.va_drug_pair_id) '
         || 'WHERE (maindp.va_drug_pair_id, maindp.version_num) IN (SELECT dp.va_drug_pair_id '
         || ',dp.version_num '
         || 'FROM ctstaging.pecs_drug_pair_version_va dp '
         || 'LEFT OUTER JOIN '
         || '(  SELECT subdp.va_drug_pair_id '
         || ',MAX (subdp.version_date) maxdate '
         || 'FROM ctstaging.pecs_drug_pair_version_va subdp '
         || 'WHERE subdp.status_id IN (5,7) '
         || 'GROUP BY subdp.va_drug_pair_id '
         || 'ORDER BY subdp.va_drug_pair_id) source_dp '
         || 'ON dp.va_drug_pair_id = source_dp.va_drug_pair_id '
         || 'WHERE version_date = maxdate)';


      EXECUTE IMMEDIATE vc_sql_statement;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) updated in ctstaging.pecs_drug_pair_version_va : '
         || TO_CHAR (SQL%ROWCOUNT));

      COMMIT;

      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   END:  Migrate_DP_Export_Date');
      RETURN (0);
   EXCEPTION
      WHEN OTHERS
      THEN
         vn_code := SQLCODE;
         vc_errm := SUBSTR (SQLERRM, 1, 64);
         sys.DBMS_OUTPUT.put_line (
               'Migrate_DP_Export_Date: Error code '
            || vn_code
            || ': '
            || vc_errm);
         sys.DBMS_OUTPUT.put_line (
            'Migrate_DP_Export_Date: last value for vc_sql_statement ');
         sys.DBMS_OUTPUT.put_line (
            'Migrate_DP_Export_Date: ' || vc_sql_statement);
         RETURN (1);
   END migrate_dp_export_date;

   -----------------------------------------------------------------------------
   -- NAME:         Migrate_DR_Export_Date
   -- TYPE:         Function
   -- DESCRIPTSION: Create objects that were in Migrate_DR_Export_Date.sql
   -- INPUTS:       None
   -- OUTPUTS:      Number: (0 - good: >0 - error)
   -----------------------------------------------------------------------------
   FUNCTION migrate_dr_export_date
      RETURN NUMBER
   IS
   BEGIN
      sys.DBMS_OUTPUT.put_line (CHR (10));
      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   START:  Migrate_DR_Export_Date');

      vc_sql_statement :=
            'UPDATE ctstaging.pecs_dose_range_version_va maindr '
         || 'SET (export_date) = '
         || '(SELECT DISTINCT maxdate '
         || 'FROM ctstaging.pecs_dose_range_version_va dr '
         || 'LEFT OUTER JOIN '
         || '(  SELECT subdr.va_dose_range_id '
         || ',MAX (subdr.version_date) maxdate '
         || 'FROM ctstaging.pecs_dose_range_version_va subdr '
         || 'WHERE subdr.status_id IN (5, 7) '
         || 'GROUP BY subdr.va_dose_range_id '
         || 'ORDER BY subdr.va_dose_range_id) source_dr '
         || 'ON dr.va_dose_range_id = source_dr.va_dose_range_id '
         || 'WHERE     version_date = maxdate '
         || 'AND dr.va_dose_range_id = maindr.va_dose_range_id) '
         || 'WHERE (maindr.va_dose_range_id, maindr.version_num) IN (SELECT dr.va_dose_range_id '
         || ',dr.version_num '
         || 'FROM ctstaging.pecs_dose_range_version_va dr '
         || 'LEFT OUTER JOIN '
         || '(  SELECT subdr.va_dose_range_id '
         || ',MAX (subdr.version_date) maxdate '
         || 'FROM ctstaging.pecs_dose_range_version_va subdr '
         || 'WHERE subdr.status_id IN (5,7) '
         || 'GROUP BY subdr.va_dose_range_id '
         || 'ORDER BY subdr.va_dose_range_id) source_dr '
         || 'ON dr.va_dose_range_id = source_dr.va_dose_range_id '
         || 'WHERE version_date = maxdate)';


      EXECUTE IMMEDIATE vc_sql_statement;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) updated in ctstaging.pecs_dose_range_version_va : '
         || TO_CHAR (SQL%ROWCOUNT));

      COMMIT;

      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   END:  Migrate_DR_Export_Date');
      RETURN (0);
   EXCEPTION
      WHEN OTHERS
      THEN
         vn_code := SQLCODE;
         vc_errm := SUBSTR (SQLERRM, 1, 64);
         sys.DBMS_OUTPUT.put_line (
               'Migrate_DR_Export_Date: Error code '
            || vn_code
            || ': '
            || vc_errm);
         sys.DBMS_OUTPUT.put_line (
            'Migrate_DR_Export_Date: last value for vc_sql_statement ');
         sys.DBMS_OUTPUT.put_line (
            'Migrate_DR_Export_Date: ' || vc_sql_statement);
         RETURN (1);
   END migrate_dr_export_date;

   -----------------------------------------------------------------------------
   -- NAME:         Migrate_DT_Export_Date
   -- TYPE:         Function
   -- DESCRIPTSION: Create objects that were in Migrate_DT_Export_Date.sql
   -- INPUTS:       None
   -- OUTPUTS:      Number: (0 - good: >0 - error)
   -----------------------------------------------------------------------------
   FUNCTION migrate_dt_export_date
      RETURN NUMBER
   IS
   BEGIN
      sys.DBMS_OUTPUT.put_line (CHR (10));
      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   START:  Migrate_DT_Export_Date');

      vc_sql_statement :=
            'UPDATE ctstaging.pecs_dupl_therapy_version_va maindt '
         || 'SET (export_date) = '
         || '(SELECT DISTINCT maxdate '
         || 'FROM ctstaging.pecs_dupl_therapy_version_va dt '
         || 'LEFT OUTER JOIN '
         || '(  SELECT subdt.va_dupl_therapy_id '
         || ',MAX (subdt.version_date) maxdate '
         || 'FROM ctstaging.pecs_dupl_therapy_version_va subdt '
         || 'WHERE subdt.status_id IN (5, 7) '
         || 'GROUP BY subdt.va_dupl_therapy_id '
         || 'ORDER BY subdt.va_dupl_therapy_id) source_dt '
         || 'ON dt.va_dupl_therapy_id = source_dt.va_dupl_therapy_id '
         || 'WHERE     version_date = maxdate '
         || 'AND dt.va_dupl_therapy_id = maindt.va_dupl_therapy_id) '
         || 'WHERE (maindt.va_dupl_therapy_id, maindt.version_num) IN (SELECT dt.va_dupl_therapy_id '
         || ',dt.version_num '
         || 'FROM ctstaging.pecs_dupl_therapy_version_va dt '
         || 'LEFT OUTER JOIN '
         || '(  SELECT subdt.va_dupl_therapy_id '
         || ',MAX (subdt.version_date) maxdate '
         || 'FROM ctstaging.pecs_dupl_therapy_version_va subdt '
         || 'WHERE subdt.status_id IN (5,7) '
         || 'GROUP BY subdt.va_dupl_therapy_id '
         || 'ORDER BY subdt.va_dupl_therapy_id) source_dt '
         || 'ON dt.va_dupl_therapy_id = source_dt.va_dupl_therapy_id '
         || 'WHERE version_date = maxdate)';


      EXECUTE IMMEDIATE vc_sql_statement;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) updated in ctstaging.pecs_dupl_therapy_version_va : '
         || TO_CHAR (SQL%ROWCOUNT));

      COMMIT;

      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   END:  Migrate_DT_Export_Date');
      RETURN (0);
   EXCEPTION
      WHEN OTHERS
      THEN
         vn_code := SQLCODE;
         vc_errm := SUBSTR (SQLERRM, 1, 64);
         sys.DBMS_OUTPUT.put_line (
               'Migrate_DT_Export_Date: Error code '
            || vn_code
            || ': '
            || vc_errm);
         sys.DBMS_OUTPUT.put_line (
            'Migrate_DT_Export_Date: last value for vc_sql_statement ');
         sys.DBMS_OUTPUT.put_line (
            'Migrate_DT_Export_Date: ' || vc_sql_statement);
         RETURN (1);
   END migrate_dt_export_date;

   -----------------------------------------------------------------------------
   -- NAME:         Migrate_Mono_Export_Date
   -- TYPE:         Function
   -- DESCRIPTSION: Create objects that were in Migrate_Mono_Export_Date.sql
   -- INPUTS:       None
   -- OUTPUTS:      Number: (0 - good: >0 - error)
   -----------------------------------------------------------------------------
   FUNCTION migrate_mono_export_date
      RETURN NUMBER
   IS
   BEGIN
      sys.DBMS_OUTPUT.put_line (CHR (10));
      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   START:  Migrate_Mono_Export_Date');

      vc_sql_statement :=
            'UPDATE ctstaging.pecs_monograph_version_va mainmono '
         || 'SET (export_date) = '
         || '(SELECT DISTINCT maxdate '
         || 'FROM ctstaging.pecs_monograph_version_va mono '
         || 'LEFT OUTER JOIN '
         || '(  SELECT submono.va_monograph_id '
         || ',MAX (submono.version_date) maxdate '
         || 'FROM ctstaging.pecs_monograph_version_va submono '
         || 'WHERE submono.status_id IN (5, 7) '
         || 'GROUP BY submono.va_monograph_id '
         || 'ORDER BY submono.va_monograph_id) source_mono '
         || 'ON mono.va_monograph_id = source_mono.va_monograph_id '
         || 'WHERE     version_date = maxdate '
         || 'AND mono.va_monograph_id = mainmono.va_monograph_id) '
         || 'WHERE (mainmono.va_monograph_id, mainmono.version_num) IN (SELECT mono.va_monograph_id '
         || ',mono.version_num '
         || 'FROM ctstaging.pecs_monograph_version_va mono '
         || 'LEFT OUTER JOIN '
         || '(  SELECT submono.va_monograph_id '
         || ',MAX (submono.version_date) maxdate '
         || 'FROM ctstaging.pecs_monograph_version_va submono '
         || 'WHERE submono.status_id IN (5,7) '
         || 'GROUP BY submono.va_monograph_id '
         || 'ORDER BY submono.va_monograph_id) source_mono '
         || 'ON mono.va_monograph_id = source_mono.va_monograph_id '
         || 'WHERE version_date = maxdate)';


      EXECUTE IMMEDIATE vc_sql_statement;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) updated in ctstaging.pecs_monograph_version_va : '
         || TO_CHAR (SQL%ROWCOUNT));

      COMMIT;

      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   END:  Migrate_Mono_Export_Date');
      RETURN (0);
   EXCEPTION
      WHEN OTHERS
      THEN
         vn_code := SQLCODE;
         vc_errm := SUBSTR (SQLERRM, 1, 64);
         sys.DBMS_OUTPUT.put_line (
               'Migrate_Mono_Export_Date: Error code '
            || vn_code
            || ': '
            || vc_errm);
         sys.DBMS_OUTPUT.put_line (
            'Migrate_Mono_Export_Date: last value for vc_sql_statement ');
         sys.DBMS_OUTPUT.put_line (
            'Migrate_Mono_Export_Date: ' || vc_sql_statement);
         RETURN (1);
   END migrate_mono_export_date;

   -----------------------------------------------------------------------------
   -- NAME:         ZIP_FILE_NAME_ADD_EXTENSION
   -- TYPE:         Function
   -- DESCRIPTSION: Create objects that were in ZIP_FILE_NAME_ADD_EXTENSION.sql
   -- INPUTS:       None
   -- OUTPUTS:      Number: (0 - good: >0 - error)
   -----------------------------------------------------------------------------
   FUNCTION zip_file_name_add_extension
      RETURN NUMBER
   IS
   BEGIN
      sys.DBMS_OUTPUT.put_line (CHR (10));
      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   START:  ZIP_FILE_NAME_ADD_EXTENSION');

      vc_sql_statement :=
            'UPDATE ctstaging.custom_update_file_va '
         || 'SET zip_file_name = CONCAT (zip_file_name, ''.ZIP'') '
         || 'WHERE zip_file_name NOT LIKE ''%.ZIP'' AND zip_file_name NOT LIKE ''%.zip'' ';


      EXECUTE IMMEDIATE vc_sql_statement;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) updated in ctstaging.custom_update_file_va : '
         || TO_CHAR (SQL%ROWCOUNT));

      COMMIT;

      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   END:  ZIP_FILE_NAME_ADD_EXTENSION');
      RETURN (0);
   EXCEPTION
      WHEN OTHERS
      THEN
         vn_code := SQLCODE;
         vc_errm := SUBSTR (SQLERRM, 1, 64);
         sys.DBMS_OUTPUT.put_line (
               'ZIP_FILE_NAME_ADD_EXTENSION: Error code '
            || vn_code
            || ': '
            || vc_errm);
         sys.DBMS_OUTPUT.put_line (
            'ZIP_FILE_NAME_ADD_EXTENSION: last value for vc_sql_statement ');
         sys.DBMS_OUTPUT.put_line (
            'ZIP_FILE_NAME_ADD_EXTENSION: ' || vc_sql_statement);
         RETURN (1);
   END zip_file_name_add_extension;

   -----------------------------------------------------------------------------
   -- NAME:         ADD_COLUMN_REVERSE_FDB_ID_TO_P
   -- TYPE:         Function
   -- DESCRIPTSION: Create objects that were in ADD_COLUMN_REVERSE_FDB_ID_TO_PECS_DDI_VA.sql
   -- INPUTS:       None
   -- OUTPUTS:      Number: (0 - good: >0 - error)
   -----------------------------------------------------------------------------
   FUNCTION add_column_reverse_fdb_id_to_p
      RETURN NUMBER
   IS
   BEGIN
      sys.DBMS_OUTPUT.put_line (CHR (10));
      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   START:  ADD_COLUMN_REVERSE_FDB_ID_TO_P');

      vc_sql_statement :=
         'ALTER TABLE ctstaging.pecs_ddi_va ADD reverse_fdb_id NUMBER';

      EXECUTE IMMEDIATE vc_sql_statement;

      COMMIT;

      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   Successfully add column reverse_fdb_id to pecs_ddi_va');
      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   END:  ADD_COLUMN_REVERSE_FDB_ID_TO_P');
      RETURN (0);
   EXCEPTION
      WHEN OTHERS
      THEN
         vn_code := SQLCODE;
         vc_errm := SUBSTR (SQLERRM, 1, 64);
         sys.DBMS_OUTPUT.put_line (
               'ADD_COLUMN_REVERSE_FDB_ID_TO_P: Error code '
            || vn_code
            || ': '
            || vc_errm);
         sys.DBMS_OUTPUT.put_line (
            'ADD_COLUMN_REVERSE_FDB_ID_TO_P: last value for vc_sql_statement ');
         sys.DBMS_OUTPUT.put_line (
            'ADD_COLUMN_REVERSE_FDB_ID_TO_P: ' || vc_sql_statement);
         RETURN (1);
   END add_column_reverse_fdb_id_to_p;

   -----------------------------------------------------------------------------
   -- NAME:         ADD_REVERSE_FDB_ID_TO_DRUG_DRU
   -- TYPE:         Function
   -- DESCRIPTSION: Create objects that were in ADD_REVERSE_FDB_ID_TO_DRUG_DRUG_INTERACTION_DTO.sql
   -- INPUTS:       None
   -- OUTPUTS:      Number: (0 - good: >0 - error)
   -----------------------------------------------------------------------------
   FUNCTION add_reverse_fdb_id_to_drug_dru
      RETURN NUMBER
   IS
      vn_maxval   NUMBER;
   BEGIN
      sys.DBMS_OUTPUT.put_line (CHR (10));
      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   START:  ADD_REVERSE_FDB_ID_TO_DRUG_DRU');

      BEGIN
         SELECT MAX (display_order) + 1 AS maxval
           INTO vn_maxval
           FROM ctstaging.field_metadata
          WHERE concept_name = 'DrugDrugInteractionDTO';
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            vn_maxval := 1;
      END;

      IF vn_maxval IS NULL
      THEN
         vn_maxval := 1;
      END IF;

      vc_sql_statement :=
            'INSERT INTO ctstaging.field_metadata (field_id '
         || ',concept_name '
         || ',field_name '
         || ',display_order '
         || ',is_query '
         || ',is_report '
         || ',is_detail '
         || ',display_name) '
         || 'SELECT ctstaging.fieldmeta_seq.NEXTVAL '
         || ',''DrugDrugInteractionDTO'' '
         || ',''REVERSE_FDB_ID'' '
         || ',:newmaxval '
         || ',''Y'' '
         || ',''Y'' '
         || ',''Y'' '
         || ',''Reverse FDB ID'' '
         || 'FROM DUAL';


      BEGIN
         EXECUTE IMMEDIATE vc_sql_statement USING vn_maxval;
      EXCEPTION
         WHEN OTHERS
         THEN
            vn_code := SQLCODE;
            vc_errm := SUBSTR (SQLERRM, 1, 64);
            sys.DBMS_OUTPUT.put_line (
                  'ADD_REVERSE_FDB_ID_TO_DRUG_DRU: Error code '
               || vn_code
               || ': '
               || vc_errm);
            sys.DBMS_OUTPUT.put_line (
               'ADD_REVERSE_FDB_ID_TO_DRUG_DRU: ' || vc_sql_statement);
      END;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) inserted into ctstaging.field_metadata : '
         || TO_CHAR (SQL%ROWCOUNT));

      COMMIT;

      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   END:  ADD_REVERSE_FDB_ID_TO_DRUG_DRU');
      RETURN (0);
   EXCEPTION
      WHEN OTHERS
      THEN
         vn_code := SQLCODE;
         vc_errm := SUBSTR (SQLERRM, 1, 64);
         sys.DBMS_OUTPUT.put_line (
               'ADD_REVERSE_FDB_ID_TO_DRUG_DRU: Error code '
            || vn_code
            || ': '
            || vc_errm);
         sys.DBMS_OUTPUT.put_line (
            'ADD_REVERSE_FDB_ID_TO_DRUG_DRU: last value for vc_sql_statement ');
         sys.DBMS_OUTPUT.put_line (
            'ADD_REVERSE_FDB_ID_TO_DRUG_DRU: ' || vc_sql_statement);
         RETURN (1);
   END add_reverse_fdb_id_to_drug_dru;

   -----------------------------------------------------------------------------
   -- NAME:         ADD_REVERSE_FDB_ID_TO_DrugPair
   -- TYPE:         Function
   -- DESCRIPTSION: Create objects that were in ADD_REVERSE_FDB_ID_TO_DrugPairDTO.sql
   -- INPUTS:       None
   -- OUTPUTS:      Number: (0 - good: >0 - error)
   -----------------------------------------------------------------------------
   FUNCTION add_reverse_fdb_id_to_drugpair
      RETURN NUMBER
   IS
      vn_maxval   NUMBER;
   BEGIN
      sys.DBMS_OUTPUT.put_line (CHR (10));
      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   START:  ADD_REVERSE_FDB_ID_TO_DrugPair');

      BEGIN
         SELECT MAX (display_order) + 1 AS maxval
           INTO vn_maxval
           FROM ctstaging.field_metadata
          WHERE concept_name = 'DrugPairDTO';
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            vn_maxval := 1;
      END;

      IF vn_maxval IS NULL
      THEN
         vn_maxval := 1;
      END IF;

      vc_sql_statement :=
            'INSERT INTO ctstaging.field_metadata (field_id '
         || ',concept_name '
         || ',field_name '
         || ',display_order '
         || ',is_query '
         || ',is_report '
         || ',is_detail '
         || ',display_name) '
         || 'SELECT ctstaging.fieldmeta_seq.NEXTVAL '
         || ',''DrugPairDTO'' '
         || ',''REVERSE_FDB_ID'' '
         || ',:newmaxval '
         || ',''Y'' '
         || ',''Y'' '
         || ',''Y'' '
         || ',''Reverse FDB ID'' '
         || 'FROM DUAL';


      BEGIN
         EXECUTE IMMEDIATE vc_sql_statement USING vn_maxval;
      EXCEPTION
         WHEN OTHERS
         THEN
            vn_code := SQLCODE;
            vc_errm := SUBSTR (SQLERRM, 1, 64);
            sys.DBMS_OUTPUT.put_line (
                  'ADD_REVERSE_FDB_ID_TO_DrugPair: Error code '
               || vn_code
               || ': '
               || vc_errm);
            sys.DBMS_OUTPUT.put_line (
               'ADD_REVERSE_FDB_ID_TO_DrugPair: ' || vc_sql_statement);
      END;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) inserted into ctstaging.field_metadata : '
         || TO_CHAR (SQL%ROWCOUNT));

      COMMIT;

      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   END:  ADD_REVERSE_FDB_ID_TO_DrugPair');
      RETURN (0);
   EXCEPTION
      WHEN OTHERS
      THEN
         vn_code := SQLCODE;
         vc_errm := SUBSTR (SQLERRM, 1, 64);
         sys.DBMS_OUTPUT.put_line (
               'ADD_REVERSE_FDB_ID_TO_DrugPair: Error code '
            || vn_code
            || ': '
            || vc_errm);
         sys.DBMS_OUTPUT.put_line (
            'ADD_REVERSE_FDB_ID_TO_DrugPair: last value for vc_sql_statement ');
         sys.DBMS_OUTPUT.put_line (
            'ADD_REVERSE_FDB_ID_TO_DrugPair: ' || vc_sql_statement);
         RETURN (1);
   END add_reverse_fdb_id_to_drugpair;

   -----------------------------------------------------------------------------
   -- NAME:         POPULATE_REVERSE_FDB_ID_FOR_PA
   -- TYPE:         Function
   -- DESCRIPTSION: Create objects that were in POPULATE_REVERSE_FDB_ID_FOR_PAST_RECORDS.sql
   -- INPUTS:       None
   -- OUTPUTS:      Number: (0 - good: >0 - error)
   -----------------------------------------------------------------------------
   FUNCTION populate_reverse_fdb_id_for_pa
      RETURN NUMBER
   IS
   BEGIN
      sys.DBMS_OUTPUT.put_line (CHR (10));
      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   START:  POPULATE_REVERSE_FDB_ID_FOR_PA');

      vc_sql_statement :=
            'UPDATE ctstaging.pecs_ddi_va ddi '
         || 'SET ddi.reverse_fdb_id = 32000 - ddi.fdb_id';


      EXECUTE IMMEDIATE vc_sql_statement;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) updated in ctstaging.custom_update_file_va : '
         || TO_CHAR (SQL%ROWCOUNT));

      COMMIT;

      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   END:  POPULATE_REVERSE_FDB_ID_FOR_PA');
      RETURN (0);
   EXCEPTION
      WHEN OTHERS
      THEN
         vn_code := SQLCODE;
         vc_errm := SUBSTR (SQLERRM, 1, 64);
         sys.DBMS_OUTPUT.put_line (
               'POPULATE_REVERSE_FDB_ID_FOR_PA: Error code '
            || vn_code
            || ': '
            || vc_errm);
         sys.DBMS_OUTPUT.put_line (
            'POPULATE_REVERSE_FDB_ID_FOR_PA: last value for vc_sql_statement ');
         sys.DBMS_OUTPUT.put_line (
            'POPULATE_REVERSE_FDB_ID_FOR_PA: ' || vc_sql_statement);
         RETURN (1);
   END populate_reverse_fdb_id_for_pa;

   -----------------------------------------------------------------------------
   -- NAME:         Execute_Concept_Export_Migrati
   -- TYPE:         Function
   -- DESCRIPTSION: Create objects that were in Execute_Concept_Export_Migration.sql
   -- INPUTS:       None
   -- OUTPUTS:      Number: (0 - good: >0 - error)
   -----------------------------------------------------------------------------
   FUNCTION execute_concept_export_migrati
      RETURN NUMBER
   IS
   BEGIN
      sys.DBMS_OUTPUT.put_line (CHR (10));
      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   START:  Execute_Concept_Export_Migrati');

      concept_export_migration ('CTSTAGING.PECS_DDI_VA'
                               ,'CTSTAGING.PECS_DDI_EXPORT_VA'
                               ,'PECS_DDI_VA_ID');

      concept_export_migration ('CTSTAGING.PECS_DRUG_PAIR_VA'
                               ,'CTSTAGING.PECS_DRUG_PAIR_EXPORT_VA'
                               ,'VA_DRUG_PAIR_ID');

      concept_export_migration ('CTSTAGING.PECS_DUPL_THERAPY_VA'
                               ,'CTSTAGING.PECS_DUPL_THERAPY_EXPORT_VA'
                               ,'VA_DUPL_THERAPY_ID');

      concept_export_migration ('CTSTAGING.PECS_DOSE_RANGE_VA'
                               ,'CTSTAGING.PECS_DOSE_RANGE_EXPORT_VA'
                               ,'VA_DOSE_RANGE_ID');

      concept_export_migration ('CTSTAGING.PECS_MONOGRAPH_VA'
                               ,'CTSTAGING.PECS_MONOGRAPH_EXPORT_VA'
                               ,'VA_MONOGRAPH_ID');

      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   END:  Execute_Concept_Export_Migrati');
      RETURN (0);
   EXCEPTION
      WHEN OTHERS
      THEN
         vn_code := SQLCODE;
         vc_errm := SUBSTR (SQLERRM, 1, 64);
         sys.DBMS_OUTPUT.put_line (
               'Execute_Concept_Export_Migrati: Error code '
            || vn_code
            || ': '
            || vc_errm);
         RETURN (1);
   END execute_concept_export_migrati;

   -----------------------------------------------------------------------------
   -- NAME:         ADD_PECS4_FOREIGN_KEYS
   -- TYPE:         Function
   -- DESCRIPTSION: Create objects that were in ADD_PECS4_FOREIGN_KEYS.sql
   -- INPUTS:       None
   -- OUTPUTS:      Number: (0 - good: >0 - error)
   -----------------------------------------------------------------------------
   FUNCTION add_pecs4_foreign_keys
      RETURN NUMBER
   IS
   BEGIN
      sys.DBMS_OUTPUT.put_line (CHR (10));
      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   START:  ADD_PECS4_FOREIGN_KEYS');

      -- Create relationships section -------------------------------------------------

      /* Already exists in present schema
      -- should be a renamed using standard naming conventions
      ALTER TABLE CTSTAGING.PECS_DDI_VERSION_VA ADD CONSTRAINT FK_CLINICAL_EFFECT1_ID FOREIGN KEY (CLINICAL_EFFECT1_ID) REFERENCES CTSTAGING.CLINICAL_EFFECT (CLINICAL_EFFECT_ID)/
      ALTER TABLE CTSTAGING.PECS_DDI_VERSION_VA ADD CONSTRAINT FK_CLINICAL_EFFECT2_ID FOREIGN KEY (CLINICAL_EFFECT2_ID) REFERENCES CTSTAGING.CLINICAL_EFFECT (CLINICAL_EFFECT_ID)/
      ALTER TABLE CTSTAGING.PECS_DRUG_PAIR_VA ADD CONSTRAINT FK_DDI_TO_DP FOREIGN KEY (PECS_DDI_VA_ID) REFERENCES CTSTAGING.PECS_DDI_VA (PECS_DDI_VA_ID)/
      ALTER TABLE CTSTAGING.FDB_UPDATE_REPORT ADD CONSTRAINT FK_RPTCONCEPT_RPT FOREIGN KEY (COMPARISON_REPORT_CONCEPT_ID) REFERENCES CTSTAGING.COMPARISON_REPORT_CONCEPT (COMPARISON_REPORT_CONCEPT_ID)/
      ALTER TABLE CTSTAGING.PECS_DDI_VERSION_VA ADD CONSTRAINT FK_SEVERITY_LEVEL_ID FOREIGN KEY (SEVERITY_LEVEL_ID) REFERENCES CTSTAGING.SEVERITY_LEVEL (SEVERITY_LEVEL_ID)/
      */

      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.PECS_DRUG_PAIR_VERSION_VA ADD CONSTRAINT FK_DPVERSION_DP FOREIGN KEY (VA_DRUG_PAIR_ID) REFERENCES CTSTAGING.PECS_DRUG_PAIR_VA (VA_DRUG_PAIR_ID)';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.PECS_DDI_VERSION_VA ADD CONSTRAINT FK_DDIVERSION_TO_DDI FOREIGN KEY (PECS_DDI_VA_ID) REFERENCES CTSTAGING.PECS_DDI_VA (PECS_DDI_VA_ID)';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.PECS_DUPL_THERAPY_VERSION_VA ADD CONSTRAINT FK_DTVERSION_TO_DT FOREIGN KEY (VA_DUPL_THERAPY_ID) REFERENCES CTSTAGING.PECS_DUPL_THERAPY_VA (VA_DUPL_THERAPY_ID)';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.PECS_DOSE_RANGE_VERSION_VA ADD CONSTRAINT FK_DRVERSION_TO_DR FOREIGN KEY (VA_DOSE_RANGE_ID) REFERENCES CTSTAGING.PECS_DOSE_RANGE_VA (VA_DOSE_RANGE_ID)';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.PECS_MONOGRAPH_VERSION_VA ADD CONSTRAINT FK_MONOVERSION_TO_MONO FOREIGN KEY (VA_MONOGRAPH_ID) REFERENCES CTSTAGING.PECS_MONOGRAPH_VA (VA_MONOGRAPH_ID)';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.PECS_MONOGRAPH_VERSION_LINE_VA ADD CONSTRAINT FK_MONOVERSLINE_TO_MONOVERSION FOREIGN KEY (VERSION_ID) REFERENCES CTSTAGING.PECS_MONOGRAPH_VERSION_VA (VERSION_ID)';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.PECS_MONOGRAPH_VERSION_LINE_VA ADD CONSTRAINT FK_MONOVERSLINE_MONOSECTLINE FOREIGN KEY (LINE_ID) REFERENCES CTSTAGING.PECS_MONOGRAPH_SECTION_LINE_VA (LINE_ID)';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.PECS_MONOGRAPH_SECTION_LINE_VA ADD CONSTRAINT FK_MONOSECTLINE_SECTORDER FOREIGN KEY (SECTION_CODE) REFERENCES CTSTAGING.MONOGRAPH_SECTION_ORDER_VA (SECTIONCODE)';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.PECS_DOSE_RANGE_VERSION_VA ADD CONSTRAINT FK_DRVERSION_DOSETYPE FOREIGN KEY (DOSE_TYPE_ID) REFERENCES CTSTAGING.DOSE_TYPE (DOSE_TYPE_ID)';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.PECS_DOSE_RANGE_VERSION_VA ADD CONSTRAINT FK_DRVERSION_DOSEROUTE FOREIGN KEY (DOSE_ROUTE_ID) REFERENCES CTSTAGING.DOSE_ROUTE (DOSE_ROUTE_ID)';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.PECS_DUPL_THERAPY_VERSION_VA ADD CONSTRAINT FK_DTVERSION_TO_STATUS FOREIGN KEY (STATUS_ID) REFERENCES CTSTAGING.PECS_STATUS (STATUS_ID)';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.PECS_MONOGRAPH_VERSION_VA ADD CONSTRAINT FK_MONOVERS_TO_STATUS FOREIGN KEY (STATUS_ID) REFERENCES CTSTAGING.PECS_STATUS (STATUS_ID)';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.PECS_DRUG_PAIR_VERSION_VA ADD CONSTRAINT FK_DPVERSION_TO_STATUS FOREIGN KEY (STATUS_ID) REFERENCES CTSTAGING.PECS_STATUS (STATUS_ID)';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.PECS_DOSE_RANGE_VERSION_VA ADD CONSTRAINT FK_DRVERSION_TO_STATUS FOREIGN KEY (STATUS_ID) REFERENCES CTSTAGING.PECS_STATUS (STATUS_ID)';

      EXECUTE IMMEDIATE vc_sql_statement;

      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.PECS_DDI_VERSION_VA ADD CONSTRAINT FK_DDIVERSION_TO_STATUS FOREIGN KEY (STATUS_ID) REFERENCES CTSTAGING.PECS_STATUS (STATUS_ID)';

      COMMIT;

      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   Successfully added forgiegn key relationsship for PECS 5');
      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   END:  ADD_PECS4_FOREIGN_KEYS');
      RETURN (0);
   EXCEPTION
      WHEN OTHERS
      THEN
         vn_code := SQLCODE;
         vc_errm := SUBSTR (SQLERRM, 1, 64);
         sys.DBMS_OUTPUT.put_line (
               'ADD_PECS4_FOREIGN_KEYS: Error code '
            || vn_code
            || ': '
            || vc_errm);
         sys.DBMS_OUTPUT.put_line (
            'ADD_PECS4_FOREIGN_KEYS: last value for vc_sql_statement ');
         sys.DBMS_OUTPUT.put_line (
            'ADD_PECS4_FOREIGN_KEYS: ' || vc_sql_statement);
         RETURN (1);
   END add_pecs4_foreign_keys;

   -----------------------------------------------------------------------------
   -- NAME:         HALF_LIFE_MIGRATION
   -- TYPE:         Function
   -- DESCRIPTSION: Create objects that were in HALF_LIFE_MIGRATION.sql
   -- INPUTS:       None
   -- OUTPUTS:      Number: (0 - good: >0 - error)
   -----------------------------------------------------------------------------
   FUNCTION half_life_migration
      RETURN NUMBER
   IS
   BEGIN
      sys.DBMS_OUTPUT.put_line (CHR (10));
      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   START:  HALF_LIFE_MIGRATION');

      --add new id column for reference
      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.PECS_DOSE_RANGE_VERSION_VA ADD (HALF_LIFE_UNIT_ID Number(11,0))';

      EXECUTE IMMEDIATE vc_sql_statement;

      --migration existing data
      vc_sql_statement :=
            'UPDATE CTSTAGING.PECS_DOSE_RANGE_VERSION_VA DR SET HALF_LIFE_UNIT_ID = '
         || '(SELECT half_life_unit_id FROM CTSTAGING.HALF_LIFE_UNIT HALFLIFE '
         || 'where HALFLIFE.CODE = DR.HALFLIFEUNITS) '
         || 'WHERE DR.HALF_LIFE_UNIT_ID IS NULL';

      EXECUTE IMMEDIATE vc_sql_statement;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) updated in ctstaging.pecs_dose_range_version_va : '
         || TO_CHAR (SQL%ROWCOUNT));

      --add foreign key for reference id field
      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.PECS_DOSE_RANGE_VERSION_VA ADD CONSTRAINT FK_HALFLIFE_DRVERSION FOREIGN KEY (HALF_LIFE_UNIT_ID) REFERENCES CTSTAGING.HALF_LIFE_UNIT (HALF_LIFE_UNIT_ID)';

      EXECUTE IMMEDIATE vc_sql_statement;

      --drop column once migration has occurred
      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.PECS_DOSE_RANGE_VERSION_VA DROP (HALFLIFEUNITS)';

      EXECUTE IMMEDIATE vc_sql_statement;

      COMMIT;
      sys.DBMS_OUTPUT.put_line ('INFORMATION -   END:  HALF_LIFE_MIGRATION');
      RETURN (0);
   EXCEPTION
      WHEN OTHERS
      THEN
         vn_code := SQLCODE;
         vc_errm := SUBSTR (SQLERRM, 1, 64);
         sys.DBMS_OUTPUT.put_line (
            'HALF_LIFE_MIGRATION: Error code ' || vn_code || ': ' || vc_errm);
         sys.DBMS_OUTPUT.put_line (
            'HALF_LIFE_MIGRATION: last value for vc_sql_statement ');
         sys.DBMS_OUTPUT.put_line (
            'HALF_LIFE_MIGRATION: ' || vc_sql_statement);
         RETURN (1);
   END half_life_migration;

   -----------------------------------------------------------------------------
   -- NAME:         DUPLICATE_ALLOWANCE_MIGRATION
   -- TYPE:         Function
   -- DESCRIPTSION: Create objects that were in DUPLICATE_ALLOWANCE_MIGRATION.sql
   -- INPUTS:       None
   -- OUTPUTS:      Number: (0 - good: >0 - error)
   -----------------------------------------------------------------------------
   FUNCTION duplicate_allowance_migration
      RETURN NUMBER
   IS
   BEGIN
      sys.DBMS_OUTPUT.put_line (CHR (10));
      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   START:  DUPLICATE_ALLOWANCE_MIGRATION');

      -- Modify PECS_DUPL_THERAPY_VERSION_VA to reference duplicate allowance lookup table

      --add new id column for reference
      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.PECS_DUPL_THERAPY_VERSION_VA ADD (CUSTOM_DUP_ALLOWANCE_ID Number(11,0))';

      EXECUTE IMMEDIATE vc_sql_statement;

      --migration existing data
      vc_sql_statement :=
            'UPDATE CTSTAGING.PECS_DUPL_THERAPY_VERSION_VA THERAPY SET CUSTOM_DUP_ALLOWANCE_ID = '
         || '(SELECT duplicate_allowance_id FROM CTSTAGING.DUPLICATE_ALLOWANCE allow '
         || 'where ALLOW.CODE = THERAPY.CUSTOM_DUP_ALLOWANCE) '
         || 'WHERE THERAPY.CUSTOM_DUP_ALLOWANCE_ID IS NULL';

      EXECUTE IMMEDIATE vc_sql_statement;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) updated in ctstaging.pecs_dupl_therapy_version_va : '
         || TO_CHAR (SQL%ROWCOUNT));

      --add foreign key for reference id field
      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.PECS_DUPL_THERAPY_VERSION_VA ADD CONSTRAINT FK_DUPALLOW_DUPLTHERAPY FOREIGN KEY (CUSTOM_DUP_ALLOWANCE_ID) REFERENCES CTSTAGING.DUPLICATE_ALLOWANCE (DUPLICATE_ALLOWANCE_ID)';

      EXECUTE IMMEDIATE vc_sql_statement;

      --add not null constraint to column
      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.PECS_DUPL_THERAPY_VERSION_VA MODIFY (CUSTOM_DUP_ALLOWANCE_ID Number(11,0) NOT NULL)';

      EXECUTE IMMEDIATE vc_sql_statement;

      --drop column once migration has occurred
      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.PECS_DUPL_THERAPY_VERSION_VA DROP  (CUSTOM_DUP_ALLOWANCE)';

      EXECUTE IMMEDIATE vc_sql_statement;

      COMMIT;

      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   END:  DUPLICATE_ALLOWANCE_MIGRATION');
      RETURN (0);
   EXCEPTION
      WHEN OTHERS
      THEN
         vn_code := SQLCODE;
         vc_errm := SUBSTR (SQLERRM, 1, 64);
         sys.DBMS_OUTPUT.put_line (
               'DUPLICATE_ALLOWANCE_MIGRATION: Error code '
            || vn_code
            || ': '
            || vc_errm);
         sys.DBMS_OUTPUT.put_line (
            'DUPLICATE_ALLOWANCE_MIGRATION: last value for vc_sql_statement ');
         sys.DBMS_OUTPUT.put_line (
            'DUPLICATE_ALLOWANCE_MIGRATION: ' || vc_sql_statement);
         RETURN (1);
   END duplicate_allowance_migration;

   -----------------------------------------------------------------------------
   -- NAME:         Drop_DupAllow_Constraint
   -- TYPE:         Function
   -- DESCRIPTSION: Create objects that were in Drop_DupAllow_Constraint.sql
   -- INPUTS:       None
   -- OUTPUTS:      Number: (0 - good: >0 - error)
   -----------------------------------------------------------------------------
   FUNCTION drop_dupallow_constraint
      RETURN NUMBER
   IS
   BEGIN
      sys.DBMS_OUTPUT.put_line (CHR (10));
      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   START:  Drop_DupAllow_Constraint');

      vc_sql_statement :=
         'ALTER TABLE ctstaging.pecs_dupl_therapy_version_va DROP CONSTRAINT fk_dupallow_dupltherapy';

      EXECUTE IMMEDIATE vc_sql_statement;


      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   Successfully dropped fk_dupallow_dupltherapy');

      COMMIT;

      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   END:  Drop_DupAllow_Constraint');
      RETURN (0);
   EXCEPTION
      WHEN OTHERS
      THEN
         vn_code := SQLCODE;
         vc_errm := SUBSTR (SQLERRM, 1, 64);
         sys.DBMS_OUTPUT.put_line (
               'Drop_DupAllow_Constraint: Error code '
            || vn_code
            || ': '
            || vc_errm);
         sys.DBMS_OUTPUT.put_line (
            'Drop_DupAllow_Constraint: last value for vc_sql_statement ');
         sys.DBMS_OUTPUT.put_line (
            'Drop_DupAllow_Constraint: ' || vc_sql_statement);
         RETURN (1);
   END drop_dupallow_constraint;

   -----------------------------------------------------------------------------
   -- NAME:         DUPLICATE_ALLOWANCE_DATA
   -- TYPE:         Function
   -- DESCRIPTSION: Create objects that were in DUPLICATE_ALLOWANCE_DATA.sql
   -- INPUTS:       None
   -- OUTPUTS:      Number: (0 - good: >0 - error)
   -----------------------------------------------------------------------------
   FUNCTION duplicate_allowance_data
      RETURN NUMBER
   IS
   BEGIN
      sys.DBMS_OUTPUT.put_line (CHR (10));
      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   START:  DUPLICATE_ALLOWANCE_DATA');

      vc_sql_statement :=
            'Insert into CTSTAGING.DUPLICATE_ALLOWANCE '
         || '(DUPLICATE_ALLOWANCE_ID, CODE, NAME, DESCRIPTION, SORT_ORDER,  '
         || 'ISACTIVE, EFFECTIVEDATE, EXPIRATIONDATE, RECORD_CREATED_BY, RECORD_CREATED_DATE,  '
         || 'RECORD_MODIFIED_BY, RECORD_MODIFIED_DATE) '
         || 'Values '
         || '(1, 0, ''0'', ''0'', 1,  '
         || '''1'', TO_DATE(''10/04/2011 00:00:00'', ''MM/DD/YYYY HH24:MI:SS''), NULL, ''CCR4795'', TO_DATE(''07/10/2012 16:19:54'', ''MM/DD/YYYY HH24:MI:SS''),  '
         || '''CCR4795'', TO_DATE(''07/10/2012 16:19:54'', ''MM/DD/YYYY HH24:MI:SS''))';

      EXECUTE IMMEDIATE vc_sql_statement;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) inserted into ctstaging.duplicate_allowance : '
         || TO_CHAR (SQL%ROWCOUNT));

      vc_sql_statement :=
            'Insert into CTSTAGING.DUPLICATE_ALLOWANCE '
         || '(DUPLICATE_ALLOWANCE_ID, CODE, NAME, DESCRIPTION, SORT_ORDER,  '
         || 'ISACTIVE, EFFECTIVEDATE, EXPIRATIONDATE, RECORD_CREATED_BY, RECORD_CREATED_DATE,  '
         || 'RECORD_MODIFIED_BY, RECORD_MODIFIED_DATE) '
         || 'Values '
         || '(2, 1, ''1'', ''1'', 1,  '
         || '''1'', TO_DATE(''10/04/2011 00:00:00'', ''MM/DD/YYYY HH24:MI:SS''), NULL, ''CCR4795'', TO_DATE(''07/10/2012 16:19:54'', ''MM/DD/YYYY HH24:MI:SS''),  '
         || '''CCR4795'', TO_DATE(''07/10/2012 16:19:54'', ''MM/DD/YYYY HH24:MI:SS''))';

      EXECUTE IMMEDIATE vc_sql_statement;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) inserted into ctstaging.duplicate_allowance : '
         || TO_CHAR (SQL%ROWCOUNT));

      vc_sql_statement :=
            'Insert into CTSTAGING.DUPLICATE_ALLOWANCE '
         || '(DUPLICATE_ALLOWANCE_ID, CODE, NAME, DESCRIPTION, SORT_ORDER,  '
         || 'ISACTIVE, EFFECTIVEDATE, EXPIRATIONDATE, RECORD_CREATED_BY, RECORD_CREATED_DATE,  '
         || 'RECORD_MODIFIED_BY, RECORD_MODIFIED_DATE) '
         || ' Values '
         || '(3, 2, ''2'', ''2'', 1,  '
         || '''1'', TO_DATE(''10/04/2011 00:00:00'', ''MM/DD/YYYY HH24:MI:SS''), NULL, ''CCR4795'', TO_DATE(''07/10/2012 16:19:54'', ''MM/DD/YYYY HH24:MI:SS''),  '
         || '''CCR4795'', TO_DATE(''07/10/2012 16:19:54'', ''MM/DD/YYYY HH24:MI:SS''))';

      EXECUTE IMMEDIATE vc_sql_statement;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) inserted into ctstaging.duplicate_allowance : '
         || TO_CHAR (SQL%ROWCOUNT));

      vc_sql_statement :=
            'Insert into CTSTAGING.DUPLICATE_ALLOWANCE '
         || '(DUPLICATE_ALLOWANCE_ID, CODE, NAME, DESCRIPTION, SORT_ORDER,  '
         || 'ISACTIVE, EFFECTIVEDATE, EXPIRATIONDATE, RECORD_CREATED_BY, RECORD_CREATED_DATE,  '
         || 'RECORD_MODIFIED_BY, RECORD_MODIFIED_DATE) '
         || 'Values '
         || '(4, 3, ''3'', ''3'', 1,  '
         || '''1'', TO_DATE(''08/26/2013 00:00:00'', ''MM/DD/YYYY HH24:MI:SS''), NULL, ''DUPLALLOW'', TO_DATE(''08/26/2013 00:00:00'', ''MM/DD/YYYY HH24:MI:SS''),  '
         || '''DUPALLOW'', TO_DATE(''08/26/2013 00:00:00'', ''MM/DD/YYYY HH24:MI:SS''))';

      EXECUTE IMMEDIATE vc_sql_statement;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) inserted into ctstaging.duplicate_allowance : '
         || TO_CHAR (SQL%ROWCOUNT));

      vc_sql_statement :=
            'Insert into CTSTAGING.DUPLICATE_ALLOWANCE '
         || '(DUPLICATE_ALLOWANCE_ID, CODE, NAME, DESCRIPTION, SORT_ORDER,  '
         || 'ISACTIVE, EFFECTIVEDATE, EXPIRATIONDATE, RECORD_CREATED_BY, RECORD_CREATED_DATE,  '
         || 'RECORD_MODIFIED_BY, RECORD_MODIFIED_DATE) '
         || 'Values '
         || '(5, 4, ''4'', ''4'', 1,  '
         || '''1'', TO_DATE(''08/26/2013 00:00:00'', ''MM/DD/YYYY HH24:MI:SS''), NULL, ''DUPLALLOW'', TO_DATE(''08/26/2013 00:00:00'', ''MM/DD/YYYY HH24:MI:SS''),  '
         || '''DUPALLOW'', TO_DATE(''08/26/2013 00:00:00'', ''MM/DD/YYYY HH24:MI:SS''))';

      EXECUTE IMMEDIATE vc_sql_statement;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) inserted into ctstaging.duplicate_allowance : '
         || TO_CHAR (SQL%ROWCOUNT));

      COMMIT;

      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   END:  DUPLICATE_ALLOWANCE_DATA');
      RETURN (0);
   EXCEPTION
      WHEN OTHERS
      THEN
         vn_code := SQLCODE;
         vc_errm := SUBSTR (SQLERRM, 1, 64);
         sys.DBMS_OUTPUT.put_line (
               'DUPLICATE_ALLOWANCE_DATA: Error code '
            || vn_code
            || ': '
            || vc_errm);
         sys.DBMS_OUTPUT.put_line (
            'DUPLICATE_ALLOWANCE_DATA: last value for vc_sql_statement ');
         sys.DBMS_OUTPUT.put_line (
            'DUPLICATE_ALLOWANCE_DATA: ' || vc_sql_statement);
         RETURN (1);
   END duplicate_allowance_data;


   -----------------------------------------------------------------------------
   -- NAME:         CONVERT_CTISSUEDATE_DATE
   -- TYPE:         Function
   -- DESCRIPTSION: Create objects that were in CONVERT_CTISSUEDATE_DATE.sql
   -- INPUTS:       None
   -- OUTPUTS:      Number: (0 - good: >0 - error)
   -----------------------------------------------------------------------------
   FUNCTION convert_ctissuedate_date
      RETURN NUMBER
   IS
   BEGIN
      sys.DBMS_OUTPUT.put_line (CHR (10));
      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   START:  CONVERT_CTISSUEDATE_DATE');

      --add NEW column
      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.CT_VERSION_VA ADD (CT_ISSUEDATE_DT TIMESTAMP(6))';

      EXECUTE IMMEDIATE vc_sql_statement;

      -- set TIMESTAMP based on value in old field
      vc_sql_statement :=
         'UPDATE CTSTAGING.CT_VERSION_VA SET ct_issuedate_dt = TO_DATE (ct_issuedate, ''yyyymmdd'')';

      EXECUTE IMMEDIATE vc_sql_statement;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) updated in ctstaging.ct_version_va : '
         || TO_CHAR (SQL%ROWCOUNT));

      --drop column once migration has occurred
      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.CT_VERSION_VA DROP COLUMN ct_issuedate';

      EXECUTE IMMEDIATE vc_sql_statement;

      --rename new column to match old
      vc_sql_statement :=
         'ALTER TABLE CTSTAGING.CT_VERSION_VA RENAME COLUMN ct_issuedate_dt TO ct_issuedate';

      EXECUTE IMMEDIATE vc_sql_statement;

      COMMIT;

      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   END:  CONVERT_CTISSUEDATE_DATE');
      RETURN (0);
   EXCEPTION
      WHEN OTHERS
      THEN
         vn_code := SQLCODE;
         vc_errm := SUBSTR (SQLERRM, 1, 64);
         sys.DBMS_OUTPUT.put_line (
               'CONVERT_CTISSUEDATE_DATE: Error code '
            || vn_code
            || ': '
            || vc_errm);
         sys.DBMS_OUTPUT.put_line (
            'CONVERT_CTISSUEDATE_DATE: last value for vc_sql_statement ');
         sys.DBMS_OUTPUT.put_line (
            'CONVERT_CTISSUEDATE_DATE: ' || vc_sql_statement);
         RETURN (1);
   END convert_ctissuedate_date;

   -----------------------------------------------------------------------------
   -- NAME:         DDI_ACTION_DATE_FIELD_METADATS
   -- TYPE:         Function
   -- DESCRIPTSION: Create objects that were in DDI_ACTION_DATE_FIELD_METADATS.sql
   -- INPUTS:       None
   -- OUTPUTS:      Number: (0 - good: >0 - error)
   -----------------------------------------------------------------------------
   FUNCTION ddi_action_date_field_metadats
      RETURN NUMBER
   IS
      vn_maxval   NUMBER;
   BEGIN
      sys.DBMS_OUTPUT.put_line (CHR (10));
      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   START:  DDI_ACTION_DATE_FIELD_METADATS');

      BEGIN
         SELECT MAX (display_order) + 1 AS maxval
           INTO vn_maxval
           FROM ctstaging.field_metadata
          WHERE concept_name = 'DrugDrugInteractionDTO';
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            vn_maxval := 1;
      END;

      IF vn_maxval IS NULL
      THEN
         vn_maxval := 1;
      END IF;

      vc_sql_statement :=
            'INSERT INTO ctstaging.field_metadata (field_id '
         || ',concept_name '
         || ',field_name '
         || ',display_order '
         || ',is_query '
         || ',is_report '
         || ',is_detail '
         || ',display_name) '
         || 'SELECT ctstaging.fieldmeta_seq.NEXTVAL '
         || ',''DrugDrugInteractionDTO'' '
         || ',''ACTION_DATE_DISPLAY_STRING'' '
         || ',:newmaxval '
         || ',''Y'' '
         || ',''Y'' '
         || ',''Y'' '
         || ',''Action Date'' '
         || 'FROM DUAL';


      BEGIN
         EXECUTE IMMEDIATE vc_sql_statement USING vn_maxval;
      EXCEPTION
         WHEN OTHERS
         THEN
            vn_code := SQLCODE;
            vc_errm := SUBSTR (SQLERRM, 1, 64);
            sys.DBMS_OUTPUT.put_line (
                  'DDI_ACTION_DATE_FIELD_METADATS: Error code '
               || vn_code
               || ': '
               || vc_errm);
            sys.DBMS_OUTPUT.put_line (
                  'DDI_ACTION_DATE_FIELD_METADATS: vn_maxval='
               || vn_maxval
               || '--');
            sys.DBMS_OUTPUT.put_line (
               'DDI_ACTION_DATE_FIELD_METADATS: ' || vc_sql_statement);
      END;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) inserted into ctstaging.field_metadata : '
         || TO_CHAR (SQL%ROWCOUNT));

      COMMIT;

      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   END:  DDI_ACTION_DATE_FIELD_METADATS');
      RETURN (0);
   EXCEPTION
      WHEN OTHERS
      THEN
         vn_code := SQLCODE;
         vc_errm := SUBSTR (SQLERRM, 1, 64);
         sys.DBMS_OUTPUT.put_line (
               'DDI_ACTION_DATE_FIELD_METADATS: Error code '
            || vn_code
            || ': '
            || vc_errm);
         sys.DBMS_OUTPUT.put_line (
            'DDI_ACTION_DATE_FIELD_METADATS: last value for vc_sql_statement ');
         sys.DBMS_OUTPUT.put_line (
            'DDI_ACTION_DATE_FIELD_METADATS: ' || vc_sql_statement);
         RETURN (1);
   END ddi_action_date_field_metadats;

   -----------------------------------------------------------------------------
   -- NAME:         DDI_EXPORT_DATE_FIELD_METADATS
   -- TYPE:         Function
   -- DESCRIPTSION: Create objects that were in DDI_EXPORT_DATE_FIELD_METADATS.sql
   -- INPUTS:       None
   -- OUTPUTS:      Number: (0 - good: >0 - error)
   -----------------------------------------------------------------------------
   FUNCTION ddi_export_date_field_metadats
      RETURN NUMBER
   IS
      vn_maxval   NUMBER;
   BEGIN
      sys.DBMS_OUTPUT.put_line (CHR (10));
      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   START:  DDI_EXPORT_DATE_FIELD_METADATS');

      BEGIN
         SELECT MAX (display_order) + 1 AS maxval
           INTO vn_maxval
           FROM ctstaging.field_metadata
          WHERE concept_name = 'DrugDrugInteractionDTO';
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            vn_maxval := 1;
      END;

      IF vn_maxval IS NULL
      THEN
         vn_maxval := 1;
      END IF;

      vc_sql_statement :=
            'INSERT INTO ctstaging.field_metadata (field_id '
         || ',concept_name '
         || ',field_name '
         || ',display_order '
         || ',is_query '
         || ',is_report '
         || ',is_detail '
         || ',display_name) '
         || 'SELECT ctstaging.fieldmeta_seq.NEXTVAL '
         || ',''DrugDrugInteractionDTO'' '
         || ',''EXPORT_DATE_DISPLAY_STRING'' '
         || ',:newmaxval '
         || ',''Y'' '
         || ',''Y'' '
         || ',''Y'' '
         || ',''Export Date'' '
         || 'FROM DUAL';


      BEGIN
         EXECUTE IMMEDIATE vc_sql_statement USING vn_maxval;
      EXCEPTION
         WHEN OTHERS
         THEN
            vn_code := SQLCODE;
            vc_errm := SUBSTR (SQLERRM, 1, 64);
            sys.DBMS_OUTPUT.put_line (
                  'DDI_EXPORT_DATE_FIELD_METADATS: Error code '
               || vn_code
               || ': '
               || vc_errm);
            sys.DBMS_OUTPUT.put_line (
               'DDI_EXPORT_DATE_FIELD_METADATS: ' || vc_sql_statement);
      END;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) inserted into ctstaging.field_metadata : '
         || TO_CHAR (SQL%ROWCOUNT));

      COMMIT;

      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   END:  DDI_EXPORT_DATE_FIELD_METADATS');
      RETURN (0);
   EXCEPTION
      WHEN OTHERS
      THEN
         vn_code := SQLCODE;
         vc_errm := SUBSTR (SQLERRM, 1, 64);
         sys.DBMS_OUTPUT.put_line (
               'DDI_EXPORT_DATE_FIELD_METADATS: Error code '
            || vn_code
            || ': '
            || vc_errm);
         sys.DBMS_OUTPUT.put_line (
            'DDI_EXPORT_DATE_FIELD_METADATS: last value for vc_sql_statement ');
         sys.DBMS_OUTPUT.put_line (
            'DDI_EXPORT_DATE_FIELD_METADATS: ' || vc_sql_statement);
         RETURN (1);
   END ddi_export_date_field_metadats;

   -----------------------------------------------------------------------------
   -- NAME:         DP_ACTION_DATE_FIELD_METADATAS
   -- TYPE:         Function
   -- DESCRIPTSION: Create objects that were in DP_ACTION_DATE_FIELD_METADATAS.sql
   -- INPUTS:       None
   -- OUTPUTS:      Number: (0 - good: >0 - error)
   -----------------------------------------------------------------------------
   FUNCTION dp_action_date_field_metadatas
      RETURN NUMBER
   IS
      vn_maxval   NUMBER;
   BEGIN
      sys.DBMS_OUTPUT.put_line (CHR (10));
      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   START:  DP_ACTION_DATE_FIELD_METADATAS');

      BEGIN
         SELECT MAX (display_order) + 1 AS maxval
           INTO vn_maxval
           FROM ctstaging.field_metadata
          WHERE concept_name = 'DrugPairDTO';
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            vn_maxval := 1;
      END;

      IF vn_maxval IS NULL
      THEN
         vn_maxval := 1;
      END IF;

      vc_sql_statement :=
            'INSERT INTO ctstaging.field_metadata (field_id '
         || ',concept_name '
         || ',field_name '
         || ',display_order '
         || ',is_query '
         || ',is_report '
         || ',is_detail '
         || ',display_name) '
         || 'SELECT ctstaging.fieldmeta_seq.NEXTVAL '
         || ',''DrugPairDTO'' '
         || ',''ACTION_DATE_DISPLAY_STRING'' '
         || ',:newmaxval '
         || ',''Y'' '
         || ',''Y'' '
         || ',''Y'' '
         || ',''Action Date'' '
         || 'FROM DUAL';


      BEGIN
         EXECUTE IMMEDIATE vc_sql_statement USING vn_maxval;
      EXCEPTION
         WHEN OTHERS
         THEN
            vn_code := SQLCODE;
            vc_errm := SUBSTR (SQLERRM, 1, 64);
            sys.DBMS_OUTPUT.put_line (
                  'DP_ACTION_DATE_FIELD_METADATAS: Error code '
               || vn_code
               || ': '
               || vc_errm);
            sys.DBMS_OUTPUT.put_line (
               'DP_ACTION_DATE_FIELD_METADATAS: ' || vc_sql_statement);
      END;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) inserted into ctstaging.field_metadata : '
         || TO_CHAR (SQL%ROWCOUNT));

      COMMIT;

      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   END:  DP_ACTION_DATE_FIELD_METADATAS');
      RETURN (0);
   EXCEPTION
      WHEN OTHERS
      THEN
         vn_code := SQLCODE;
         vc_errm := SUBSTR (SQLERRM, 1, 64);
         sys.DBMS_OUTPUT.put_line (
               'DP_ACTION_DATE_FIELD_METADATAS: Error code '
            || vn_code
            || ': '
            || vc_errm);
         sys.DBMS_OUTPUT.put_line (
            'DP_ACTION_DATE_FIELD_METADATAS: last value for vc_sql_statement ');
         sys.DBMS_OUTPUT.put_line (
            'DP_ACTION_DATE_FIELD_METADATAS: ' || vc_sql_statement);
         RETURN (1);
   END dp_action_date_field_metadatas;

   -----------------------------------------------------------------------------
   -- NAME:         DP_EXPORT_DATE_FIELD_METADATAS
   -- TYPE:         Function
   -- DESCRIPTSION: Create objects that were in DP_EXPORT_DATE_FIELD_METADATAS.sql
   -- INPUTS:       None
   -- OUTPUTS:      Number: (0 - good: >0 - error)
   -----------------------------------------------------------------------------
   FUNCTION dp_export_date_field_metadatas
      RETURN NUMBER
   IS
      vn_maxval   NUMBER;
   BEGIN
      sys.DBMS_OUTPUT.put_line (CHR (10));
      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   START:  DP_EXPORT_DATE_FIELD_METADATAS');

      BEGIN
         SELECT MAX (display_order) + 1 AS maxval
           INTO vn_maxval
           FROM ctstaging.field_metadata
          WHERE concept_name = 'DrugPairDTO';
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            vn_maxval := 1;
      END;

      IF vn_maxval IS NULL
      THEN
         vn_maxval := 1;
      END IF;

      vc_sql_statement :=
            'INSERT INTO ctstaging.field_metadata (field_id '
         || ',concept_name '
         || ',field_name '
         || ',display_order '
         || ',is_query '
         || ',is_report '
         || ',is_detail '
         || ',display_name) '
         || 'SELECT ctstaging.fieldmeta_seq.NEXTVAL '
         || ',''DrugPairDTO'' '
         || ',''EXPORT_DATE_DISPLAY_STRING'' '
         || ',:newmaxval '
         || ',''Y'' '
         || ',''Y'' '
         || ',''Y'' '
         || ',''Export Date'' '
         || 'FROM DUAL';


      BEGIN
         EXECUTE IMMEDIATE vc_sql_statement USING vn_maxval;
      EXCEPTION
         WHEN OTHERS
         THEN
            vn_code := SQLCODE;
            vc_errm := SUBSTR (SQLERRM, 1, 64);
            sys.DBMS_OUTPUT.put_line (
                  'DP_EXPORT_DATE_FIELD_METADATAS: Error code '
               || vn_code
               || ': '
               || vc_errm);
            sys.DBMS_OUTPUT.put_line (
               'DP_EXPORT_DATE_FIELD_METADATAS: ' || vc_sql_statement);
      END;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) inserted into ctstaging.field_metadata : '
         || TO_CHAR (SQL%ROWCOUNT));

      COMMIT;

      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   END:  DP_EXPORT_DATE_FIELD_METADATAS');
      RETURN (0);
   EXCEPTION
      WHEN OTHERS
      THEN
         vn_code := SQLCODE;
         vc_errm := SUBSTR (SQLERRM, 1, 64);
         sys.DBMS_OUTPUT.put_line (
               'DP_EXPORT_DATE_FIELD_METADATAS: Error code '
            || vn_code
            || ': '
            || vc_errm);
         sys.DBMS_OUTPUT.put_line (
            'DP_EXPORT_DATE_FIELD_METADATAS: last value for vc_sql_statement ');
         sys.DBMS_OUTPUT.put_line (
            'DP_EXPORT_DATE_FIELD_METADATAS: ' || vc_sql_statement);
         RETURN (1);
   END dp_export_date_field_metadatas;

   -----------------------------------------------------------------------------
   -- NAME:         DR_ACTION_DATE_FIELD_METADATAS
   -- TYPE:         Function
   -- DESCRIPTSION: Create objects that were in DR_ACTION_DATE_FIELD_METADATAS.sql
   -- INPUTS:       None
   -- OUTPUTS:      Number: (0 - good: >0 - error)
   -----------------------------------------------------------------------------
   FUNCTION dr_action_date_field_metadatas
      RETURN NUMBER
   IS
      vn_maxval   NUMBER;
   BEGIN
      sys.DBMS_OUTPUT.put_line (CHR (10));
      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   START:  DR_ACTION_DATE_FIELD_METADATAS');

      BEGIN
         SELECT MAX (display_order) + 1 AS maxval
           INTO vn_maxval
           FROM ctstaging.field_metadata
          WHERE concept_name = 'DoseRangeDTO';
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            vn_maxval := 1;
      END;

      IF vn_maxval IS NULL
      THEN
         vn_maxval := 1;
      END IF;

      vc_sql_statement :=
            'INSERT INTO ctstaging.field_metadata (field_id '
         || ',concept_name '
         || ',field_name '
         || ',display_order '
         || ',is_query '
         || ',is_report '
         || ',is_detail '
         || ',display_name) '
         || 'SELECT ctstaging.fieldmeta_seq.NEXTVAL '
         || ',''DoseRangeDTO'' '
         || ',''ACTION_DATE_DISPLAY_STRING'' '
         || ',:newmaxval '
         || ',''Y'' '
         || ',''Y'' '
         || ',''Y'' '
         || ',''Action Date'' '
         || 'FROM DUAL';


      BEGIN
         EXECUTE IMMEDIATE vc_sql_statement USING vn_maxval;
      EXCEPTION
         WHEN OTHERS
         THEN
            vn_code := SQLCODE;
            vc_errm := SUBSTR (SQLERRM, 1, 64);
            sys.DBMS_OUTPUT.put_line (
                  'DR_ACTION_DATE_FIELD_METADATAS: Error code '
               || vn_code
               || ': '
               || vc_errm);
            sys.DBMS_OUTPUT.put_line (
               'DR_ACTION_DATE_FIELD_METADATAS: ' || vc_sql_statement);
      END;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) inserted into ctstaging.field_metadata : '
         || TO_CHAR (SQL%ROWCOUNT));

      COMMIT;

      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   END:  DR_ACTION_DATE_FIELD_METADATAS');
      RETURN (0);
   EXCEPTION
      WHEN OTHERS
      THEN
         vn_code := SQLCODE;
         vc_errm := SUBSTR (SQLERRM, 1, 64);
         sys.DBMS_OUTPUT.put_line (
               'DR_ACTION_DATE_FIELD_METADATAS: Error code '
            || vn_code
            || ': '
            || vc_errm);
         sys.DBMS_OUTPUT.put_line (
            'DR_ACTION_DATE_FIELD_METADATAS: last value for vc_sql_statement ');
         sys.DBMS_OUTPUT.put_line (
            'DR_ACTION_DATE_FIELD_METADATAS: ' || vc_sql_statement);
         RETURN (1);
   END dr_action_date_field_metadatas;

   -----------------------------------------------------------------------------
   -- NAME:         DR_EXPORT_DATE_FIELD_METADATAS
   -- TYPE:         Function
   -- DESCRIPTSION: Create objects that were in DR_EXPORT_DATE_FIELD_METADATAS.sql
   -- INPUTS:       None
   -- OUTPUTS:      Number: (0 - good: >0 - error)
   -----------------------------------------------------------------------------
   FUNCTION dr_export_date_field_metadatas
      RETURN NUMBER
   IS
      vn_maxval   NUMBER;
   BEGIN
      sys.DBMS_OUTPUT.put_line (CHR (10));
      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   START:  DR_EXPORT_DATE_FIELD_METADATAS');

      BEGIN
         SELECT MAX (display_order) + 1 AS maxval
           INTO vn_maxval
           FROM ctstaging.field_metadata
          WHERE concept_name = 'DoseRangeDTO';
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            vn_maxval := 1;
      END;

      IF vn_maxval IS NULL
      THEN
         vn_maxval := 1;
      END IF;

      vc_sql_statement :=
            'INSERT INTO ctstaging.field_metadata (field_id '
         || ',concept_name '
         || ',field_name '
         || ',display_order '
         || ',is_query '
         || ',is_report '
         || ',is_detail '
         || ',display_name) '
         || 'SELECT ctstaging.fieldmeta_seq.NEXTVAL '
         || ',''DoseRangeDTO'' '
         || ',''EXPORT_DATE_DISPLAY_STRING'' '
         || ',:newmaxval '
         || ',''Y'' '
         || ',''Y'' '
         || ',''Y'' '
         || ',''Export Date'' '
         || 'FROM DUAL';


      BEGIN
         EXECUTE IMMEDIATE vc_sql_statement USING vn_maxval;
      EXCEPTION
         WHEN OTHERS
         THEN
            vn_code := SQLCODE;
            vc_errm := SUBSTR (SQLERRM, 1, 64);
            sys.DBMS_OUTPUT.put_line (
                  'DR_EXPORT_DATE_FIELD_METADATAS: Error code '
               || vn_code
               || ': '
               || vc_errm);
            sys.DBMS_OUTPUT.put_line (
               'DR_EXPORT_DATE_FIELD_METADATAS: ' || vc_sql_statement);
      END;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) inserted into ctstaging.field_metadata : '
         || TO_CHAR (SQL%ROWCOUNT));

      COMMIT;

      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   END:  DR_EXPORT_DATE_FIELD_METADATAS');
      RETURN (0);
   EXCEPTION
      WHEN OTHERS
      THEN
         vn_code := SQLCODE;
         vc_errm := SUBSTR (SQLERRM, 1, 64);
         sys.DBMS_OUTPUT.put_line (
               'DR_EXPORT_DATE_FIELD_METADATAS: Error code '
            || vn_code
            || ': '
            || vc_errm);
         sys.DBMS_OUTPUT.put_line (
            'DR_EXPORT_DATE_FIELD_METADATAS: last value for vc_sql_statement ');
         sys.DBMS_OUTPUT.put_line (
            'DR_EXPORT_DATE_FIELD_METADATAS: ' || vc_sql_statement);
         RETURN (1);
   END dr_export_date_field_metadatas;

   -----------------------------------------------------------------------------
   -- NAME:         DT_ACTION_DATE_FIELD_METADATAS
   -- TYPE:         Function
   -- DESCRIPTSION: Create objects that were in DT_ACTION_DATE_FIELD_METADATAS.sql
   -- INPUTS:       None
   -- OUTPUTS:      Number: (0 - good: >0 - error)
   -----------------------------------------------------------------------------
   FUNCTION dt_action_date_field_metadatas
      RETURN NUMBER
   IS
      vn_maxval   NUMBER;
   BEGIN
      sys.DBMS_OUTPUT.put_line (CHR (10));
      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   START:  DT_ACTION_DATE_FIELD_METADATAS');

      BEGIN
         SELECT MAX (display_order) + 1 AS maxval
           INTO vn_maxval
           FROM ctstaging.field_metadata
          WHERE concept_name = 'DuplicateTherapyDTO';
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            vn_maxval := 1;
      END;

      IF vn_maxval IS NULL
      THEN
         vn_maxval := 1;
      END IF;

      vc_sql_statement :=
            'INSERT INTO ctstaging.field_metadata (field_id '
         || ',concept_name '
         || ',field_name '
         || ',display_order '
         || ',is_query '
         || ',is_report '
         || ',is_detail '
         || ',display_name) '
         || 'SELECT ctstaging.fieldmeta_seq.NEXTVAL '
         || ',''DuplicateTherapyDTO'' '
         || ',''ACTION_DATE_DISPLAY_STRING'' '
         || ',:newmaxval '
         || ',''Y'' '
         || ',''Y'' '
         || ',''Y'' '
         || ',''Action Date'' '
         || 'FROM DUAL';


      BEGIN
         EXECUTE IMMEDIATE vc_sql_statement USING vn_maxval;
      EXCEPTION
         WHEN OTHERS
         THEN
            vn_code := SQLCODE;
            vc_errm := SUBSTR (SQLERRM, 1, 64);
            sys.DBMS_OUTPUT.put_line (
                  'DT_ACTION_DATE_FIELD_METADATAS: Error code '
               || vn_code
               || ': '
               || vc_errm);
            sys.DBMS_OUTPUT.put_line (
               'DT_ACTION_DATE_FIELD_METADATAS: ' || vc_sql_statement);
      END;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) inserted into ctstaging.field_metadata : '
         || TO_CHAR (SQL%ROWCOUNT));

      COMMIT;

      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   END:  DT_ACTION_DATE_FIELD_METADATAS');
      RETURN (0);
   EXCEPTION
      WHEN OTHERS
      THEN
         vn_code := SQLCODE;
         vc_errm := SUBSTR (SQLERRM, 1, 64);
         sys.DBMS_OUTPUT.put_line (
               'DT_ACTION_DATE_FIELD_METADATAS: Error code '
            || vn_code
            || ': '
            || vc_errm);
         sys.DBMS_OUTPUT.put_line (
            'DT_ACTION_DATE_FIELD_METADATAS: last value for vc_sql_statement ');
         sys.DBMS_OUTPUT.put_line (
            'DT_ACTION_DATE_FIELD_METADATAS: ' || vc_sql_statement);
         RETURN (1);
   END dt_action_date_field_metadatas;

   -----------------------------------------------------------------------------
   -- NAME:         DT_EXPORT_DATE_FIELD_METADATAS
   -- TYPE:         Function
   -- DESCRIPTSION: Create objects that were in DT_EXPORT_DATE_FIELD_METADATAS.sql
   -- INPUTS:       None
   -- OUTPUTS:      Number: (0 - good: >0 - error)
   -----------------------------------------------------------------------------
   FUNCTION dt_export_date_field_metadatas
      RETURN NUMBER
   IS
      vn_maxval   NUMBER;
   BEGIN
      sys.DBMS_OUTPUT.put_line (CHR (10));
      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   START:  DT_EXPORT_DATE_FIELD_METADATAS');

      BEGIN
         SELECT MAX (display_order) + 1 AS maxval
           INTO vn_maxval
           FROM ctstaging.field_metadata
          WHERE concept_name = 'DuplicateTherapyDTO';
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            vn_maxval := 1;
      END;

      IF vn_maxval IS NULL
      THEN
         vn_maxval := 1;
      END IF;

      vc_sql_statement :=
            'INSERT INTO ctstaging.field_metadata (field_id '
         || ',concept_name '
         || ',field_name '
         || ',display_order '
         || ',is_query '
         || ',is_report '
         || ',is_detail '
         || ',display_name) '
         || 'SELECT ctstaging.fieldmeta_seq.NEXTVAL '
         || ',''DuplicateTherapyDTO'' '
         || ',''EXPORT_DATE_DISPLAY_STRING'' '
         || ',:newmaxval '
         || ',''Y'' '
         || ',''Y'' '
         || ',''Y'' '
         || ',''Export Date'' '
         || 'FROM DUAL';


      BEGIN
         EXECUTE IMMEDIATE vc_sql_statement USING vn_maxval;
      EXCEPTION
         WHEN OTHERS
         THEN
            vn_code := SQLCODE;
            vc_errm := SUBSTR (SQLERRM, 1, 64);
            sys.DBMS_OUTPUT.put_line (
                  'DT_EXPORT_DATE_FIELD_METADATAS: Error code '
               || vn_code
               || ': '
               || vc_errm);
            sys.DBMS_OUTPUT.put_line (
               'DT_EXPORT_DATE_FIELD_METADATAS: ' || vc_sql_statement);
      END;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) inserted into ctstaging.field_metadata : '
         || TO_CHAR (SQL%ROWCOUNT));

      COMMIT;

      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   END:  DT_EXPORT_DATE_FIELD_METADATAS');
      RETURN (0);
   EXCEPTION
      WHEN OTHERS
      THEN
         vn_code := SQLCODE;
         vc_errm := SUBSTR (SQLERRM, 1, 64);
         sys.DBMS_OUTPUT.put_line (
               'DT_EXPORT_DATE_FIELD_METADATAS: Error code '
            || vn_code
            || ': '
            || vc_errm);
         sys.DBMS_OUTPUT.put_line (
            'DT_EXPORT_DATE_FIELD_METADATAS: last value for vc_sql_statement ');
         sys.DBMS_OUTPUT.put_line (
            'DT_EXPORT_DATE_FIELD_METADATAS: ' || vc_sql_statement);
         RETURN (1);
   END dt_export_date_field_metadatas;

   -----------------------------------------------------------------------------
   -- NAME:         PM_ACTION_DATE_FIELD_METADATAS
   -- TYPE:         Function
   -- DESCRIPTSION: Create objects that were in PM_ACTION_DATE_FIELD_METADATAS.sql
   -- INPUTS:       None
   -- OUTPUTS:      Number: (0 - good: >0 - error)
   -----------------------------------------------------------------------------
   FUNCTION pm_action_date_field_metadatas
      RETURN NUMBER
   IS
      vn_maxval   NUMBER;
   BEGIN
      sys.DBMS_OUTPUT.put_line (CHR (10));
      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   START:  PM_ACTION_DATE_FIELD_METADATAS');

      BEGIN
         SELECT MAX (display_order) + 1 AS maxval
           INTO vn_maxval
           FROM ctstaging.field_metadata
          WHERE concept_name = 'MonographDTO';
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            vn_maxval := 1;
      END;

      IF vn_maxval IS NULL
      THEN
         vn_maxval := 1;
      END IF;

      vc_sql_statement :=
            'INSERT INTO ctstaging.field_metadata (field_id '
         || ',concept_name '
         || ',field_name '
         || ',display_order '
         || ',is_query '
         || ',is_report '
         || ',is_detail '
         || ',display_name) '
         || 'SELECT ctstaging.fieldmeta_seq.NEXTVAL '
         || ',''MonographDTO'' '
         || ',''ACTION_DATE_DISPLAY_STRING'' '
         || ',:newmaxval '
         || ',''Y'' '
         || ',''Y'' '
         || ',''Y'' '
         || ',''Action Date'' '
         || 'FROM DUAL';


      BEGIN
         EXECUTE IMMEDIATE vc_sql_statement USING vn_maxval;
      EXCEPTION
         WHEN OTHERS
         THEN
            vn_code := SQLCODE;
            vc_errm := SUBSTR (SQLERRM, 1, 64);
            sys.DBMS_OUTPUT.put_line (
                  'PM_ACTION_DATE_FIELD_METADATAS: Error code '
               || vn_code
               || ': '
               || vc_errm);
            sys.DBMS_OUTPUT.put_line (
               'PM_ACTION_DATE_FIELD_METADATAS: ' || vc_sql_statement);
      END;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) inserted into ctstaging.field_metadata : '
         || TO_CHAR (SQL%ROWCOUNT));

      COMMIT;

      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   END:  PM_ACTION_DATE_FIELD_METADATAS');
      RETURN (0);
   EXCEPTION
      WHEN OTHERS
      THEN
         vn_code := SQLCODE;
         vc_errm := SUBSTR (SQLERRM, 1, 64);
         sys.DBMS_OUTPUT.put_line (
               'PM_ACTION_DATE_FIELD_METADATAS: Error code '
            || vn_code
            || ': '
            || vc_errm);
         sys.DBMS_OUTPUT.put_line (
            'PM_ACTION_DATE_FIELD_METADATAS: last value for vc_sql_statement ');
         sys.DBMS_OUTPUT.put_line (
            'PM_ACTION_DATE_FIELD_METADATAS: ' || vc_sql_statement);
         RETURN (1);
   END pm_action_date_field_metadatas;

   -----------------------------------------------------------------------------
   -- NAME:         PM_EXPORT_DATE_FIELD_METADATAS
   -- TYPE:         Function
   -- DESCRIPTSION: Create objects that were in PM_EXPORT_DATE_FIELD_METADATAS.sql
   -- INPUTS:       None
   -- OUTPUTS:      Number: (0 - good: >0 - error)
   -----------------------------------------------------------------------------
   FUNCTION pm_export_date_field_metadatas
      RETURN NUMBER
   IS
      vn_maxval   NUMBER;
   BEGIN
      sys.DBMS_OUTPUT.put_line (CHR (10));
      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   START:  PM_EXPORT_DATE_FIELD_METADATAS');

      BEGIN
         SELECT MAX (display_order) + 1 AS maxval
           INTO vn_maxval
           FROM ctstaging.field_metadata
          WHERE concept_name = 'MonographDTO';
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            vn_maxval := 1;
      END;

      IF vn_maxval IS NULL
      THEN
         vn_maxval := 1;
      END IF;

      vc_sql_statement :=
            'INSERT INTO ctstaging.field_metadata (field_id '
         || ',concept_name '
         || ',field_name '
         || ',display_order '
         || ',is_query '
         || ',is_report '
         || ',is_detail '
         || ',display_name) '
         || 'SELECT ctstaging.fieldmeta_seq.NEXTVAL '
         || ',''MonographDTO'' '
         || ',''EXPORT_DATE_DISPLAY_STRING'' '
         || ',:newmaxval '
         || ',''Y'' '
         || ',''Y'' '
         || ',''Y'' '
         || ',''Export Date'' '
         || 'FROM DUAL';


      BEGIN
         EXECUTE IMMEDIATE vc_sql_statement USING vn_maxval;
      EXCEPTION
         WHEN OTHERS
         THEN
            vn_code := SQLCODE;
            vc_errm := SUBSTR (SQLERRM, 1, 64);
            sys.DBMS_OUTPUT.put_line (
                  'PM_EXPORT_DATE_FIELD_METADATAS: Error code '
               || vn_code
               || ': '
               || vc_errm);
            sys.DBMS_OUTPUT.put_line (
               'PM_EXPORT_DATE_FIELD_METADATAS: ' || vc_sql_statement);
      END;

      sys.DBMS_OUTPUT.put_line (
            'INFORMATION -   Number of row(s) inserted into ctstaging.field_metadata : '
         || TO_CHAR (SQL%ROWCOUNT));

      COMMIT;

      sys.DBMS_OUTPUT.put_line (
         'INFORMATION -   END:  PM_EXPORT_DATE_FIELD_METADATAS');
      RETURN (0);
   EXCEPTION
      WHEN OTHERS
      THEN
         vn_code := SQLCODE;
         vc_errm := SUBSTR (SQLERRM, 1, 64);
         sys.DBMS_OUTPUT.put_line (
               'PM_EXPORT_DATE_FIELD_METADATAS: Error code '
            || vn_code
            || ': '
            || vc_errm);
         sys.DBMS_OUTPUT.put_line (
            'PM_EXPORT_DATE_FIELD_METADATAS: last value for vc_sql_statement ');
         sys.DBMS_OUTPUT.put_line (
            'PM_EXPORT_DATE_FIELD_METADATAS: ' || vc_sql_statement);
         RETURN (1);
   END pm_export_date_field_metadatas;
-----------------------------------------------------------------------------
-- MAIN SECTION
-----------------------------------------------------------------------------
BEGIN
   sys.DBMS_OUTPUT.put_line (CHR (10));
   sys.DBMS_OUTPUT.put_line (
      'INFORMATION -   Checking to see if PECS 5 table(s) already exists.');

   vn_continue := check_for_pecs5;

   IF vn_continue > 0
   THEN
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -     ONE OR MORE PECS 5 TABLES ALREADY EXISTS. STOPPING MIGRATION.');
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -   EXECUTE PECS 5 ROLLBACK PROCESS');
      sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
      RETURN;
   END IF;

   vn_continue := create_fdb_comments_revised;

   IF vn_continue > 0
   THEN
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -   EXECUTE PECS 5 ROLLBACK PROCESS');
      sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
      RETURN;
   END IF;

   vn_continue := ddi_crossreference_migration;

   IF vn_continue > 0
   THEN
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -   EXECUTE PECS 5 ROLLBACK PROCESS');
      sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
      RETURN;
   END IF;

   vn_continue := create_ddi_xref_link;

   IF vn_continue > 0
   THEN
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -   EXECUTE PECS 5 ROLLBACK PROCESS');
      sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
      RETURN;
   END IF;

   vn_continue := doserange_crossreference_migra;

   IF vn_continue > 0
   THEN
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -   EXECUTE PECS 5 ROLLBACK PROCESS');
      sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
      RETURN;
   END IF;

   vn_continue := create_doserange_xref_link;

   IF vn_continue > 0
   THEN
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -   EXECUTE PECS 5 ROLLBACK PROCESS');
      sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
      RETURN;
   END IF;

   vn_continue := dt_crossreference_migration;

   IF vn_continue > 0
   THEN
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -   EXECUTE PECS 5 ROLLBACK PROCESS');
      sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
      RETURN;
   END IF;

   vn_continue := create_dt_xref_link;

   IF vn_continue > 0
   THEN
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -   EXECUTE PECS 5 ROLLBACK PROCESS');
      sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
      RETURN;
   END IF;

   vn_continue := mono_crossreference_migration;

   IF vn_continue > 0
   THEN
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -   EXECUTE PECS 5 ROLLBACK PROCESS');
      sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
      RETURN;
   END IF;

   vn_continue := create_mono_xref_link;

   IF vn_continue > 0
   THEN
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -   EXECUTE PECS 5 ROLLBACK PROCESS');
      sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
      RETURN;
   END IF;

   vn_continue := create_pecs_contact;

   IF vn_continue > 0
   THEN
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -   EXECUTE PECS 5 ROLLBACK PROCESS');
      sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
      RETURN;
   END IF;

   vn_continue := field_metadata_data;

   IF vn_continue > 0
   THEN
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -   EXECUTE PECS 5 ROLLBACK PROCESS');
      sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
      RETURN;
   END IF;

   vn_continue := field_metadata_classnameupdate;

   IF vn_continue > 0
   THEN
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -   EXECUTE PECS 5 ROLLBACK PROCESS');
      sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
      RETURN;
   END IF;

   vn_continue := comparison_report_concept_dose;

   IF vn_continue > 0
   THEN
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -   EXECUTE PECS 5 ROLLBACK PROCESS');
      sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
      RETURN;
   END IF;

   vn_continue := ddi_export_date_column_add;

   IF vn_continue > 0
   THEN
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -   EXECUTE PECS 5 ROLLBACK PROCESS');
      sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
      RETURN;
   END IF;

   vn_continue := ddi_export_date_field_metadata;

   IF vn_continue > 0
   THEN
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -   EXECUTE PECS 5 ROLLBACK PROCESS');
      sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
      RETURN;
   END IF;

   vn_continue := dr_export_date_column_add;

   IF vn_continue > 0
   THEN
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -   EXECUTE PECS 5 ROLLBACK PROCESS');
      sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
      RETURN;
   END IF;

   vn_continue := dr_export_date_field_metadata;

   IF vn_continue > 0
   THEN
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -   EXECUTE PECS 5 ROLLBACK PROCESS');
      sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
      RETURN;
   END IF;

   vn_continue := drugpairs_export_date_column_a;

   IF vn_continue > 0
   THEN
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -   EXECUTE PECS 5 ROLLBACK PROCESS');
      sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
      RETURN;
   END IF;

   vn_continue := drugpairs_export_date_field_me;

   IF vn_continue > 0
   THEN
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -   EXECUTE PECS 5 ROLLBACK PROCESS');
      sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
      RETURN;
   END IF;

   vn_continue := dt_export_date_column_add;

   IF vn_continue > 0
   THEN
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -   EXECUTE PECS 5 ROLLBACK PROCESS');
      sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
      RETURN;
   END IF;

   vn_continue := dt_export_date_field_metadata;

   IF vn_continue > 0
   THEN
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -   EXECUTE PECS 5 ROLLBACK PROCESS');
      sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
      RETURN;
   END IF;

   vn_continue := pm_export_date_column_add;

   IF vn_continue > 0
   THEN
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -   EXECUTE PECS 5 ROLLBACK PROCESS');
      sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
      RETURN;
   END IF;

   vn_continue := pm_export_date_field_metadata;

   IF vn_continue > 0
   THEN
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -   EXECUTE PECS 5 ROLLBACK PROCESS');
      sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
      RETURN;
   END IF;

   vn_continue := migrate_ddi_export_date;

   IF vn_continue > 0
   THEN
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -   EXECUTE PECS 5 ROLLBACK PROCESS');
      sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
      RETURN;
   END IF;

   vn_continue := migrate_dp_export_date;

   IF vn_continue > 0
   THEN
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -   EXECUTE PECS 5 ROLLBACK PROCESS');
      sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
      RETURN;
   END IF;

   vn_continue := migrate_dr_export_date;

   IF vn_continue > 0
   THEN
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -   EXECUTE PECS 5 ROLLBACK PROCESS');
      sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
      RETURN;
   END IF;

   vn_continue := migrate_dt_export_date;

   IF vn_continue > 0
   THEN
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -   EXECUTE PECS 5 ROLLBACK PROCESS');
      sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
      RETURN;
   END IF;

   vn_continue := migrate_mono_export_date;

   IF vn_continue > 0
   THEN
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -   EXECUTE PECS 5 ROLLBACK PROCESS');
      sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
      RETURN;
   END IF;

   vn_continue := zip_file_name_add_extension;

   IF vn_continue > 0
   THEN
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -   EXECUTE PECS 5 ROLLBACK PROCESS');
      sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
      RETURN;
   END IF;

   vn_continue := add_column_reverse_fdb_id_to_p;

   IF vn_continue > 0
   THEN
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -   EXECUTE PECS 5 ROLLBACK PROCESS');
      sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
      RETURN;
   END IF;

   vn_continue := add_reverse_fdb_id_to_drug_dru;

   IF vn_continue > 0
   THEN
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -   EXECUTE PECS 5 ROLLBACK PROCESS');
      sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
      RETURN;
   END IF;

   vn_continue := add_reverse_fdb_id_to_drugpair;

   IF vn_continue > 0
   THEN
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -   EXECUTE PECS 5 ROLLBACK PROCESS');
      sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
      RETURN;
   END IF;

   vn_continue := populate_reverse_fdb_id_for_pa;

   IF vn_continue > 0
   THEN
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -   EXECUTE PECS 5 ROLLBACK PROCESS');
      sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
      RETURN;
   END IF;

   vn_continue := pecs_ddi_export_va;

   IF vn_continue > 0
   THEN
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -   EXECUTE PECS 5 ROLLBACK PROCESS');
      sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
      RETURN;
   END IF;

   vn_continue := pecs_dp_export_va;

   IF vn_continue > 0
   THEN
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -   EXECUTE PECS 5 ROLLBACK PROCESS');
      sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
      RETURN;
   END IF;

   vn_continue := pecs_dr_export_va;

   IF vn_continue > 0
   THEN
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -   EXECUTE PECS 5 ROLLBACK PROCESS');
      sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
      RETURN;
   END IF;

   vn_continue := pecs_dt_export_va;

   IF vn_continue > 0
   THEN
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -   EXECUTE PECS 5 ROLLBACK PROCESS');
      sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
      RETURN;
   END IF;

   vn_continue := pecs_mono_export_va;

   IF vn_continue > 0
   THEN
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -   EXECUTE PECS 5 ROLLBACK PROCESS');
      sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
      RETURN;
   END IF;

   vn_continue := execute_concept_export_migrati;

   IF vn_continue > 0
   THEN
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -   EXECUTE PECS 5 ROLLBACK PROCESS');
      sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
      RETURN;
   END IF;

   vn_continue := add_pecs4_foreign_keys;

   IF vn_continue > 0
   THEN
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -   EXECUTE PECS 5 ROLLBACK PROCESS');
      sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
      RETURN;
   END IF;

   vn_continue := half_life_migration;

   IF vn_continue > 0
   THEN
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -   EXECUTE PECS 5 ROLLBACK PROCESS');
      sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
      RETURN;
   END IF;

   vn_continue := duplicate_allowance_migration;

   IF vn_continue > 0
   THEN
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -   EXECUTE PECS 5 ROLLBACK PROCESS');
      sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
      RETURN;
   END IF;

   vn_continue := drop_dupallow_constraint;

   IF vn_continue > 0
   THEN
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -   EXECUTE PECS 5 ROLLBACK PROCESS');
      sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
      RETURN;
   END IF;

   vn_continue := duplicate_allowance;

   IF vn_continue > 0
   THEN
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -   EXECUTE PECS 5 ROLLBACK PROCESS');
      sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
      RETURN;
   END IF;

   vn_continue := duplicate_allowance_data;

   IF vn_continue > 0
   THEN
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -   EXECUTE PECS 5 ROLLBACK PROCESS');
      sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
      RETURN;
   END IF;

   vn_continue := create_dupallow_constraint;

   IF vn_continue > 0
   THEN
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -   EXECUTE PECS 5 ROLLBACK PROCESS');
      sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
      RETURN;
   END IF;

   vn_continue := convert_ctissuedate_date;

   IF vn_continue > 0
   THEN
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -   EXECUTE PECS 5 ROLLBACK PROCESS');
      sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
      RETURN;
   END IF;

   vn_continue := ddi_action_date_field_metadats;

   IF vn_continue > 0
   THEN
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -   EXECUTE PECS 5 ROLLBACK PROCESS');
      sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
      RETURN;
   END IF;

   vn_continue := ddi_export_date_field_metadats;

   IF vn_continue > 0
   THEN
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -   EXECUTE PECS 5 ROLLBACK PROCESS');
      sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
      RETURN;
   END IF;

   vn_continue := dp_action_date_field_metadatas;

   IF vn_continue > 0
   THEN
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -   EXECUTE PECS 5 ROLLBACK PROCESS');
      sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
      RETURN;
   END IF;

   vn_continue := dp_export_date_field_metadatas;

   IF vn_continue > 0
   THEN
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -   EXECUTE PECS 5 ROLLBACK PROCESS');
      sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
      RETURN;
   END IF;

   vn_continue := dr_action_date_field_metadatas;

   IF vn_continue > 0
   THEN
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -   EXECUTE PECS 5 ROLLBACK PROCESS');
      sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
      RETURN;
   END IF;

   vn_continue := dr_export_date_field_metadatas;

   IF vn_continue > 0
   THEN
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -   EXECUTE PECS 5 ROLLBACK PROCESS');
      sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
      RETURN;
   END IF;

   vn_continue := dt_action_date_field_metadatas;

   IF vn_continue > 0
   THEN
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -   EXECUTE PECS 5 ROLLBACK PROCESS');
      sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
      RETURN;
   END IF;

   vn_continue := dt_export_date_field_metadatas;

   IF vn_continue > 0
   THEN
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -   EXECUTE PECS 5 ROLLBACK PROCESS');
      sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
      RETURN;
   END IF;

   vn_continue := pm_action_date_field_metadatas;

   IF vn_continue > 0
   THEN
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -   EXECUTE PECS 5 ROLLBACK PROCESS');
      sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
      RETURN;
   END IF;

   vn_continue := pm_export_date_field_metadatas;

   IF vn_continue > 0
   THEN
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -   EXECUTE PECS 5 ROLLBACK PROCESS');
      sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
      RETURN;
   END IF;

   -- Purge recycle bin as CTSTAGING user
   vc_sql_statement := 'purge RECYCLEBIN';
   COMMIT;

   EXECUTE IMMEDIATE vc_sql_statement;

   -- Recompile Schema and Gather Statistics
   DBMS_UTILITY.compile_schema (schema => 'CTSTAGING');
   DBMS_STATS.gather_schema_stats ('CTSTAGING');
EXCEPTION
   WHEN OTHERS
   THEN
      vn_code := SQLCODE;
      vc_errm := SUBSTR (SQLERRM, 1, 64);
      sys.DBMS_OUTPUT.put_line (
         'MAIN: Error code ' || vn_code || ': ' || vc_errm);
END;
/

DROP PROCEDURE concept_export_migration;


SELECT 'Migration Stopped: ' || CURRENT_TIMESTAMP FROM DUAL;


SPOOL OFF
EXIT